

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "private\vw_common_exports.h"
#include <string>


//********************************************************************
namespace vertexware {


//====================================================================
// Base exception class.
// All vertexware exception-classes should derive from this one.
//====================================================================
class VW_COMMON_API Exception
{
public:
	Exception();
	virtual ~Exception(); //enable RTTI

	void  SetMessage(const std::string& message);
	void  SetFileLine(const std::string& file, uint line);

	const std::string   GetClassName() const;

	bool                ContainsMessage() const { return bMessage; }
	const std::string&  GetMessage() const { return message; }

	bool                ContainsFileLine() const { return bFileLine; }
	const std::string&  GetFile() const { return file; }
	const uint          GetLine() const { return line; }

	const std::string   MakeDescription(bool printClass = true,
								bool printFileLine = true, bool printMessage = true) const;


private:
	bool         bMessage;
	std::string  message;

	bool         bFileLine;
	std::string  file;
	uint         line;
};


//====================================================================
// Exception declaration and throw macros.
//====================================================================
#define  VW_DERIVE_EXCEPTION(DerivedExceptionClass, BaseExceptionClass) \
	class DerivedExceptionClass : public BaseExceptionClass {};


//====================================================================
#define  VW_DECLARE_EXCEPTION(DerivedExceptionClass) \
	VW_DERIVE_EXCEPTION(DerivedExceptionClass, vertexware::Exception)


//====================================================================
#define  VW_THROW_FILE_LINE(ExceptionClass, Message, File, Line) \
	{ ExceptionClass e; \
	e.SetMessage(Message); \
	e.SetFileLine(File, Line); \
	throw e; }


//====================================================================
#define  VW_THROW(ExceptionClass, Message) \
	VW_THROW_FILE_LINE(ExceptionClass, Message, __FILE__, __LINE__)


//====================================================================
#define  VW_IF_THROW_ELSE_RETURN(Expression, ReturnValue, ExceptionClass, Message) \
	if(Expression) VW_THROW_FILE_LINE(ExceptionClass, Message, __FILE__, __LINE__) \
	else return ReturnValue;


//====================================================================
// Common exceptions.
//====================================================================
VW_DECLARE_EXCEPTION( InvalidUsageException );
VW_DECLARE_EXCEPTION( SystemResourceException );


//********************************************************************
}//namespace vertexware


