

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\SinglePassIterator.h"
#include <set>


//********************************************************************
namespace vertexware {


//====================================================================
// Observer pattern.
// Example usage:
//    class StateObserver {};
//    class StateObject : public Observable<StateObserver> {}
//    StateObject state;
//    StateObserver ob1, ob2;
//    state.RegisterObserver(ob1);
//    state.RegisterObserver(ob2);
//    StateObject::ObserverIterator it = state.GetObserverIterator();
//    StateObject::ObserverConstIterator cit = state.GetObserverConstIterator();
//====================================================================
template<typename TObserver>
class Observable
{
private:
	typedef  std::set<TObserver*>  OBSERVER_CONTAINER_T;
	OBSERVER_CONTAINER_T  observers;

public:
	virtual ~Observable() {}

	//-----------------------
	// Observer registration.
	//-----------------------
	bool IsObserverRegistered(TObserver& o) const {
		return (observers.find(&o) != observers.end());
	}
	void RegisterObserver(TObserver& o) { observers.insert(&o); }
	void UnregisterObserver(TObserver& o) { observers.erase(&o); }
	void UnregisterAllObservers() { observers.clear(); }

public:
	//-----------------------
	// class ObserverIterator
	// GetObserverIterator()
	//-----------------------
	VW_DECLARE_NESTED_SINGLEPASSITERATOR(ObserverIterator, OBSERVER_CONTAINER_T, observers);

	//----------------------------
	// class ObserverConstIterator
	// GetObserverConstIterator()
	//----------------------------
	VW_DECLARE_NESTED_SINGLEPASSCONSTITERATOR(ObserverConstIterator, OBSERVER_CONTAINER_T, observers);
};


//********************************************************************
}//namespace vertexware


