

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\CompilerConfig.h"
#include "private\vw_common_exports.h"
#include <cstddef>
#include <string>
#include <iosfwd>


//********************************************************************
namespace vertexware {


//====================================================================
// Type aliases.
//====================================================================
typedef  unsigned long   ulong;  //32 bits
typedef  unsigned int    uint;   //32 bits
typedef  unsigned short  ushort; //16 bits
typedef           char   byte;   // 8 bits
typedef  unsigned char   ubyte;  // 8 bits


//====================================================================
// Unicode helpers.
//====================================================================
#define  VW_WIDE_TEXT_INDIRECT(a)   L ## a
#define  VW_WIDE_TEXT(a)            VW_WIDE_TEXT_INDIRECT(a)


//====================================================================
// External-variable definition helpers.
//====================================================================
#define  VW_DECLARE_VARIABLE( Type, Name ) \
	extern Type Name;


//====================================================================
#define  VW_DEFINE_VARIABLE( Type, Name, Value ) \
	extern Type Name = Value;


//====================================================================
#define  VW_DEFINE_VARIABLE_DEFAULT_VALUE( Type, Name ) \
	extern Type Name = Type();


//====================================================================
// External-constant definition helpers.
//====================================================================
#define  VW_DECLARE_CONSTANT( Type, Name ) \
	extern const Type Name;


//====================================================================
#define  VW_DEFINE_CONSTANT( Type, Name, Value ) \
	extern const Type Name = Value;


//====================================================================
#define  VW_DEFINE_CONSTANT_DEFAULT_VALUE( Type, Name ) \
	extern const Type Name = Type();


//====================================================================
// Unique identifier.
// Sequence contains 4 unsigned integers (128 bits).
//====================================================================
class VW_COMMON_API UID
{
public:
	UID(); //sets sequence to zeros
	UID(const uint* a); //must point to 4 unsigned integers
	UID(uint a0, uint a1, uint a2, uint a3);

	uint&        operator[](uint i)       { return sequence[i]; }
	const uint&  operator[](uint i) const { return sequence[i]; }

	bool  operator==(const UID& rhs) const;
	bool  operator!=(const UID& rhs) const { return !(*this == rhs); }

	std::string  ToString() const;


private:
	uint  sequence[4];
};


//====================================================================
VW_COMMON_API
__forceinline std::ostream& operator<<(std::ostream& os, const UID& x) {
	return os << x.ToString();
}


//====================================================================
// UID definition helpers.
//====================================================================
#define  VW_DECLARE_UID(name) \
	extern const vertexware::UID name;


//====================================================================
#define  VW_IMPLEMENT_UID(name, a0, a1, a2, a3) \
	const vertexware::UID name(a0, a1, a2, a3);


//********************************************************************
}//namespace vertexware

