

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "BaseTools\Exception.h"
#include "Framework\FrameworkExceptions.h"
#include "private\vw_direct3d8_exports.h"
#include <string>
#include <d3d8.h>


//********************************************************************
namespace vertexware {


//********************************************************************
namespace d3d8_tools {


//====================================================================
VW_DERIVE_EXCEPTION( D3D8Exception, VideoRendererException )


//====================================================================
// Returns D3D8 error description.
//====================================================================
VW_DIRECT3D8_API
std::string __fastcall FormatD3DError(HRESULT hr,
									const std::string& methodName = "",
									const std::string& testPointName = "");


//====================================================================
// Returns true if <hr> indicates D3D8 error.
//====================================================================
VW_DIRECT3D8_API
bool __fastcall D3DErrorCheck(HRESULT hr,
								const std::string& methodName = "",
								const std::string& testPointName = "",
								std::string* pOutErrDesc = 0);


//====================================================================
// Throws D3D8Exception exception if <hr> indicates D3D8 error.
//====================================================================
VW_DIRECT3D8_API
void __fastcall D3DErrorCheckE(HRESULT hr,
								const std::string& methodName = "",
								const std::string& testPointName = "");


//====================================================================
// Throws TException exception if <hr> indicates D3D8 error.
//====================================================================
template<typename TException>
inline void __fastcall D3DErrorCheckET(HRESULT hr,
								const std::string& methodName = "",
								const std::string& testPointName = "")
{
	if(hr != D3D_OK) {
		VW_THROW(TException, FormatD3DError(hr, methodName, testPointName));
	}
}


//********************************************************************
}//namespace d3d8_tools


//********************************************************************
}//namespace vertexware


