

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "MathTools\Matrix44.h"
#include "private\vw_direct3d8_exports.h"
#include <d3d8.h>


//********************************************************************
namespace vertexware {


//********************************************************************
namespace d3d8_tools {


//====================================================================
__forceinline VW_DIRECT3D8_API
const D3DMATRIX& __fastcall ReinterpretCast(const mat4& m)
{
	return reinterpret_cast<const D3DMATRIX&>(m);
}


//====================================================================
__forceinline VW_DIRECT3D8_API
const D3DMATRIX* __fastcall ReinterpretCast(const mat4* m)
{
	return reinterpret_cast<const D3DMATRIX*>(m);
}


//====================================================================
__forceinline VW_DIRECT3D8_API
HRESULT __fastcall SetRenderState_Enable(IDirect3DDevice8& device,
							D3DRENDERSTATETYPE renderState, bool bEnable)
{
	return device.SetRenderState(renderState, bEnable ? TRUE : FALSE);
}


//====================================================================
__forceinline VW_DIRECT3D8_API
HRESULT __fastcall SetRenderState_AlphaRef(IDirect3DDevice8& device, float alphaRef)
{
	return device.SetRenderState(D3DRS_ALPHAREF, 0x000000FF & DWORD(alphaRef * 255.0f));
}


//====================================================================
class StateBlockCompileSentry;


//====================================================================
class VW_DIRECT3D8_API StateBlock
{
friend class StateBlockCompileSentry;

public:
	StateBlock(IDirect3DDevice8& device);
	StateBlock(IDirect3DDevice8& device, D3DSTATEBLOCKTYPE blockType);

	virtual ~StateBlock();

	IDirect3DDevice8*  GetD3DDevice() const { return &device; }
	DWORD              GetD3DToken() const { return token; }

	bool  IsValid() const { return (token != 0); }
	void  Delete();

	HRESULT  CompilePredefined(D3DSTATEBLOCKTYPE blockType);
	HRESULT  Capture() { return device.CaptureStateBlock(token); }
	HRESULT  Apply() const { return device.ApplyStateBlock(token); }


private:
	IDirect3DDevice8&  device;
	DWORD              token;
};


//====================================================================
class VW_DIRECT3D8_API StateBlockCompileSentry
{
public:
	StateBlockCompileSentry(StateBlock& sb);
	~StateBlockCompileSentry();


private:
	StateBlock& sb;
};


//********************************************************************
}//namespace d3d8_tools


//********************************************************************
}//namespace vertexware


