

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "DataObjects\ImageBase.h"
#include "DataObjects\Signal.h"


//********************************************************************
namespace vertexware {


//====================================================================
// Typed image.
//====================================================================
template<typename TSample>
class Image : public ImageBase, public Signal<TSample>
{
public:
	typedef  TSample  SampleType;


public:
	// constructor
	Image(eSampleFormat sampleFormat =
			eSampleFormat(sample_formats::UnknownSampleFormat(sizeof(TSample))),
			uint numRows = 0, uint numColumns = 0)
		:
		SignalBase(sizeof(TSample)),
		ImageBase(sizeof(TSample))
	{
		SetSampleFormat(sampleFormat);
		if(numRows  &&  numColumns) {
			ImageBase::ReserveSampleStorage(numRows, numColumns);
		}
	}

	// storage configuration (avoid inheritance via dominance)
	virtual  void  ReserveSampleStorage(uint numRows, uint numColumns) {
		ImageBase::ReserveSampleStorage(numRows, numColumns);
	}
	virtual  void  ReserveSampleStorage(uint numSamples) {
		ImageBase::ReserveSampleStorage(numSamples);
	}
	virtual  void  ReleaseSampleStorage() {
		ImageBase::ReleaseSampleStorage();
	}

	// storage access
	__forceinline  TSample*  Sample(uint row, uint column) {
		CheckAccess2D(row, column);
		return (GetSamples() + (row * GetNumColumns() + column));
	}
	__forceinline  const TSample*  Sample(uint row, uint column) const {
		return const_cast<Image<TSample >* >(this)->Sample(row, column);
	}
};


//====================================================================
// Standard image types.
//====================================================================
typedef  Image<Sample8>    Image8;
typedef  Image<Sample16>   Image16;
typedef  Image<Sample24>   Image24;
typedef  Image<Sample32>   Image32;
typedef  Image<Sample48>   Image48;
typedef  Image<Sample64>   Image64;
typedef  Image<Sample96>   Image96;
typedef  Image<Sample128>  Image128;


//********************************************************************
}//namespace vertexware


