

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "private\vw_common_exports.h"


//********************************************************************
namespace vertexware {


//====================================================================
struct VW_COMMON_API Sample8
{
	union {
		ubyte  b[1];
		struct {
			ubyte  b0;
		};
	};
};


//====================================================================
struct VW_COMMON_API Sample16
{
	union {
		ubyte  b[2];
		struct {
			ubyte  b0;
			ubyte  b1;
		};

		ushort  s[1];
		struct {
			ushort  s0;
		};
	};
};


//====================================================================
struct VW_COMMON_API Sample24
{
	union {
		ubyte  b[3];
		struct {
			ubyte  b0;
			ubyte  b1;
			ubyte  b2;
		};
	};
};


//====================================================================
struct VW_COMMON_API Sample32
{
	union {
		ubyte  b[4];
		struct {
			ubyte  b0;
			ubyte  b1;
			ubyte  b2;
			ubyte  b3;
		};

		ushort  s[2];
		struct {
			ushort  s0;
			ushort  s1;
		};

		uint  i[1];
		struct {
			uint  i0;
		};

		float  f[1];
		struct {
			float  f0;
		};
	};
};


//====================================================================
struct VW_COMMON_API Sample48
{
	union {
		ubyte  b[6];
		struct {
			ubyte  b0;
			ubyte  b1;
			ubyte  b2;
			ubyte  b3;
			ubyte  b4;
			ubyte  b5;
		};

		ushort  s[3];
		struct {
			ushort  s0;
			ushort  s1;
			ushort  s2;
		};
	};
};


//====================================================================
struct VW_COMMON_API Sample64
{
	union {
		ubyte  b[8];
		struct {
			ubyte  b0;
			ubyte  b1;
			ubyte  b2;
			ubyte  b3;
			ubyte  b4;
			ubyte  b5;
			ubyte  b6;
			ubyte  b7;
		};

		ushort  s[4];
		struct {
			ushort  s0;
			ushort  s1;
			ushort  s2;
			ushort  s3;
		};

		uint  i[2];
		struct {
			uint  i0;
			uint  i1;
		};

		float  f[2];
		struct {
			float  f0;
			float  f1;
		};
	};
};


//====================================================================
struct VW_COMMON_API Sample96
{
	union {
		ubyte  b[12];
		struct {
			ubyte  b0;
			ubyte  b1;
			ubyte  b2;
			ubyte  b3;
			ubyte  b4;
			ubyte  b5;
			ubyte  b6;
			ubyte  b7;
			ubyte  b8;
			ubyte  b9;
			ubyte  b10;
			ubyte  b11;
		};

		ushort  s[6];
		struct {
			ushort  s0;
			ushort  s1;
			ushort  s2;
			ushort  s3;
			ushort  s4;
			ushort  s5;
		};

		uint  i[3];
		struct {
			uint  i0;
			uint  i1;
			uint  i2;
		};

		float  f[3];
		struct {
			float  f0;
			float  f1;
			float  f2;
		};
	};
};


//====================================================================
struct VW_COMMON_API Sample128
{
	union {
		ubyte  b[16];
		struct {
			ubyte  b0;
			ubyte  b1;
			ubyte  b2;
			ubyte  b3;
			ubyte  b4;
			ubyte  b5;
			ubyte  b6;
			ubyte  b7;
			ubyte  b8;
			ubyte  b9;
			ubyte  b10;
			ubyte  b11;
			ubyte  b12;
			ubyte  b13;
			ubyte  b14;
			ubyte  b15;
		};

		ushort  s[8];
		struct {
			ushort  s0;
			ushort  s1;
			ushort  s2;
			ushort  s3;
			ushort  s4;
			ushort  s5;
			ushort  s6;
			ushort  s7;
		};

		uint  i[4];
		struct {
			uint  i0;
			uint  i1;
			uint  i2;
			uint  i3;
		};

		float  f[4];
		struct {
			float  f0;
			float  f1;
			float  f2;
			float  f3;
		};
	};
};


//********************************************************************
namespace sample_formats {


//====================================================================
VW_COMMON_API
std::string __fastcall MapSampleFormatToName(uint sampleFormat);


//====================================================================
VW_COMMON_API
uint __fastcall MapNameToSampleFormat(const std::string& sampleFormatName);


//====================================================================
enum
{
	SampleByteSizeMask         = 0xFF000000,
	SampleByteSizeShift        = 24,
	ComponentLayoutMask        = 0x00FFFFFF,
	ComponentLayoutUnknown     = 0x00000000,
	ComponentLayoutUserDefined = 0x00010000,

	SampleByteSize_1  = (1  << SampleByteSizeShift),
	SampleByteSize_2  = (2  << SampleByteSizeShift),
	SampleByteSize_3  = (3  << SampleByteSizeShift),
	SampleByteSize_4  = (4  << SampleByteSizeShift),
	SampleByteSize_6  = (6  << SampleByteSizeShift),
	SampleByteSize_8  = (8  << SampleByteSizeShift),
	SampleByteSize_12 = (12 << SampleByteSizeShift),
	SampleByteSize_16 = (16 << SampleByteSizeShift),

	// not used
	Force32Bits = 0x7FFFFFFF
};


//====================================================================
VW_COMMON_API __forceinline
uint __fastcall UnknownSampleFormat(uint sampleByteSize)
{
	return (sampleByteSize << SampleByteSizeShift)  |  ComponentLayoutUnknown;
}


//====================================================================
VW_COMMON_API __forceinline
uint __fastcall ExtractByteSize(uint sampleFormat)
{
	return uint(sampleFormat & SampleByteSizeMask) >> SampleByteSizeShift;
}


//====================================================================
VW_COMMON_API __forceinline
uint __fastcall ExtractComponentLayout(uint sampleFormat)
{
	return (sampleFormat & ComponentLayoutMask);
}


//====================================================================
// UTPackage should be unsigned type.
//====================================================================
template<typename UTPackage>
__forceinline
UTPackage __fastcall Unpack(UTPackage package, UTPackage bitMask, UTPackage bitShift)
{
	return (package >> bitShift) & bitMask;
}


//====================================================================
template<typename UTPackage, uint BitMask, uint BitShift>
struct ComponentUnpacker
{
	__forceinline static UTPackage __fastcall Unpack(UTPackage package) {
		return (package >> BitShift) & UTPackage(BitMask);
	}
	__forceinline UTPackage operator()(UTPackage package) const {
		return Unpack(package);
	}
};


//====================================================================
template<typename UTPackage>
__forceinline
void __fastcall Pack(UTPackage& package, UTPackage componentValue,
							UTPackage bitMask, UTPackage bitShift)
{
	package &= ~(bitMask << bitShift);
	package |= ((componentValue & bitMask) << bitShift);
}


//====================================================================
template<typename UTPackage, uint BitMask, uint BitShift>
struct ComponentPacker
{
	__forceinline static void __fastcall Pack(UTPackage& package, UTPackage componentValue) {
		package &= ~(UTPackage(UTPackage(BitMask) << BitShift));
		package |= ((componentValue & UTPackage(BitMask)) << BitShift);
	}
	__forceinline void operator()(UTPackage& package, UTPackage componentValue) const {
		Pack(package, componentValue);
	}
};


//====================================================================
// 0-component starts at msb in register.
//====================================================================
template<typename UTPackage,
		uint BitMask0, uint BitShift0,
		uint BitMask1, uint BitShift1,
		uint BitMask2, uint BitShift2,
		uint BitMask3, uint BitShift3
		>
struct PackedSample4
{
	typedef  UTPackage  PackageType;

	// unpack
	static __forceinline UTPackage __fastcall Unpack0(UTPackage package) {
		return ComponentUnpacker<UTPackage, BitMask0, BitShift0>::Unpack(package);
	}
	static __forceinline UTPackage __fastcall Unpack1(UTPackage package) {
		return ComponentUnpacker<UTPackage, BitMask1, BitShift1>::Unpack(package);
	}
	static __forceinline UTPackage __fastcall Unpack2(UTPackage package) {
		return ComponentUnpacker<UTPackage, BitMask2, BitShift2>::Unpack(package);
	}
	static __forceinline UTPackage __fastcall Unpack3(UTPackage package) {
		return ComponentUnpacker<UTPackage, BitMask3, BitShift3>::Unpack(package);
	}

	// pack
	static __forceinline void __fastcall Pack0(UTPackage& package, UTPackage componentValue) {
		ComponentPacker<UTPackage, BitMask0, BitShift0>::Pack(package, componentValue);
	}
	static __forceinline void __fastcall Pack1(UTPackage& package, UTPackage componentValue) {
		ComponentPacker<UTPackage, BitMask1, BitShift1>::Pack(package, componentValue);
	}
	static __forceinline void __fastcall Pack2(UTPackage& package, UTPackage componentValue) {
		ComponentPacker<UTPackage, BitMask2, BitShift2>::Pack(package, componentValue);
	}
	static __forceinline void __fastcall Pack3(UTPackage& package, UTPackage componentValue) {
		ComponentPacker<UTPackage, BitMask3, BitShift3>::Pack(package, componentValue);
	}
};


//====================================================================
typedef  PackedSample4<uint, 0xFF, 24, 0xFF, 16, 0xFF, 8, 0xFF, 0>  PackedSample_UInt8888;


//====================================================================
typedef  PackedSample4<ushort, 0xFF, 8, 0xFF, 0, 0x00, 0, 0x00, 0>  PackedSample_UShort88;


//====================================================================
typedef  PackedSample4<ushort, 0x0F, 12, 0x0F, 8, 0x0F, 4, 0x0F, 0>  PackedSample_UShort4444;


//====================================================================
typedef  PackedSample4<ushort, 0x01, 15, 0x1F, 10, 0x1F, 5, 0x1F, 0>  PackedSample_UShort1555;


//====================================================================
typedef  PackedSample4<ubyte, 0x0F, 4, 0x0F, 0, 0x00, 0, 0x00, 0>  PackedSample_UByte44;


//********************************************************************
}//namespace sample_formats


//====================================================================
// Standard sample formats.
// Names of formats reflect in-register byte-order, except where noted.
//
// L  - Luminance
// A  - Alpha
// R  - Red
// G  - Green
// B  - Blue
// X  - Unused
// ZB - Z-buffer floating-point value: range [0-1]
// WB - W-buffer floating-point value: range [0-1]
//
//====================================================================
enum eSampleFormat
{
	SampleFormat_Invalid = 0,

	// 8-bit integer formats
	SampleFormat_L8 = (sample_formats::SampleByteSize_1  |  0x00000001),
	SampleFormat_A8 = (sample_formats::SampleByteSize_1  |  0x00000002),

	// 16-bit integer formats
	SampleFormat_A8L8     = (sample_formats::SampleByteSize_2  |  0x00000001),
	SampleFormat_A4R4G4B4 = (sample_formats::SampleByteSize_2  |  0x00000010),
	SampleFormat_A1R5G5B5 = (sample_formats::SampleByteSize_2  |  0x00000020),

	// 32-bit integer formats
	SampleFormat_X8R8G8B8 = (sample_formats::SampleByteSize_4  |  0x00000001),
	SampleFormat_A8R8G8B8 = (sample_formats::SampleByteSize_4  |  0x00000002),

	// 32-bit floating-point formats
	SampleFormat_ZB32 = (sample_formats::SampleByteSize_4  |  0x00001001),
	SampleFormat_WB32 = (sample_formats::SampleByteSize_4  |  0x00001002),

	// not used
	SampleFormat_Force32Bits = 0x7FFFFFFF
};


//********************************************************************
}//namespace vertexware


