

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "DataObjects\SignalIO.h"


//********************************************************************
namespace vertexware {


//====================================================================
// VertexWareDataPackage signal-encoder.
// Format/optional-parameter support:
// 1) VWDP (byte-order-big-endian)
//====================================================================
class VW_COMMON_API SignalEncoderVWDP : public SignalEncoder
{
public:
	SignalEncoderVWDP();

	virtual  bool  SaveSignal(const SignalBase& sourceSignal,
							const std::string& fileName,
							const std::string& fileFormat,
							const NamedParameterList& parameterList);
};


//====================================================================
// VertexWareDataPackage signal-decoder.
// Format/optional-parameter support:
// 1) VWDP
//====================================================================
class VW_COMMON_API SignalDecoderVWDP : public SignalDecoder
{
public:
	SignalDecoderVWDP();

	virtual  SignalBase*  LoadSignal(const std::string& fileName,
								const NamedParameterList& parameterList);
};


//====================================================================
// VertexWareDataPackage image-encoder.
// Format/optional-parameter support:
// 1) VWDP (byte-order-big-endian, origin-bottom)
//====================================================================
class VW_COMMON_API ImageEncoderVWDP : public ImageEncoder
{
public:
	ImageEncoderVWDP();

	virtual  bool  SaveImage(const ImageBase& sourceImage,
							const std::string& fileName,
							const std::string& fileFormat,
							const NamedParameterList& parameterList);
};


//====================================================================
// VertexWareDataPackage image-decoder.
// Format/optional-parameter support:
// 1) VWDP
//====================================================================
class VW_COMMON_API ImageDecoderVWDP : public ImageDecoder
{
public:
	ImageDecoderVWDP();

	virtual  ImageBase*  LoadImage(const std::string& fileName,
								const NamedParameterList& parameterList);
};


//====================================================================
// VertexWareDataPackage volume-encoder.
// Format/optional-parameter support:
// 1) VWDP (byte-order-big-endian, origin-bottom, origin-back)
//====================================================================
class VW_COMMON_API VolumeEncoderVWDP : public VolumeEncoder
{
public:
	VolumeEncoderVWDP();

	virtual  bool  SaveVolume(const VolumeBase& sourceVolume,
							const std::string& fileName,
							const std::string& fileFormat,
							const NamedParameterList& parameterList);
};


//====================================================================
// VertexWareDataPackage volume-decoder.
// Format/optional-parameter support:
// 1) VWDP
//====================================================================
class VW_COMMON_API VolumeDecoderVWDP : public VolumeDecoder
{
public:
	VolumeDecoderVWDP();

	virtual  VolumeBase*  LoadVolume(const std::string& fileName,
								const NamedParameterList& parameterList);
};


//********************************************************************
}//namespace vertexware


