

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "DataObjects\Signal.h"
#include "BaseTools\Singleton.h"
#include "BaseTools\Color.h"
#include <map>


//********************************************************************
namespace vertexware {


//====================================================================
// Inplace low-level helpers.
//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_FlipByteOrder(Signal16& target);


//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_FlipByteOrder(Signal32& target);


//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_FlipByteOrder(SignalBase& target);


//====================================================================
// target-sample-size must be >= 3.
// <tripletOffset> must be <= (target-sample-size - 3).
//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_3BytesFlipByteOrder(SignalBase& target, uint tripletOffset);


//====================================================================
// <byteOffset1> and <byteOffset2> must be < target-sample-size.
//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_SwapBytes(SignalBase& target,
									uint byteOffset1, uint byteOffset2);


//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_Clear(SignalBase& target);


//====================================================================
// <byteOffset> must be < target-sample-size.
//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_Fill1Byte(SignalBase& target,
										uint byteOffset, ubyte value);


//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_FillNBytes(SignalBase& target, ubyte value);


//====================================================================
// target-sample-size must be >= 3.
// <tripletOffset> must be <= (target-sample-size - 3).
//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_Fill3Bytes(SignalBase& target, uint tripletOffset,
										ubyte value0, ubyte value1, ubyte value2);


//====================================================================
// <byteOffset> must be < target-sample-size.
//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_Fill1Nibble(SignalBase& target,
								uint byteOffset, bool bNibbleHigh, ubyte value);


//====================================================================
VW_COMMON_API __forceinline
void __fastcall ProcessSamples_Fill1Nibble_MapValueRange(SignalBase& target,
								uint byteOffset, bool bNibbleHigh, ubyte value)
{
	ProcessSamples_Fill1Nibble(target, byteOffset, bNibbleHigh, (value >> 4));
}


//====================================================================
// sample-format sizes must be as specified.
//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_FillBits_UShort(SignalBase& target, ushort value,
												ushort bitMask, ushort bitShift);


//====================================================================
// <target> must be 4 bytes and is interpreted as A8R8G8B8.
// Copies BGR luminance to A component, then thresholds BGR components.
//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_BGRToA_ThresholdBGR(SignalBase& target);


//====================================================================
// <target> must be 4 bytes and is interpreted as A8R8G8B8.
// Sets pixel's alpha to <alphaMatch> if corresponding BGR matches
// <rgbColorKey>, otherwise sets pixel's alpha to <alphaMismatch>.
//====================================================================
VW_COMMON_API
void __fastcall ProcessSamples_AlphaFromColorKey(
								SignalBase& target, color4b rgbColorKey,
								ubyte alphaMatch = 0, ubyte alphaMismatch = 255);


//====================================================================
// Source to destination low-level helpers.
// <dst> and <src> sample-counts must be equal.
//====================================================================
VW_COMMON_API
void __fastcall CopySamples_FlippedByteOrder(Signal16& dst, const Signal16& src);


//====================================================================
VW_COMMON_API
void __fastcall CopySamples_FlippedByteOrder(Signal32& dst, const Signal32& src);


//====================================================================
// dst-sample-size must be >= 3.
// <dstTripletOffset> must be <= (dst-sample-size - 3).
// src-sample-size must be >= 3.
// <srcTripletOffset> must be <= (src-sample-size - 3).
//====================================================================
VW_COMMON_API
void __fastcall CopySamples_3BytesFlippedByteOrder(SignalBase& dst, uint dstTripletOffset,
									const SignalBase& src, uint srcTripletOffset);


//====================================================================
// <dstLOffset> must be < dst-sample-size.
// src-sample-size must be >= 3.
// <srcBGROffset> must be <= (src-sample-size - 3).
//====================================================================
VW_COMMON_API
void __fastcall CopySamples_3BytesBGRTo1ByteL(SignalBase& dst, uint dstLOffset,
									const SignalBase& src, uint srcBGROffset);


//====================================================================
// dst-sample-size must be >= 3.
// <dstBGROffset> must be <= (dst-sample-size - 3).
// <srcLOffset> must be < src-sample-size.
//====================================================================
VW_COMMON_API
__forceinline
void __fastcall CopySamples_1ByteLTo3BytesBGR(SignalBase& dst, uint dstBGROffset,
									const SignalBase& src, uint srcLOffset);


//====================================================================
// sample-format sizes must be as specified.
//====================================================================
VW_COMMON_API
void __fastcall CopySamples_UInt8888_To_UShort4444(SignalBase& dst, const SignalBase& src);


//====================================================================
VW_COMMON_API
void __fastcall CopySamples_UShort4444_To_UInt8888(SignalBase& dst, const SignalBase& src);


//====================================================================
VW_COMMON_API
void __fastcall CopySamples_UInt8888_To_UShort1555(SignalBase& dst, const SignalBase& src);


//====================================================================
VW_COMMON_API
void __fastcall CopySamples_UShort1555_To_UInt8888(SignalBase& dst, const SignalBase& src);


//====================================================================
// FP32Norm is floating-point value normalized to range [0-1]
//====================================================================
VW_COMMON_API
void __fastcall CopySamples_FP32Norm_To_UInt8(SignalBase& dst, const SignalBase& src);


//====================================================================
VW_COMMON_API
void __fastcall CopySamples_UInt8_To_FP32Norm(SignalBase& dst, const SignalBase& src);


//====================================================================
// <numBytesPerSample> must be <= dst-sample-size.
// <dstByteOffset> must be <= (dst-sample-size - <numBytesPerSample>).
// <numBytesPerSample> must be <= src-sample-size.
// <srcByteOffset> must be <= (src-sample-size - <numBytesPerSample>).
// NOTE: Flat-copy is performed if possible.
//====================================================================
VW_COMMON_API
void __fastcall CopySamples_NBytes(SignalBase& dst, uint dstByteOffset,
							const SignalBase& src, uint srcByteOffset,
							uint numBytesPerSample);


//====================================================================
VW_COMMON_API __forceinline
void __fastcall CopySamples(SignalBase& dst, const SignalBase& src)
{
	CopySamples_NBytes(dst, 0, src, 0, dst.GetSampleByteSize());
}


//====================================================================
// Signal-operations manager (singleton)
//====================================================================
class VW_COMMON_API SignalOperationsManager
{
public:
	//------------------------------------
	// Transfer-samples handler interface.
	//------------------------------------
	struct VW_COMMON_API TransferSamplesHandler {
		virtual ~TransferSamplesHandler() {}
		virtual  void  Handle(SignalBase& dst, const SignalBase& src) {}
	};

	//-----------------------------------
	// Convert-samples handler interface.
	//-----------------------------------
	struct VW_COMMON_API ConvertSamplesHandler {
		virtual ~ConvertSamplesHandler() {}
		virtual  void  Handle(SignalBase& target, eSampleFormat newSampleFormat) {}
	};


public:
	VW_DECLARE_SINGLETON( SignalOperationsManager );


public:
	SignalOperationsManager();
	~SignalOperationsManager();


	//----------------------------------------------------------
	// If handler already exists, it is replaced by new handler.
	// Only handler-address is stored.
	//----------------------------------------------------------
	void  RegisterTransferHandler(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat,
												TransferSamplesHandler& tsh);
	void  RegisterDefaultTransferHandlers();

	void  RegisterConvertHandler(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat,
												ConvertSamplesHandler& csh);
	void  RegisterDefaultConvertHandlers();

	void  UnregisterTransferHandler(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat);
	void  UnregisterConvertHandler(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat);

	void  UnregisterAllTransferHandlers();
	void  UnregisterAllConvertHandlers();


	//-------------------------------------------------------
	// Returns handler if transfer is supported, otherwise 0.
	//-------------------------------------------------------
	TransferSamplesHandler*  GetTransferHandler(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat);
	ConvertSamplesHandler*   GetConvertHandler(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat);

	//---------------------------------------
	// Returns true if transfer is supported.
	//---------------------------------------
	bool IsTransferSupported(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat) {
		return (GetTransferHandler(dstSampleFormat, srcSampleFormat) != 0);
	}
	bool IsConversionSupported(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat) {
		return (GetConvertHandler(dstSampleFormat, srcSampleFormat) != 0);
	}


private:
	//--------------
	// Handler keys.
	//--------------
	struct VW_COMMON_API HandlerKey {
		eSampleFormat  dstFmt;
		eSampleFormat  srcFmt;

		HandlerKey(eSampleFormat _dstFmt, eSampleFormat _srcFmt) {
			dstFmt = _dstFmt;
			srcFmt = _srcFmt;
		}

		bool operator<(const HandlerKey& rhs) const {
			return (dstFmt < rhs.dstFmt) ||
			((dstFmt == rhs.dstFmt) && (srcFmt < rhs.srcFmt));
		}
	};

	typedef  std::map<HandlerKey, TransferSamplesHandler*>  TRANSFER_HANDLER_MAP;
	typedef  TRANSFER_HANDLER_MAP::iterator                 TRANSFER_HANDLER_ITER;

	typedef  std::map<HandlerKey, ConvertSamplesHandler*>   CONVERT_HANDLER_MAP;
	typedef  CONVERT_HANDLER_MAP::iterator                  CONVERT_HANDLER_ITER;

	//--------------
	// Handler maps.
	//--------------
	TRANSFER_HANDLER_MAP  transferHandlerMap;
	CONVERT_HANDLER_MAP   convertHandlerMap;
};


//====================================================================
// Returns 0 if sample-byte-size is unsupported.
//====================================================================
VW_COMMON_API
SignalBase* __fastcall ManufactureSignal(eSampleFormat sampleFormat, uint numSamples = 0);


//====================================================================
// Returns 0 if sample-byte-size is unsupported.
//====================================================================
VW_COMMON_API
SignalBase* __fastcall CloneSignal(const SignalBase& src);


//====================================================================
// Sample-formats must match.
// Offsets and counts must be within valid range.
// If <count> is 0, full source-size is copied.
//====================================================================
VW_COMMON_API
void __fastcall CopySamples1D(SignalBase& dst, const SignalBase& src,
							uint count = 0, uint dstOffset = 0, uint srcOffset = 0);


//====================================================================
// Returns false if transfer is not supported.
// If <bAutoResize> is false, sample-counts must be equal.
//====================================================================
VW_COMMON_API
bool __fastcall TransferSamples1D(SignalBase& dst, const SignalBase& src,
													bool bAutoResize = true);


//====================================================================
// If <bAutoResize> is false, sample-counts must be equal.
//====================================================================
VW_COMMON_API
void __fastcall TransferSamples1D(SignalBase& dst, const SignalBase& src,
									SignalOperationsManager::TransferSamplesHandler& tsh,
									bool bAutoResize = true);


//====================================================================
// Returns false if conversion is not supported.
// If conversion is supported, samples are converted to <newSampleFormat>
// and target's sample-format-flags are automatically set to <newSampleFormat>.
//====================================================================
VW_COMMON_API
bool __fastcall ConvertSamples(SignalBase& target, eSampleFormat newSampleFormat);


//====================================================================
// Resampling filters.
//====================================================================
enum eResampleFilter
{
	ResampleFilter_Nearest = 0,
	ResampleFilter_Linear,

	ResampleFilter_Count,
	ResampleFilter_Force32Bits = 0x7FFFFFFF
};


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
//====================================================================
VW_COMMON_API
void __fastcall Resample1D_Nearest(SignalBase& dst, const SignalBase& src);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
// <numComponentsPerSample> must be <= (sample-size / sizeof(float)).
// <componentOffset> must be <= ((sample-size / sizeof(float)) - <numComponentsPerSample>).
// Each component is assumed to be a separate channel.
//====================================================================
VW_COMMON_API
void __fastcall Resample1D_Linear_NFloats(SignalBase& dst, const SignalBase& src,
								uint componentOffset, uint numComponentsPerSample);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
// <numBytesPerSample> must be <= sample-size.
// <byteOffset> must be <= (sample-size - <numBytesPerSample>).
// Each byte is assumed to be a separate channel.
//====================================================================
VW_COMMON_API
void __fastcall Resample1D_Linear_NBytes(SignalBase& dst, const SignalBase& src,
										uint byteOffset, uint numBytesPerSample);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// sample-format byte-size must be 2.
// dst-sample-format and src-sample-format must be equal.
// Each bit-group is assumed to be a separate channel.
//====================================================================
VW_COMMON_API
void __fastcall Resample1D_Linear_PackedSample_UShort4444(SignalBase& dst, const SignalBase& src);


//====================================================================
VW_COMMON_API
void __fastcall Resample1D_Linear_PackedSample_UShort1555(SignalBase& dst, const SignalBase& src);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
// Returns false if filter-type/sample-format combo is not supported.
//====================================================================
VW_COMMON_API
bool __fastcall Resample1D(SignalBase& dst, const SignalBase& src,
							eResampleFilter filterType = ResampleFilter_Linear);


//====================================================================
VW_COMMON_API
void __fastcall FlipSamples(SignalBase& target);


//====================================================================
// Fill_*() family returns false if fill is not supported on
// target's sample-format. No conversions are performed.
// These are useful for byte-order testing.
//====================================================================
VW_COMMON_API
bool __fastcall Fill_Luminance(SignalBase& target, ubyte luminance);


//====================================================================
VW_COMMON_API
bool __fastcall Fill_ARGB(SignalBase& target, color4b argb);


//====================================================================
VW_COMMON_API
bool __fastcall Fill_RGB(SignalBase& target, color4b rgb);


//====================================================================
VW_COMMON_API
bool __fastcall Fill_Alpha(SignalBase& target, ubyte alpha);


//====================================================================
VW_COMMON_API
bool __fastcall Fill_Red(SignalBase& target, ubyte red);


//====================================================================
VW_COMMON_API
bool __fastcall Fill_Green(SignalBase& target, ubyte green);


//====================================================================
VW_COMMON_API
bool __fastcall Fill_Blue(SignalBase& target, ubyte blue);


//====================================================================
// Generates gradient between <startIndex> and <endIndex> which
// must be in valid range. Indices are inclusive.
// Returns false if target-sample-format is not supported.
//====================================================================
VW_COMMON_API
bool __fastcall GradientFill(SignalBase& target,
							const color4f& rgbaStart, const color4f& rgbaEnd,
							uint startIndex, uint endIndex);


//====================================================================
// <byteOffset> must be < sample-size.
// All values <  <thresholdValue> will be set to <lowValue>.
// All values == <thresholdValue> will be set to <newThresholdValue>.
// All values >  <thresholdValue> will be untouched.
//====================================================================
VW_COMMON_API
void __fastcall HighPassFilter(SignalBase& target, uint byteOffset,
							ubyte thresholdValue, ubyte newThresholdValue, ubyte lowValue);


//====================================================================
// <byteOffset> must be < sample-size.
// All values <  <thresholdValue> will be untouched.
// All values == <thresholdValue> will be set to <newThresholdValue>.
// All values >  <thresholdValue> will be set to <highValue>.
//====================================================================
VW_COMMON_API
void __fastcall LowPassFilter(SignalBase& target, uint byteOffset,
							ubyte thresholdValue, ubyte newThresholdValue, ubyte highValue);


//====================================================================
// <byteOffset> must be < sample-size.
// All values <  <thresholdValue> will be set to <lowValue>.
// All values == <thresholdValue> will be set to <midValue>.
// All values >  <thresholdValue> will be set to <highValue>.
//====================================================================
VW_COMMON_API
void __fastcall ThresholdSegmentation(SignalBase& target, uint byteOffset,
							ubyte thresholdValue,
							ubyte lowValue, ubyte midValue, ubyte highValue);


//====================================================================
// <numBytesPerSample> must be <= sample-size - <byteOffset>.
//====================================================================
VW_COMMON_API
void __fastcall ThresholdSegmentationNBytes(SignalBase& target,
							uint byteOffset, uint numBytesPerSample,
							ubyte thresholdValue,
							ubyte lowValue, ubyte midValue, ubyte highValue);


//********************************************************************
}//namespace vertexware


