

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "DataObjects\SignalOperations.h"
#include "DataObjects\Volume.h"


//********************************************************************
namespace vertexware {


//====================================================================
// Returns 0 if sample-byte-size is unsupported.
//====================================================================
VW_COMMON_API
VolumeBase* __fastcall ManufactureVolume(eSampleFormat sampleFormat,
							uint numLayers = 0, uint numRows = 0, uint numColumns = 0);


//====================================================================
// Returns 0 if sample-byte-size is unsupported.
//====================================================================
VW_COMMON_API
VolumeBase* __fastcall CloneVolume(const VolumeBase& src);


//====================================================================
// Sample-formats must match.
// Offsets and counts must be within valid range.
// If <layerCount> is 0, full source-layer-count is copied.
// If <rowCount> is 0, full source-row-count is copied.
// If <columnCount> is 0, full source-column-count is copied.
//====================================================================
VW_COMMON_API
void __fastcall CopySamples3D(VolumeBase& dst, const VolumeBase& src,
						uint layerCount = 0, uint rowCount = 0, uint columnCount = 0,
						uint dstLayerOffset = 0, uint dstRowOffset = 0, uint dstColumnOffset = 0,
						uint srcLayerOffset = 0, uint srcRowOffset = 0, uint srcColumnOffset = 0,
						bool bFlipLayers = false, bool bFlipRows = false);


//====================================================================
// Returns false if transfer is not supported.
// If <bAutoResize> is false, dimensions must be equal.
//====================================================================
VW_COMMON_API
bool __fastcall TransferSamples3D(VolumeBase& dst, const VolumeBase& src,
											bool bAutoResize = true);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
//====================================================================
VW_COMMON_API
void __fastcall Resample3D_Nearest(VolumeBase& dst, const VolumeBase& src);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
// <numComponentsPerSample> must be <= (sample-size / sizeof(float)).
// <componentOffset> must be <= ((sample-size / sizeof(float)) - <numComponentsPerSample>).
// Each component is assumed to be a separate channel.
//====================================================================
VW_COMMON_API
void __fastcall Resample3D_Linear_NFloats(VolumeBase& dst, const VolumeBase& src,
								uint componentOffset, uint numComponentsPerSample);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
// <numBytesPerSample> must be <= sample-size.
// <byteOffset> must be <= (sample-size - <numBytesPerSample>).
// Each byte is assumed to be a separate channel.
//====================================================================
VW_COMMON_API
void __fastcall Resample3D_Linear_NBytes(VolumeBase& dst, const VolumeBase& src,
										uint byteOffset, uint numBytesPerSample);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// sample-format byte-size must be 2.
// dst-sample-format and src-sample-format must be equal.
// Each bit-group is assumed to be a separate channel.
//====================================================================
VW_COMMON_API
void __fastcall Resample3D_Linear_PackedSample_UShort4444(VolumeBase& dst, const VolumeBase& src);


//====================================================================
VW_COMMON_API
void __fastcall Resample3D_Linear_PackedSample_UShort1555(VolumeBase& dst, const VolumeBase& src);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
// Returns false if filter-type/sample-format combo is not supported.
//====================================================================
VW_COMMON_API
bool __fastcall Resample3D(VolumeBase& dst, const VolumeBase& src,
							eResampleFilter filterType = ResampleFilter_Linear);


//====================================================================
// dst-layer-byte-size must be equal to src-total-byte-size.
// <dstLayerIndex> must be < dst-num-layers.
//====================================================================
VW_COMMON_API
void __fastcall CopyToLayer(VolumeBase& dst, uint dstLayerIndex, const SignalBase& src);


//====================================================================
// src-layer-byte-size must be equal to dst-total-byte-size.
// <srcLayerIndex> must be < src-num-layers.
//====================================================================
VW_COMMON_API
void __fastcall CopyFromLayer(SignalBase& dst, const VolumeBase& src, uint srcLayerIndex);


//====================================================================
VW_COMMON_API
void __fastcall FlipLayers(VolumeBase& target);


//====================================================================
VW_COMMON_API
void __fastcall FlipRows(VolumeBase& target);


//====================================================================
VW_COMMON_API
void __fastcall FlipColumns(VolumeBase& target);


//********************************************************************
}//namespace vertexware


