

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\Plugin.h"
#include "BaseTools\WinUtils.h"
#include "private\vw_common_exports.h"


//********************************************************************
namespace vertexware {


//====================================================================
class VW_COMMON_API DirectInputConfig
{
public:
	DirectInputConfig();

	bool  operator==(const DirectInputConfig& rhs) const;
	bool  operator!=(const DirectInputConfig& rhs) const { return !(*this == rhs); }

	bool  IsLogicallyEqual(const DirectInputConfig& rhs) const;

	void  Dump(std::ostream& os, uint tabLevel = 0) const;

public:
	bool  enableDirectInput;
	bool  useLastVersion;
	uint  directInputVersion; //used if <useLastVersion> is false
	bool  enableFullscreenExclusiveMode;
	bool  useAtDeviceInit;
};


//====================================================================
class VW_COMMON_API DirectInputDeviceInstanceInfo
{
public:
	win_utils::Guid  instanceGuid;
	win_utils::Guid  productGuid;

	std::string  instanceName;
	std::string  productName;


public:
	DirectInputDeviceInstanceInfo() {}

	DirectInputDeviceInstanceInfo(
			const win_utils::Guid& instanceGuid, const std::string& instanceName,
			const win_utils::Guid& productGuid, const std::string& productName);

	bool operator==(const DirectInputDeviceInstanceInfo& rhs) const;
	bool operator!=(const DirectInputDeviceInstanceInfo& rhs) const { return !(*this == rhs); }

	void  DumpInfo(std::ostream& os, uint tabLevel = 0) const;
};


//====================================================================
class VW_COMMON_API DirectInputProxy : public Plugin
{
public:
	//------------------------------------
	// Ensure auto-shutdown by destructor.
	//------------------------------------
	virtual ~DirectInputProxy();

	//------------------------------------
	// Return handled DirectInput version.
	//------------------------------------
	virtual  std::string  GetDirectInputProxyInterfaceName() const = 0;

	//--------------------------------------
	// Exceptions allowed from Initialize().
	//--------------------------------------
	virtual  void  Initialize() = 0;
	virtual  void  Shutdown() = 0;


	bool  IsAllowedFullscreenExclusiveMode() const { return bAllowFullscreenExclusiveMode; }
	void  SetAllowFullscreenExclusiveMode(bool bAllow) { bAllowFullscreenExclusiveMode = bAllow; }


protected:
	DirectInputProxy();


private:
	bool  bAllowFullscreenExclusiveMode;
};


//====================================================================
class VW_COMMON_API DirectInputProxyPluginDescriptor : public PluginDescriptor
{
public:
	virtual  Plugin*  CreatePlugin() {
		return CreateDirectInputProxy();
	}

	virtual  DirectInputProxy*  CreateDirectInputProxy() = 0;


protected:
	DirectInputProxyPluginDescriptor(const PluginInfo& pi);
};


//====================================================================
// DirectInputProxy plugin access-point declaration.
//====================================================================
#define  VW_DECLARE_PLUGIN_ACCESS_POINT_DIRECT_INPUT_PROXY( \
				ClassNamePlugin, StringPluginName, \
				UIntPluginVersionMajor, UIntPluginVersionMinor, \
				UIntPluginVersionRevision, UIntPluginVersionBuildFlags, \
				StringPluginDescription, StringPluginDeveloperInfo) \
\
struct ClassNamePlugin##__PluginDescriptor : public DirectInputProxyPluginDescriptor \
{ \
	ClassNamePlugin##__PluginDescriptor() \
		: DirectInputProxyPluginDescriptor(PluginInfo( \
				vertexware::library_info::LibraryVersion(), \
				PluginType_DirectInputProxy, \
				StringPluginName, \
				vertexware::PluginVersion(UIntPluginVersionMajor, UIntPluginVersionMinor, \
										UIntPluginVersionRevision, UIntPluginVersionBuildFlags), \
				VW_BUILD_TIMESTAMP_STRING, \
				StringPluginDescription, \
				StringPluginDeveloperInfo)) \
	{ } \
\
	virtual  DirectInputProxy*  CreateDirectInputProxy() { \
		return new ClassNamePlugin; \
	} \
}; \
VW_DECLARE_PLUGIN_ACCESS_POINT( ClassNamePlugin##__PluginDescriptor )


//********************************************************************
}//namespace vertexware


