

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\DirectInputProxy.h"
#include "private\vw_common_exports.h"

#define  DIRECTINPUT_VERSION  0x0800
#include <dinput.h>


//********************************************************************
namespace vertexware {


//====================================================================
class VW_COMMON_API DirectInputDeviceInstanceInfo8 : public DIDEVICEINSTANCE
{
public:
	DirectInputDeviceInstanceInfo8() {
	}
	DirectInputDeviceInstanceInfo8(const DIDEVICEINSTANCE& rhs) {
		this->operator=(rhs);
	}
	DirectInputDeviceInstanceInfo8& operator=(const DIDEVICEINSTANCE& rhs) {
		*static_cast<DIDEVICEINSTANCE*>(this) = rhs;
		return *this;
	}
	bool operator==(const DirectInputDeviceInstanceInfo8& rhs) const;
	bool operator!=(const DirectInputDeviceInstanceInfo8& rhs) const { return !(*this == rhs); }

	void  CopyInfo(DirectInputDeviceInstanceInfo& didii) const;
	void  DumpInfo(std::ostream& os, uint tabLevel = 0) const;


private:
	static void __fastcall FormatDeviceType(uint iDeviceType, uint iDeviceSubType,
								std::string& sDeviceType, std::string& sDeviceSubType);
};


//====================================================================
// This wrappers auto-initialize DIPROPHEADER member for
// each of DirectInput property structures.
//====================================================================
template<typename TDIPropStruct>
struct AutoInitializedDIPropStruct : public TDIPropStruct
{
	AutoInitializedDIPropStruct(DWORD _dwObj = 0, DWORD _dwHow = DIPH_DEVICE) {
		diph.dwSize = sizeof(TDIPropStruct);
		diph.dwHeaderSize = sizeof(DIPROPHEADER);
		diph.dwObj = _dwObj;
		diph.dwHow = _dwHow;
	}
	AutoInitializedDIPropStruct(const TDIPropStruct& rhs) {
		this->operator=(rhs);
	}
	AutoInitializedDIPropStruct& operator=(const TDIPropStruct& rhs) {
		*static_cast<TPropStruct*>(this) = rhs;
		return *this;
	}

	operator DIPROPHEADER*() { return &diph; }
	operator const DIPROPHEADER*() const { return &diph; }

	void  SetPropertyIdentifiers(DWORD _dwObj = 0, DWORD _dwHow = DIPH_DEVICE) {
		diph.dwObj = _dwObj;
		diph.dwHow = _dwHow;
	}
	void  GetPropertyIdentifiers(DWORD& _dwObj, DWORD& _dwHow) {
		_dwObj = diph.dwObj;
		_dwHow = diph.dwHow;
	}
};


//====================================================================
typedef  AutoInitializedDIPropStruct<DIPROPCPOINTS>      DIPropCPoints;
typedef  AutoInitializedDIPropStruct<DIPROPDWORD>        DIPropDWord;
typedef  AutoInitializedDIPropStruct<DIPROPGUIDANDPATH>  DIPropGuidAndPath;
typedef  AutoInitializedDIPropStruct<DIPROPPOINTER>      DIPropPointer;
typedef  AutoInitializedDIPropStruct<DIPROPRANGE>        DIPropRange;
typedef  AutoInitializedDIPropStruct<DIPROPSTRING>       DIPropString;


//====================================================================
class VW_COMMON_API DirectInputProxy8 : public DirectInputProxy
{
public:
	static std::string __fastcall GetCooperativeLevelDesc(DWORD flags);


public:
	virtual  std::string  GetDirectInputProxyInterfaceName() const;

	//----------------------------
	// DirectInputProxy8 interface
	//----------------------------
	virtual  IDirectInput8*  GetDirectInputObject() = 0;

	virtual  const GUID&  GetGuidSysKeyboard() = 0;
	virtual  const GUID&  GetGuidSysMouse() = 0;

	virtual  const DIDATAFORMAT&  GetDataFormatKeyboard() = 0;
	virtual  const DIDATAFORMAT&  GetDataFormatMouse() = 0;
	virtual  const DIDATAFORMAT&  GetDataFormatMouse2() = 0;
	virtual  const DIDATAFORMAT&  GetDataFormatJoystick() = 0;
	virtual  const DIDATAFORMAT&  GetDataFormatJoystick2() = 0;


protected:
	DirectInputProxy8();
};


//====================================================================
#define  VW_INTERFACE_NAME_DirectInputProxy8   "DirectInputProxy8"


//********************************************************************
}//namespace vertexware


