

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\Kernel.h"
#include "GUI\GUI_Renderer.h"
#include "private\vw_common_exports.h"
#include <cfloat>
#include <deque>


//********************************************************************
namespace vertexware {


//====================================================================
class VW_COMMON_API FrameStatisticsCollector : public FrameObserver, public ReconfigObserver
{
public:
	FrameStatisticsCollector();

	void    SetMinMaxStabilizationPeriod(double secs = 2);
	double  GetMinMaxStabilizationPeriod() const { return secsMinMaxStabilizationPeriod; }

	void    SetSmoothingInterval(double secsInterval = 1);
	double  GetSmoothingInterval() const { return secsSmoothingInterval; }

	void    SetHistoryInterval(double secsInterval = 1);
	double  GetHistoryInterval() const { return secsHistoryInterval; }

	// choose smoothing or history-evaluation
	void    EnableHistoryEvaluation(bool bEnable = true);
	bool    IsEnabledHistoryEvaluation() const { return bEnableHistoryEvaluation; }

	void    Reset();

	uint    GetTotalFrameCount() const { return (totalFrameStartCount - 1); }
	double  GetTotalFrameTime() const { return totalFrameTime; }

	// last frame only
	double  GetFrameRateImmediate() const { return frameRateImmediate; }
	double  GetFrameRateImmediateMin() const { return (frameRateImmediateMin != FLT_MAX ? frameRateImmediateMin : 0); }
	double  GetFrameRateImmediateMax() const { return frameRateImmediateMax; }

	// rate smoothed over N last frames (N is internal dynamic parameter)
	double  GetFrameRateSmooth() const { return frameRateSmooth; }
	double  GetFrameRateSmoothMin() const { return (frameRateSmoothMin != FLT_MAX ? frameRateSmoothMin : 0); }
	double  GetFrameRateSmoothMax() const { return frameRateSmoothMax; }

	//------------------------------
	// FrameObserver implementation.
	//------------------------------
	virtual  void  OnFrame(bool& bAutoUnregister, uint frameNumber, double secsFrameTimeDelta);

	//-----------------------------
	// ReconfigeObserver overrides.
	//-----------------------------
	virtual  void  OnVideoReconfig();
	virtual  void  OnVSyncReconfig(bool bVSyncEnabled);


private:
	typedef  std::deque<double>        TIME_CONTAINER;
	typedef  TIME_CONTAINER::iterator  TIME_ITER;

	double  secsMinMaxStabilizationPeriod;
	double  secsHistoryInterval;
	double  secsSmoothingInterval;
	bool    bEnableHistoryEvaluation;

	// history
	TIME_CONTAINER  frameStartTimes;

	// smoothing
	uint    accumFrameCount;
	double  accumFrameTime;

	uint    totalFrameStartCount;
	double  totalFrameTime;

	double  frameRateImmediate;
	double  frameRateImmediateMin;
	double  frameRateImmediateMax;

	double  frameRateSmooth;
	double  frameRateSmoothMin;
	double  frameRateSmoothMax;
};


//====================================================================
class VW_COMMON_API GUIRendererStatsCollector : public FrameObserver
{
public:
	GUIRendererStatsCollector();

	void  Reset();
	bool  IsRunning() const { return bRunning; }
	void  Run();
	void  Stop();
	void  CalcAverage(); //called by Stop() internally

	uint                              GetFrameCount() const { return frameCount; }
	const gui::Renderer::Statistics&  GetStatsAccum() const { return sAcc; }
	const gui::Renderer::Statistics&  GetStatsMin() const { return sMin; }
	const gui::Renderer::Statistics&  GetStatsMax() const { return sMax; }
	const gui::Renderer::Statistics&  GetStatsAverage() const { return sAvg; }

	//------------------------------
	// FrameObserver implementation.
	//------------------------------
	virtual  void  OnFrame(bool& bAutoUnregister, uint frameNumber, double secsFrameTimeDelta);


private:
	bool                       bRunning;
	uint                       frameCount;
	gui::Renderer::Statistics  sAcc;
	gui::Renderer::Statistics  sMin;
	gui::Renderer::Statistics  sMax;
	gui::Renderer::Statistics  sAvg;
};


//********************************************************************
}//namespace vertexware


