

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Renderer.h"
#include "Framework\VideoSubsystemOpenGL.h"
#include "Framework\VideoRendererOpenGL.h"
#include "BaseTools\Memory.h"
#include "GLTools\DisplayLists.h"
#include "GLTools\Textures.h"
#include "GLTools\VertexBuffers.h"
#include <stack>


//********************************************************************
namespace vertexware {


//====================================================================
class VW_OPENGL_API GUIRendererOpenGL : public gui::Renderer
{
public:
	GUIRendererOpenGL(VideoRendererOpenGL& vrParent);
	virtual ~GUIRendererOpenGL();


	//------------------------------
	// gui::Renderer implementation.
	//------------------------------
	virtual  void  GetRendererParams(RendererParams& rp) const;

	virtual  bool  SetViewport(const ViewportParams& vp);
	virtual  void  ClearBuffers(const ClearParams& cp);

	virtual  void  SetPixelParams(const PixelParams& pp);
	virtual  void  GetPixelParams(PixelParams& pp) const;

	virtual  void  SetBaseDepthParams(const DepthParams& dp);
	virtual  void  GetBaseDepthParams(DepthParams& dp) const;

	virtual  video_renderer_objects::Texture2D*  CreateResourceTexture2D(
					eSampleFormat sampleFormat, bool bMultiLevel, uint height, uint width);

	virtual  bool  IsInsideBeginEnd() { return bInsideBeginEnd; }
	virtual  void  Begin();
	virtual  void  End();

	virtual  bool  IsInsideBeginEndInterleaved() { return bInsideBeginEndInterleaved; }
	virtual  void  BeginInterleaved();
	virtual  void  EndInterleaved();

	virtual  bool  QueryCreatesManagedResourcesOnly();

	virtual  void  SetDepthParams(const DepthParams& dp);
	virtual  void  GetDepthParams(DepthParams& dp) const;

	virtual  void  SetTransform(const mat4& t);
	virtual  void  GetTransform(mat4& t) const;

	virtual  void  DrawPrimitive(const gui::VertexBufferC& vertexBuffer,
								gui::ePrimitiveType primitiveType,
								uint numPrimitives,
								uint startVertex);

	virtual  void  DrawPrimitive(const gui::Texture& texture,
								const gui::VertexBufferCT& vertexBuffer,
								gui::ePrimitiveType primitiveType,
								uint numPrimitives,
								uint startVertex);

	virtual  void  DrawText(const gui::FontTexture& fontTexture,
							uint textLen, const char* pText,
							const gui::DrawTextParams& dtp);


protected:
	//-------------------------------
	// Video-subsystem notifications.
	//-------------------------------
	virtual  void  ConnectVideoSubsystem(VideoSubsystem& vs);
	virtual  void  DisconnectVideoSubsystem();

	virtual  void  CreateVideoResources(bool bPostReset);
	virtual  void  ReleaseVideoResources(bool bPreReset);

	virtual  void  OnWindowResize(uint width, uint height);


private:
	//---------------------------------------
	// Application memory-pool vertex buffer.
	//---------------------------------------
	class AppPoolVertexBuffer : public gl_tools::AdaptiveVertexBuffer<gui::VertexCT>
	{
	public:
		AppPoolVertexBuffer(uint minSize, uint minStableCycles) :
			gl_tools::AdaptiveVertexBuffer<gui::VertexCT>(minSize, minStableCycles)
		{ }

	protected:
		virtual  void  DoHookPointersToGL(const gui::VertexCT* pVertexZero) const;
	};


private:
	void  Compile_Begin();
	void  Compile_End();
	void  Compile_PrepareTextureParams();

	void  Send_DepthParamsRanges(const DepthParams& dp);
	void  Send_DepthParamsFlags(const DepthParams& dp);
	void  Send_EnableAlphaTest(bool bEnable);
	void  Send_DisableTexturing();
	void  Send_BindTextureAndEnableTexturing(const gui::Texture& texture);

	void  ConstructInternalTransform();

	__forceinline  void  Send_Transform() { glLoadMatrixf(matTransform * matInternalTransform); }

	// returns 0 on error
	static GLenum __fastcall TranslatePrimitiveParams(gui::ePrimitiveType pt,
												uint numPrimitives, uint& outVertexCount);

	__forceinline  void  InvalidateLastTexParams()
	{
		pLastBoundTexture = 0;
		lastTexMipFilter = gui::Texture::Filter_Invalid;
	}


private:
	PixelParams  pixelParams;
	DepthParams  depthParamsBase;
	DepthParams  depthParams;
	mat4         matInternalTransform;
	mat4         matTransform;

	bool  bInsideBeginEnd;
	bool  bInsideBeginEndInterleaved;
	bool  bTexturingEnabled;
	bool  bAlphaTestEnabled;

	const gui::Texture*    pLastBoundTexture;
	gui::Texture::eFilter  lastTexMipFilter;

	AppPoolVertexBuffer  appPoolVertexBuffer;

	AutoPtr<gl_tools::DisplayList>  apDL_Begin;
	AutoPtr<gl_tools::DisplayList>  apDL_End;
	AutoPtr<gl_tools::DisplayList>  apDL_PrepareTextureParams;
};


//********************************************************************
}//namespace vertexware


