

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\HumanInterfaceDevice.h"
#include "private\vw_common_exports.h"


//********************************************************************
namespace vertexware {


//====================================================================
class VW_COMMON_API HIDController
{
public:
	virtual ~HIDController();

	HumanInterfaceDevice&        GetParentHID();
	const HumanInterfaceDevice&  GetParentHID() const { return const_cast<HIDController*>(this)->GetParentHID(); }

	//---------------------------
	// Controller-interface name.
	//---------------------------
	virtual  std::string  GetControllerInterfaceName() const = 0;

	//----------------------------
	// Print controller info/caps.
	//----------------------------
	virtual  void  DumpControllerInfo(std::ostream& os, uint tabLevel = 0) const;

	//--------------------------
	// Reset all input-readings.
	//--------------------------
	virtual  void  ResetInputReadings() = 0;

	//--------------------------
	// Pre/post sampling update.
	//--------------------------
	virtual  void  PreSampleUpdate() = 0;
	virtual  void  PostSampleUpdate() = 0;
	virtual  void  OnBeginAsyncSampleRead() = 0;
	virtual  void  OnEndAsyncSampleRead() = 0;


protected:
	HIDController();

	void  SetParentHID(HumanInterfaceDevice& hid) { parentHID = &hid; }


private:
	HumanInterfaceDevice*  parentHID;
};


//********************************************************************
}//namespace vertexware


