

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\HumanInterfaceDevice.h"
#include "Framework\Plugin.h"
#include "private\vw_common_exports.h"


//********************************************************************
namespace vertexware {


//====================================================================
class VW_COMMON_API HIDDriver : public Plugin
{
public:
	virtual  std::string  GetHIDDriverInterfaceName() = 0;

	//-----------------------------------------------
	// Return default driver priority (0 is highest).
	//-----------------------------------------------
	virtual  uint  GetDefaultDriverPriority() = 0;

	//--------------------------
	// Clear cached information.
	// Exceptions NOT allowed.
	//--------------------------
	virtual  void  ClearCache() = 0;


protected:
	HIDDriver();
};


//====================================================================
class VW_COMMON_API HIDDriverPluginDescriptor : public PluginDescriptor
{
public:
	virtual  Plugin*  CreatePlugin() {
		return CreateHIDDriver();
	}

	virtual  HIDDriver*  CreateHIDDriver() = 0;


protected:
	HIDDriverPluginDescriptor(const PluginInfo& pi);
};


//====================================================================
// HIDDriver plugin access-point declaration.
//====================================================================
#define  VW_DECLARE_PLUGIN_ACCESS_POINT_HID_DRIVER( \
				ClassNamePlugin, StringPluginName, \
				UIntPluginVersionMajor, UIntPluginVersionMinor, \
				UIntPluginVersionRevision, UIntPluginVersionBuildFlags, \
				StringPluginDescription, StringPluginDeveloperInfo) \
\
struct ClassNamePlugin##__PluginDescriptor : public HIDDriverPluginDescriptor \
{ \
	ClassNamePlugin##__PluginDescriptor() \
		: HIDDriverPluginDescriptor(PluginInfo( \
				vertexware::library_info::LibraryVersion(), \
				PluginType_HIDDriver, \
				StringPluginName, \
				vertexware::PluginVersion(UIntPluginVersionMajor, UIntPluginVersionMinor, \
										UIntPluginVersionRevision, UIntPluginVersionBuildFlags), \
				VW_BUILD_TIMESTAMP_STRING, \
				StringPluginDescription, \
				StringPluginDeveloperInfo)) \
	{ } \
\
	virtual  HIDDriver*  CreateHIDDriver() { \
		return new ClassNamePlugin; \
	} \
}; \
VW_DECLARE_PLUGIN_ACCESS_POINT( ClassNamePlugin##__PluginDescriptor )


//********************************************************************
}//namespace vertexware


