

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\HIDDriver.h"
#include "Framework\DirectInputProxy.h"
#include "private\vw_common_exports.h"


//********************************************************************
namespace vertexware {


//====================================================================
// DirectInput-independent device-driver.
//====================================================================
class VW_COMMON_API HIDDriver0 : public HIDDriver
{
public:
	//--------------------------
	// HIDDriver implementation.
	//--------------------------
	virtual  std::string  GetHIDDriverInterfaceName();

	//----------------------------------------------------------
	// If device does not have DirectInput drivers just return
	// false, otherwise return true if device-info matches any
	// of the devices that can be handled through native driver.
	// This prevents generic direct-input drivers from handling
	// the device, while it is already handled by native-driver.
	// Exceptions NOT allowed.
	//----------------------------------------------------------
	virtual  bool  CanHandleDevice(const DirectInputDeviceInstanceInfo& didii) = 0;

	//----------------------------------------------------
	// Probe all devices handled through native-drivers.
	// Return total number of devices that can be created.
	//----------------------------------------------------
	virtual  uint  ProbeDevices() = 0;

	//-------------------------------------------------
	// Create device at given index. If ProbeDevices()
	// returned <n>, this function is called <n> times.
	// Exceptions NOT allowed.
	//-------------------------------------------------
	virtual  HumanInterfaceDevice*  CreateDevice(uint i) = 0;


protected:
	HIDDriver0();
};


//====================================================================
#define  VW_INTERFACE_NAME_HIDDriver0   "HIDDriver0"


//********************************************************************
}//namespace vertexware


