

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\VideoRenderer.h"
#include "Framework\VideoSubsystemOpenGL.h"
#include "GLTools\Extensions.h"
#include "private\vw_opengl_exports.h"
#include <set>


//********************************************************************
namespace vertexware {


//====================================================================
// OpenGL direct-access interface.
//====================================================================
class VW_OPENGL_API VideoRendererOpenGL : virtual public VideoRenderer
{
public:
	virtual ~VideoRendererOpenGL();


	//----------------------------
	// GL implementation features.
	//----------------------------
	const std::string&  GetExtensionsString() { return extensionsString; }

	bool  IsExtensionEnabled(gl_ext::eExtension e);
	void  EnableExtension(gl_ext::eExtension e, bool bEnable);
	void  EnableAllExtensions(bool bEnable);
	void  RefreshExtensionPointers();

	void  DumpDriverInfo(std::ostream& os, uint tabLevel = 1,
				bool bVendor = true, bool bExtensions = true, bool bAdditional = true);

	void  DumpDisabledExtensions(std::ostream& os, uint tabLevel = 1);


	//------------------------------
	// VideoRenderer implementation.
	//------------------------------
	virtual  std::string  GetVideoRendererInterfaceName();
	virtual  std::string  GetMatchingVideoSubsystemInterfaceName();


protected:
	VideoRendererOpenGL();

	VideoSubsystemOpenGL*  GetVideoSubsystemOpenGL() { return pVideoSubsystemOpenGL; }

	//------------------------------
	// VideoRenderer implementation.
	//------------------------------
	virtual  void  ConnectVideoSubsystem(VideoSubsystem& vs);
	virtual  void  DisconnectVideoSubsystem();

	virtual  void  CreateVideoResources(bool bPostReset);
	virtual  void  ReleaseVideoResources(bool bPreReset);


private:
	void  SetupGLVersion(bool bInit);
	void  SetupGLExtensions(bool bInit);
	void  FillExtensionDescList(void* v);


private:
	VideoSubsystemOpenGL*  pVideoSubsystemOpenGL;

	// GL implementation features
	std::string                   extensionsString;
	std::set<gl_ext::eExtension>  disabledExtensions;
};


//====================================================================
#define  VW_INTERFACE_NAME_VideoRendererOpenGL   "VideoRendererOpenGL"


//********************************************************************
}//namespace vertexware


