

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "GLTools\States.h"
#include "private\vw_opengl_exports.h"
#include <string>
#include <gl\gl.h>


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gl_tools {


//====================================================================
// Matrix stacks.
//====================================================================
class VW_OPENGL_API MatrixStack
{
public:
	static std::string __fastcall GetStackName(GLenum stack);

	static uint __fastcall GetMaxStackDepth(GLenum stack);
	static uint __fastcall GetStackDepth(GLenum stack);
	static uint __fastcall GetAvailableStackSpace(GLenum stack);

	__forceinline static GLenum __fastcall GetActiveMatrixMode() {
		return gl_tools::GetInteger(GL_MATRIX_MODE);
	}


public:
	MatrixStack(GLenum _stack) {
		stack = _stack;
	}

	uint  GetMaxDepth() const { return GetMaxStackDepth(stack); }
	uint  GetDepth() const { return GetStackDepth(stack); }
	uint  GetAvailableSpace() const { return GetAvailableStackSpace(stack); }

	void  Activate() { glMatrixMode(stack); }
	void  Push();
	void  Pop();


private:
	GLenum  stack;
};


//====================================================================
// Matrix-stack-sentry provides automatic push/pop.
//====================================================================
class VW_OPENGL_API MatrixStackSentry
{
public:
	MatrixStackSentry(GLenum stack);
	~MatrixStackSentry();

private:
	GLenum  stack;
};


//====================================================================
// Attribute stacks.
//====================================================================
class VW_OPENGL_API AttribStack
{
public:
	__forceinline static uint __fastcall GetMaxStackDepth() {
		return gl_tools::GetInteger(GL_MAX_ATTRIB_STACK_DEPTH);
	}
	__forceinline static uint __fastcall GetStackDepth() {
		return gl_tools::GetInteger(GL_ATTRIB_STACK_DEPTH);
	}
	static uint __fastcall GetAvailableStackSpace();


public:
	uint  GetMaxDepth() const { return GetMaxStackDepth(); }
	uint  GetDepth() const { return GetStackDepth(); }
	uint  GetAvailableSpace() const { return GetAvailableStackSpace(); }

	void  Push(GLbitfield attribMask);
	void  Pop();
};


//====================================================================
// Attribute-stack-sentry provides automatic push/pop.
//====================================================================
class VW_OPENGL_API AttribStackSentry
{
public:
	AttribStackSentry(GLbitfield attribMask);
	~AttribStackSentry();
};


//====================================================================
class VW_OPENGL_API ClientAttribStack
{
public:
	__forceinline static uint __fastcall GetMaxStackDepth() {
		return gl_tools::GetInteger(GL_MAX_CLIENT_ATTRIB_STACK_DEPTH);
	}
	__forceinline static uint __fastcall GetStackDepth() {
		return gl_tools::GetInteger(GL_CLIENT_ATTRIB_STACK_DEPTH);
	}
	static uint __fastcall GetAvailableStackSpace();


public:
	uint  GetMaxDepth() const { return GetMaxStackDepth(); }
	uint  GetDepth() const { return GetStackDepth(); }
	uint  GetAvailableSpace() const { return GetAvailableStackSpace(); }

	void  Push(GLbitfield clientAttribMask);
	void  Pop();
};


//====================================================================
// Client-attribute-stack-sentry provides automatic push/pop.
//====================================================================
class VW_OPENGL_API ClientAttribStackSentry
{
public:
	ClientAttribStackSentry(GLbitfield clientAttribMask);
	~ClientAttribStackSentry();
};


//====================================================================
// Debug-mode dump of stack depths.
//====================================================================
VW_OPENGL_API
void __fastcall DumpAllStacks(std::ostream& os);


//********************************************************************
}//namespace gl_tools


//********************************************************************
}//namespace vertexware


