

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "private\vw_opengl_exports.h"
#include <string>
#include <gl\gl.h>


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gl_tools {


//====================================================================
// Single value queries.
//====================================================================
VW_OPENGL_API __forceinline
GLboolean __fastcall GetBoolean(GLenum pname)
{
	GLboolean param;
	glGetBooleanv(pname, &param);
	return param;
}


//====================================================================
VW_OPENGL_API __forceinline
GLdouble __fastcall GetDouble(GLenum pname)
{
	GLdouble param;
	glGetDoublev(pname, &param);
	return param;
}


//====================================================================
VW_OPENGL_API __forceinline
GLfloat __fastcall GetFloat(GLenum pname)
{
	GLfloat param;
	glGetFloatv(pname, &param);
	return param;
}


//====================================================================
VW_OPENGL_API __forceinline
GLint __fastcall GetInteger(GLenum pname)
{
	GLint param;
	glGetIntegerv(pname, &param);
	return param;
}


//====================================================================
VW_OPENGL_API __forceinline
GLvoid* __fastcall GetPointer(GLenum pname)
{
	GLvoid* param;
	glGetPointerv(pname, &param);
	return param;
}


//====================================================================
// Enable / disable wrappers.
//====================================================================
VW_OPENGL_API __forceinline
void __fastcall Enable(GLenum cap, bool bEnable)
{
	bEnable ? glEnable(cap) : glDisable(cap);
}


//====================================================================
VW_OPENGL_API __forceinline
void __fastcall EnableClientState(GLenum array, bool bEnable)
{
	bEnable ? glEnableClientState(array) : glDisableClientState(array);
}


//====================================================================
// PixelStore wrappers.
//====================================================================
VW_OPENGL_API
void __fastcall SetupPixelPack(GLboolean bSwapBytes = GL_FALSE,
								uint rowAlignmentInBytes = 1,
								uint rowLengthInPixels = 0,
								uint imageHeightInPixels = 0);


//====================================================================
class VW_OPENGL_API PixelPackSentry
{
public:
	PixelPackSentry(GLboolean bSwapBytes = GL_FALSE,
					uint rowAlignmentInBytes = 1,
					uint rowLengthInPixels = 0,
					uint imageHeightInPixels = 0);
	~PixelPackSentry();


private:
	GLboolean  swapBytes;
	uint       rowAlignment;
	uint       rowLength;
	uint       imageHeight;
};


//====================================================================
VW_OPENGL_API
void __fastcall SetupPixelUnpack(GLboolean bSwapBytes = GL_FALSE,
								uint rowAlignmentInBytes = 1,
								uint rowLengthInPixels = 0,
								uint imageHeightInPixels = 0);


//====================================================================
class VW_OPENGL_API PixelUnpackSentry
{
public:
	PixelUnpackSentry(GLboolean bSwapBytes = GL_FALSE,
						uint rowAlignmentInBytes = 1,
						uint rowLengthInPixels = 0,
						uint imageHeightInPixels = 0);
	~PixelUnpackSentry();


private:
	GLboolean  swapBytes;
	uint       rowAlignment;
	uint       rowLength;
	uint       imageHeight;
};


//====================================================================
class VW_OPENGL_API ReadBufferSentry
{
public:
	ReadBufferSentry(GLenum newReadBufferMode = GL_BACK);
	~ReadBufferSentry();


private:
	GLint  lastReadBufferMode;
};


//====================================================================
class VW_OPENGL_API DrawBufferSentry
{
public:
	DrawBufferSentry(GLenum newDrawBufferMode = GL_BACK);
	~DrawBufferSentry();


private:
	GLint  lastDrawBufferMode;
};


//********************************************************************
}//namespace gl_tools


//********************************************************************
}//namespace vertexware


