

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\Dialog.h"
#include "GUI_CS0\LogoSpinners.h"
#include "GUI_CS0\ShaderNames.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
class VW_GUI_CS0_API AboutDialog : public gui_cs0::Dialog
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		vec2   logoSize;
		float  infoTextHeight;
		float  infoLineHeight;
		float  libraryInfoTextHeight;

		GeomParams();
	};


public:
	//-----------------------
	// AboutDialog interface.
	//-----------------------
	AboutDialog(const std::string& title = "About");

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() { return geomParams; }

	void  SetAboutDialogShaderNames(
						const std::string& logoSpinner = DefaultShaderName(Shader_LogoSpinner),
						const std::string& labelInfo = DefaultShaderName(Shader_Label_Static),
						const std::string& labelLibraryInfo = DefaultShaderName(Shader_Label_Link));

	void  SetAboutDialogSubcomponentLayout(
						bool bShowLogo = true);

	void  SetAboutDialogSubcomponentText(
						const std::string& sInfoText = "Info:\ncopyright and legal here",
						const gui::TileParams& tileParamsLogo = gui::std_tile_textures::MakeTileParams(gui::std_tile_textures::Tile_Stop));

	void  SetAboutDialogSubcomponentToolTipText(
						const std::string& sLogo = "your logo here");


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  bool  OnCommand(uint commandID);

	//------------------
	// Dialog overrides.
	//------------------
	virtual  void  OnStdDialogControlsLayouterUpdate();
	virtual  void  OnDialogOpened() {}
	virtual  void  OnDialogClosed() {}
	virtual  void  OnApplySettings() {}

	//-------------------------
	// Derived-class interface.
	//-------------------------
	enum {
		CID_LibraryInfo = gui::CID_User,
		CID_Info,
		CID_Logo
	};

	virtual  void  OnCommandLibraryInfo();
	virtual  void  OnCommandInfo() {}
	virtual  void  OnCommandLogo() {}


private:
	//----------------
	// Sub-components.
	//----------------
	gui::layouters::RowLayouter*  pLayouter;

	Label*        pLabelLibraryInfo;
	Label*        pLabelInfo;
	LogoSpinner*  pLogoSpinner;

	//------------
	// Properties.
	//------------
	GeomParams  geomParams;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


