

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Component.h"
#include "GUI\GUI_Shader.h"
#include "private\vw_gui_cs0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
// Feature support:
// CanDisable. CanFocus.
//====================================================================
class VW_GUI_CS0_API EditBox : public gui::Component, public gui::CommandSender
{
	VW_GUI_COMPONENT_DECLARE_SHADER_DLL(VW_GUI_CS0_API, Shader);
	VW_GUI_COMPONENT_DECLARE_CHECK_SHADER(Shader);


public:
	//----------------------
	// Text change observer.
	//----------------------
	class VW_GUI_CS0_API Observer
	{
	public:
		virtual ~Observer() {}

		// check <newText>, return false to reject change
		virtual  bool  ConfirmTextChange(const std::string& lastText, const std::string& newText)
		{ return true; }

		virtual  void  OnTextChanged(const std::string& newText) {}
	};


public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		vec2   preferredSize;
		float  textPadding;
		float  textHeight;

		GeomParams();
	};


public:
	//-------------------
	// EditBox interface.
	//-------------------
	EditBox(gui::Component* parent = 0,
			gui::MessageHandler* commandTarget = 0,
			uint commandID = 0);

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void                SetText(const std::string& text = "");
	const std::string&  GetText() const { return text; }

	std::string  ExtractTextBeforeCaret() const;
	std::string  ExtractTextAfterCaret() const;

	void  SetCaretPosition(uint pos);
	uint  GetCaretPosition() const { return caretPos; }

	void  SetMaxChars(uint maxChars = 0); //0 means unlimited
	uint  GetMaxChars() const { return maxChars; }

	void  SetShiftFocusToAncestor(bool bShiftOnEnter = true, bool bShiftOnTab = true);
	void  GetShiftFocusToAncestor(bool& bShiftOnEnter, bool& bShiftOnTab) const;

	void  SetSendCommand(bool bSendOnEnterRepeat = false, bool bSendOnKeyboardFocusLost = true);
	void  GetSendCommand(bool& bSendOnEnterRepeat, bool& bSendOnKeyboardFocusLost) const;

	void  SetResetCaretPositionOnKeyboardFocusLost(bool bReset = true) { bResetCaretPositionOnKeyboardFocusLost = bReset; }
	bool  GetResetCaretPositionOnKeyboardFocusLost() const { return bResetCaretPositionOnKeyboardFocusLost; }

	void                SetObserver(EditBox::Observer* observer = 0);
	EditBox::Observer*  GetObserver() const { return pObserver; }

	//-----------------------------------------------
	// Additional methods intended for use by shader.
	//-----------------------------------------------
	float  GetCaretDrawPosition() const { return fCaretDrawPos; }
	void   GetVisibleTextInfo(uint& firstVisible, uint& maxVisible) const;


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  RecalcSize();
	virtual  void  UpdateGeometryCache();
	virtual  void  PostCacheShaderHandles();
	virtual  bool  OnMouseButton(bool bDown, eMouseButton buttonCode, bool bDoubleClick, uint repeatCount, const vec2& mousePos);
	virtual  bool  OnKeyboardQueryFocus(bool bFocusByMouse, eMouseButton buttonCode, bool& bWantFocus, bool& bNeedCharacterData);
	virtual  bool  OnKeyboardFocus(bool bGotFocus);
	virtual  bool  OnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch);

	//------------------
	// Internal helpers.
	//------------------
	void  SetCaretPosition(const vec2& mousePos);
	void  UpdateTextAndCaretDrawCache();
	bool  ConfirmTextChange(const std::string& lastText, const std::string& newText);
	void  NotifyTextChanged(const std::string& newText);


protected:
	//------------
	// Properties.
	//------------
	GeomParams  geomParams;
	uint        maxChars;
	bool        bSendCommandOnEnterRepeat;
	bool        bSendCommandOnKeyboardFocusLost;
	bool        bShiftFocusToAncestorOnEnter;
	bool        bShiftFocusToAncestorOnTab;
	bool        bResetCaretPositionOnKeyboardFocusLost;

	//--------
	// States.
	//--------
	EditBox::Observer*  pObserver;
	std::string         text;
	bool                bChangedSinceLastConfirm;
	uint                caretPos;
	uint                firstVisible;
	uint                maxVisible;
	std::vector<float>  textWidths;

	//-------
	// Cache.
	//-------
	const EditBox::Shader*  pEditBoxShader;
	float                   fMaxVisibleTextWidth;
	float                   fCaretDrawPos;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


