

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Component.h"
#include "private\vw_gui_cs0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
// Feature support:
// CanFocus.
//====================================================================
class VW_GUI_CS0_API FontTest : public gui::Component
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		float  cellSize; //valid range [8, 48]

		GeomParams();
	};

	//------------------------------
	struct VW_GUI_CS0_API DrawParams
	{
		color4b  colorBackground;
		color4b  colorGrid;
		color4b  colorBoxMonospace;
		color4b  colorBoxVarPitch;
		color4b  colorFontGlyph;

		DrawParams();
	};

	//---------------------------------------------------
	// Font-texture box display mode, useful for tweaking
	// font-texture generation parameters.
	//---------------------------------------------------
	enum eBoxDisplayMode
	{
		BoxDisplayMode_Hide = 0,
		BoxDisplayMode_Monospace,
		BoxDisplayMode_VarPitch,

		BoxDisplayMode_Count,
		BoxDisplayMode_Force32Bits = 0x7FFFFFFF
	};


public:
	//--------------------
	// FontTest interface.
	//--------------------
	FontTest();

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void               SetDrawParams(const DrawParams& dp = DrawParams());
	const DrawParams&  GetDrawParams() const { return drawParams; }

	void             SetBoxDisplayMode(eBoxDisplayMode mode = BoxDisplayMode_Hide);
	eBoxDisplayMode  GetBoxDisplayMode() const { return boxDisplayMode; }

	void  SetDrawGrid(bool bDrawGrid = true);
	bool  GetDrawGrid() const { return bDrawGrid; }

	void  ShowNextFont();


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  UpdateGeometryCache();
	virtual  void  DrawSelf(gui::Renderer& gr);
	virtual  void  OnFontListChanged(gui::Manager& gm);
	virtual  bool  CacheFontHandles(gui::Manager& gm);
	virtual  bool  OnMouseButton(bool bDown, eMouseButton buttonCode, bool bDoubleClick, uint repeatCount, const vec2& mousePos);
	virtual  bool  OnKeyboardQueryFocus(bool bFocusByMouse, eMouseButton buttonCode, bool& bWantFocus, bool& bNeedCharacterData);
	virtual  bool  OnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch);
	virtual  bool  OnQueryToolTipParams(gui::ToolTipParams& ttp);

	//------------------
	// Internal helpers.
	//------------------
	std::string  GetNextRegisteredFontName();
	void         SetupToolTip();
	void         ResetToolTip();


protected:
	//------------
	// Properties.
	//------------
	GeomParams  geomParams;
	DrawParams  drawParams;

	//--------
	// States.
	//--------
	eBoxDisplayMode  boxDisplayMode;
	bool             bDrawGrid;

	//-------
	// Cache.
	//-------
	std::string              activeFontName;
	const gui::Font*         pFont;
	const gui::FontTexture*  pFontTexture;
	vec2                     glyphSize;
	gui::RectangleC          rBackground;
	gui::RectangleC          rGridCell;
	gui::RectangleC          rBoxMono;
	gui::RectangleC          rBoxVar;
	gui::RectangleCT         rGlyphCell;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


