

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\Dialog.h"
#include "Framework\DirectInputProxy.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
class VW_GUI_CS0_API HIDConfigDialog : public gui_cs0::Dialog, public ReconfigObserver
{
public:
	//---------------------------
	// HIDConfigDialog interface.
	//---------------------------
	HIDConfigDialog(const std::string& title = "Input Configuration",
					uint hidReconfigCode = 0);

	void  SetHIDConfigDialogShaderNames(
				const std::string& label = DefaultShaderName(Shader_Label_Static),
				const std::string& labelCB = DefaultShaderName(Shader_Label_Active),
				const std::string& checkBox = DefaultShaderName(Shader_CheckBox),
				const std::string& comboBox = DefaultShaderName(Shader_ComboBox),
				const std::string& horzTextSeparator = DefaultShaderName(Shader_HorzTextSeparator));

	void  SetHIDConfigDialogSubcomponentToolTipText(
				const std::string& sEnableDInput = "Allow input subsystem\nto use DirectInput",
				const std::string& sLastVersion = "Use latest supported\nversion of DirectInput",
				const std::string& sExclusiveMode = "Allow devices to use DirectInput\nexclusive mode (fullscreen only)",
				const std::string& sUseAtInit = "Allow devices to use DirectInput\nimmediately after initialization",
				const std::string& sSelectVersion = "Select DirectInput version");

	// code used when calling Kernel::RequestHIDReconfig()
	void  SetHIDReconfigCode(uint rc = 0) { iHIDReconfigCode = rc; }


protected:
	//----------------------------
	// ReconfigObserver overrides.
	//----------------------------
	virtual  void  OnHIDReconfig();

	//---------------------
	// Component overrides.
	//---------------------
	virtual  bool  OnCommand(uint commandID);

	//------------------
	// Dialog overrides.
	//------------------
	virtual  void  OnDialogOpened();
	virtual  void  OnDialogClosed() {}
	virtual  void  OnApplySettings();

	//---------------------
	// Command identifiers.
	//---------------------
	enum {
		CID_DIEnable = gui::CID_User,
		CID_DILastVersion,
		CID_DIVersion,
		CID_DIExclusiveMode,
		CID_DIUseAtInit
	};

	//------------------
	// Internal helpers.
	//------------------
	void  MatchActiveConfig();
	void  UpdateControlDependencies();

	//-------------------------
	// Derived-class interface.
	//-------------------------
	virtual  void  OnPreApplySettings(DirectInputConfig& diConfig) {}


protected:
	//------------
	// Properties.
	//------------
	uint  iHIDReconfigCode;

	//----------------
	// Sub-components.
	//----------------
	gui::layouters::RowLayouter*  pLayouter;

	HorzTextSeparator*  pHorzTextSeparatorDInput;
	Label*              pLabelEnableDInput;
	Label*              pLabelLastVersion;
	Label*              pLabelExclusiveMode;
	Label*              pLabelUseAtInit;
	Label*              pLabelVersion;
	CheckBox*           pCheckBoxEnableDInput;
	CheckBox*           pCheckBoxLastVersion;
	CheckBox*           pCheckBoxExclusiveMode;
	CheckBox*           pCheckBoxUseAtInit;
	ComboBox*           pComboBoxVersion;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


