

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Layouters.h"
#include "GUI_CS0\Buttons.h"
#include "GUI_CS0\Label.h"
#include "GUI_CS0\ShaderNames.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
// Feature support:
// CanFocus.
//====================================================================
class VW_GUI_CS0_API PixelOffsetControlPane : public gui::Component
{
	VW_GUI_COMPONENT_DECLARE_SHADER_DLL(VW_GUI_CS0_API, Shader);
	VW_GUI_COMPONENT_DECLARE_CHECK_SHADER(Shader);


public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		float  screenBorderWidth;
		vec2   controlAreaSize;
		float  iconButtonSize;
		float  iconButtonTileSize;
		float  labelTextHeight;

		GeomParams();
	};


public:
	//----------------------------------
	// PixelOffsetControlPane interface.
	//----------------------------------
	PixelOffsetControlPane();

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void   SetPixelOffsetGranularity(float pog = 0.125f); //[0, 0.5]
	float  GetPixelOffsetGranularity() const { return pixelOffsetGranularity; }

	void  SetPOCPShaderNames(
				const std::string& pane = DefaultShaderName(Shader_PixelOffsetControlPane),
				const std::string& iconButtons = DefaultShaderName(Shader_PixelOffsetControlPane_IconButton),
				const std::string& labels = DefaultShaderName(Shader_PixelOffsetControlPane_Label));

	void   SetPOCPSubcomponentToolTipText(
				const std::string& buttonClose = "Close",
				const std::string& buttonArrows = "Adjust pixel-offset",
				const std::string& labelOffsetX = "Pixel-offset-x",
				const std::string& labelOffsetY = "Pixel-offset-y");

	void   SetPOCPSubcomponentTileParams(
				const gui::TileParams& buttonClose = gui::std_tile_textures::MakeTileParams(gui::std_tile_textures::Tile_X),
				const gui::TileParams& buttonLeft = gui::std_tile_textures::MakeTileParams(gui::std_tile_textures::Tile_ArrowLeft),
				const gui::TileParams& buttonRight = gui::std_tile_textures::MakeTileParams(gui::std_tile_textures::Tile_ArrowRight),
				const gui::TileParams& buttonUp = gui::std_tile_textures::MakeTileParams(gui::std_tile_textures::Tile_ArrowUp),
				const gui::TileParams& buttonDown = gui::std_tile_textures::MakeTileParams(gui::std_tile_textures::Tile_ArrowDown));

	//-----------------------------------------------
	// Additional methods intended for use by shader.
	//-----------------------------------------------
	void  GetDrawRectangles(gui::Rectangle& rControlArea,
							gui::Rectangle& rBorderLeft,
							gui::Rectangle& rBorderRight,
							gui::Rectangle& rBorderTop,
							gui::Rectangle& rBorderBottom) const;


protected:
	//-----------------------------
	// VideoResourceUser overrides.
	//-----------------------------
	virtual  void  OnVideoRendererChanged(VideoRenderer* vr);

	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  RecalcLayout();
	virtual  void  UpdateGeometryCache();
	virtual  void  OnWindowResize(uint width, uint height);
	virtual  bool  HitTestSelf(const vec2& testPoint) const;
	virtual  bool  OnCommand(uint commandID);
	virtual  bool  OnKeyboardQueryFocus(bool bFocusByMouse, eMouseButton buttonCode, bool& bWantFocus, bool& bNeedCharacterData);
	virtual  bool  OnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch);

	//------------------
	// Internal helpers.
	//------------------
	void  SetupRendererPixelOffset(float dx, float dy);
	void  UpdatePixelOffsetText();


protected:
	//----------------
	// Sub-components.
	//----------------
	gui::layouters::RowLayouter*  pLayouter;

	IconButton*  pButtonClose;
	IconButton*  pButtonLeft;
	IconButton*  pButtonRight;
	IconButton*  pButtonUp;
	IconButton*  pButtonDown;
	Label*       pLabelPixelOffsetX;
	Label*       pLabelPixelOffsetY;

	//------------
	// Properties.
	//------------
	GeomParams  geomParams;
	float       pixelOffsetGranularity;

	//-------
	// Cache.
	//-------
	vec2            controlAreaOffset;
	vec2            buttonOffset;
	gui::Rectangle  rControlArea;
	gui::Rectangle  rBorderLeft;
	gui::Rectangle  rBorderRight;
	gui::Rectangle  rBorderTop;
	gui::Rectangle  rBorderBottom;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


