

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\Buttons.h"
#include "GUI_CS0\ShaderNames.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
// Feature support:
// CanFocus.
//
// Internal hierarchy:
// * TopLevelPane
//   * IconButton (Close)
//====================================================================
class VW_GUI_CS0_API TopLevelPane : public gui::Component
{
	VW_GUI_COMPONENT_DECLARE_SHADER_DLL(VW_GUI_CS0_API, Shader);
	VW_GUI_COMPONENT_DECLARE_CHECK_SHADER(Shader);


public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		float  titleTextHeight;
		vec2   titleOffsetTopLeft;
		vec2   buttonOffsetTopRight;
		float  iconButtonPaneControlSize;
		float  iconButtonPaneControlTileSize;

		GeomParams();
	};

	//------------------------------------------------------------
	static gui::TextAttributes __fastcall DefaultTextAttributes();


public:
	//------------------------
	// TopLevelPane interface.
	//------------------------
	TopLevelPane(const std::string& title = "TopLevelPane");

	// default is empty font-name (not used)
	void                        SetTextAttributes(const gui::TextAttributes& ta = gui::TextAttributes());
	const gui::TextAttributes&  GetTextAttributes() const { return textAtts; }

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void                SetTitleText(const std::string& text = "TopLevelPane") { titleText = text; }
	const std::string&  GetTitleText() const { return titleText; }

	bool  IsTitleVisible() const { return bTitleVisible; }

	void  SetTLPShaderNames(
				const std::string& topLevelPane = DefaultShaderName(Shader_TopLevelPane),
				const std::string& iconButtonPaneControl = DefaultShaderName(Shader_TopLevelPane_IconButton_PaneControl));

	void  SetTLPSubcomponentLayout(
				bool bShowTitle = true,
				bool bHideTitleOnMouseExit = false,
				bool bShowCloseButton = true,
				bool bHideCloseButtonOnMouseExit = true);

	void  SetTLPSubcomponentToolTipText(
				const std::string& buttonClose = "Close");

	void  SetTLPSubcomponentTileParams(
				const gui::TileParams& buttonClose = gui::std_tile_textures::MakeTileParams(gui::std_tile_textures::Tile_X));

	void  SetClosePaneOnKey(bool bCloseOnKey = true, eKey kc = Key_Escape);

	// area not used by title-text & control-buttons
	void  CalcTLPMainArea(float offsetY, float sizeY) const;


protected:
	//-----------------------------
	// VideoResourceUser overrides.
	//-----------------------------
	virtual  bool  CacheFontHandles(gui::Manager& gm);

	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  RecalcLayout();
	virtual  gui::eCommandArea  HitTestCommandAreaSelf(const vec2& testPoint) const { return gui::CommandArea_Move; }
	virtual  void  OnChangedVisible();
	virtual  bool  OnCommand(uint commandID);
	virtual  bool  OnDeactivate(bool bDeactivateByMouse);
	virtual  bool  OnMouseEnterExit(bool bEnter, bool bMouseOverDirect, const vec2& mousePos);
	virtual  bool  OnMouseButton(bool bDown, eMouseButton buttonCode, bool bDoubleClick, uint repeatCount, const vec2& mousePos);
	virtual  bool  OnKeyboardQueryFocus(bool bFocusByMouse, eMouseButton buttonCode, bool& bWantFocus, bool& bNeedCharacterData);
	virtual  bool  OnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch);

	//------------------
	// Internal helpers.
	//------------------
	const gui::Font*  GetFont() const { return pFont; }

	//--------------------------------
	// Derived-class template-methods.
	//--------------------------------
	virtual  void  OnPaneDock(); //called after LMB double-click, default calls window resize
	virtual  void  OnPaneOpened() {}
	virtual  void  OnPaneClosed() {}


protected:
	//----------------
	// Sub-components.
	//----------------
	IconButton*  pButtonClose;

	//------------
	// Properties.
	//------------
	GeomParams           geomParams;
	gui::TextAttributes  textAtts;
	std::string          titleText;
	bool                 bShowTitle;
	bool                 bHideTitleOnMouseExit;
	bool                 bShowCloseButton;
	bool                 bHideCloseButtonOnMouseExit;
	bool                 bClosePaneOnKey;
	eKey             keyCodeClose;

	//--------
	// States.
	//--------
	bool  bTitleVisible;

	//-------
	// Cache.
	//-------
	const gui::Font*  pFont;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


