

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\Buttons.h"
#include "private\vw_gui_ss0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
// Parameter sets.
//====================================================================
struct VW_GUI_SS0_API ShaderParams_Button_PushOpaque
{
	// <colorPressedGlyph> is used only if button-style is Style_Press.
	// <pressOffset> is used only if button-style is Style_Push.
	color4b  colorDisabledBorder;
	color4b  colorDisabledGlyph;
	color4b  colorDisabledBgr;
	color4b  colorEnabledBorder;
	color4b  colorEnabledGlyph;
	color4b  colorEnabledBgrTop;
	color4b  colorEnabledBgrBottom;
	color4b  colorPressedBorder;
	color4b  colorPressedBgrTop;
	color4b  colorPressedBgrBottom;
	color4b  colorPressedGlyph;
	color4b  colorHotBorder;
	vec2     pressOffset;
	float    mainBorderWidth;
	float    hotBorderWidth;

	ShaderParams_Button_PushOpaque();

	void  ChooseParams(const gui::Button& button,
					color4b& cTop, color4b& cBottom,
					color4b& cGlyph, color4b& cMainBorder,
					vec2& pressOffset) const;
};


//====================================================================
struct VW_GUI_SS0_API ShaderParams_Button_PushTranslucent
{
	color4b  colorDisabled;
	color4b  colorEnabled;
	color4b  colorHot;
	ubyte    alphaPressedBgr;
	ubyte    alphaHotBgr;

	ShaderParams_Button_PushTranslucent();

	void  ChooseParams(const gui::Button& button,
						color4b& cGlyph, color4b& cBgr, bool& bDrawBgr) const;
};


//====================================================================
struct VW_GUI_SS0_API ShaderParams_Button_ToggleOpaque
{
	color4b  colorDisabled;
	color4b  colorDisabledToggled;
	color4b  colorDisabledBgr;
	color4b  colorDisabledBgrToggled;
	color4b  colorEnabledBorder;
	color4b  colorUntoggledGlyph;
	color4b  colorUntoggledPressedGlyph;
	color4b  colorToggledGlyph;
	color4b  colorToggledPressedGlyph;
	color4b  colorUntoggledBgrTop;
	color4b  colorUntoggledBgrBottom;
	color4b  colorUntoggledPressedBgrTop;
	color4b  colorUntoggledPressedBgrBottom;
	color4b  colorToggledBgrTop;
	color4b  colorToggledBgrBottom;
	color4b  colorToggledPressedBgrTop;
	color4b  colorToggledPressedBgrBottom;
	color4b  colorHotBorder;
	float    mainBorderWidthUntoggled;
	float    mainBorderWidthUntoggledPressed;
	float    mainBorderWidthToggled;
	float    mainBorderWidthToggledPressed;
	float    hotBorderWidth;

	ShaderParams_Button_ToggleOpaque();

	void  ChooseParams(const gui::Button& button,
					color4b& cTop, color4b& cBottom,
					color4b& cGlyph, color4b& cMainBorder,
					float& mainBorderWidth) const;
};


//====================================================================
struct VW_GUI_SS0_API ShaderParams_Button_ToggleTranslucent
{
	color4b  colorDisabled;
	color4b  colorDisabledToggled;
	color4b  colorUntoggled;
	color4b  colorUntoggledPressed;
	color4b  colorToggled;
	color4b  colorToggledPressed;
	color4b  colorBgr;
	ubyte    alphaPressedBgr;
	ubyte    alphaHotBgr;

	ShaderParams_Button_ToggleTranslucent();

	void  ChooseParams(const gui::Button& button,
						color4b& cGlyph, color4b& cBgr, bool& bDrawBgr) const;
};


//====================================================================
// Intended for button-styles: Style_Push, Style_Press
//====================================================================
class VW_GUI_SS0_API CS0_TextButtonShader_PushOpaque : public gui_cs0::TextButton::Shader
{
public:
	//-------------------------------------------
	// CS0_TextButtonShader_PushOpaque interface.
	//-------------------------------------------
	CS0_TextButtonShader_PushOpaque(const std::string& shaderName = "",
									const ShaderParams_Button_PushOpaque& sp = ShaderParams_Button_PushOpaque(),
									const gui::TextAttributes& ta = DefaultTextAttributes());

	void                                   SetShaderParams(const ShaderParams_Button_PushOpaque& sp = ShaderParams_Button_PushOpaque()) { shaderParams = sp; }
	const ShaderParams_Button_PushOpaque&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams_Button_PushOpaque  shaderParams;
};


//====================================================================
// Intended for button-styles: Style_OnOff, Style_Select
//====================================================================
class VW_GUI_SS0_API CS0_TextButtonShader_ToggleOpaque : public gui_cs0::TextButton::Shader
{
public:
	//---------------------------------------------
	// CS0_TextButtonShader_ToggleOpaque interface.
	//---------------------------------------------
	CS0_TextButtonShader_ToggleOpaque(const std::string& shaderName = "",
									const ShaderParams_Button_ToggleOpaque& sp = ShaderParams_Button_ToggleOpaque(),
									const gui::TextAttributes& ta = DefaultTextAttributes());

	void                                     SetShaderParams(const ShaderParams_Button_ToggleOpaque& sp = ShaderParams_Button_ToggleOpaque()) { shaderParams = sp; }
	const ShaderParams_Button_ToggleOpaque&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams_Button_ToggleOpaque  shaderParams;
};


//====================================================================
class VW_GUI_SS0_API CS0_TextButtonShader_Translucent : public gui_cs0::TextButton::Shader
{
protected:
	CS0_TextButtonShader_Translucent(const std::string& shaderName = "",
								const gui::TextAttributes& ta = DefaultTextAttributes());

	void  DrawTextButton(const gui_cs0::TextButton& textButton, gui::Renderer& gr,
						color4b colorGlyph, color4b colorBgr, bool bDrawBgr) const;
};


//====================================================================
// Intended for button-styles: Style_Push, Style_Press
//====================================================================
class VW_GUI_SS0_API CS0_TextButtonShader_PushTranslucent : public CS0_TextButtonShader_Translucent
{
public:
	//------------------------------------------------
	// CS0_TextButtonShader_PushTranslucent interface.
	//------------------------------------------------
	CS0_TextButtonShader_PushTranslucent(const std::string& shaderName = "",
									const ShaderParams_Button_PushTranslucent& sp = ShaderParams_Button_PushTranslucent(),
									const gui::TextAttributes& ta = DefaultTextAttributes());

	void                                        SetShaderParams(const ShaderParams_Button_PushTranslucent& sp = ShaderParams_Button_PushTranslucent()) { shaderParams = sp; }
	const ShaderParams_Button_PushTranslucent&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams_Button_PushTranslucent  shaderParams;
};


//====================================================================
// Intended for button-styles: Style_OnOff, Style_Select
//====================================================================
class VW_GUI_SS0_API CS0_TextButtonShader_ToggleTranslucent : public CS0_TextButtonShader_Translucent
{
public:
	//--------------------------------------------------
	// CS0_TextButtonShader_ToggleTranslucent interface.
	//--------------------------------------------------
	CS0_TextButtonShader_ToggleTranslucent(const std::string& shaderName = "",
									const ShaderParams_Button_ToggleTranslucent& sp = ShaderParams_Button_ToggleTranslucent(),
									const gui::TextAttributes& ta = DefaultTextAttributes());

	void                                          SetShaderParams(const ShaderParams_Button_ToggleTranslucent& sp = ShaderParams_Button_ToggleTranslucent()) { shaderParams = sp; }
	const ShaderParams_Button_ToggleTranslucent&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams_Button_ToggleTranslucent  shaderParams;
};


//====================================================================
// Intended for button-styles: Style_Push, Style_Press
//====================================================================
class VW_GUI_SS0_API CS0_IconButtonShader_PushOpaque : public gui_cs0::IconButton::Shader
{
public:
	//-------------------------------------------
	// CS0_IconButtonShader_PushOpaque interface.
	//-------------------------------------------
	CS0_IconButtonShader_PushOpaque(const std::string& shaderName = "",
								const ShaderParams_Button_PushOpaque& sp = ShaderParams_Button_PushOpaque());

	void                                   SetShaderParams(const ShaderParams_Button_PushOpaque& sp = ShaderParams_Button_PushOpaque()) { shaderParams = sp; }
	const ShaderParams_Button_PushOpaque&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams_Button_PushOpaque  shaderParams;
};


//====================================================================
// Intended for button-styles: Style_OnOff, Style_Select
//====================================================================
class VW_GUI_SS0_API CS0_IconButtonShader_ToggleOpaque : public gui_cs0::IconButton::Shader
{
public:
	//---------------------------------------------
	// CS0_IconButtonShader_ToggleOpaque interface.
	//---------------------------------------------
	CS0_IconButtonShader_ToggleOpaque(const std::string& shaderName = "",
									const ShaderParams_Button_ToggleOpaque& sp = ShaderParams_Button_ToggleOpaque());

	void                                     SetShaderParams(const ShaderParams_Button_ToggleOpaque& sp = ShaderParams_Button_ToggleOpaque()) { shaderParams = sp; }
	const ShaderParams_Button_ToggleOpaque&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams_Button_ToggleOpaque  shaderParams;
};


//====================================================================
class VW_GUI_SS0_API CS0_IconButtonShader_Translucent : public gui_cs0::IconButton::Shader
{
protected:
	CS0_IconButtonShader_Translucent(const std::string& shaderName = "");

	static void __fastcall DrawIconButton(const gui_cs0::IconButton& iconButton,
										gui::Renderer& gr,
										color4b colorGlyph, color4b colorBgr, bool bDrawBgr);
};


//====================================================================
// Intended for button-styles: Style_Push, Style_Press
//====================================================================
class VW_GUI_SS0_API CS0_IconButtonShader_PushTranslucent : public CS0_IconButtonShader_Translucent
{
public:
	//------------------------------------------------
	// CS0_IconButtonShader_PushTranslucent interface.
	//------------------------------------------------
	CS0_IconButtonShader_PushTranslucent(const std::string& shaderName = "",
									const ShaderParams_Button_PushTranslucent& sp = ShaderParams_Button_PushTranslucent());

	void                                        SetShaderParams(const ShaderParams_Button_PushTranslucent& sp = ShaderParams_Button_PushTranslucent()) { shaderParams = sp; }
	const ShaderParams_Button_PushTranslucent&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams_Button_PushTranslucent  shaderParams;
};


//====================================================================
// Intended for button-styles: Style_OnOff, Style_Select
//====================================================================
class VW_GUI_SS0_API CS0_IconButtonShader_ToggleTranslucent : public CS0_IconButtonShader_Translucent
{
public:
	//--------------------------------------------------
	// CS0_IconButtonShader_ToggleTranslucent interface.
	//--------------------------------------------------
	CS0_IconButtonShader_ToggleTranslucent(const std::string& shaderName = "",
									const ShaderParams_Button_ToggleTranslucent& sp = ShaderParams_Button_ToggleTranslucent());

	void                                          SetShaderParams(const ShaderParams_Button_ToggleTranslucent& sp = ShaderParams_Button_ToggleTranslucent()) { shaderParams = sp; }
	const ShaderParams_Button_ToggleTranslucent&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams_Button_ToggleTranslucent  shaderParams;
};


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


