

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\Menu.h"
#include "GUI_SS0\Buttons.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
class VW_GUI_SS0_API CS0_MenuBarShader : public gui_cs0::MenuBar::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ShaderParams
	{
		color4b  colorBgrTop;
		color4b  colorBgrBottom;
		color4b  colorBorder;
		color4b  colorBgrTopActive;
		color4b  colorBgrBottomActive;
		color4b  colorBorderActive;
		float    borderWidth;
		color4b  colorBorderMoving;
		float    borderWidthMoving;
		color4b  colorBorderMovingInDockArea;
		float    borderWidthMovingInDockArea;

		ShaderParams();
	};


public:
	//-----------------------------
	// CS0_MenuBarShader interface.
	//-----------------------------
	CS0_MenuBarShader(const std::string& shaderName = "",
						const ShaderParams& sp = ShaderParams());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams()) { shaderParams = sp; }
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;
};


//====================================================================
class VW_GUI_SS0_API CS0_MenuTitleShader : public gui_cs0::MenuTitle::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ShaderParams
	{
		color4b  colorDisabled;
		color4b  colorEnabled;
		color4b  colorActive;
		color4b  colorHot;

		ShaderParams();
	};


public:
	//-------------------------------
	// CS0_MenuTitleShader interface.
	//-------------------------------
	CS0_MenuTitleShader(const std::string& shaderName = "",
						const ShaderParams& sp = ShaderParams(),
						const gui::TextAttributes& ta = DefaultTextAttributes());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams()) { shaderParams = sp; }
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;
};


//====================================================================
class VW_GUI_SS0_API CS0_MenuPaneShader : public gui_cs0::MenuPane::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ShaderParams
	{
		color4b  colorBgrLeft;
		color4b  colorBgrRight;
		color4b  colorBorder;
		float    widthBorder;

		ShaderParams();
	};


public:
	//------------------------------
	// CS0_MenuPaneShader interface.
	//------------------------------
	CS0_MenuPaneShader(const std::string& shaderName = "",
						const ShaderParams& sp = ShaderParams());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams()) { shaderParams = sp; }
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;
};


//====================================================================
class VW_GUI_SS0_API CS0_MenuSeparatorShader : public gui_cs0::MenuSeparator::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ShaderParams
	{
		color4b  colorLeft;
		color4b  colorRight;
		float    padding;
		float    lineWidth;

		ShaderParams();
	};


public:
	//-----------------------------------
	// CS0_MenuSeparatorShader interface.
	//-----------------------------------
	CS0_MenuSeparatorShader(const std::string& shaderName = "",
							const ShaderParams& sp = ShaderParams());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams()) { shaderParams = sp; }
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;
};


//====================================================================
class VW_GUI_SS0_API CS0_MenuItemShader : public gui_cs0::MenuItem::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ShaderParams
	{
		color4b  colorTextDisabled;
		color4b  colorTextEnabled;
		color4b  colorTextHot;
		color4b  colorBgrLeftHot;
		color4b  colorBgrRightHot;
		color4b  colorTileDisabled;
		color4b  colorTileEnabled;
		color4b  colorTileHot;
		float    tileShrink;

		gui::TileParams  tileParamsOn;
		gui::TileParams  tileParamsSelected;
		gui::TileParams  tileParamsOpen;

		ShaderParams();
	};


public:
	//------------------------------
	// CS0_MenuItemShader interface.
	//------------------------------
	CS0_MenuItemShader(const std::string& shaderName = "",
						const ShaderParams& sp = ShaderParams(),
						const gui::TextAttributes& ta = DefaultTextAttributes());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams());
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------------
	// Internal helpers.
	//------------------
	void  PrepareTileRectangle(gui::RectangleCT& r, color4b color, float leftOffset,
					const gui_cs0::MenuItem& mi, const gui_cs0::MenuItem::GeomParams& gp) const;

	//-----------------------------
	// VideoResourceUser overrides.
	//-----------------------------
	virtual  bool  CacheTileTextureHandles(gui::Manager& gm);
	virtual  void  PostCacheTileTextureHandles();


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;

	//-------
	// Cache.
	//-------
	const gui::TileTexture*   pTileTexOn;
	const gui::TileTexture*   pTileTexSelected;
	const gui::TileTexture*   pTileTexOpen;
	mutable gui::RectangleCT  rTileOn;
	mutable gui::RectangleCT  rTileSelected;
	mutable gui::RectangleCT  rTileOpen;
};


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


