

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\ScrollBar.h"
#include "private\vw_gui_ss0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
class VW_GUI_SS0_API CS0_ScrollBarShader : public gui_cs0::ScrollBar::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ParamBlockArea
	{
		color4b  c1;
		color4b  c2;
		color4b  cBorder;
		float    widthBorder;
	};

	struct VW_GUI_SS0_API ParamBlockTile
	{
		color4b  color;
		float    shrink;
	};

	struct VW_GUI_SS0_API ShaderParams
	{
		ParamBlockArea  baseDisabled;
		ParamBlockArea  baseEnabled;
		ParamBlockArea  baseFocused;
		ParamBlockArea  baseHot;

		ParamBlockArea  thumbDisabled;
		ParamBlockArea  thumbEnabled;
		ParamBlockArea  thumbHot;
		ParamBlockArea  thumbGrabbed;

		ParamBlockArea  buttonDisabled;
		ParamBlockArea  buttonEnabled;
		ParamBlockArea  buttonHot;
		ParamBlockArea  buttonPressed;

		ParamBlockTile  tButtonDisabled;
		ParamBlockTile  tButtonEnabled;
		ParamBlockTile  tButtonHot;
		ParamBlockTile  tButtonPressed;

		gui::TileParams  tileParamsUp;
		gui::TileParams  tileParamsDown;
		gui::TileParams  tileParamsLeft;
		gui::TileParams  tileParamsRight;

		ShaderParams();

		void  ChooseParams(const gui_cs0::ScrollBarButton& button,
					ParamBlockArea& pba, ParamBlockTile& pbt) const;

		void  ChooseParams(const gui_cs0::ScrollBar& sbar,
					ParamBlockArea& pbaBase, ParamBlockArea& pbaThumb,
					ParamBlockArea& pbaButtonBackward, ParamBlockTile& pbtButtonBackward,
					ParamBlockArea& pbaButtonForward, ParamBlockTile& pbtButtonForward) const;
	};


public:
	//-------------------------------
	// CS0_ScrollBarShader interface.
	//-------------------------------
	CS0_ScrollBarShader(const std::string& shaderName = "",
						const ShaderParams& sp = ShaderParams());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams());
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//-----------------------------
	// VideoResourceUser overrides.
	//-----------------------------
	virtual  bool  CacheTileTextureHandles(gui::Manager& gm);
	virtual  void  PostCacheTileTextureHandles();


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;

	//-------
	// Cache.
	//-------
	const gui::TileTexture*   pTileTexUp;
	const gui::TileTexture*   pTileTexDown;
	const gui::TileTexture*   pTileTexLeft;
	const gui::TileTexture*   pTileTexRight;
	mutable gui::RectangleCT  rTileUp;
	mutable gui::RectangleCT  rTileDown;
	mutable gui::RectangleCT  rTileLeft;
	mutable gui::RectangleCT  rTileRight;
};


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


