

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\ShaderNames.h"
#include "GUI_SS0\Buttons.h"
#include "GUI_SS0\CheckBox.h"
#include "GUI_SS0\ComboBox.h"
#include "GUI_SS0\Console.h"
#include "GUI_SS0\EditBox.h"
#include "GUI_SS0\Label.h"
#include "GUI_SS0\LogoSpinners.h"
#include "GUI_SS0\Menu.h"
#include "GUI_SS0\PixelOffsetControlPane.h"
#include "GUI_SS0\PropertyFrame.h"
#include "GUI_SS0\RadioButton.h"
#include "GUI_SS0\Rubberband.h"
#include "GUI_SS0\ScrollBar.h"
#include "GUI_SS0\Separators.h"
#include "GUI_SS0\ToolTip.h"
#include "GUI_SS0\TopLevelFrame.h"
#include "GUI_SS0\TopLevelPane.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
struct VW_GUI_SS0_API ShaderFactory
{
	//------------------------
	// Shader creation-params.
	//------------------------
	CS0_LabelShader_Static::ShaderParams  spLabelStatic;
	CS0_LabelShader_Active::ShaderParams  spLabelActive;
	CS0_LabelShader_Link::ShaderParams    spLabelLink;

	ShaderParams_Button_PushOpaque         spTextButtonPush;
	ShaderParams_Button_ToggleOpaque       spTextButtonToggle;
	ShaderParams_Button_PushOpaque         spIconButtonPush;
	ShaderParams_Button_ToggleOpaque       spIconButtonToggle;

	CS0_CheckBoxShader_Opaque::ShaderParams    spCheckBox;
	CS0_RadioButtonShader::ShaderParams        spRadioButton;
	CS0_EditBoxShader::ShaderParams            spEditBox;
	CS0_ComboBoxShader::ShaderParams           spComboBox;
	CS0_ScrollBarShader::ShaderParams          spScrollBar;
	CS0_HorzSeparatorShader::ShaderParams      spHorzSeparator;
	CS0_HorzTextSeparatorShader::ShaderParams  spHorzTextSeparator;
	CS0_ControlGroupFrameShader::ShaderParams  spControlGroupFrame;
	CS0_LogoSpinnerShader::ShaderParams        spLogoSpinner;
	CS0_ToolTipShader::ShaderParams            spToolTip;
	CS0_RubberbandShader::ShaderParams         spRubberBand;

	CS0_MenuBarShader::ShaderParams        spMenuBar;
	ShaderParams_Button_PushTranslucent    spMenuBarIconButtonBarControl;
	CS0_MenuTitleShader::ShaderParams      spMenuTitle;
	CS0_MenuPaneShader::ShaderParams       spMenuPane;
	CS0_MenuSeparatorShader::ShaderParams  spMenuSeparator;
	CS0_MenuItemShader::ShaderParams       spMenuItem;

	CS0_PixelOffsetControlPaneShader::ShaderParams  spPixelOffsetControlPane;
	ShaderParams_Button_PushTranslucent             spPixelOffsetControlPaneIconButton;
	CS0_LabelShader_Static::ShaderParams            spPixelOffsetControlPaneLabel;
	gui::TextAttributes                             taPixelOffsetControlPaneLabel;

	CS0_PropertyTabShader::ShaderParams   spPropertyTab;
	CS0_PropertyPageShader::ShaderParams  spPropertyPage;

	CS0_ConsoleOutputBgrShader::ShaderParams  spConsoleOutputBgr;
	CS0_EditBoxShader::ShaderParams           spConsoleEditBox;
	CS0_ScrollBarShader::ShaderParams         spConsoleScrollBar;
	gui::TextAttributes                       taConsoleEditBox;

	CS0_TopLevelFrameShader::ShaderParams  spTopLevelFrame;
	ShaderParams_Button_PushTranslucent    spTopLevelFrameIconButtonFrameControlPush;
	ShaderParams_Button_ToggleTranslucent  spTopLevelFrameIconButtonFrameControlToggle;
	ShaderParams_Button_PushTranslucent    spTopLevelFrameTextButtonFrameControlPush;
	ShaderParams_Button_ToggleTranslucent  spTopLevelFrameTextButtonFrameControlToggle;

	CS0_TopLevelPaneShader::ShaderParams  spTopLevelPane;
	ShaderParams_Button_PushTranslucent   spTopLevelPaneIconButtonPaneControl;

	CS0_LabelShader_Static::ShaderParams          spOverlayLabelStatic;
	ShaderParams_Button_PushTranslucent           spOverlayTextButtonPush;
	ShaderParams_Button_ToggleTranslucent         spOverlayTextButtonToggle;
	ShaderParams_Button_PushTranslucent           spOverlayIconButtonPush;
	ShaderParams_Button_ToggleTranslucent         spOverlayIconButtonToggle;
	CS0_CheckBoxShader_Translucent::ShaderParams  spOverlayCheckBox;

	//------------------------------------------------------
	// Creation interface.
	// Set corresponding flags to false if particular shader
	// is not required (default is true for all shaders),
	// then call CreateShaders.
	//------------------------------------------------------
	ShaderFactory();

	void  SetAllCreateFlags(bool bCreate = true);
	void  SetCreateFlag(gui_cs0::eShader s, bool bCreate = true);
	bool  IsCreateFlagSet(gui_cs0::eShader s) const { return createFlags[s]; }

	void  CreateShaders() const;


private:
	bool  createFlags[ gui_cs0::Shader_Count ];
};


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


