

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\OutputForms.h"
#include "MathTools\Matrix22.h"
#include "MathTools\Matrix33.h"
#include "MathTools\Matrix44.h"
#include "private\vw_common_exports.h"


//********************************************************************
namespace vertexware {


//====================================================================
// Handles formatted output of matrices.
//====================================================================
class VW_COMMON_API MatrixOutputForm : public NumberSequenceOutputForm
{
VW_DECLARE_NUMBER_SEQUENCE_OUTPUT_FORM( MatrixOutputForm )


public:
	//----------------
	// Bind to matrix.
	//----------------
	const NumberSequenceBoundOutputForm_float operator()(const mat2& m) const {
		return NumberSequenceBoundOutputForm_float(*this, m.ToPtr(), 4, 2);
	}
	const NumberSequenceBoundOutputForm_float operator()(const mat3& m) const {
		return NumberSequenceBoundOutputForm_float(*this, m.ToPtr(), 9, 3);
	}
	const NumberSequenceBoundOutputForm_float operator()(const mat4& m) const {
		return NumberSequenceBoundOutputForm_float(*this, m.ToPtr(), 16, 4);
	}
};


//====================================================================
// Outputs the matrix using the current form.
//====================================================================
__forceinline std::ostream& __fastcall operator<<(std::ostream &os, const mat2& m)
{
	NumberSequenceBoundOutputForm_float
			boundForm(MatrixOutputForm::GetCurrentOutputForm(), m.ToPtr(), 4, 2);
	return os << boundForm;
}


//====================================================================
__forceinline std::ostream& __fastcall operator<<(std::ostream &os, const mat3& m)
{
	NumberSequenceBoundOutputForm_float
			boundForm(MatrixOutputForm::GetCurrentOutputForm(), m.ToPtr(), 9, 3);
	return os << boundForm;
}


//====================================================================
__forceinline std::ostream& __fastcall operator<<(std::ostream &os, const mat4& m)
{
	NumberSequenceBoundOutputForm_float
			boundForm(MatrixOutputForm::GetCurrentOutputForm(), m.ToPtr(), 16, 4);
	return os << boundForm;
}


//********************************************************************
}//namespace vertexware

