

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\OutputForms.h"
#include "MathTools\Quaternion.h"
#include "private\vw_common_exports.h"


//********************************************************************
namespace vertexware {


//====================================================================
// Handles formatted output of quaternions.
//====================================================================
class VW_COMMON_API QuaternionOutputForm : public NumberSequenceOutputForm
{
VW_DECLARE_NUMBER_SEQUENCE_OUTPUT_FORM( QuaternionOutputForm )


public:
	QuaternionOutputForm() {
		AxisProlog();
		AxisEpilog();
	}

	void  AxisProlog(const std::string& s = "(") { axisProlog = s; }
	const std::string&  AxisProlog() const { return axisProlog; }

	void  AxisEpilog(const std::string& s = ")") { axisEpilog = s; }
	const std::string&  AxisEpilog() const { return axisEpilog; }

	//---------------------
	// Base-form overrides.
	//---------------------
	virtual  bool  GetNumberProlog(uint numberPosition, std::string& outProlog) const {
		if(numberPosition == 0) {
			outProlog = axisProlog;
			return true;
		}
		return false;
	}

	virtual  bool  GetNumberEpilog(uint numberPosition, std::string& outEpilog) const {
		if(numberPosition == 2) {
			outEpilog = axisEpilog;
			return true;
		}
		return false;
	}

	//--------------------
	// Bind to quat.
	//--------------------
	const NumberSequenceBoundOutputForm_float operator()(const quat& q) const {
		return NumberSequenceBoundOutputForm_float(*this, q.ToPtr(), 4, 4);
	}


private:
	std::string  axisProlog;
	std::string  axisEpilog;
};


//====================================================================
// Outputs quat using the current form.
//====================================================================
__forceinline std::ostream& __fastcall operator<<(std::ostream &os, const quat& q)
{
	NumberSequenceBoundOutputForm_float
			boundForm(QuaternionOutputForm::GetCurrentOutputForm(), q.ToPtr(), 4, 4);
	return os << boundForm;
}


//********************************************************************
}//namespace vertexware

