

//--------------------------------------------------------------------
#pragma once


//====================================================================
// Base header.
//====================================================================
#include "BaseTools\CompilerConfig.h"


//====================================================================
// Header package: BaseTools
//====================================================================
#include "BaseTools\AutoCleanup.h"
#include "BaseTools\Color.h"
#include "BaseTools\ColorOutput.h"
#include "BaseTools\CommandInterpreter.h"
#include "BaseTools\CommandLineParser.h"
//#include "BaseTools\CompilerConfig.h" //included on top as base-header
#include "BaseTools\ConsoleBuffers.h"
#include "BaseTools\ConsoleOutputWindow.h"
#include "BaseTools\Constructs.h"
#include "BaseTools\Debug.h"
#include "BaseTools\DynamicLinkLibrary.h"
#include "BaseTools\ElementProcessors.h"
#include "BaseTools\Exception.h"
#include "BaseTools\FileSystem.h"
#include "BaseTools\HeapTracked.h"
#include "BaseTools\LibraryInfo.h"
#include "BaseTools\Memory.h"
#include "BaseTools\NamedParameter.h"
#include "BaseTools\NoCopy.h"
#include "BaseTools\Observable.h"
#include "BaseTools\OutputForms.h"
#include "BaseTools\PlatformInfo.h"
#include "BaseTools\Profiler.h"
#include "BaseTools\Prototype.h"
#include "BaseTools\ScrollGadget.h"
#include "BaseTools\Serializable.h"
#include "BaseTools\Serializers.h"
#include "BaseTools\SinglePassIterator.h"
#include "BaseTools\Singleton.h"
#include "BaseTools\STLExtensions.h"
#include "BaseTools\Stream.h"
#include "BaseTools\String.h"
#include "BaseTools\SyncPrimitives.h"
#include "BaseTools\Thread.h"
#include "BaseTools\Time.h"
#include "BaseTools\TopLevelWindow.h"
#include "BaseTools\Types.h"
#include "BaseTools\VariableProxy.h"
#include "BaseTools\WinUtils.h"


//====================================================================
// Header package: MathTools
//====================================================================
#include "MathTools\MathBase.h"
#include "MathTools\MathProcessors.h"
#include "MathTools\Matrix22.h"
#include "MathTools\Matrix33.h"
#include "MathTools\Matrix44.h"
#include "MathTools\MatrixBase.h"
#include "MathTools\MatrixOutput.h"
#include "MathTools\Quaternion.h"
#include "MathTools\QuaternionOutput.h"
#include "MathTools\TransformTools.h"
#include "MathTools\Vector2.h"
#include "MathTools\Vector3.h"
#include "MathTools\Vector4.h"
#include "MathTools\VectorBase.h"
#include "MathTools\VectorOutput.h"


//====================================================================
// Header package: Framework
//====================================================================
#include "Framework\Application.h"
#include "Framework\DirectInputProxy.h"
#include "Framework\DirectInputProxy8.h"
#include "Framework\FrameObservers.h"
#include "Framework\FrameworkCommands.h"
#include "Framework\FrameworkExceptions.h"
#include "Framework\FrameworkProfiler.h"
#include "Framework\GenericExtension.h"
#include "Framework\HIDController.h"
#include "Framework\HIDDriver.h"
#include "Framework\HIDDriver0.h"
#include "Framework\HIDDriver8.h"
#include "Framework\HumanInterfaceDevice.h"
#include "Framework\Kernel.h"
//#include "Framework\KernelMacros.h" //do not include by default (contains unprefixed macros)
#include "Framework\LibWrapperDXErr8.h"
#include "Framework\Plugin.h"
#include "Framework\StandardCommandInterpreter.h"
#include "Framework\StandardPlugins.h"
#include "Framework\VideoProperties.h"
#include "Framework\VideoRenderer.h"
#include "Framework\VideoSubsystem.h"


//====================================================================
// Header package: DataObjects
//====================================================================
#include "DataObjects\DataCodecManager.h"
#include "DataObjects\DataPackage.h"
#include "DataObjects\Image.h"
#include "DataObjects\ImageBase.h"
#include "DataObjects\ImageOperations.h"
#include "DataObjects\MipmapChains.h"
#include "DataObjects\Sample.h"
#include "DataObjects\SampleProcessors.h"
#include "DataObjects\Signal.h"
#include "DataObjects\SignalBase.h"
#include "DataObjects\SignalCodecGdiplus.h"
#include "DataObjects\SignalCodecVWDP.h"
#include "DataObjects\SignalIO.h"
#include "DataObjects\SignalOperations.h"
#include "DataObjects\Volume.h"
#include "DataObjects\VolumeBase.h"
#include "DataObjects\VolumeOperations.h"


//====================================================================
// Header package: VideoRendererObjects
//====================================================================
#include "VideoRendererObjects\VRO_Resources.h"
#include "VideoRendererObjects\VRO_Textures.h"


//====================================================================
// Header package: HIDControllers
//====================================================================
#include "HIDControllers\GameController.h"
#include "HIDControllers\Glove.h"
#include "HIDControllers\GloveGestures.h"
#include "HIDControllers\Keyboard.h"
#include "HIDControllers\KeyboardCodes.h"
#include "HIDControllers\Mouse.h"
#include "HIDControllers\OrientationTracker.h"
#include "HIDControllers\PositionTracker.h"


//====================================================================
// Header package: GUI
//====================================================================
#include "GUI\GUI_Base.h"
#include "GUI\GUI_BehaviorBases.h"
#include "GUI\GUI_CommonID.h"
#include "GUI\GUI_Component.h"
#include "GUI\GUI_Cursor.h"
#include "GUI\GUI_InterleavedRenderArea.h"
#include "GUI\GUI_Layouters.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_Renderer.h"
#include "GUI\GUI_Shader.h"
#include "GUI\GUI_StdObjects.h"
#include "GUI\GUI_Tools.h"


