

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include <vw_common.h>
#include <vw_common_autolink.h>

#include <vw_opengl.h>
#include <vw_opengl_autolink.h>

#include <vw_direct3d8.h>
#include <vw_direct3d8_autolink.h>

#include <vw_gui_cs0.h>
#include <vw_gui_cs0_autolink.h>

#include <vw_gui_ss0.h>
#include <vw_gui_ss0_autolink.h>

#include <PluginParams\PluginParams_vs_direct3d8.h>


//--------------------------------------------------------------------
using namespace vertexware;


//====================================================================
class app_null : public Application
{
private:
	bool                     bUseOpenGL;
	bool                     bUseFullscreen;
	bool                     bUseResizable;
	const gui::Font*         pFont;
	gui::Renderer*           pGR;
	VideoRenderer*           pVR;
	VideoRendererOpenGL*     pVROpenGL;
	VideoRendererDirect3D8*  pVRDirect3D8;


public:
	app_null();

	virtual ~app_null();

	virtual  UID   GetApplicationUID();
	
	virtual  void  BeginStartup(const StartupInfo& si, StartupResult& sr);
	
	virtual  void  QueryAutoInit(AutoInitInfo& aii);
	
	virtual  void  Initialize();
	
	virtual  void  EndStartup();
	
	virtual  void  Cleanup(bool bCleanExit, const FailureDesc& fd);
	
	virtual  bool  ConfirmGenericExtension(const std::string& pluginName,
							const std::string& genericExtensionInterfaceName);
							
	virtual  void  ConnectGenericExtensions();

	virtual  bool  ConfirmDirectInputVersion(uint directInputVersion);
	
	virtual  bool  ConfirmHIDDriver(const std::string& pluginName,
									uint defaultDriverPriority,
									uint& outDriverPriority);

	virtual  void  QueryHIDConfig(DirectInputConfig& diConfig,
								uint iReconfigCount, uint reconfigCode);

	virtual  bool  QueryIgnoreHIDDriver(const std::string& pluginName);

	virtual  void  BeginConfirmHumanInterfaceDevices();
	
	virtual  bool  ConfirmHumanInterfaceDevice(HumanInterfaceDevice& hid);
	
	virtual  void  EndConfirmHumanInterfaceDevices();

	virtual  void  BeginConnectHumanInterfaceDevices(const DirectInputConfig& diConfig);
	
	virtual  void  ConnectHumanInterfaceDevice(HumanInterfaceDevice& hid);
	
	virtual  void  EndConnectHumanInterfaceDevices();

	virtual  void  SetupAsyncHIDSampling();

	virtual  void  DisconnectHumanInterfaceDevices();

	virtual  void  FilterVideoProfile(const VideoProfile& fullProfile,
									VideoProfile& outFilteredProfile);

	virtual  bool  ConfirmVideoRenderer(const std::string& pluginName,
										const std::string& interfaceName);

	virtual  void  QueryVideoConfig(std::string& vrPluginName, VideoConfig& vc,
									uint iReconfigCount, uint reconfigCode);

	virtual  void  PostCreateMainWindow(MainWindowProperties& mwp, MainWindowGeometry& mwg,
									const std::string& vrPluginName, const VideoConfig& vc,
									uint iCreateCount);

	virtual  void  PreDestroyMainWindow(const MainWindowGeometry& mwg,
								const std::string& vrPluginName, const VideoConfig& vc);

	virtual  void  OnVSyncUpdate(bool bVSyncEnabled);

	virtual  void  ConnectVideoRenderer(VideoRenderer& vr);

	virtual  void  DisconnectVideoRenderer();

	virtual  void  InitVideoRendererStates(bool bPostReset);

	virtual  void  CreateVideoResources(bool bPostReset);

	virtual  void  ReleaseVideoResources(bool bPreReset);

	virtual  void  GenericReconfig(uint iReconfigCount, uint reconfigCode);

	virtual  void  InitializeSubsystems(uint iReconfigCount, uint reconfigCode);
	
	virtual  void  CleanupSubsystems(uint iReconfigCount, uint reconfigCode);

	virtual  void  ConnectSubsystemsToAppWindow(HWND hwnd, bool bFullscreen);
	
	virtual  void  DisconnectSubsystemsFromAppWindow();

	virtual  bool  CheckSubsystems();
	
	virtual  void  FixSubsystems();

	virtual  void  OnKernelLevelChanged(eKernelLevel activeLevel);

	virtual  void  OnWindowActive(bool bActive);
	
	virtual  void  OnWindowResize(uint width, uint height);

	virtual  void  FrameBegin(double secsFrameTimeDelta);
	
	virtual  void  FrameSampleInput();
	
	virtual  void  FrameUpdateStates(double secsFrameTimeDelta);
	
	virtual  void  FramePreRenderVideo();
	
	virtual  void  FrameRenderVideo();
	
	virtual  void  FrameRenderArea(const gui::InterleavedRenderArea& area);
	
	virtual  void  FramePostRenderVideo();
	
	virtual  void  FrameEnd();

	virtual  bool  GUIOnCommand(uint commandID);
	
	virtual  bool  GUIOnActivate(bool bActivate, bool bByMouse);
	
	virtual  bool  GUIOnMouseCapture(bool bCaptureBegin);
	
	virtual  bool  GUIOnMouseEnterExit(bool bEnter, const vec2& mousePos);
	
	virtual  bool  GUIOnMouseQueryCursor(const gui::Cursor* &pCursor, const vec2& mousePos);
	
	virtual  bool  GUIOnMouseMove(bool bHover, double hoverTimeSecs, const vec2& mousePos);
	
	virtual  bool  GUIOnMouseWheel(int delta, const vec2& mousePos);
	
	virtual  bool  GUIOnMouseButton(bool bDown, eMouseButton buttonCode, bool bDoubleClick, uint repeatCount, const vec2& mousePos);
	
	virtual  bool  GUIOnKeyboardQueryFocus(bool bFocusByMouse, eMouseButton buttonCode, bool& bWantFocus, bool& bNeedCharacterData);
	
	virtual  bool  GUIOnKeyboardFocus(bool bGotFocus);
	
	virtual  bool  GUIOnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch);
};

