

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_CommonID.h" // this header contains only standard gui identifiers
#include "Framework\HIDController.h"


//--------------------------------------------------------------------
using namespace vertexware;


//====================================================================
// Application specific identifiers
//====================================================================
enum
{
	CID_Dummy = vertexware::gui::CID_User,
	CID_OpenConsole,
	CID_OpenProfilerDialog,
	CID_OpenFrameStatsPane,
	CID_VSync,
	CID_OpenVideoConfigDialog,
	CID_OpenPixelOffsetPane,
	CID_OpenHIDConfigDialog,
	CID_HIDReconfigCycle,
	CID_OpenMouseStatePane,
	CID_OpenKeyboardStatePane,
	CID_OpenGameControllerStatePane,
	CID_OpenOrientationTrackerStatePane,
	CID_OpenPositionTrackerStatePane,
	CID_OpenGloveStatePane,
	CID_VSD3D8Cleanup,
	CID_OpenFontTest,
	CID_OpenClockTest,
	CID_OpenToolBarTest1,
	CID_OpenToolBarTest2,
	CID_OpenDemoOptions,
	CID_OpenDemoAbout,
	CID_LightColor,
	CID_EnableRubberband,
	CID_EventLogShow,
	CID_EventLogQueryCursor,
	CID_EventLogMouseMove,
	CID_RubberBand,
	CID_ToolBarTest1Button,
	CID_ToolBarTest2Button,
	CID_CreateDynamicTestPane,
	CID_DestroyDynamicTestPane,
	CID_AppSubsysReconfigCycle,
	CID_ExitProgram
};


//====================================================================
enum
{
	RC_CreateDynamicTestPane = 1,
	RC_DestroyDynamicTestPane
};


//====================================================================
enum
{
	AreaCode_InterleavedTest = 1,
};


//====================================================================
enum eViewController
{
	ViewController_None = 0,
	ViewController_Keyboard,
	ViewController_Mouse,
	ViewController_GameController,
};


//====================================================================
struct ViewConfig
{
	eViewController  activeController;
	bool             bReverseViewControllerVerticalAxis;
	bool             bCombineTrackerReadings;
	bool             bReverseTrackerHead;
	bool             bReverseTrackerPitch;
	bool             bReverseTrackerRoll;

	ViewConfig(eViewController _activeController = ViewController_None,
				bool _bReverseViewControllerVerticalAxis = false,
				bool _bCombineTrackerReadings = false,
				bool _bReverseTrackerHead = false,
				bool _bReverseTrackerPitch = false,
				bool _bReverseTrackerRoll = false)
	{
		activeController = _activeController;
		bReverseViewControllerVerticalAxis = _bReverseViewControllerVerticalAxis;
		bCombineTrackerReadings = _bCombineTrackerReadings;
		bReverseTrackerHead = _bReverseTrackerHead;
		bReverseTrackerPitch = _bReverseTrackerPitch;
		bReverseTrackerRoll = _bReverseTrackerRoll;
	}
};


//====================================================================
struct InterleavedViewConfig
{
	float  maxHorz;
	float  maxVert;

	float  pageHorz;
	float  pageVert;

	float  posHorz;
	float  posVert;

	InterleavedViewConfig(float posH = 0, float posV = 0)
	{
		posHorz = posH;
		posVert = posV;
	}
};


//====================================================================
enum eAsyncController
{
	AsyncController_Mouse = 0,
	AsyncController_Keyboard,
	AsyncController_GameController,
	AsyncController_OrientationTracker,

	AsyncController_Count
};


//====================================================================
struct AsyncSampleConfig
{
	HIDController*  controller;
	bool            bAsync;
	bool            bLimitRate;
	double          maxRate;

	AsyncSampleConfig(HIDController* c = 0, bool async = false,
						bool limit = false, double max = 200)
	{
		controller = c;
		bAsync = async;
		bLimitRate = limit;
		maxRate = max;
	}
};


//====================================================================
struct MiscConfig
{
	std::string  screenshotFormat;

	MiscConfig(const std::string& _screenshotFormat = "")
	{
		screenshotFormat = _screenshotFormat;
	}
};

