

//--------------------------------------------------------------------
#include "private\GUITests.h"
#include "private\app_test1.h"
#include "MathTools\MathBase.h"


//====================================================================
ToolBarTest::ToolBarTest(gui::MessageHandler* pCommandTarget,
						uint commandID,
						bool bVertical,
						const std::string& title)
	:
	gui_cs0::ToolBar(title, bVertical)
{
	SetCommandTarget(pCommandTarget);
	SetCommandID(commandID);
}


//====================================================================
bool  ToolBarTest::ProcessToolBarControlCommand(uint commandID)
{
	// Any button push enables all components
	{for(uint i = 0; i < 4; i++) {
		pButtonsS0[i]->SetEnabled(true);
		pButtonsS1[i]->SetEnabled(true);
	}}

	{for(uint i = 0; i < 3; i++) {
		pButtonsOO[i]->SetEnabled(true);
		pButtonsP[i]->SetEnabled(true);
	}}

	// If selection button is toggled, disable corresponding button group
	if(commandID >= CID_ButtonSelect  &&  commandID < CID_ButtonSelect + 4) {
		switch(commandID - CID_ButtonSelect) {
		case 0:
			{for(uint i = 0; i < 4; i++) {
				pButtonsS0[i]->SetEnabled(false);
			}}
			break;
		case 1:
			{for(uint i = 0; i < 3; i++) {
				pButtonsOO[i]->SetEnabled(false);
			}}
			break;
		case 2:
			{for(uint i = 0; i < 4; i++) {
				pButtonsS1[i]->SetEnabled(false);
			}}
			break;
		case 3:
			{for(uint i = 0; i < 3; i++) {
				pButtonsP[i]->SetEnabled(false);
			}}
			break;
		}
	}

	// Report button press to external target
	SendCommandMessageToTarget();
	return true;
}


//====================================================================
ToolBarTest1::ToolBarTest1(gui::MessageHandler* pCommandTarget,
							uint commandID,
							const std::string& title)
	:
	ToolBarTest(pCommandTarget, commandID, false, title)
{
	using namespace vertexware::gui::std_tile_textures;

	IconButton* pButton = 0;

	// Add selection group 0
	{for(uint i = 0; i < 4; i++) {

		pButton = new IconButton(this, this, CID_ButtonSelect + i, Button::Style_Selection, 0);
		pButton->SetTileParams(MakeTileParams(eTile(Tile_ArrowLeft + i)));
		pButton->SetToolTipText("Selection group 0 / button " + ToString(i));

		AttachControl(pButton);
		pButtonsS0[i] = pButton;
	}}

	// Add on/off group
	{for(uint i = 0; i < 3; i++) {

		pButton = new IconButton(this, this, 0, Button::Style_OnOff);
		pButton->SetTileParams(MakeTileParams(eTile(Tile_X + i)));
		pButton->SetToolTipText("On/Off button " + ToString(i));

		AttachControl(pButton);
		pButtonsOO[i] = pButton;
	}}

	// Add selection group 1
	{for(uint i = 0; i < 4; i++) {

		pButton = new IconButton(this, this, CID_ButtonSelect + i, Button::Style_Selection, 1);
		pButton->SetTileParams(MakeTileParams(eTile(Tile_ArrowNS + i)));
		pButton->SetToolTipText("Selection group 1 / button " + ToString(i));

		AttachControl(pButton);
		pButtonsS1[i] = pButton;
	}}

	// Add push group
	{for(uint i = 0; i < 3; i++) {

		pButton = new IconButton(this, this, 0, Button::Style_Push);
		pButton->SetTileParams(MakeTileParams(eTile(Tile_Maximize + i)));
		pButton->SetToolTipText("Push button " + ToString(i));

		AttachControl(pButton);
		pButtonsP[i] = pButton;
	}}

	// Add spacers
	AddSpacer(pButtonsOO[0]);
	AddSpacer(pButtonsS1[0]);
	AddSpacer(pButtonsP[0]);
}


//====================================================================
ToolBarTest2::ToolBarTest2(gui::MessageHandler* pCommandTarget,
							uint commandID,
							const std::string& title)
	:
	ToolBarTest(pCommandTarget, commandID, true, title)
{
	TextButton* pButton = 0;

	// Add selection group 0
	{for(uint i = 0; i < 4; i++) {

		pButton = new TextButton(this, this, CID_ButtonSelect + i, Button::Style_Selection, 0, "Select0");
		pButton->SetToolTipText("Selection group 0 / button " + ToString(i));

		AttachControl(pButton);
		pButtonsS0[i] = pButton;
	}}

	// Add on/off group
	{for(uint i = 0; i < 3; i++) {

		pButton = new TextButton(this, this, 0, Button::Style_OnOff, 0, "On/Off");
		pButton->SetToolTipText("On/Off button " + ToString(i));

		AttachControl(pButton);
		pButtonsOO[i] = pButton;
	}}

	// Add selection group 1
	{for(uint i = 0; i < 4; i++) {

		pButton = new TextButton(this, this, CID_ButtonSelect + i, Button::Style_Selection, 1, "Select1");
		pButton->SetToolTipText("Selection group 1 / button " + ToString(i));

		AttachControl(pButton);
		pButtonsS1[i] = pButton;
	}}

	// Add push group
	{for(uint i = 0; i < 3; i++) {

		pButton = new TextButton(this, this, 0, Button::Style_Push, 0, "Push");
		pButton->SetToolTipText("Push button " + ToString(i));

		AttachControl(pButton);
		pButtonsP[i] = pButton;
	}}

	// Add spacers
	AddSpacer(pButtonsOO[0]);
	AddSpacer(pButtonsS1[0]);
	AddSpacer(pButtonsP[0]);
}


//====================================================================
ClockTestPane::ClockTestPane()
{
	SetTLPSubcomponentLayout(false, false, true, true);
	SetSize(96, 32);

	pClock = new Clock(this);
	pClock->SetDefaultCommandArea(CommandArea_Move);
}


//====================================================================
void  ClockTestPane::RecalcLayout()
{
	TopLevelPane::RecalcLayout();

	// center clock label
	vec2 offset = (GetSize() - pClock->GetSize()) * 0.5f;
	pClock->SetPosition(offset);
}


//====================================================================
DynamicTestPane::DynamicTestPane(gui::MessageHandler* _pCommandTarget,
								uint _commandID,
								const std::string& _title)
	:
	TopLevelPane(_title)
{
	SetCommandTarget(_pCommandTarget);
	SetCommandID(_commandID);

	SetSize(340, 280);

	// Create components
	using namespace gui::layouters;
	pLayouter = new RowLayouter(this);
	pLayouter->SetPosition(20, 20);
	RowLayouter::GeomParams gpRL;
	gpRL.preferredSize = vec2(GetSize() - pLayouter->GetPosition() * 2);
	pLayouter->SetGeomParams(gpRL);

	pCheckBoxUseOverlayShaders = new gui_cs0::CheckBox(pLayouter, this, CID_UseOverlayShaders);
	pCheckBoxUseOverlayShaders->SetShaderName(DefaultShaderName(Shader_Overlay_CheckBox));
	pCheckBoxUseOverlayShaders->SetOn(true);

	pLabelUseOverlayShaders = new gui_cs0::Label(pLayouter, 0, 0, "Use overlay shaders", 0, true);
	pLabelUseOverlayShaders->SetShaderName(DefaultShaderName(Shader_Overlay_Label_Static));

	pTextButtonPress1 = new gui_cs0::TextButton(pLayouter, 0, 0, Button::Style_Press, 0, "Press1");
	pTextButtonPress2 = new gui_cs0::TextButton(pLayouter, 0, 0, Button::Style_Press, 0, "Press2");
	pTextButtonPush1 = new gui_cs0::TextButton(pLayouter, 0, 0, Button::Style_Push, 0, "Push1");
	pTextButtonPush2 = new gui_cs0::TextButton(pLayouter, 0, 0, Button::Style_Push, 0, "Push2");
	pTextButtonToggle1 = new gui_cs0::TextButton(pLayouter, 0, 0, Button::Style_OnOff, 0, "Toggle1");
	pTextButtonToggle2 = new gui_cs0::TextButton(pLayouter, 0, 0, Button::Style_OnOff, 0, "Toggle2");

	using namespace std_tile_textures;

	pIconButtonPress1 = new gui_cs0::IconButton(pLayouter, 0, 0, Button::Style_Press, 0, MakeTileParams(Tile_ArrowPointer));
	pIconButtonPress2 = new gui_cs0::IconButton(pLayouter, 0, 0, Button::Style_Press, 0, MakeTileParams(Tile_ArrowUpDown));
	pIconButtonPush1 = new gui_cs0::IconButton(pLayouter, 0, 0, Button::Style_Push, 0, MakeTileParams(Tile_ArrowPointer));
	pIconButtonPush2 = new gui_cs0::IconButton(pLayouter, 0, 0, Button::Style_Push, 0, MakeTileParams(Tile_ArrowUpDown));
	pIconButtonToggle1 = new gui_cs0::IconButton(pLayouter, 0, 0, Button::Style_OnOff, 0, MakeTileParams(Tile_ArrowPointer));
	pIconButtonToggle2 = new gui_cs0::IconButton(pLayouter, 0, 0, Button::Style_OnOff, 0, MakeTileParams(Tile_ArrowUpDown));

	pCheckBox1 = new gui_cs0::CheckBox(pLayouter);
	pCheckBox2 = new gui_cs0::CheckBox(pLayouter);

	pTextButtonPress2->SetEnabled(false);
	pTextButtonPush2->SetEnabled(false);
	pTextButtonToggle2->SetEnabled(false);
	pIconButtonPress2->SetEnabled(false);
	pIconButtonPush2->SetEnabled(false);
	pIconButtonToggle2->SetEnabled(false);
	pCheckBox2->SetOn(true);
	pCheckBox2->SetEnabled(false);

	// Specify layout
	using namespace gui::layouters;
	RowLayouter& l = *pLayouter;

	float hgt = 40;

	l.Clear();
	l.FixedHeightRow(hgt).Target(pCheckBoxUseOverlayShaders).Spacer().Target(pLabelUseOverlayShaders);
	l.FixedHeightRow(hgt).Target(pTextButtonPress1).AutoSpacer().Target(pTextButtonPush1).AutoSpacer().Target(pTextButtonToggle1);
	l.FixedHeightRow(hgt).Target(pTextButtonPress2).AutoSpacer().Target(pTextButtonPush2).AutoSpacer().Target(pTextButtonToggle2);
	l.FixedHeightRow(hgt).Target(pIconButtonPress1).AutoSpacer().Target(pIconButtonPush1).AutoSpacer().Target(pIconButtonToggle1);
	l.FixedHeightRow(hgt).Target(pIconButtonPress2).AutoSpacer().Target(pIconButtonPush2).AutoSpacer().Target(pIconButtonToggle2);
	l.FixedHeightRow(hgt).Target(pCheckBox1).AutoSpacer().Target(pCheckBox2);

	// Assign initial shaders
	SetShaders(pCheckBoxUseOverlayShaders->IsOn());
}


//====================================================================
void  DynamicTestPane::SetShaders(bool bUseOverlayShaders)
{
	std::string s = DefaultShaderName(bUseOverlayShaders ? Shader_Overlay_TextButton_Press : Shader_TextButton_Press);
	pTextButtonPress1->SetShaderName(s);
	pTextButtonPress1->SetToolTipText(pTextButtonPress1->GetShaderName());
	pTextButtonPress2->SetShaderName(s);
	pTextButtonPress2->SetToolTipText(pTextButtonPress2->GetShaderName());

	s = DefaultShaderName(bUseOverlayShaders ? Shader_Overlay_TextButton_Push : Shader_TextButton_Push);
	pTextButtonPush1->SetShaderName(s);
	pTextButtonPush1->SetToolTipText(pTextButtonPush1->GetShaderName());
	pTextButtonPush2->SetShaderName(s);
	pTextButtonPush2->SetToolTipText(pTextButtonPush2->GetShaderName());

	s = DefaultShaderName(bUseOverlayShaders ? Shader_Overlay_TextButton_Toggle : Shader_TextButton_Toggle);
	pTextButtonToggle1->SetShaderName(s);
	pTextButtonToggle1->SetToolTipText(pTextButtonToggle1->GetShaderName());
	pTextButtonToggle2->SetShaderName(s);
	pTextButtonToggle2->SetToolTipText(pTextButtonToggle2->GetShaderName());

	s = DefaultShaderName(bUseOverlayShaders ? Shader_Overlay_IconButton_Press : Shader_IconButton_Press);
	pIconButtonPress1->SetShaderName(s);
	pIconButtonPress1->SetToolTipText(pIconButtonPress1->GetShaderName());
	pIconButtonPress2->SetShaderName(s);
	pIconButtonPress2->SetToolTipText(pIconButtonPress2->GetShaderName());

	s = DefaultShaderName(bUseOverlayShaders ? Shader_Overlay_IconButton_Push : Shader_IconButton_Push);
	pIconButtonPush1->SetShaderName(s);
	pIconButtonPush1->SetToolTipText(pIconButtonPush1->GetShaderName());
	pIconButtonPush2->SetShaderName(s);
	pIconButtonPush2->SetToolTipText(pIconButtonPush2->GetShaderName());

	s = DefaultShaderName(bUseOverlayShaders ? Shader_Overlay_IconButton_Toggle : Shader_IconButton_Toggle);
	pIconButtonToggle1->SetShaderName(s);
	pIconButtonToggle1->SetToolTipText(pIconButtonToggle1->GetShaderName());
	pIconButtonToggle2->SetShaderName(s);
	pIconButtonToggle2->SetToolTipText(pIconButtonToggle2->GetShaderName());

	s = DefaultShaderName(bUseOverlayShaders ? Shader_Overlay_CheckBox : Shader_CheckBox);
	pCheckBox1->SetShaderName(s);
	pCheckBox1->SetToolTipText(pCheckBox1->GetShaderName());
	pCheckBox2->SetShaderName(s);
	pCheckBox2->SetToolTipText(pCheckBox2->GetShaderName());
}


//====================================================================
bool  DynamicTestPane::OnCommand(uint commandID)
{
	if(TopLevelPane::OnCommand(commandID)) return true;

	switch(commandID) {
		case CID_UseOverlayShaders:
			SetShaders(pCheckBoxUseOverlayShaders->IsOn());
			return true;
	}
	return false;
}


//====================================================================
void  DynamicTestPane::OnPaneClosed()
{
	SendCommandMessageToTarget();
}


//====================================================================
AppTest1Page::AppTest1Page(app_test1& app, gui_cs0::PropertyTab* _parent)
	:
	PropertyPage(_parent),
	appTest1(app)
{
}


//====================================================================
ViewPage::ViewPage(app_test1& app, gui_cs0::PropertyTab* _parent)
	:
	AppTest1Page(app, _parent)
{
	using namespace gui::layouters;

	// create components
	pLayouter = new gui::layouters::RowLayouter(this);
	RowLayouter::GeomParams gpRL;
	gpRL.bAutoFitToParentOnRecalcLayout = true;
	pLayouter->SetGeomParams(gpRL);

	// view
	pCGFView = new ControlGroupFrame(pLayouter, "View controller");
	RowLayouter::GeomParams gpCGF;
	gpCGF.preferredSize = vec2(420, 150);
	pCGFView->RowLayouter::SetGeomParams(gpCGF);

	pRadioNone = new RadioButton(pCGFView, this, CID_ControlValuesChanged);
	pLabelNone = new Label(pCGFView, 0, 0, "None", pRadioNone);

	pRadioKeyboard = new RadioButton(pCGFView, this, CID_ControlValuesChanged);
	pLabelKeyboard = new Label(pCGFView, 0, 0, "Keyboard", pRadioKeyboard);

	pRadioMouse = new RadioButton(pCGFView, this, CID_ControlValuesChanged);
	pLabelMouse = new Label(pCGFView, 0, 0, "Mouse", pRadioMouse);
	pLabelMouse->SetToolTipText("To activate, click on application area,\nthen cycle through view-controllers\n(check scrolling help)");
	pRadioMouse->SetEnabled(false);
	pLabelMouse->SetEnabled(false);

	pRadioGameController = new RadioButton(pCGFView, this, CID_ControlValuesChanged);
	pLabelGameController = new Label(pCGFView, 0, 0, "GameController", pRadioGameController);

	pCheckBoxReverseVertAxis = new CheckBox(pCGFView, this, CID_ControlValuesChanged);
	pLabelReverseVertAxis = new Label(pCGFView, 0, 0, "Reverse vertical axis", pCheckBoxReverseVertAxis);

	// tracker
	pCGFTracker = new ControlGroupFrame(pLayouter, "Orientation tracker");
	gpCGF.preferredSize = vec2(420, 120);
	pCGFTracker->RowLayouter::SetGeomParams(gpCGF);

	pCheckBoxCombineTracker = new CheckBox(pCGFTracker, this, CID_ControlValuesChanged);
	pCheckBoxCombineTracker->SetOn(true);
	pCheckBoxCombineTracker->SetToolTipText("Combine tracker readings\nwith view controller readings");

	pLabelCombineTracker = new Label(pCGFTracker, 0, 0, "Combine\ntracker\nreadings", pCheckBoxCombineTracker);

	pCheckBoxReverseHead = new CheckBox(pCGFTracker, this, CID_ControlValuesChanged);
	pLabelReverseHead = new Label(pCGFTracker, 0, 0, "Reverse head (y-axis)", pCheckBoxReverseHead);

	pCheckBoxReversePitch = new CheckBox(pCGFTracker, this, CID_ControlValuesChanged);
	pLabelReversePitch = new Label(pCGFTracker, 0, 0, "Reverse pitch (x-axis)", pCheckBoxReversePitch);

	pCheckBoxReverseRoll = new CheckBox(pCGFTracker, this, CID_ControlValuesChanged);
	pLabelReverseRoll = new Label(pCGFTracker, 0, 0, "Reverse roll (z-axis)", pCheckBoxReverseRoll);

	// specify layout
	pLayouter->TargetHeightRow(pCGFView).Target(pCGFView);
	pLayouter->FixedHeightRow(10);
	pLayouter->TargetHeightRow(pCGFTracker).Target(pCGFTracker);

	pCGFView->FixedHeightRow(20);
	pCGFView->FixedHeightRow().Spacer().Target(pRadioNone).Spacer().Target(pLabelNone);
	pCGFView->LastRow().AutoSpacer().Target(pCheckBoxReverseVertAxis).Spacer().Target(pLabelReverseVertAxis).Spacer();
	pCGFView->FixedHeightRow().Spacer().Target(pRadioKeyboard).Spacer().Target(pLabelKeyboard);
	pCGFView->FixedHeightRow().Spacer().Target(pRadioMouse).Spacer().Target(pLabelMouse);
	pCGFView->FixedHeightRow().Spacer().Target(pRadioGameController).Spacer().Target(pLabelGameController);

	pCGFTracker->FixedHeightRow(20);
	pCGFTracker->FixedHeightRow().AutoSpacer().Target(pCheckBoxReverseHead).Spacer().Target(pLabelReverseHead).Spacer();
	pCGFTracker->FixedHeightRow().Spacer().Target(pCheckBoxCombineTracker).Spacer().Target(pLabelCombineTracker);
	pCGFTracker->LastRow().Target(pCheckBoxReversePitch, SF_Target, 1, AF_Target, pCheckBoxReverseHead).Spacer();
	pCGFTracker->LastRow().Target(pLabelReversePitch, SF_Target, 1, AF_Target, pLabelReverseHead);
	pCGFTracker->FixedHeightRow().AutoSpacer();
	pCGFTracker->LastRow().Target(pCheckBoxReverseRoll, SF_Target, 1, AF_Target, pCheckBoxReverseHead).Spacer();
	pCGFTracker->LastRow().Target(pLabelReverseRoll, SF_Target, 1, AF_Target, pLabelReverseHead);
}


//====================================================================
void  ViewPage::UpdateControlDependencies()
{
	ViewConfig  vc;
	bool        bHasGameController, bHasOrientationTracker;

	appTest1.GetViewConfig(vc, bHasGameController, bHasOrientationTracker);

	pRadioGameController->SetEnabled(bHasGameController);
	pLabelGameController->SetEnabled(bHasGameController);

	switch(vc.activeController) {
		case ViewController_None:
			pRadioNone->Select(false);
			break;
		case ViewController_Keyboard:
			pRadioKeyboard->Select(false);
			break;
		case ViewController_Mouse:
			pRadioMouse->Select(false);
			break;
		case ViewController_GameController:
			pRadioGameController->Select(false);
			break;
	}

	pCheckBoxCombineTracker->SetEnabled(bHasOrientationTracker  &&
									vc.bCombineTrackerReadings  &&
									vc.activeController != ViewController_None);

	pCheckBoxCombineTracker->SetEnabled(bHasOrientationTracker);
	pLabelCombineTracker->SetEnabled(bHasOrientationTracker);

	bool b = pCheckBoxCombineTracker->IsEnabled();
	pCheckBoxReverseHead->SetEnabled(b);
	pLabelReverseHead->SetEnabled(b);
	pCheckBoxReversePitch->SetEnabled(b);
	pLabelReversePitch->SetEnabled(b);
	pCheckBoxReverseRoll->SetEnabled(b);
	pLabelReverseRoll->SetEnabled(b);
}


//====================================================================
void  ViewPage::ApplyControlValues()
{
	ViewConfig vc;

	if(pRadioNone->IsSelected()) vc.activeController = ViewController_None;
	if(pRadioKeyboard->IsSelected()) vc.activeController = ViewController_Keyboard;
	if(pRadioGameController->IsSelected()) vc.activeController = ViewController_GameController;

	vc.bReverseViewControllerVerticalAxis = pCheckBoxReverseVertAxis->IsOn();

	vc.bCombineTrackerReadings = pCheckBoxCombineTracker->IsEnabled()  &&  pCheckBoxCombineTracker->IsOn();
	vc.bReverseTrackerHead = pCheckBoxReverseHead->IsOn();
	vc.bReverseTrackerPitch = pCheckBoxReversePitch->IsOn();
	vc.bReverseTrackerRoll = pCheckBoxReverseRoll->IsOn();

	appTest1.SetViewConfig(vc);
}


//====================================================================
SamplingPage::SamplingPage(app_test1& app, gui_cs0::PropertyTab* _parent)
	:
	AppTest1Page(app, _parent)
{
	// create components & specify layout
	using namespace gui::layouters;
	pLayouter = new RowLayouter(this);
	RowLayouter::GeomParams gp;
	gp.bAutoFitToParentOnRecalcLayout = true;
	pLayouter->SetGeomParams(gp);

	const char* titles[] = { "Mouse", "Keyboard", "GameController", "OrientationTracker" };

	for(uint i = 0; i < AsyncController_Count; i++) {
		CreateGroup(i, titles[i]);
		if(i < AsyncController_Count - 1) pLayouter->AutoHeightRow();
	}
}


//====================================================================
void  SamplingPage::CreateGroup(uint i, const std::string& title)
{
	using namespace gui::layouters;

	pHTS[i] = new gui_cs0::HorzTextSeparator(pLayouter, title);
	pCheckBoxAsync[i] = new gui_cs0::CheckBox(pLayouter, this, CID_ControlValuesChanged);
	pLabelAsync[i] = new gui_cs0::Label(pLayouter, 0, 0, "Async", pCheckBoxAsync[i]);
	pCheckBoxLimitRate[i] = new gui_cs0::CheckBox(pLayouter, this, CID_ControlValuesChanged);
	pLabelLimitRate[i] = new gui_cs0::Label(pLayouter, 0, 0, "LimitRate", pCheckBoxLimitRate[i]);
	pLabelMaxRate[i] = new gui_cs0::Label(pLayouter, 0, 0, "MaxRate:");
	pEditBoxMaxRate[i] = new gui_cs0::EditBox(pLayouter, this, CID_ControlValuesChanged);

	RowLayouter& l = *pLayouter;
	l.TargetHeightRow(pHTS[i]).Target(pHTS[i], SF_Variable);
	l.FixedHeightRow().Target(pCheckBoxAsync[i]).Spacer().Target(pLabelAsync[i]);
	l.LastRow().AutoSpacer().Target(pCheckBoxLimitRate[i]).Spacer().Target(pLabelLimitRate[i]);
	l.LastRow().AutoSpacer().Target(pLabelMaxRate[i]).Spacer().Target(pEditBoxMaxRate[i], SF_Variable, 0.6f);
}


//====================================================================
void  SamplingPage::UpdateControlDependencies()
{
	AsyncSampleConfig  asConfig[AsyncController_Count];
	appTest1.GetAsyncConfig(asConfig);

	for(uint i = 0; i < AsyncController_Count; i++) {
		bool b = (asConfig[i].controller != 0);

		// reset all
		pCheckBoxAsync[i]->SetEnabled(b);
		pCheckBoxAsync[i]->SetOn(false);
		pLabelAsync[i]->SetEnabled(b);
		pCheckBoxLimitRate[i]->SetEnabled(b);
		pCheckBoxLimitRate[i]->SetOn(false);
		pLabelLimitRate[i]->SetEnabled(b);
		pLabelMaxRate[i]->SetEnabled(b);
		pEditBoxMaxRate[i]->SetEnabled(b);
		pEditBoxMaxRate[i]->SetText("");

		// update available only
		if(! b) continue;
		AsyncSampleConfig& asc = asConfig[i];

		pCheckBoxAsync[i]->SetOn(asc.bAsync);
		pCheckBoxLimitRate[i]->SetOn(asc.bLimitRate);
		pEditBoxMaxRate[i]->SetText(ToString(asc.maxRate));
	}
}


//====================================================================
void  SamplingPage::ApplyControlValues()
{
	AsyncSampleConfig  asConfig[AsyncController_Count];

	for(uint i = 0; i < AsyncController_Count; i++) {
		AsyncSampleConfig& asc = asConfig[i];

		asc.bAsync = pCheckBoxAsync[i]->IsOn();
		asc.bLimitRate = pCheckBoxLimitRate[i]->IsOn();
		ToValue<double>(pEditBoxMaxRate[i]->GetText(), asc.maxRate, 200);
	}
	appTest1.SetAsyncConfig(asConfig);
}


//====================================================================
InterleavedRenderPage::InterleavedRenderPage(app_test1& app, gui_cs0::PropertyTab* _parent)
	:
	AppTest1Page(app, _parent)
{
	using namespace gui::layouters;

	// create components
	pLayouter = new RowLayouter(this);
	RowLayouter::GeomParams  gpRL;
	gpRL.bAutoFitToParentOnRecalcLayout = true;
	pLayouter->SetGeomParams(gpRL);

	pScrollBarV = new ScrollBar(pLayouter, this, CID_RotateView, true);
	pScrollBarH = new ScrollBar(pLayouter, this, CID_RotateView, false);

	pArea = new InterleavedRenderArea(pLayouter, AreaCode_InterleavedTest);

	// specify layout
	Cell cArea(pArea, SF_Variable, 1, AF_Center, 0, SF_Variable, 1);
	Cell cSBV(pScrollBarV, SF_Target, 1, AF_Center, 0, SF_Variable, 1);

	pLayouter->AutoHeightRow().Append(cArea).Spacer().Append(cSBV);
	pLayouter->FixedHeightRow(10);
	pLayouter->TargetHeightRow(pScrollBarH).Target(pScrollBarH, SF_Variable, 1);
}


//====================================================================
void  InterleavedRenderPage::UpdateControlDependencies()
{
	InterleavedViewConfig ivc;
	appTest1.GetInterleavedViewConfig(ivc);

	pScrollBarH->SetTotalScrollSize(ivc.maxHorz);
	pScrollBarH->SetVisibleScrollSize(ivc.pageHorz);
	pScrollBarH->SetScrollPosition(ivc.posHorz);

	pScrollBarV->SetTotalScrollSize(ivc.maxVert);
	pScrollBarV->SetVisibleScrollSize(ivc.pageVert);
	pScrollBarV->SetScrollPosition(ivc.posVert);
}


//====================================================================
void  InterleavedRenderPage::ApplyControlValues()
{
	InterleavedViewConfig ivc;
	ivc.posHorz = pScrollBarH->GetScrollPosition();
	ivc.posVert = pScrollBarV->GetScrollPosition();

	appTest1.SetInterleavedViewConfig(ivc);
}


//====================================================================
bool  InterleavedRenderPage::OnCommand(uint commandID)
{
	switch(commandID) {
		case CID_RotateView: {
			ApplyControlValues();
			return true;
		}
	}
	return PropertyPage::OnCommand(commandID);
}


//====================================================================
MiscPage::MiscPage(app_test1& app, gui_cs0::PropertyTab* _parent)
	:
	AppTest1Page(app, _parent)
{
	using namespace gui::layouters;

	// create components
	pLayouter = new gui::layouters::RowLayouter(this);
	RowLayouter::GeomParams gpRL;
	gpRL.bAutoFitToParentOnRecalcLayout = true;
	pLayouter->SetGeomParams(gpRL);

	// screenshot
	pCGFFormat = new ControlGroupFrame(pLayouter, "Screenshot");
	RowLayouter::GeomParams gpCGF;
	gpCGF.preferredSize = vec2(140, 160);
	pCGFFormat->RowLayouter::SetGeomParams(gpCGF);

	pRadioJpeg = new RadioButton(pCGFFormat, this, CID_ControlValuesChanged);
	pRadioTiff = new RadioButton(pCGFFormat, this, CID_ControlValuesChanged);
	pRadioPng = new RadioButton(pCGFFormat, this, CID_ControlValuesChanged);
	pRadioBmp = new RadioButton(pCGFFormat, this, CID_ControlValuesChanged);
	pLabelJpeg = new Label(pCGFFormat, 0, 0, "Jpeg", pRadioJpeg);
	pLabelTiff = new Label(pCGFFormat, 0, 0, "Tiff", pRadioTiff);
	pLabelPng = new Label(pCGFFormat, 0, 0, "Png", pRadioPng);
	pLabelBmp = new Label(pCGFFormat, 0, 0, "Bmp", pRadioBmp);

	// combo-box tests
	pLayouterCombo = new gui::layouters::RowLayouter(pLayouter);
	pLayouterCombo->SetGeomParams(gpCGF);

	ComboBox::GeomParams gpCombo;

	pComboBottom = new gui_cs0::ComboBox(pLayouterCombo);
	gpCombo.panePosition = ComboBox::PanePosition_Bottom;
	pComboBottom->SetGeomParams(gpCombo);

	pComboCenter = new gui_cs0::ComboBox(pLayouterCombo);
	gpCombo.panePosition = ComboBox::PanePosition_Center;
	pComboCenter->SetGeomParams(gpCombo);

	pComboTop = new gui_cs0::ComboBox(pLayouterCombo);
	gpCombo.bOpenButtonRightAlign = false;
	gpCombo.panePosition = ComboBox::PanePosition_Top;
	pComboTop->SetGeomParams(gpCombo);

	FillComboBoxes(true);

	// some buttons to control other stuff
	TextButton::GeomParams gpText;
	gpText.minSize.y = 44;

	pButtonEnableRadio = new gui_cs0::TextButton(pLayouter, this, CID_EnableRadio, gui::Button::Style_OnOff, 0, "Enable\nradio");
	pButtonEnableRadio->SetGeomParams(gpText);
	pButtonEnableRadio->SetOn(true);

	pButtonEnableCombo = new gui_cs0::TextButton(pLayouter, this, CID_EnableCombo, gui::Button::Style_OnOff, 0, "Enable\ncombos");
	pButtonEnableCombo->SetGeomParams(gpText);
	pButtonEnableCombo->SetOn(true);

	pButtonClearCombo = new gui_cs0::TextButton(pLayouter, this, CID_ClearCombo, gui::Button::Style_Push, 0, "Clear\ncombos");
	pButtonClearCombo->SetGeomParams(gpText);

	pButtonFillCombo = new gui_cs0::TextButton(pLayouter, this, CID_FillCombo, gui::Button::Style_Push, 0, " Fill\ncombos");
	pButtonFillCombo->SetGeomParams(gpText);

	// specify layout
	pLayouter->TargetHeightRow(pCGFFormat).Target(pCGFFormat).Spacer().Target(pLayouterCombo, SF_Variable);
	pLayouter->FixedHeightRow();
	pLayouter->TargetHeightRow(pButtonEnableRadio).AutoSpacer().Target(pButtonEnableRadio).AutoSpacer();
	pLayouter->LastRow().Target(pButtonEnableCombo).AutoSpacer();
	pLayouter->LastRow().Target(pButtonClearCombo).AutoSpacer().Target(pButtonFillCombo).AutoSpacer();

	pCGFFormat->FixedHeightRow(20);
	pCGFFormat->FixedHeightRow().Spacer().Target(pRadioJpeg).Spacer().Target(pLabelJpeg);
	pCGFFormat->FixedHeightRow().Spacer().Target(pRadioTiff).Spacer().Target(pLabelTiff);
	pCGFFormat->FixedHeightRow().Spacer().Target(pRadioPng).Spacer().Target(pLabelPng);
	pCGFFormat->FixedHeightRow().Spacer().Target(pRadioBmp).Spacer().Target(pLabelBmp);

	pLayouterCombo->AutoHeightRow();
	pLayouterCombo->TargetHeightRow(pComboBottom).Target(pComboBottom, SF_Variable);
	pLayouterCombo->FixedHeightRow();
	pLayouterCombo->TargetHeightRow(pComboCenter).Target(pComboCenter, SF_Variable);
	pLayouterCombo->FixedHeightRow();
	pLayouterCombo->TargetHeightRow(pComboTop).Target(pComboTop, SF_Variable);
	pLayouterCombo->AutoHeightRow();
}


//====================================================================
void  MiscPage::UpdateControlDependencies()
{
	// pull application-states
	using namespace signal_codec_parameters;

	MiscConfig mc;
	appTest1.GetMiscConfig(mc);

	const std::string& s = mc.screenshotFormat;

	if(s == Value_ImageFileFormat_JPEG) pRadioJpeg->Select(false);
	else if(s == Value_ImageFileFormat_TIFF) pRadioTiff->Select(false);
	else if(s == Value_ImageFileFormat_PNG) pRadioPng->Select(false);
	else if(s == Value_ImageFileFormat_BMP) pRadioBmp->Select(false);
}


//====================================================================
void  MiscPage::ApplyControlValues()
{
	// set application-states
	using namespace signal_codec_parameters;
	std::string s = Value_ImageFileFormat_JPEG;

	if(pRadioTiff->IsSelected()) s = Value_ImageFileFormat_TIFF;
	else if(pRadioPng->IsSelected()) s = Value_ImageFileFormat_PNG;
	else if(pRadioBmp->IsSelected()) s = Value_ImageFileFormat_BMP;

	MiscConfig mc(s);
	appTest1.SetMiscConfig(mc);
}


//====================================================================
bool  MiscPage::OnCommand(uint commandID)
{
	switch(commandID) {
		case CID_EnableRadio: {
			bool b = pButtonEnableRadio->IsOn();
			pRadioJpeg->SetEnabled(b);
			pRadioTiff->SetEnabled(b);
			pRadioPng->SetEnabled(b);
			pRadioBmp->SetEnabled(b);
			pLabelJpeg->SetEnabled(b);
			pLabelTiff->SetEnabled(b);
			pLabelPng->SetEnabled(b);
			pLabelBmp->SetEnabled(b);
			return true;
		}
		case CID_EnableCombo: {
			bool b = pButtonEnableCombo->IsOn();
			pComboBottom->SetEnabled(b);
			pComboCenter->SetEnabled(b);
			pComboTop->SetEnabled(b);
			return true;
		}
		case CID_ClearCombo:
			FillComboBoxes(false);
			return true;

		case CID_FillCombo:
			FillComboBoxes(true);
			return true;
	}
	return PropertyPage::OnCommand(commandID);
}


//====================================================================
void  MiscPage::FillComboBoxes(bool bFill)
{
	pComboBottom->DeleteAllItems();
	pComboCenter->DeleteAllItems();
	pComboTop->DeleteAllItems();

	if(! bFill) return;

	uint maxItems = 10;

	{for(uint i = 0; i < UnitRandom<float>() * maxItems; i++) {
		pComboBottom->AddItem(new ComboBoxItem("Item Text " + ToString(i) + " BottomOpen RightButton"));
	}}
	{for(uint i = 0; i < UnitRandom<float>() * maxItems; i++) {
		pComboCenter->AddItem(new ComboBoxItem("Item Text " + ToString(i) + " CenterOpen RightButton"));
	}}
	{for(uint i = 0; i < UnitRandom<float>() * maxItems; i++) {
		pComboTop->AddItem(new ComboBoxItem("Item Text " + ToString(i) + " TopOpen LeftButton"));
	}}
}


//====================================================================
OptionsDialog::OptionsDialog(app_test1& app, const std::string& title)
	:
	Dialog(title),
	appTest1(app)
{
	// create property-frame which serves as a container for property-tabs
	pFrame = new gui_cs0::PropertyFrame(this);

	// create property-tab / property-page pairs and attach them to property-frame
	pTabView = new gui_cs0::PropertyTab(pFrame, this, CID_Selected, "View");
	pPageView = new ViewPage(appTest1, pTabView);

	pTabSampling = new gui_cs0::PropertyTab(pFrame, this, CID_Selected, "Sampling");
	pPageSampling = new SamplingPage(appTest1, pTabSampling);

	pTabInterleavedRender = new gui_cs0::PropertyTab(pFrame, this, CID_Selected, "Interleaved render");
	pPageInterleavedRender = new InterleavedRenderPage(appTest1, pTabInterleavedRender);

	pTabMisc = new gui_cs0::PropertyTab(pFrame, this, CID_Selected, "Misc");
	pPageMisc = new MiscPage(appTest1, pTabMisc);

	// choose initially opened tab
	pTabView->Select();

	// setup dialog params
	TopLevelFrame::GeomParams  gpTLF = TopLevelFrame::GetGeomParams();
	gpTLF.bResizable = true;
	gpTLF.minSize = vec2(500, 430);
	gpTLF.maxSize = vec2(640, 480);
	TopLevelFrame::SetGeomParams(gpTLF);

	SetSize(gpTLF.minSize);
	SetDialogSubcomponentLayout(true, true, true, false);
}


//====================================================================
bool  OptionsDialog::OnCommand(uint commandID)
{
	switch(commandID) {
		// disable apply-button after different page is selected
		case CID_Selected:
			DisableDialogApplyButton();
			return true;
	}
	return Dialog::OnCommand(commandID);
}


//====================================================================
void  OptionsDialog::RecalcLayout()
{
	Dialog::RecalcLayout();

	// position the property-frame within dialog
	pFrame->SetSize(CalcDialogWorkAreaSize());
	pFrame->SetPosition(CalcDialogWorkAreaOffset());
}


//====================================================================
void  OptionsDialog::OnDialogOpened()
{
	// update control dependencies on all pages
	pFrame->UpdateControlDependencies(false);
}


//====================================================================
void  OptionsDialog::OnApplySettings()
{
	// apply settings from selected page only
	pFrame->ApplyControlValues(true);
}

