

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Layouters.h"
#include "GUI_CS0\Clock.h"
#include "GUI_CS0\Dialog.h"
#include "GUI_CS0\PropertyFrame.h"
#include "GUI_CS0\ToolBar.h"
#include "GUI_CS0\TopLevelPane.h"
#include "GUI_CS0\ScrollBar.h"
#include "private\AppCommon.h"


//--------------------------------------------------------------------
using namespace vertexware;
using namespace vertexware::gui;
using namespace vertexware::gui_cs0;


//--------------------------------------------------------------------
class app_test1;


//====================================================================
// All buttons will send the same command-id to target.
//====================================================================
class ToolBarTest : public gui_cs0::ToolBar, public gui::CommandSender
{
public:
	ToolBarTest(gui::MessageHandler* pCommandTarget = 0,
				uint commandID = 0,
				bool bVertical = true,
				const std::string& title = "ToolBarTest");

protected:
	enum {
		CID_ButtonSelect = gui::CID_User
	};

	virtual  bool  ProcessToolBarControlCommand(uint commandID);

protected:
	Button*  pButtonsS0[4];
	Button*  pButtonsOO[3];
	Button*  pButtonsS1[4];
	Button*  pButtonsP[3];
};


//====================================================================
// Horizontal toolbar / icon-button test
//====================================================================
class ToolBarTest1 : public ToolBarTest
{
public:
	ToolBarTest1(gui::MessageHandler* pCommandTarget = 0,
				uint commandID = 0,
				const std::string& title = "ToolBarTest1");
};


//====================================================================
// Vertical toolbar / text-button test
//====================================================================
class ToolBarTest2 : public ToolBarTest
{
public:
	ToolBarTest2(gui::MessageHandler* pCommandTarget = 0,
				uint commandID = 0,
				const std::string& title = "TBTest2");
};


//====================================================================
// Clock test
//====================================================================
class ClockTestPane : public gui_cs0::TopLevelPane
{
public:
	ClockTestPane();

protected:
	virtual  void  RecalcLayout();

protected:
	gui_cs0::Clock*  pClock;
};


//====================================================================
// Tests dynamic-component-creation / dynamic-shader-change
//====================================================================
class DynamicTestPane : public gui_cs0::TopLevelPane, public gui::CommandSender
{
public:
	DynamicTestPane(gui::MessageHandler* pCommandTarget,
					uint commandID = CID_DestroyDynamicTestPane,
					const std::string& title = "DynamicTestPane");

	void  SetShaders(bool bUseOverlayShaders = true);

protected:
	enum { CID_UseOverlayShaders = gui::CID_User };

	virtual  bool  OnCommand(uint commandID);
	virtual  void  OnPaneClosed();

protected:
	gui::layouters::RowLayouter*  pLayouter;

	gui_cs0::Label*     pLabelUseOverlayShaders;
	gui_cs0::CheckBox*  pCheckBoxUseOverlayShaders;

	gui_cs0::TextButton*  pTextButtonPress1;
	gui_cs0::TextButton*  pTextButtonPress2;
	gui_cs0::TextButton*  pTextButtonPush1;
	gui_cs0::TextButton*  pTextButtonPush2;
	gui_cs0::TextButton*  pTextButtonToggle1;
	gui_cs0::TextButton*  pTextButtonToggle2;

	gui_cs0::IconButton*  pIconButtonPress1;
	gui_cs0::IconButton*  pIconButtonPress2;
	gui_cs0::IconButton*  pIconButtonPush1;
	gui_cs0::IconButton*  pIconButtonPush2;
	gui_cs0::IconButton*  pIconButtonToggle1;
	gui_cs0::IconButton*  pIconButtonToggle2;

	gui_cs0::CheckBox*  pCheckBox1;
	gui_cs0::CheckBox*  pCheckBox2;
};


//====================================================================
class AppTest1Page : public gui_cs0::PropertyPage
{
public:
	AppTest1Page(app_test1& app, gui_cs0::PropertyTab* parent = 0);

protected:
	app_test1&  appTest1;
};


//====================================================================
class ViewPage : public AppTest1Page
{
public:
	ViewPage(app_test1& app, gui_cs0::PropertyTab* parent = 0);

protected:
	virtual  void  UpdateControlDependencies();
	virtual  void  ApplyControlValues();

	gui::layouters::RowLayouter*  pLayouter;

	gui_cs0::ControlGroupFrame*   pCGFView;
	gui_cs0::Label*               pLabelNone;
	gui_cs0::Label*               pLabelKeyboard;
	gui_cs0::Label*               pLabelMouse;
	gui_cs0::Label*               pLabelGameController;
	gui_cs0::RadioButton*         pRadioNone;
	gui_cs0::RadioButton*         pRadioKeyboard;
	gui_cs0::RadioButton*         pRadioMouse;
	gui_cs0::RadioButton*         pRadioGameController;
	gui_cs0::CheckBox*            pCheckBoxReverseVertAxis;
	gui_cs0::Label*               pLabelReverseVertAxis;

	gui_cs0::ControlGroupFrame*   pCGFTracker;
	gui_cs0::CheckBox*            pCheckBoxCombineTracker;
	gui_cs0::Label*               pLabelCombineTracker;
	gui_cs0::CheckBox*            pCheckBoxReverseHead;
	gui_cs0::Label*               pLabelReverseHead;
	gui_cs0::CheckBox*            pCheckBoxReversePitch;
	gui_cs0::Label*               pLabelReversePitch;
	gui_cs0::CheckBox*            pCheckBoxReverseRoll;
	gui_cs0::Label*               pLabelReverseRoll;
};


//====================================================================
class SamplingPage : public AppTest1Page
{
public:
	SamplingPage(app_test1& app, gui_cs0::PropertyTab* parent = 0);

protected:
	virtual  void  UpdateControlDependencies();
	virtual  void  ApplyControlValues();

	void  CreateGroup(uint i, const std::string& title);

protected:
	gui::layouters::RowLayouter*  pLayouter;
	gui_cs0::HorzTextSeparator*   pHTS[AsyncController_Count];
	gui_cs0::Label*               pLabelAsync[AsyncController_Count];
	gui_cs0::CheckBox*            pCheckBoxAsync[AsyncController_Count];
	gui_cs0::Label*               pLabelLimitRate[AsyncController_Count];
	gui_cs0::CheckBox*            pCheckBoxLimitRate[AsyncController_Count];
	gui_cs0::Label*               pLabelMaxRate[AsyncController_Count];
	gui_cs0::EditBox*             pEditBoxMaxRate[AsyncController_Count];
};


//====================================================================
class InterleavedRenderPage : public AppTest1Page
{
public:
	InterleavedRenderPage(app_test1& app, gui_cs0::PropertyTab* parent = 0);

protected:
	virtual  bool  OnCommand(uint commandID);
	virtual  void  UpdateControlDependencies();
	virtual  void  ApplyControlValues();

	enum {
		CID_RotateView = gui::CID_User
	};

protected:
	gui::layouters::RowLayouter*  pLayouter;
	gui::InterleavedRenderArea*   pArea;
	gui_cs0::ScrollBar*           pScrollBarV;
	gui_cs0::ScrollBar*           pScrollBarH;
};


//====================================================================
class MiscPage : public AppTest1Page
{
public:
	MiscPage(app_test1& app, gui_cs0::PropertyTab* parent = 0);

protected:
	virtual  bool  OnCommand(uint commandID);
	virtual  void  UpdateControlDependencies();
	virtual  void  ApplyControlValues();

	void  FillComboBoxes(bool bFill);

	enum {
		CID_EnableRadio = gui::CID_User,
		CID_EnableCombo,
		CID_ClearCombo,
		CID_FillCombo
	};

protected:
	gui::layouters::RowLayouter*  pLayouter;
	gui::layouters::RowLayouter*  pLayouterCombo;

	gui_cs0::ControlGroupFrame*   pCGFFormat;
	gui_cs0::Label*               pLabelJpeg;
	gui_cs0::Label*               pLabelTiff;
	gui_cs0::Label*               pLabelPng;
	gui_cs0::Label*               pLabelBmp;
	gui_cs0::RadioButton*         pRadioJpeg;
	gui_cs0::RadioButton*         pRadioTiff;
	gui_cs0::RadioButton*         pRadioPng;
	gui_cs0::RadioButton*         pRadioBmp;

	gui_cs0::ComboBox*  pComboBottom;
	gui_cs0::ComboBox*  pComboCenter;
	gui_cs0::ComboBox*  pComboTop;

	gui_cs0::TextButton*  pButtonEnableRadio;
	gui_cs0::TextButton*  pButtonEnableCombo;
	gui_cs0::TextButton*  pButtonClearCombo;
	gui_cs0::TextButton*  pButtonFillCombo;
};


//====================================================================
class OptionsDialog : public gui_cs0::Dialog
{
public:
	OptionsDialog(app_test1& app, const std::string& title = "Options");

protected:
	virtual  bool  OnCommand(uint commandID);
	virtual  void  RecalcLayout();
	virtual  void  OnDialogOpened();
	virtual  void  OnApplySettings();

protected:
	app_test1&  appTest1;

	gui_cs0::PropertyFrame*  pFrame;
	gui_cs0::PropertyTab*    pTabView;
	gui_cs0::PropertyTab*    pTabSampling;
	gui_cs0::PropertyTab*    pTabInterleavedRender;
	gui_cs0::PropertyTab*    pTabMisc;
	ViewPage*                pPageView;
	SamplingPage*            pPageSampling;
	InterleavedRenderPage*   pPageInterleavedRender;
	MiscPage*                pPageMisc;
};


