

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
// Include everything
//--------------------------------------------------------------------
#include "vw_common.h"
#include "vw_opengl.h"
#include "vw_direct3d8.h"
#include "vw_gui_cs0.h"
#include "vw_gui_ss0.h"


//--------------------------------------------------------------------
// Additional library-header, must be included manually because it
// contains unprefixed macros:
//    olog <-- shorthand notation for GetLogOut()
//    ocon <-- shorthand notation for GetConOut()
//--------------------------------------------------------------------
#include "Framework\KernelMacros.h"


//--------------------------------------------------------------------
// Plugin-specific headers
//--------------------------------------------------------------------
#include "PluginParams\PluginParams_vs_direct3d8.h"


//--------------------------------------------------------------------
// Application-specific headers
//--------------------------------------------------------------------
#include "private\AppCommon.h"
#include "private\GUITests.h"


//--------------------------------------------------------------------
using namespace vertexware;
using namespace vertexware::gui;
using namespace vertexware::gui_cs0;
using namespace vertexware::gui_ss0;


//====================================================================
// Custom vertex-formats for background rendering.
//====================================================================
struct Vertex
{
	vec3   position;
	vec3   normal;
	vec2   texCoords;
	float  alpha;
};


//====================================================================
struct VertexD3D8
{
	enum {
		FVF = D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_DIFFUSE | D3DFVF_TEX1 | D3DFVF_TEXCOORDSIZE2(0)
	};
	vec3   position;
	vec3   normal;
	DWORD  diffuse;
	vec2   texCoords;
};


//====================================================================
const uint gc_NumImages = 5;


//====================================================================
class app_test1 : public Application
{
public:
	app_test1();
	virtual ~app_test1();

	//--------------------------------------
	// Methods used for dialog communication
	//--------------------------------------
	void  GetViewConfig(ViewConfig& vc, bool& bHasGameController, bool& bHasOrientationTracker) const;
	void  SetViewConfig(const ViewConfig& vc);

	void  GetAsyncConfig(AsyncSampleConfig asConfig[AsyncController_Count]) const;
	void  SetAsyncConfig(const AsyncSampleConfig asConfig[AsyncController_Count]);

	void  GetInterleavedViewConfig(InterleavedViewConfig& ivc) const;
	void  SetInterleavedViewConfig(const InterleavedViewConfig& ivc);

	void  GetMiscConfig(MiscConfig& mc) const;
	void  SetMiscConfig(const MiscConfig& mc);

	void  DrawBackgroundInterleaved(const vec2& pos, const vec2& size);

	//----------------------
	// Application overrides
	//----------------------
	virtual  UID   GetApplicationUID();
	virtual  void  BeginStartup(const StartupInfo& si, StartupResult& sr);
	virtual  void  QueryAutoInit(AutoInitInfo& aii);
	virtual  void  Initialize();
	virtual  void  EndStartup();
	virtual  void  Cleanup(bool bCleanExit, const FailureDesc& fd);

	virtual  bool  ConfirmGenericExtension(const std::string& pluginName,
							const std::string& genericExtensionInterfaceName);
	virtual  void  ConnectGenericExtensions();

	virtual  bool  ConfirmDirectInputVersion(uint directInputVersion);

	virtual  bool  ConfirmHIDDriver(const std::string& pluginName,
									uint defaultDriverPriority,
									uint& outDriverPriority);

	virtual  void  QueryHIDConfig(DirectInputConfig& diConfig,
								uint iReconfigCount, uint reconfigCode);

	virtual  bool  QueryIgnoreHIDDriver(const std::string& pluginName);
	virtual  void  BeginConfirmHumanInterfaceDevices();
	virtual  bool  ConfirmHumanInterfaceDevice(HumanInterfaceDevice& hid);
	virtual  void  EndConfirmHumanInterfaceDevices();
	virtual  void  BeginConnectHumanInterfaceDevices(const DirectInputConfig& diConfig);
	virtual  void  ConnectHumanInterfaceDevice(HumanInterfaceDevice& hid);
	virtual  void  EndConnectHumanInterfaceDevices();
	virtual  void  SetupAsyncHIDSampling();
	virtual  void  DisconnectHumanInterfaceDevices();

	virtual  void  FilterVideoProfile(const VideoProfile& fullProfile,
									VideoProfile& outFilteredProfile);

	virtual  bool  ConfirmVideoRenderer(const std::string& pluginName,
										const std::string& interfaceName);

	virtual  void  QueryVideoConfig(std::string& pluginName, VideoConfig& vc,
									uint iReconfigCount, uint reconfigCode);

	virtual  void  PostCreateMainWindow(MainWindowProperties& mwp, MainWindowGeometry& mwg,
								const std::string& vrPluginName, const VideoConfig& vc,
								uint iCreateCount);

	virtual  void  PreDestroyMainWindow(const MainWindowGeometry& mwg,
								const std::string& vrPluginName, const VideoConfig& vc);

	virtual  void  OnVSyncUpdate(bool bVSyncEnabled);

	virtual  void  ConnectVideoRenderer(VideoRenderer& vr);
	virtual  void  DisconnectVideoRenderer();
	virtual  void  InitVideoRendererStates(bool bPostReset);
	virtual  void  CreateVideoResources(bool bPostReset);
	virtual  void  ReleaseVideoResources(bool bPreReset);

	virtual  void  GenericReconfig(uint iReconfigCount, uint reconfigCode);

	virtual  void  InitializeSubsystems(uint iReconfigCount, uint reconfigCode);
	virtual  void  CleanupSubsystems(uint iReconfigCount, uint reconfigCode);
	virtual  void  ConnectSubsystemsToAppWindow(HWND hwnd, bool bFullscreen);
	virtual  void  DisconnectSubsystemsFromAppWindow();
	virtual  bool  CheckSubsystems();
	virtual  void  FixSubsystems();

	virtual  void  OnKernelLevelChanged(eKernelLevel activeLevel);

	virtual  void  OnWindowActive(bool bActive);
	virtual  void  OnWindowResize(uint width, uint height);

	virtual  void  FrameBegin(double secsFrameTimeDelta);
	virtual  void  FrameSampleInput();
	virtual  void  FrameUpdateStates(double secsFrameTimeDelta);
	virtual  void  FramePreRenderVideo();
	virtual  void  FrameRenderVideo();
	virtual  void  FrameRenderArea(const gui::InterleavedRenderArea& area);
	virtual  void  FramePostRenderVideo();
	virtual  void  FrameEnd();

	virtual  bool  GUIOnCommand(uint commandID);
	virtual  bool  GUIOnActivate(bool bActivate, bool bByMouse);
	virtual  bool  GUIOnMouseCapture(bool bCaptureBegin);
	virtual  bool  GUIOnMouseEnterExit(bool bEnter, const vec2& mousePos);
	virtual  bool  GUIOnMouseQueryCursor(const gui::Cursor* &pCursor, const vec2& mousePos);
	virtual  bool  GUIOnMouseMove(bool bHover, double hoverTimeSecs, const vec2& mousePos);
	virtual  bool  GUIOnMouseWheel(int delta, const vec2& mousePos);
	virtual  bool  GUIOnMouseButton(bool bDown, eMouseButton buttonCode, bool bDoubleClick, uint repeatCount, const vec2& mousePos);
	virtual  bool  GUIOnKeyboardQueryFocus(bool bFocusByMouse, eMouseButton buttonCode, bool& bWantFocus, bool& bNeedCharacterData);
	virtual  bool  GUIOnKeyboardFocus(bool bGotFocus);
	virtual  bool  GUIOnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch);


private:
	// various
	void  AddEventToLog(const std::string& eventName);
	void  SyncGUIWithEventLogStates();
	void  SyncGUIWithVideoRendererConfig();
	void  UpdateLightColorFromGUI();

	// rendering
	void  CreateBackgroundTextures();
	void  ReleaseBackgroundTextures();

	void  DrawBackground();
	void  DrawBackgroundInterleaved();

	void  GenerateSphereSectionVertices(
							AutoPtr<Vertex>& dst,
							uint subdivH, uint subdivV,
							float maxAngleH, float maxAngleV,
							float radius);

	void  GenerateSphereSectionVerticesD3D8(
							const AutoPtr<Vertex>& src,
							AutoPtr<VertexD3D8>& dst,
							AutoPtr<float>& dstAlpha,
							uint& vertexCount,
							uint subdivH, uint subdivV);

	void  FeedSphereSectionVerticesGL(
							const AutoPtr<Vertex>& src,
							uint subdivH, uint subdivV,
							color4f color);

	void  UpdateSphereSectionVertexAlphaD3D8(
							AutoPtr<VertexD3D8>& dst,
							const AutoPtr<float>& refAlpha,
							uint vertexCount,
							float alphaMultiplier);

	void  FeedSphereSectionVertexBufferD3D8(
							IDirect3DDevice8& dev,
							const AutoPtr<VertexD3D8>& src,
							uint subdivH, uint subdivV);


private:
	// config backtrack
	MainWindowGeometry  lastMainWindowGeom;
	VideoConfig         lastVideoConfig;

	// misc config
	MiscConfig   miscConfig;
	uint         iScreenShotSerial;
	std::string  sProfileDirectoryName;

	// worker objects
	AutoPtr<FrameStatisticsCollector>              apFrameStatsCollector;
	AutoPtr<std_command_interpreter::Interpreter>  apCommandInterpreter;

	// gui components (deleted by gui-manager on exit)
	ToolTip*  pToolTip;

	MenuBar*        pMenuBar;
	MenuTitle*      pMenuTitleSystem;
	MenuPane*       pMenuPaneSystem;
	MenuItem*       pMenuItemFrameStats;
	MenuItem*       pMenuItemConsole;
	MenuItem*       pMenuItemProfiler;
	MenuItem*       pMenuItemExit;
	MenuSeparator*  pMenuSeparatorSystem1;

	MenuTitle*      pMenuTitleConfig;
	MenuPane*       pMenuPaneConfig;
	MenuItem*       pMenuItemVSync;
	MenuItem*       pMenuItemVideoConfig;
	MenuItem*       pMenuItemPixelOffset;
	MenuSeparator*  pMenuSeparatorConfig1;
	MenuItem*       pMenuItemInputConfig;
	MenuItem*       pMenuItemHIDReconfigCycle;
	MenuItem*       pMenuItemHIDDrivers;
	MenuPane*       pMenuPaneHIDDrivers;
	MenuItem*       pMenuItemHIDDriverGenericGameController;
	MenuItem*       pMenuItemHIDDriverInterSenseTracker;
	MenuItem*       pMenuItemHIDDriver5DTDataGlove;
	MenuItem*       pMenuItemControllerStates;
	MenuPane*       pMenuPaneControllerStates;
	MenuItem*       pMenuItemControllerStateMouse;
	MenuItem*       pMenuItemControllerStateKeyboard;
	MenuItem*       pMenuItemControllerStateGameController;
	MenuItem*       pMenuItemControllerStateOrientationTracker;
	MenuItem*       pMenuItemControllerStatePositionTracker;
	MenuItem*       pMenuItemControllerStateGlove;
	MenuSeparator*  pMenuSeparatorConfig2;
	MenuItem*       pMenuItemVSD3D8;
	MenuPane*       pMenuPaneVSD3D8;
	MenuItem*       pMenuItemVSD3D8Cleanup;

	MenuTitle*      pMenuTitleTools;
	MenuPane*       pMenuPaneTools;
	MenuItem*       pMenuItemFontTest;
	MenuItem*       pMenuItemClockTest;
	MenuItem*       pMenuItemToolBarTest1;
	MenuItem*       pMenuItemToolBarTest2;
	MenuSeparator*  pMenuSeparatorTools1;
	MenuItem*       pMenuItemDynamicTestPane;
	MenuItem*       pMenuItemSubsysReconfig;

	MenuTitle*      pMenuTitleDemo;
	MenuPane*       pMenuPaneDemo;
	MenuItem*       pMenuItemOptions;
	MenuItem*       pMenuItemAbout;

	MenuPane*       pMenuPaneContext;
	MenuItem*       pMenuItemLightWhite;
	MenuItem*       pMenuItemLightRed;
	MenuItem*       pMenuItemLightGreen;
	MenuItem*       pMenuItemLightBlue;
	MenuSeparator*  pMenuSeparatorContext1;
	MenuItem*       pMenuItemEnableRubberband;
	MenuSeparator*  pMenuSeparatorContext2;
	MenuItem*       pMenuItemEventLogShow;
	MenuItem*       pMenuItemEventLogQueryCursor;
	MenuItem*       pMenuItemEventLogMouseMove;

	RubberBand*              pRubberBand;
	Console*                 pConsole;
	ProfilerControlDialog*   pProfilerControlDialog;
	FrameStatisticsPane*     pFrameStatsPane;
	VideoConfigDialog*       pVideoConfigDialog;
	PixelOffsetControlPane*  pPixelOffsetControlPane;
	HIDConfigDialog*         pHIDConfigDialog;
	FontTest*                pFontTest;
	ClockTestPane*           pClockTestPane;
	ToolBar*                 pToolBarTest1;
	ToolBar*                 pToolBarTest2;
	DynamicTestPane*         pDynamicTestPane;
	OptionsDialog*           pOptionsDialog;
	AboutDialog*             pAboutDialog;

	MouseStatePane*               pMouseStatePane;
	KeyboardStatePane*            pKeyboardStatePane;
	GameControllerStatePane*      pGameControllerStatePane;
	OrientationTrackerStatePane*  pOrientationTrackerStatePane;
	PositionTrackerStatePane*     pPositionTrackerStatePane;
	GloveStatePane*               pGloveStatePane;

	// cached pointers
	const gui::Font* pFont;

	// help text
	bool                 bShowHelp;
	float                fHelpSpeed;
	std::string          sHelpText;
	gui::DrawTextParams  dtpHelpText;
	vec2                 posHelpText;
	RectangleC           rHelpTextBgr;

	// info-labels
	bool                 bShowInfoLabels;
	vec3                 speedInfoLabels;
	float                fInfoLabelAlpha;
	float                fInfoLabelAlphaSpeed;
	std::string          sRenderer;
	std::string          sActiveController;
	std::string          sHeadTracker;
	gui::DrawTextParams  dtpInfoLabels;
	vec3                 posInfoLabels;

	// hpr-label
	bool                 bShowHPRLabel;
	std::string          sHPR;
	gui::DrawTextParams  dtpHPRLabel;

	// event-log
	bool                    bEventLogShow;
	bool                    bEventLogQueryCursor;
	bool                    bEventLogMouseMove;
	std::list<std::string>  eventLog;
	uint                    maxLogEvents;
	gui::DrawTextParams     dtpEventLog;
	gui::RectangleC         rEventLogBgr;

	// rubberband trail
	bool             bDrawRubberbandTrail;
	color4f          rubberbandTrailColor;
	gui::RectangleC  rRubberbandTrail;

	// animation variables
	float    lightIntensity, lightIntensitySpeed;
	float    bgr0Alpha, bgr0AlphaSpeed;
	color4f  lightColor;

	// transforms, rotations represents view-direction-HPR in standard RHS
	InterleavedViewConfig  interleavedViewConfig;
	ViewConfig             viewConfig;
	vec3                   bgr0Rot, bgr0RotSpeed;
	vec3                   bgr1Rot, bgr1RotSpeed;
	vec3                   trackerRot, trackerU, trackerV, trackerN;
	vec3                   finalViewRot;

	// master image-storage
	AutoPtr<ImageBase>  apImage[gc_NumImages];

	// master vertex-data (RHS, tex-coords OpenGL style)
	AutoPtr<Vertex>  apVerticesSphereSection0;
	AutoPtr<Vertex>  apVerticesSphereSection1;
	AutoPtr<Vertex>  apVerticesSphereSection2;

	// OpenGL resources
	AutoPtr<gl_tools::Texture2D>  apTextureGL[gc_NumImages];

	// Direct3D8 resources
	AutoPtr<d3d8_tools::Texture2D>  apTextureD3D8[gc_NumImages];
	AutoPtr<VertexD3D8>             apUPVertexBufferD3D8[3];
	AutoPtr<float>                  apVertexAlphaD3D8[3];
	uint                            vertexCountD3D8[3];
	D3DLIGHT8                       lightD3D8;

	// renderer connections
	VideoRenderer*           pVRend;
	VideoRendererOpenGL*     pVRendGL;
	VideoRendererDirect3D8*  pVRendD3D8;
	gui::Renderer*           pGRend;

	// available input devices
	Mouse*               pMouse;
	Keyboard*            pKeyboard;
	GameController*      pGameController;
	PositionTracker*     pPositionTracker;
	OrientationTracker*  pOrientationTracker;
	Glove*               pGlove;

	// async sampling configurations
	AsyncSampleConfig   asyncSampleConfigs[AsyncController_Count];
};

