

//--------------------------------------------------------------------
#include "private\hidd0_intersense_tracker.h"
#include "private\InterSenseOrientationTracker.h"
#include "BaseTools\Stream.h"
#include "Framework\KernelMacros.h"

using namespace vertexware;


//====================================================================
#define  DEFAULT_DRIVER_PRIORITY  555444


//====================================================================
hidd0_intersense_tracker::hidd0_intersense_tracker()
{
}


//====================================================================
hidd0_intersense_tracker::~hidd0_intersense_tracker()
{
}


//====================================================================
uint  hidd0_intersense_tracker::GetDefaultDriverPriority()
{
	return DEFAULT_DRIVER_PRIORITY;
}


//====================================================================
void  hidd0_intersense_tracker::ClearCache()
{
	availableTrackers.clear();
}


//====================================================================
bool  hidd0_intersense_tracker::CanHandleDevice(const DirectInputDeviceInstanceInfo& didii)
{
	// RevisionCheckPoint:
	// Add GUIDs for other intersense models.

	// ProductGUID: {00010A94-0000-0000-0000-504944564944}
	// ProductName: InterTrax 2 USB V1.0
	if(didii.productGuid == UID(0x00010A94, 0x00000000, 0x00005049, 0x44564944)) {
		return true;
	}

	// ProductGUID: {00641634-0000-0000-0000-504944564944}
	// ProductName: InterSense Joystick
	if(didii.productGuid == UID(0x00641634, 0x00000000, 0x00005049, 0x44564944)) {
		return true;
	}

	// not my device
	return false;
}


//====================================================================
uint  hidd0_intersense_tracker::ProbeDevices()
{
	// open all available trackers
	while(true) {

		// probe next tracker and check result
		ISD_TRACKER_HANDLE  trackerHandle = ISD_OpenTracker(0, 0, FALSE, FALSE);

		if(trackerHandle == 0) {
			// cannot load driver or driver-DLL-s are missing
			olog << e_error << "Cannot load InterSense drivers.\n";
			olog << "Check the file: isense.dll\n" << e_info;
			break;
		}
		if(trackerHandle < 0) {
			// detection failure or maximum number of devices is reached
			break;
		}

		// store tracker handle
		availableTrackers.push_back(trackerHandle);
	}

	// return number of available trackers
	return availableTrackers.size();
}


//====================================================================
HumanInterfaceDevice*  hidd0_intersense_tracker::CreateDevice(uint i)
{
	// create head-tracker object for each available tracker-handle
	HumanInterfaceDevice* hid = new InterSenseOrientationTracker(availableTrackers[i]);
	return hid;
}


