//////////////////////////////////////////////////////////////////////////////
//
//      File Name:      isense.c
//      Description:    DLL access point
//      Created:        12/7/98
//      Author:         Yury Altshuler
//
//      Copyright:      InterSense 2002 - All rights Reserved.
//
//      Comments:        
//
//////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "isense.h"


#if defined(_WIN32) || defined(WIN32) || defined(__WIN32__)
#define ISD_LIB_NAME "isense.dll"
#else
#include <dlfcn.h>
#define ISD_LIB_NAME "libisense"
#endif


typedef void DLL;
DLL *hLib = NULL;


static DLL_EP  dll_entrypoint( DLL *dll, const char *name );
static DLL    *dll_load( const char *name );
static void    dll_unload( DLL *dll );


typedef ISD_TRACKER_HANDLE (*ISD_OPEN_FN)           ( HWND, DWORD, BOOL, BOOL );
typedef DWORD              (*ISD_OPEN_ALL_FN)       ( HWND, ISD_TRACKER_HANDLE *, BOOL, BOOL );
typedef BOOL               (*ISD_COMMAND_FN)        ( ISD_TRACKER_HANDLE );
typedef BOOL               (*ISD_COMM_INFO_FN)      ( ISD_TRACKER_HANDLE, ISD_TRACKER_INFO_TYPE * );
typedef BOOL               (*ISD_SYSTEM_CONFIG_FN)  ( ISD_TRACKER_HANDLE, ISD_TRACKER_INFO_TYPE *, BOOL );
typedef BOOL               (*ISD_STATION_CONFIG_FN) ( ISD_TRACKER_HANDLE, ISD_STATION_INFO_TYPE *, WORD, BOOL );
typedef BOOL               (*ISD_DATA_FN)           ( ISD_TRACKER_HANDLE, ISD_TRACKER_DATA_TYPE * );
typedef BOOL               (*ISD_CAMERA_DATA_FN)    ( ISD_TRACKER_HANDLE, ISD_CAMERA_DATA_TYPE * );
typedef BOOL               (*ISD_SCRIPT_FN)         ( ISD_TRACKER_HANDLE, char * );
typedef BOOL               (*ISD_COUNT_FN)          ( WORD * );
typedef BOOL               (*ISD_RESET_ANGLES_FN)   ( ISD_TRACKER_HANDLE, float, float, float );
typedef BOOL               (*ISD_RESET_HEADING_FN)  ( ISD_TRACKER_HANDLE, WORD );
typedef HWND               (*ISD_WINDOW_FN)         ( ISD_TRACKER_HANDLE );
typedef float              (*ISD_GET_TIME)          ( void );


/* dll function pointers */
ISD_OPEN_FN             _ISD_OpenTracker        = NULL;
ISD_OPEN_ALL_FN         _ISD_OpenAllTrackers    = NULL;
ISD_COMMAND_FN          _ISD_CloseTracker       = NULL;
ISD_COMM_INFO_FN        _ISD_GetCommInfo        = NULL;
ISD_SYSTEM_CONFIG_FN    _ISD_GetTrackerConfig   = NULL;
ISD_SYSTEM_CONFIG_FN    _ISD_SetTrackerConfig   = NULL;
ISD_STATION_CONFIG_FN   _ISD_GetStationConfig   = NULL;
ISD_STATION_CONFIG_FN   _ISD_SetStationConfig   = NULL;
ISD_DATA_FN             _ISD_GetData            = NULL;   
ISD_CAMERA_DATA_FN      _ISD_GetCameraData      = NULL;   
ISD_SCRIPT_FN           _ISD_SendScript         = NULL;
ISD_COUNT_FN            _ISD_NumOpenTrackers    = NULL;
ISD_WINDOW_FN           _ISD_OpenConfigWindow   = NULL;
ISD_RESET_ANGLES_FN     _ISD_ResetAngles        = NULL; 
ISD_RESET_HEADING_FN    _ISD_ResetHeading       = NULL;
ISD_GET_TIME            _ISD_GetTime            = NULL;



/*****************************************************************************
*
*   functionName:   load_ISLIB
*   Description:    loads isense.dll
*   Created:        12/7/98
*   Author:         Yury Altshuler
*
*   Comments:       
*
******************************************************************************/
static DLL *load_ISLIB( void )
{
    if( hLib = dll_load( ISD_LIB_NAME ) )
    {
        _ISD_OpenTracker      = ( ISD_OPEN_FN )           dll_entrypoint( hLib, "ISD_OpenTracker" );
        _ISD_OpenAllTrackers  = ( ISD_OPEN_ALL_FN )       dll_entrypoint( hLib, "ISD_OpenAllTrackers" );
        _ISD_CloseTracker     = ( ISD_COMMAND_FN )        dll_entrypoint( hLib, "ISD_CloseTracker" );
        _ISD_GetCommInfo      = ( ISD_COMM_INFO_FN )      dll_entrypoint( hLib, "ISD_GetCommInfo" );
        _ISD_GetTrackerConfig = ( ISD_SYSTEM_CONFIG_FN )  dll_entrypoint( hLib, "ISD_GetTrackerConfig" );
        _ISD_SetTrackerConfig = ( ISD_SYSTEM_CONFIG_FN )  dll_entrypoint( hLib, "ISD_SetTrackerConfig" );
        _ISD_GetStationConfig = ( ISD_STATION_CONFIG_FN ) dll_entrypoint( hLib, "ISD_GetStationConfig" );
        _ISD_SetStationConfig = ( ISD_STATION_CONFIG_FN ) dll_entrypoint( hLib, "ISD_SetStationConfig" );
        _ISD_GetData          = ( ISD_DATA_FN )           dll_entrypoint( hLib, "ISD_GetData" );
        _ISD_GetCameraData    = ( ISD_CAMERA_DATA_FN )    dll_entrypoint( hLib, "ISD_GetCameraData" );
        _ISD_SendScript       = ( ISD_SCRIPT_FN )         dll_entrypoint( hLib, "ISD_SendScript" );
        _ISD_NumOpenTrackers  = ( ISD_COUNT_FN )          dll_entrypoint( hLib, "ISD_NumOpenTrackers" );
        _ISD_OpenConfigWindow = ( ISD_WINDOW_FN )         dll_entrypoint( hLib, "ISD_OpenConfigWindow" );
        _ISD_ResetAngles      = ( ISD_RESET_ANGLES_FN )   dll_entrypoint( hLib, "ISD_ResetAngles" );
        _ISD_ResetHeading     = ( ISD_RESET_HEADING_FN )  dll_entrypoint( hLib, "ISD_ResetHeading" );
        _ISD_GetTime          = ( ISD_GET_TIME )          dll_entrypoint( hLib, "ISD_GetTime" );
    }

    if( hLib == NULL )
    {
        printf( "Could not load %s\n", ISD_LIB_NAME );
    }

    return hLib;
}


/*****************************************************************************
*
*   functionName:   ISD_FreeLib
*   Description:    frees isense.dll
*   Created:        12/8/98
*   Author:         Yury Altshuler
*
*   Comments:       
*
******************************************************************************/
static void ISD_FreeLib( void )
{
    _ISD_OpenTracker      = NULL;
    _ISD_OpenAllTrackers  = NULL;
    _ISD_CloseTracker     = NULL;
    _ISD_GetCommInfo      = NULL;
    _ISD_GetTrackerConfig = NULL;
    _ISD_SetTrackerConfig = NULL;
    _ISD_GetStationConfig = NULL;
    _ISD_SetStationConfig = NULL;
    _ISD_GetData          = NULL;  
    _ISD_GetCameraData    = NULL;
    _ISD_SendScript       = NULL;
    _ISD_NumOpenTrackers  = NULL;
    _ISD_OpenConfigWindow = NULL;
    _ISD_ResetAngles      = NULL;
    _ISD_ResetHeading     = NULL;
    _ISD_GetTime          = NULL;
   
    if( hLib )
    {
    	dll_unload( hLib );  /* free the dll */
        hLib = NULL;
    }
}


/******************************************************************************/
ISD_TRACKER_HANDLE ISD_OpenTracker( 
                                   HWND hParent,
                                   DWORD commPort, 
                                   BOOL infoScreen, 
                                   BOOL verbose 
                                   )
{
    if( !_ISD_OpenTracker ) /* this will be NULL if dll not loaded */
    {
        if( !hLib ) load_ISLIB();

        if( !hLib )  /* failed to load dll */
        {
            return 0;
        }
    }
    if( _ISD_OpenTracker )
    {
        return( (*_ISD_OpenTracker)( hParent, commPort, infoScreen, verbose ) );
    }
    return FALSE;
}


/******************************************************************************/
DWORD ISD_OpenAllTrackers( 
                          HWND hParent, 
                          ISD_TRACKER_HANDLE *handle, 
                          BOOL infoScreen, 
                          BOOL verbose 
                          )
{
    if( !_ISD_OpenAllTrackers ) /* this will be NULL if dll is not loaded */
    {
        if( !hLib ) load_ISLIB();

        if( !hLib )  /* failed to load dll */
        {
            return 0;
        }
    }
    if( _ISD_OpenAllTrackers )
    {
        return( (*_ISD_OpenAllTrackers)( hParent, handle, infoScreen, verbose ) );
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_CloseTracker( ISD_TRACKER_HANDLE handle )
{
    BOOL ret;
    WORD num;

    if( _ISD_CloseTracker )
    {
        ret = (*_ISD_CloseTracker)( handle );

        /* if all trackers are closed the dll can be freed */
        if( ISD_NumOpenTrackers( &num ) )
        {
            if(num == 0)
            {
                ISD_FreeLib( );
            }
        }
        return ret;
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_NumOpenTrackers( WORD *num )
{
    if( _ISD_NumOpenTrackers ) 
    {
        return( (*_ISD_NumOpenTrackers)( num ) );
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_GetCommInfo( 
                     ISD_TRACKER_HANDLE handle, 
                     ISD_TRACKER_INFO_TYPE *Tracker 
                     )
{
    if( _ISD_GetCommInfo )
    {
        return( (*_ISD_GetCommInfo)( handle, Tracker ) );
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_GetTrackerConfig( 
                          ISD_TRACKER_HANDLE handle, 
                          ISD_TRACKER_INFO_TYPE *Tracker, 
                          BOOL verbose 
                          )
{
    if( _ISD_GetTrackerConfig )
    {
        return( (*_ISD_GetTrackerConfig)( handle, Tracker, verbose ) );
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_SetTrackerConfig( 
                          ISD_TRACKER_HANDLE handle, 
                          ISD_TRACKER_INFO_TYPE *Tracker,
                          BOOL verbose 
                          )
{
    if( _ISD_SetTrackerConfig )
    {
        return( (*_ISD_SetTrackerConfig)( handle, Tracker, verbose ) );
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_SetStationConfig( 
                          ISD_TRACKER_HANDLE handle, 
                          ISD_STATION_INFO_TYPE *Station, 
                          WORD stationNum, 
                          BOOL verbose 
                          )
{
    if( _ISD_SetStationConfig )
    {
        return( (*_ISD_SetStationConfig)( handle, Station, stationNum, verbose ) );
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_GetStationConfig( 
                          ISD_TRACKER_HANDLE handle, 
                          ISD_STATION_INFO_TYPE *Station, 
                          WORD stationNum, 
                          BOOL verbose 
                          )
{
    if( _ISD_GetStationConfig )
    {
        return( (*_ISD_GetStationConfig)( handle, Station, stationNum, verbose ) );
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_GetData( 
                 ISD_TRACKER_HANDLE handle, 
                 ISD_TRACKER_DATA_TYPE *Data 
                 )
{
    if( _ISD_GetData )
    {
        return( (*_ISD_GetData)( handle, Data ) );
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_GetCameraData( 
                       ISD_TRACKER_HANDLE handle, 
                       ISD_CAMERA_DATA_TYPE *Data 
                       )
{
    if( _ISD_GetCameraData )
    {
        return( (*_ISD_GetCameraData)( handle, Data ) );
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_SendScript( 
                    ISD_TRACKER_HANDLE handle, 
                    char *script 
                    )
{
    if( _ISD_SendScript )
    {
        return( (*_ISD_SendScript)( handle, script ) );
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_ResetAngles( 
                     ISD_TRACKER_HANDLE handle, 
                     float yaw, 
                     float pitch,
                     float roll 
                     )
{
    if( _ISD_ResetAngles ) 
    {
        return( (*_ISD_ResetAngles)( handle, yaw, pitch, roll ) );
    }
    return FALSE;
}


/******************************************************************************/
BOOL ISD_ResetHeading( 
                      ISD_TRACKER_HANDLE handle, 
                      WORD stationNum 
                      )
{
    if( _ISD_ResetHeading ) 
    {
        return( (*_ISD_ResetHeading)( handle, stationNum ) );
    }
    return FALSE;
}


/******************************************************************************/
HWND ISD_OpenConfigWindow( ISD_TRACKER_HANDLE handle )
{
    if( _ISD_OpenConfigWindow )
    {
        return( (*_ISD_OpenConfigWindow)( handle ) );
    }
    return (HWND) NULL;
}


/******************************************************************************/
float ISD_GetTime( void )
{
    if( _ISD_GetTime )
    {
        return( (*_ISD_GetTime)() );
    }
    return 0.0f;
}


/******************************************************************************/
DLL_EP dll_entrypoint( DLL *dll, const char *name )
{
#if defined(_WIN32) || defined(WIN32) || defined(__WIN32__)

    FARPROC proc;

    proc = GetProcAddress( (HMODULE) dll, (LPCSTR) name );
    if( proc == NULL )
    {
        printf( "Failed to load %s. Error code %d\n", name, GetLastError() );
    }
    return (DLL_EP) proc;

#else // UNIX

	void *handle = (void *) dll;
	DLL_EP ep;
	ep = (DLL_EP) dlsym(handle, name);
	return ( dlerror() == 0 ) ? ep : (DLL_EP) NULL;

#endif
}	

/******************************************************************************/
DLL *dll_load( const char *name )
{
#if defined(_WIN32) || defined(WIN32) || defined(__WIN32__)

    return (DLL *) LoadLibrary( name );

#else // UNIX

	DLL *dll;

    char dllname[200+1+2+1];
	strcpy(dllname, name);
	strcat(dllname, ".so");
	dll = dlopen(dllname, RTLD_NOW);
	if(!dll) printf("%s\n", (char *) dlerror());
	return dll;

#endif
}

/******************************************************************************/
void dll_unload( DLL *dll )
{
#if defined(_WIN32) || defined(WIN32) || defined(__WIN32__)

    HINSTANCE hInst = (HINSTANCE) dll;
    FreeLibrary( hInst );

#else // UNIX

	void *handle = (void *) dll;
	dlclose(handle);

#endif
}


