

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "HIDControllers\GameController.h"
#include "Framework\DirectInputProxy8.h"

using namespace vertexware;


//====================================================================
class GenericGameController : public HumanInterfaceDevice,
								public GameController
{
public:
	GenericGameController(DirectInputProxy8& dip,
						const DirectInputDeviceInstanceInfo8& didii);

	virtual ~GenericGameController();


	//-------------------------------------
	// HumanInterfaceDevice implementation.
	//-------------------------------------
	virtual  uint            GetNumControllers() { return 1; }
	virtual  HIDController&  GetController(uint i) { return *this; }

	virtual  void  ConnectToAppWindow(TopLevelWindow* pAppWindow,
									HWND hwndAppWindow, bool bFullscreen);

	virtual  void  DisconnectFromAppWindow();

	virtual  bool  SampleInputData();


protected:
	//-------------------------------------
	// HumanInterfaceDevice implementation.
	//-------------------------------------
	virtual  void  OnUseDirectInput(bool bUseDI);

	//-------------------------------
	// GameController implementation.
	//-------------------------------
	virtual  void  OnInputModifiersChanged();


private:
	void  CreateDIDevice();
	void  ReleaseDIDevice();
	void  ExtractHardwareProperties();
	void  SetupHardwareProperties();
	bool  GetDIPropertyRange(float* rangeVals, REFGUID rguidProp, DWORD dwObj, DWORD dwHow = DIPH_BYOFFSET);
	bool  GetDIPropertyValue(DWORD& val, REFGUID rguidProp, DWORD dwObj, DWORD dwHow = DIPH_BYOFFSET);
	bool  SetDIPropertyValue(DWORD val, REFGUID rguidProp, DWORD dwObj, DWORD dwHow = DIPH_BYOFFSET);

	// normalize raw-value to [-1, 1]
	static __forceinline  float  NormalizeAxisValue(float val, const float* rangeVals)
	{
		return (val - rangeVals[RangeVal_Min]) * rangeVals[RangeVal_TwoOverRange] - 1.0f;
	}

	// apply dead-zone/saturation to normalized range [-1, 1]
	static __forceinline  float  ApplyAxisModifiers(float val,
										float deadZone, float saturation, float oneOverRange)
	{
		float absVal = Abs(val);
		if(absVal <= deadZone) return 0;
		if(absVal >= saturation) return Sign(val);
		return (absVal - deadZone) * oneOverRange * Sign(val);
	}

	struct DeviceObjectsEnumStats
	{
		DeviceObjectsEnumStats();

		bool  bX, bY, bZ, bRX, bRY, bRZ;
		uint  numSliders, numPOVHats, numButtons;
	};
	static BOOL CALLBACK DIEnumDeviceObjectsCallback(const DIDEVICEOBJECTINSTANCE* pdoi, void* pData);

	static DWORD __fastcall MapAxisToDIOffset(eAxis a);
	void   CalcAxisModifiers();


private:
	DirectInputProxy8*              pDirectInputProxy8;
	DirectInputDeviceInstanceInfo8  diDeviceInfo;
	IDirectInputDevice8*            iDIDevice;
	HWND                            hwndAppWindow;

	bool  bDIDeviceConnectedToWindow;
	bool  bUsingExclusiveMode;

	DIJOYSTATE  diJoystickState;

	// ranges reported by hardware
	enum {
		RangeVal_Min = 0,
		RangeVal_Max = 1,
		RangeVal_TwoOverRange = 2,
		RangeVal_Count
	};

	float  axisRangeVals[Axis_Count][RangeVal_Count];
	float  sliderRangeVals[2][RangeVal_Count];

	float  axisModsOneOverRange[Axis_Count];
	float  sliderModsOneOverRange[2];
};

