

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\VideoSubsystemDirect3D8.h"

using namespace vertexware;


//====================================================================
class vs_direct3d8 : public VideoSubsystemDirect3D8
{
public:
	vs_direct3d8();
	virtual ~vs_direct3d8();


	//------------------
	// Plugin overrides.
	//------------------
	virtual  bool  SetPluginParameter(const NamedParameter& np);
	virtual  bool  GetPluginParameter(const std::string& parameterName, NamedParameter& out) const;


	//-------------------------------
	// VideoSubsystem implementation.
	//-------------------------------
	virtual  std::string  GetVideoSubsystemInterfaceName();

	virtual  bool  DoBuildVideoProfile(std::ostream& log,
										VideoProfile& outVideoProfile);

	virtual  VideoSubsystemWindow*  InitializeWindow(const VideoConfig& videoConfig,
													bool bAutoShowWindow);

	virtual  VideoSubsystemWindow*  GetWindow();

	virtual  void  CleanupWindow();

	virtual  void  PresentBackBuffer();

	virtual  void  ErrorCheck();

	virtual  void  Reset();

	virtual  void  UpdateVSync(bool bEnable);

	virtual  bool  CaptureFrameBuffer(AutoPtr<ImageBase>& out,
									uint frameBufferFlags,
									bool bForceColorSampleFormat);


	//----------------------------------------
	// VideoSubsystemDirect3D8 implementation.
	//----------------------------------------
	virtual  IDirect3D8*        GetD3DObject();
	virtual  IDirect3DDevice8*  GetD3DDevice();


private:
	//-------------------------------
	// VideoSubsystemWindow override.
	//-------------------------------
	class Window : public VideoSubsystemWindow
	{
	friend class vs_direct3d8;
	public:
		Window(vs_direct3d8& _subsys, bool bFullscreen) :
			VideoSubsystemWindow(bFullscreen), subsys(_subsys)
		{ }

	protected:
		virtual  void  OnResized(eResizeAction action);

	private:
		vs_direct3d8& subsys;
	};
	friend class Window;


	//----------------------------
	// System-resource management.
	//----------------------------
	bool   CreateD3DObject();
	void   CreateD3DDevice();
	void   StabilizeColorBuffers();


private:
	//----------------------------
	// Plugin-specific parameters.
	//----------------------------
	bool  bDestroyWindowBeforeD3DObjectRelease;

	//------------------
	// System resources.
	//------------------
	Window*                pWindow;
	IDirect3D8*            iObject;
	IDirect3DDevice8*      iDevice;
	D3DDEVTYPE             deviceType;
	D3DPRESENT_PARAMETERS  presentationParameters;
};


