

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\VideoSubsystemOpenGL.h"

using namespace vertexware;


//====================================================================
class vs_opengl : public VideoSubsystemOpenGL
{
public:
	vs_opengl();
	virtual ~vs_opengl();


	//-------------------------------
	// VideoSubsystem implementation.
	//-------------------------------
	virtual  std::string  GetVideoSubsystemInterfaceName();

	virtual  bool  DoBuildVideoProfile(std::ostream& log,
										VideoProfile& outVideoProfile);

	virtual  VideoSubsystemWindow*  InitializeWindow(const VideoConfig& vc,
													bool bAutoShowWindow);

	virtual  VideoSubsystemWindow*  GetWindow();

	virtual  void  CleanupWindow();

	virtual  void  PresentBackBuffer();

	virtual  void  ErrorCheck();

	virtual  void  Reset();

	virtual  void  UpdateVSync(bool bEnable);

	virtual  bool  CaptureFrameBuffer(AutoPtr<ImageBase>& out,
									uint frameBufferFlags,
									bool bForceColorSampleFormat);


private:
	//-------------------------
	// Display-mode management.
	//-------------------------
	class DisplayModeSwitchSentry {
	public:
		DisplayModeSwitchSentry() : bSwitched(false) {}
		DisplayModeSwitchSentry(const DisplayModeConfig& dmc);
		~DisplayModeSwitchSentry() { Restore(); }

		bool  Switch(const DisplayModeConfig& dmc);
		void  Restore();
		bool  IsSwitched() const { return bSwitched; }

	private:
		bool  bSwitched;
	};


	//-------------------------------
	// VideoSubsystemWindow override.
	//-------------------------------
	class Window : public VideoSubsystemWindow
	{
	friend class vs_opengl;
	public:
		Window(vs_opengl& _subsys, bool bFullscreen) :
			VideoSubsystemWindow(bFullscreen), subsys(_subsys)
		{ }

	protected:
		virtual  void  OnActivated(bool bActive);
		virtual  void  OnResized(eResizeAction action);

	private:
		vs_opengl& subsys;
	};
	friend class Window;


	//----------------------------
	// System-resource management.
	//----------------------------
	void   CreateRenderingContext();
	void   DestroyRenderingContext();
	void   SetupVSync(bool bEnable);

	bool   IsRCBoundToCurrentThread();
	void   BindRCToCurrentThread();
	void   UnbindRCFromCurrentThread();

	void   StabilizeColorBuffers();
	void   InitExtensionPointers();
	void   InvalidateExtensionPointers();

	void   PostWindowInitialize();
	void   PreWindowCleanup();


private:
	//------------------
	// System resources.
	//------------------
	DisplayModeSwitchSentry  dmss;
	Window*                  pWindow;
	HGLRC                    hglrc;
	std::string              errorDesc;

	//-----------------------------------
	// Internally used OpenGL extensions.
	//-----------------------------------
	typedef  BOOL  (APIENTRY *PFNWGLSWAPINTERVALEXT)(int);
	typedef  int   (APIENTRY *PFNWGLGETSWAPINTERVALEXT)();

	PFNWGLSWAPINTERVALEXT     wglSwapIntervalEXT;
	PFNWGLGETSWAPINTERVALEXT  wglGetSwapIntervalEXT;
};

