

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\CompilerConfig.h"
#include "private\vw_common_exports.h"


//********************************************************************
namespace vertexware {


//====================================================================
// Standard cleanup-handler through member function
//====================================================================
template<typename TTarget>
class CleanupHandlerMFN
{
public:
	typedef  void (TTarget::* PMFNCleanup)();

public:
	CleanupHandlerMFN(PMFNCleanup _pmfnCleanup) {
		pmfnCleanup = _pmfnCleanup;
	}

	__forceinline  void  operator()(TTarget& target) {
		(target.*pmfnCleanup)();
	}

private:
	PMFNCleanup  pmfnCleanup;
};


//====================================================================
// template<typename TTarget>
// struct CleanupHandler {
//     __forceinline  void  operator()(TTarget& target) {}
// };
// NOTES: Copy-construction must be available.
//====================================================================
template<typename TTarget, typename TCleanupHandler = CleanupHandlerMFN<TTarget> >
class AutoCleanup
{
public:
	AutoCleanup(TTarget* _pTarget = 0, bool _bEnabled = true,
				TCleanupHandler& _cleanupHandler = TCleanupHandler())
		:
		pTarget(_pTarget), cleanupHandler(_cleanupHandler), bEnabled(_bEnabled)
	{ }

	~AutoCleanup() {
		if(bEnabled  &&  pTarget) {
			cleanupHandler(*pTarget);
		}
	}

	bool  IsEnabled() { return bEnabled; }
	void  Enable() { bEnabled = true; }
	void  Disable() { bEnabled = false; }

	TTarget*  GetTarget() { return pTarget; }
	void      SetTarget(TTarget* _pTarget) { pTarget = _pTarget; }

	TCleanupHandler&  GetCleanupHandler() { return cleanupHandler; }
	void              SetCleanupHandler(TCleanupHandler& _cleanupHandler) { cleanupHandler = _cleanupHandler; }


private:
	TTarget*         pTarget;
	TCleanupHandler  cleanupHandler;
	bool             bEnabled;
};


//********************************************************************
}//namespace vertexware

