

//--------------------------------------------------------------------
#include "BaseTools\CommandInterpreter.h"
#include "BaseTools\Debug.h"


//********************************************************************
namespace vertexware {


//====================================================================
Command::Command()
{
	SetDeveloperModeOnly(false);
	SetDefaultName("command");
}


//====================================================================
Command::~Command()
{
}


//====================================================================
void  Command::SetName(const std::string& s)
{
	name = s;

	if(name.empty()) {
		VW_ASSERT(0, "Using empty-string for command-name.");
		name = VW_GET_CLASS_NAME(*this);
	}
}


//====================================================================
void  Command::SetDefaultName(const std::string& s)
{
	defaultName = s;

	if(defaultName.empty()) {
		VW_ASSERT(0, "Using empty-string for default-command-name.");
		defaultName = VW_GET_CLASS_NAME(*this);
	}
	SetName(defaultName);
}


//====================================================================
void  Command::SetShortDesc(const std::string& s)
{
	shortDesc = s;
}


//====================================================================
CommandInterpreter::~CommandInterpreter()
{
}


//====================================================================
uint  CommandInterpreter::InterpretScript(std::ostream& os, const std::string& script,
										bool bStopOnError)
{
	uint numErrors = 0;

	// break script into lines
	std::vector<std::string>  lines;
	BreakIntoLines(script, lines, '\n');

	// interpret lines
	for(uint i = 0; i < lines.size(); i++) {
		os << lines[i] << nl;

		if(! InterpretCommandLine(lines[i], os)) {
			numErrors++;
			if(bStopOnError) {
				// return 1-based-line-number of first-error
				return (i + 1);
			}
		}
	}
	return numErrors;
}


//====================================================================
std::string  CommandInterpreter::GetInterpreterDesc() const
{
	return "Null-interpreter";
}


//====================================================================
bool  CommandInterpreter::InterpretCommandLine(const std::string& commandLine,
												std::ostream& os)
{
	os << "Command not recognized." << std::endl;
	return false;
}


//====================================================================
bool  CommandInterpreter::CompleteCommandLine(const std::string& commandLinePrefix,
											std::string& completedCommandLine,
											std::ostream& os)
{
	return false;
}


//********************************************************************
}//namespace vertexware


