

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "BaseTools\CommandLineParser.h"
#include "BaseTools\Stream.h"
#include "BaseTools\String.h"


//********************************************************************
namespace vertexware {


//====================================================================
// Command-name & argument matching should be case-sensitive.
//====================================================================
class VW_COMMON_API Command
{
public:
	Command();
	virtual ~Command();

	void  SetDeveloperModeOnly(bool bDevModeOnly = false) { bDeveloperModeOnly = bDevModeOnly; }
	bool  IsDeveloperModeOnly() const { return bDeveloperModeOnly; }

	// empty names are not accepted
	void                SetName(const std::string& s);
	const std::string&  GetName() const { return name; }

	void                SetDefaultName(const std::string& s);
	const std::string&  GetDefaultName() const { return defaultName; }

	void                SetShortDesc(const std::string& s);
	const std::string&  GetShortDesc() const { return shortDesc; }

	//--------------------------------------------------------------------------
	// Parse either raw command-line or pre-processed command-line then execute.
	// Return execution-result. Use <os> for output.
	// If parsing is done by processing <rawCommandLine> do NOT use hardcoded
	// string to match the command-name, as it may be changed through public
	// interface. Always check the command-name through GetName().
	//--------------------------------------------------------------------------
	virtual  bool  ParseAndExecute(std::ostream& os,
									const CommandLineParser& clp,
									const std::string& rawCommandLine) = 0;

	// dump help
	virtual  void  DumpHelp(std::ostream& os, const CommandLineParser& clp) = 0;


private:
	bool         bDeveloperModeOnly;
	std::string  name;
	std::string  defaultName;
	std::string  shortDesc;
};


//====================================================================
class VW_COMMON_API CommandInterpreter
{
public:
	virtual ~CommandInterpreter();

	//------------------------------------------------------
	// Interpret script (newline separated commands),
	// Returns 0 on success.
	// Returns error-count if <bStopOnError> is false,
	// otherwise returns 1-based-line-number of first-error.
	//------------------------------------------------------
	virtual  uint  InterpretScript(std::ostream& os, const std::string& script,
									bool bStopOnError = false);

	// return interpreter description
	virtual  std::string  GetInterpreterDesc() const;

	// return true if interpreted
	virtual  bool  InterpretCommandLine(const std::string& commandLine,
										std::ostream& os);

	// return true if completed
	virtual  bool  CompleteCommandLine(const std::string& commandLinePrefix,
										std::string& completedCommandLine,
										std::ostream& os);
};


//********************************************************************
}//namespace vertexware


