

//--------------------------------------------------------------------
#include "BaseTools\ConsoleOutputWindow.h"
#include "BaseTools\LibraryInfo.h"
#include "BaseTools\String.h"
#include "BaseTools\PlatformInfo.h"
#include "BaseTools\Time.h"


//--------------------------------------------------------------------
#define  COWIN_MIN_DESKTOP_LUMINANCE  0.2f
#define  COWIN_DEFAULT_DESKTOP_COLOR  color4b(0, 78, 152)
#define  COWIN_UNIT                   16
#define  COWIN_MIN_WIDTH              (COWIN_UNIT * 27)
#define  COWIN_BAR_HEIGHT             (COWIN_UNIT * 3)
#define  COWIN_BAR_HEIGHT2            (COWIN_BAR_HEIGHT * 2)
#define  COWIN_MIN_HEIGHT             (COWIN_BAR_HEIGHT2)
#define  COWIN_BAR_FONT_NAME          "Tahoma"
#define  COWIN_TEXT_FONT_NAME         "Courier New"
#define  COWIN_BAR_FONT_SIZE          (COWIN_UNIT)
#define  COWIN_GADGET_SIZE            (COWIN_UNIT)
#define  COWIN_GADGET_INTENSITY       3
#define  COWIN_GADGET_VOFFSET         (COWIN_UNIT * 1)
#define  COWIN_GADGET_CLOSE_HOFFSET   (COWIN_UNIT * 1)
#define  COWIN_GADGET_MIN_HOFFSET     (COWIN_UNIT * 6)
#define  COWIN_GADGET_MAX_HOFFSET     (COWIN_UNIT * 4)
#define  COWIN_GADGET_SIZE_HOFFSET    (COWIN_UNIT * 2)
#define  COWIN_SCROLL_THUMB_MIN_SIZE  (COWIN_UNIT / 4)
#define  COWIN_TEXT_HEIGHT            16
#define  COWIN_TEXT_WIDTH             10.25 //determined by experiment
#define  COWIN_TEXT_ROW_HEIGHT        (COWIN_UNIT)
#define  COWIN_TEXT_HMARGIN           (COWIN_UNIT)
#define  COWIN_SCROLLBAR_WIDTH        (COWIN_UNIT / 4)
#define  COWIN_MOVE_SNAP              (COWIN_UNIT * 2)
#define  COWIN_SIZE_SNAP              (COWIN_UNIT * 2)


//********************************************************************
namespace vertexware {


//====================================================================
ConsoleOutputWindow::ConsoleOutputWindow()
	:
	TopLevelWindow(true),
	pTextSource(0),
	bHookedToTextSource(false),
	bTextAreaVisible(false),
	inactiveColor(Gdiplus::Color::Black),
	activeColor(Gdiplus::Color::Yellow),
	inactivePen(Gdiplus::Color::Black, COWIN_GADGET_INTENSITY),
	activePen(Gdiplus::Color::Yellow, COWIN_GADGET_INTENSITY),
	inactiveBrush(Gdiplus::Color::Black),
	activeBrush(activeColor),
	textBgrBrush(Gdiplus::Color::Black),
	textBrushInfo(Gdiplus::Color::White),
	textBrushWarning(Gdiplus::Color::White),
	textBrushError(Gdiplus::Color::White),
	textBrushAttention(Gdiplus::Color::White),
	textBrushQuery(Gdiplus::Color::White),
	textBrushEcho(Gdiplus::Color::White),
	textBrushDebug(Gdiplus::Color::White),
	barFontFamily(VW_WIDE_TEXT(COWIN_BAR_FONT_NAME)),
	barFont(&barFontFamily, COWIN_BAR_FONT_SIZE, Gdiplus::FontStyleBold, Gdiplus::UnitPixel),
	barStringFormat(),
	textFontFamily(VW_WIDE_TEXT(COWIN_TEXT_FONT_NAME)),
	textFont(&textFontFamily, COWIN_TEXT_HEIGHT, 0, Gdiplus::UnitPixel),
	textStringFormat()
{
	// setup base-properties
	CreateSystemWindow(0, WS_POPUP, WS_EX_APPWINDOW);

	EnableSystemCommandEmulation(true);

	SetMinSize(COWIN_MIN_WIDTH, COWIN_MIN_HEIGHT);

	UpdateMaximizedPlacement();
	CenterOnDesktop(COWIN_MIN_WIDTH, COWIN_MIN_HEIGHT);

	// setup specific-properties
	SetOutputTypeColor(OutputStream::OutputType_Info, color4b::GREEN);
	SetOutputTypeColor(OutputStream::OutputType_Warning, color4b(255, 128, 0));
	SetOutputTypeColor(OutputStream::OutputType_Error, color4b::RED);
	SetOutputTypeColor(OutputStream::OutputType_Attention, color4b::YELLOW);
	SetOutputTypeColor(OutputStream::OutputType_Query, color4b::GREEN);
	SetOutputTypeColor(OutputStream::OutputType_Echo, color4b::WHITE);
	SetOutputTypeColor(OutputStream::OutputType_Debug, color4b::CYAN);

	SetMouseWheelStep();

	bUpdatedSizeAfterDisplayChange = true;

	scrollHorz.SetBarPosX(COWIN_TEXT_HMARGIN);
	scrollHorz.SetBarWidth(COWIN_SCROLLBAR_WIDTH);
	scrollHorz.SetMinThumbLength(COWIN_SCROLL_THUMB_MIN_SIZE);
	scrollHorz.EnableMaxPixelsPerStep(false);

	scrollVert.SetBarPosY(COWIN_BAR_HEIGHT + COWIN_TEXT_ROW_HEIGHT / 2);
	scrollVert.SetBarWidth(COWIN_SCROLLBAR_WIDTH);
	scrollVert.SetMinThumbLength(COWIN_SCROLL_THUMB_MIN_SIZE);
	scrollVert.EnableMaxPixelsPerStep(false);

	// create initial back-buffer
	SetupBackBuffer();

	// prepare drawing resources
	OnSystemColorsChanged();

	barStringFormat.SetAlignment(Gdiplus::StringAlignmentCenter);
	barStringFormat.SetLineAlignment(Gdiplus::StringAlignmentCenter);

	textStringFormat.SetAlignment(Gdiplus::StringAlignmentNear);
	textStringFormat.SetLineAlignment(Gdiplus::StringAlignmentCenter);

	SetTitleText(GetText());
	SetStatusText(std::string("powered by ") + library_info::GetLibraryName());
}


//====================================================================
ConsoleOutputWindow::~ConsoleOutputWindow()
{
	UnhookFromTextSource();
}


//====================================================================
void  ConsoleOutputWindow::SetTitleText(const std::string& s)
{
	AsciiToUnicode(s, wsTitleText);
	PostRepaint();
}


//====================================================================
void  ConsoleOutputWindow::SetStatusText(const std::string& s)
{
	AsciiToUnicode(s, wsStatusText);
	PostRepaint();
}


//====================================================================
void  ConsoleOutputWindow::SetTextSource(ConsoleOutputBuffer* cob)
{
	UnhookFromTextSource();
	pTextSource = cob;

	scrollHorz.ResetLogicalParameters();
	scrollVert.ResetLogicalParameters();

	HookToTextSource();
	OnResized(ResizeAction_Resize);
}


//====================================================================
void  ConsoleOutputWindow::SetOutputTypeColor(OutputStream::eOutputType type, color4b c)
{
	Gdiplus::SolidBrush* brush = 0;
	switch(type) {
		case OutputStream::OutputType_Info: brush = &textBrushInfo; break;
		case OutputStream::OutputType_Warning: brush = &textBrushWarning; break;
		case OutputStream::OutputType_Error: brush = &textBrushError; break;
		case OutputStream::OutputType_Attention: brush = &textBrushAttention; break;
		case OutputStream::OutputType_Query: brush = &textBrushQuery; break;
		case OutputStream::OutputType_Echo: brush = &textBrushEcho; break;
		case OutputStream::OutputType_Debug: brush = &textBrushDebug; break;
	}
	if(brush) {
		brush->SetColor(Gdiplus::Color(c.r, c.g, c.b));
	}
}


//====================================================================
void  ConsoleOutputWindow::SetMouseWheelStep(uint step)
{
	mouseWheelStep = step;
}


//====================================================================
void  ConsoleOutputWindow::EnableCloseGadget(bool bEnable)
{
	bCloseGadgetEnabled = bEnable;
	EnableCloseThroughAltF4(bCloseGadgetEnabled);
	PostRepaint();
}


//====================================================================
void  ConsoleOutputWindow::FocusOnLastLine()
{
	scrollVert.SetPositionToMax();
	PostRepaint();
}


//====================================================================
bool  ConsoleOutputWindow::IsSplashSized() const
{
	return (GetWidth() == COWIN_MIN_WIDTH  &&  GetHeight() == COWIN_MIN_HEIGHT);
}


//====================================================================
ConsoleOutputWindow::eWindowArea  ConsoleOutputWindow::MouseHitTest(const win_utils::Point& mousePosClient, uint& areaSubCode)
{
	areaSubCode = 0;
	LONG  w = GetWidth();

	// test top bar
	if(mousePosClient.y < COWIN_BAR_HEIGHT) {
		return WindowArea_Move;
	}
	// test bottom bar
	else if(mousePosClient.y > GetHeight() - COWIN_BAR_HEIGHT) {

		// test gadgets
		LONG  gadgetRefVoffset = GetHeight() - (COWIN_BAR_HEIGHT - COWIN_GADGET_VOFFSET);

		// size
		if(mousePosClient.HitTestRectangle(w - COWIN_GADGET_SIZE_HOFFSET, gadgetRefVoffset,
											COWIN_GADGET_SIZE, COWIN_GADGET_SIZE)) {
			return WindowArea_SizeBottomRight;
		}
		// maximize/restore
		if(mousePosClient.HitTestRectangle(w - COWIN_GADGET_MAX_HOFFSET, gadgetRefVoffset,
											COWIN_GADGET_SIZE, COWIN_GADGET_SIZE)) {
			return WindowArea_MaximizeRestore;
		}
		// minimize
		if(mousePosClient.HitTestRectangle(w - COWIN_GADGET_MIN_HOFFSET, gadgetRefVoffset,
											COWIN_GADGET_SIZE, COWIN_GADGET_SIZE)) {
			return WindowArea_Minimize;
		}
		if(bCloseGadgetEnabled  &&  mousePosClient.HitTestRectangle(
						COWIN_GADGET_CLOSE_HOFFSET, gadgetRefVoffset,
						COWIN_GADGET_SIZE, COWIN_GADGET_SIZE)) {
			return WindowArea_Close;
		}
		return WindowArea_Move;
	}
	return WindowArea_Client;
}


//====================================================================
void  ConsoleOutputWindow::OnSystemColorsChanged()
{
	// get desktop color
	desktopColor = win_utils::GetSystemColor(win_utils::SystemColor_Desktop);

	// if desktop color is too dark, use default color
	color4f c = win_utils::GdiplusConvertColor(desktopColor);

	if(c.Luminance() <= COWIN_MIN_DESKTOP_LUMINANCE) {
		desktopColor = win_utils::GdiplusConvertColor(COWIN_DEFAULT_DESKTOP_COLOR);
	}

	// setup other colors according to base-color
	inactiveColor = desktopColor;
	inactiveBrush.SetColor(inactiveColor);
	inactivePen.SetColor(inactiveColor);
}


//====================================================================
void  ConsoleOutputWindow::OnSystemSettingsChanged()
{
	UpdateMaximizedPlacement();
}


//====================================================================
void  ConsoleOutputWindow::OnDisplaySettingsChanged()
{
	bUpdatedSizeAfterDisplayChange = false;
	UpdateSizeAfterDisplayChange();
}


//====================================================================
void  ConsoleOutputWindow::OnActivated(bool bActive)
{
	OnMouseEnter(GetCurrentMouseArea(), GetCurrentMouseAreaSubCode());
	PostRepaint();
}


//====================================================================
void  ConsoleOutputWindow::OnResized(eResizeAction action)
{
	switch(action) {
		case ResizeAction_Minimize:
		{
			UnhookFromTextSource();
			break;
		}
		case ResizeAction_Resize:
		case ResizeAction_Maximize:
		{
			if(! bUpdatedSizeAfterDisplayChange) {
				UpdateSizeAfterDisplayChange();
			}

			HookToTextSource();
			bool  bFocusOnLastLine = scrollVert.IsPositionAtMaximum();

			int hBarLength = GetWidth() - COWIN_TEXT_HMARGIN * 2;
			int numCols = double(hBarLength) / double(COWIN_TEXT_WIDTH);

			scrollHorz.SetVisibleSize(numCols);
			scrollHorz.SetBarPosY(GetHeight() - (COWIN_BAR_HEIGHT + COWIN_SCROLLBAR_WIDTH));
			scrollHorz.SetBarLength(hBarLength);

			int vTextArea = GetHeight() - COWIN_BAR_HEIGHT2;
			int numRows  = ClampedLow<int>(vTextArea, 0) / COWIN_TEXT_ROW_HEIGHT;
			bTextAreaVisible = (numRows > 0);
			if(bTextAreaVisible) numRows--;

			scrollVert.SetVisibleSize(numRows);
			scrollVert.SetBarPosX(GetWidth() - COWIN_TEXT_HMARGIN + COWIN_SCROLLBAR_WIDTH);
			scrollVert.SetBarLength(vTextArea - COWIN_TEXT_ROW_HEIGHT);

			if(bFocusOnLastLine) {
				scrollVert.SetPositionToMax();
			}
			else {
				scrollVert.SetPosition(scrollVert.GetPosition());
			}
			break;
		}
	}
}


//====================================================================
void  ConsoleOutputWindow::OnPaint()
{
	Render();
}


//====================================================================
void  ConsoleOutputWindow::UpdateMaximizedPlacement()
{
	uint width, height, left, top;
	platform_info::GetWorkArea(&width, &height, &left, &top);

	SetMaximizedPosition(left, top);
	SetMaxSize(width, height);
}


//====================================================================
void  ConsoleOutputWindow::UpdateSizeAfterDisplayChange()
{
	if(IsMinimized()) return;

	Restore();
	UpdateMaximizedPlacement();
	CenterOnDesktop(COWIN_MIN_WIDTH, COWIN_MIN_HEIGHT);
	SetupBackBuffer();

	bUpdatedSizeAfterDisplayChange = true;
}


//====================================================================
void  ConsoleOutputWindow::SetupBackBuffer()
{
	uint desktopW, desktopH;
	platform_info::GetDesktopDisplayMode(&desktopW, &desktopH, 0, 0);

	// initializing back-buffer-bitmap with suitable target improves drawing performance
	Gdiplus::Graphics  frontBuffer(GetHDC());
	pBackBufferBitmap.Assign(new Gdiplus::Bitmap(desktopW, desktopH, &frontBuffer));
	pBackBuffer.Assign(new Gdiplus::Graphics(pBackBufferBitmap));
}


//====================================================================
void  ConsoleOutputWindow::Render()
{
	using namespace Gdiplus;

	// cache parameters
	const int w = GetWidth();
	const int h = GetHeight();

	// draw visible area into back-buffer
	RenderFrameArea(*pBackBuffer, w, h);
	if(bTextAreaVisible) {
		RenderTextArea(*pBackBuffer, w, h);
	}

	// copy pixels into front-buffer
	Graphics  frontBuffer(GetHDC());
	frontBuffer.DrawImage(pBackBufferBitmap, 0, 0, 0, 0, w, h, UnitPixel);
}


//====================================================================
void  ConsoleOutputWindow::RenderFrameArea(Gdiplus::Graphics& g, int w, int h)
{
	using namespace Gdiplus;

	//--------
	// top-bar
	Rect topBarRect(0, 0, w, COWIN_BAR_HEIGHT + 1);
	LinearGradientBrush topBarBrush(topBarRect, desktopColor, Color::Black, LinearGradientModeVertical);
	g.FillRectangle(&topBarBrush, topBarRect);

	//-----------
	// bottom-bar
	Rect botBarRect(0, h - COWIN_BAR_HEIGHT, w, COWIN_BAR_HEIGHT + 1);
	LinearGradientBrush botBarBrush(botBarRect, Color::Black, desktopColor, LinearGradientModeVertical);
	g.FillRectangle(&botBarBrush, botBarRect);

	//-----------------------
	// system-command gadgets
	const INT gadgetVOffsetRef = h - (COWIN_BAR_HEIGHT - COWIN_GADGET_VOFFSET);

	//------
	// close
	if(bCloseGadgetEnabled) {
		Pen& p = IsMouseInArea(WindowArea_Close) ? activePen : inactivePen;
		g.DrawLine(&p, COWIN_GADGET_CLOSE_HOFFSET, gadgetVOffsetRef,
						COWIN_GADGET_CLOSE_HOFFSET + COWIN_GADGET_SIZE,
						gadgetVOffsetRef + COWIN_GADGET_SIZE);
		g.DrawLine(&p, COWIN_GADGET_CLOSE_HOFFSET + COWIN_GADGET_SIZE, gadgetVOffsetRef,
						COWIN_GADGET_CLOSE_HOFFSET, gadgetVOffsetRef + COWIN_GADGET_SIZE);
	}

	//---------
	// minimize
	Brush& brushMin = IsMouseInArea(WindowArea_Minimize) ? activeBrush : inactiveBrush;
	g.FillRectangle(&brushMin, w - COWIN_GADGET_MIN_HOFFSET,
					gadgetVOffsetRef + COWIN_GADGET_SIZE - COWIN_GADGET_INTENSITY,
					COWIN_GADGET_SIZE, COWIN_GADGET_INTENSITY);

	//-----------------
	// maximize/restore
	Pen& penMax = IsMouseInArea(WindowArea_MaximizeRestore) ? activePen : inactivePen;
	if(IsMaximized()) {
		const uint halfSize = COWIN_GADGET_SIZE / 2;
		g.DrawRectangle(&penMax, w - COWIN_GADGET_MAX_HOFFSET + halfSize, 
						gadgetVOffsetRef, halfSize, halfSize);
		g.DrawRectangle(&penMax, w - COWIN_GADGET_MAX_HOFFSET, 
						gadgetVOffsetRef + halfSize, halfSize, halfSize);
	}
	else {
		g.DrawRectangle(&penMax, w - COWIN_GADGET_MAX_HOFFSET, gadgetVOffsetRef,
						COWIN_GADGET_SIZE, COWIN_GADGET_SIZE);
	}

	//-----
	// size
	if(! IsMaximized()) {
		Brush& brushSize = IsMouseInArea(WindowArea_SizeBottomRight) ? activeBrush : inactiveBrush;
		INT  sizeRefHOffset = w - COWIN_GADGET_SIZE_HOFFSET;
		Point sizePoints[3] = {
			Point(sizeRefHOffset, gadgetVOffsetRef + COWIN_GADGET_SIZE),
			Point(sizeRefHOffset + COWIN_GADGET_SIZE, gadgetVOffsetRef + COWIN_GADGET_SIZE),
			Point(sizeRefHOffset + COWIN_GADGET_SIZE, gadgetVOffsetRef)
		};
		g.FillPolygon(&brushSize, sizePoints, 3);
	}

	//------------------
	// print status-text
	RectF textRect(0, h - COWIN_BAR_HEIGHT, w, COWIN_BAR_HEIGHT);
	g.DrawString(wsStatusText.c_str(), wsStatusText.length(),
				&barFont, textRect, &barStringFormat, &inactiveBrush);

	//-----------------
	// print title-text
	Brush& appInfoBrush = IsActive() ? activeBrush: inactiveBrush;
	textRect.Y = 0;
	g.DrawString(wsTitleText.c_str(), wsTitleText.length(),
					&barFont, textRect, &barStringFormat, &appInfoBrush);
}


//====================================================================
void  ConsoleOutputWindow::RenderTextArea(Gdiplus::Graphics& g, int w, int h)
{
	using namespace Gdiplus;

	// draw background
	g.FillRectangle(&textBgrBrush, 0, COWIN_BAR_HEIGHT, w, h - COWIN_BAR_HEIGHT2);
	if(! pTextSource) return;

	// determine view parameters
	int numVisibleCols = scrollHorz.GetVisibleSize();
	int hPosition = scrollHorz.GetPosition();

	//-----------
	// scrollbars
	scrollVert.UpdateThumbParameters();
	if(scrollVert.IsVisibleLessThanTotal()) {
		g.FillRectangle(&inactiveBrush, scrollVert.GetBarPosX(), scrollVert.GetBarPosY() + scrollVert.GetThumbOffset(),
						scrollVert.GetBarWidth(), scrollVert.GetThumbLength());
	}

	scrollHorz.UpdateThumbParameters();
	if(scrollHorz.IsVisibleLessThanTotal()) {
		g.FillRectangle(&inactiveBrush, scrollHorz.GetBarPosX() + scrollHorz.GetThumbOffset(),
						scrollHorz.GetBarPosY(), scrollHorz.GetThumbLength(), scrollHorz.GetBarWidth());
	}

	//-----------------
	// print text lines
	pTextSource->Lock();

	std::wstring ws;
	int yDrawOffset = COWIN_BAR_HEIGHT;
	int wTextArea   = GetWidth() - COWIN_TEXT_HMARGIN * 2;

	const uint lastLine = scrollVert.GetLastVisiblePosition();
	for(uint i = scrollVert.GetPosition(); i <= lastLine; i++)
	{
		// get one line
		uint outputType;
		const std::string& s = pTextSource->GetLine(i, outputType);

		if(s.length() > hPosition) {

			// convert string to gdiplus-native unicode
			AsciiToUnicode(s, ws);
			const wchar_t* firstCharToDraw = ws.c_str() + hPosition;
			int numCharsToDraw  = ClampedHigh<int>(ws.length() - hPosition, numVisibleCols);

			RectF rect(COWIN_TEXT_HMARGIN, yDrawOffset, wTextArea, COWIN_TEXT_ROW_HEIGHT);

			// choose brush
			Gdiplus::Brush* brush = &textBrushInfo; //default
			switch(outputType) {
				case OutputStream::OutputType_Warning: brush = &textBrushWarning; break;
				case OutputStream::OutputType_Error: brush = &textBrushError; break;
				case OutputStream::OutputType_Attention: brush = &textBrushAttention; break;
				case OutputStream::OutputType_Query: brush = &textBrushQuery; break;
				case OutputStream::OutputType_Echo: brush = &textBrushEcho; break;
				case OutputStream::OutputType_Debug: brush = &textBrushDebug; break;
			}

			// draw one line
			g.DrawString(firstCharToDraw, numCharsToDraw, &textFont,
							rect, &textStringFormat, brush);
		}
		yDrawOffset += COWIN_TEXT_ROW_HEIGHT;
	}
	pTextSource->Unlock();
}


//====================================================================
void  ConsoleOutputWindow::OnMouseEnter(eWindowArea area, uint areaSubCode)
{
	PostRepaint();
}


//====================================================================
void  ConsoleOutputWindow::OnMouseWheel(int delta, const win_utils::Point& mousePosClient,
																WPARAM keyFlags)
{
	int mult = (scrollVert.GetVisibleSize() < mouseWheelStep) ? 1 : mouseWheelStep;
	int lineShift = - delta * mult;

	if(keyFlags  &  MK_CONTROL) {
		scrollHorz.OffsetPosition(lineShift);
	}
	else {
		scrollVert.OffsetPosition(lineShift);
	}
	PostRepaint();
}


//====================================================================
void  ConsoleOutputWindow::OnKeyDown(WPARAM virtKeyCode, LPARAM keyData)
{
	switch(virtKeyCode) {

		case VK_ESCAPE:
			if(bCloseGadgetEnabled) {
				PostQuit();
			}
			break;

		case VK_SPACE:
			// control-space
			if(IsKeyDown(VK_CONTROL)) {
				Minimize();
				break;
			}
			// shift-space
			if(IsKeyDown(VK_SHIFT)) {
				if(IsMaximized()) Restore();
				SetSize(COWIN_MIN_WIDTH, COWIN_MIN_HEIGHT);
				break;
			}
			// space only
			if(IsMaximized()) {
				Restore();
			}
			else {
				Maximize();
			}
			break;

		case VK_PRIOR:
			scrollVert.OffsetPosition(- scrollVert.GetVisibleSize());
			break;

		case VK_NEXT:
			scrollVert.OffsetPosition(scrollVert.GetVisibleSize());
			break;

		case VK_END:
			scrollVert.SetPosition(scrollVert.GetMaxPosition());
			break;

		case VK_HOME:
			scrollVert.SetPosition(0);
			break;

		case VK_LEFT:
			if(IsKeyDown(VK_SHIFT)) {
				if(IsKeyDown(VK_CONTROL)) UpdatePosition(- COWIN_MOVE_SNAP, 0);
				else UpdateSize(- COWIN_SIZE_SNAP, 0);
			}
			else if(IsKeyDown(VK_CONTROL)) scrollHorz.OffsetPosition(- scrollHorz.GetVisibleSize());
			else scrollHorz.OffsetPosition(- 1);
			break;

		case VK_UP:
			if(IsKeyDown(VK_SHIFT)) {
				if(IsKeyDown(VK_CONTROL)) UpdatePosition(0, - COWIN_MOVE_SNAP);
				else UpdateSize(0, - COWIN_SIZE_SNAP);
			}
			else if(IsKeyDown(VK_CONTROL)) scrollVert.OffsetPosition(- scrollVert.GetVisibleSize());
			else scrollVert.OffsetPosition(- 1);
			break;

		case VK_RIGHT:
			if(IsKeyDown(VK_SHIFT)) {
				if(IsKeyDown(VK_CONTROL)) UpdatePosition(COWIN_MOVE_SNAP, 0);
				else UpdateSize(COWIN_SIZE_SNAP, 0);
			}
			else if(IsKeyDown(VK_CONTROL)) scrollHorz.OffsetPosition(scrollHorz.GetVisibleSize());
			else scrollHorz.OffsetPosition(1);
			break;

		case VK_DOWN:
			if(IsKeyDown(VK_SHIFT)) {
				if(IsKeyDown(VK_CONTROL)) UpdatePosition(0, COWIN_MOVE_SNAP);
				else UpdateSize(0, COWIN_SIZE_SNAP);
			}
			else if(IsKeyDown(VK_CONTROL)) scrollVert.OffsetPosition(scrollVert.GetVisibleSize());
			else scrollVert.OffsetPosition(1);
			break;

		case 'c':
		case 'C':
			if(IsMaximized()) return;
			CenterOnDesktop(COWIN_MIN_WIDTH, COWIN_MIN_HEIGHT);
			break;

		default:
			return;
	}
	PostRepaint();
}


//====================================================================
void  ConsoleOutputWindow::OnChanged(const ConsoleOutputBuffer& cob)
{
	if(IsMinimized()) return;

	bool  bFocusOnLastLine = scrollVert.IsPositionAtMaximum();
	cob.Lock();
	scrollHorz.SetTotalSize(cob.GetMaxColumns());
	scrollVert.SetTotalSize(cob.GetNumLines());
	cob.Unlock();
	if(bFocusOnLastLine) {
		scrollVert.SetPositionToMax();
	}

	// repaint window
	if(bTextAreaVisible) {
		PostRepaint();
	}
}


//====================================================================
void  ConsoleOutputWindow::HookToTextSource()
{
	if(pTextSource) {
		if(! bHookedToTextSource) {
			bHookedToTextSource = true;
			pTextSource->AddObserver(*this);
		}
	}
}


//====================================================================
void  ConsoleOutputWindow::UnhookFromTextSource()
{
	if(pTextSource) {
		if(bHookedToTextSource) {
			bHookedToTextSource = false;
			pTextSource->RemoveObserver(*this);
		}
	}
}


//********************************************************************
}//namespace vertexware

