

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\TopLevelWindow.h"
#include "BaseTools\ConsoleBuffers.h"
#include "BaseTools\Memory.h"
#include "BaseTools\ScrollGadget.h"
#include "private\vw_common_exports.h"


//********************************************************************
namespace vertexware {


//====================================================================
class VW_COMMON_API ConsoleOutputWindow : public TopLevelWindow,
											public ConsoleOutputBuffer::Observer
{
public:
	ConsoleOutputWindow();
	virtual ~ConsoleOutputWindow();

	void  SetTitleText(const std::string& s);
	void  SetStatusText(const std::string& s);
	void  SetTextSource(ConsoleOutputBuffer* cob = 0);
	void  SetOutputTypeColor(OutputStream::eOutputType type, color4b c);
	void  SetMouseWheelStep(uint step = 3);
	void  EnableCloseGadget(bool bEnable);
	void  FocusOnLastLine();

	bool  IsSplashSized() const;


private:
	//----------------------
	// Base-class overrides.
	//----------------------
	virtual  eWindowArea  MouseHitTest(const win_utils::Point& mousePosClient, uint& areaSubCode);

	virtual  void  OnSystemColorsChanged();
	virtual  void  OnSystemSettingsChanged();
	virtual  void  OnDisplaySettingsChanged();
	virtual  void  OnActivated(bool bActive);
	virtual  void  OnResized(eResizeAction action);
	virtual  void  OnPaint();
	virtual  void  OnMouseEnter(eWindowArea area, uint areaSubCode);
	virtual  void  OnMouseWheel(int delta, const win_utils::Point& mousePosClient, WPARAM keyFlags);
	virtual  void  OnKeyDown(WPARAM virtKeyCode, LPARAM keyData);

	//----------------------------------------------
	// ConsoleOutputBuffer::Observer implementation.
	//----------------------------------------------
	virtual  void  OnChanged(const ConsoleOutputBuffer& cob);

	//------------------
	// Internal helpers.
	//------------------
	void  UpdateMaximizedPlacement();
	void  UpdateSizeAfterDisplayChange();
	void  SetupBackBuffer();

	void  Render();
	void  RenderFrameArea(Gdiplus::Graphics& g, int w, int h);
	void  RenderTextArea(Gdiplus::Graphics& g, int w, int h);

	void  HookToTextSource();
	void  UnhookFromTextSource();


private:
	ConsoleOutputBuffer*  pTextSource;
	bool                  bHookedToTextSource;

	ScrollGadget  scrollHorz;
	ScrollGadget  scrollVert;
	bool          bTextAreaVisible;

	std::wstring  wsTitleText;
	std::wstring  wsStatusText;

	int   mouseWheelStep;
	bool  bCloseGadgetEnabled;

	bool  bUpdatedSizeAfterDisplayChange;

	//-------------
	// Back-buffer.
	//-------------
	typedef  AutoPtr<Gdiplus::Graphics>  APTR_GRAPHICS;
	typedef  AutoPtr<Gdiplus::Bitmap>    APTR_BITMAP;

	APTR_BITMAP    pBackBufferBitmap;
	APTR_GRAPHICS  pBackBuffer;

	//-------------------
	// Gdiplus resources.
	//-------------------
	Gdiplus::Color         desktopColor;
	Gdiplus::Color         inactiveColor;
	Gdiplus::Color         activeColor;

	Gdiplus::Pen           inactivePen;
	Gdiplus::Pen           activePen;

	Gdiplus::SolidBrush    inactiveBrush;
	Gdiplus::SolidBrush    activeBrush;
	Gdiplus::SolidBrush    textBgrBrush;

	Gdiplus::SolidBrush    textBrushInfo;
	Gdiplus::SolidBrush    textBrushWarning;
	Gdiplus::SolidBrush    textBrushError;
	Gdiplus::SolidBrush    textBrushAttention;
	Gdiplus::SolidBrush    textBrushQuery;
	Gdiplus::SolidBrush    textBrushEcho;
	Gdiplus::SolidBrush    textBrushDebug;

	Gdiplus::FontFamily    barFontFamily;
	Gdiplus::Font          barFont;
	Gdiplus::StringFormat  barStringFormat;

	Gdiplus::FontFamily    textFontFamily;
	Gdiplus::Font          textFont;
	Gdiplus::StringFormat  textStringFormat;
};


//********************************************************************
}//namespace vertexware


