

//--------------------------------------------------------------------
#include "BaseTools\Debug.h"
#include <sstream>
#include <cctype>


//********************************************************************
namespace {


//--------------------------------------------------------------------
using namespace vertexware;


//====================================================================
template<typename T>
void __fastcall CheckTypeSize(uint expectedSize, bool& bOK, std::ostream& os,
								const T& dummy = T())
{
	if(sizeof(T) != expectedSize) {
		bOK = false;
		os << typeid(T).name() << " has size " << sizeof(T);
		os << " byte(s). Expected " << expectedSize << " byte(s)." << std::endl;
	}
}


//********************************************************************
}//namespace


//********************************************************************
namespace vertexware {


//====================================================================
bool __fastcall CheckBuiltInTypeSizes(std::ostream& os)
{
	bool ok = true;
	CheckTypeSize<bool>(1, ok, os);
	CheckTypeSize<ubyte>(1, ok, os);
	CheckTypeSize<byte>(1, ok, os);
	CheckTypeSize<ushort>(2, ok, os);
	CheckTypeSize<short>(2, ok, os);
	CheckTypeSize<uint>(4, ok, os);
	CheckTypeSize<int>(4, ok, os);
	CheckTypeSize<ulong>(4, ok, os);
	CheckTypeSize<long>(4, ok, os);
	CheckTypeSize<float>(4, ok, os);
	CheckTypeSize<double>(8, ok, os);
	return ok;
}


//====================================================================
std::string __fastcall FormatSourceCodeLocation(const std::string& file, uint line,
												bool bForceWinKernelName)
{
	std::ostringstream  oss;
	oss << file << "(" << line << ")";
	std::string  s = oss.str();

	if(bForceWinKernelName) {
		ForceWinKernelName(s);
	}
	return s;
}


//====================================================================
std::string __fastcall FormatClassName(const std::string& nameFromTypeInfo)
{
	std::string s = std::string(nameFromTypeInfo);

	// remove "class " && "struct " prefix generated by VC60
	if(s.length() > 7  &&  s.substr(0, 7) == "struct ") {
		return s.substr(7);
	}
	else if(s.length() > 6  &&  s.substr(0, 6) == "class ") {
		return s.substr(6);
	}
	return s;
}


//====================================================================
std::string __fastcall FormatObjectName(const std::string& nameFromTypeInfo,
										const void* objectAddress)
{
	std::ostringstream  oss;
	oss << FormatClassName(nameFromTypeInfo) << "@0x" << objectAddress;
	return oss.str();
}


//====================================================================
bool __fastcall ShowDialogAbortRetryIgnore(bool& outIgnoreAlways,
						uint defaultButton,
						const std::string& caption, const std::string& message,
						const std::string& file, uint line)
{
	if(outIgnoreAlways) return false;

	std::ostringstream  oss;
	oss << "@ " << file << "(" << line << ")" << "\n! " << message;
	std::string  text = oss.str();

	switch(defaultButton) {
		case 2:  defaultButton = MB_DEFBUTTON2; break;
		case 3:  defaultButton = MB_DEFBUTTON3; break;
		default: defaultButton = MB_DEFBUTTON1; break;
	}

	int result = ::MessageBox(0, text.c_str(), caption.c_str(),
							defaultButton | MB_ABORTRETRYIGNORE |
							MB_ICONSTOP | MB_SETFOREGROUND | MB_TASKMODAL | MB_SETFOREGROUND);
	if(result == IDABORT) {
		outIgnoreAlways = false;
		return true;
	}
	if(result == IDRETRY) {
		outIgnoreAlways = false;
		return false;
	}
	//IDIGNORE
	outIgnoreAlways = true;
	return false;
}


//====================================================================
bool __fastcall ShowDialogCancelOk(uint defaultButton,
						const std::string& caption, const std::string& message)
{
	switch(defaultButton) {
		case 2:  defaultButton = MB_DEFBUTTON2; break;
		default: defaultButton = MB_DEFBUTTON1; break;
	}

	int result = ::MessageBox(NULL, message.c_str(), caption.c_str(),
								defaultButton | MB_OKCANCEL |
								MB_ICONSTOP | MB_SETFOREGROUND | MB_TASKMODAL);
	return (result == IDCANCEL);
}


//====================================================================
void __fastcall ShowDialogOk(const std::string& caption, const std::string& message)
{
	::MessageBox(NULL, message.c_str(), caption.c_str(),
					MB_OK | MB_ICONSTOP | MB_SETFOREGROUND | MB_TASKMODAL);
}


//********************************************************************
}//namespace vertexware


