

//--------------------------------------------------------------------
#include "BaseTools\Debug.h"
#include "BaseTools\Exception.h"
#include <sstream>


//********************************************************************
namespace vertexware {


//====================================================================
Exception::Exception() :
	bMessage(false), message(),
	bFileLine(false), file(), line(0)
{
}


//====================================================================
Exception::~Exception()
{
}


//====================================================================
void  Exception::SetMessage(const std::string& _message)
{
	bMessage = true;
	message = _message;
}


//====================================================================
void  Exception::SetFileLine(const std::string& _file, uint _line)
{
	bFileLine = true;
	file = _file;
	line = _line;
}


//====================================================================
const std::string  Exception::GetClassName() const
{
	return VW_GET_CLASS_NAME(*this);
}


//====================================================================
const std::string  Exception::MakeDescription(bool printClass,
									bool printFileLine, bool printMessage) const
{
	const char* szFileSep = "\n@ ";
	const char* szMsgSep  = "\n! ";

	std::ostringstream oss;

	if(! bFileLine) printFileLine = false;
	if(! bMessage)  printMessage  = false;

	if(printClass) {
		oss << GetClassName();

		if(printFileLine) {
			oss << szFileSep;
		}
		else if(printMessage) {
			oss << szMsgSep;
		}
	}
	if(printFileLine) {
		oss << file << "(" << line << ")";
		if(printMessage) {
			oss << szMsgSep;
		}
	}
	if(printMessage) {
		oss << message;
	}

	return oss.str();
}


//********************************************************************
}//namespace vertexware


