

//--------------------------------------------------------------------
#include "BaseTools\HeapTracked.h"
#include "BaseTools\Debug.h"
#include <algorithm>


//********************************************************************
namespace vertexware {


//====================================================================
bool  HeapTracked::IsOnHeap() const
{
	const void* pRawAddress = dynamic_cast<const void*>(this);
	return (s_rawAddresses.find(pRawAddress) != s_rawAddresses.end());
}


//====================================================================
void*  HeapTracked::operator new(size_t size)
{
	void* pRawAddress = ::operator new(size);
	if(pRawAddress) {
		s_rawAddresses.insert(pRawAddress);
	}
	return pRawAddress;
}


//====================================================================
void  HeapTracked::operator delete(void *ptr)
{
	s_rawAddresses.erase(ptr);
	::operator delete(ptr);
}


//====================================================================
void*  HeapTracked::operator new[](size_t size)
{
	VW_ASSERT(0, "Invalid call: calling new[] for heap-tracked class.");
	return 0;
}


//====================================================================
void  HeapTracked::operator delete[](void *ptr)
{
	VW_ASSERT(0, "Invalid call: calling delete[] for heap-tracked class.");
}


//====================================================================
HeapTracked::RAW_ADDRESS_SET  HeapTracked::s_rawAddresses;


//********************************************************************
}//namespace vertexware


