

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "private\vw_common_exports.h"
#include <set>


//********************************************************************
namespace vertexware {


//====================================================================
// Heap-tracked pattern.
// Can be used to check whether an object is allocated by using
// standard operator new.
// WARNING: problems arise when used in conjuction with virtual bases.
//====================================================================
class VW_COMMON_API HeapTracked
{
public:
	virtual ~HeapTracked() {}

	// call to determine whether the object is created on heap
	bool  IsOnHeap() const;

	static  void*  operator new(size_t size);
	static  void   operator delete(void* ptr);

	// do NOT use these (or any other non-standard forms)
	static  void*  operator new[](size_t size);
	static  void   operator delete[](void* ptr);


protected:
	HeapTracked() {}


private:
	typedef  std::set<const void*>            RAW_ADDRESS_SET;
	typedef  RAW_ADDRESS_SET::iterator        RAW_ADDRESS_ITER;
	typedef  RAW_ADDRESS_SET::const_iterator  RAW_ADDRESS_CITER;

	static   RAW_ADDRESS_SET  s_rawAddresses;
};


//********************************************************************
}//namespace vertexware

