

//--------------------------------------------------------------------
#include "BaseTools\LibraryInfo.h"
#include <sstream>


//********************************************************************
namespace {


//====================================================================
// vertexware library information.
//====================================================================
#define  LIBRARY_NAME               "vertexware"
#define  LIBRARY_COPYRIGHT_NOTICE   "Copyright FER.Zagreb.Croatia"


//====================================================================
// Developer information.
//====================================================================
const char* szDeveloperInfo[] =
{
	"petar.basic.2002-2002 (original author, design, coding)"
};


//********************************************************************
}//namespace


//********************************************************************
namespace vertexware {


//********************************************************************
namespace library_info {


//====================================================================
LibraryVersion::LibraryVersion(uint _major, uint _minor, uint _revision, uint _buildFlags)
{
	major = _major;
	minor = _minor;
	revision = _revision;
	buildFlags = _buildFlags;
}


//====================================================================
bool  LibraryVersion::operator==(const LibraryVersion& rhs) const
{
	return
	(major == rhs.major)  &&  (minor == rhs.minor)  &&
	(revision == rhs.revision)  &&  (buildFlags == rhs.buildFlags);
}


//====================================================================
bool  LibraryVersion::IsBuildTypeDebug() const
{
	return (buildFlags == VW_LIBRARY_VERSION_BUILDTYPE_DEBUG);
}


//====================================================================
bool  LibraryVersion::IsBuildTypeRelease() const
{
	return (buildFlags == VW_LIBRARY_VERSION_BUILDTYPE_RELEASE);
}


//====================================================================
std::string  LibraryVersion::MakeString() const
{
	const char sep = '.';

	std::ostringstream oss;
	oss << major << sep;
	oss << minor << sep;
	oss << revision << sep;

	if(IsBuildTypeDebug()) oss << "debug";
	else if(IsBuildTypeRelease()) oss << "release";
	else oss << "UnknownBuildType";

	return oss.str();
}


//====================================================================
bool  LibraryVersion::IsCompatibleWith(const LibraryVersion& rhs) const
{
	return
	(major == rhs.major)  &&  (minor == rhs.minor)  &&
	(buildFlags == rhs.buildFlags);
}


//====================================================================
LibraryVersion __fastcall GetLibraryVersion()
{
	return LibraryVersion();
}


//====================================================================
std::string __fastcall GetBuildTimestamp()
{
	return VW_BUILD_TIMESTAMP_STRING;
}


//====================================================================
std::string __fastcall GetLibraryName()
{
	return LIBRARY_NAME;
}


//====================================================================
std::string __fastcall GetCopyrightNotice()
{
	return LIBRARY_COPYRIGHT_NOTICE;
}


//====================================================================
uint __fastcall GetNumDevelopers()
{
	return (sizeof(szDeveloperInfo) / sizeof(szDeveloperInfo[0]));
}


//====================================================================
std::string __fastcall GetDeveloperInfo(uint i)
{
	if(i >= GetNumDevelopers()) return "";
	return szDeveloperInfo[i];
}


//********************************************************************
}//namespace library_info


//********************************************************************
}//namespace vertexware


