

//--------------------------------------------------------------------
#include "BaseTools\OutputForms.h"
#include "BaseTools\Stream.h"


//********************************************************************
namespace vertexware {


//====================================================================
NumberOutputForm::NumberOutputForm()
{
	NumberWidth();
	FPNumberPrecision();
	FPNumberFormat();
}


//====================================================================
NumberSequenceOutputForm::NumberSequenceOutputForm()
{
	Multiline();
	SequenceProlog();
	NumberSeparator();
	SequenceEpilog();
}


//====================================================================
std::ostream& __fastcall operator<<(std::ostream &os,
									const NumberSequenceBoundOutputForm& boundForm)
{
	// setup stream flags
	StreamFormatFlagsSentry sffs(os);

	os.precision(boundForm.form.FPNumberPrecision());
	os.setf(boundForm.form.FPNumberFormat(), std::ios_base::floatfield);

	// cache parameters
	NumberSequenceBoundOutputForm::eValueType  valueType = boundForm.GetValueType();

	const uint maxElem = boundForm.elemCount - 1;
	std::string  sTemp;

	// output values
	uint slotCount = 0;
	for(uint elem = 0; elem <= maxElem; elem++) {

		// per-value prolog and setup
		if(slotCount == 0) {
			os << boundForm.form.SequenceProlog();
			slotCount = boundForm.lineLength;
		}
		if(boundForm.form.GetNumberProlog(elem, sTemp)) {
			os << sTemp;
		}
		os.width(boundForm.form.NumberWidth());

		// value-type dependant output
		switch(valueType) {
			case NumberSequenceBoundOutputForm::ValueType_Int:
				os << boundForm.GetValueInt(elem);
				break;
			case NumberSequenceBoundOutputForm::ValueType_UInt:
				os << boundForm.GetValueUInt(elem);
				break;
			case NumberSequenceBoundOutputForm::ValueType_Double:
				os << boundForm.GetValueDouble(elem);
				break;
			default:
				os << "BadValueType";
				break;
		}

		// per-value epilog and setup
		if(boundForm.form.GetNumberEpilog(elem, sTemp)) {
			os << sTemp;
		}
		slotCount--;

		if(slotCount != 0  &&  elem != maxElem) {
			os << boundForm.form.NumberSeparator();
		}
		if(slotCount == 0) {
			os << boundForm.form.SequenceEpilog();
			if(boundForm.form.Multiline()  &&  elem != maxElem) {
				os << '\n';
			}
		}
	}
	return os;
}


//********************************************************************
}//namespace vertexware

