

//--------------------------------------------------------------------
#include "BaseTools\PlatformInfo.h"
#include "BaseTools\Constructs.h"
#include "BaseTools\Memory.h"
#include <Lmcons.h>
#include <sstream>


//********************************************************************
namespace {


//--------------------------------------------------------------------
using namespace vertexware;


//====================================================================
__forceinline bool __fastcall Verify2000(uint major, uint minor) {
	return (major == 5  &&  minor == 0);
}


//====================================================================
__forceinline bool __fastcall VerifyXP(uint major, uint minor) {
	return (major == 5  &&  minor == 1);
}


//********************************************************************
}//namespace


//********************************************************************
namespace vertexware {


//********************************************************************
namespace platform_info {


//====================================================================
std::string __fastcall GetMachineName()
{
	const uint buflen = MAX_COMPUTERNAME_LENGTH + 1;

	char  nameBuffer[buflen];
	DWORD nameLen = buflen;
	::GetComputerName(nameBuffer, &nameLen);

	return std::string(nameBuffer, nameLen);
}


//====================================================================
std::string __fastcall GetUserName()
{
	const uint buflen = UNLEN + 1;

	char  nameBuffer[buflen];
	DWORD nameLen = buflen;
	::GetUserName(nameBuffer, &nameLen);

	return std::string(nameBuffer, nameLen);
}


//====================================================================
bool __fastcall ExpandEnvironmentVariable(const std::string& envVar, std::string& out)
{
	out.erase();
	uint len = ::ExpandEnvironmentStrings(envVar.c_str(), 0, 0);
	Buffer<char> buffer(len);
	len = ::ExpandEnvironmentStrings(envVar.c_str(), buffer, len);
	if(len) {
		out.assign(buffer);
		return (envVar != out);
	}
	return false;
}


//====================================================================
bool __fastcall GetOperatingSystemVersion(
							uint* pMajor, uint* pMinor, uint* pBuild,
							uint* pServicePackMajor, uint* pServicePackMinor)
{
	OSVERSIONINFOEX osvi;
	::ZeroMemory(&osvi, sizeof(osvi));
	osvi.dwOSVersionInfoSize = sizeof(osvi);
	if(! ::GetVersionEx(reinterpret_cast<OSVERSIONINFO*>(&osvi))) {
		return false;
	}
	SafeAssign<uint>(pMajor, osvi.dwMajorVersion);
	SafeAssign<uint>(pMinor, osvi.dwMinorVersion);
	SafeAssign<uint>(pBuild, osvi.dwBuildNumber);
	SafeAssign<uint>(pServicePackMajor, osvi.wServicePackMajor);
	SafeAssign<uint>(pServicePackMinor, osvi.wServicePackMinor);
	return true;
}


//====================================================================
bool __fastcall RunningOnWindows2000()
{
	uint major = 0, minor = 0;
	GetOperatingSystemVersion(&major, &minor, 0, 0, 0);
	return Verify2000(major, minor);
}


//====================================================================
bool __fastcall RunningOnWindowsXP()
{
	uint major = 0, minor = 0;
	GetOperatingSystemVersion(&major, &minor, 0, 0, 0);
	return VerifyXP(major, minor);
}


//====================================================================
std::string __fastcall GetOperatingSystemDescription()
{
	uint  major, minor, build, spMajor, spMinor;
	GetOperatingSystemVersion(&major, &minor, &build, &spMajor, &spMinor);

	const char  dot = '.';
	const char* sep = " / ";

	std::ostringstream oss;

	oss << "Microsoft Windows";
	if(Verify2000(major, minor)) oss << " 2000";
	if(VerifyXP(major, minor)) oss << " XP";
	oss << sep << major << dot << minor << dot << build;
	oss << sep << "Service Pack " << spMajor << dot << spMinor;

	return oss.str();
}


//====================================================================
bool __fastcall GetWorkArea(uint* pWidth, uint* pHeight, uint* pLeft, uint* pTop)
{
	RECT rect;
	if(::SystemParametersInfo(SPI_GETWORKAREA, 0, &rect, 0) != 0) {
		SafeAssign<uint>(pLeft, rect.left);
		SafeAssign<uint>(pTop, rect.top);
		SafeAssign<uint>(pWidth, rect.right - rect.left);
		SafeAssign<uint>(pHeight, rect.bottom - rect.top);
	}
	return false;
}


//====================================================================
bool __fastcall GetDesktopDisplayMode(DEVMODE& dm)
{
	::ZeroMemory(&dm, sizeof(dm));
	dm.dmSize = sizeof(dm);
	if(! ::EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &dm)) {
		return false;
	}
	return true;
}


//====================================================================
bool __fastcall GetDesktopDisplayMode(uint* pWidth, uint* pHeight,
									uint* pBitsPerPixel, uint* pRefreshFrequency)
{
	DEVMODE dm;
	if(! GetDesktopDisplayMode(dm)) {
		return false;
	}
	SafeAssign<uint>(pWidth, dm.dmPelsWidth);
	SafeAssign<uint>(pHeight, dm.dmPelsHeight);
	SafeAssign<uint>(pBitsPerPixel, dm.dmBitsPerPel);
	SafeAssign<uint>(pRefreshFrequency, dm.dmDisplayFrequency);
	return true;
}


//====================================================================
std::string __fastcall GetDesktopDisplayModeDescription()
{
	uint  width = 0, height = 0, bpp = 0, freq = 0;
	GetDesktopDisplayMode(&width, &height, &bpp, &freq);

	const char* sep1 = " x ";
	const char* sep2 = " @ ";

	std::ostringstream oss;
	oss << width << sep1 << height << sep1 << bpp << sep2 << freq;
	return oss.str();
}


//********************************************************************
}//namespace platform_info


//********************************************************************
}//namespace vertexware


