

//--------------------------------------------------------------------
#include "BaseTools\ScrollGadget.h"


//********************************************************************
namespace vertexware {


//====================================================================
ScrollGadget::ScrollGadget()
{
	ResetLogicalParameters();
	ResetDrawingParameters();
}


//====================================================================
ScrollGadget::~ScrollGadget()
{
}


//====================================================================
void  ScrollGadget::ResetLogicalParameters()
{
	total   = 1;
	visible = 1;
	pos     = 0;
}


//====================================================================
void  ScrollGadget::ResetDrawingParameters()
{
	barLength        = 1;
	minThumbLength   = 1;
	maxPixelsPerStep = 1;
	bEnableMaxPixelsPerStep = true;

	thumbOffset = 0;
	thumbLength = 1;

	barPosX  = 0;
	barPosY  = 0;
	barWidth = 1;
}


//====================================================================
void  ScrollGadget::UpdateThumbParameters()
{
	float fNumSteps = float(GetTotalSize()) - float(GetVisibleSize());
	if(fNumSteps > 0) {

		float ratioVisibleToTotal = float(GetVisibleSize()) / float(GetTotalSize());
		float fThumbLength = ClampedLow<float>(ratioVisibleToTotal * barLength, minThumbLength);

		if(bEnableMaxPixelsPerStep) {
			if(((barLength - fThumbLength) / fNumSteps) > maxPixelsPerStep) {
				ClampLow<float>(fThumbLength, barLength - fNumSteps * maxPixelsPerStep);
			}
		}
		thumbLength = fThumbLength;

		float ratioPosToMaxPos = float(GetPosition()) / float(GetMaxPosition());
		thumbOffset = ratioPosToMaxPos * (barLength - thumbLength);
	}
	else {
		thumbLength = barLength;
		thumbOffset = 0;
	}
}


//====================================================================
void  ScrollGadget::SetTotalSize(uint _total)
{
	total = ClampedLow<uint>(_total, 1);
	SetPosition(pos); //recompute
}


//====================================================================
void  ScrollGadget::SetVisibleSize(uint _visible)
{
	visible = ClampedLow<uint>(_visible, 1);
	SetPosition(pos); //recompute
}


//====================================================================
uint  ScrollGadget::GetMaxPosition() const
{
	if(visible >= total) return 0;
	return (total - visible);
}


//====================================================================
uint  ScrollGadget::GetLastVisiblePosition() const
{
	return ClampedHigh<uint>(pos + visible - 1, total - 1);
}


//====================================================================
void  ScrollGadget::SetPosition(int _pos)
{
	pos = Clamped<int>(_pos, 0, GetMaxPosition());
}


//********************************************************************
}//namespace vertexware


