

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\CompilerConfig.h"
#include "BaseTools\Exception.h"
#include "private\vw_common_exports.h"
#include <iosfwd>


//********************************************************************
namespace vertexware {


//====================================================================
VW_DECLARE_EXCEPTION( SerializationException )
VW_DECLARE_EXCEPTION( DeserializationException )


//====================================================================
// Serializable pattern.
// Defines low-level binary storage interface.
//====================================================================
class VW_COMMON_API Serializable
{
public:
	virtual  ~Serializable() { }

	virtual  void  Serialize(std::ostream& os) const = 0;
	virtual  void  Deserialize(std::istream& is) = 0;
};


//====================================================================
__forceinline  std::ostream& operator<=(std::ostream& os, const Serializable& s) {
	s.Serialize(os);
	return os;
}


//====================================================================
__forceinline  std::istream& operator>=(std::istream& is, Serializable& s) {
	s.Deserialize(is);
	return is;
}


//********************************************************************
}//namespace vertexware


