

//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "BaseTools\String.h"
#include "BaseTools\Memory.h"
#include <sstream>


//--------------------------------------------------------------------
#define  VW_BOOL_STRING_0      "0"
#define  VW_BOOL_STRING_1      "1"
#define  VW_BOOL_STRING_FALSE  "false"
#define  VW_BOOL_STRING_TRUE   "true"
#define  VW_BOOL_STRING_OFF    "off"
#define  VW_BOOL_STRING_ON     "on"
#define  VW_BOOL_STRING_NO     "no"
#define  VW_BOOL_STRING_YES    "yes"


//********************************************************************
namespace {


//====================================================================
template<typename T>
bool __fastcall ToValueImp(const std::string& src, T& dst)
{
	std::string trimmedSrc;
	vertexware::TrimWhitespace(src, trimmedSrc);

	T  temp;
	std::istringstream iss(trimmedSrc);
	iss >> temp;
	if(iss.fail()  ||  ! iss.eof()) {
		return false;
	}
	dst = temp;
	return true;
}


//********************************************************************
}//namespace


//********************************************************************
namespace vertexware {


//====================================================================
bool __fastcall AsciiToUnicode(const std::string& src, std::wstring& dst)
{
	dst.erase();
	if(! src.length()) return true;
	dst.reserve(src.length());

	// get required buffer size
	int  bufferSize = ::MultiByteToWideChar(CP_UTF8, 0, src.c_str(), src.length(), 0, 0);
	if(! bufferSize) return false;

	// do actual conversion
	Buffer<wchar_t>  buffer(bufferSize);
	int numChars = ::MultiByteToWideChar(CP_UTF8, 0, src.c_str(), src.length(),
											buffer, bufferSize);
	dst.assign(buffer, numChars);

	return (numChars != 0);
}


//====================================================================
bool __fastcall UnicodeToAscii(const std::wstring& src, std::string& dst)
{
	dst.erase();
	if(! src.length()) return true;
	dst.reserve(src.length());

	// get required buffer size
	int  bufferSize = ::WideCharToMultiByte(CP_UTF8, 0, src.c_str(), src.length(),
											0, 0, 0, 0);
	if(! bufferSize) return false;

	// do actual conversion
	Buffer<char>  buffer(bufferSize);
	int numChars = ::WideCharToMultiByte(CP_UTF8, 0, src.c_str(), src.length(),
											buffer, bufferSize, 0, 0);
	dst.assign(buffer, numChars);

	return (numChars != 0);
}


//====================================================================
bool __fastcall ToValue(const std::string& src, bool& dst)
{
	std::string s = LowerCase(src);

	if(s == VW_BOOL_STRING_0  ||  s == VW_BOOL_STRING_FALSE ||
		s == VW_BOOL_STRING_OFF  ||  s == VW_BOOL_STRING_NO)
	{
		dst = false;
		return true;
	}
	if(s == VW_BOOL_STRING_1  ||  s == VW_BOOL_STRING_TRUE ||
		s == VW_BOOL_STRING_ON  ||  s == VW_BOOL_STRING_YES)
	{
		dst = true;
		return true;
	}
	return false;
}


//====================================================================
bool __fastcall ToValue(const std::string& src, byte& dst) {
	return ToValueImp(src, dst);
}


//====================================================================
bool __fastcall ToValue(const std::string& src, ubyte& dst) {
	return ToValueImp(src, dst);
}


//====================================================================
bool __fastcall ToValue(const std::string& src, short& dst) {
	return ToValueImp(src, dst);
}


//====================================================================
bool __fastcall ToValue(const std::string& src, ushort& dst) {
	return ToValueImp(src, dst);
}


//====================================================================
bool __fastcall ToValue(const std::string& src, int& dst) {
	return ToValueImp(src, dst);
}


//====================================================================
bool __fastcall ToValue(const std::string& src, uint& dst) {
	return ToValueImp(src, dst);
}


//====================================================================
bool __fastcall ToValue(const std::string& src, float& dst) {
	return ToValueImp(src, dst);
}


//====================================================================
bool __fastcall ToValue(const std::string& src, double& dst) {
	return ToValueImp(src, dst);
}


//====================================================================
bool __fastcall ToValue(const std::string& src, void*& dst) {
	return ToValueImp(src, dst);
}


//====================================================================
void __fastcall ToString(std::string& out, bool src)
{
	out.assign(src ? VW_BOOL_STRING_TRUE : VW_BOOL_STRING_FALSE);
}


//====================================================================
void __fastcall ToString(std::string& out, int src)
{
	std::ostringstream oss;
	oss << src;
	out.assign(oss.str());
}


//====================================================================
void __fastcall ToString(std::string& out, uint src)
{
	std::ostringstream oss;
	oss << src;
	out.assign(oss.str());
}


//====================================================================
void __fastcall ToStringHex(std::string& out, uint src, uint width)
{
	std::ostringstream oss;
	oss.setf(std::ios::hex, std::ios::basefield);
	oss.width(width);
	oss.fill('0');
	oss << src;
	out.assign(oss.str());
}


//====================================================================
void __fastcall ToString(std::string& out, long src)
{
	std::ostringstream oss;
	oss << src;
	out.assign(oss.str());
}


//====================================================================
void __fastcall ToString(std::string& out, ulong src)
{
	std::ostringstream oss;
	oss << src;
	out.assign(oss.str());
}


//====================================================================
void __fastcall ToStringHex(std::string& out, ulong src, uint width)
{
	std::ostringstream oss;
	oss.setf(std::ios::hex, std::ios::basefield);
	oss.width(width);
	oss.fill('0');
	oss << src;
	out.assign(oss.str());
}


//====================================================================
void __fastcall ToString(std::string& out, double src, uint width, uint maxDigits)
{
	std::ostringstream oss;
	oss.width(width);
	oss.setf(std::ios_base::fmtflags(0), std::ios_base::floatfield);
	oss.precision(maxDigits);
	oss << src;
	out.assign(oss.str());
}


//====================================================================
void __fastcall ToStringFix(std::string& out, double src, uint width, uint maxPrec,
							bool bShowPlusSign, bool bInternalPadding)
{
	std::ostringstream oss;
	oss.width(width);
	oss.setf(std::ios_base::fixed, std::ios_base::floatfield);
	oss.precision(maxPrec);
	if(bShowPlusSign) {
		oss.setf(std::ios_base::showpos);
	}
	if(bInternalPadding) {
		oss.setf(std::ios_base::internal, std::ios_base::adjustfield);
	}
	oss << src;
	out.assign(oss.str());
}


//====================================================================
void __fastcall ToStringSci(std::string& out, double src, uint width, uint maxPrec,
							bool bShowPlusSign, bool bInternalPadding)
{
	std::ostringstream oss;
	oss.width(width);
	oss.setf(std::ios_base::scientific, std::ios_base::floatfield);
	oss.precision(maxPrec);
	if(bShowPlusSign) {
		oss.setf(std::ios_base::showpos);
	}
	if(bInternalPadding) {
		oss.setf(std::ios_base::internal, std::ios_base::adjustfield);
	}
	oss << src;
	out.assign(oss.str());
}


//====================================================================
void __fastcall ToString(std::string& out, void* src)
{
	std::ostringstream oss;
	oss << "0x" << src;
	out.assign(oss.str());
}


//====================================================================
std::string __fastcall LowerCase(const std::string& s)
{
	std::string sLower;
	sLower.reserve(s.length());
	for(uint i = 0; i < s.length(); i++) {
		sLower.append(1, tolower(s[i]));
	}
	return sLower;
}


//====================================================================
std::string __fastcall UpperCase(const std::string& s)
{
	std::string sUpper;
	sUpper.reserve(s.length());
	for(uint i = 0; i < s.length(); i++) {
		sUpper.append(1, toupper(s[i]));
	}
	return sUpper;
}


//====================================================================
void __fastcall TrimWhitespace(std::string& s)
{
	// remove trailing whitespace
	while(s.size()) {
		uint last = s.size() - 1;
		if(s[last] != ' '  &&  s[last] != '\t') break;
		s.erase(last, 1);
	}
	// remove leading whitespace
	while(s.size()) {
		if(s[0] != ' '  &&  s[0] != '\t') break;
		s.erase(0, 1);
	}
}


//====================================================================
void __fastcall TrimWhitespace(const std::string& src, std::string& dst)
{
	const char* szWhitespaceChars = " \t";

	uint first = src.find_first_not_of(szWhitespaceChars);
	if(first != std::string::npos) {

		uint last = src.find_last_not_of(szWhitespaceChars);
		dst.assign(src, first, last - first + 1);
	}
	else {
		// no non-whitespace characters
		dst.erase();
	}
}


//====================================================================
void __fastcall TrimQuotes(std::string& s)
{
	const char singleQuote = '\'';
	const char doubleQuote = '"';

	if(s.size() >= 2) {
		if((s[0] == singleQuote  &&  s[s.size() - 1] == singleQuote) ||
			(s[0] == doubleQuote  &&  s[s.size() - 1] == doubleQuote))
		{
			s.assign(s.substr(1, s.size() - 2));
		}
	}
}


//====================================================================
bool __fastcall MatchNext(const std::string& s, const std::string& choiceChars)
{
	if(s.empty()) return false;
	return  (choiceChars.find(s[0]) != std::string::npos);
}


//====================================================================
bool __fastcall MatchNext(const std::string& s, char ch)
{
	return  (s.size()  &&  s[0] == ch);
}


//====================================================================
uint __fastcall EatLeading(std::string& s, const std::string& choiceChars)
{
	uint eatCount = 0;
	while(eatCount < s.size()  &&  choiceChars.find(s[eatCount]) != std::string::npos) {
		eatCount++;
	}
	if(eatCount) {
		s.erase(0, eatCount);
	}
	return  eatCount;
}


//====================================================================
uint __fastcall EatNext(std::string& s, char ch)
{
	if(s.size()  &&  s[0] == ch) {
		s.erase(0, 1);
		return  1;
	}
	return  0;
}


//====================================================================
bool __fastcall MatchPrefix(const std::string& s, const std::string& prefix)
{
	return  (s.compare(0, prefix.size(), prefix) == 0);
}


//====================================================================
bool __fastcall EatPrefix(std::string& s, const std::string& prefix)
{
	if(s.compare(0, prefix.size(), prefix) == 0) {
		s.erase(0, prefix.size());
		return  true;
	}
	return  false;
}


//====================================================================
bool __fastcall EatToDelimiter(std::string& s, const std::string& delimChars, bool eraseIfNotFound)
{
	uint delimPos = s.find_first_of(delimChars);
	if(delimPos != std::string::npos) {
		s.erase(0, delimPos);
		return  true;
	}
	if(eraseIfNotFound) {
		s.erase();
	}
	return  false;
}


//====================================================================
std::string __fastcall FitToLineLengthBreakAtWhitespaceOnly(
								const std::string& s, uint maxLineLength)
{
	std::istringstream iss(s);
	std::string formatted;
	std::string word;

	formatted.reserve(s.length());
	uint lineLen = 0;

	while(iss >> word) {
		uint totalLen = lineLen + 1 + word.length();

		if(totalLen <= maxLineLength) {
			if(lineLen != 0) {
				formatted.append(1, ' ');
				lineLen ++;
			}
			formatted.append(word);
			lineLen += word.length();
		}
		else {
			if(formatted.length()) {
				formatted.append(1, '\n');
			}
			formatted.append(word);
			lineLen = word.length();
		}
	}
	return formatted;
}


//====================================================================
std::string __fastcall AddLinePrefix(const std::string& s,
									const std::string& linePrefix)
{
	std::string formatted;
	formatted.reserve(s.length() * 2);

	std::string::size_type seekPos = 0;
	while(seekPos < s.length()) {

		formatted.append(linePrefix);
		std::string::size_type breakPos = s.find('\n', seekPos);
		if(breakPos == std::string::npos) {
			formatted.append(s.substr(seekPos));
			break;
		}
		breakPos ++; //include newline
		formatted.append(s.substr(seekPos, breakPos - seekPos));
		seekPos = breakPos;
	}

	if(formatted.empty()) {
		formatted.append(linePrefix);
	}
	return formatted;
}


//====================================================================
void __fastcall PadToLength(std::string& s, uint minLength, char padChar)
{
	int padLen = minLength - s.length();
	if(padLen > 0) {
		s.append(padLen, padChar);
	}
}


//====================================================================
void __fastcall BreakIntoLines(const std::string& s,
								std::vector<std::string>& lines,
								char breakChar)
{
	lines.clear();

	uint pos = 0;
	while(pos < s.length()) {
		uint end = s.find(breakChar, pos);

		if(end != std::string::npos) {
			lines.push_back(s.substr(pos, end - pos));
			pos = end + 1;
		}
		else {
			lines.push_back(s.substr(pos));
			pos = std::string::npos;
		}
	}
}


//====================================================================
// Standard identifier helpers: alpha-numeric, underscore, cannot
// start with number, no whitespace.
//====================================================================
bool __fastcall IsStandardIdentifier(const std::string& s)
{
	for(uint i = 0; i < s.length(); i++) {
		char ch = s[i];

		int bAlpha = isalpha(ch)  ||  (ch == '_');
		int bDigit = isdigit(ch);

		if(i == 0) {
			if(! bAlpha) return false;
		}
		else {
			if(! bAlpha && ! bDigit) return false;
		}
	}
	return true;
}


//====================================================================
// Removes characters that are not allowed, kills whitespace,
// trims head-characters that are number.
//====================================================================
std::string __fastcall ForceStandardIdentifier(const std::string& s)
{
	std::string out;
	out.reserve(s.length());

	for(uint i = 0; i < s.length(); i++) {
		char ch = s[i];

		int bAlpha = isalpha(ch)  ||  (ch == '_');
		int bDigit = isdigit(ch);

		if(out.empty()) {
			if(bAlpha) out.append(1, ch);
		}
		else {
			if(bAlpha || bDigit) out.append(1, ch);
		}
	}
	return out;
}


//********************************************************************
}//namespace vertexware


