

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\CompilerConfig.h"
#include "BaseTools\Types.h"
#include <string>
#include <vector>


//********************************************************************
namespace vertexware {


//====================================================================
// Ascii / Unicode helpers.
//
// Functions return true on success.
//====================================================================
__forceinline  bool __fastcall IsAscii(char ch) {
	return (ch <= 127);
}


//====================================================================
__forceinline  bool __fastcall IsDisplayableAscii(char ch) {
	return (ch >= 32  &&  ch <= 126);
}


//====================================================================
VW_COMMON_API
bool __fastcall AsciiToUnicode(const std::string& src, std::wstring& dst);


//====================================================================
VW_COMMON_API
bool __fastcall UnicodeToAscii(const std::wstring& src, std::string& dst);


//====================================================================
// Conversions std::string ===> value.
//
// Functions return true on success.
// <dst> argument is preserved if conversion fails.
//====================================================================


//====================================================================
// Accepts the following strings:
// "0", "false", "off", "no"  ===> as false
// "1", "true",  "on",  "yes" ===> as true
//====================================================================
VW_COMMON_API
bool __fastcall ToValue(const std::string& src, bool& dst);


//====================================================================
VW_COMMON_API
bool __fastcall ToValue(const std::string& src, byte& dst);


//====================================================================
VW_COMMON_API
bool __fastcall ToValue(const std::string& src, ubyte& dst);


//====================================================================
VW_COMMON_API
bool __fastcall ToValue(const std::string& src, short& dst);


//====================================================================
VW_COMMON_API
bool __fastcall ToValue(const std::string& src, ushort& dst);


//====================================================================
VW_COMMON_API
bool __fastcall ToValue(const std::string& src, int& dst);


//====================================================================
VW_COMMON_API
bool __fastcall ToValue(const std::string& src, uint& dst);


//====================================================================
VW_COMMON_API
bool __fastcall ToValue(const std::string& src, float& dst);


//====================================================================
VW_COMMON_API
bool __fastcall ToValue(const std::string& src, double& dst);


//====================================================================
__forceinline  bool __fastcall ToValue(const std::string& src, std::string& dst) {
	dst = src;
	return true;
}


//====================================================================
VW_COMMON_API
bool __fastcall ToValue(const std::string& src, void*& dst);


//====================================================================
// Conversions std::string ===> value.
//
// <failValue> argument is copied to <dst> if conversion fails.
//====================================================================
template<typename T>
__forceinline  void __fastcall ToValue(const std::string& src, T& dst, const T& failValue) {
	if(! ToValue(src, dst)) {
		dst = failValue;
	}
}


//====================================================================
// Conversions value ===> std::string.
//
// <width> argument specifies minimum width of the constructed string.
//         0 indicates as many as needed.
// <maxDigits> argument specifies maximum total number of digits
//             in constructed string.
// <maxPrec> argument specifies maximum number of digits after the
//             decimal point in constructed string.
//====================================================================
VW_COMMON_API
void __fastcall ToString(std::string& out, bool src);


//====================================================================
__forceinline std::string __fastcall ToString(bool src) {
	std::string s; ToString(s, src); return s;
}


//====================================================================
VW_COMMON_API
void __fastcall ToString(std::string& out, int src);


//====================================================================
__forceinline std::string __fastcall ToString(int src) {
	std::string s; ToString(s, src); return s;
}


//====================================================================
VW_COMMON_API
void __fastcall ToString(std::string& out, uint src);


//====================================================================
__forceinline std::string __fastcall ToString(uint src) {
	std::string s; ToString(s, src); return s;
}


//====================================================================
VW_COMMON_API
void __fastcall ToStringHex(std::string& out, uint src, uint width = 0);


//====================================================================
__forceinline std::string __fastcall ToStringHex(uint src, uint width = 0) {
	std::string s; ToStringHex(s, src, width); return s;
}


//====================================================================
VW_COMMON_API
void __fastcall ToString(std::string& out, long src);


//====================================================================
__forceinline std::string __fastcall ToString(long src) {
	std::string s; ToString(s, src); return s;
}


//====================================================================
VW_COMMON_API
void __fastcall ToString(std::string& out, ulong src);


//====================================================================
__forceinline std::string __fastcall ToString(ulong src) {
	std::string s; ToString(s, src); return s;
}


//====================================================================
VW_COMMON_API
void __fastcall ToStringHex(std::string& out, ulong src, uint width = 0);


//====================================================================
__forceinline std::string __fastcall ToStringHex(ulong src, uint width = 0) {
	std::string s; ToStringHex(s, src, width); return s;
}


//====================================================================
VW_COMMON_API
void __fastcall ToString(std::string& out, double src, uint width = 0, uint maxDigits = 6);


//====================================================================
__forceinline std::string __fastcall ToString(double src, uint width = 0, uint maxDigits = 6) {
	std::string s; ToString(s, src, width, maxDigits); return s;
}


//====================================================================
VW_COMMON_API
void __fastcall ToStringFix(std::string& out, double src, uint width = 0, uint maxPrec = 6,
							bool bShowPlusSign = false, bool bInternalPadding = false);


//====================================================================
__forceinline std::string __fastcall ToStringFix(double src, uint width = 0, uint maxPrec = 6,
							bool bShowPlusSign = false, bool bInternalPadding = false)
{
	std::string s; ToStringFix(s, src, width, maxPrec, bShowPlusSign, bInternalPadding); return s;
}


//====================================================================
VW_COMMON_API
void __fastcall ToStringSci(std::string& out, double src, uint width = 0, uint maxPrec = 6,
							bool bShowPlusSign = false, bool bInternalPadding = false);


//====================================================================
__forceinline std::string __fastcall ToStringSci(double src, uint width = 0, uint maxPrec = 6,
							bool bShowPlusSign = false, bool bInternalPadding = false)
{
	std::string s; ToStringSci(s, src, width, maxPrec, bShowPlusSign, bInternalPadding); return s;
}


//====================================================================
VW_COMMON_API
void __fastcall ToString(std::string& out, void* src);


//====================================================================
__forceinline std::string __fastcall ToString(void* src) {
	std::string s; ToString(s, src); return s;
}


//====================================================================
__forceinline std::string __fastcall ToString(const std::string& src) {
	return src;
}


//====================================================================
// Force character case.
//====================================================================
VW_COMMON_API
std::string __fastcall LowerCase(const std::string& s);


//====================================================================
VW_COMMON_API
std::string __fastcall UpperCase(const std::string& s);


//====================================================================
// Removes leading & trailing whitespace.
//====================================================================
VW_COMMON_API
void __fastcall TrimWhitespace(std::string& s);


//====================================================================
VW_COMMON_API
void __fastcall TrimWhitespace(const std::string& src, std::string& dst);


//====================================================================
// Removes leading & trailing quotes (single or double).
//====================================================================
VW_COMMON_API
void __fastcall TrimQuotes(std::string& s);


//====================================================================
// Returns true if first character in <s> matches any of the
// characters from <choiceChars>.
//====================================================================
VW_COMMON_API
bool __fastcall MatchNext(const std::string& s, const std::string& choiceChars);


//====================================================================
// Returns true if first character in <s> matches <ch>.
//====================================================================
VW_COMMON_API
bool __fastcall MatchNext(const std::string& s, char ch);


//====================================================================
// Eats all leading characters in <s> that match any of the
// characters from <choiceChars>. Returns number of eaten characters.
//====================================================================
VW_COMMON_API
uint __fastcall EatLeading(std::string& s, const std::string& choiceChars);


//====================================================================
// Eats leading character from <s> if it matches <ch>.
// Returns number of eaten characters (0 or 1).
//====================================================================
VW_COMMON_API
uint __fastcall EatNext(std::string& s, char ch);


//====================================================================
// Returns true if <s> starts with <prefix>.
//====================================================================
VW_COMMON_API
bool __fastcall MatchPrefix(const std::string& s, const std::string& prefix);


//====================================================================
// If <s> starts with <prefix>, returns true and removes <prefix>
// from <s>. Otherwise returns false.
//====================================================================
VW_COMMON_API
bool __fastcall EatPrefix(std::string& s, const std::string& prefix);


//====================================================================
// If any of the <delimChars> is found, all characters positioned
// before delimiter occurence are removed and true is returned.
// If delimiter is not found false is returned. If <eraseIfNotFound>
// is true <s> is erased.
//====================================================================
VW_COMMON_API
bool __fastcall EatToDelimiter(std::string& s, const std::string& delimChars,
														bool eraseIfNotFound);


//====================================================================
// Fits the string to given line-length. Line-breaks are inserted
// only at whitespace positions. All whitespace is replaced by
// single-space. Words longer than <maxLineLength>
// are placed in single line. No trailing newline is appended.
//====================================================================
VW_COMMON_API
std::string __fastcall FitToLineLengthBreakAtWhitespaceOnly(
								const std::string& s, uint maxLineLength);


//====================================================================
// Adds prefix to each line.
//====================================================================
VW_COMMON_API
std::string __fastcall AddLinePrefix(const std::string& s,
									const std::string& linePrefix);


//====================================================================
// Pad to length if shorter.
//====================================================================
VW_COMMON_API
void __fastcall PadToLength(std::string& s, uint minLength, char padChar);


//====================================================================
// Break into multiple-lines.
//====================================================================
VW_COMMON_API
void __fastcall BreakIntoLines(const std::string& s,
								std::vector<std::string>& lines,
								char breakChar = '\n');


//====================================================================
// Standard identifier helpers: alpha, digit, underscore, cannot
// start with digit, no whitespace.
//====================================================================
VW_COMMON_API
bool __fastcall IsStandardIdentifier(const std::string& s);


//====================================================================
// Removes characters that are not allowed, kills whitespace,
// trims head-characters that are digit.
//====================================================================
VW_COMMON_API
std::string __fastcall ForceStandardIdentifier(const std::string& s);


//********************************************************************
}//namespace vertexware

