

//--------------------------------------------------------------------
#include "BaseTools\SyncPrimitives.h"
#include "BaseTools\Exception.h"


//********************************************************************
namespace {


//====================================================================
void __fastcall UpdateKernelNamePosition(const std::string s,
							const std::string& prefix, vertexware::uint& pos)
{
	if(! s.compare(0, prefix.length(), prefix)) {
		pos = prefix.length();
	}
}


//********************************************************************
}//namespace


//********************************************************************
namespace vertexware {


//====================================================================
void __fastcall ForceWinKernelName(std::string& s)
{
	// force character-count
	if(s.length() > MAX_PATH) {
		s.erase(MAX_PATH);
	}
	uint pos = 0;

	// reserved prefixes are the only place where backslash is allowed
	UpdateKernelNamePosition(s, "Local\\", pos);
	UpdateKernelNamePosition(s, "Global\\", pos);
	UpdateKernelNamePosition(s, "Session\\", pos);

	// convert backslashes to slashes
	for(uint i = pos; i < s.length(); i++) {
		if(s[i] == '\\') {
			s[i] = '/';
		}
	}
}


//====================================================================
Event::Event(bool initiallySet, bool manualReset,
			const std::string& _name, bool bForceWinKernelName) :
	name(_name)
{
	if(name.length() == 0) {
		// unnamed event
		handle = ::CreateEvent(0, manualReset, initiallySet, 0);
	}
	else {
		// named event
		if(bForceWinKernelName) {
			ForceWinKernelName(name);
		}
		handle = ::CreateEvent(0, manualReset, initiallySet, name.c_str());
	}
	// check for errors
	if(! handle) {
		VW_THROW(SystemResourceException, "CreateEvent() failed.");
	}
}


//====================================================================
Event::~Event()
{
	::CloseHandle(handle);
}


//====================================================================
bool  Event::WaitInfinite()
{
	DWORD result = ::WaitForSingleObject(handle, INFINITE);
	if(result == WAIT_OBJECT_0) return true;
	return false;
}


//====================================================================
bool  Event::Wait(uint msecs, bool& bTimeout)
{
	DWORD result = ::WaitForSingleObject(handle, msecs);
	if(result == WAIT_OBJECT_0) {
		bTimeout = false;
		return true;
	}
	if(result == WAIT_TIMEOUT) {
		bTimeout = true;
		return true;
	}
	bTimeout = false;
	return false;
}


//====================================================================
SingleInstanceFlagSentry::SingleInstanceFlagSentry(const std::string& _name,
													bool bForceWinKernelName) :
	name(_name)
{
	handle = 0;
	if(name.empty()) return;

	if(bForceWinKernelName) {
		ForceWinKernelName(name);
	}

	// create OS-kernel-object
	::SetLastError(ERROR_SUCCESS);
	handle = ::CreateMutex(0, TRUE, name.c_str());

	if(handle) {
		if(::GetLastError() == ERROR_ALREADY_EXISTS) {
			// opened existing mutex, this is not the first instance
			::CloseHandle(handle);
			handle = 0;
		}
		// first owner, this is the first instance
	}
	// could not create mutex
}


//====================================================================
SingleInstanceFlagSentry::~SingleInstanceFlagSentry()
{
	if(handle) {
		::CloseHandle(handle);
	}
}


//********************************************************************
}//namespace vertexware

