

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "BaseTools\Exception.h"
#include "BaseTools\SyncPrimitives.h"


//********************************************************************
namespace vertexware {


//====================================================================
class VW_COMMON_API Thread
{
public:
	Thread();
	virtual ~Thread();

	//---------------------------------------------------
	// Priority control (as defined in platform-sdk under
	// SetThreadPriority, SetThreadPriorityBoost).
	//---------------------------------------------------
	void  SetPriority(int priority = THREAD_PRIORITY_NORMAL);
	int   GetPriority() const { return priority; }

	void  EnablePriorityBoost(bool bEnable = true);
	bool  IsEnabledPriorityBoost() const { return bEnablePriorityBoost; }

	//---------------------
	// Cooperative control.
	//---------------------
	void  Run();
	bool  IsRunning();

	bool  WaitReady();

	void  SignalExit();
	void  WaitExit(bool bCloseThreadHandle = true);
	uint  GetLastExitCode();

	bool  PostStandardMessage(UINT message, WPARAM wparam = 0, LPARAM lparam = 0);
	bool  PostStandardNullMessage() { return PostStandardMessage(WM_NULL); }
	bool  PostStandardQuitMessage() { return PostStandardMessage(WM_QUIT); }

	// If <maxRetries> is 0, wait-test is executed infinite number of times,
	// otherwise it is executed <maxRetries> times with <msecSleepInterval>
	// sleep-pause between consecutive wait-tests. Returns false if failed.
	bool  WaitMessageQueueReady(uint maxRetries = 0, uint msecSleepInterval = 0);

	//---------------------
	// Brute-force control.
	//---------------------
	bool  Suspend();
	bool  Resume();

	//----------------
	// OS data access.
	//----------------
	HANDLE  GetThreadHandle() const { return threadHandle; }
	uint    GetThreadID() const { return threadID; }


private:
	HANDLE  threadHandle;
	uint    threadID;

	int     priority;
	bool    bEnablePriorityBoost;

	uint    exitCode;
	Event   exitEvent;
	Event   readyEvent;

	//---------------------------------------------------
	// This flag can be tested by work-function instead
	// of more heavy-weight exit-event tests.
	// Intended for use in simple looping work-functions.
	//---------------------------------------------------
	volatile  bool  bContinue;


private:
	//--------------------
	// OS-callback mapper.
	//--------------------
	static  uint __stdcall WorkFunctionMapper(void*);


protected:
	//-------------------------
	// Derived-class interface.
	//-------------------------
	bool    QueryContinue() { return bContinue; }
	bool    IsExitSignaled();
	void    WaitExitSignal();
	Event&  GetExitEvent() { return exitEvent; }

	void    SignalReady();
	Event&  GetReadyEvent() { return readyEvent; }

	//--------------------------------------------
	// Thread's work-function (returns exit-code).
	//--------------------------------------------
	virtual  uint  DoWork() = 0;
};


//********************************************************************
}//namespace vertexware

