

//--------------------------------------------------------------------
#include "DataObjects\ImageBase.h"


//********************************************************************
namespace vertexware {


//====================================================================
ImageBase::ImageBase(uint sampleByteSize)
	:
	SignalBase(sampleByteSize)
{
	numRows = 0;
	numColumns = 0;
	rowByteSize = 0;
}


//====================================================================
void  ImageBase::SetupInternalImageProperties(uint _numRows, uint _numColumns)
{
	// if any dimension is 0, force all to 0
	if(! _numRows  ||  ! _numColumns) {
		_numRows = 0;
		_numColumns = 0;
	}

	numRows = _numRows;
	numColumns = _numColumns;
	rowByteSize = numColumns * GetSampleByteSize();
}


//====================================================================
void  ImageBase::ReserveSampleStorage(uint _numRows, uint _numColumns)
{
	SetupInternalImageProperties(_numRows, _numColumns);
	SignalBase::ReserveSampleStorage(numRows * numColumns);
}


//====================================================================
void  ImageBase::ReserveSampleStorage(uint numSamples)
{
	ImageBase::ReserveSampleStorage(1, numSamples);
}


//====================================================================
void  ImageBase::ReleaseSampleStorage()
{
	SetupInternalImageProperties(0, 0);
	SignalBase::ReleaseSampleStorage();
}


//====================================================================
bool  ImageBase::CheckDimensionMatch(const ImageBase& img) const
{
	return
	(numRows == img.GetNumRows())  &&
	(numColumns == img.GetNumColumns());
}


//====================================================================
ImageBaseFromExternalStorageAdapter::ImageBaseFromExternalStorageAdapter(
													eSampleFormat sampleFormat)
	:
	SignalBase(sample_formats::ExtractByteSize(sampleFormat)),
	ImageBase(sample_formats::ExtractByteSize(sampleFormat))
{
	SetSampleFormat(sampleFormat);
}


//====================================================================
void  ImageBaseFromExternalStorageAdapter::SetExternalStorage(
						ubyte* pStorage, uint numRows, uint numColumns)
{
	if(pStorage) {
		VW_ASSERT(numRows && numColumns,
				"Invalid argument combination passed to ImageBase adapter.");

		SetupInternalImageProperties(numRows, numColumns);
		AllocateStorage(numRows * numColumns, pStorage);
	}
	else {
		SetupInternalImageProperties(0, 0);
		DeallocateStorage();
	}
}


//====================================================================
void  ImageBaseFromExternalStorageAdapter::ReserveSampleStorage(uint, uint)
{
	VW_THROW(InvalidUsageException, "Cannot manage external storage through ImageBase adapter.");
}


//====================================================================
void  ImageBaseFromExternalStorageAdapter::ReserveSampleStorage(uint)
{
	VW_THROW(InvalidUsageException, "Cannot manage external storage through ImageBase adapter.");
}


//********************************************************************
}//namespace vertexware


