

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "DataObjects\SignalBase.h"


//********************************************************************
namespace vertexware {


//====================================================================
// ImageBase provides sample-type independent operations.
// Size can be 0x0, but samples must not be accessed in that case.
//====================================================================
class VW_COMMON_API ImageBase : virtual public SignalBase
{
public:
	// storage configuration
	virtual  void  ReserveSampleStorage(uint numRows, uint numColumns);
	virtual  void  ReserveSampleStorage(uint numSamples);
	virtual  void  ReleaseSampleStorage();

	// storage info
	__forceinline  uint  GetNumRows() const { return numRows; }
	__forceinline  uint  GetNumColumns() const { return numColumns; }
	__forceinline  uint  GetRowByteSize() const { return rowByteSize; }

	bool  CheckDimensionMatch(const ImageBase& img) const;

	// storage access
	using SignalBase::SampleB;

	__forceinline  ubyte*  SampleB(uint row, uint column) {
		CheckAccess2D(row, column);
		return (GetSamplesB() + GetSampleByteSize() * (row * numColumns + column));
	}
	__forceinline  const ubyte*  SampleB(uint row, uint column) const {
		return const_cast<ImageBase*>(this)->SampleB(row, column);
	}


protected:
	// constructor
	ImageBase(uint sampleByteSize);

	// debug helpers
	__forceinline  void  CheckAccess2D(uint row, uint column) const {
		VW_ASSERT(GetNumSamples(), "Accessing 0x0 image.");
		VW_ASSERT(row < numRows, "Row-index out of range.");
		VW_ASSERT(column < numColumns,  "Column-index out of range.");
	}

	// low-level setup used by reserve/release overrides
	void  SetupInternalImageProperties(uint numRows, uint numColumns);


private:
	uint  numRows;
	uint  numColumns;
	uint  rowByteSize;
};


//====================================================================
// Adapts externally allocated storage so that it appears as ImageBase.
// Reserve is not allowed. External storage must be tightly packed.
//====================================================================
class VW_COMMON_API ImageBaseFromExternalStorageAdapter : public ImageBase
{
public:
	ImageBaseFromExternalStorageAdapter(eSampleFormat sampleFormat);

	void  SetExternalStorage(ubyte* pStorage, uint numRows, uint numColumns);

	// storage configuration (reserve throws exception if called)
	virtual  void  ReserveSampleStorage(uint, uint);
	virtual  void  ReserveSampleStorage(uint);
	virtual  void  ReleaseSampleStorage() {}
};


//********************************************************************
}//namespace vertexware


