

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "DataObjects\SignalOperations.h"
#include "DataObjects\Image.h"


//********************************************************************
namespace vertexware {


//====================================================================
VW_COMMON_API
void __fastcall GdiplusFillBitmapData(Gdiplus::BitmapData& bitmapData,
							Gdiplus::PixelFormat pixelFormat, const ImageBase& image);


//====================================================================
// Transfer pixels to/from user-buffer.
// <bitmapData> must be prepared.
//====================================================================
VW_COMMON_API
bool __fastcall GdiplusTransferBitmapPixels(Gdiplus::Bitmap& bitmap,
							Gdiplus::BitmapData& bitmapData, bool bToUserBuffer);


//====================================================================
// Returns 0 if sample-byte-size is unsupported.
//====================================================================
VW_COMMON_API
ImageBase* __fastcall ManufactureImage(eSampleFormat sampleFormat,
									uint numRows = 0, uint numColumns = 0);


//====================================================================
// Returns 0 if sample-byte-size is unsupported.
//====================================================================
VW_COMMON_API
ImageBase* __fastcall CloneImage(const ImageBase& src);


//====================================================================
// Sample-formats must match.
// Offsets and counts must be within valid range.
// If <rowCount> is 0, full source-row-count is copied.
// If <columnCount> is 0, full source-column-count is copied.
//====================================================================
VW_COMMON_API
void __fastcall CopySamples2D(ImageBase& dst, const ImageBase& src,
							uint rowCount = 0, uint columnCount = 0,
							uint dstRowOffset = 0, uint dstColumnOffset = 0,
							uint srcRowOffset = 0, uint srcColumnOffset = 0,
							bool bFlipRows = false);


//====================================================================
// Returns false if transfer is not supported.
// If <bAutoResize> is false, dimensions must be equal.
//====================================================================
VW_COMMON_API
bool __fastcall TransferSamples2D(ImageBase& dst, const ImageBase& src,
											bool bAutoResize = true);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
//====================================================================
VW_COMMON_API
void __fastcall Resample2D_Nearest(ImageBase& dst, const ImageBase& src);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
// <numComponentsPerSample> must be <= (sample-size / sizeof(float)).
// <componentOffset> must be <= ((sample-size / sizeof(float)) - <numComponentsPerSample>).
// Each component is assumed to be a separate channel.
//====================================================================
VW_COMMON_API
void __fastcall Resample2D_Linear_NFloats(ImageBase& dst, const ImageBase& src,
								uint componentOffset, uint numComponentsPerSample);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
// <numBytesPerSample> must be <= sample-size.
// <byteOffset> must be <= (sample-size - <numBytesPerSample>).
// Each byte is assumed to be a separate channel.
//====================================================================
VW_COMMON_API
void __fastcall Resample2D_Linear_NBytes(ImageBase& dst, const ImageBase& src,
										uint byteOffset, uint numBytesPerSample);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// sample-format byte-size must be 2.
// dst-sample-format and src-sample-format must be equal.
// Each bit-group is assumed to be a separate channel.
//====================================================================
VW_COMMON_API
void __fastcall Resample2D_Linear_PackedSample_UShort4444(ImageBase& dst, const ImageBase& src);


//====================================================================
VW_COMMON_API
void __fastcall Resample2D_Linear_PackedSample_UShort1555(ImageBase& dst, const ImageBase& src);


//====================================================================
// <src> or <dst> dimensions cannot be 0.
// dst-sample-format and src-sample-format must be equal.
// Returns false if filter-type/sample-format combo is not supported.
//====================================================================
VW_COMMON_API
bool __fastcall Resample2D(ImageBase& dst, const ImageBase& src,
							eResampleFilter filterType = ResampleFilter_Linear);


//====================================================================
VW_COMMON_API
void __fastcall FlipRows(ImageBase& target);


//====================================================================
VW_COMMON_API
void __fastcall FlipColumns(ImageBase& target);


//====================================================================
// <byteOffset> must be < target-sample-size.
// <topRow> must be < target-num-rows.
// <topRow> + <countRows> must be <= target-num-rows.
// <leftColumn> must be < target-num-columns.
// <leftColumn> + <countColumns> must be <= target-num-columns.
//====================================================================
VW_COMMON_API
void __fastcall FillBlock(ImageBase& target, uint byteOffset, ubyte value,
						uint topRow, uint countRows, uint leftColumn, uint countColumns);


//====================================================================
// <byteOffset> must be < target-sample-size.
// Desired target-dimensions must be set before the call.
//====================================================================
VW_COMMON_API
void __fastcall GenerateCheckerboard(ImageBase& target, uint byteOffset,
							ubyte value1, ubyte value2,
							uint vertBlockSize, uint horzBlockSize);


//====================================================================
// target-sample-size must be >= 3.
// <tripletOffset> must be <= (target-sample-size - 3).
// Desired target-dimensions must be set before the call.
//====================================================================
VW_COMMON_API
void __fastcall GenerateCheckerboard(ImageBase& target, uint tripletOffset,
							color4b rgb1, color4b rgb2,
							uint vertBlockSize, uint horzBlockSize);


//====================================================================
// Desired target-dimensions must be set before the call.
// Minimum useful size == 64 x 64
// Recommended size >= 256 x 256
//====================================================================
VW_COMMON_API
void __fastcall GenerateRGBATest(ImageBase& target);


//====================================================================
// Returns true if font is supported.
//====================================================================
VW_COMMON_API
bool __fastcall CanGenerateFont(const std::string& fontFamilyName,
								bool bBold, bool bItalic);


//====================================================================
// If font is unsupported generic font will be used.
// Pass empty string for <fontFamilyName> to use generic font.
// <target> is resized to minimum possible size, depending on
// <cellSize>, <numGlyphs> and <numGlyphsPerRow>.
// if <bAsciiSubsetOnly> is false, otherwise it is resized to
// (cellSize * 8)-rows x (cellSize * 16)-columns.
// Multiples of 16 are recommended for <cellSize>.
// <bAntialias> is not recommended in conjuction with small <cellSize>.
// <bDrawCells> is provided for debugging & tweaking purposes.
//====================================================================
VW_COMMON_API
void __fastcall GenerateFontGlyphs(ImageBase& target,
								uint numGlyphs,
								const wchar_t* glyphs,
								const std::string& fontFamilyName = "Arial",
								bool bBold = false,
								bool bItalic = false,
								uint numGlyphsPerRow = 16,
								uint cellSize = 32,
								float scaleEmHeight = 1,
								float glyphOffsetH = 0,
								float glyphOffsetV = 0,
								float glyphSizeModifyH = 0,
								float glyphSizeModifyV = 0,
								bool bAntialias = true,
								bool bAlphaPremultiply = false,
								bool bDrawCells = false);


//====================================================================
// If font is unsupported generic font will be used.
// Generates ASCII glyphs. Accepted range is [0-255].
//====================================================================
VW_COMMON_API
void __fastcall GenerateFontGlyphsAscii(ImageBase& target,
								uint firstAsciiGlyph = 0,
								uint lastAsciiGlyph = 127,
								const std::string& fontFamilyName = "Arial",
								bool bBold = false,
								bool bItalic = false,
								uint numGlyphsPerRow = 16,
								uint cellSize = 32,
								float scaleEmHeight = 1,
								float glyphOffsetH = 0,
								float glyphOffsetV = 0,
								float glyphSizeModifyH = 0,
								float glyphSizeModifyV = 0,
								bool bAntialias = true,
								bool bAlphaPremultiply = false,
								bool bDrawCells = false);


//********************************************************************
}//namespace vertexware


