

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "DataObjects\SignalBase.h"


//********************************************************************
namespace vertexware {


//====================================================================
// Typed signal.
//====================================================================
template<typename TSample>
class Signal : virtual public SignalBase
{
public:
	typedef  TSample  SampleType;


public:
	// constructor
	Signal(eSampleFormat sampleFormat =
			eSampleFormat(sample_formats::UnknownSampleFormat(sizeof(TSample))),
			uint numSamples = 0)
		:
		SignalBase(sizeof(TSample))
	{
		SetSampleFormat(sampleFormat);
		if(numSamples) {
			SignalBase::ReserveSampleStorage(numSamples);
		}
	}

	// storage access
	__forceinline  TSample*  GetSamples() {
		return reinterpret_cast<TSample*>(GetSamplesB());
	}
	__forceinline  const TSample*  GetSamples() const {
		return reinterpret_cast<const TSample*>(GetSamplesB());
	}

	__forceinline  TSample*  Sample(uint sampleIndex) {
		CheckAccess1D(sampleIndex);
		return (GetSamples() + sampleIndex);
	}
	__forceinline  const TSample*  Sample(uint sampleIndex) const {
		return const_cast<Signal<TSample >* >(this)->Sample(sampleIndex);
	}
};


//====================================================================
// Standard signal types.
//====================================================================
typedef  Signal<Sample8>    Signal8;
typedef  Signal<Sample16>   Signal16;
typedef  Signal<Sample24>   Signal24;
typedef  Signal<Sample32>   Signal32;
typedef  Signal<Sample48>   Signal48;
typedef  Signal<Sample64>   Signal64;
typedef  Signal<Sample96>   Signal96;
typedef  Signal<Sample128>  Signal128;


//********************************************************************
}//namespace vertexware


