

//--------------------------------------------------------------------
#include "DataObjects\SignalOperations.h"
#include "DataObjects\SampleProcessors.h"
#include "MathTools\MathBase.h"
#include "BaseTools\Memory.h"


//********************************************************************
namespace {


//--------------------------------------------------------------------
using namespace vertexware;


//====================================================================
struct Process_AlphaFromColorKey_A8R8G8B8 : public ElementProcessorInplace<ubyte>
{
	color4b  rgbColorKey;
	ubyte    alphaMatch;
	ubyte    alphaMismatch;

	Process_AlphaFromColorKey_A8R8G8B8(color4b _rgbColorKey, ubyte _alphaMatch, ubyte _alphaMismatch)
	{
		rgbColorKey = _rgbColorKey;
		alphaMatch = _alphaMatch;
		alphaMismatch = _alphaMismatch;
	}

	__forceinline  void  Process(ubyte* elem)
	{
		if(elem[0] == rgbColorKey.b  &&  elem[1] == rgbColorKey.g  &&  elem[2] == rgbColorKey.r) {
			elem[3] = alphaMatch;
		}
		else {
			elem[3] = alphaMismatch;
		}
	}
};


//====================================================================
template<typename TElementProcessor>
__forceinline
void __fastcall Call_ResampleElements1D_Nearest_ubyte_ubyte(
									TElementProcessor& copyProcessor,
									SignalBase& dst, const SignalBase& src)
{
	ResampleElements1D_Nearest<ubyte, ubyte>(
			copyProcessor,
			dst.GetSamplesB(), dst.GetSampleByteSize(), dst.GetNumSamples(),
			src.GetSamplesB(), src.GetSampleByteSize(), src.GetNumSamples());
}


//====================================================================
template<typename TLinearInterpolator1D>
__forceinline
void __fastcall Call_ResampleElements1D_Linear_PackedSample_UShort(
									TLinearInterpolator1D& interpolator,
									SignalBase& dst, const SignalBase& src)
{
	VW_ASSERT(dst.GetNumSamples(), "Destination is empty.");
	VW_ASSERT(src.GetNumSamples(), "Source is empty.");
	VW_ASSERT(dst.GetSampleByteSize() == 2, "Invalid sample-format size.");
	VW_ASSERT(dst.CheckSampleFormatMatch(src), "Sample-format mismatch.");

	ResampleElements1D_Linear<ubyte, ubyte>(
			interpolator,
			dst.GetSamplesB(), dst.GetSampleByteSize(), dst.GetNumSamples(),
			src.GetSamplesB(), src.GetSampleByteSize(), src.GetNumSamples());
}


//====================================================================
// Internal transfer handlers
//====================================================================
struct TSH_CopySamples : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples(dst, src);
	}
};
TSH_CopySamples tsh_CopySamples;


//====================================================================
template<uint DstByteOffset, uint SrcByteOffset, uint NumBytesPerSample>
struct TSH_CopySamples_NBytes : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_NBytes(dst, DstByteOffset, src, SrcByteOffset, NumBytesPerSample);
	}
};
TSH_CopySamples_NBytes<0, 0, 1>  tsh_CopySamples_NBytes_d0_s0_n1;
TSH_CopySamples_NBytes<0, 1, 1>  tsh_CopySamples_NBytes_d0_s1_n1;
TSH_CopySamples_NBytes<1, 0, 1>  tsh_CopySamples_NBytes_d1_s0_n1;
TSH_CopySamples_NBytes<0, 3, 1>  tsh_CopySamples_NBytes_d0_s3_n1;
TSH_CopySamples_NBytes<3, 0, 1>  tsh_CopySamples_NBytes_d3_s0_n1;


//====================================================================
template<uint DstLOffset, uint SrcBGROffset>
struct TSH_CopySamples_3BytesBGRTo1ByteL : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_3BytesBGRTo1ByteL(dst, DstLOffset, src, SrcBGROffset);
	}
};
TSH_CopySamples_3BytesBGRTo1ByteL<0, 0>  tsh_CopySamples_3BytesBGRTo1ByteL_d0_s0;


//====================================================================
template<uint DstBGROffset, uint SrcLOffset>
struct TSH_CopySamples_1ByteLTo3BytesBGR : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_1ByteLTo3BytesBGR(dst, DstBGROffset, src, SrcLOffset);
	}
};
TSH_CopySamples_1ByteLTo3BytesBGR<0, 0>  tsh_CopySamples_1ByteLTo3BytesBGR_d0_s0;


//====================================================================
template<uint DstLOffset, uint SrcBGROffset, uint DstCloneOffset, uint SrcCloneOffset>
struct TSH_CopySamples_3BytesBGRTo1ByteL1ByteClone : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_3BytesBGRTo1ByteL(dst, DstLOffset, src, SrcBGROffset);
		CopySamples_NBytes(dst, DstCloneOffset, src, SrcCloneOffset, 1);
	}
};
TSH_CopySamples_3BytesBGRTo1ByteL1ByteClone<0, 0, 1, 3>  tsh_CopySamples_3BytesBGRTo1ByteL1ByteClone_d0_s0_d1_s3;


//====================================================================
template<uint DstAOffset, uint DstBGROffset, uint SrcBGROffset>
struct TSH_CopySamples_3BytesBGRTo4BytesBGRA : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_3BytesBGRTo1ByteL(dst, DstAOffset, src, SrcBGROffset);
		CopySamples_NBytes(dst, DstBGROffset, src, SrcBGROffset, 3);
	}
};
TSH_CopySamples_3BytesBGRTo4BytesBGRA<3, 0, 0>  tsh_CopySamples_3BytesBGRTo4BytesBGRA_d3_0_s0;


//====================================================================
template<uint DstBGROffset, uint SrcLOffset, uint DstCloneOffset, uint SrcCloneOffset>
struct TSH_CopySamples_1ByteLTo3BytesBGR1ByteClone : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_1ByteLTo3BytesBGR(dst, DstBGROffset, src, SrcLOffset);
		CopySamples_NBytes(dst, DstCloneOffset, src, SrcCloneOffset, 1);
	}
};
TSH_CopySamples_1ByteLTo3BytesBGR1ByteClone<0, 0, 3, 1>  tsh_CopySamples_1ByteLTo3BytesBGR1ByteClone_d0_s0_d3_s1;


//====================================================================
struct TSH_CopySamples_UInt8888_To_UShort4444 : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_UInt8888_To_UShort4444(dst, src);
	}
};
TSH_CopySamples_UInt8888_To_UShort4444  tsh_CopySamples_UInt8888_To_UShort4444;


//====================================================================
struct TSH_CopySamples_UShort4444_To_UInt8888 : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_UShort4444_To_UInt8888(dst, src);
	}
};
TSH_CopySamples_UShort4444_To_UInt8888  tsh_CopySamples_UShort4444_To_UInt8888;


//====================================================================
struct TSH_CopySamples_UInt8888_To_UShort1555 : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_UInt8888_To_UShort1555(dst, src);
	}
};
TSH_CopySamples_UInt8888_To_UShort1555  tsh_CopySamples_UInt8888_To_UShort1555;


//====================================================================
struct TSH_CopySamples_UShort1555_To_UInt8888 : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_UShort1555_To_UInt8888(dst, src);
	}
};
TSH_CopySamples_UShort1555_To_UInt8888  tsh_CopySamples_UShort1555_To_UInt8888;


//====================================================================
struct TSH_CopySamples_FP32Norm_To_UInt8 : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_FP32Norm_To_UInt8(dst, src);
	}
};
TSH_CopySamples_FP32Norm_To_UInt8  tsh_CopySamples_FP32Norm_To_UInt8;


//====================================================================
struct TSH_CopySamples_UInt8_To_FP32Norm : public SignalOperationsManager::TransferSamplesHandler
{
	virtual  void  Handle(SignalBase& dst, const SignalBase& src) {
		CopySamples_UInt8_To_FP32Norm(dst, src);
	}
};
TSH_CopySamples_UInt8_To_FP32Norm  tsh_CopySamples_UInt8_To_FP32Norm;


//====================================================================
// Internal convert handlers
//====================================================================
struct CSH_Null : public SignalOperationsManager::ConvertSamplesHandler
{
	virtual  void  Handle(SignalBase& target, uint newSampleFormat) {
	}
};
CSH_Null  csh_Null;


//********************************************************************
}//namespace


//********************************************************************
namespace vertexware {


//====================================================================
void __fastcall ProcessSamples_FlipByteOrder(Signal16& target)
{
	ProcessElements1D(element_processors::Process_FlipOrder2<ubyte>(),
						target.GetNumSamples(), target.GetSamplesB(), 2);
}


//====================================================================
void __fastcall ProcessSamples_FlipByteOrder(Signal32& target)
{
	ProcessElements1D(element_processors::Process_FlipOrder4<ubyte>(),
						target.GetNumSamples(), target.GetSamplesB(), 4);
}


//====================================================================
void __fastcall ProcessSamples_FlipByteOrder(SignalBase& target)
{
	ProcessElements1D(
			element_processors::Process_FlipOrderN<ubyte>(target.GetSampleByteSize()),
			target.GetNumSamples(), target.GetSamplesB(), target.GetSampleByteSize());
}


//====================================================================
void __fastcall ProcessSamples_3BytesFlipByteOrder(SignalBase& target, uint tripletOffset)
{
	VW_ASSERT((target.GetSampleByteSize() >= 3)  &&
			(tripletOffset <= target.GetSampleByteSize() - 3),
			"Triplet-offset out of range.");

	ProcessElements1D(
			element_processors::Process_FlipOrder3<ubyte>(),
			target.GetNumSamples(), target.GetSamplesB() + tripletOffset,
			target.GetSampleByteSize());
}


//====================================================================
void __fastcall ProcessSamples_SwapBytes(SignalBase& target,
									uint byteOffset1, uint byteOffset2)
{
	VW_ASSERT(byteOffset1 < target.GetSampleByteSize(), "Byte-offset-1 out of range.");
	VW_ASSERT(byteOffset2 < target.GetSampleByteSize(), "Byte-offset-2 out of range.");

	ProcessElements1D(
			element_processors::Process_Swap1<ubyte>(byteOffset2),
			target.GetNumSamples(), target.GetSamplesB() + byteOffset1,
			target.GetSampleByteSize());
}


//====================================================================
void __fastcall ProcessSamples_Clear(SignalBase& target)
{
	ProcessElements1D(
			element_processors::Process_FillN<ubyte>(target.GetSampleByteSize(), 0),
			target.GetNumSamples(), target.GetSamplesB(), target.GetSampleByteSize());
}


//====================================================================
void __fastcall ProcessSamples_Fill1Byte(SignalBase& target,
										uint byteOffset, ubyte value)
{
	VW_ASSERT(byteOffset < target.GetSampleByteSize(), "Byte-offset out of range.");

	ProcessElements1D(
			element_processors::Process_Fill1<ubyte>(value),
			target.GetNumSamples(), target.GetSamplesB() + byteOffset,
			target.GetSampleByteSize());
}


//====================================================================
void __fastcall ProcessSamples_FillNBytes(SignalBase& target, ubyte value)
{
	for(uint i = 0; i < target.GetSampleByteSize(); i++) {
		ProcessSamples_Fill1Byte(target, i, value);
	}
}


//====================================================================
void __fastcall ProcessSamples_Fill3Bytes(SignalBase& target, uint tripletOffset,
										ubyte value0, ubyte value1, ubyte value2)
{
	VW_ASSERT((target.GetSampleByteSize() >= 3)  &&
			(tripletOffset <= target.GetSampleByteSize() - 3),
			"Triplet-offset out of range.");

	ProcessElements1D(
			element_processors::Process_Fill3<ubyte>(value0, value1, value2),
			target.GetNumSamples(), target.GetSamplesB() + tripletOffset,
			target.GetSampleByteSize());
}


//====================================================================
void __fastcall ProcessSamples_Fill1Nibble(SignalBase& target,
								uint byteOffset, bool bNibbleHigh, ubyte value)
{
	VW_ASSERT(byteOffset < target.GetSampleByteSize(), "Byte-offset out of range.");

	if(bNibbleHigh) {
		ProcessElements1D(
				sample_processors::Process_Fill1NibbleHigh(value),
				target.GetNumSamples(), target.GetSamplesB() + byteOffset,
				target.GetSampleByteSize());
	}
	else {
		ProcessElements1D(
				sample_processors::Process_Fill1NibbleLow(value),
				target.GetNumSamples(), target.GetSamplesB() + byteOffset,
				target.GetSampleByteSize());
	}
}


//====================================================================
void __fastcall ProcessSamples_FillBits_UShort(SignalBase& target, ushort value,
												ushort bitMask, ushort bitShift)
{
	VW_ASSERT(target.GetSampleByteSize() == 2, "Invalid target-sample-size.");

	ProcessElements1D(
			sample_processors::Process_FillBits<ushort>(value, bitMask, bitShift),
			target.GetNumSamples(), target.GetSamplesB(), target.GetSampleByteSize());
}


//====================================================================
void __fastcall ProcessSamples_BGRToA_ThresholdBGR(SignalBase& target)
{
	if(target.GetSampleByteSize() < 4) return;
	CopySamples_3BytesBGRTo1ByteL(target, 3, target, 0);
	ThresholdSegmentationNBytes(target, 0, 3, 0, 0, 0, 255);
}


//====================================================================
void __fastcall ProcessSamples_AlphaFromColorKey(
								SignalBase& target, color4b rgbColorKey,
								ubyte alphaMatch, ubyte alphaMismatch)
{
	if(target.GetSampleByteSize() != 4) return;

	ProcessElements1D(
			Process_AlphaFromColorKey_A8R8G8B8(rgbColorKey, alphaMatch, alphaMismatch),
			target.GetNumSamples(), target.GetSamplesB(),
			target.GetSampleByteSize());
}


//====================================================================
void __fastcall CopySamples_FlippedByteOrder(Signal16& dst, const Signal16& src)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");

	ProcessElements1D(element_processors::Copy_FlippedOrder2<ubyte>(),
		dst.GetNumSamples(), dst.GetSamplesB(), 2, src.GetSamplesB(), 2);
}


//====================================================================
void __fastcall CopySamples_FlippedByteOrder(Signal32& dst, const Signal32& src)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");

	ProcessElements1D(element_processors::Copy_FlippedOrder4<ubyte>(),
		dst.GetNumSamples(), dst.GetSamplesB(), 4, src.GetSamplesB(), 4);
}


//====================================================================
void __fastcall CopySamples_3BytesFlippedByteOrder(SignalBase& dst, uint dstTripletOffset,
									const SignalBase& src, uint srcTripletOffset)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");

	VW_ASSERT((dst.GetSampleByteSize() >= 3)  &&
			(dstTripletOffset <= dst.GetSampleByteSize() - 3),
			"Destination-triplet-offset out of range.");

	VW_ASSERT((src.GetSampleByteSize() >= 3)  &&
			(srcTripletOffset <= src.GetSampleByteSize() - 3),
			"Source-triplet-offset out of range.");

	ProcessElements1D(
			element_processors::Copy_FlippedOrder3<ubyte>(), dst.GetNumSamples(),
			dst.GetSamplesB() + dstTripletOffset, dst.GetSampleByteSize(),
			src.GetSamplesB() + srcTripletOffset, src.GetSampleByteSize());
}


//====================================================================
void __fastcall CopySamples_3BytesBGRTo1ByteL(SignalBase& dst, uint dstLOffset,
									const SignalBase& src, uint srcBGROffset)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");
	VW_ASSERT(dstLOffset < dst.GetSampleByteSize(), "Destination-L-offset out of range.");

	VW_ASSERT((src.GetSampleByteSize() >= 3)  &&
			(srcBGROffset <= src.GetSampleByteSize() - 3),
			"Source-BGR-offset out of range.");

	ProcessElements1D(
			sample_processors::CopyConverter_B8G8R8_To_L8(), dst.GetNumSamples(),
			dst.GetSamplesB() + dstLOffset, dst.GetSampleByteSize(),
			src.GetSamplesB() + srcBGROffset, src.GetSampleByteSize());
}


//====================================================================
void __fastcall CopySamples_1ByteLTo3BytesBGR(SignalBase& dst, uint dstBGROffset,
									const SignalBase& src, uint srcLOffset)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");
	VW_ASSERT(srcLOffset < src.GetSampleByteSize(), "Source-L-offset out of range.");

	VW_ASSERT((dst.GetSampleByteSize() >= 3)  &&
			(dstBGROffset <= dst.GetSampleByteSize() - 3),
			"Destination-BGR-offset out of range.");

	ProcessElements1D(
			sample_processors::CopyConverter_L8_To_B8G8R8(), dst.GetNumSamples(),
			dst.GetSamplesB() + dstBGROffset, dst.GetSampleByteSize(),
			src.GetSamplesB() + srcLOffset, src.GetSampleByteSize());
}


//====================================================================
void __fastcall CopySamples_UInt8888_To_UShort4444(SignalBase& dst, const SignalBase& src)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");
	VW_ASSERT(dst.GetSampleByteSize() == 2, "Invalid destination-sample-size.");
	VW_ASSERT(src.GetSampleByteSize() == 4, "Invalid source-sample-size.");

	ProcessElements1D(
		sample_processors::CopyConverter_UInt8888_To_UShort4444(), dst.GetNumSamples(),
		dst.GetSamplesB(), dst.GetSampleByteSize(),
		src.GetSamplesB(), src.GetSampleByteSize());
}


//====================================================================
void __fastcall CopySamples_UShort4444_To_UInt8888(SignalBase& dst, const SignalBase& src)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");
	VW_ASSERT(dst.GetSampleByteSize() == 4, "Invalid destination-sample-size.");
	VW_ASSERT(src.GetSampleByteSize() == 2, "Invalid source-sample-size.");

	ProcessElements1D(
		sample_processors::CopyConverter_UShort4444_To_UInt8888(), dst.GetNumSamples(),
		dst.GetSamplesB(), dst.GetSampleByteSize(),
		src.GetSamplesB(), src.GetSampleByteSize());
}


//====================================================================
void __fastcall CopySamples_UInt8888_To_UShort1555(SignalBase& dst, const SignalBase& src)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");
	VW_ASSERT(dst.GetSampleByteSize() == 2, "Invalid destination-sample-size.");
	VW_ASSERT(src.GetSampleByteSize() == 4, "Invalid source-sample-size.");

	ProcessElements1D(
		sample_processors::CopyConverter_UInt8888_To_UShort1555(), dst.GetNumSamples(),
		dst.GetSamplesB(), dst.GetSampleByteSize(),
		src.GetSamplesB(), src.GetSampleByteSize());
}


//====================================================================
void __fastcall CopySamples_UShort1555_To_UInt8888(SignalBase& dst, const SignalBase& src)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");
	VW_ASSERT(dst.GetSampleByteSize() == 4, "Invalid destination-sample-size.");
	VW_ASSERT(src.GetSampleByteSize() == 2, "Invalid source-sample-size.");

	ProcessElements1D(
		sample_processors::CopyConverter_UShort1555_To_UInt8888(), dst.GetNumSamples(),
		dst.GetSamplesB(), dst.GetSampleByteSize(),
		src.GetSamplesB(), src.GetSampleByteSize());
}


//====================================================================
void __fastcall CopySamples_FP32Norm_To_UInt8(SignalBase& dst, const SignalBase& src)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");
	VW_ASSERT(dst.GetSampleByteSize() == 1, "Invalid destination-sample-size.");
	VW_ASSERT(src.GetSampleByteSize() == 4, "Invalid source-sample-size.");

	ProcessElements1D(
		sample_processors::CopyConverter_FP32Norm_To_UInt8(), dst.GetNumSamples(),
		dst.GetSamplesB(), dst.GetSampleByteSize(),
		src.GetSamplesB(), src.GetSampleByteSize());
}


//====================================================================
void __fastcall CopySamples_UInt8_To_FP32Norm(SignalBase& dst, const SignalBase& src)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");
	VW_ASSERT(dst.GetSampleByteSize() == 4, "Invalid destination-sample-size.");
	VW_ASSERT(src.GetSampleByteSize() == 1, "Invalid source-sample-size.");

	ProcessElements1D(
		sample_processors::CopyConverter_UInt8_To_FP32Norm(), dst.GetNumSamples(),
		dst.GetSamplesB(), dst.GetSampleByteSize(),
		src.GetSamplesB(), src.GetSampleByteSize());
}


//====================================================================
void __fastcall CopySamples_NBytes(SignalBase& dst, uint dstByteOffset,
									const SignalBase& src, uint srcByteOffset,
									uint numBytesPerSample)
{
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");
	VW_ASSERT(numBytesPerSample <= dst.GetSampleByteSize(), "Num-bytes-per-sample out of range.");
	VW_ASSERT(dstByteOffset <= dst.GetSampleByteSize() - numBytesPerSample, "Destination-byte-offset out of range.");
	VW_ASSERT(numBytesPerSample <= src.GetSampleByteSize(), "Num-bytes-per-sample out of range.");
	VW_ASSERT(srcByteOffset <= src.GetSampleByteSize() - numBytesPerSample, "Source-byte-offset out of range.");

	if((numBytesPerSample == dst.GetSampleByteSize())  &&
		(numBytesPerSample == src.GetSampleByteSize()))
	{
		// use flat-copy if possible
		::CopyMemory(dst.GetSamplesB(), src.GetSamplesB(), src.GetTotalByteSize());
	}
	else if(numBytesPerSample == 1)
	{
		// 1-byte optimized copy-processor
		ProcessElements1D(
				element_processors::Copy_1<ubyte>(), dst.GetNumSamples(),
				dst.GetSamplesB() + dstByteOffset, dst.GetSampleByteSize(),
				src.GetSamplesB() + srcByteOffset, src.GetSampleByteSize());
	}
	else
	{
		// general copy-processor
		ProcessElements1D(
				element_processors::Copy_N<ubyte>(numBytesPerSample),
				dst.GetNumSamples(),
				dst.GetSamplesB() + dstByteOffset, dst.GetSampleByteSize(),
				src.GetSamplesB() + srcByteOffset, src.GetSampleByteSize());
	}
}


//====================================================================
VW_IMPLEMENT_SINGLETON( SignalOperationsManager );


//====================================================================
SignalOperationsManager::SignalOperationsManager()
{
	VW_IMPLEMENT_SINGLETON_CTOR( SignalOperationsManager );
	RegisterDefaultTransferHandlers();
	RegisterDefaultConvertHandlers();
}


//====================================================================
SignalOperationsManager::~SignalOperationsManager()
{
	UnregisterAllConvertHandlers();
	UnregisterAllTransferHandlers();
	VW_IMPLEMENT_SINGLETON_DTOR( SignalOperationsManager );
}


//====================================================================
void  SignalOperationsManager::RegisterTransferHandler(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat,
												SignalOperationsManager::TransferSamplesHandler& tsh)
{
	// store handler address only
	HandlerKey key(dstSampleFormat, srcSampleFormat);
	transferHandlerMap[key] = &tsh;
}


//====================================================================
void  SignalOperationsManager::RegisterConvertHandler(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat,
												SignalOperationsManager::ConvertSamplesHandler& csh)
{
	// store handler address only
	HandlerKey key(dstSampleFormat, srcSampleFormat);
	convertHandlerMap[key] = &csh;
}


//====================================================================
void  SignalOperationsManager::RegisterDefaultTransferHandlers()
{
	//------------------
	// 8-bit <===> 8-bit
	RegisterTransferHandler(SampleFormat_L8, SampleFormat_L8, tsh_CopySamples);
	RegisterTransferHandler(SampleFormat_L8, SampleFormat_A8, tsh_CopySamples);

	RegisterTransferHandler(SampleFormat_A8, SampleFormat_L8, tsh_CopySamples);
	RegisterTransferHandler(SampleFormat_A8, SampleFormat_A8, tsh_CopySamples);

	//------------------
	// 8-bit <=== 16-bit
	RegisterTransferHandler(SampleFormat_L8, SampleFormat_A8L8, tsh_CopySamples_NBytes_d0_s0_n1);

	RegisterTransferHandler(SampleFormat_A8, SampleFormat_A8L8, tsh_CopySamples_NBytes_d0_s1_n1);

	//------------------
	// 16-bit <=== 8-bit
	RegisterTransferHandler(SampleFormat_A8L8, SampleFormat_L8, tsh_CopySamples_NBytes_d0_s0_n1);
	RegisterTransferHandler(SampleFormat_A8L8, SampleFormat_A8, tsh_CopySamples_NBytes_d1_s0_n1);

	//--------------------
	// 16-bit <===> 16-bit
	RegisterTransferHandler(SampleFormat_A8L8, SampleFormat_A8L8, tsh_CopySamples);

	RegisterTransferHandler(SampleFormat_A4R4G4B4, SampleFormat_A4R4G4B4, tsh_CopySamples);

	//------------------
	// 8-bit <=== 32-bit
	RegisterTransferHandler(SampleFormat_L8, SampleFormat_X8R8G8B8, tsh_CopySamples_3BytesBGRTo1ByteL_d0_s0);
	RegisterTransferHandler(SampleFormat_L8, SampleFormat_A8R8G8B8, tsh_CopySamples_3BytesBGRTo1ByteL_d0_s0);

	RegisterTransferHandler(SampleFormat_A8, SampleFormat_X8R8G8B8, tsh_CopySamples_3BytesBGRTo1ByteL_d0_s0);
	RegisterTransferHandler(SampleFormat_A8, SampleFormat_A8R8G8B8, tsh_CopySamples_NBytes_d0_s3_n1);

	//------------------
	// 32-bit <=== 8-bit
	RegisterTransferHandler(SampleFormat_X8R8G8B8, SampleFormat_L8, tsh_CopySamples_1ByteLTo3BytesBGR_d0_s0);
	RegisterTransferHandler(SampleFormat_X8R8G8B8, SampleFormat_A8, tsh_CopySamples_1ByteLTo3BytesBGR_d0_s0);

	RegisterTransferHandler(SampleFormat_A8R8G8B8, SampleFormat_L8, tsh_CopySamples_1ByteLTo3BytesBGR_d0_s0);
	RegisterTransferHandler(SampleFormat_A8R8G8B8, SampleFormat_A8, tsh_CopySamples_NBytes_d3_s0_n1);

	//-------------------
	// 16-bit <=== 32-bit
	RegisterTransferHandler(SampleFormat_A8L8, SampleFormat_X8R8G8B8, tsh_CopySamples_3BytesBGRTo1ByteL_d0_s0);
	RegisterTransferHandler(SampleFormat_A8L8, SampleFormat_A8R8G8B8, tsh_CopySamples_3BytesBGRTo1ByteL1ByteClone_d0_s0_d1_s3);

	RegisterTransferHandler(SampleFormat_A4R4G4B4, SampleFormat_X8R8G8B8, tsh_CopySamples_UInt8888_To_UShort4444);
	RegisterTransferHandler(SampleFormat_A4R4G4B4, SampleFormat_A8R8G8B8, tsh_CopySamples_UInt8888_To_UShort4444);

	RegisterTransferHandler(SampleFormat_A1R5G5B5, SampleFormat_X8R8G8B8, tsh_CopySamples_UInt8888_To_UShort1555);
	RegisterTransferHandler(SampleFormat_A1R5G5B5, SampleFormat_A8R8G8B8, tsh_CopySamples_UInt8888_To_UShort1555);

	//-------------------
	// 32-bit <=== 16-bit
	RegisterTransferHandler(SampleFormat_X8R8G8B8, SampleFormat_A8L8,     tsh_CopySamples_1ByteLTo3BytesBGR_d0_s0);
	RegisterTransferHandler(SampleFormat_X8R8G8B8, SampleFormat_A4R4G4B4, tsh_CopySamples_UShort4444_To_UInt8888);
	RegisterTransferHandler(SampleFormat_X8R8G8B8, SampleFormat_A1R5G5B5, tsh_CopySamples_UShort1555_To_UInt8888);

	RegisterTransferHandler(SampleFormat_A8R8G8B8, SampleFormat_A8L8,     tsh_CopySamples_1ByteLTo3BytesBGR1ByteClone_d0_s0_d3_s1);
	RegisterTransferHandler(SampleFormat_A8R8G8B8, SampleFormat_A4R4G4B4, tsh_CopySamples_UShort4444_To_UInt8888);
	RegisterTransferHandler(SampleFormat_A8R8G8B8, SampleFormat_A1R5G5B5, tsh_CopySamples_UShort1555_To_UInt8888);

	//--------------------
	// 32-bit <===> 32-bit
	RegisterTransferHandler(SampleFormat_X8R8G8B8, SampleFormat_X8R8G8B8, tsh_CopySamples);
	RegisterTransferHandler(SampleFormat_X8R8G8B8, SampleFormat_A8R8G8B8, tsh_CopySamples);

	RegisterTransferHandler(SampleFormat_A8R8G8B8, SampleFormat_X8R8G8B8, tsh_CopySamples);
	RegisterTransferHandler(SampleFormat_A8R8G8B8, SampleFormat_A8R8G8B8, tsh_CopySamples);

	//---------------------------------
	// 8-bit <=== 32-bit floating-point
	RegisterTransferHandler(SampleFormat_L8, SampleFormat_ZB32, tsh_CopySamples_FP32Norm_To_UInt8);
	RegisterTransferHandler(SampleFormat_L8, SampleFormat_WB32, tsh_CopySamples_FP32Norm_To_UInt8);

	//---------------------------------
	// 32-bit floating-point <=== 8-bit
	RegisterTransferHandler(SampleFormat_ZB32, SampleFormat_L8, tsh_CopySamples_UInt8_To_FP32Norm);
	RegisterTransferHandler(SampleFormat_WB32, SampleFormat_L8, tsh_CopySamples_UInt8_To_FP32Norm);
}


//====================================================================
void  SignalOperationsManager::RegisterDefaultConvertHandlers()
{
	//------------------
	// 8-bit <===> 8-bit
	RegisterConvertHandler(SampleFormat_L8, SampleFormat_A8, csh_Null);
	RegisterConvertHandler(SampleFormat_A8, SampleFormat_L8, csh_Null);

	//--------------------
	// 16-bit <===> 16-bit

	//--------------------
	// 32-bit <===> 32-bit
	RegisterConvertHandler(SampleFormat_X8R8G8B8, SampleFormat_A8R8G8B8, csh_Null);
	RegisterConvertHandler(SampleFormat_A8R8G8B8, SampleFormat_X8R8G8B8, csh_Null);
}


//====================================================================
void  SignalOperationsManager::UnregisterTransferHandler(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat)
{
	HandlerKey key(dstSampleFormat, srcSampleFormat);
	transferHandlerMap.erase(key);
}


//====================================================================
void  SignalOperationsManager::UnregisterConvertHandler(eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat)
{
	HandlerKey key(dstSampleFormat, srcSampleFormat);
	convertHandlerMap.erase(key);
}


//====================================================================
void  SignalOperationsManager::UnregisterAllTransferHandlers()
{
	transferHandlerMap.clear();
}


//====================================================================
void  SignalOperationsManager::UnregisterAllConvertHandlers()
{
	convertHandlerMap.clear();
}


//====================================================================
SignalOperationsManager::TransferSamplesHandler*  SignalOperationsManager::GetTransferHandler(
												eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat)
{
	HandlerKey key(dstSampleFormat, srcSampleFormat);
	TRANSFER_HANDLER_ITER it = transferHandlerMap.find(key);
	if(it != transferHandlerMap.end()) return (it->second);
	return 0;
}


//====================================================================
SignalOperationsManager::ConvertSamplesHandler*  SignalOperationsManager::GetConvertHandler(
												eSampleFormat dstSampleFormat, eSampleFormat srcSampleFormat)
{
	HandlerKey key(dstSampleFormat, srcSampleFormat);
	CONVERT_HANDLER_ITER it = convertHandlerMap.find(key);
	if(it != convertHandlerMap.end()) return (it->second);
	return 0;
}


//====================================================================
SignalBase* __fastcall ManufactureSignal(eSampleFormat sampleFormat, uint numSamples)
{
	switch(sample_formats::ExtractByteSize(sampleFormat))
	{
		case 1: return new Signal8(sampleFormat, numSamples);
		case 2: return new Signal16(sampleFormat, numSamples);
		case 3: return new Signal24(sampleFormat, numSamples);
		case 4: return new Signal32(sampleFormat, numSamples);
		case 6: return new Signal48(sampleFormat, numSamples);
		case 8: return new Signal64(sampleFormat, numSamples);
		case 12: return new Signal96(sampleFormat, numSamples);
		case 16: return new Signal128(sampleFormat, numSamples);
	}
	return 0;
}


//====================================================================
SignalBase* __fastcall CloneSignal(const SignalBase& src)
{
	SignalBase* dst = ManufactureSignal(src.GetSampleFormat(), src.GetNumSamples());
	if(! dst) return 0;
	CopySamples1D(*dst, src);
	return dst;
}


//====================================================================
void __fastcall CopySamples1D(SignalBase& dst, const SignalBase& src,
							uint count, uint dstOffset, uint srcOffset)
{
	if(! count) count = src.GetNumSamples();

	VW_ASSERT(dst.CheckSampleFormatMatch(src), "Sample-format mismatch.");
	VW_ASSERT(dst.GetNumSamples(), "Destination is empty.");
	VW_ASSERT(src.GetNumSamples(), "Source is empty.");
	VW_ASSERT(dstOffset + count <= dst.GetNumSamples(), "Destination-offset/count out of range.");
	VW_ASSERT(srcOffset + count <= src.GetNumSamples(), "Source-offset/count out of range.");

	::CopyMemory(dst.SampleB(dstOffset), src.SampleB(srcOffset), src.GetSampleByteSize() * count);
}


//====================================================================
bool __fastcall TransferSamples1D(SignalBase& dst, const SignalBase& src, bool bAutoResize)
{
	SignalOperationsManager::TransferSamplesHandler* tsh =
		SignalOperationsManager::Instance().GetTransferHandler(dst.GetSampleFormat(), src.GetSampleFormat());

	if(! tsh) return false; //not supported

	TransferSamples1D(dst, src, *tsh, bAutoResize);
	return true;
}


//====================================================================
void __fastcall TransferSamples1D(SignalBase& dst, const SignalBase& src,
								SignalOperationsManager::TransferSamplesHandler& tsh, bool bAutoResize)
{
	if(bAutoResize) {
		dst.ReserveSampleStorage(src.GetNumSamples());
	}
	VW_ASSERT(dst.CheckDimensionMatch(src), "Sample-count mismatch.");

	tsh.Handle(dst, src);
}


//====================================================================
bool __fastcall ConvertSamples(SignalBase& target, eSampleFormat newSampleFormat)
{
	if(target.GetSampleFormat() == newSampleFormat) return true;
	if(! target.IsValidSampleFormat(newSampleFormat)) return false;

	SignalOperationsManager::ConvertSamplesHandler* csh =
		SignalOperationsManager::Instance().GetConvertHandler(newSampleFormat, target.GetSampleFormat());

	if(! csh) return false; //not supported

	csh->Handle(target, newSampleFormat);
	target.SetSampleFormat(newSampleFormat);
	return true;
}


//====================================================================
void __fastcall Resample1D_Nearest(SignalBase& dst, const SignalBase& src)
{
	VW_ASSERT(dst.GetNumSamples(), "Destination is empty.");
	VW_ASSERT(src.GetNumSamples(), "Source is empty.");
	VW_ASSERT(dst.CheckSampleFormatMatch(src), "Sample-format mismatch.");

	switch(dst.GetSampleByteSize()) {
		case 1:
			Call_ResampleElements1D_Nearest_ubyte_ubyte(
				element_processors::Copy_1<ubyte>(), dst, src);
			break;

		case 2:
			Call_ResampleElements1D_Nearest_ubyte_ubyte(
				element_processors::Copy_1<ushort>(), dst, src);
			break;

		case 4:
			Call_ResampleElements1D_Nearest_ubyte_ubyte(
				element_processors::Copy_1<uint>(), dst, src);
			break;

		default:
			Call_ResampleElements1D_Nearest_ubyte_ubyte(
				element_processors::Copy_N<ubyte>(dst.GetSampleByteSize()), dst, src);
			break;
	}
}


//====================================================================
void __fastcall Resample1D_Linear_NFloats(SignalBase& dst, const SignalBase& src,
								uint componentOffset, uint numComponentsPerSample)
{
	uint numComponents = dst.GetSampleByteSize() / sizeof(float);
	uint byteOffset = componentOffset * sizeof(float);

	VW_ASSERT(dst.GetNumSamples(), "Destination is empty.");
	VW_ASSERT(src.GetNumSamples(), "Source is empty.");
	VW_ASSERT(dst.CheckSampleFormatMatch(src), "Sample-format mismatch.");
	VW_ASSERT(numComponentsPerSample <= numComponents, "Num-components-per-sample out of range.");
	VW_ASSERT(componentOffset <= numComponents - numComponentsPerSample, "Component-offset out of range.");

	ResampleElements1D_Linear<ubyte, ubyte>(
			sample_processors::LinearInterpolator1D_NFloats(numComponentsPerSample),
			dst.GetSamplesB() + byteOffset, dst.GetSampleByteSize(), dst.GetNumSamples(),
			src.GetSamplesB() + byteOffset, src.GetSampleByteSize(), src.GetNumSamples());
}


//====================================================================
void __fastcall Resample1D_Linear_NBytes(SignalBase& dst, const SignalBase& src,
										uint byteOffset, uint numBytesPerSample)
{
	VW_ASSERT(dst.GetNumSamples(), "Destination is empty.");
	VW_ASSERT(src.GetNumSamples(), "Source is empty.");
	VW_ASSERT(dst.CheckSampleFormatMatch(src), "Sample-format mismatch.");
	VW_ASSERT(numBytesPerSample <= dst.GetSampleByteSize(), "Num-bytes-per-sample out of range.");
	VW_ASSERT(byteOffset <= dst.GetSampleByteSize() - numBytesPerSample, "Byte-offset out of range.");

	ResampleElements1D_Linear<ubyte, ubyte>(
			sample_processors::LinearInterpolator1D_NBytes(numBytesPerSample),
			dst.GetSamplesB() + byteOffset, dst.GetSampleByteSize(), dst.GetNumSamples(),
			src.GetSamplesB() + byteOffset, src.GetSampleByteSize(), src.GetNumSamples());
}


//====================================================================
void __fastcall Resample1D_Linear_PackedSample_UShort4444(SignalBase& dst, const SignalBase& src)
{
	Call_ResampleElements1D_Linear_PackedSample_UShort(
		sample_processors::LinearInterpolator1D_PackedSample_UShort4444(), dst, src);
}


//====================================================================
void __fastcall Resample1D_Linear_PackedSample_UShort1555(SignalBase& dst, const SignalBase& src)
{
	Call_ResampleElements1D_Linear_PackedSample_UShort(
		sample_processors::LinearInterpolator1D_PackedSample_UShort1555(), dst, src);
}


//====================================================================
bool __fastcall Resample1D(SignalBase& dst, const SignalBase& src,
										eResampleFilter filterType)
{
	VW_ASSERT(dst.GetNumSamples(), "Destination is empty.");
	VW_ASSERT(src.GetNumSamples(), "Source is empty.");
	VW_ASSERT(dst.CheckSampleFormatMatch(src), "Sample-format mismatch.");

	// nearest-point resampling
	if(filterType == ResampleFilter_Nearest) {

		Resample1D_Nearest(dst, src);
		return true;
	}

	// linear-interpolation resampling
	if(filterType == ResampleFilter_Linear) {

		switch(dst.GetSampleFormat()) {
			case SampleFormat_L8:
			case SampleFormat_A8:
			case SampleFormat_A8L8:
			case SampleFormat_A8R8G8B8:
				Resample1D_Linear_NBytes(dst, src, 0, dst.GetSampleByteSize());
				return true;

			case SampleFormat_X8R8G8B8:
				Resample1D_Linear_NBytes(dst, src, 0, 3);
				return true;

			case SampleFormat_A4R4G4B4:
				Resample1D_Linear_PackedSample_UShort4444(dst, src);
				return true;

			case SampleFormat_A1R5G5B5:
				Resample1D_Linear_PackedSample_UShort1555(dst, src);
				return true;

			case SampleFormat_ZB32:
			case SampleFormat_WB32:
				Resample1D_Linear_NFloats(dst, src, 0, dst.GetSampleByteSize() / sizeof(float));
				return true;
		}
	}

	// unsupported filter-type / sample-type
	return false;
}


//====================================================================
void __fastcall FlipSamples(SignalBase& target)
{
	uint halfNumSamples = target.GetNumSamples() / 2;
	if(! halfNumSamples) return;

	ProcessElements1D(element_processors::Swap_N<ubyte>(target.GetSampleByteSize()),
						halfNumSamples,
						target.GetSamplesB(), target.GetSampleByteSize(),
						target.SampleB(target.GetNumSamples() - 1), - target.GetSampleByteSize());
}


//====================================================================
bool __fastcall Fill_Luminance(SignalBase& target, ubyte luminance)
{
	switch(target.GetSampleFormat()) {
		case SampleFormat_L8:
		case SampleFormat_A8L8:
			ProcessSamples_Fill1Byte(target, 0, luminance);
			return true;
	}
	return false;
}


//====================================================================
bool __fastcall Fill_ARGB(SignalBase& target, color4b argb)
{
	switch(target.GetSampleFormat()) {
		case SampleFormat_A4R4G4B4:
		case SampleFormat_A1R5G5B5:
		case SampleFormat_A8R8G8B8:
			Fill_RGB(target, argb);
			Fill_Alpha(target, argb.a);
			return true;
	}
	return false;
}


//====================================================================
bool __fastcall Fill_RGB(SignalBase& target, color4b rgb)
{
	switch(target.GetSampleFormat()) {
		case SampleFormat_A4R4G4B4:
		case SampleFormat_A1R5G5B5:
			Fill_Red(target, rgb.r);
			Fill_Green(target, rgb.g);
			Fill_Blue(target, rgb.b);
			return true;

		case SampleFormat_X8R8G8B8:
		case SampleFormat_A8R8G8B8:
			ProcessSamples_Fill3Bytes(target, 0, rgb.b, rgb.g, rgb.r);
			return true;
	}
	return false;
}


//====================================================================
bool __fastcall Fill_Alpha(SignalBase& target, ubyte alpha)
{
	switch(target.GetSampleFormat()) {
		case SampleFormat_A8:
			ProcessSamples_Fill1Byte(target, 0, alpha);
			return true;

		case SampleFormat_A8L8:
			ProcessSamples_Fill1Byte(target, 1, alpha);
			return true;

		case SampleFormat_A4R4G4B4:
			ProcessSamples_Fill1Nibble_MapValueRange(target, 1, true, alpha);
			return true;

		case SampleFormat_A1R5G5B5:
			ProcessSamples_FillBits_UShort(target, (alpha >> 7), 0x0001, 15);
			return true;

		case SampleFormat_A8R8G8B8:
			ProcessSamples_Fill1Byte(target, 3, alpha);
			return true;
	}
	return false;
}


//====================================================================
bool __fastcall Fill_Red(SignalBase& target, ubyte red)
{
	switch(target.GetSampleFormat()) {
		case SampleFormat_A4R4G4B4:
			ProcessSamples_Fill1Nibble_MapValueRange(target, 1, false, red);
			return true;

		case SampleFormat_A1R5G5B5:
			ProcessSamples_FillBits_UShort(target, (red >> 3), 0x001F, 10);
			return true;

		case SampleFormat_X8R8G8B8:
		case SampleFormat_A8R8G8B8:
			ProcessSamples_Fill1Byte(target, 2, red);
			return true;
	}
	return false;
}


//====================================================================
bool __fastcall Fill_Green(SignalBase& target, ubyte green)
{
	switch(target.GetSampleFormat()) {
		case SampleFormat_A4R4G4B4:
			ProcessSamples_Fill1Nibble_MapValueRange(target, 0, true, green);
			return true;

		case SampleFormat_A1R5G5B5:
			ProcessSamples_FillBits_UShort(target, (green >> 3), 0x001F, 5);
			return true;

		case SampleFormat_X8R8G8B8:
		case SampleFormat_A8R8G8B8:
			ProcessSamples_Fill1Byte(target, 1, green);
			return true;
	}
	return false;
}


//====================================================================
bool __fastcall Fill_Blue(SignalBase& target, ubyte blue)
{
	switch(target.GetSampleFormat()) {
		case SampleFormat_A4R4G4B4:
			ProcessSamples_Fill1Nibble_MapValueRange(target, 0, false, blue);
			return true;

		case SampleFormat_A1R5G5B5:
			ProcessSamples_FillBits_UShort(target, (blue >> 3), 0x001F, 0);
			return true;

		case SampleFormat_X8R8G8B8:
		case SampleFormat_A8R8G8B8:
			ProcessSamples_Fill1Byte(target, 0, blue);
			return true;
	}
	return false;
}


//====================================================================
bool __fastcall GradientFill(SignalBase& target,
							const color4f& rgbaStart, const color4f& rgbaEnd,
							uint startIndex, uint endIndex)
{
	VW_ASSERT(target.GetNumSamples(), "Signal is empty.");
	VW_ASSERT(startIndex < target.GetNumSamples(), "Start-index out of range.");
	VW_ASSERT(endIndex < target.GetNumSamples(), "End-index out of range.");
	VW_ASSERT(startIndex <= endIndex, "End-index preceds start-index.");

	uint  numSamples = ClampedLow<uint>((endIndex - startIndex + 1), 1);

	color4f  rgbaInc = (rgbaEnd - rgbaStart) / numSamples;
	color4f  rgbaCur = rgbaStart;

	Signal32  fillTarget(SampleFormat_A8R8G8B8, numSamples);

	for(uint i = 0; i < numSamples; i++) {
		fillTarget.Sample(i)->i0 = color4b(rgbaCur);
		rgbaCur += rgbaInc;
	}

	AutoPtr<SignalBase>  apCopySource(&fillTarget, false);
	if(target.GetSampleFormat() != apCopySource->GetSampleFormat()) {
		apCopySource.Assign(ManufactureSignal(target.GetSampleFormat()));
		if(! TransferSamples1D(*apCopySource, fillTarget, true)) return false;
	}
	CopySamples1D(target, *apCopySource, numSamples, startIndex, 0);
	return true;
}


//====================================================================
void __fastcall HighPassFilter(SignalBase& target, uint byteOffset,
							ubyte thresholdValue, ubyte newThresholdValue, ubyte lowValue)
{
	VW_ASSERT(byteOffset < target.GetSampleByteSize(), "Byte-offset out of range.");

	ProcessElements1D(
			element_processors::Process_HighPassFilter<ubyte>(
								thresholdValue, newThresholdValue, lowValue),
			target.GetNumSamples(), target.GetSamplesB() + byteOffset,
			target.GetSampleByteSize());
}


//====================================================================
void __fastcall LowPassFilter(SignalBase& target, uint byteOffset,
							ubyte thresholdValue, ubyte newThresholdValue, ubyte highValue)
{
	VW_ASSERT(byteOffset < target.GetSampleByteSize(), "Byte-offset out of range.");

	ProcessElements1D(
			element_processors::Process_LowPassFilter<ubyte>(
								thresholdValue, newThresholdValue, highValue),
			target.GetNumSamples(), target.GetSamplesB() + byteOffset,
			target.GetSampleByteSize());
}


//====================================================================
void __fastcall ThresholdSegmentation(SignalBase& target, uint byteOffset,
							ubyte thresholdValue,
							ubyte lowValue, ubyte midValue, ubyte highValue)
{
	VW_ASSERT(byteOffset < target.GetSampleByteSize(), "Byte-offset out of range.");

	ProcessElements1D(
			element_processors::Process_ThresholdSegmentation<ubyte>(
								thresholdValue, lowValue, midValue, highValue),
			target.GetNumSamples(), target.GetSamplesB() + byteOffset,
			target.GetSampleByteSize());
}


//====================================================================
void __fastcall ThresholdSegmentationNBytes(SignalBase& target,
							uint byteOffset, uint numBytesPerSample,
							ubyte thresholdValue,
							ubyte lowValue, ubyte midValue, ubyte highValue)
{
	VW_ASSERT(byteOffset < target.GetSampleByteSize(), "Byte-offset out of range.");
	VW_ASSERT(numBytesPerSample <= target.GetSampleByteSize() - byteOffset, "Num-bytes-per-sample out of range.");

	for(uint i = byteOffset; i < byteOffset + numBytesPerSample; i++) {
		ThresholdSegmentation(target, i, thresholdValue, lowValue, midValue, highValue);
	}
}


//********************************************************************
}//namespace vertexware


