

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "DataObjects\VolumeBase.h"
#include "DataObjects\Signal.h"


//********************************************************************
namespace vertexware {


//====================================================================
// Typed volume.
//====================================================================
template<typename TSample>
class Volume : public VolumeBase, public Signal<TSample>
{
public:
	typedef  TSample  SampleType;


public:
	// constructor
	Volume(eSampleFormat sampleFormat =
			eSampleFormat(sample_formats::UnknownSampleFormat(sizeof(TSample))),
			uint numLayers = 0, uint numRows = 0, uint numColumns = 0)
		:
		SignalBase(sizeof(TSample)),
		VolumeBase(sizeof(TSample))
	{
		SetSampleFormat(sampleFormat);
		if(numLayers  &&  numRows  &&  numColumns) {
			ReserveSampleStorage(numLayers, numRows, numColumns);
		}
	}

	// storage configuration (avoid inheritance via dominance)
	virtual  void  ReserveSampleStorage(uint numLayers, uint numRows, uint numColumns) {
		VolumeBase::ReserveSampleStorage(numLayers, numRows, numColumns);
	}
	virtual  void  ReserveSampleStorage(uint numSamples) {
		VolumeBase::ReserveSampleStorage(numSamples);
	}
	virtual  void  ReleaseSampleStorage() {
		VolumeBase::ReleaseSampleStorage();
	}

	// storage access
	__forceinline  TSample*  Sample(uint layer, uint row, uint column) {
		CheckAccess3D(layer, row, column);
		return (GetSamples() +
				(layer * GetNumSamplesPerLayer() + row * GetNumColumns() + column));
	}
	__forceinline  const TSample*  Sample(uint layer, uint row, uint column) const {
		return const_cast<Volume<TSample >* >(this)->Sample(layer, row, column);
	}
};


//====================================================================
// Standard volume types.
//====================================================================
typedef  Volume<Sample8>    Volume8;
typedef  Volume<Sample16>   Volume16;
typedef  Volume<Sample24>   Volume24;
typedef  Volume<Sample32>   Volume32;
typedef  Volume<Sample48>   Volume48;
typedef  Volume<Sample64>   Volume64;
typedef  Volume<Sample96>   Volume96;
typedef  Volume<Sample128>  Volume128;


//********************************************************************
}//namespace vertexware


