

//--------------------------------------------------------------------
#include "DataObjects\VolumeBase.h"


//********************************************************************
namespace vertexware {


//====================================================================
VolumeBase::VolumeBase(uint sampleByteSize)
	:
	SignalBase(sampleByteSize)
{
	numLayers = 0;
	numRows = 0;
	numColumns = 0;
	numSamplesPerLayer = 0;
	rowByteSize = 0;
	layerByteSize = 0;
}


//====================================================================
void  VolumeBase::SetupInternalVolumeProperties(uint _numLayers, uint _numRows, uint _numColumns)
{
	// if any dimension is 0, force all to 0
	if(! _numLayers  ||  ! _numRows  ||  ! _numColumns) {
		_numLayers = 0;
		_numRows = 0;
		_numColumns = 0;
	}

	numLayers = _numLayers;
	numRows = _numRows;
	numColumns = _numColumns;
	numSamplesPerLayer = numRows * numColumns;
	rowByteSize = numColumns * GetSampleByteSize();
	layerByteSize = numRows * rowByteSize;
}


//====================================================================
void  VolumeBase::ReserveSampleStorage(uint _numLayers, uint _numRows, uint _numColumns)
{
	SetupInternalVolumeProperties(_numLayers, _numRows, _numColumns);
	SignalBase::ReserveSampleStorage(numLayers * numRows * numColumns);
}


//====================================================================
void  VolumeBase::ReserveSampleStorage(uint numSamples)
{
	VolumeBase::ReserveSampleStorage(1, 1, numSamples);
}


//====================================================================
void  VolumeBase::ReleaseSampleStorage()
{
	SetupInternalVolumeProperties(0, 0, 0);
	SignalBase::ReleaseSampleStorage();
}


//====================================================================
bool  VolumeBase::CheckDimensionMatch(const VolumeBase& vol) const
{
	return
	(numLayers == vol.GetNumLayers())  &&
	(numRows == vol.GetNumRows())  &&
	(numColumns == vol.GetNumColumns());
}


//====================================================================
VolumeBaseFromExternalStorageAdapter::VolumeBaseFromExternalStorageAdapter(
													eSampleFormat sampleFormat)
	:
	SignalBase(sample_formats::ExtractByteSize(sampleFormat)),
	VolumeBase(sample_formats::ExtractByteSize(sampleFormat))
{
	SetSampleFormat(sampleFormat);
}


//====================================================================
void  VolumeBaseFromExternalStorageAdapter::SetExternalStorage(
						ubyte* pStorage, uint numLayers, uint numRows, uint numColumns)
{
	if(pStorage) {
		VW_ASSERT(numLayers && numRows && numColumns,
				"Invalid argument combination passed to VolumeBase adapter.");

		SetupInternalVolumeProperties(numLayers, numRows, numColumns);
		AllocateStorage(numLayers * numRows * numColumns, pStorage);
	}
	else {
		SetupInternalVolumeProperties(0, 0, 0);
		DeallocateStorage();
	}
}


//====================================================================
void  VolumeBaseFromExternalStorageAdapter::ReserveSampleStorage(uint, uint, uint)
{
	VW_THROW(InvalidUsageException, "Cannot manage external storage through VolumeBase adapter.");
}


//====================================================================
void  VolumeBaseFromExternalStorageAdapter::ReserveSampleStorage(uint)
{
	VW_THROW(InvalidUsageException, "Cannot manage external storage through VolumeBase adapter.");
}


//********************************************************************
}//namespace vertexware


