

//--------------------------------------------------------------------
#include "Framework\DirectInputProxy.h"
#include "BaseTools\WinUtils.h"
#include "BaseTools\Stream.h"


//********************************************************************
namespace vertexware {


//====================================================================
DirectInputConfig::DirectInputConfig()
{
	enableDirectInput = true;
	useLastVersion = true;
	directInputVersion = 0;
	enableFullscreenExclusiveMode = true;
	useAtDeviceInit = true;
}


//====================================================================
bool  DirectInputConfig::operator==(const DirectInputConfig& rhs) const
{
	return
	(enableDirectInput == rhs.enableDirectInput) &&
	(useLastVersion == rhs.useLastVersion) &&
	(directInputVersion == rhs.directInputVersion) &&
	(enableFullscreenExclusiveMode == rhs.enableFullscreenExclusiveMode) &&
	(useAtDeviceInit == rhs.useAtDeviceInit);
}


//====================================================================
bool  DirectInputConfig::IsLogicallyEqual(const DirectInputConfig& rhs) const
{
	if(enableDirectInput != rhs.enableDirectInput) return false;

	if(enableDirectInput) {
		if(useLastVersion != rhs.useLastVersion) return false;

		if(! useLastVersion) {
			if(directInputVersion != rhs.directInputVersion) return false;
		}
		return
		(enableFullscreenExclusiveMode == rhs.enableFullscreenExclusiveMode) &&
		(useAtDeviceInit == rhs.useAtDeviceInit);
	}
	return true;
}


//====================================================================
void  DirectInputConfig::Dump(std::ostream& os, uint tabLevel) const
{
	TabEffector te(tabLevel);
	os << te << "EnableDirectInput............: " << ToString(enableDirectInput) << nl;
	os << te << "UseLastVersion...............: " << ToString(useLastVersion) << nl;
	os << te << "DirectInputVersion...........: " << ToString(directInputVersion) << nl;
	os << te << "EnableFullscreenExclusiveMode: " << ToString(enableFullscreenExclusiveMode) << nl;
	os << te << "UseAtDeviceInit..............: " << ToString(useAtDeviceInit) << nl;
}


//====================================================================
DirectInputDeviceInstanceInfo::DirectInputDeviceInstanceInfo(
			const win_utils::Guid& _instanceGuid, const std::string& _instanceName,
			const win_utils::Guid& _productGuid, const std::string& _productName)
	:
	instanceGuid(_instanceGuid),
	productGuid(_productGuid),
	instanceName(_instanceName),
	productName(_productName)
{
}


//====================================================================
bool DirectInputDeviceInstanceInfo::operator==(const DirectInputDeviceInstanceInfo& rhs) const
{
	return
	(instanceGuid == rhs.instanceGuid) &&
	(productGuid == rhs.productGuid) &&
	(instanceName == rhs.instanceName) &&
	(productName == rhs.productName);
}


//====================================================================
void  DirectInputDeviceInstanceInfo::DumpInfo(std::ostream& os, uint tabLevel) const
{
	TabEffector te(tabLevel);

	os << te << "InstanceGUID: " << instanceGuid << nl;
	os << te << "InstanceName: " << instanceName << nl;
	os << te << "ProductGUID.: " << productGuid << nl;
	os << te << "ProductName.: " << productName << nl;
}


//====================================================================
DirectInputProxy::DirectInputProxy()
{
	bAllowFullscreenExclusiveMode = false;
}


//====================================================================
DirectInputProxy::~DirectInputProxy()
{
}


//====================================================================
DirectInputProxyPluginDescriptor::DirectInputProxyPluginDescriptor(const PluginInfo& pi)
	:
	PluginDescriptor(PluginInfo(pi, PluginType_DirectInputProxy))
{
}


//********************************************************************
}//namespace vertexware


