

//--------------------------------------------------------------------
#include "Framework\DirectInputProxy8.h"
#include "BaseTools\WinUtils.h"
#include "BaseTools\Debug.h"
#include "BaseTools\Stream.h"
#include <sstream>
#include <iomanip>
#include <dxerr8.h>


//********************************************************************
namespace vertexware {


//====================================================================
bool  DirectInputDeviceInstanceInfo8::operator==(const DirectInputDeviceInstanceInfo8& rhs) const
{
	return
	(dwSize == rhs.dwSize) &&
	(win_utils::Guid(guidInstance) == win_utils::Guid(rhs.guidInstance)) &&
	(win_utils::Guid(guidProduct) == win_utils::Guid(rhs.guidProduct)) &&
	(dwDevType == rhs.dwDevType) &&
	(std::string(tszInstanceName) == std::string(rhs.tszInstanceName)) &&
	(std::string(tszProductName) == std::string(rhs.tszProductName)) &&
	(win_utils::Guid(guidFFDriver) == win_utils::Guid(rhs.guidFFDriver)) &&
	(wUsagePage == rhs.wUsagePage) &&
    (wUsage == rhs.wUsage);
}


//====================================================================
void  DirectInputDeviceInstanceInfo8::CopyInfo(DirectInputDeviceInstanceInfo& didii) const
{
	didii.instanceGuid = guidInstance;
	didii.productGuid  = guidProduct;
	didii.instanceName = std::string(tszInstanceName);
	didii.productName  = std::string(tszProductName);
}


//====================================================================
void  DirectInputDeviceInstanceInfo8::DumpInfo(std::ostream& os, uint tabLevel) const
{
	TabEffector te(tabLevel);

	os << te << "InstanceGUID.: " << guidInstance << nl;
	os << te << "InstanceName.: " << tszInstanceName << nl;
	os << te << "ProductGUID..: " << guidProduct << nl;
	os << te << "ProductName..: " << tszProductName << nl;

	uint  iDeviceType    = GET_DIDEVICE_TYPE(dwDevType);
	uint  iDeviceSubType = GET_DIDEVICE_SUBTYPE(dwDevType);

	std::string  sDeviceType, sDeviceSubType;
	FormatDeviceType(iDeviceType, iDeviceSubType, sDeviceType, sDeviceSubType);

	const char* sep = " | ";
	os << te << "DeviceType...: " << std::setw(2) << iDeviceType << sep << sDeviceType << nl;
	os << te << "DeviceSubType: " << std::setw(2) << iDeviceSubType << sep << sDeviceSubType << nl;
}


//====================================================================
void  DirectInputDeviceInstanceInfo8::FormatDeviceType(uint iDeviceType, uint iDeviceSubType,
										std::string& sDeviceType, std::string& sDeviceSubType)
{
	sDeviceType = "unknown";
	sDeviceSubType = "unknown";

	switch(iDeviceType) {
		case DI8DEVTYPE_1STPERSON:
			sDeviceType = "first-person-game controller";
			switch(iDeviceSubType) {
				case DI8DEVTYPE1STPERSON_LIMITED:
					sDeviceSubType = "limited";
					break;
				case DI8DEVTYPE1STPERSON_SHOOTER:
					sDeviceSubType = "shooter";
					break;
				case DI8DEVTYPE1STPERSON_SIXDOF:
					sDeviceSubType = "6 degrees of freedom";
					break;
			}
			return;

		case DI8DEVTYPE_DEVICE:
			sDeviceType = "unclassified";
			return;

		case DI8DEVTYPE_DEVICECTRL:
			sDeviceType = "device controller";
			switch(iDeviceSubType) {
				case DI8DEVTYPEDEVICECTRL_COMMSSELECTION:
					sDeviceSubType = "communications selector";
					break;
				case DI8DEVTYPEDEVICECTRL_COMMSSELECTION_HARDWIRED:
					sDeviceSubType = "hardwired communications selector";
					break;
			}
			return;

		case DI8DEVTYPE_DRIVING:
			sDeviceType = "steering controller";
			switch(iDeviceSubType) {
				case DI8DEVTYPEDRIVING_COMBINEDPEDALS:
					sDeviceSubType = "combined pedals";
					break;
				case DI8DEVTYPEDRIVING_DUALPEDALS:
					sDeviceSubType = "dual pedals";
					break;
				case DI8DEVTYPEDRIVING_HANDHELD:
					sDeviceSubType = "hand held";
					break;
				case DI8DEVTYPEDRIVING_LIMITED:
					sDeviceSubType = "limited";
					break;
				case DI8DEVTYPEDRIVING_THREEPEDALS:
					sDeviceSubType = "three pedals";
					break;
			}
			return;

		case DI8DEVTYPE_FLIGHT:
			sDeviceType = "flight controller";
			switch(iDeviceSubType) {
				case DI8DEVTYPEFLIGHT_LIMITED:
					sDeviceSubType = "limited";
					break;
				case DI8DEVTYPEFLIGHT_RC:
					sDeviceSubType = "remote control";
					break;
				case DI8DEVTYPEFLIGHT_STICK:
					sDeviceSubType = "joystick";
					break;
				case DI8DEVTYPEFLIGHT_YOKE:
					sDeviceSubType = "yoke";
					break;
			}
			return;

		case DI8DEVTYPE_GAMEPAD:
			sDeviceType = "gamepad";
			switch(iDeviceSubType) {
				case DI8DEVTYPEGAMEPAD_LIMITED:
					sDeviceSubType = "limited";
					break;
				case DI8DEVTYPEGAMEPAD_STANDARD:
					sDeviceSubType = "standard";
					break;
				case DI8DEVTYPEGAMEPAD_TILT:
					sDeviceSubType = "tilt";
					break;
			}
			return;

		case DI8DEVTYPE_JOYSTICK:
			sDeviceType = "joystick";
			switch(iDeviceSubType) {
				case DI8DEVTYPEJOYSTICK_LIMITED:
					sDeviceSubType = "limited";
					break;
				case DI8DEVTYPEJOYSTICK_STANDARD:
					sDeviceSubType = "standard";
					break;
			}
			return;

		case DI8DEVTYPE_KEYBOARD:
			sDeviceType = "keyboard";
			switch(iDeviceSubType) {
				case DI8DEVTYPEKEYBOARD_PCXT:
					sDeviceSubType = "IBM PC/XT 83-key keyboard";
					break;
				case DI8DEVTYPEKEYBOARD_OLIVETTI:
					sDeviceSubType = "Olivetti 102-key keyboard";
					break;
				case DI8DEVTYPEKEYBOARD_PCAT:
					sDeviceSubType = "IBM PC/AT 84-key keyboard";
					break;
				case DI8DEVTYPEKEYBOARD_PCENH:
					sDeviceSubType = "IBM PC Enhanced 101/102-key or Microsoft Natural keyboard";
					break;
				case DI8DEVTYPEKEYBOARD_NOKIA1050:
					sDeviceSubType = "Nokia 1050 keyboard";
					break;
				case DI8DEVTYPEKEYBOARD_NOKIA9140:
					sDeviceSubType = "Nokia 9140 keyboard";
					break;
				case DI8DEVTYPEKEYBOARD_NEC98:
					sDeviceSubType = "Japanese NEC PC98 keyboard";
					break;
				case DI8DEVTYPEKEYBOARD_NEC98LAPTOP:
					sDeviceSubType = "Japanese NEC PC98 laptop keyboard";
					break;
				case DI8DEVTYPEKEYBOARD_NEC98106:
					sDeviceSubType = "Japanese NEC PC98 106-key keyboard";
					break;
				case DI8DEVTYPEKEYBOARD_JAPAN106:
					sDeviceSubType = "Japanese 106-key keyboard";
					break;
				case DI8DEVTYPEKEYBOARD_JAPANAX:
					sDeviceSubType = "Japanese AX keyboard";
					break;
				case DI8DEVTYPEKEYBOARD_J3100:
					sDeviceSubType = "Japanese J3100 keyboard";
					break;
			}
			return;

		case DI8DEVTYPE_MOUSE:
			sDeviceType = "mouse";
			switch(iDeviceSubType) {
				case DI8DEVTYPEMOUSE_ABSOLUTE:
					sDeviceSubType = "absolute axis data";
					break;
				case DI8DEVTYPEMOUSE_FINGERSTICK:
					sDeviceSubType = "fingerstick";
					break;
				case DI8DEVTYPEMOUSE_TOUCHPAD:
					sDeviceSubType = "touchpad";
					break;
				case DI8DEVTYPEMOUSE_TRACKBALL:
					sDeviceSubType = "trackball";
					break;
				case DI8DEVTYPEMOUSE_TRADITIONAL:
					sDeviceSubType = "traditional";
					break;
			}
			return;

		case DI8DEVTYPE_REMOTE: 
			sDeviceType = "remote controller";
			return;

		case DI8DEVTYPE_SCREENPOINTER:
			sDeviceType = "screen pointer";
			switch(iDeviceSubType) {
				case DI8DEVTYPESCREENPTR_LIGHTGUN:
					sDeviceSubType = "light gun";
					break;
				case DI8DEVTYPESCREENPTR_LIGHTPEN:
					sDeviceSubType = "light pen";
					break;
				case DI8DEVTYPESCREENPTR_TOUCH:
					sDeviceSubType = "touch screen";
					break;
			}
			return;

		case DI8DEVTYPE_SUPPLEMENTAL:
			sDeviceType = "supplemental controller";
			switch(iDeviceSubType) {
				case DI8DEVTYPESUPPLEMENTAL_2NDHANDCONTROLLER:
					sDeviceSubType = "secondary hand held controller";
					break;
				case DI8DEVTYPESUPPLEMENTAL_COMBINEDPEDALS:
					sDeviceSubType = "combined pedals";
					break;
				case DI8DEVTYPESUPPLEMENTAL_DUALPEDALS:
					sDeviceSubType = "dual pedals";
					break;
				case DI8DEVTYPESUPPLEMENTAL_HANDTRACKER:
					sDeviceSubType = "hand tracker";
					break;
				case DI8DEVTYPESUPPLEMENTAL_HEADTRACKER:
					sDeviceSubType = "head tracker";
					break;
				case DI8DEVTYPESUPPLEMENTAL_RUDDERPEDALS:
					sDeviceSubType = "rudder pedals";
					break;
				case DI8DEVTYPESUPPLEMENTAL_SHIFTER:
					sDeviceSubType = "gear selection axis";
					break;
				case DI8DEVTYPESUPPLEMENTAL_SHIFTSTICKGATE:
					sDeviceSubType = "gear selection buttons";
					break;
				case DI8DEVTYPESUPPLEMENTAL_SPLITTHROTTLE:
					sDeviceSubType = "multiple throttle controls";
					break;
				case DI8DEVTYPESUPPLEMENTAL_THREEPEDALS:
					sDeviceSubType = "three pedals";
					break;
				case DI8DEVTYPESUPPLEMENTAL_THROTTLE:
					sDeviceSubType = "single throttle controls";
					break;
			}
			return;
	}
}


//====================================================================
DirectInputProxy8::DirectInputProxy8()
{
}


//====================================================================
std::string  DirectInputProxy8::GetDirectInputProxyInterfaceName() const
{
	return VW_INTERFACE_NAME_DirectInputProxy8;
}


//====================================================================
std::string __fastcall DirectInputProxy8::GetCooperativeLevelDesc(DWORD flags)
{
	std::string s;
	if(flags & DISCL_BACKGROUND) {
		s.append("background");
	}
	else if(flags & DISCL_FOREGROUND) {
		s.append("foreground");
	}
	if(flags & DISCL_EXCLUSIVE) {
		s.append(" exclusive");
	}
	else if(flags & DISCL_NONEXCLUSIVE) {
		s.append(" non-exclusive");
	}
	if(flags & DISCL_NOWINKEY) {
		s.append(" no-win-key");
	}
	return s;
}


//********************************************************************
}//namespace vertexware


