

//--------------------------------------------------------------------
#include "Framework\Plugin.h"
#include "BaseTools\Stream.h"
#include "BaseTools\Debug.h"


//********************************************************************
namespace vertexware {


//====================================================================
PluginVersion::PluginVersion(uint _major, uint _minor, uint _revision, uint _buildFlags)
{
	major = _major;
	minor = _minor;
	revision = _revision;
	buildFlags = _buildFlags;
}


//====================================================================
bool  PluginVersion::operator==(const PluginVersion& rhs) const
{
	return
	(major == rhs.major)  &&  (minor == rhs.minor)  &&
	(revision == rhs.revision)  &&  (buildFlags == rhs.buildFlags);
}


//====================================================================
std::string  PluginVersion::MakeString() const
{
	const char sep = '.';

	std::ostringstream oss;
	oss << major << sep;
	oss << minor << sep;
	oss << revision << sep;
	oss << buildFlags;

	return oss.str();
}


//====================================================================
Plugin::Plugin()
{
	pPluginInfo = 0;
}


//====================================================================
Plugin::~Plugin()
{
}


//====================================================================
const PluginInfo&  Plugin::GetPluginInfo() const
{
	VW_ASSERT(pPluginInfo, "Plugin-info not available.");
	return *pPluginInfo;
}


//====================================================================
bool  Plugin::SetPluginParameter(const NamedParameter& np)
{
	return false;
}


//====================================================================
bool  Plugin::GetPluginParameter(const std::string& parameterName, NamedParameter& out) const
{
	return false;
}


//====================================================================
PluginInfo::PluginInfo(
				const library_info::LibraryVersion& _libraryVersion,
				ePluginType _pluginType,
				const std::string& _pluginName,
				const PluginVersion& _pluginVersion,
				const std::string& _buildTimestamp,
				const std::string& _description,
				const std::string& _developerInfo)
		:
		libraryVersion(_libraryVersion),
		pluginType(_pluginType),
		pluginName(_pluginName),
		pluginVersion(_pluginVersion),
		buildTimestamp(_buildTimestamp),
		description(_description),
		developerInfo(_developerInfo)
{
}


//====================================================================
PluginInfo::PluginInfo(const PluginInfo& pi, ePluginType _pluginType)
{
	*this = pi;
	pluginType = _pluginType;
}


//====================================================================
bool  PluginInfo::operator==(const PluginInfo& rhs) const
{
	return
	(libraryVersion == rhs.libraryVersion) &&
	(pluginType == rhs.pluginType) &&
	(pluginName == rhs.pluginName) &&
	(pluginVersion == rhs.pluginVersion) &&
	(buildTimestamp == rhs.buildTimestamp) &&
	(description == rhs.description) &&
	(developerInfo == rhs.developerInfo);
}


//====================================================================
void  PluginInfo::Dump(std::ostream& os, uint tabLevel) const
{
	TabEffector te(tabLevel);

	os << te << "LibraryVersion: " << libraryVersion.MakeString() << nl;
	os << te << "PluginType....: " << FormatPluginType() << nl;
	os << te << "PluginName....: " << pluginName << nl;
	os << te << "PluginVersion.: " << pluginVersion.MakeString() << nl;
	os << te << "BuildTimestamp: " << buildTimestamp << nl;
	os << te << "Description...: " << description << nl;
	os << te << "DeveloperInfo.: " << developerInfo << nl;
}


//====================================================================
std::string  PluginInfo::FormatPluginType() const
{
	switch(pluginType) {

		case PluginType_GenericExtension:
			return "GenericExtension";

		case PluginType_Application:
			return "Application";

		case PluginType_DirectInputProxy:
			return "DirectInputProxy";

		case PluginType_HIDDriver:
			return "HIDDriver";

		case PluginType_VideoSubsystem:
			return "VideoSubsystem";

		case PluginType_VideoRenderer:
			return "VideoRenderer";
	}
	return "BadPluginType";
}


//====================================================================
PluginDescriptor::PluginDescriptor(const PluginInfo& _pluginInfo)
	:
	pluginInfo(_pluginInfo)
{
}


//====================================================================
Plugin*  PluginDescriptor::CreateAndSetupPlugin()
{
	Plugin* pPlugin = CreatePlugin();
	if(pPlugin) {
		pPlugin->SetPluginInfo(&pluginInfo);
	}
	return pPlugin;
}


//********************************************************************
}//namespace vertexware

