

//--------------------------------------------------------------------
#include "GUI\GUI_Base.h"
#include "GUI\GUI_Manager.h"
#include "BaseTools\STLExtensions.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui {


//====================================================================
void  VideoResourceUser::RequestTileTextureRecache()
{
	gui::Manager::Instance().RequestTileTextureRecache(*this);
}


//====================================================================
void  VideoResourceUser::RequestFontRecache()
{
	gui::Manager::Instance().RequestFontRecache(*this);
}


//====================================================================
bool  MessageHandler::DispatchMessage(Message& msg)
{
	msg.target = this;

	// processing by hooks
	for(uint i = 0; i < hooks.size(); i++) {
		if(hooks[i]->HandleMessage(msg)) return true;
	}

	// processing by original target
	return HandleMessage(msg);
}


//====================================================================
void  MessageHandler::InstallMessageHook(MessageHandler& hook,
										bool bInstallAsFirst,
										const MessageHandler* pInstallBefore)
{
	// cannot install self as a hook
	if(&hook == this) return;

	// multiple install-points not allowed
	UninstallMessageHook(hook);

	// do install
	if(bInstallAsFirst) {
		hooks.insert(hooks.begin(), &hook);
	}
	else {
		if(pInstallBefore) {
			MESSAGE_HANDLER_ITER it = std::find(hooks.begin(), hooks.end(), pInstallBefore);
			if(it != hooks.end()) {
				hooks.insert(it, &hook);
				return;
			}
		}
		hooks.push_back(&hook);
	}
}


//====================================================================
void  MessageHandler::UninstallMessageHook(MessageHandler& hook)
{
	EraseFromContainer(hooks, &hook);
}


//====================================================================
void  MessageHandler::UninstallAllMessageHooks()
{
	hooks.clear();
}


//====================================================================
bool  MessageHandler::SendMessageToServiceProvider(uint serviceID, Message& msg)
{
	return Manager::Instance().SendMessageToServiceProvider(serviceID, msg);
}


//====================================================================
bool  MessageHandler::BroadcastMessage(Message& msg)
{
	return Manager::Instance().BroadcastMessage(msg);
}


//====================================================================
bool  MessageHandler::SendMessageTo(MessageHandler& mh, Message& msg)
{
	return mh.DispatchMessage(msg);
}


//====================================================================
bool  MessageHandler::SendMessageToSelf(Message& msg)
{
	return this->DispatchMessage(msg);
}


//====================================================================
bool  MessageHandler::IsRegisteredAsServiceProvider(uint serviceID) const
{
	return (Manager::Instance().GetServiceProvider(serviceID) == this);
}


//====================================================================
void  MessageHandler::RegisterAsServiceProvider(uint serviceID)
{
	Manager::Instance().RegisterServiceProvider(serviceID, *this);
}


//====================================================================
void  MessageHandler::UnregisterAsServiceProvider(uint serviceID)
{
	Manager::Instance().UnregisterServiceProvider(serviceID, *this);
}


//====================================================================
CommandSender::CommandSender(MessageHandler* _pCommandTarget, uint _commandID)
{
	pCommandTarget = _pCommandTarget;
	commandID = _commandID;
}


//====================================================================
bool  CommandSender::SendMessageToTarget(Message& msg)
{
	if(pCommandTarget) {
		return pCommandTarget->DispatchMessage(msg);
	}
	return false;
}


//====================================================================
bool  CommandSender::SendCommandMessageToTarget()
{
	if(pCommandTarget) {
		return pCommandTarget->DispatchMessage(Message_Command(commandID));
	}
	return false;
}


//********************************************************************
}//namespace gui


//********************************************************************
}//namespace vertexware


