

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Base.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui {


//====================================================================
class VW_COMMON_API Cursor : public VideoResourceUser
{
friend class Manager;

public:
	static void        __fastcall SetGlobalSize(const vec2& size = vec2(16, 16));
	static const vec2& __fastcall GetGlobalSize() { return s_globalSize; }

	static void    __fastcall SetGlobalColor(color4b c = color4b::WHITE);
	static color4b __fastcall GetGlobalColor() { return s_globalColor; }


public:
	Cursor(const std::string& name = "",
			const vec2& hotSpot = vec2::ZERO,
			const vec2& privateSize = vec2(16, 16),
			color4b privateColor = color4b::WHITE);

	virtual ~Cursor();

	void                SetName(const std::string& s = "");
	const std::string&  GetName() const { return cursorName; }

	// range [0, 1] x=left-to-right, y=top-to-bottom
	void         SetHotSpot(const vec2& hotSpot = vec2::ZERO);
	const vec2&  GetHotSpot() const { return hotSpot; }

	void         SetPrivateSize(bool bUsePrivateSize = false, const vec2& size = vec2(16, 16));
	const vec2&  GetPrivateSize() const { return privateSize; }
	bool         IsUsingPrivateSize() const { return (pActiveSize == &privateSize); }

	void     SetPrivateColor(bool bUsePrivateColor = false, color4b c = color4b::WHITE);
	color4b  GetPrivateColor() const { return privateColor; }
	bool     IsUsingPrivateColor() const { return (pActiveColor == &privateColor); }

	const vec2&  GetActiveSize() const { return *pActiveSize; }
	color4b      GetActiveColor() const { return *pActiveColor; }

	// draw from [0, 0] to [sizeX, sizeY], ignore the hot spot
	virtual  void  DrawSelf(gui::Renderer& gr);
	virtual  void  OnTimeDelta(double secs) {}


protected:
	virtual  void  UpdateColorCache() {}
	virtual  void  UpdateGeometryCache() {}


protected:
	//------------
	// Properties.
	//------------
	static  vec2     s_globalSize;
	static  color4b  s_globalColor;

	std::string  cursorName;
	vec2         hotSpot;
	vec2         privateSize;
	color4b      privateColor;

	//-------
	// Cache.
	//-------
	vec2*     pActiveSize;
	color4b*  pActiveColor;
};


//====================================================================
class VW_COMMON_API ImageCursor : public Cursor
{
public:
	ImageCursor(const std::string& name = "",
				const gui::TileParams& tileParams = gui::TileParams(),
				const vec2& hotSpot = vec2::ZERO,
				const vec2& privateSize = vec2(16, 16),
				color4b privateColor = color4b::WHITE);

	void                    SetTileParams(const gui::TileParams& tileParams = gui::TileParams());
	const gui::TileParams&  GetTileParams() const { return tileParams; }

	virtual  void  DrawSelf(gui::Renderer& gr);


protected:
	virtual  void  UpdateColorCache();
	virtual  void  UpdateGeometryCache();

	virtual  bool  CacheTileTextureHandles(gui::Manager& gm);
	virtual  void  PostCacheTileTextureHandles();
	virtual  void  InvalidateTileTextureHandles();


protected:
	//------------
	// Properties.
	//------------
	gui::TileParams  tileParams;

	//-------
	// Cache.
	//-------
	const TileTexture*  pTileTexture;
	RectangleCT         rDrawArea;
};


//********************************************************************
namespace std_cursors {


//====================================================================
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_Void );
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_Client );
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_Move );
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_ResizeN );
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_ResizeS );
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_ResizeW );
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_ResizeE );
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_ResizeNW );
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_ResizeNE );
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_ResizeSW );
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_ResizeSE );
VW_COMMON_API  VW_DECLARE_CONSTANT( char*, CursorName_Stop );


//********************************************************************
}//namespace std_cursors


//********************************************************************
}//namespace gui


//********************************************************************
}//namespace vertexware


