

//--------------------------------------------------------------------
#include "GUI\GUI_Renderer.h"
#include "Framework\VideoRenderer.h"
#include "DataObjects\ImageOperations.h"
#include "DataObjects\SignalIO.h"
#include "DataObjects\ImageOperations.h"
#include "MathTools\TransformTools.h"
#include "BaseTools\Stream.h"


//====================================================================
#define    FONT_TEXTURE_GLYPH_SIZE_MIN    0.01f


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui {


//====================================================================
void  Rectangle::Set(float _left, float _top, float _width, float _height)
{
	left   = _left;
	top    = _top;
	width  = _width;
	height = _height;
}


//====================================================================
void  Rectangle::Set(const vec2& _position, const vec2& _size)
{
	left   = _position.x;
	top    = _position.y;
	width  = _size.x;
	height = _size.y;
}


//====================================================================
void  Rectangle::SetPosition(float _left, float _top)
{
	left = _left;
	top = _top;
}


//====================================================================
void  Rectangle::SetPosition(const vec2& _positionTopLeft)
{
	left = _positionTopLeft.x;
	top = _positionTopLeft.y;
}


//====================================================================
void  Rectangle::SetSize(float _width, float _height)
{
	width = _width;
	height = _height;
}


//====================================================================
void  Rectangle::SetSize(const vec2& _size)
{
	width = _size.x;
	height = _size.y;
}


//====================================================================
void  Rectangle::Offset(float x, float y)
{
	left += x;
	top  += y;
}


//====================================================================
void  Rectangle::Offset(const vec2& off)
{
	left += off.x;
	top  += off.y;
}


//====================================================================
void  Rectangle::Grow(float delta)
{
	float delta2 = delta * 2;
	left   -= delta;
	width  += delta2;
	top    -= delta;
	height += delta2;
}


//====================================================================
void  Rectangle::Shrink(float delta)
{
	float delta2 = delta * 2;
	if(delta2 < width) {
		left  += delta;
		width -= delta2;
	}
	if(delta2 < height) {
		top    += delta;
		height -= delta2;
	}
}


//====================================================================
void  Rectangle::CopyGeometryParamsFrom(const Rectangle& r)
{
	left   = r.left;
	top    = r.top;
	width  = r.width;
	height = r.height;
}


//====================================================================
bool  Rectangle::HitTest(float x, float y) const
{
	if(x < left) return false;
	if(x > left + width) return false;
	if(y < top) return false;
	if(y > top + height) return false;
	return true;
}


//====================================================================
bool  Rectangle::Intersects(const Rectangle& r) const
{
	// test horizontal
	if(left <= r.left) {
		if(left + width < r.left) return false;
	}
	else {
		if(r.left + r.width < left) return false;
	}

	// test vertical
	if(top <= r.top) {
		if(top + height < r.top) return false;
	}
	else {
		if(r.top + r.height < top) return false;
	}
	return true;
}


//====================================================================
bool  Rectangle::Contains(const Rectangle& r) const
{
	if(left > r.left) return false;
	if(left + width < r.left + r.width) return false;
	if(top > r.top) return false;
	if(top + height < r.top + r.height) return false;
	return true;
}


//====================================================================
void  RectangleC::SetColor(color4b c)
{
	colorTL = c;
	colorTR = c;
	colorBL = c;
	colorBR = c;
}


//====================================================================
void  RectangleC::SetColorLeft(color4b c)
{
	colorTL = c;
	colorBL = c;
}


//====================================================================
void  RectangleC::SetColorRight(color4b c)
{
	colorTR = c;
	colorBR = c;
}


//====================================================================
void  RectangleC::SetColorTop(color4b c)
{
	colorTL = c;
	colorTR = c;
}


//====================================================================
void  RectangleC::SetColorBottom(color4b c)
{
	colorBL = c;
	colorBR = c;
}


//====================================================================
void  RectangleC::CopyColorParamsFrom(const RectangleC& r)
{
	colorTL = r.colorTL;
	colorTR = r.colorTR;
	colorBL = r.colorBL;
	colorBR = r.colorBR;
}


//====================================================================
void  RectangleCT::SetStandardTexCoords()
{
	texCoordLeft   = 0.0f;
	texCoordRight  = 1.0f;
	texCoordTop    = 0.0f;
	texCoordBottom = 1.0f;
}


//====================================================================
void  RectangleCT::FillTransformedTexCoords(VertexCT& vertexTL, VertexCT& vertexTR,
											VertexCT& vertexBL, VertexCT& vertexBR) const
{
	switch(texTransform) {
		case TexTransform_Identity:
			vertexTL.texCoord.Set(texCoordLeft, texCoordTop);
			vertexTR.texCoord.Set(texCoordRight, texCoordTop);
			vertexBL.texCoord.Set(texCoordLeft, texCoordBottom);
			vertexBR.texCoord.Set(texCoordRight, texCoordBottom);
			return;

		case TexTransform_FlipH:
			vertexTL.texCoord.Set(texCoordRight, texCoordTop);
			vertexTR.texCoord.Set(texCoordLeft, texCoordTop);
			vertexBL.texCoord.Set(texCoordRight, texCoordBottom);
			vertexBR.texCoord.Set(texCoordLeft, texCoordBottom);
			return;

		case TexTransform_FlipV:
			vertexTL.texCoord.Set(texCoordLeft, texCoordBottom);
			vertexTR.texCoord.Set(texCoordRight, texCoordBottom);
			vertexBL.texCoord.Set(texCoordLeft, texCoordTop);
			vertexBR.texCoord.Set(texCoordRight, texCoordTop);
			return;

		case TexTransform_Rotate90:
			vertexTL.texCoord.Set(texCoordRight, texCoordTop);
			vertexTR.texCoord.Set(texCoordRight, texCoordBottom);
			vertexBL.texCoord.Set(texCoordLeft, texCoordTop);
			vertexBR.texCoord.Set(texCoordLeft, texCoordBottom);
			return;

		case TexTransform_Rotate180:
			vertexTL.texCoord.Set(texCoordRight, texCoordBottom);
			vertexTR.texCoord.Set(texCoordLeft, texCoordBottom);
			vertexBL.texCoord.Set(texCoordRight, texCoordTop);
			vertexBR.texCoord.Set(texCoordLeft, texCoordTop);
			return;

		case TexTransform_Rotate270:
			vertexTL.texCoord.Set(texCoordLeft, texCoordBottom);
			vertexTR.texCoord.Set(texCoordLeft, texCoordTop);
			vertexBL.texCoord.Set(texCoordRight, texCoordBottom);
			vertexBR.texCoord.Set(texCoordRight, texCoordTop);
			return;
	}
}


//====================================================================
void  RectangleCT::CopyTexCoordParamsFrom(const RectangleCT& r)
{
	texCoordLeft   = r.texCoordLeft;
	texCoordRight  = r.texCoordRight;
	texCoordTop    = r.texCoordTop;
	texCoordBottom = r.texCoordBottom;
}


//====================================================================
TextMetrics::TextMetrics(float _textHeight, bool _bMonospace,
						bool _bUseFontDefaultGlyphSpacing, float _customGlyphSpacing,
						float _scaleGlyphWidth, float _scaleGlyphSpacing)
{
	textHeight = _textHeight;
	bMonospace = _bMonospace;
	bUseFontDefaultGlyphSpacing = _bUseFontDefaultGlyphSpacing;
	customGlyphSpacing = _customGlyphSpacing;
	scaleGlyphWidth = _scaleGlyphWidth;
	scaleGlyphSpacing = _scaleGlyphSpacing;
}


//====================================================================
TextMetricsL::TextMetricsL(const TextMetrics& _tm, float _lineHeight)
	:
	TextMetrics(_tm)
{
	lineHeight = _lineHeight;
}


//====================================================================
TextAttributes::TextAttributes(const std::string& _fontName, const TextMetricsL& _textMetricsL)
	:
	fontName(fontName),
	textMetricsL(_textMetricsL)
{
}


//====================================================================
DrawTextParams::DrawTextParams(color4b _color, const vec2& _positionTopLeft,
								const gui::TextMetrics& _textMetrics)
	:
	tm(_textMetrics)
{
	color = _color;
	positionTopLeft = _positionTopLeft;
}


//====================================================================
void  Transform::GenerateMV(mat4& out) const
{
	mat4* pOut = &out;
	mat4  temp;

	if(! translate.IsZero()) {
		GenerateTranslateMV(*pOut, translate.x, translate.y, translate.z);
		ConcatToOut(out, temp, pOut);
	}
	if(rotate.z) {
		GenerateRotateMV(*pOut, rotate.z, Axis_Z);
		ConcatToOut(out, temp, pOut);
	}
	if(rotate.x) {
		GenerateRotateMV(*pOut, rotate.x, Axis_X);
		ConcatToOut(out, temp, pOut);
	}
	if(rotate.y) {
		GenerateRotateMV(*pOut, rotate.y, Axis_Y);
		ConcatToOut(out, temp, pOut);
	}
	if(scale.x != 1  ||  scale.y != 1  ||  scale.z != 1) {
		GenerateNonUniformScale(*pOut, scale.x, scale.y, scale.z);
		ConcatToOut(out, temp, pOut);
	}
	if(pOut == &out) {
		out.SetIdentity();
	}
}


//====================================================================
void  Transform::GenerateVM(mat4& out) const
{
	mat4* pOut = &out;
	mat4  temp;

	if(scale.x != 1  ||  scale.y != 1  ||  scale.z != 1) {
		GenerateNonUniformScale(*pOut, scale.x, scale.y, scale.z);
		ConcatToOut(out, temp, pOut);
	}
	if(rotate.y) {
		GenerateRotateVM(*pOut, rotate.y, Axis_Y);
		ConcatToOut(out, temp, pOut);
	}
	if(rotate.x) {
		GenerateRotateVM(*pOut, rotate.x, Axis_X);
		ConcatToOut(out, temp, pOut);
	}
	if(rotate.z) {
		GenerateRotateVM(*pOut, rotate.z, Axis_Z);
		ConcatToOut(out, temp, pOut);
	}
	if(! translate.IsZero()) {
		GenerateTranslateVM(*pOut, translate.x, translate.y, translate.z);
		ConcatToOut(out, temp, pOut);
	}
	if(pOut == &out) {
		out.SetIdentity();
	}
}


//====================================================================
VideoResource::VideoResource()
{
}


//====================================================================
Texture::Texture(const ImageBase& srcImage, bool _bMultiLevel)
{
	sampleFormat = srcImage.GetSampleFormat();
	bMultiLevel = _bMultiLevel;
	height = srcImage.GetNumRows();
	width = srcImage.GetNumColumns();

	SetMipFilter();
	bLoadBottomToTop = false;

	CheckParameters();
	SetSourceImage(&srcImage);
}


//====================================================================
Texture::Texture(eSampleFormat _sampleFormat, bool _bMultiLevel, uint _height, uint _width)
{
	sampleFormat = _sampleFormat;
	bMultiLevel = _bMultiLevel;
	height = _height;
	width = _width;

	SetMipFilter();
	bLoadBottomToTop = false;

	CheckParameters();
}


//====================================================================
void  Texture::SetSourceImage(const ImageBase* pSrcImage, bool _bLoadBottomToTop)
{
	VW_ASSERT(! pSrcImage  ||
			(pSrcImage->GetNumRows() == height  &&  pSrcImage->GetNumColumns() == width),
			"GUI-texture / source-image dimension mismatch.");

	bLoadBottomToTop = _bLoadBottomToTop;

	bool bOK = false;
	if(pSrcImage) {
		apSrcImage.Assign(ManufactureImage(sampleFormat));
		bOK = TransferSamples2D(*apSrcImage, *pSrcImage, true);
	}
	if(! bOK) {
		apSrcImage.SafeDelete();
	}
	srcFileName.erase();
}


//====================================================================
void  Texture::SetSourceImage(const std::string& fileName, bool _bLoadBottomToTop)
{
	bLoadBottomToTop = _bLoadBottomToTop;

	srcFileName = fileName;
	apSrcImage.SafeDelete();
}


//====================================================================
void  Texture::CreateVideoRendererResource(VideoRenderer& vr)
{
	ReleaseVideoRendererResource();

	apVideoRendererResource.Assign(vr.GetGUIRenderer().CreateResourceTexture2D(sampleFormat, bMultiLevel, height, width));
	if(! apVideoRendererResource) {
		VW_THROW(GUIException, "Failed to create gui-texture resource.");
	}
	UpdateVideoRendererResource();
}


//====================================================================
void  Texture::ReleaseVideoRendererResource()
{
	apVideoRendererResource.SafeDelete();
}


//====================================================================
void  Texture::UpdateVideoRendererResource()
{
	if(! apVideoRendererResource) return;

	bool bBottomToTop = false;
	ImageBase* pSrcImage = GenerateSourceImage(bBottomToTop);

	video_renderer_objects::Texture2D::SourceRowOrderSentry  sros(bBottomToTop);

	if(pSrcImage) {
		apVideoRendererResource->LoadLevels(*pSrcImage);
		ReleaseSourceImage(pSrcImage);
	}
	else {
		apVideoRendererResource->LoadLevels(255);
	}
}


//====================================================================
ImageBase*  Texture::GenerateSourceImage(bool& bBottomToTop)
{
	bBottomToTop = bLoadBottomToTop;

	if(apSrcImage) {
		// use cached image
		return apSrcImage.Get();
	}
	else if(srcFileName.length()) {
		// reload image
		NamedParameterList loadNPL;
		loadNPL.Insert(NamedParameter(signal_codec_parameters::Param_OriginBottom, bLoadBottomToTop));
		ImageBase* pLoadedImage = LoadImage(srcFileName, &loadNPL);
		if(pLoadedImage) return pLoadedImage;
	}
	return 0;
}


//====================================================================
void  Texture::ReleaseSourceImage(ImageBase* pGeneratedSourceImage)
{
	if(apSrcImage != pGeneratedSourceImage) {
		SafeDelete(pGeneratedSourceImage);
	}
}


//====================================================================
void  Texture::CheckParameters()
{
	if(! IsPowerOf2(height)  ||  ! IsPowerOf2(width)) {
		VW_THROW(GUIException, "Texture dimensions are not power-of-2.");
	}
	switch(sampleFormat) {
		case SampleFormat_X8R8G8B8:
		case SampleFormat_A8R8G8B8:
			break;
		default:
			VW_THROW(GUIException, "Texture sample-format not supported by gui.");
	}
}


//====================================================================
TileTexture::TileTexture(const ImageBase& srcImage, bool _bMultiLevel,
						uint _numTileRows, uint _numTileColumns)
	:
	Texture(srcImage, _bMultiLevel)
{
	numTileRows = _numTileRows;
	numTileColumns = _numTileColumns;

	CalcParameters();
}


//====================================================================
TileTexture::TileTexture(eSampleFormat _sampleFormat, bool _bMultiLevel,
						uint _height, uint _width,
						uint _numTileRows, uint _numTileColumns)
	:
	Texture(_sampleFormat, _bMultiLevel, _height, _width)
{
	numTileRows = _numTileRows;
	numTileColumns = _numTileColumns;

	CalcParameters();
}


//====================================================================
void  TileTexture::CalcParameters()
{
	ClampLow<uint>(numTileRows, 1);
	ClampLow<uint>(numTileColumns, 1);

	tileSizeU = 1.0f / numTileColumns;
	tileSizeV = 1.0f / numTileRows;

	tilePixelSizeU = GetWidth() / float(numTileColumns);
	tilePixelSizeV = GetHeight() / float(numTileRows);
}


//====================================================================
void  TileTexture::GetTileCoords(float tileRow, float tileColumn,
								float& left, float& right,
								float& top, float& bottom) const
{
	left   = GetTileCoordU(tileColumn);
	right  = left + tileSizeU;
	top    = GetTileCoordV(tileRow);
	bottom = top + tileSizeV;
}


//====================================================================
void  TileTexture::GetTileCoords(float tileRow, float tileColumn, RectangleCT& r) const
{
	GetTileCoords(tileRow, tileColumn,
			r.texCoordLeft, r.texCoordRight, r.texCoordTop, r.texCoordBottom);
}


//====================================================================
Font::Font()
{
	SetFamilyName();
	SetBold();
	SetItalic();
	SetAsciiSubsetOnly();
	SetDefaultSpacingMono();
	SetDefaultSpacing();
}


//====================================================================
float  Font::CalcSpacingWidth(const TextMetrics& tm) const
{
	return
	tm.scaleGlyphSpacing * tm.textHeight *
	(tm.bUseFontDefaultGlyphSpacing ? GetDefaultSpacing(tm.bMonospace) : tm.customGlyphSpacing);
}


//====================================================================
uint __fastcall FontTexture::FixTextureWidth(uint textureWidth)
{
	if(textureWidth < 512) return 256;
	if(textureWidth < 1024) return 512;
	return 1024;
}


//====================================================================
FontTexture::CreationParams::CreationParams()
{
	familyName = "Arial";
	bBold = false;
	bItalic = false;

	defaultSpacingMono = 0;
	defaultSpacing = 0;

	bAsciiSubsetOnly = true;
	textureWidth = 512;
	bMultiLevel = true;

	igScaleEmHeight = 1;
	igGlyphOffsetH = 0;
	igGlyphOffsetV = 0;
	igGlyphSizeModifyH = 0;
	igGlyphSizeModifyV = 0;

	autoCalcWidthEmptyGlyph = 0.5f;
	autoCalcAddPixelsLeft = 1;
	autoCalcAddPixelsRight = 1;
	autoCalcAddPixelsTop = 1;
	autoCalcAddPixelsBottom = 1;
};


//====================================================================
void  FontTexture::CreationParams::Dump(std::ostream& os, uint tabLevel) const
{
	TabEffector  te(tabLevel);

	os << te << "Family............: " << familyName << nl;
	os << te << "Bold..............: " << ToString(bBold) << nl;
	os << te << "Italic............: " << ToString(bItalic) << nl;

	os << te << "DefaultSpacingMono: " << defaultSpacingMono << nl;
	os << te << "DefaultSpacing....: " << defaultSpacing << nl;

	os << te << "AsciiSubsetOnly...: " << ToString(bAsciiSubsetOnly) << nl;
	os << te << "TextureWidth......: " << textureWidth << nl;
	os << te << "TextureMultiLevel.: " << ToString(bMultiLevel) << nl;

	os << te << "IGScaleEmHeight...: " << igScaleEmHeight << nl;
	os << te << "IGGlyphOffsetH....: " << igGlyphOffsetH << nl;
	os << te << "IGGlyphOffsetV....: " << igGlyphOffsetV << nl;
	os << te << "IGGlyphSizeModifyH: " << igGlyphSizeModifyH << nl;
	os << te << "IGGlyphSizeModifyV: " << igGlyphSizeModifyV << nl;

	os << te << "ACWidthEmptyGlyph.: " << autoCalcWidthEmptyGlyph << nl;
	os << te << "ACAddPixelsLeft...: " << autoCalcAddPixelsLeft << nl;
	os << te << "ACAddPixelsRight..: " << autoCalcAddPixelsRight << nl;
	os << te << "ACAddPixelsTop....: " << autoCalcAddPixelsTop << nl;
	os << te << "ACAddPixelsBottom.: " << autoCalcAddPixelsBottom << nl;
}


//====================================================================
FontTexture* __fastcall FontTexture::CreateFontTexture(const CreationParams& cp)
{
	FontTexture* p = new FontTexture(cp.bAsciiSubsetOnly, cp.textureWidth, cp.bMultiLevel);

	p->SetFamilyName(cp.familyName);
	p->SetBold(cp.bBold);
	p->SetItalic(cp.bItalic);

	p->SetDefaultSpacingMono(cp.defaultSpacingMono);
	p->SetDefaultSpacing(cp.defaultSpacing);

	p->SetImageGenerationParameters(cp.igScaleEmHeight,
					cp.igGlyphOffsetH, cp.igGlyphOffsetV,
					cp.igGlyphSizeModifyH, cp.igGlyphSizeModifyV);

	p->CalcGlyphPropertiesFromImage(cp.autoCalcWidthEmptyGlyph,
					cp.autoCalcAddPixelsLeft, cp.autoCalcAddPixelsRight,
					cp.autoCalcAddPixelsTop, cp.autoCalcAddPixelsBottom);
	return p;
}


//====================================================================
void  FontTexture::GetCreationParameters(CreationParams& cp) const
{
	cp.familyName = GetFamilyName();
	cp.bBold = IsBold();
	cp.bItalic = IsItalic();

	cp.defaultSpacingMono = GetDefaultSpacingMono();
	cp.defaultSpacing = GetDefaultSpacing();

	cp.bAsciiSubsetOnly = IsAsciiSubsetOnly();
	cp.textureWidth = GetWidth();
	cp.bMultiLevel = IsMultiLevel();

	cp.igScaleEmHeight = fIGScaleEmHeight;
	cp.igGlyphOffsetH = fIGGlyphOffsetH;
	cp.igGlyphOffsetV = fIGGlyphOffsetV;
	cp.igGlyphSizeModifyH = fIGGlyphSizeModifyH;
	cp.igGlyphSizeModifyV = fIGGlyphSizeModifyV;

	cp.autoCalcWidthEmptyGlyph = autoCalcWidthEmptyGlyph;
	cp.autoCalcAddPixelsLeft = autoCalcAddPixelsLeft;
	cp.autoCalcAddPixelsRight = autoCalcAddPixelsRight;
	cp.autoCalcAddPixelsTop = autoCalcAddPixelsTop;
	cp.autoCalcAddPixelsBottom = autoCalcAddPixelsBottom;
}


//====================================================================
FontTexture::FontTexture(bool _bAsciiSubsetOnly, uint textureWidth, bool bMultiLevel)
	:
	TileTexture(SampleFormat_A8R8G8B8, bMultiLevel,
				FixTextureWidth(textureWidth) / (_bAsciiSubsetOnly ? 2 : 1),
				FixTextureWidth(textureWidth),
				FontTexture::NumGlyphColumns / (_bAsciiSubsetOnly ? 2 : 1),
				FontTexture::NumGlyphColumns)
{
	SetAsciiSubsetOnly(_bAsciiSubsetOnly);

	SetGenerateImageFromSystemFont(true);
	SetImageGenerationParameters();
	ClearGlyphProperties();
}


//====================================================================
void  FontTexture::SetGenerateImageFromSystemFont(bool _bGenerateImageFromSystemFont)
{
	bGenerateImageFromSystemFont = _bGenerateImageFromSystemFont;
}


//====================================================================
ImageBase*  FontTexture::GenerateImageFromSystemFont()
{
	ImageBase* pImage = ManufactureImage(SampleFormat_A8R8G8B8);
	if(! pImage) return 0;

	// creates generic-monospace font if requested font is not available
	GenerateFontGlyphsAscii(*pImage, 0, GetNumGlyphs() - 1,
							GetFamilyName(), IsBold(), IsItalic(),
							GetNumGlyphColumns(), GetGlyphPixelSizeU(),
							fIGScaleEmHeight,
							fIGGlyphOffsetH, fIGGlyphOffsetV,
							fIGGlyphSizeModifyH, fIGGlyphSizeModifyV,
							true, false, false);

	// sync internal-data if requested font is not available
	if(! CanGenerateFont(GetFamilyName(), IsBold(), IsItalic())) {
		SetFamilyName("GenericMonospace");
		SetBold(false);
		SetItalic(false);
	}

	return pImage;
}


//====================================================================
void  FontTexture::SetImageGenerationParameters(float _scaleEmHeight,
							float _glyphOffsetH, float _glyphOffsetV,
							float _glyphSizeModifyH, float _glyphSizeModifyV)
{
	fIGScaleEmHeight = _scaleEmHeight;
	fIGGlyphOffsetH = _glyphOffsetH;
	fIGGlyphOffsetV = _glyphOffsetV;
	fIGGlyphSizeModifyH = _glyphSizeModifyH;
	fIGGlyphSizeModifyV = _glyphSizeModifyV;
}


//====================================================================
void  FontTexture::GetImageGenerationParameters(float& _scaleEmHeight,
							float& _glyphOffsetH, float& _glyphOffsetV,
							float& _glyphSizeModifyH, float& _glyphSizeModifyV)
{
	_scaleEmHeight = fIGScaleEmHeight;
	_glyphOffsetH = fIGGlyphOffsetH;
	_glyphOffsetV = fIGGlyphOffsetV;
	_glyphSizeModifyH = fIGGlyphSizeModifyH;
	_glyphSizeModifyV = fIGGlyphSizeModifyV;
}


//====================================================================
void  FontTexture::CalcGlyphPropertiesFromImage(float widthEmptyGlyph,
								float addPixelsLeft, float addPixelsRight,
								float addPixelsTop, float addPixelsBottom)
{
	// generate font-image
	bool bBottomToTop = false;
	ImageBase* pSrcImage = GenerateSourceImage(bBottomToTop);
	if(! pSrcImage) {
		ClearGlyphProperties();
		return;
	}

	VW_ASSERT(pSrcImage->GetSampleFormat() == SampleFormat_A8R8G8B8, "Invalid sample-format.");
	VW_ASSERT((pSrcImage->GetNumRows() == GetGlyphPixelSizeV() * GetNumGlyphRows()) &&
			(pSrcImage->GetNumColumns() == GetGlyphPixelSizeU() * GetNumGlyphColumns()),
			"Image dimension mismatch.");

	Image32* img = static_cast<Image32*>(pSrcImage);

	// calculate common parameters
	addPixelsLeft = Clamped<float>(addPixelsLeft, 0, GetGlyphPixelSizeU());
	addPixelsRight = Clamped<float>(addPixelsRight, 0, GetGlyphPixelSizeU());
	addPixelsTop = Clamped<float>(addPixelsTop, 0, GetGlyphPixelSizeV());
	addPixelsBottom = Clamped<float>(addPixelsBottom, 0, GetGlyphPixelSizeV());

	float oneOverTileSizeU = 1.0f / GetGlyphPixelSizeU();
	float oneOverTileSizeV = 1.0f / GetGlyphPixelSizeV();

	Clamp<float>(widthEmptyGlyph, 0, 1);
	float  emptyGlyphMinHOffset = (1 - widthEmptyGlyph) * 0.5f;
	float  emptyGlyphMaxHOffset = 1 - emptyGlyphMinHOffset;

	float  heightEmptyGlyph = widthEmptyGlyph * 0.5f;
	float  emptyGlyphMinVOffset = (1 - heightEmptyGlyph) * 0.5f;
	float  emptyGlyphMaxVOffset = 1 - emptyGlyphMinVOffset;

	float  minVOffset = 1;
	float  maxVOffset = 0;
	float  minHOffset = 1;
	float  maxHOffset = 0;
	float  widths[256];
	float  hOffsets[256];

	// clear all
	{for(uint i = 0; i < 256; i++) {
		widths[i] = 0;
		hOffsets[i] = 0;
	}}

	// analyze pixels of each glyph
	for(uint r = 0; r < GetNumGlyphRows(); r++) {
		for(uint c = 0; c < GetNumGlyphColumns(); c++) {

			uint trMin = r * GetGlyphPixelSizeV();
			uint trMax = trMin + GetGlyphPixelSizeV();
			uint tcMin = c * GetGlyphPixelSizeU();
			uint tcMax = tcMin + GetGlyphPixelSizeU();

			float rMin = trMax;
			float rMax = trMin;
			float cMin = tcMax;
			float cMax = tcMin;

			// find glyph's bounding-box
			bool bFoundPixel = false;

			for(uint tr = trMin; tr < trMax; tr++) {
				for(uint tc = tcMin; tc < tcMax; tc++) {

					Sample32* pixel = img->Sample(tr, tc);
					if(pixel->i0) {
						bFoundPixel = true;

						rMin = Min<float>(rMin, tr);
						rMax = Max<float>(rMax, tr);
						cMin = Min<float>(cMin, tc);
						cMax = Max<float>(cMax, tc);
					}
				}
			}

			if(bFoundPixel) {
				rMin -= trMin;
				rMax -= trMin;
				cMin -= tcMin;
				cMax -= tcMin;
			}
			else {
				rMin = emptyGlyphMinVOffset * GetGlyphPixelSizeV();
				rMax = emptyGlyphMaxVOffset * GetGlyphPixelSizeV();
				cMin = emptyGlyphMinHOffset * GetGlyphPixelSizeU();
				cMax = emptyGlyphMaxHOffset * GetGlyphPixelSizeU();
			}

			// add guard-band (pixels on each side)
			rMin = ClampedLow<float>(rMin - addPixelsTop, 0);
			rMax = ClampedHigh<float>(rMax + addPixelsBottom, GetGlyphPixelSizeU());
			cMin = ClampedLow<float>(cMin - addPixelsLeft, 0);
			cMax = ClampedHigh<float>(cMax + addPixelsRight, GetGlyphPixelSizeV());

			// accumulate results & update properties
			uint ch = r * GetNumTileColumns() + c;

			maxVOffset = Max(maxVOffset, rMax * oneOverTileSizeV);
			minVOffset = Min(minVOffset, rMin * oneOverTileSizeV);

			maxHOffset = Max(maxHOffset, cMax * oneOverTileSizeU);
			minHOffset = Min(minHOffset, cMin * oneOverTileSizeU);

			widths[ch] = (cMax - cMin) * oneOverTileSizeU;
			hOffsets[ch] = cMin * oneOverTileSizeU;
		}
	}

	// apply results
	SetGlyphWidthMono(maxHOffset - minHOffset);
	SetGlyphHOffsetMono(minHOffset);
	SetGlyphHeight(maxVOffset - minVOffset);
	SetGlyphVOffset(minVOffset);
	SetGlyphWidths(widths);
	SetGlyphHOffsets(hOffsets);

	// release font-image
	ReleaseSourceImage(pSrcImage);

	// store auto-generation params
	autoCalcWidthEmptyGlyph = widthEmptyGlyph;
	autoCalcAddPixelsLeft = addPixelsLeft;
	autoCalcAddPixelsRight = addPixelsRight;
	autoCalcAddPixelsTop = addPixelsTop;
	autoCalcAddPixelsBottom = addPixelsBottom;
}


//====================================================================
void  FontTexture::ClearGlyphProperties()
{
	SetGlyphHeight();
	SetGlyphVOffset();
	SetGlyphWidthMono();
	SetGlyphHOffsetMono();
	SetGlyphWidths();
	SetGlyphHOffsets();

	// auto-generation params
	autoCalcWidthEmptyGlyph = 0.5f;
	autoCalcAddPixelsLeft = 0;
	autoCalcAddPixelsRight = 0;
	autoCalcAddPixelsTop = 0;
	autoCalcAddPixelsBottom = 0;
}


//====================================================================
void  FontTexture::SetGlyphHeight(float h)
{
	glyphHeight = Clamped<float>(h, FONT_TEXTURE_GLYPH_SIZE_MIN, 1);
	oneOverGlyphHeight = 1 / glyphHeight;
	CalcVerticalGlyphCoords();
}


//====================================================================
void  FontTexture::SetGlyphVOffset(float o)
{
	glyphVOffset = Clamped<float>(o, 0, 1);
	CalcVerticalGlyphCoords();
}


//====================================================================
void  FontTexture::SetGlyphWidthMono(float w)
{
	glyphWidthMono = Clamped<float>(w, FONT_TEXTURE_GLYPH_SIZE_MIN, 1);
	CalcHorizontalGlyphCoords();
}


//====================================================================
void  FontTexture::SetGlyphHOffsetMono(float o)
{
	glyphHOffsetMono = Clamped<float>(o, 0, 1);
	CalcHorizontalGlyphCoords();
}


//====================================================================
void  FontTexture::SetGlyphWidths(const float widths[256])
{
	for(uint i = 0; i < 256; i++) {
		glyphWidths[i] = Clamped<float>(widths[i], FONT_TEXTURE_GLYPH_SIZE_MIN, 1);
	}
	CalcHorizontalGlyphCoords();
}


//====================================================================
void  FontTexture::SetGlyphWidths(float w)
{
	float widths[256];
	for(uint i = 0; i < 256; i++) widths[i] = w;
	SetGlyphWidths(widths);
}


//====================================================================
void  FontTexture::SetGlyphHOffsets(const float offsets[256])
{
	for(uint i = 0; i < 256; i++) {
		glyphHOffsets[i] = Clamped<float>(offsets[i], 0, 1);
	}
	CalcHorizontalGlyphCoords();
}


//====================================================================
void  FontTexture::SetGlyphHOffsets(float o)
{
	float offsets[256];
	for(uint i = 0; i < 256; i++) offsets[i] = o;
	SetGlyphHOffsets(offsets);
}


//====================================================================
void  FontTexture::CalcVerticalGlyphCoords()
{
	float offsetV = GetTileSizeV() * glyphVOffset;
	float heightV = GetTileSizeV() * glyphHeight;

	for(uint i = 0; i < 256; i++) {
		glyphCoordsTop[i] = GetTileCoordV(i / NumGlyphColumns) + offsetV;
		glyphCoordsBottom[i] = glyphCoordsTop[i] + heightV;
	}
}


//====================================================================
void  FontTexture::CalcHorizontalGlyphCoords()
{
	// var-pitch coords
	{
		float tsU = GetTileSizeU();

		for(uint i = 0; i < 256; i++) {
			glyphCoordsLeft[i] = GetTileCoordU(i % NumGlyphColumns) + tsU * glyphHOffsets[i];
			glyphCoordsRight[i] = glyphCoordsLeft[i] + tsU * glyphWidths[i];
		}
	}

	// monospace coords
	{
		float offsetU = GetTileSizeU() * glyphHOffsetMono;
		float widthU = GetTileSizeU() * glyphWidthMono;

		for(uint i = 0; i < 256; i++) {
			glyphCoordsLeftMono[i] = GetTileCoordU(i % NumGlyphColumns) + offsetU;
			glyphCoordsRightMono[i] = glyphCoordsLeftMono[i] + widthU;
		}
	}
}


//====================================================================
ImageBase*  FontTexture::GenerateSourceImage(bool& bBottomToTop)
{
	if(bGenerateImageFromSystemFont) {
		bBottomToTop = false;
		return GenerateImageFromSystemFont();
	}
	return TileTexture::GenerateSourceImage(bBottomToTop);
}


//====================================================================
void  FontTexture::ReleaseSourceImage(ImageBase* pGeneratedSourceImage)
{
	if(bGenerateImageFromSystemFont) {
		SafeDelete(pGeneratedSourceImage);
	}
	else {
		TileTexture::ReleaseSourceImage(pGeneratedSourceImage);
	}
}


//====================================================================
void  FontTexture::DrawText(gui::Renderer& gr,
							uint textLen, const char* pText,
							const gui::DrawTextParams& dtp) const
{
	gr.DrawText(*this, textLen, pText, dtp);
}


//====================================================================
float  FontTexture::CalcGlyphWidth(const TextMetrics& tm, char ch) const
{
	float gwMult = CalcGlyphWidthMultiplier(tm);

	// monospace
	if(tm.bMonospace) {
		return (glyphWidthMono * gwMult);
	}

	// proportional
	return (glyphWidths[ static_cast<ubyte>(ch) ] * gwMult);
}


//====================================================================
float  FontTexture::CalcTextWidth(const TextMetrics& tm, uint textLen, const char* pText) const
{
	float w = 0;

	// monospace
	if(tm.bMonospace) {
		w = CalcGlyphWidth(tm, 0) * textLen;
	}
	// proportional
	else {
		for(uint i = 0; i < textLen; i++) {
			w += glyphWidths[ static_cast<ubyte>(pText[i]) ];
		}
		w *= CalcGlyphWidthMultiplier(tm);
	}

	// add spacing
	if(textLen > 0) {
		w += (textLen - 1) * CalcSpacingWidth(tm);
	}
	return w;
}


//====================================================================
void  FontTexture::CalcTextWidths(std::vector<float>& outWidths,
								const TextMetrics& tm,
								uint textLen, const char* pText) const
{
	outWidths.clear();
	outWidths.reserve(textLen);

	float spacing = CalcSpacingWidth(tm);

	// monospace
	if(tm.bMonospace) {
		float w = CalcGlyphWidth(tm, 0);

		float curWidth = 0;
		for(uint i = 0; i < textLen; i++) {

			curWidth += w;
			outWidths.push_back(curWidth);
			curWidth += spacing;
		}
	}

	// proportional
	else {
		float gwMult = CalcGlyphWidthMultiplier(tm);

		float curWidth = 0;
		for(uint i = 0; i < textLen; i++) {

			curWidth += glyphWidths[ static_cast<ubyte>(pText[i]) ] * gwMult;
			outWidths.push_back(curWidth);
			curWidth += spacing;
		}
	}
}


//====================================================================
void  FontTexture::CalcMaxVisibleText(uint& outLen, float& outWidth, float maxWidth,
								const TextMetrics& tm,
								uint textLen, const char* pText) const
{
	outLen = 0;
	outWidth = 0;

	float spacing = CalcSpacingWidth(tm);

	// monospace
	if(tm.bMonospace) {
		float w = CalcGlyphWidth(tm, 0);

		float curWidth = 0;
		for(uint i = 0; i < textLen; i++) {

			curWidth += w;
			if(curWidth > maxWidth) break;
			outWidth = curWidth;
			outLen++;
			curWidth += spacing;
		}
	}

	// proportional
	else {
		float gwMult = CalcGlyphWidthMultiplier(tm);

		float curWidth = 0;
		for(uint i = 0; i < textLen; i++) {

			curWidth += glyphWidths[ static_cast<ubyte>(pText[i]) ] * gwMult;
			if(curWidth > maxWidth) break;
			outWidth = curWidth;
			outLen++;
			curWidth += spacing;
		}
	}
}


//====================================================================
Renderer::Statistics::Statistics()
{
	Clear();
}


//====================================================================
void  Renderer::Statistics::Clear(uint val)
{
	numClearBuffers = val;
	numSetViewport = val;
	numBegin = val;
	numEnd = val;
	numBeginInterleaved = val;
	numEndInterleaved = val;
	numSetDepthParams = val;
	numSetTransform = val;
	numDrawPrimitiveColored = val;
	numDrawPrimitiveTextured = val;
	numDrawText = val;
	numDrawRectangleColored = val;
	numDrawRectangleTextured = val;
	numDrawRectangleFrameColored = val;
	numTextureChanges = val;
	numPrimitivesColored = val;
	numPrimitivesTextured = val;
	numTextGlyphs = val;
}


//====================================================================
void  Renderer::Statistics::Accum(const Statistics& s)
{
	numClearBuffers += s.numClearBuffers;
	numSetViewport += s.numSetViewport;
	numBegin += s.numBegin;
	numEnd += s.numEnd;
	numBeginInterleaved += s.numBeginInterleaved;
	numEndInterleaved += s.numEndInterleaved;
	numSetDepthParams += s.numSetDepthParams;
	numSetTransform += s.numSetTransform;
	numDrawPrimitiveColored += s.numDrawPrimitiveColored;
	numDrawPrimitiveTextured += s.numDrawPrimitiveTextured;
	numDrawText += s.numDrawText;
	numDrawRectangleColored += s.numDrawRectangleColored;
	numDrawRectangleTextured += s.numDrawRectangleTextured;
	numDrawRectangleFrameColored += s.numDrawRectangleFrameColored;
	numTextureChanges += s.numTextureChanges;
	numPrimitivesColored += s.numPrimitivesColored;
	numPrimitivesTextured += s.numPrimitivesTextured;
	numTextGlyphs += s.numTextGlyphs;
}


//====================================================================
void  Renderer::Statistics::Average(const Statistics& s, uint frameCount)
{
	double val = frameCount ? 1.0 / frameCount : 0;

	numClearBuffers = s.numClearBuffers * val;
	numSetViewport = s.numSetViewport * val;
	numBegin = s.numBegin * val;
	numEnd = s.numEnd * val;
	numBeginInterleaved = s.numBeginInterleaved * val;
	numEndInterleaved = s.numEndInterleaved * val;
	numSetDepthParams = s.numSetDepthParams * val;
	numSetTransform = s.numSetTransform * val;
	numDrawPrimitiveColored = s.numDrawPrimitiveColored * val;
	numDrawPrimitiveTextured = s.numDrawPrimitiveTextured * val;
	numDrawText = s.numDrawText * val;
	numDrawRectangleColored = s.numDrawRectangleColored * val;
	numDrawRectangleTextured = s.numDrawRectangleTextured * val;
	numDrawRectangleFrameColored = s.numDrawRectangleFrameColored * val;
	numTextureChanges = s.numTextureChanges * val;
	numPrimitivesColored = s.numPrimitivesColored * val;
	numPrimitivesTextured = s.numPrimitivesTextured * val;
	numTextGlyphs = s.numTextGlyphs * val;
}


//====================================================================
void  Renderer::Statistics::Min(const Statistics& s1, const Statistics& s2)
{
	numClearBuffers = vertexware::Min(s1.numClearBuffers, s2.numClearBuffers);
	numSetViewport = vertexware::Min(s1.numSetViewport, s2.numSetViewport);
	numBegin = vertexware::Min(s1.numBegin, s2.numBegin);
	numEnd = vertexware::Min(s1.numEnd, s2.numEnd);
	numBeginInterleaved = vertexware::Min(s1.numBeginInterleaved, s2.numBeginInterleaved);
	numEndInterleaved = vertexware::Min(s1.numEndInterleaved, s2.numEndInterleaved);
	numSetDepthParams = vertexware::Min(s1.numSetDepthParams, s2.numSetDepthParams);
	numSetTransform = vertexware::Min(s1.numSetTransform, s2.numSetTransform);
	numDrawPrimitiveColored = vertexware::Min(s1.numDrawPrimitiveColored, s2.numDrawPrimitiveColored);
	numDrawPrimitiveTextured = vertexware::Min(s1.numDrawPrimitiveTextured, s2.numDrawPrimitiveTextured);
	numDrawText = vertexware::Min(s1.numDrawText, s2.numDrawText);
	numDrawRectangleColored = vertexware::Min(s1.numDrawRectangleColored, s2.numDrawRectangleColored);
	numDrawRectangleTextured = vertexware::Min(s1.numDrawRectangleTextured, s2.numDrawRectangleTextured);
	numDrawRectangleFrameColored = vertexware::Min(s1.numDrawRectangleFrameColored, s2.numDrawRectangleFrameColored);
	numTextureChanges = vertexware::Min(s1.numTextureChanges, s2.numTextureChanges);
	numPrimitivesColored = vertexware::Min(s1.numPrimitivesColored, s2.numPrimitivesColored);
	numPrimitivesTextured = vertexware::Min(s1.numPrimitivesTextured, s2.numPrimitivesTextured);
	numTextGlyphs = vertexware::Min(s1.numTextGlyphs, s2.numTextGlyphs);
}


//====================================================================
void  Renderer::Statistics::Max(const Statistics& s1, const Statistics& s2)
{
	numClearBuffers = vertexware::Max(s1.numClearBuffers, s2.numClearBuffers);
	numSetViewport = vertexware::Max(s1.numSetViewport, s2.numSetViewport);
	numBegin = vertexware::Max(s1.numBegin, s2.numBegin);
	numEnd = vertexware::Max(s1.numEnd, s2.numEnd);
	numBeginInterleaved = vertexware::Max(s1.numBeginInterleaved, s2.numBeginInterleaved);
	numEndInterleaved = vertexware::Max(s1.numEndInterleaved, s2.numEndInterleaved);
	numSetDepthParams = vertexware::Max(s1.numSetDepthParams, s2.numSetDepthParams);
	numSetTransform = vertexware::Max(s1.numSetTransform, s2.numSetTransform);
	numDrawPrimitiveColored = vertexware::Max(s1.numDrawPrimitiveColored, s2.numDrawPrimitiveColored);
	numDrawPrimitiveTextured = vertexware::Max(s1.numDrawPrimitiveTextured, s2.numDrawPrimitiveTextured);
	numDrawText = vertexware::Max(s1.numDrawText, s2.numDrawText);
	numDrawRectangleColored = vertexware::Max(s1.numDrawRectangleColored, s2.numDrawRectangleColored);
	numDrawRectangleTextured = vertexware::Max(s1.numDrawRectangleTextured, s2.numDrawRectangleTextured);
	numDrawRectangleFrameColored = vertexware::Max(s1.numDrawRectangleFrameColored, s2.numDrawRectangleFrameColored);
	numTextureChanges = vertexware::Max(s1.numTextureChanges, s2.numTextureChanges);
	numPrimitivesColored = vertexware::Max(s1.numPrimitivesColored, s2.numPrimitivesColored);
	numPrimitivesTextured = vertexware::Max(s1.numPrimitivesTextured, s2.numPrimitivesTextured);
	numTextGlyphs = vertexware::Max(s1.numTextGlyphs, s2.numTextGlyphs);
}


//====================================================================
void  Renderer::Statistics::Dump(std::ostream& os, uint tabLevel) const
{
	TabEffector te(tabLevel);

	os << te << "NumClearBuffers.............: " << numClearBuffers << nl;
	os << te << "NumSetViewport..............: " << numSetViewport << nl;
	os << te << "NumBegin....................: " << numBegin << nl;
	os << te << "NumEnd......................: " << numEnd << nl;
	os << te << "NumBeginInterleaved.........: " << numBeginInterleaved << nl;
	os << te << "NumEndInterleaved...........: " << numEndInterleaved << nl;
	os << te << "NumSetDepthParams...........: " << numSetDepthParams << nl;
	os << te << "NumSetTransform.............: " << numSetTransform << nl;
	os << te << "NumDrawPrimitiveColored.....: " << numDrawPrimitiveColored << nl;
	os << te << "NumDrawPrimitiveTextured....: " << numDrawPrimitiveTextured << nl;
	os << te << "NumDrawText.................: " << numDrawText << nl;
	os << te << "NumDrawRectangleColored.....: " << numDrawRectangleColored << nl;
	os << te << "NumDrawRectangleTextured....: " << numDrawRectangleTextured << nl;
	os << te << "NumDrawRectangleFrameColored: " << numDrawRectangleFrameColored << nl;
	os << te << "NumTextureChanges...........: " << numTextureChanges << nl;
	os << te << "NumPrimitivesColored........: " << numPrimitivesColored << nl;
	os << te << "NumPrimitivesTextured.......: " << numPrimitivesTextured << nl;
	os << te << "NumTextGlyphs...............: " << numTextGlyphs << nl;
}


//====================================================================
Renderer::PixelParams::PixelParams(float _offsetX, float _offsetY)
{
	offsetX = _offsetX;
	offsetY = _offsetY;
}


//====================================================================
Renderer::ClearParams::ClearParams(bool _bClearColor, color4b _color,
								bool _bClearDepth, float _depth,
								bool _bClearStencil, uint _stencil,
								bool _bUseRectangle, const gui::Rectangle& r)
{
	bClearColor = _bClearColor;
	bClearDepth = _bClearDepth;
	bClearStencil = _bClearStencil;
	bUseRectangle = _bUseRectangle;

	color = _color;
	depth = _depth;
	stencil = _stencil;
	rectangle = r;
}


//====================================================================
Renderer::ViewportParams::ViewportParams(const vec2& _topLeft, const vec2& _size,
								float _renderDepthNear, float _renderDepthFar)
{
	topLeft = _topLeft;
	size = _size;
	renderDepthNear = _renderDepthNear;
	renderDepthFar = _renderDepthFar;
}


//====================================================================
Renderer::DepthParams::DepthParams(
							float _clippingPlaneNear, float _clippingPlaneFar,
							float _renderDepthNear, float _renderDepthFar,
							bool _bEnableDepthTest, bool _bEnableDepthWrite,
							bool _bUseWBuffering)
{
	clippingPlaneNear = _clippingPlaneNear;
	clippingPlaneFar  = _clippingPlaneFar;

	renderDepthNear = _renderDepthNear;
	renderDepthFar  = _renderDepthFar;

	bEnableDepthTest = _bEnableDepthTest;
	bEnableDepthWrite = _bEnableDepthWrite;
	bUseWBuffering = _bUseWBuffering;
}


//====================================================================
Renderer::Renderer(VideoRenderer& vrParent)
	:
	parentVideoRenderer(vrParent)
{
	statistics.Clear();

	windowWidth = 1;
	windowHeight = 1;

	tempVertexBufferC.Allocate(10);
	tempVertexBufferCT.Allocate(4);
}


//====================================================================
Renderer::~Renderer()
{
}


//====================================================================
void  Renderer::OnWindowResize(uint width, uint height)
{
	windowWidth = ClampedLow<uint>(width, 1);
	windowHeight = ClampedLow<uint>(height, 1);
}


//====================================================================
void  Renderer::DrawRectangle(const gui::RectangleC& rect)
{
	statistics.numDrawRectangleColored++;

	// uses 4 C-vertices
	float posRight  = rect.left + rect.width;
	float posBottom = rect.top  + rect.height;

	tempVertexBufferC[0].position.Set(rect.left, rect.top, 0);
	tempVertexBufferC[0].color = rect.colorTL;

	tempVertexBufferC[1].position.Set(rect.left, posBottom, 0);
	tempVertexBufferC[1].color = rect.colorBL;

	tempVertexBufferC[2].position.Set(posRight, posBottom, 0);
	tempVertexBufferC[2].color = rect.colorBR;

	tempVertexBufferC[3].position.Set(posRight, rect.top, 0);
	tempVertexBufferC[3].color = rect.colorTR;

	DrawPrimitive(tempVertexBufferC, PrimitiveType_TriangleFan, 2, 0);
}


//====================================================================
void  Renderer::DrawRectangle(const gui::Texture& texture, const gui::RectangleCT& rect)
{
	statistics.numDrawRectangleTextured++;

	// uses 4 CT-vertices
	float posRight  = rect.left + rect.width;
	float posBottom = rect.top  + rect.height;

	tempVertexBufferCT[0].position.Set(rect.left, rect.top, 0);
	tempVertexBufferCT[0].color = rect.colorTL;

	tempVertexBufferCT[1].position.Set(rect.left, posBottom, 0);
	tempVertexBufferCT[1].color = rect.colorBL;

	tempVertexBufferCT[2].position.Set(posRight, posBottom, 0);
	tempVertexBufferCT[2].color = rect.colorBR;

	tempVertexBufferCT[3].position.Set(posRight, rect.top, 0);
	tempVertexBufferCT[3].color = rect.colorTR;

	rect.FillTransformedTexCoords(tempVertexBufferCT[0], tempVertexBufferCT[3],
								tempVertexBufferCT[1], tempVertexBufferCT[2]);

	DrawPrimitive(texture, tempVertexBufferCT, PrimitiveType_TriangleFan, 2, 0);
}


//====================================================================
void  Renderer::DrawRectangleFrame(const gui::RectangleC& rectOuter, float frameWidth)
{
	statistics.numDrawRectangleFrameColored++;

	// uses 10 C-vertices
	ClampHigh<float>(frameWidth, Min(rectOuter.width * 0.5f, rectOuter.height * 0.5f));

	float  posLeftOut   = rectOuter.left;
	float  posLeftIn    = posLeftOut + frameWidth;
	float  posRightOut  = rectOuter.left + rectOuter.width;
	float  posRightIn   = posRightOut - frameWidth;
	float  posTopOut    = rectOuter.top;
	float  posTopIn     = posTopOut + frameWidth;
	float  posBottomOut = rectOuter.top  + rectOuter.height;
	float  posBottomIn  = posBottomOut - frameWidth;

	tempVertexBufferC[0].position.Set(posLeftOut, posTopOut, 0);
	tempVertexBufferC[0].color = rectOuter.colorTL;

	tempVertexBufferC[1].position.Set(posLeftIn, posTopIn, 0);
	tempVertexBufferC[1].color = rectOuter.colorTL;

	tempVertexBufferC[2].position.Set(posRightOut, posTopOut, 0);
	tempVertexBufferC[2].color = rectOuter.colorTR;

	tempVertexBufferC[3].position.Set(posRightIn, posTopIn, 0);
	tempVertexBufferC[3].color = rectOuter.colorTR;

	tempVertexBufferC[4].position.Set(posRightOut, posBottomOut, 0);
	tempVertexBufferC[4].color = rectOuter.colorBR;

	tempVertexBufferC[5].position.Set(posRightIn, posBottomIn, 0);
	tempVertexBufferC[5].color = rectOuter.colorBR;

	tempVertexBufferC[6].position.Set(posLeftOut, posBottomOut, 0);
	tempVertexBufferC[6].color = rectOuter.colorBL;

	tempVertexBufferC[7].position.Set(posLeftIn, posBottomIn, 0);
	tempVertexBufferC[7].color = rectOuter.colorBL;

	tempVertexBufferC[8] = tempVertexBufferC[0];
	tempVertexBufferC[9] = tempVertexBufferC[1];

	DrawPrimitive(tempVertexBufferC, PrimitiveType_TriangleStrip, 8, 0);
}


//********************************************************************
}//namespace gui


//********************************************************************
}//namespace vertexware


