

//--------------------------------------------------------------------
#include "GUI\GUI_Shader.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_StdObjects.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui {


//====================================================================
gui::TextAttributes __fastcall BaseShader::DefaultTextAttributes()
{
	gui::TextAttributes ta;
	ta.fontName = gui::std_fonts::FontName_Default;
	return ta;
}


//====================================================================
BaseShader::BaseShader(const std::string& _name)
	:
	shaderName(_name)
{
	pFont = 0;

	gui::Manager::Instance().NotifyObjectCreateDelete(*this, true);
}


//====================================================================
BaseShader::~BaseShader()
{
	gui::Manager::Instance().NotifyObjectCreateDelete(*this, false);
}


//====================================================================
void  BaseShader::SetName(const std::string& s)
{
	shaderName = s;
	gui::Manager::Instance().NotifyNameChanged(*this);
}


//====================================================================
void  BaseShader::SetTextAttributes(const gui::TextAttributes& ta)
{
	textAtts = ta;
	textMetricsLCache = textAtts.textMetricsL;
	RequestFontRecache();
}


//====================================================================
gui::TextMetricsL  BaseShader::GetTextMetricsL(float textHeight, float lineHeight) const
{
	gui::TextMetricsL tml = textAtts.textMetricsL;
	tml.textHeight = textHeight;
	tml.lineHeight = lineHeight;
	return tml;
}


//====================================================================
bool  BaseShader::CacheFontHandles(gui::Manager& gm)
{
	if(textAtts.fontName.empty()) {
		pFont = 0;
		return true;
	}
	pFont = gm.GetFont(textAtts.fontName);
	return (pFont != 0);
}


//********************************************************************
}//namespace gui


//********************************************************************
}//namespace vertexware


