

//--------------------------------------------------------------------
#include "GUI\GUI_StdObjects.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_Cursor.h"
#include "DataObjects\SignalIO.h"
#include "DataObjects\ImageOperations.h"
#include "BaseTools\Stream.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui {


//********************************************************************
namespace std_tile_textures {


//====================================================================
VW_DEFINE_CONSTANT( char*, TileTextureName_Set0, "std/set0" );
VW_DEFINE_CONSTANT( char*, DefaultFileName_Set0, "gui_std_tiletexture_set0.tif" );


//====================================================================
bool __fastcall RegisterTileTexture_Set0(std::ostream& os,
										bool bMultiLevel,
										const std::string& registerUnderName,
										const std::string& loadFromFile)
{
	bool bLoadBottomToTop = true;
	NamedParameterList nplLoad;
	nplLoad.Insert(NamedParameter(signal_codec_parameters::Param_OriginBottom, bLoadBottomToTop));

	AutoPtr<ImageBase>  apLoadedImage(LoadImage(loadFromFile, &nplLoad));
	if(apLoadedImage) {

		// convert BGR-intesity to alpha, perform BGR threshold-segmentation
		ProcessSamples_BGRToA_ThresholdBGR(*apLoadedImage);

		AutoPtr<TileTexture> apTT(new TileTexture(
									SampleFormat_A8R8G8B8, bMultiLevel,
									Set0_NumRows * Set0_TileSize,
									Set0_NumCols * Set0_TileSize,
									Set0_NumRows, Set0_NumCols));

		apTT->SetSourceImage(apLoadedImage, bLoadBottomToTop);

		Manager::Instance().RegisterTileTexture(registerUnderName, apTT);
		return true;
	}

	os << e_error << "Failed to load tile-texure: " << registerUnderName;
	os << " from file: " << loadFromFile << nl << e_info;
	return false;
}


//====================================================================
gui::TileParams __fastcall MakeTileParams(eTile t)
{
	using namespace gui;
	using namespace gui::std_tile_textures;

	switch(t) {
	case Tile_X: return TileParams(TileTextureName_Set0, Set0_Row_X, Set0_Col_X);
	case Tile_Check: return TileParams(TileTextureName_Set0, Set0_Row_Check, Set0_Col_Check);
	case Tile_Bullet: return TileParams(TileTextureName_Set0, Set0_Row_Bullet, Set0_Col_Bullet);
	case Tile_ArrowLeft: return TileParams(TileTextureName_Set0, Set0_Row_ArrowRight, Set0_Col_ArrowRight, RectangleCT::TexTransform_FlipH);
	case Tile_ArrowRight: return TileParams(TileTextureName_Set0, Set0_Row_ArrowRight, Set0_Col_ArrowRight);
	case Tile_ArrowUp: return TileParams(TileTextureName_Set0, Set0_Row_ArrowRight, Set0_Col_ArrowRight, RectangleCT::TexTransform_Rotate90);
	case Tile_ArrowDown: return TileParams(TileTextureName_Set0, Set0_Row_ArrowRight, Set0_Col_ArrowRight, RectangleCT::TexTransform_Rotate270);
	case Tile_ArrowUpDown: return TileParams(TileTextureName_Set0, Set0_Row_ArrowUpDown, Set0_Col_ArrowUpDown);
	case Tile_ArrowPointer: return TileParams(TileTextureName_Set0, Set0_Row_ArrowPointer, Set0_Col_ArrowPointer);
	case Tile_Arrow4Way: return TileParams(TileTextureName_Set0, Set0_Row_Arrow4Way, Set0_Col_Arrow4Way);
	case Tile_ArrowNS: return TileParams(TileTextureName_Set0, Set0_Row_ArrowWE, Set0_Col_ArrowWE, RectangleCT::TexTransform_Rotate90);
	case Tile_ArrowWE: return TileParams(TileTextureName_Set0, Set0_Row_ArrowWE, Set0_Col_ArrowWE);
	case Tile_ArrowNESW: return TileParams(TileTextureName_Set0, Set0_Row_ArrowNESW, Set0_Col_ArrowNESW);
	case Tile_ArrowNWSE: return TileParams(TileTextureName_Set0, Set0_Row_ArrowNESW, Set0_Col_ArrowNESW, RectangleCT::TexTransform_Rotate90);
	case Tile_Maximize: return TileParams(TileTextureName_Set0, Set0_Row_Maximize, Set0_Col_Maximize);
	case Tile_Restore: return TileParams(TileTextureName_Set0, Set0_Row_Restore, Set0_Col_Restore);
	case Tile_Stop: return TileParams(TileTextureName_Set0, Set0_Row_Stop, Set0_Col_Stop);
	}
	return gui::TileParams();
}


//********************************************************************
}//namespace std_resources


//********************************************************************
namespace std_fonts {


//====================================================================
VW_DEFINE_CONSTANT( char*, FontName_Default,   "std/default");
VW_DEFINE_CONSTANT( char*, SystemFont_Default, "Tahoma");

VW_DEFINE_CONSTANT( char*, FontName_Console,   "std/console");
VW_DEFINE_CONSTANT( char*, SystemFont_Console, "Lucida Console");


//====================================================================
VW_COMMON_API
void __fastcall RegisterFont_Default(std::ostream& os,
						bool bAsciiSubsetOnly,
						uint textureWidth,
						bool bMultiLevel,
						const std::string& registerUnderName)
{
	AutoPtr<FontTexture> apF(new FontTexture(bAsciiSubsetOnly, textureWidth, bMultiLevel));

	apF->SetFamilyName(SystemFont_Default);
	apF->SetGlyphWidthMono(0.8f);
	apF->SetGlyphHOffsetMono(0.1f);
	apF->SetGlyphWidths(0.7f);
	apF->SetGlyphHOffsets(0.15f);
	apF->SetGlyphHeight(0.95f);
	apF->SetGlyphVOffset(0.025f);
	apF->CalcGlyphPropertiesFromImage(0.4f);
	apF->SetDefaultSpacingMono(0.0f);
	apF->SetDefaultSpacing(0.05f);

	Manager::Instance().RegisterFont(registerUnderName, apF);
}


//====================================================================
VW_COMMON_API
void __fastcall RegisterFont_Console(std::ostream& os,
						bool bAsciiSubsetOnly,
						uint textureWidth,
						bool bMultiLevel,
 						const std::string& registerUnderName)
{
	AutoPtr<FontTexture> apF(new FontTexture(bAsciiSubsetOnly, textureWidth, bMultiLevel));

	apF->SetFamilyName(SystemFont_Console);
	apF->SetImageGenerationParameters(0.9, 0, 0, 0, 0);
	apF->SetGlyphWidthMono(0.55f);
	apF->SetGlyphHOffsetMono(0.225f);
	apF->SetGlyphWidths(0.5f);
	apF->SetGlyphHOffsets(0.25f);
	apF->SetGlyphHeight(0.95f);
	apF->SetGlyphVOffset(0.025f);
	apF->CalcGlyphPropertiesFromImage(0.4f);

	Manager::Instance().RegisterFont(registerUnderName, apF);
}


//********************************************************************
}//namespace std_fonts


//********************************************************************
namespace std_cursors {


//====================================================================
void __fastcall RegisterCursors(std::ostream& os)
{
	using namespace std_tile_textures;

	const vec2  hsHalf(0.5f, 0.5f);

	new ImageCursor(CursorName_Void, MakeTileParams(Tile_ArrowPointer), vec2::ZERO);
	new ImageCursor(CursorName_Client, MakeTileParams(Tile_ArrowPointer), vec2::ZERO);
	new ImageCursor(CursorName_Move, MakeTileParams(Tile_Arrow4Way), hsHalf);
	new ImageCursor(CursorName_ResizeN, MakeTileParams(Tile_ArrowNS), hsHalf);
	new ImageCursor(CursorName_ResizeS, MakeTileParams(Tile_ArrowNS), hsHalf);
	new ImageCursor(CursorName_ResizeW, MakeTileParams(Tile_ArrowWE), hsHalf);
	new ImageCursor(CursorName_ResizeE, MakeTileParams(Tile_ArrowWE), hsHalf);
	new ImageCursor(CursorName_ResizeNW, MakeTileParams(Tile_ArrowNWSE), hsHalf);
	new ImageCursor(CursorName_ResizeNE, MakeTileParams(Tile_ArrowNESW), hsHalf);
	new ImageCursor(CursorName_ResizeSW, MakeTileParams(Tile_ArrowNESW), hsHalf);
	new ImageCursor(CursorName_ResizeSE, MakeTileParams(Tile_ArrowNWSE), hsHalf);
	new ImageCursor(CursorName_Stop, MakeTileParams(Tile_Stop), hsHalf);
}


//********************************************************************
}//namespace std_cursors


//********************************************************************
}//namespace gui


//********************************************************************
}//namespace vertexware

