

//--------------------------------------------------------------------
#include "GUI\GUI_Tools.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui {


//====================================================================
void __fastcall CalcTextSize(const gui::Font& font,
							const gui::TextMetricsL& tml,
							const std::vector<std::string>& textLines,
							vec2& outTextSize,
							std::vector<float>* pOutWidths)
{
	if(pOutWidths) {
		pOutWidths->reserve(textLines.size());
	}
	float fAbsLineHeight = tml.AbsLineHeight();

	outTextSize.Set(0, tml.textHeight);

	for(uint i = 0; i < textLines.size(); i++) {

		float w = font.CalcTextWidth(tml, textLines[i]);
		if(pOutWidths) (*pOutWidths)[i] = w;
		outTextSize.x = Max(outTextSize.x, w);
	}
	if(textLines.size() > 1) {
		outTextSize.y += (textLines.size() - 1) * fAbsLineHeight;
	}
}


//====================================================================
void __fastcall DrawTextLines(gui::Renderer& gr, const gui::Font& font,
							color4b textColor, const vec2& offsetTopLeft,
							const TextMetricsL& tml,
							const std::vector<std::string>& textLines)
{
	float fAbsLineHeight = tml.AbsLineHeight();

	gui::DrawTextParams  dtp(textColor, offsetTopLeft, tml);

	for(uint i = 0; i < textLines.size(); i++) {

		font.DrawText(gr, textLines[i], dtp);
		dtp.positionTopLeft.y += fAbsLineHeight;
	}
}


//====================================================================
void __fastcall CalcPointsFromSizeAndTopLeft(
						const vec2& size, const vec2& topLeft,
						vec2& topRight, vec2& bottomLeft, vec2& bottomRight)
{
	bottomRight = topLeft + size;
	topRight.Set(bottomRight.x, topLeft.y);
	bottomLeft.Set(topLeft.x, bottomRight.y);
}


//********************************************************************
}//namespace gui


//********************************************************************
}//namespace vertexware


