

//--------------------------------------------------------------------
#include "HIDControllers\KeyboardCodes.h"


//********************************************************************
namespace {


//--------------------------------------------------------------------
using namespace vertexware;


//====================================================================
class KeyNameMap
{
public:
	KeyNameMap();

	__forceinline  const char* operator[](eKey keyCode) const
	{
		if(keyCode < Key_Count) return names[keyCode];
		return "BadKeyCode";
	}

private:
	const char*  names[Key_Count];
}
g_keyNameMap;


//====================================================================
KeyNameMap::KeyNameMap()
{
	// keyboard row 1
	names[Key_Escape] = "Escape";
	names[Key_F1]     = "F1";
	names[Key_F2]     = "F2";
	names[Key_F3]     = "F3";
	names[Key_F4]     = "F4";
	names[Key_F5]     = "F5";
	names[Key_F6]     = "F6";
	names[Key_F7]     = "F7";
	names[Key_F8]     = "F8";
	names[Key_F9]     = "F9";
	names[Key_F10]    = "F10";
	names[Key_F11]    = "F11";
	names[Key_F12]    = "F12";

	// keyboard row 2
	names[Key_GraveAccent] = "GraveAccent";
	names[Key_1]           = "1";
	names[Key_2]           = "2";
	names[Key_3]           = "3";
	names[Key_4]           = "4";
	names[Key_5]           = "5";
	names[Key_6]           = "6";
	names[Key_7]           = "7";
	names[Key_8]           = "8";
	names[Key_9]           = "9";
	names[Key_0]           = "0";
	names[Key_Minus]       = "Minus";
	names[Key_Equals]      = "Equals";
	names[Key_Backspace]   = "Backspace";

	// keyboard row 3
	names[Key_Tab]          = "Tab";
	names[Key_Q]            = "Q";
	names[Key_W]            = "W";
	names[Key_E]            = "E";
	names[Key_R]            = "R";
	names[Key_T]            = "T";
	names[Key_Y]            = "Y";
	names[Key_U]            = "U";
	names[Key_I]            = "I";
	names[Key_O]            = "O";
	names[Key_P]            = "P";
	names[Key_LeftBracket]  = "L-Bracket";
	names[Key_RightBracket] = "R-Bracket";
	names[Key_Enter]        = "Enter";

	// keyboard row 4
	names[Key_CapsLock]   = "CapsLock";
	names[Key_A]          = "A";
	names[Key_S]          = "S";
	names[Key_D]          = "D";
	names[Key_F]          = "F";
	names[Key_G]          = "G";
	names[Key_H]          = "H";
	names[Key_J]          = "J";
	names[Key_K]          = "K";
	names[Key_L]          = "L";
	names[Key_Semicolon]  = "Semicolon";
	names[Key_Apostrophe] = "Apostrophe";
	names[Key_Backslash]  = "Backslash";

	// keyboard row 5
	names[Key_LeftShift]  = "L-Shift";
	names[Key_OEM102]     = "OEM102";
	names[Key_Z]          = "Z";
	names[Key_X]          = "X";
	names[Key_C]          = "C";
	names[Key_V]          = "V";
	names[Key_B]          = "B";
	names[Key_N]          = "N";
	names[Key_M]          = "M";
	names[Key_Comma]      = "Comma";
	names[Key_Period]     = "Period";
	names[Key_Slash]      = "Slash";
	names[Key_RightShift] = "R-Shift";

	// keyboard row 6
	names[Key_LeftControl]  = "L-Control";
	names[Key_LeftWin]      = "L-Win";
	names[Key_LeftAlt]      = "L-Alt";
	names[Key_Space]        = "Space";
	names[Key_RightAlt]     = "R-Alt";
	names[Key_RightWin]     = "R-Win";
	names[Key_AppMenu]      = "AppMenu";
	names[Key_RightControl] = "R-Control";

	// keyboard special 1
	names[Key_PrintScreen] = "PrintScreen";
	names[Key_ScrollLock]  = "ScrollLock";
	names[Key_Pause]       = "Pause";

	// keyboard special 2
	names[Key_Insert]   = "Insert";
	names[Key_Delete]   = "Delete";
	names[Key_Home]     = "Home";
	names[Key_End]      = "End";
	names[Key_PageUp]   = "PageUp";
	names[Key_PageDown] = "PageDown";

	// keyboard special 3
	names[Key_ArrowUp]    = "ArrowUp";
	names[Key_ArrowDown]  = "ArrowDown";
	names[Key_ArrowLeft]  = "ArrowLeft";
	names[Key_ArrowRight] = "ArrowRight";

	// keyboard numpad
	names[Key_NumLock]     = "NumLock";
	names[Key_NumDivide]   = "NumDivide";
	names[Key_NumMultiply] = "NumMultiply";
	names[Key_NumSubtract] = "NumSubtract";
	names[Key_NumAdd]      = "NumAdd";
	names[Key_NumEnter]    = "NumEnter";
	names[Key_NumDecimal]  = "NumDecimal";
	names[Key_Num0]        = "Num0";
	names[Key_Num1]        = "Num1";
	names[Key_Num2]        = "Num2";
	names[Key_Num3]        = "Num3";
	names[Key_Num4]        = "Num4";
	names[Key_Num5]        = "Num5";
	names[Key_Num6]        = "Num6";
	names[Key_Num7]        = "Num7";
	names[Key_Num8]        = "Num8";
	names[Key_Num9]        = "Num9";
}


//********************************************************************
}//namespace


//********************************************************************
namespace vertexware {


//====================================================================
std::string __fastcall GetKeyModifierName(eKeyModifier keyModCode)
{
	switch(keyModCode) {
		case KeyModifier_Shift: return "Shift";
		case KeyModifier_Control: return "Control";
		case KeyModifier_Alt: return "Alt";
	}
	return "";
}


//====================================================================
std::string __fastcall GetKeyName(eKey keyCode)
{
	return g_keyNameMap[keyCode];
}


//********************************************************************
}//namespace vertexware


