

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "private\vw_common_exports.h"


//********************************************************************
namespace vertexware {


//====================================================================
enum eKeyModifier
{
	KeyModifier_Shift = 0,
	KeyModifier_Control,
	KeyModifier_Alt,

	KeyModifier_Count,
	KeyModifier_Force32Bits = 0x7FFFFFFF
};


//====================================================================
VW_COMMON_API
std::string __fastcall GetKeyModifierName(eKeyModifier keyModCode);


//====================================================================
// Keyboard codes use US-layout and key-names.
// WARNING: Codes must start with 0. Code-holes are NOT allowed.
//====================================================================
enum eKey
{
	// keyboard row 1
	Key_Escape = 0,
	Key_F1     = 1,
	Key_F2     = 2,
	Key_F3     = 3,
	Key_F4     = 4,
	Key_F5     = 5,
	Key_F6     = 6,
	Key_F7     = 7,
	Key_F8     = 8,
	Key_F9     = 9,
	Key_F10    = 10,
	Key_F11    = 11,
	Key_F12    = 12,

	// keyboard row 2
	Key_GraveAccent = 13,
	Key_1           = 14,
	Key_2           = 15,
	Key_3           = 16,
	Key_4           = 17,
	Key_5           = 18,
	Key_6           = 19,
	Key_7           = 20,
	Key_8           = 21,
	Key_9           = 22,
	Key_0           = 23,
	Key_Minus       = 24,
	Key_Equals      = 25,
	Key_Backspace   = 26,

	// keyboard row 3
	Key_Tab          = 27,
	Key_Q            = 28,
	Key_W            = 29,
	Key_E            = 30,
	Key_R            = 31,
	Key_T            = 32,
	Key_Y            = 33,
	Key_U            = 34,
	Key_I            = 35,
	Key_O            = 36,
	Key_P            = 37,
	Key_LeftBracket  = 38,
	Key_RightBracket = 39,
	Key_Enter        = 40,

	// keyboard row 4
	Key_CapsLock   = 41,
	Key_A          = 42,
	Key_S          = 43,
	Key_D          = 44,
	Key_F          = 45,
	Key_G          = 46,
	Key_H          = 47,
	Key_J          = 48,
	Key_K          = 49,
	Key_L          = 50,
	Key_Semicolon  = 51,
	Key_Apostrophe = 52,
	Key_Backslash  = 53,

	// keyboard row 5
	Key_LeftShift  = 54,
	Key_OEM102     = 55,
	Key_Z          = 56,
	Key_X          = 57,
	Key_C          = 58,
	Key_V          = 59,
	Key_B          = 60,
	Key_N          = 61,
	Key_M          = 62,
	Key_Comma      = 63,
	Key_Period     = 64,
	Key_Slash      = 65,
	Key_RightShift = 66,

	// keyboard row 6
	Key_LeftControl  = 67,
	Key_LeftWin      = 68,
	Key_LeftAlt      = 69,
	Key_Space        = 70,
	Key_RightAlt     = 71,
	Key_RightWin     = 72,
	Key_AppMenu      = 73,
	Key_RightControl = 74,

	// keyboard special 1
	Key_PrintScreen = 75,
	Key_ScrollLock  = 76,
	Key_Pause       = 77,

	// keyboard special 2
	Key_Insert   = 78,
	Key_Delete   = 79,
	Key_Home     = 80,
	Key_End      = 81,
	Key_PageUp   = 82,
	Key_PageDown = 83,

	// keyboard special 3
	Key_ArrowUp    = 84,
	Key_ArrowDown  = 85,
	Key_ArrowLeft  = 86,
	Key_ArrowRight = 87,

	// keyboard numpad
	Key_NumLock     = 88,
	Key_NumDivide   = 89,
	Key_NumMultiply = 90,
	Key_NumSubtract = 91,
	Key_NumAdd      = 92,
	Key_NumEnter    = 93,
	Key_NumDecimal  = 94,
	Key_Num0        = 95,
	Key_Num1        = 96,
	Key_Num2        = 97,
	Key_Num3        = 98,
	Key_Num4        = 99,
	Key_Num5        = 100,
	Key_Num6        = 101,
	Key_Num7        = 102,
	Key_Num8        = 103,
	Key_Num9        = 104,

	Key_Count,
	Key_Force32Bits = 0x7FFFFFFF
};


//====================================================================
VW_COMMON_API
std::string __fastcall GetKeyName(eKey keyCode);


//********************************************************************
}//namespace vertexware


