

//--------------------------------------------------------------------
#include "HIDControllers\PositionTracker.h"
#include "MathTools\Vector4.h"
#include "BaseTools\String.h"
#include "BaseTools\Stream.h"


//********************************************************************
namespace vertexware {


//====================================================================
PositionTracker::PositionTracker()
{
	ResetInputReadings();

	SetReverseX(false);
	SetReverseY(false);
	SetReverseZ(false);

	SetTransform();

	ReportHardwareCaps(false, false, false, false);
}


//====================================================================
PositionTracker::~PositionTracker()
{
}


//====================================================================
std::string  PositionTracker::GetControllerInterfaceName() const
{
	return VW_INTERFACE_NAME_PositionTracker;
}


//====================================================================
void  PositionTracker::DumpControllerInfo(std::ostream& os, uint tabLevel) const
{
	TabEffector te(tabLevel);
	os << te << "ResetPosition: " << ToString(IsSupportedResetPosition()) << nl;
	os << te << "ResetX.......: " << ToString(IsSupportedResetX()) << nl;
	os << te << "ResetY.......: " << ToString(IsSupportedResetY()) << nl;
	os << te << "ResetZ.......: " << ToString(IsSupportedResetZ()) << nl;
}


//====================================================================
void  PositionTracker::ResetInputReadings()
{
	finalPosition = vec3::ZERO;
	rawPosition = vec3::ZERO;
}


//====================================================================
void  PositionTracker::PreSampleUpdate()
{
}


//====================================================================
void  PositionTracker::PostSampleUpdate()
{
}


//====================================================================
void  PositionTracker::OnBeginAsyncSampleRead()
{
}


//====================================================================
void  PositionTracker::OnEndAsyncSampleRead()
{
}


//====================================================================
bool  PositionTracker::ResetPosition()
{
	if(! bSupportsResetPosition) return false;
	return DoResetPosition();
}


//====================================================================
bool  PositionTracker::ResetX()
{
	if(! bSupportsResetX) return false;
	return DoResetX();
}


//====================================================================
bool  PositionTracker::ResetY()
{
	if(! bSupportsResetY) return false;
	return DoResetY();
}


//====================================================================
bool  PositionTracker::ResetZ()
{
	if(! bSupportsResetZ) return false;
	return DoResetZ();
}


//====================================================================
void  PositionTracker::SetReverseX(bool bReverse)
{
	fReverseX = bReverse ? -1 : 1;
}


//====================================================================
void  PositionTracker::SetReverseY(bool bReverse)
{
	fReverseY = bReverse ? -1 : 1;
}


//====================================================================
void  PositionTracker::SetReverseZ(bool bReverse)
{
	fReverseZ = bReverse ? -1 : 1;
}


//====================================================================
void  PositionTracker::SetTransform(const mat4& _transformMV)
{
	transformMV = _transformMV;
}


//====================================================================
void  PositionTracker::ReportHardwareCaps(
										bool _bSupportsResetPosition,
										bool _bSupportsResetX,
										bool _bSupportsResetY,
										bool _bSupportsResetZ)
{
	bSupportsResetPosition = _bSupportsResetPosition;
	bSupportsResetX = _bSupportsResetX;
	bSupportsResetY = _bSupportsResetY;
	bSupportsResetZ = _bSupportsResetZ;
}


//====================================================================
void  PositionTracker::SetPosition(const vec3& position)
{
	rawPosition = position;

	// calculate final position
	vec4  tPos(
				fReverseX * rawPosition.x,
				fReverseY * rawPosition.y,
				fReverseZ * rawPosition.z,
				1);

	tPos = transformMV * tPos;
	tPos.Normalize();

	finalPosition.Set(tPos.x, tPos.y, tPos.z);
}


//********************************************************************
}//namespace vertexware


