

//--------------------------------------------------------------------
#include "MathTools\MathBase.h"
#include <cfloat>


//====================================================================
#define  CONSTANT_PI          (4.0 * atan(1.0))
#define  CONSTANT_LOG10_OF_2  (log10(2.0))


//********************************************************************
namespace vertexware {


//********************************************************************
namespace math_constants {


//====================================================================
VW_DEFINE_CONSTANT( double, Pi, CONSTANT_PI );
VW_DEFINE_CONSTANT( double, PiOver180, CONSTANT_PI / 180 );
VW_DEFINE_CONSTANT( double, _180OverPi, 180 / CONSTANT_PI );
VW_DEFINE_CONSTANT( double, PlusInfinity, + DBL_MAX );
VW_DEFINE_CONSTANT( double, MinusInfinity, - DBL_MAX );
VW_DEFINE_CONSTANT( double, Log10_Of_2, CONSTANT_LOG10_OF_2 );


//********************************************************************
}//namespace math_constants


//====================================================================
uint __fastcall GEqualPowerOf2(uint val)
{
	// special cases
	const uint max = 0x80000000;
	if(val >= max) return max;

	uint ref = 1;
	while(ref < val) ref <<= 1; // multiply by 2
	return ref;
}


//====================================================================
uint __fastcall LEqualPowerOf2(uint val)
{
	// special cases
	const uint max = 0x80000000;
	if(val >= max) return max;

	uint ref = 1;
	while(ref < val) ref <<= 1; // multiply by 2

	if(ref == val) return ref;
	return (ref >> 1); // divide by 2 if greater
}


//====================================================================
uint __fastcall ClosestPowerOf2(uint val)
{
	uint lequal = LEqualPowerOf2(val);
	uint gequal = GEqualPowerOf2(val);

	// return value with smallest distance
	return ((val - lequal) < (gequal - val)) ? lequal : gequal;
}


//********************************************************************
}//namespace vertexware


