

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "private\vw_common_exports.h"
#include <cmath>


//********************************************************************
namespace vertexware {


//********************************************************************
namespace math_constants {


//====================================================================
VW_COMMON_API  VW_DECLARE_CONSTANT( double, Pi );
VW_COMMON_API  VW_DECLARE_CONSTANT( double, PiOver180 );
VW_COMMON_API  VW_DECLARE_CONSTANT( double, _180OverPi );
VW_COMMON_API  VW_DECLARE_CONSTANT( double, PlusInfinity );
VW_COMMON_API  VW_DECLARE_CONSTANT( double, MinusInfinity );
VW_COMMON_API  VW_DECLARE_CONSTANT( double, Log10_Of_2 );


//********************************************************************
}//namespace math_constants


//====================================================================
// Radians / degrees conversions.
//====================================================================
template<typename T>
__forceinline  T __fastcall RadToDeg(T rad)
{
	return  rad * T(math_constants::_180OverPi);
}


//====================================================================
template<typename T>
__forceinline  T __fastcall DegToRad(T deg)
{
	return  deg * T(math_constants::PiOver180);
}


//====================================================================
// Various.
//====================================================================
template<typename T>
__forceinline  T __fastcall Ceiling(T val)
{
	return static_cast<T>(ceil(val));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall Floor(T val)
{
	return static_cast<T>(floor(val));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall Exp(T exponent)
{
	return static_cast<T>(exp(exponent));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall Pow(T base, T exponent)
{
	return static_cast<T>(pow(base, exponent));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall LogE(T val)
{
	return static_cast<T>(log(val));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall Log10(T val)
{
	return static_cast<T>(log10(val));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall Log2(T val)
{
	return static_cast<T>(log10(val) / math_constants::Log10_Of_2);
}


//====================================================================
template<typename T>
__forceinline  T __fastcall LogN(uint logBase, T val)
{
	return static_cast<T>(log10(val) / log10(logBase));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall CosDeg(T deg)
{
	return static_cast<T>(cos(DegToRad<double>(deg)));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall SinDeg(T deg)
{
	return static_cast<T>(sin(DegToRad<double>(deg)));
}


//====================================================================
template<typename T>
__forceinline  void __fastcall SinCosDeg(T deg, T& outSin, T& outCos)
{
	double rad = DegToRad<double>(deg);
	outSin = static_cast<T>(sin(rad));
	outCos = static_cast<T>(cos(rad));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall TanDeg(T deg)
{
	return static_cast<T>(tan(DegToRad<double>(deg)));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall ArcCosDeg(T cos)
{
	return static_cast<T>(RadToDeg(acos(cos)));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall ArcSinDeg(T sin)
{
	return static_cast<T>(RadToDeg(asin(sin)));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall Atan2Deg(T numerator, T denominator)
{
	return static_cast<T>(RadToDeg(atan2(numerator, denominator)));
}


//====================================================================
template<typename T>
__forceinline  T __fastcall Sqr(T val)
{
	return static_cast<T>(val * val);
}


//====================================================================
template<typename T>
__forceinline  T __fastcall Sqrt(T val)
{
	return static_cast<T>(sqrt(val));
}


//====================================================================
template<typename T>
__forceinline T __fastcall UnitRandom()
{
	return T(double(rand()) / double(RAND_MAX));
}


//====================================================================
template<typename T>
__forceinline T __fastcall SymmetricRandom()
{
	return T(double(rand()) / double(RAND_MAX) * 2.0 - 1.0);
}


//====================================================================
// Power-of-2 helpers.
//====================================================================
VW_COMMON_API
uint __fastcall GEqualPowerOf2(uint val);


//====================================================================
VW_COMMON_API
uint __fastcall LEqualPowerOf2(uint val);


//====================================================================
VW_COMMON_API
uint __fastcall ClosestPowerOf2(uint val);


//====================================================================
VW_COMMON_API
__forceinline bool __fastcall IsPowerOf2(uint val) {
	return (GEqualPowerOf2(val) == val);
}


//********************************************************************
}//namespace vertexware


