

//--------------------------------------------------------------------
#include "MathTools\Matrix22.h"
#include "BaseTools\Serializers.h"


//********************************************************************
namespace vertexware {


//====================================================================
const mat2  mat2::ZERO     = mat2(0, 0, 0, 0);
const mat2  mat2::IDENTITY = mat2(1, 0, 0, 1);


//====================================================================
mat2&  mat2::Mul(const mat2& lhs, const mat2& rhs)
{
	m00 = Dot00(lhs, rhs);
	m01 = Dot01(lhs, rhs);
	m10 = Dot10(lhs, rhs);
	m11 = Dot11(lhs, rhs);
	return *this;
}


//====================================================================
mat2&  mat2::Inverse(const mat2& src, bool* pFailed)
{
	// NOTE: this/src aliasing allowed
	float  det = src.Determinant();
	if(Abs(det) >= math_constants::MatrixInverseDefaultDeterminantTolerance) {
		det = 1 / det;
		this->Set(src.m11 * det, -src.m01 * det, -src.m10 * det, src.m00 * det);
		SafeAssign(pFailed, false);
	}
	else {
		*this = mat2::IDENTITY;
		SafeAssign(pFailed, true);
	}
	return *this;
}


//====================================================================
std::ostream& __fastcall operator<=(std::ostream &os, const mat2& m)
{
	for(uint i = 0; i < 4; i++) {
		os <= m.m[i];
	}
	return os;
}


//====================================================================
std::istream& __fastcall operator>=(std::istream &is, mat2& m)
{
	for(uint i = 0; i < 4; i++) {
		is >= m.m[i];
	}
	return is;
}


//********************************************************************
}//namespace vertexware


