

//--------------------------------------------------------------------
#include "MathTools\Matrix33.h"
#include "BaseTools\Serializers.h"


//********************************************************************
namespace vertexware {


//====================================================================
const mat3  mat3::ZERO     = mat3(0, 0, 0, 0, 0, 0, 0, 0, 0);
const mat3  mat3::IDENTITY = mat3(1, 0, 0, 0, 1, 0, 0, 0, 1);


//====================================================================
mat3&  mat3::Mul(const mat3& lhs, const mat3& rhs)
{
	m00 = Dot00(lhs, rhs);
	m01 = Dot01(lhs, rhs);
	m02 = Dot02(lhs, rhs);
	m10 = Dot10(lhs, rhs);
	m11 = Dot11(lhs, rhs);
	m12 = Dot12(lhs, rhs);
	m20 = Dot20(lhs, rhs);
	m21 = Dot21(lhs, rhs);
	m22 = Dot22(lhs, rhs);
	return *this;
}


//====================================================================
mat3&  mat3::Inverse(const mat3& src, bool* pFailed)
{
	// transposed cofactors
	m00 = src.m11 * src.m22 - src.m12 * src.m21; //A00
	m01 = src.m02 * src.m21 - src.m01 * src.m22; //A10
	m02 = src.m01 * src.m12 - src.m02 * src.m11; //A20

	m10 = src.m12 * src.m20 - src.m10 * src.m22; //A01
	m11 = src.m00 * src.m22 - src.m02 * src.m20; //A11
	m12 = src.m02 * src.m10 - src.m00 * src.m12; //A21

	m20 = src.m10 * src.m21 - src.m11 * src.m20; //A02
	m21 = src.m01 * src.m20 - src.m00 * src.m21; //A12
	m22 = src.m00 * src.m11 - src.m01 * src.m10; //A22

	// compute determinant using cofactors
	// det = a00 * A00 + a01 * A01 + a02 * A02
	float  det = (src.m00 * m00) + (src.m01 * m10) + (src.m02 * m20);

	if(Abs(det) >= math_constants::MatrixInverseDefaultDeterminantTolerance) {
		// divide by determinant
		*this /= det;
		SafeAssign(pFailed, false);
	}
	else {
		*this = mat3::IDENTITY;
		SafeAssign(pFailed, true);
	}
	return *this;
}


//====================================================================
bool  mat3::operator==(const mat3& rhs) const
{
	return
	(m00 == rhs.m00)  &&
	(m01 == rhs.m01)  &&
	(m02 == rhs.m02)  &&
	(m10 == rhs.m10)  &&
	(m11 == rhs.m11)  &&
	(m12 == rhs.m12)  &&
	(m20 == rhs.m20)  &&
	(m21 == rhs.m21)  &&
	(m22 == rhs.m22);
}


//====================================================================
std::ostream& __fastcall operator<=(std::ostream &os, const mat3& m)
{
	for(uint i = 0; i < 9; i++) {
		os <= m.m[i];
	}
	return os;
}


//====================================================================
std::istream& __fastcall operator>=(std::istream &is, mat3& m)
{
	for(uint i = 0; i < 9; i++) {
		is >= m.m[i];
	}
	return is;
}


//********************************************************************
}//namespace vertexware

