

//--------------------------------------------------------------------
#include "MathTools\Vector2.h"
#include "BaseTools\Serializers.h"


//********************************************************************
namespace vertexware {


//====================================================================
const vec2  vec2::ZERO     = vec2(0, 0);
const vec2  vec2::ONE      = vec2(1, 1);
const vec2  vec2::INFINITY = vec2(math_constants::PlusInfinity,
											math_constants::PlusInfinity);
const vec2  vec2::UNIT_X   = vec2(1, 0);
const vec2  vec2::UNIT_Y   = vec2(0, 1);


//====================================================================
bool  vec2::IsZero(float tolerance) const
{
	return
	InRange(x, -tolerance, tolerance)  &&
	InRange(y, -tolerance, tolerance);
}


//====================================================================
bool  vec2::HasZeroComponent(float tolerance) const
{
	return
	InRange(x, -tolerance, tolerance)  ||
	InRange(y, -tolerance, tolerance);
}


//====================================================================
void __fastcall OrthoNormalize(vec2& v0, vec2& v1)
{
	v0.Unitize();

	v1 -= v1.ProjectionTo(v0);
	v1.Unitize();
}


//====================================================================
std::ostream& __fastcall operator<=(std::ostream &os, const vec2& v)
{
	os <= v.x <= v.y;
	return os;
}


//====================================================================
std::istream& __fastcall operator>=(std::istream &is, vec2& v)
{
	is >= v.x >= v.y;
	return is;
}


//********************************************************************
}//namespace vertexware


