

//--------------------------------------------------------------------
#include "MathTools\Vector4.h"
#include "BaseTools\Serializers.h"


//********************************************************************
namespace vertexware {


//====================================================================
const vec4  vec4::ZERO     = vec4(0, 0, 0, 0);
const vec4  vec4::ONE      = vec4(1, 1, 1, 1);
const vec4  vec4::INFINITY = vec4(math_constants::PlusInfinity,
											math_constants::PlusInfinity,
											math_constants::PlusInfinity,
											math_constants::PlusInfinity);
const vec4  vec4::UNIT_X   = vec4(1, 0, 0, 0);
const vec4  vec4::UNIT_Y   = vec4(0, 1, 0, 0);
const vec4  vec4::UNIT_Z   = vec4(0, 0, 1, 0);
const vec4  vec4::UNIT_W   = vec4(0, 0, 0, 0);


//====================================================================
bool  vec4::IsZero(float tolerance) const
{
	return
	InRange(x, -tolerance, tolerance)  &&
	InRange(y, -tolerance, tolerance)  &&
	InRange(z, -tolerance, tolerance)  &&
	InRange(w, -tolerance, tolerance);
}


//====================================================================
bool  vec4::HasZeroComponent(float tolerance) const
{
	return
	InRange(x, -tolerance, tolerance)  ||
	InRange(y, -tolerance, tolerance)  ||
	InRange(z, -tolerance, tolerance)  ||
	InRange(w, -tolerance, tolerance);
}


//====================================================================
std::ostream& __fastcall operator<=(std::ostream &os, const vec4& v)
{
	os <= v.x <= v.y <= v.z <= v.w;
	return os;
}


//====================================================================
std::istream& __fastcall operator>=(std::istream &is, vec4& v)
{
	is >= v.x >= v.y >= v.z >= v.w;
	return is;
}


//********************************************************************
}//namespace vertexware

