

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "BaseTools\NoCopy.h"
#include "private\vw_common_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace video_renderer_objects {


//====================================================================
enum eResourceType
{
	ResourceType_Texture1D = 0,
	ResourceType_Texture2D,
	ResourceType_Texture3D,

	ResourceType_User        = 0x10000000,
	ResourceType_Force32Bits = 0x7FFFFFFF
};


//====================================================================
class VW_COMMON_API Resource : public NoCopy
{
public:
	enum {
		LowestPriority  = 0x00000000,
		DefaultPriority = 0x7FFFFFFF,
		HighestPriority = 0xFFFFFFFF
	};


public:
	virtual ~Resource();

	eResourceType  GetResourceType() const { return resourceType; }

	void  SetPriority(uint newPriority = Resource::DefaultPriority);
	uint  GetPriority() const { return priority; }

	void  SetUserData(uint _userData) { userData = _userData; }
	uint  GetUserData() const { return userData; }


protected:
	Resource(eResourceType resourceType);

	virtual  void  UpdatePriority() = 0;


private:
	eResourceType  resourceType;
	uint           priority;
	uint           userData;
};


//********************************************************************
}//namespace video_renderer_objects


//********************************************************************
}//namespace vertexware


