

//--------------------------------------------------------------------
#include "D3D8Tools\States.h"
#include "D3D8Tools\ErrorHandling.h"
#include "BaseTools\Exception.h"
#include <string>


//********************************************************************
namespace vertexware {


//********************************************************************
namespace d3d8_tools {


//====================================================================
StateBlockCompileSentry::StateBlockCompileSentry(StateBlock& _sb) :
	sb(_sb)
{
	sb.Delete();
	HRESULT hr = sb.device.BeginStateBlock();
	D3DErrorCheckE(hr, "IDirect3DDevice8::BeginStateBlock");
}


//====================================================================
StateBlockCompileSentry::~StateBlockCompileSentry()
{
	HRESULT hr = sb.device.EndStateBlock(&sb.token);
	D3DErrorCheckE(hr, "IDirect3DDevice8::EndStateBlock");
}


//====================================================================
StateBlock::StateBlock(IDirect3DDevice8& _device) :
	device(_device)
{
	token = 0;
}


//====================================================================
StateBlock::StateBlock(IDirect3DDevice8& _device, D3DSTATEBLOCKTYPE blockType) :
	device(_device)
{
	token = 0;
	HRESULT hr = CompilePredefined(blockType);
	D3DErrorCheckE(hr, "IDirect3DDevice8::CreateStateBlock");
}


//====================================================================
StateBlock::~StateBlock()
{
	Delete();
}


//====================================================================
HRESULT  StateBlock::CompilePredefined(D3DSTATEBLOCKTYPE blockType)
{
	Delete();
	return device.CreateStateBlock(blockType, &token);
}


//====================================================================
void  StateBlock::Delete()
{
	if(token) {
		device.DeleteStateBlock(token);
		token = 0;
	}
}


//********************************************************************
}//namespace d3d8_tools


//********************************************************************
}//namespace vertexware


