

//--------------------------------------------------------------------
#include "D3D8Tools\Transforms.h"
#include "D3D8Tools\States.h"
#include "MathTools\MathBase.h"
#include "MathTools\TransformTools.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace d3d8_tools {


//====================================================================
HRESULT __fastcall Viewport(IDirect3DDevice8& device,
						uint x, uint y, uint width, uint height, float minZ, float maxZ)
{
	D3DVIEWPORT8 vp;

	vp.X = x;
	vp.Y = y;
	vp.Width = width;
	vp.Height = height;
	vp.MinZ = minZ;
	vp.MaxZ = maxZ;

	return device.SetViewport(&vp);
}


//====================================================================
HRESULT __fastcall Perspective(IDirect3DDevice8& device,
							float fovYDeg, float aspectWToH, float zNear, float zFar)
{
	float halfH = zNear * TanDeg(fovYDeg * 0.5f);
	float halfW = halfH * aspectWToH;

	mat4  mat;
	float ZfMinZn = zFar - zNear;

	float oneOverZfMinZn = ZfMinZn ? (1 / ZfMinZn) : 0;
	float oneOverHalfW   = halfW ? (1 / halfW) : 0;
	float oneOverHalfH   = halfH ? (1 / halfH) : 0;

	mat.Set(
		zNear * oneOverHalfW,   0,    0,               0,
		0,   zNear * oneOverHalfH,    0,               0,
		0,   0,   zFar * oneOverZfMinZn,               1,
		0,   0,      - zNear * zFar * oneOverZfMinZn,  0);

	return device.SetTransform(D3DTS_PROJECTION, ReinterpretCast(&mat));
}


//====================================================================
HRESULT __fastcall View(IDirect3DDevice8& device,
					const vec3& rotation,
					const vec3& position)
{
	mat4  mT, mY, mX, mZ;

	GenerateTranslateVM(mT, -position.x, -position.y, -position.z);
	GenerateRotateVM(mY, +rotation.y, Axis_Y); //L2R * V2M = -1 * -1 = +1
	GenerateRotateVM(mX, -rotation.x, Axis_X); //L2R * V2M = +1 * -1 = -1
	GenerateRotateVM(mZ, +rotation.z, Axis_Z); //L2R * V2M = -1 * -1 = +1

	mat4  mFinal = mT * mY * mX * mZ;

	return device.SetTransform(D3DTS_VIEW, ReinterpretCast(&mFinal));
}


//********************************************************************
}//namespace d3d8_tools


//********************************************************************
}//namespace vertexware


