

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Renderer.h"
#include "Framework\VideoSubsystemDirect3D8.h"
#include "Framework\VideoRendererDirect3D8.h"
#include "BaseTools\Memory.h"
#include "MathTools\Matrix44.h"
#include "D3D8Tools\States.h"
#include <stack>


//********************************************************************
namespace vertexware {


//====================================================================
class VW_DIRECT3D8_API GUIRendererDirect3D8 : public gui::Renderer
{
public:
	GUIRendererDirect3D8(VideoRendererDirect3D8& vrParent);
	virtual ~GUIRendererDirect3D8();


	//------------------------------
	// gui::Renderer implementation.
	//------------------------------
	virtual  void  GetRendererParams(RendererParams& rp) const;

	virtual  bool  SetViewport(const ViewportParams& vp);
	virtual  void  ClearBuffers(const ClearParams& cp);

	virtual  void  SetPixelParams(const PixelParams& pp);
	virtual  void  GetPixelParams(PixelParams& pp) const;

	virtual  void  SetBaseDepthParams(const DepthParams& dp);
	virtual  void  GetBaseDepthParams(DepthParams& dp) const;

	virtual  video_renderer_objects::Texture2D*  CreateResourceTexture2D(
					eSampleFormat sampleFormat, bool bMultiLevel, uint height, uint width);

	virtual  bool  IsInsideBeginEnd() { return bInsideBeginEnd; }
	virtual  void  Begin();
	virtual  void  End();

	virtual  bool  IsInsideBeginEndInterleaved() { return bInsideBeginEndInterleaved; }
	virtual  void  BeginInterleaved();
	virtual  void  EndInterleaved();

	virtual  bool  QueryCreatesManagedResourcesOnly();

	virtual  void  SetDepthParams(const DepthParams& dp);
	virtual  void  GetDepthParams(DepthParams& dp) const;

	virtual  void  SetTransform(const mat4& m);
	virtual  void  GetTransform(mat4& m) const;

	virtual  void  DrawPrimitive(const gui::VertexBufferC& vertexBuffer,
								gui::ePrimitiveType primitiveType,
								uint numPrimitives,
								uint startVertex);

	virtual  void  DrawPrimitive(const gui::Texture& texture,
								const gui::VertexBufferCT& vertexBuffer,
								gui::ePrimitiveType primitiveType,
								uint numPrimitives,
								uint startVertex);

	virtual  void  DrawText(const gui::FontTexture& fontTexture,
							uint textLen, const char* pText,
							const gui::DrawTextParams& dtp);


protected:
	//-------------------------------
	// Video-subsystem notifications.
	//-------------------------------
	virtual  void  ConnectVideoSubsystem(VideoSubsystem& vs);
	virtual  void  DisconnectVideoSubsystem();

	virtual  void  CreateVideoResources(bool bPostReset);
	virtual  void  ReleaseVideoResources(bool bPreReset);

	virtual  void  OnWindowResize(uint width, uint height);


private:
	//---------------------------------------
	// Application memory-pool vertex buffer.
	//---------------------------------------
	class AppPoolVertexBuffer : public AdaptiveBuffer<gui::VertexCT>
	{
	public:
		typedef  ElementType  VertexType;

	public:
		AppPoolVertexBuffer(uint minSize, uint minStableCycles) :
			AdaptiveBuffer<gui::VertexCT>(minSize, minStableCycles)
		{ }
	};


private:
	void  CompileStateBlock_BeginEnd(AutoPtr<d3d8_tools::StateBlock>& apSB);

	void  Send_DepthParams(const DepthParams& dp);
	void  Send_EnableAlphaTest(bool bEnable);
	void  Send_DisableTexturing();
	void  Send_BindTextureAndEnableTexturing(const gui::Texture& texture);

	void  ConstructInternalTransform();

	__forceinline  void  Send_Transform() { pdev->SetTransform(D3DTS_WORLD, d3d8_tools::ReinterpretCast(&(matTransform * matInternalTransform))); }

	// returns 0 on error
	static D3DPRIMITIVETYPE __fastcall TranslatePrimitiveParams(gui::ePrimitiveType pt,
												uint numPrimitives, uint& outVertexCount);

	__forceinline  void  InvalidateLastTexParams()
	{
		pLastBoundTexture = 0;
		lastTexSampleFormat = SampleFormat_Invalid;
		lastTexMipFilter = gui::Texture::Filter_Invalid;
	}


private:
	VideoSubsystemDirect3D8*  pD3DVideoSubsystem;
	IDirect3DDevice8*         pdev;

	PixelParams  pixelParams;
	DepthParams  depthParamsBase;
	DepthParams  depthParams;
	mat4         matInternalTransform;
	mat4         matTransform;

	bool  bInsideBeginEnd;
	bool  bInsideBeginEndInterleaved;
	bool  bTexturingEnabled;
	bool  bAlphaTestEnabled;

	const gui::Texture*    pLastBoundTexture;
	eSampleFormat          lastTexSampleFormat;
	gui::Texture::eFilter  lastTexMipFilter;

	AppPoolVertexBuffer  appPoolVertexBuffer;

	AutoPtr<d3d8_tools::StateBlock>  apSB_Begin;
	AutoPtr<d3d8_tools::StateBlock>  apSB_End;

	// device caps
	uint  numTextureStages;
	bool  bSupportsWBuffering;
	bool  bSupportsMipFilter;
};


//********************************************************************
}//namespace vertexware


