

//--------------------------------------------------------------------
#include "GUI_CS0\AboutDialog.h"
#include "Framework\Application.h"
#include "BaseTools\LibraryInfo.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
AboutDialog::GeomParams::GeomParams()
{
	logoSize = vec2(60, 60);
	infoTextHeight = 16;
	infoLineHeight = 1.25;
	libraryInfoTextHeight = 14;
}


//====================================================================
AboutDialog::AboutDialog(const std::string& _title)
	:
	Dialog(_title)
{
	// Create components
	pLayouter = new gui::layouters::RowLayouter(this);

	pLabelInfo = new Label(pLayouter, this, CID_Info, "");
	pLogoSpinner = new LogoSpinner(pLayouter, this, CID_Logo);

	pLabelLibraryInfo = new Label(pLayouterStdDialogControls, this, CID_LibraryInfo, "powered by " + library_info::GetLibraryName());
	pLabelLibraryInfo->SetToolTipText("Keep off.\nException inside.");

	SetDialogSubcomponentLayout(false, true, false, true);
	SetDialogSubcomponentToolTipText("", "", "");
	SetAboutDialogSubcomponentLayout();
	SetAboutDialogSubcomponentText();
	SetAboutDialogSubcomponentToolTipText();
	SetAboutDialogShaderNames();

	SetGeomParams();
	SetSize(340, 200);

	// Specify layout
	using namespace gui::layouters;
	RowLayouter& l = *pLayouter;

	l.Clear();
	l.AutoHeightRow().Target(pLabelInfo, SF_Target, 1, AF_Near, 0, AF_Near).AutoSpacer();
	l.LastRow().Target(pLogoSpinner, SF_Target, 1, AF_Far, 0, AF_Near);

	SetWorkAreaLayouter(pLayouter);
}


//====================================================================
void  AboutDialog::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;

	LogoSpinner::GeomParams  gpLogo;
	gpLogo.preferredSize = geomParams.logoSize;
	pLogoSpinner->SetGeomParams(gpLogo);

	Label::GeomParams  gpLabel;
	gpLabel.textHeight = geomParams.libraryInfoTextHeight;
	gpLabel.minSize = vec2::ZERO;
	pLabelLibraryInfo->SetGeomParams(gpLabel);

	gpLabel.textHeight = geomParams.infoTextHeight;
	gpLabel.lineHeight = geomParams.infoLineHeight;
	pLabelInfo->SetGeomParams(gpLabel);

	ReportDirtyGeometry();
}


//====================================================================
void  AboutDialog::SetAboutDialogShaderNames(
							const std::string& logoSpinner,
							const std::string& labelInfo,
							const std::string& labelLibraryInfo)
{
	pLogoSpinner->SetShaderName(logoSpinner);
	pLabelInfo->SetShaderName(labelInfo);
	pLabelLibraryInfo->SetShaderName(labelLibraryInfo);
}


//====================================================================
void  AboutDialog::SetAboutDialogSubcomponentLayout(bool bShowLogo)
{
	pLogoSpinner->SetVisible(bShowLogo);
}


//====================================================================
void  AboutDialog::SetAboutDialogSubcomponentText(
							const std::string& sInfoText,
							const gui::TileParams& tileParamsLogo)
{
	pLabelInfo->SetText(sInfoText);
	pLogoSpinner->SetTileParams(tileParamsLogo);
}


//====================================================================
void  AboutDialog::SetAboutDialogSubcomponentToolTipText(const std::string& sLogo)
{
	pLogoSpinner->SetToolTipText(sLogo);
}


//====================================================================
void  AboutDialog::OnStdDialogControlsLayouterUpdate()
{
	using namespace gui::layouters;
	RowLayouter& l = *pLayouterStdDialogControls;

	l.LastRow().Prepend(Cell(pLabelLibraryInfo));
}


//====================================================================
bool  AboutDialog::OnCommand(uint commandID)
{
	switch(commandID) {
		case CID_LibraryInfo:
			OnCommandLibraryInfo();
			return true;

		case CID_Info:
			OnCommandInfo();
			return true;

		case CID_Logo:
			OnCommandLogo();
			return true;
	}
	return Dialog::OnCommand(commandID);
}


//====================================================================
void  AboutDialog::OnCommandLibraryInfo()
{
	VW_THROW(Exception, "Forced exception.");
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


