

//--------------------------------------------------------------------
#include "GUI_CS0\Buttons.h"
#include "GUI_CS0\ShaderNames.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_Tools.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
TextButton::GeomParams::GeomParams()
{
	textHeight = 18;
	lineHeight = 1;
	minSize = vec2(0, 24);
	padding = 8;
}


//====================================================================
TextButton::TextButton(gui::Component* _parent,
					gui::MessageHandler* _commandTarget,
					uint _commandID,
					eStyle _style,
					uint _groupID,
					const std::string& _text)
	:
	Button(_parent, _commandTarget, _commandID, _style, _groupID)
{
	SetGeomParams();
	SetText(_text);

	switch(_style) {
		case Style_Press:
			SetShaderName(DefaultShaderName(Shader_TextButton_Press));
			break;
		case Style_Push:
			SetShaderName(DefaultShaderName(Shader_TextButton_Push));
			break;
		default:
			SetShaderName(DefaultShaderName(Shader_TextButton_Toggle));
			break;
	}
}


//====================================================================
void  TextButton::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;
	ReportDirtyGeometry();
}


//====================================================================
void  TextButton::SetText(const std::string& _text)
{
	text = _text;
	BreakIntoLines(text, textLines);
	textLineWidths.resize(textLines.size());
	ReportDirtyGeometry();
}


//====================================================================
void  TextButton::RecalcSize()
{
	gui::ConstShaderPtr<TextButton::Shader> pShader = GetBaseShader();
	if(! pShader) return;

	// calc & cache text size
	const gui::Font* pFont = pShader->GetFont();
	if(! pFont) return;

	gui::CalcTextSize(*pFont,
				pShader->GetTextMetricsL(geomParams.textHeight, geomParams.lineHeight),
				textLines, textSize, &textLineWidths);

	// update size depending on minimum-specified-size & text-size
	SetSize(ComponentMax(geomParams.minSize, textSize + vec2(2 * geomParams.padding, 0)));
}


//====================================================================
IconButton::GeomParams::GeomParams()
{
	preferredSize = vec2(36, 36);
	iconSize = vec2(28, 28);
}


//====================================================================
IconButton::IconButton(gui::Component* _parent,
					gui::MessageHandler* _commandTarget,
					uint _commandID,
					eStyle _style,
					uint _groupID,
					const gui::TileParams& _tp)
	:
	Button(_parent, _commandTarget, _commandID, _style, _groupID)
{
	pTileTexture = 0;

	SetGeomParams();
	SetTileParams(_tp);

	switch(_style) {
		case Style_Press:
			SetShaderName(DefaultShaderName(Shader_IconButton_Press));
			break;
		case Style_Push:
			SetShaderName(DefaultShaderName(Shader_IconButton_Push));
			break;
		default:
			SetShaderName(DefaultShaderName(Shader_IconButton_Toggle));
			break;
	}
}


//====================================================================
void  IconButton::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;
	ReportDirtyGeometry();
}


//====================================================================
void  IconButton::SetTileParams(const gui::TileParams& tp)
{
	tileParams = tp;
	RequestTileTextureRecache();
}


//====================================================================
bool  IconButton::CacheTileTextureHandles(gui::Manager& gm)
{
	if(tileParams.texName.empty()) {
		pTileTexture = 0;
		return true;
	}
	pTileTexture = gm.GetTileTexture(tileParams.texName);
	return (pTileTexture != 0);
}


//====================================================================
void  IconButton::PostCacheTileTextureHandles()
{
	if(pTileTexture) {
		pTileTexture->GetTileCoords(tileParams.texRow, tileParams.texCol, rTile);
	}
	rTile.SetTexTransform(tileParams.texTransform);
}


//====================================================================
void  IconButton::RecalcSize()
{
	SetSize(geomParams.preferredSize);
}


//====================================================================
void  IconButton::UpdateGeometryCache()
{
	rTile.Set((GetSize() - geomParams.iconSize) * 0.5f, geomParams.iconSize);
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


