

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_BehaviorBases.h"
#include "private\vw_gui_cs0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
class VW_GUI_CS0_API TextButton : public gui::Button
{
public:
	VW_GUI_COMPONENT_DECLARE_SHADER_DLL(VW_GUI_CS0_API, Shader);
	VW_GUI_COMPONENT_DECLARE_CHECK_SHADER(Shader);


public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		float  textHeight;
		float  lineHeight;
		float  padding;
		vec2   minSize;

		GeomParams();
	};


public:
	//----------------------
	// TextButton interface.
	//----------------------
	TextButton(gui::Component* parent = 0,
			gui::MessageHandler* commandTarget = 0,
			uint commandID = 0,
			eStyle style = Style_Push,
			uint groupID = 0,
			const std::string& text = "Button");

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void                SetText(const std::string& text = "Button");
	const std::string&  GetText() const { return text; }

	//-----------------------------------------------
	// Additional methods intended for use by shader.
	//-----------------------------------------------
	const std::vector<std::string>  GetTextLines() const { return textLines; }
	uint                            GetNumTextLines() const { return textLines.size(); }
	const std::string&              GetTextLine(uint i) const { return textLines[i]; }
	float                           GetTextLineWidth(uint i) const { return textLineWidths[i]; }
	const vec2&                     GetTextSize() const { return textSize; }


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  RecalcSize();


protected:
	//------------
	// Properties.
	//------------
	GeomParams   geomParams;
	std::string  text;

	//-------
	// Cache.
	//-------
	std::vector<std::string>  textLines;
	std::vector<float>        textLineWidths;
	vec2                      textSize;
};


//====================================================================
class VW_GUI_CS0_API IconButton : public gui::Button
{
	VW_GUI_COMPONENT_DECLARE_SHADER_DLL(VW_GUI_CS0_API, Shader);
	VW_GUI_COMPONENT_DECLARE_CHECK_SHADER(Shader);


public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		vec2  preferredSize;
		vec2  iconSize;

		GeomParams();
	};


public:
	//----------------------
	// IconButton interface.
	//----------------------
	IconButton(gui::Component* parent = 0,
				gui::MessageHandler* commandTarget = 0,
				uint commandID = 0,
				eStyle style = Style_Push,
				uint groupID = 0,
				const gui::TileParams& tp = gui::TileParams());

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void                    SetTileParams(const gui::TileParams& tp = gui::TileParams());
	const gui::TileParams&  GetTileParams() const { return tileParams; }

	//-----------------------------------------------
	// Additional methods intended for use by shader.
	//-----------------------------------------------
	const gui::TileTexture*  GetTileTexture() const { return pTileTexture; }

	// Returned rectangle is already filled with geometry & texture coords,
	// can be used by shader as drawing cache
	gui::RectangleCT&  GetTileRectangle() const { return rTile; }


protected:
	//-----------------------------
	// VideoResourceUser overrides.
	//-----------------------------
	virtual  bool  CacheTileTextureHandles(gui::Manager& gm);
	virtual  void  PostCacheTileTextureHandles();

	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  RecalcSize();
	virtual  void  UpdateGeometryCache();


protected:
	//------------
	// Properties.
	//------------
	GeomParams       geomParams;
	gui::TileParams  tileParams;

	//-------
	// Cache.
	//-------
	const gui::TileTexture*   pTileTexture;
	mutable gui::RectangleCT  rTile;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


