

//--------------------------------------------------------------------
#include "GUI_CS0\Clock.h"
#include "GUI_CS0\ShaderNames.h"
#include "GUI\GUI_Manager.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
Clock::Clock(gui::Component* _parent,
			gui::MessageHandler* _commandTarget,
			uint _commandID)
	:
	Label(_parent, _commandTarget, _commandID, "00:00:00")
{
	bShowTime = true;
	accumTime = 0;

	SetDateTimeSeparators();
	UpdateTextFromSystemClock();
	SetToolTipText("RMB time/date");
	SetShaderName(DefaultShaderName(Shader_Overlay_Label_Static));

	PropagateChangedBranchVisible(IsVisible());
}


//====================================================================
void  Clock::SetDateTimeSeparators(const time::DateTimeSeparators& _dts)
{
	dateTimeSeparators = _dts;
}


//====================================================================
bool  Clock::OnMouseButton(bool bDown, eMouseButton buttonCode, bool bDoubleClick, uint repeatCount, const vec2& mousePos)
{
	// switch between time/date display
	if(bDown  &&  buttonCode == MouseButton_Right  &&  ! repeatCount) {
		Flip(bShowTime);
		accumTime = 0;
		UpdateTextFromSystemClock();
		return true;
	}
	return Label::OnMouseButton(bDown, buttonCode, bDoubleClick, repeatCount, mousePos);
}


//====================================================================
void  Clock::OnChangedBranchVisible(bool bBranchVisible)
{
	gui::Manager::Instance().TimeNotifyRegister(*this, bBranchVisible && IsVisible());
}


//====================================================================
void  Clock::OnChangedVisible()
{
	gui::Manager::Instance().TimeNotifyRegister(*this, IsBranchVisible());
}


//====================================================================
void  Clock::OnTimeDelta(double secs)
{
	accumTime += secs;
	if(accumTime >= 1.0f) {
		accumTime -= 1.0f;

		UpdateTextFromSystemClock();
	}
}


//====================================================================
void  Clock::UpdateTextFromSystemClock()
{
	std::string s;

	if(bShowTime) {
		time::GetFormattedSystemTime(s, false, dateTimeSeparators);
	}
	else {
		time::GetFormattedSystemDate(s, true, false, dateTimeSeparators);
	}
	Label::SetText(s);
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


