

//--------------------------------------------------------------------
#include "GUI_CS0\Dialog.h"
#include "GUI\GUI_Tools.h"
#include "GUI\GUI_Manager.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
Dialog::GeomParams::GeomParams()
{
	buttonSpacing = 4;
	buttonMinWidth = 70;
	buttonRowHeightExtend = 0;
	horzSeparatorHeight = 8;
	horzSeparatorPadding = 0;
}


//====================================================================
Dialog::Dialog(const std::string& _title)
	:
	TopLevelFrame(_title)
{
	bUsesStdControls = true;

	// Create components
	pLayouterWorkArea = 0;
	pLayouterStdDialogControls = new gui::layouters::RowLayouter(this);

	pHorzSeparatorStdDialogButtons = new HorzSeparator(pLayouterStdDialogControls);
	pButtonApply = new TextButton(pLayouterStdDialogControls, this, gui::CID_Apply, gui::Button::Style_Push);
	pButtonOK = new TextButton(pLayouterStdDialogControls, this, gui::CID_OK, gui::Button::Style_Push);
	pButtonCancel = new TextButton(pLayouterStdDialogControls, this, gui::CID_Close, gui::Button::Style_Push);

	SetSize(400, 300);
	SetGeomParams();
	SetDialogSubcomponentLayout();
	SetDialogSubcomponentText();
	SetDialogSubcomponentToolTipText();
	SetConfirmSettingsOnKey();
	SetTLFSubcomponentLayout(true, false, true);
	SetDialogShaderNames();
}


//====================================================================
void  Dialog::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;

	TextButton::GeomParams  gpButton;
	gpButton.minSize.x = geomParams.buttonMinWidth;

	pButtonApply->SetGeomParams(gpButton);
	pButtonOK->SetGeomParams(gpButton);
	pButtonCancel->SetGeomParams(gpButton);

	HorzSeparator::GeomParams  gpHorzSep;
	gpHorzSep.preferredSize.y = geomParams.horzSeparatorHeight;
	gpHorzSep.parentPadding = geomParams.horzSeparatorPadding;

	pHorzSeparatorStdDialogButtons->SetGeomParams(gpHorzSep);

	BuildLayout();
	ReportDirtyGeometry();
}


//====================================================================
void  Dialog::SetDialogShaderNames(const std::string& textButtonStd,
								const std::string& horzSeparatorStd)
{
	pHorzSeparatorStdDialogButtons->SetShaderName(horzSeparatorStd);
	pButtonApply->SetShaderName(textButtonStd);
	pButtonOK->SetShaderName(textButtonStd);
	pButtonCancel->SetShaderName(textButtonStd);
}


//====================================================================
void  Dialog::SetDialogSubcomponentLayout(bool bVisibleApply,
										bool bVisibleOK,
										bool bVisibleCancel,
										bool bVisibleSeparator)
{
	pHorzSeparatorStdDialogButtons->SetVisible(bVisibleSeparator);
	pButtonApply->SetVisible(bVisibleApply);
	pButtonOK->SetVisible(bVisibleOK);
	pButtonCancel->SetVisible(bVisibleCancel);

	bool bStdControlsVisible = bVisibleApply  ||  bVisibleOK  ||  bVisibleCancel  ||  bVisibleSeparator;
	pLayouterStdDialogControls->SetVisible(bStdControlsVisible);

	BuildLayout();
}


//====================================================================
void  Dialog::SetDialogSubcomponentText(const std::string& sTextApply,
										const std::string& sTextOK,
										const std::string& sTextCancel)
{
	pButtonApply->SetText(sTextApply);
	pButtonOK->SetText(sTextOK);
	pButtonCancel->SetText(sTextCancel);
}


//====================================================================
void  Dialog::SetDialogSubcomponentToolTipText(const std::string& sApply,
											const std::string& sOK,
											const std::string& sCancel)
{
	pButtonApply->SetToolTipText(sApply);
	pButtonOK->SetToolTipText(sOK);
	pButtonCancel->SetToolTipText(sCancel);
}


//====================================================================
void  Dialog::SetConfirmSettingsOnKey(bool _bConfirmOnKey, eKey _kcOK, eKeyModifier _kmApply)
{
	bConfirmOnKey = _bConfirmOnKey;
	kcOK = _kcOK;
	kmApply = _kmApply;
}


//====================================================================
void  Dialog::RecalcLayout()
{
	TopLevelFrame::RecalcLayout();

	// adjust geometry of std-dialog-control layouter
	if(bUsesStdControls) {

		pLayouterStdDialogControls->SetSize(CalcTLFWorkAreaSize());
		pLayouterStdDialogControls->SetPosition(CalcTLFWorkAreaOffset());

		// calculate here so that derived-class or work-area-layouter can use the results
		pLayouterStdDialogControls->PropagateRecalcLayout();
	}

	// adjust geometry of work-area layouter
	if(pLayouterWorkArea) {

		if(bUsesStdControls) {
			pLayouterWorkArea->SetSize(CalcDialogWorkAreaSize());
			pLayouterWorkArea->SetPosition(CalcDialogWorkAreaOffset());
		}
		else {
			pLayouterWorkArea->SetSize(CalcTLFWorkAreaSize());
			pLayouterWorkArea->SetPosition(CalcTLFWorkAreaOffset());
		}
	}
}


//====================================================================
void  Dialog::OnChangedVisible()
{
	if(IsVisible()) {
		pButtonApply->SetEnabled(false);
		OnDialogOpened();
	}
}


//====================================================================
bool  Dialog::OnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch)
{
	if(bDown  &&  ! repeatCount) {
		if(bConfirmOnKey  &&  keyCode == kcOK) {
			uint commandID = gui::CID_OK;
			if(kmApply != KeyModifier_Count  &&  gui::Manager::Instance().IsKeyModifierPressed(kmApply)) {
				commandID = gui::CID_Apply;
			}
			SendMessageToSelf(gui::Message_Command(commandID));
			return true;
		}
	}
	return TopLevelFrame::OnKeyboardKey(bDown, keyCode, repeatCount, ch);
}


//====================================================================
void  Dialog::BuildLayout()
{
	bool bSep = pHorzSeparatorStdDialogButtons->IsVisible();
	bool bApply = pButtonApply->IsVisible();
	bool bOK = pButtonOK->IsVisible();
	bool bCancel = pButtonCancel->IsVisible();

	bUsesStdControls = bApply || bOK || bCancel;

	// Specify layout
	using namespace gui::layouters;
	RowLayouter& l = *pLayouterStdDialogControls;

	l.Clear();
	l.AutoHeightRow();
	if(bSep) {
		l.TargetHeightRow(pHorzSeparatorStdDialogButtons).Target(pHorzSeparatorStdDialogButtons, SF_Variable);
	}
	else {
		l.FixedHeightRow(geomParams.horzSeparatorHeight);
	}
	if(bUsesStdControls) {
		l.TargetHeightRow(0, SF_TargetAdd, geomParams.buttonRowHeightExtend).AutoSpacer();
		Row& lastRow = l.LastRow();

		if(bApply) {
			lastRow.target = pButtonApply;
			lastRow.Target(pButtonApply);
		}
		if(bOK) {
			if(bApply) lastRow.Spacer(geomParams.buttonSpacing);
			lastRow.target = pButtonOK;
			lastRow.Target(pButtonOK);
		}
		if(bCancel) {
			if(bApply  ||  bOK) lastRow.Spacer(geomParams.buttonSpacing);
			lastRow.target = pButtonCancel;
			lastRow.Target(pButtonCancel);
		}
	}
	OnStdDialogControlsLayouterUpdate();
}


//====================================================================
vec2  Dialog::CalcDialogWorkAreaSize() const
{
	if(bUsesStdControls) {
		const TopLevelFrame::GeomParams& gpTLF = TopLevelFrame::GetGeomParams();

		return vec2(GetSize().x - (gpTLF.frameBorderWidth + gpTLF.workAreaPaddingH) * 2,
					pLayouterStdDialogControls->GetRow(1).outputOffsetY);
	}
	return CalcTLFWorkAreaSize();
}


//====================================================================
void  Dialog::CalcDialogWorkArea(vec2& size,
						vec2& topLeft, vec2& topRight,
						vec2& bottomLeft, vec2& bottomRight) const
{
	size = CalcDialogWorkAreaSize();
	topLeft = CalcDialogWorkAreaOffset();
	gui::CalcPointsFromSizeAndTopLeft(size, topLeft, topRight, bottomLeft, bottomRight);
}


//====================================================================
bool  Dialog::OnCommand(uint commandID)
{
	switch(commandID) {
		case gui::CID_ControlValuesChanged: {
			pButtonApply->SetEnabled(true);
			return true;
		}
		case gui::CID_Close: {
			ShowTopLevel(false);
			OnDialogClosed();
			return true;
		}
		case gui::CID_Apply: {
			pButtonApply->SetEnabled(false);
			OnApplySettings();
			return true;
		}
		case gui::CID_OK: {
			ShowTopLevel(false);
			OnDialogClosed();
			OnApplySettings();
			return true;
		}
	}
	return TopLevelFrame::OnCommand(commandID);
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


