

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Layouters.h"
#include "GUI_CS0\TopLevelFrame.h"
#include "GUI_CS0\Buttons.h"
#include "GUI_CS0\Separators.h"
#include "GUI_CS0\CheckBox.h"
#include "GUI_CS0\ComboBox.h"
#include "GUI_CS0\EditBox.h"
#include "GUI_CS0\Label.h"
#include "GUI_CS0\RadioButton.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
// Internal hierarchy:
//
// * Dialog
//   * TextButton (Apply)
//   * TextButton (OK)
//   * TextButton (Cancel)
//   * HorzSeparator
//====================================================================
class VW_GUI_CS0_API Dialog : public gui_cs0::TopLevelFrame
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		float  buttonSpacing;
		float  buttonMinWidth;
		float  buttonRowHeightExtend;
		float  horzSeparatorHeight;
		float  horzSeparatorPadding;

		GeomParams();
	};


public:
	//------------------
	// Dialog interface.
	//------------------
	Dialog(const std::string& title = "Dialog");

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void  SetDialogShaderNames(
				const std::string& textButtonStd = DefaultShaderName(Shader_TextButton_Push),
				const std::string& horzSeparatorStd = DefaultShaderName(Shader_HorzSeparator));

	void  SetDialogSubcomponentLayout(
				bool bVisibleApply = true,
				bool bVisibleOK = true,
				bool bVisibleCancel = true,
				bool bVisibleSeparator = true);

	void  SetDialogSubcomponentText(
				const std::string& sTextApply = "Apply",
				const std::string& sTextOK = "OK",
				const std::string& sTextCancel = "Cancel");

	void  SetDialogSubcomponentToolTipText(
				const std::string& sApply = "Apply settings but\nleave the dialog opened",
				const std::string& sOK = "Apply settings and\nclose the dialog",
				const std::string& sCancel = "Close the dialog without\napplying the settings");

	void  SetConfirmSettingsOnKey(bool bConfirmOnKey = true, eKey kcOK = Key_Enter, eKeyModifier kmApply = KeyModifier_Control);

	// area not used by title-bar, border, work-area-padding & standard dialog buttons
	vec2  CalcDialogWorkAreaOffset() const { return TopLevelFrame::CalcTLFWorkAreaOffset(); }
	vec2  CalcDialogWorkAreaSize() const;

	void  CalcDialogWorkArea(vec2& size,
						vec2& topLeft, vec2& topRight,
						vec2& bottomLeft, vec2& bottomRight) const;

	// simple helpers
	void  DisableDialogApplyButton() { pButtonApply->SetEnabled(false); }
	void  UpdateControlDependencies() { OnDialogOpened(); }


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  RecalcLayout();
	virtual  bool  OnCommand(uint commandID);
	virtual  void  OnChangedVisible();
	virtual  bool  OnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch);

	//----------------
	// Layout control.
	//----------------
	void           BuildLayout();
	virtual  void  OnStdDialogControlsLayouterUpdate() {}

	// Use this to avoid overriding RecalcLayout() and setting the work-area-layouter
	// sizes & position manually.
	void             SetWorkAreaLayouter(gui::Component* pLayouter = 0) { pLayouterWorkArea = pLayouter; }
	gui::Component*  GetWorkAreaLayouter() { return pLayouterWorkArea; }

	//-------------------------------------------------
	// OnDialogOpened() is called after the
	// dialog transitions from hidden to visible state,
	// and after UpdateControlDependencies() is called.
	//
	// Buttons generate the following events:
	// 1) Apply-button:
	//        OnApplySettings()
	// 2) OK-button:
	//        OnCloseDialog()
	//        OnApplySettings()
	// 3) Cancel-button, Close-button (on title-bar):
	//        OnCloseDialog()
	//-------------------------------------------------
	virtual  void  OnDialogOpened() {}
	virtual  void  OnDialogClosed() {}
	virtual  void  OnApplySettings() {}


protected:
	//----------------
	// Sub-components.
	//----------------
	gui::Component*               pLayouterWorkArea;
	gui::layouters::RowLayouter*  pLayouterStdDialogControls;

	HorzSeparator*  pHorzSeparatorStdDialogButtons;
	TextButton*     pButtonApply;
	TextButton*     pButtonOK;
	TextButton*     pButtonCancel;

	//------------
	// Properties.
	//------------
	GeomParams    geomParams;
	bool          bConfirmOnKey;
	eKey          kcOK;
	eKeyModifier  kmApply;


private:
	//--------
	// States.
	//--------
	bool bUsesStdControls;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


