

//--------------------------------------------------------------------
#include "GUI_CS0\FrameStatisticsPane.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
FrameStatisticsPane::DrawParams::DrawParams()
{
	textOffsetTopLeft.Set(10, 10);
	textSpacingH = 10;
	colorText = color4b(255, 255, 255, 160);
	colorTextActive = color4b(255, 255, 255);
	colorTextHot = color4b(255, 255, 255);
	frameRatePrecision = 2;
	timePrecision = 3;
}


//====================================================================
FrameStatisticsPane::FrameStatisticsPane(FrameStatisticsCollector* _collector,
										const DrawParams& _dp,
										const gui::TextAttributes& _ta)
	:
	TopLevelPane("")
{
	bUseToolTip = true;

	SetCollector(_collector);
	SetTextAttributes(_ta);
	SetDrawParams(_dp);

	SetInfoMode();
	SetText();

	SetSize(260, 36);
	SetTLPSubcomponentLayout(false, false, true, true);
}


//====================================================================
void  FrameStatisticsPane::SetCollector(FrameStatisticsCollector* c)
{
	pCollector = c;
	SetupToolTipText();
}


//====================================================================
void  FrameStatisticsPane::SetDrawParams(const DrawParams& dp)
{
	drawParams = dp;
}


//====================================================================
void  FrameStatisticsPane::SetInfoMode(eInfoMode mode)
{
	infoMode = mode;
}


//====================================================================
void  FrameStatisticsPane::SetText(const std::string& _sNoCollector,
								const std::string& sImmediate,
								const std::string& sImmediateMin,
								const std::string& sImmediateMax,
								const std::string& sSmooth,
								const std::string& sSmoothMin,
								const std::string& sSmoothMax,
								const std::string& sTotalCount,
								const std::string& sTotalTime)
{
	sNoCollector = _sNoCollector;

	sText[InfoMode_FrameRateImmediate] = sImmediate;
	sText[InfoMode_FrameRateImmediateMin] = sImmediateMin;
	sText[InfoMode_FrameRateImmediateMax] = sImmediateMax;
	sText[InfoMode_FrameRateSmooth] = sSmooth;
	sText[InfoMode_FrameRateSmoothMin] = sSmoothMin;
	sText[InfoMode_FrameRateSmoothMax] = sSmoothMax;
	sText[InfoMode_TotalFrameCount] = sTotalCount;
	sText[InfoMode_TotalFrameTime] = sTotalTime;

	CalcTextWidths();
}


//====================================================================
void  FrameStatisticsPane::PostCacheFontHandles()
{
	TopLevelPane::PostCacheFontHandles();
	CalcTextWidths();
}


//====================================================================
bool  FrameStatisticsPane::OnQueryToolTipParams(gui::ToolTipParams& ttp)
{
	if(bUseToolTip) {
		return TopLevelPane::OnQueryToolTipParams(ttp);
	}
	return false;
}


//====================================================================
void  FrameStatisticsPane::DrawSelf(gui::Renderer& gr)
{
	TopLevelPane::DrawSelf(gr);

	// check & choose params
	if(! pFont) return;

	color4b cText = drawParams.colorText;
	if(IsMouseOver()) cText = drawParams.colorTextHot;
	else if(IsActive()) cText = drawParams.colorTextActive;

	gui::DrawTextParams  dtp(cText, drawParams.textOffsetTopLeft, textAtts.textMetricsL);

	if(! pCollector) {
		gr.DrawText(*pFont, sNoCollector, dtp);
		return;
	}

	// print frame stats
	uint  prec = drawParams.frameRatePrecision;

	gr.DrawText(*pFont, sText[infoMode], dtp);
	dtp.positionTopLeft.x += textWidths[infoMode] + drawParams.textSpacingH;

	std::string s;

	switch(infoMode) {
		case InfoMode_FrameRateImmediate:
			s = ToStringFix(pCollector->GetFrameRateImmediate(), 0, prec);
			break;

		case InfoMode_FrameRateImmediateMin:
			s = ToStringFix(pCollector->GetFrameRateImmediateMin(), 0, prec);
			break;

		case InfoMode_FrameRateImmediateMax:
			s = ToStringFix(pCollector->GetFrameRateImmediateMax(), 0, prec);
			break;

		case InfoMode_FrameRateSmooth:
			s = ToStringFix(pCollector->GetFrameRateSmooth(), 0, prec);
			break;

		case InfoMode_FrameRateSmoothMin:
			s = ToStringFix(pCollector->GetFrameRateSmoothMin(), 0, prec);
			break;

		case InfoMode_FrameRateSmoothMax:
			s = ToStringFix(pCollector->GetFrameRateSmoothMax(), 0, prec);
			break;

		case InfoMode_TotalFrameCount:
			s = ToString(pCollector->GetTotalFrameCount());
			break;

		case InfoMode_TotalFrameTime:
			s = ToStringFix(pCollector->GetTotalFrameTime(), 0, drawParams.timePrecision);
			break;
	}
	gr.DrawText(*pFont, s, dtp);
}


//====================================================================
bool  FrameStatisticsPane::OnMouseWheel(int delta, const vec2& mousePos)
{
	int next = infoMode + (delta >= 0 ? -1 : 1);
	if(next >= InfoMode_Count) next -= InfoMode_Count;
	if(next < 0) next += InfoMode_Count;
	SetInfoMode(eInfoMode(next));
	return true;
}


//====================================================================
bool  FrameStatisticsPane::OnMouseButton(bool bDown, eMouseButton buttonCode, bool bDoubleClick, uint repeatCount, const vec2& mousePos)
{
	if(TopLevelPane::OnMouseButton(bDown, buttonCode, bDoubleClick, repeatCount, mousePos)) return true;

	if(bDown  &&  buttonCode == MouseButton_Right  &&  ! repeatCount) {
		SetInfoMode(eInfoMode((infoMode + 1) % InfoMode_Count));
		return true;
	}
	return false;
}


//====================================================================
bool  FrameStatisticsPane::OnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch)
{
	if(bDown  &&  ! repeatCount) {
		switch(keyCode) {
			case Key_R: {
				if(pCollector) pCollector->Reset();
				return true;
			}
			case Key_T: {
				Flip(bUseToolTip);
				return true;
			}
			case Key_H: {
				if(pCollector) {
					pCollector->EnableHistoryEvaluation(! pCollector->IsEnabledHistoryEvaluation());
				}
				SetupToolTipText();
				
				return true;
			}
		}
	}
	return TopLevelPane::OnKeyboardKey(bDown, keyCode, repeatCount, ch);
}


//====================================================================
void  FrameStatisticsPane::CalcTextWidths()
{
	for(uint i = 0; i < InfoMode_Count; i++) {
		textWidths[i] = pFont ? pFont->CalcTextWidth(textAtts.textMetricsL, sText[i]) : 0;
	}
}


//====================================================================
void  FrameStatisticsPane::SetupToolTipText()
{
	std::string s = "RMB, Wheel - cycle info\nT - tooltip on/off\nR - reset collector\n";
	if(pCollector) {
		if(pCollector->IsEnabledHistoryEvaluation()) {
			s.append("H - disable history evaluation");
		}
		else {
			s.append("H - enable history evaluation");
		}
	}
	SetToolTipText(s);

	// reset tooltip
	if(IsMouseOverDirect()) {
		SendMessageToServiceProvider(gui::SID_ToolTip, gui::Message_ServiceCommand_ToolTip(gui::SCID_Reset));
	}
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


