

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\TopLevelPane.h"
#include "Framework\FrameObservers.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
class VW_GUI_CS0_API FrameStatisticsPane : public gui_cs0::TopLevelPane
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API DrawParams
	{
		vec2     textOffsetTopLeft;
		float    textSpacingH;
		color4b  colorText;
		color4b  colorTextActive;
		color4b  colorTextHot;
		uint     frameRatePrecision;
		uint     timePrecision;

		DrawParams();
	};

	//-------
	// Modes.
	//-------
	enum eInfoMode
	{
		InfoMode_FrameRateImmediate,
		InfoMode_FrameRateImmediateMin,
		InfoMode_FrameRateImmediateMax,
		InfoMode_FrameRateSmooth,
		InfoMode_FrameRateSmoothMin,
		InfoMode_FrameRateSmoothMax,
		InfoMode_TotalFrameCount,
		InfoMode_TotalFrameTime,

		InfoMode_Count,
		InfoMode_Force32Bits = 0x7FFFFFFF
	};


public:
	//-------------------------------
	// FrameStatisticsPane interface.
	//-------------------------------
	FrameStatisticsPane(FrameStatisticsCollector* collector = 0,
						const DrawParams& dp = DrawParams(),
						const gui::TextAttributes& ta = TopLevelPane::DefaultTextAttributes());

	void                       SetCollector(FrameStatisticsCollector* c = 0);
	FrameStatisticsCollector*  GetCollector() const { return pCollector; }

	void               SetDrawParams(const DrawParams& dp = DrawParams());
	const DrawParams&  GetDrawParams() const { return drawParams; }

	void  SetInfoMode(eInfoMode mode = InfoMode_FrameRateSmooth);

	void  SetText(const std::string& sNoCollector = "Collector == 0",
				const std::string& sImmediate = "FpsI:",
				const std::string& sImmediateMin = "FpsMinI:",
				const std::string& sImmediateMax = "FpsMaxI:",
				const std::string& sSmooth = "Fps:",
				const std::string& sSmoothMin = "FpsMin:",
				const std::string& sSmoothMax = "FpsMax:",
				const std::string& sTotalCount = "Frame:",
				const std::string& sTotalTime = "Time:");


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  PostCacheFontHandles();
	virtual  void  DrawSelf(gui::Renderer& gr);
	virtual  bool  OnQueryToolTipParams(gui::ToolTipParams& ttp);
	virtual  bool  OnMouseWheel(int delta, const vec2& mousePos);
	virtual  bool  OnMouseButton(bool bDown, eMouseButton buttonCode, bool bDoubleClick, uint repeatCount, const vec2& mousePos);
	virtual  bool  OnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch);

	//------------------
	// Internal helpers.
	//------------------
	void  CalcTextWidths();
	void  SetupToolTipText();


protected:
	//------------
	// Properties.
	//------------
	DrawParams   drawParams;
	std::string  sNoCollector;
	std::string  sText[InfoMode_Count];

	//--------
	// States.
	//--------
	eInfoMode                  infoMode;
	bool                       bUseToolTip;
	FrameStatisticsCollector*  pCollector;

	//-------
	// Cache.
	//-------
	float  textWidths[InfoMode_Count];
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


