

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\TopLevelPane.h"
#include "Framework\Kernel.h"
#include "HIDControllers\Mouse.h"
#include "HIDControllers\Keyboard.h"
#include "HIDControllers\PositionTracker.h"
#include "HIDControllers\Glove.h"
#include "HIDControllers\GloveGestures.h"
#include "HIDControllers\GameController.h"
#include "HIDControllers\OrientationTracker.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
class VW_GUI_CS0_API HIDControllerPane : public gui_cs0::TopLevelPane, public ReconfigObserver
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API DrawParams
	{
		vec2     textOffsetTopLeft;
		float    textSpacingH;
		float    textSpacingV;
		color4b  colorText;
		color4b  colorTextActive;
		color4b  colorTextHot;

		DrawParams();

		color4b  ChooseTextColor(const HIDControllerPane& pane) const;
	};


public:
	//-----------------------------
	// HIDControllerPane interface.
	//-----------------------------
	void               SetDrawParams(const DrawParams& dp = DrawParams());
	const DrawParams&  GetDrawParams() const { return drawParams; }

	void                SetHIDControllerName(const std::string& s = "HIDController");
	const std::string&  GetHIDControllerName() { return controllerName; }

	void  SetControllerOrdinalNumber(uint iOrdinalNumber = 0);
	uint  GetControllerOrdinalNumber() const { return iControllerOrdinalNumber; }

	const std::string&  GetControllerInterfaceName() const { return sControllerInterfaceName; }


protected:
	HIDControllerPane(const std::string& sControllerInterfaceName,
						uint iControllerOrdinalNumber = 0,
						const std::string& controllerName = "HIDController",
						const DrawParams& dp = DrawParams(),
						const gui::TextAttributes& ta = TopLevelPane::DefaultTextAttributes());

	void  SetControllerInterfaceName(const std::string& s) { sControllerInterfaceName = s; }

	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  PostCacheFontHandles();
	virtual  void  DrawSelf(gui::Renderer& gr);

	//----------------------------
	// ReconfigObserver overrides.
	//----------------------------
	virtual  void  OnHIDReconfig();

	//------------------
	// Internal helpers.
	//------------------
	void  UpdatePaneTitle();

	//--------------------------------------------
	// Derived-class interface & template-methods.
	//--------------------------------------------
	__forceinline  void  DrawOneLine(const std::string& text, gui::Renderer& gr, const gui::Font& font,
									gui::DrawTextParams& dtp, float lineSpacing)
	{
		gr.DrawText(font, text, dtp);
		dtp.positionTopLeft.y += lineSpacing;
	}

	virtual  void  DrawInfoText(gui::Renderer& gr,
								const gui::Font& font, gui::DrawTextParams& dtp,
								const DrawParams& dp, float lineSpacing) {}

	virtual  void  RecalcInfoTextCache() {}

	//---------------------------------------------------
	// Check controller and cache pointer,
	// <hidc> is 0 if none is found, return true if used.
	//---------------------------------------------------
	virtual  bool  CacheHIDController(HIDController* hidc) { return false; }


protected:
	//------------
	// Properties.
	//------------
	std::string  controllerName;
	DrawParams   drawParams;
	std::string  sControllerInterfaceName;
	uint         iControllerOrdinalNumber;

	//-------
	// Cache.
	//-------
	HIDController*  pController;
};


//====================================================================
class VW_GUI_CS0_API MouseStatePane : public gui_cs0::HIDControllerPane
{
public:
	//--------------------------
	// MouseStatePane interface.
	//--------------------------
	MouseStatePane(const std::string& controllerName = "Mouse",
					uint iMouseOrdinalNumber = 0,
					const DrawParams& dp = DrawParams(),
					const gui::TextAttributes& ta = TopLevelPane::DefaultTextAttributes());


protected:
	//-----------------------------
	// HIDControllerPane overrides.
	//-----------------------------
	virtual  void  DrawInfoText(gui::Renderer& gr,
								const gui::Font& font, gui::DrawTextParams& dtp,
								const DrawParams& dp, float lineSpacing);

	virtual  bool  CacheHIDController(HIDController* hidc);


protected:
	Mouse*  pMouse;
};


//====================================================================
class VW_GUI_CS0_API KeyboardStatePane : public gui_cs0::HIDControllerPane
{
public:
	//-----------------------------
	// KeyboardStatePane interface.
	//-----------------------------
	KeyboardStatePane(const std::string& controllerName = "Keyboard",
					uint iKeyboardOrdinalNumber = 0,
					const DrawParams& dp = DrawParams(),
					const gui::TextAttributes& ta = TopLevelPane::DefaultTextAttributes());


protected:
	//-----------------------------
	// HIDControllerPane overrides.
	//-----------------------------
	virtual  void  DrawInfoText(gui::Renderer& gr,
								const gui::Font& font, gui::DrawTextParams& dtp,
								const DrawParams& dp, float lineSpacing);

	virtual  bool  CacheHIDController(HIDController* hidc);


protected:
	Keyboard*  pKeyboard;
};


//====================================================================
class VW_GUI_CS0_API PositionTrackerStatePane : public gui_cs0::HIDControllerPane
{
public:
	//------------------------------------
	// PositionTrackerStatePane interface.
	//------------------------------------
	PositionTrackerStatePane(const std::string& controllerName = "PositionTracker",
					uint iPositionTrackerOrdinalNumber = 0,
					const DrawParams& dp = DrawParams(),
					const gui::TextAttributes& ta = TopLevelPane::DefaultTextAttributes());


protected:
	//-----------------------------
	// HIDControllerPane overrides.
	//-----------------------------
	virtual  void  DrawInfoText(gui::Renderer& gr,
								const gui::Font& font, gui::DrawTextParams& dtp,
								const DrawParams& dp, float lineSpacing);

	virtual  bool  CacheHIDController(HIDController* hidc);


protected:
	PositionTracker*  pTracker;
};


//====================================================================
class VW_GUI_CS0_API GloveStatePane : public gui_cs0::HIDControllerPane
{
public:
	//--------------------------
	// GloveStatePane interface.
	//--------------------------
	GloveStatePane(const std::string& controllerName = "Glove",
					uint iGloveOrdinalNumber = 0,
					const DrawParams& dp = DrawParams(),
					const gui::TextAttributes& ta = TopLevelPane::DefaultTextAttributes());


protected:
	//--------------------------
	// GloveStatePane overrides.
	//--------------------------
	virtual  void  DrawInfoText(gui::Renderer& gr,
								const gui::Font& font, gui::DrawTextParams& dtp,
								const DrawParams& dp, float lineSpacing);

	virtual  bool  CacheHIDController(HIDController* hidc);


protected:
	static_gestures::GestureAnalyzer  gestureAnalyzer;
	Glove*                            pGlove;
};


//====================================================================
class VW_GUI_CS0_API GameControllerStatePane : public gui_cs0::HIDControllerPane
{
public:
	//-----------------------------------
	// GameControllerStatePane interface.
	//-----------------------------------
	GameControllerStatePane(const std::string& controllerName = "GameController",
					uint iGameControllerOrdinalNumber = 0,
					const DrawParams& dp = DrawParams(),
					const gui::TextAttributes& ta = TopLevelPane::DefaultTextAttributes());


protected:
	//-----------------------------
	// HIDControllerPane overrides.
	//-----------------------------
	virtual  void  DrawInfoText(gui::Renderer& gr,
								const gui::Font& font, gui::DrawTextParams& dtp,
								const DrawParams& dp, float lineSpacing);

	virtual  bool  CacheHIDController(HIDController* hidc);


protected:
	GameController*  pGameController;
};


//====================================================================
class VW_GUI_CS0_API OrientationTrackerStatePane : public gui_cs0::HIDControllerPane
{
public:
	//---------------------------------------
	// OrientationTrackerStatePane interface.
	//---------------------------------------
	OrientationTrackerStatePane(const std::string& controllerName = "OrientationTracker",
					uint iOreintationTrackerOrdinalNumber = 0,
					const DrawParams& dp = DrawParams(),
					const gui::TextAttributes& ta = TopLevelPane::DefaultTextAttributes());


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  bool  OnQueryToolTipParams(gui::ToolTipParams& ttp);
	virtual  bool  OnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch);

	//-----------------------------
	// HIDControllerPane overrides.
	//-----------------------------
	virtual  void  DrawInfoText(gui::Renderer& gr,
								const gui::Font& font, gui::DrawTextParams& dtp,
								const DrawParams& dp, float lineSpacing);

	virtual  bool  CacheHIDController(HIDController* hidc);


protected:
	bool                 bUseToolTip;
	OrientationTracker*  pTracker;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


