

//--------------------------------------------------------------------
#include "GUI_CS0\Label.h"
#include "GUI_CS0\ShaderNames.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_Tools.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
Label::GeomParams::GeomParams()
{
	textHeight = 18;
	lineHeight = 1;
	minSize = vec2(0, 24);
}


//====================================================================
Label::Label(gui::Component* _parent,
			gui::MessageHandler* _commandTarget,
			uint _commandID,
			const std::string& _text,
			gui::Component* _pBuddy,
			bool _bGhost)
{
	pBuddy = _pBuddy;

	SetCommandTarget(_commandTarget);
	SetCommandID(_commandID);

	SetShaderName(DefaultShaderName(pBuddy ? Shader_Label_Active : Shader_Label_Static));
	SetGhost(_bGhost);
	SetText(_text);
	SetGeomParams();

	if(_parent) AttachTo(*_parent);
}


//====================================================================
void  Label::SetBuddy(gui::Component* _pBuddy)
{
	if(_pBuddy == this) _pBuddy = 0;
	pBuddy = _pBuddy;
}


//====================================================================
void  Label::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;
	ReportDirtyGeometry();
}


//====================================================================
void  Label::SetText(const std::string& _text)
{
	text = _text;
	BreakIntoLines(text, textLines);
	textLineWidths.resize(textLines.size());
	ReportDirtyGeometry();
}


//====================================================================
void  Label::RecalcSize()
{
	gui::ConstShaderPtr<Label::Shader> pShader = GetBaseShader();
	if(! pShader) return;

	// calc & cache text-size
	const gui::Font* pFont = pShader->GetFont();
	if(! pFont) return;

	gui::CalcTextSize(*pFont,
			pShader->GetTextMetricsL(geomParams.textHeight, geomParams.lineHeight),
			textLines, textSize, &textLineWidths);

	// update size depending on minimum-specified-size & text-size
	SetSize(ComponentMax(geomParams.minSize, textSize));
}


//====================================================================
bool  Label::OnQueryToolTipParams(gui::ToolTipParams& ttp)
{
	if(pBuddy) {
		if(pBuddy->GetToolTipText().length()) {
			ttp.bDisplay = true;
			ttp.text = pBuddy->GetToolTipText();
			ttp.mouseMoveIgnoreAreaPosition = GetAbsolutePosition();
			ttp.mouseMoveIgnoreAreaSize = GetSize();
			return true;
		}
	}
	return PushButton::OnQueryToolTipParams(ttp);
}


//====================================================================
void  Label::ProcessButtonRelease()
{
	// notify buddy
	if(pBuddy  &&  pBuddy->IsEnabled()) {
		SendMessageTo(*pBuddy, gui::Message_SendCommand());
	}
	// always use standard target
	SendCommandMessageToTarget();
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


