

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Component.h"
#include "GUI\GUI_BehaviorBases.h"
#include "GUI\GUI_Shader.h"
#include "private\vw_gui_cs0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
class VW_GUI_CS0_API Label : public gui::PushButton, public gui::CommandSender
{
	VW_GUI_COMPONENT_DECLARE_SHADER_DLL(VW_GUI_CS0_API, Shader);
	VW_GUI_COMPONENT_DECLARE_CHECK_SHADER(Shader);


public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		float  textHeight;
		float  lineHeight;
		vec2   minSize;

		GeomParams();
	};


public:
	//-----------------
	// Label interface.
	//-----------------
	Label(gui::Component* parent = 0,
			gui::MessageHandler* commandTarget = 0,
			uint commandID = 0,
			const std::string& text = "Label",
			gui::Component* pBuddy = 0,
			bool bGhost = false);

	//------------------------------------------------------------------
	// If buddy is hooked it receives Message_SendCommand after label is
	// released. Also, label uses buddy's tooltip text.
	//------------------------------------------------------------------
	void             SetBuddy(gui::Component* _pBuddy = 0);
	gui::Component*  GetBuddy() const { return pBuddy; }

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void                SetText(const std::string& text = "Label");
	const std::string&  GetText() const { return text; }

	//-----------------------------------------------
	// Additional methods intended for use by shader.
	//-----------------------------------------------
	const std::vector<std::string>  GetTextLines() const { return textLines; }
	uint                            GetNumTextLines() const { return textLines.size(); }
	const std::string&              GetTextLine(uint i) const { return textLines[i]; }
	float                           GetTextLineWidth(uint i) const { return textLineWidths[i]; }
	const vec2&                     GetTextSize() const { return textSize; }


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  RecalcSize();
	virtual  bool  OnQueryToolTipParams(gui::ToolTipParams& ttp);

	//---------------------------
	// PushButton implementation.
	//---------------------------
	virtual  void  ProcessButtonRelease();


protected:
	//------------
	// Properties.
	//------------
	gui::Component*  pBuddy;
	GeomParams       geomParams;
	std::string      text;

	//-------
	// Cache.
	//-------
	std::vector<std::string>  textLines;
	std::vector<float>        textLineWidths;
	vec2                      textSize;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


