

//--------------------------------------------------------------------
#include "GUI_CS0\LogoSpinners.h"
#include "GUI_CS0\ShaderNames.h"
#include "GUI\GUI_Manager.h"
#include "MathTools\TransformTools.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
LogoSpinner::GeomParams::GeomParams()
{
	preferredSize = vec2(48, 48);
}


//====================================================================
LogoSpinner::LogoSpinner(gui::Component* _parent,
						gui::MessageHandler* _commandTarget,
						uint _commandID)
{
	pTileTexture = 0;

	SetCommandTarget(_commandTarget);
	SetCommandID(_commandID);

	SetRotationRefPosition();
	SetRotationSpeed();

	SetShaderName(DefaultShaderName(Shader_LogoSpinner));
	SetGeomParams();
	SetTileParams();

	if(_parent) AttachTo(*_parent);

	PropagateChangedBranchVisible(IsVisible());
}


//====================================================================
void  LogoSpinner::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;
	ReportDirtyGeometry();
}


//====================================================================
void  LogoSpinner::SetTileParams(const gui::TileParams& tp)
{
	tileParams = tp;
	RequestTileTextureRecache();
}


//====================================================================
void  LogoSpinner::SetRotationRefPosition(const vec3& rotRefPos)
{
	rotationRefPos = rotRefPos;
	drawTransform.rotate = rotationRefPos;
}


//====================================================================
void  LogoSpinner::SetRotationSpeed(const vec3& rotSpeed)
{
	rotationSpeed = rotSpeed;
}


//====================================================================
bool  LogoSpinner::CacheTileTextureHandles(gui::Manager& gm)
{
	pTileTexture = gm.GetTileTexture(tileParams.texName);
	return (pTileTexture != 0);
}


//====================================================================
void  LogoSpinner::PostCacheTileTextureHandles()
{
	if(pTileTexture) {
		pTileTexture->GetTileCoords(tileParams.texRow, tileParams.texCol, rFront);

		// horz-flip the back side
		rBack.texCoordLeft   = rFront.texCoordRight;
		rBack.texCoordRight  = rFront.texCoordLeft;
		rBack.texCoordTop    = rFront.texCoordTop;
		rBack.texCoordBottom = rFront.texCoordBottom;
	}
	rFront.SetTexTransform(tileParams.texTransform);
	rBack.SetTexTransform(tileParams.texTransform);
}


//====================================================================
void  LogoSpinner::RecalcSize()
{
	SetSize(geomParams.preferredSize);
}


//====================================================================
void  LogoSpinner::UpdateGeometryCache()
{
	vec2  hs = GetSize() * 0.5f;

	rFront.Set(-hs.x, -hs.y, GetSize().x, GetSize().y);
	rBack.Set(hs.x, -hs.y, -GetSize().x, GetSize().y);

	drawTransform.translate = hs;
}


//====================================================================
bool  LogoSpinner::OnMouseButton(bool bDown, eMouseButton buttonCode, bool bDoubleClick, uint repeatCount, const vec2& mousePos)
{
	if(! bDown  ||  repeatCount) return false;

	if(buttonCode == MouseButton_Left) {
		if(bDoubleClick) {
			// reset reference position
			SetRotationRefPosition(rotationRefPos);
		}
		else {
			SendCommandMessageToTarget();
		}
		return true;
	}

	if(buttonCode == MouseButton_Right) {

		// reverse rotation direction
		SetRotationSpeed(- rotationSpeed);
		return true;
	}
	return false;
}


//====================================================================
void  LogoSpinner::OnChangedBranchVisible(bool bBranchVisible)
{
	gui::Manager::Instance().TimeNotifyRegister(*this, bBranchVisible && IsVisible());
}


//====================================================================
void  LogoSpinner::OnChangedVisible()
{
	gui::Manager::Instance().TimeNotifyRegister(*this, IsBranchVisible());
}


//====================================================================
void  LogoSpinner::OnTimeDelta(double secs)
{
	drawTransform.rotate += rotationSpeed * secs;

	RemapAngleTo180DegRange(drawTransform.rotate.x);
	RemapAngleTo180DegRange(drawTransform.rotate.y);
	RemapAngleTo180DegRange(drawTransform.rotate.z);
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


