

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Component.h"
#include "GUI\GUI_Shader.h"
#include "private\vw_gui_cs0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
class VW_GUI_CS0_API LogoSpinner : public gui::Component, public gui::CommandSender
{
	VW_GUI_COMPONENT_DECLARE_SHADER_DLL(VW_GUI_CS0_API, Shader);
	VW_GUI_COMPONENT_DECLARE_CHECK_SHADER(Shader);


public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		vec2  preferredSize;

		GeomParams();
	};


public:
	//-----------------------
	// LogoSpinner interface.
	//-----------------------
	LogoSpinner(gui::Component* parent = 0,
				gui::MessageHandler* commandTarget = 0,
				uint commandID = 0);

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void                    SetTileParams(const gui::TileParams& tp = gui::TileParams());
	const gui::TileParams&  GetTileParams() const { return tileParams; }

	// degrees around each axis
	void         SetRotationRefPosition(const vec3& rotRefPos = vec3::ZERO);
	const vec3&  GetRotationRefPosition() const { return rotationRefPos; }

	// degrees-per-second around each axis
	void         SetRotationSpeed(const vec3& rotSpeed = vec3(0, 90, 0));
	const vec3&  GetRotationSpeed() const { return rotationSpeed; }

	//-----------------------------------------------
	// Additional methods intended for use by shader.
	//-----------------------------------------------
	const gui::Transform&    GetDrawTransform() const { return drawTransform; }
	const gui::TileTexture*  GetTileTexture() const { return pTileTexture; }

	// Returned rectangles are already filled with geometry & texture coords,
	// can be used by shader as drawing cache
	gui::RectangleCT&  GetTileRectangleFront() const { return rFront; }
	gui::RectangleCT&  GetTileRectangleBack() const { return rBack; }


protected:
	//-----------------------------
	// VideoResourceUser overrides.
	//-----------------------------
	virtual  bool  CacheTileTextureHandles(gui::Manager& gm);
	virtual  void  PostCacheTileTextureHandles();

	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  RecalcSize();
	virtual  void  UpdateGeometryCache();
	virtual  bool  OnMouseButton(bool bDown, eMouseButton buttonCode, bool bDoubleClick, uint repeatCount, const vec2& mousePos);
	virtual  void  OnChangedBranchVisible(bool bBranchVisible);
	virtual  void  OnChangedVisible();
	virtual  void  OnTimeDelta(double secs);


protected:
	//------------
	// Properties.
	//------------
	GeomParams       geomParams;
	gui::TileParams  tileParams;
	vec3             rotationRefPos;
	vec3             rotationSpeed;

	//-------
	// Cache.
	//-------
	const gui::TileTexture*   pTileTexture;
	mutable gui::RectangleCT  rFront;
	mutable gui::RectangleCT  rBack;
	gui::Transform            drawTransform;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


