

//--------------------------------------------------------------------
#include "GUI_CS0\PixelOffsetControlPane.h"
#include "GUI_CS0\Buttons.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_StdObjects.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
PixelOffsetControlPane::GeomParams::GeomParams()
{
	screenBorderWidth = 1;
	controlAreaSize = vec2(120, 120);
	iconButtonSize = 20;
	iconButtonTileSize = 16;
	labelTextHeight = 20;
}


//====================================================================
PixelOffsetControlPane::PixelOffsetControlPane()
{
	SetVisible(false);
	SetCanFocus(true);

	// Create components
	pLayouter = new gui::layouters::RowLayouter(this);

	pButtonClose = new IconButton(pLayouter, this, gui::CID_Close, gui::Button::Style_Push);
	pButtonLeft = new IconButton(pLayouter, this, gui::CID_Left, gui::Button::Style_Press);
	pButtonRight = new IconButton(pLayouter, this, gui::CID_Right, gui::Button::Style_Press);
	pButtonUp = new IconButton(pLayouter, this, gui::CID_Up, gui::Button::Style_Press);
	pButtonDown = new IconButton(pLayouter, this, gui::CID_Down, gui::Button::Style_Press);
	pLabelPixelOffsetX = new Label(pLayouter, 0, 0, "");
	pLabelPixelOffsetY = new Label(pLayouter, 0, 0, "");

	SetGeomParams();
	SetPixelOffsetGranularity();
	UpdatePixelOffsetText();
	SetPOCPSubcomponentToolTipText();
	SetPOCPSubcomponentTileParams();
	SetPOCPShaderNames();
}


//====================================================================
void  PixelOffsetControlPane::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;

	IconButton::GeomParams  gpIconButton;
	gpIconButton.preferredSize = vec2(geomParams.iconButtonSize, geomParams.iconButtonSize);
	gpIconButton.iconSize = vec2(geomParams.iconButtonTileSize, geomParams.iconButtonTileSize);

	pButtonClose->SetGeomParams(gpIconButton);
	pButtonLeft->SetGeomParams(gpIconButton);
	pButtonRight->SetGeomParams(gpIconButton);
	pButtonUp->SetGeomParams(gpIconButton);
	pButtonDown->SetGeomParams(gpIconButton);

	Label::GeomParams gpLabel;
	gpLabel.textHeight = geomParams.labelTextHeight;
	gpLabel.minSize = vec2::ZERO;

	pLabelPixelOffsetX->SetGeomParams(gpLabel);
	pLabelPixelOffsetY->SetGeomParams(gpLabel);

	// Specify layout
	using namespace gui::layouters;
	RowLayouter& l = *pLayouter;

	l.Clear();
	l.FixedHeightRow(geomParams.iconButtonSize).AutoSpacer().Target(pButtonUp).AutoSpacer();
	l.FixedHeightRow(geomParams.iconButtonSize).AutoSpacer().Target(pButtonLeft).Target(pButtonClose).Target(pButtonRight).AutoSpacer();
	l.FixedHeightRow(geomParams.iconButtonSize).AutoSpacer().Target(pButtonDown).AutoSpacer();
	l.AutoHeightRow().AutoSpacer().Target(pLabelPixelOffsetX).AutoSpacer();
	l.AutoHeightRow().AutoSpacer().Target(pLabelPixelOffsetY).AutoSpacer();

	ReportDirtyGeometry();
}


//====================================================================
void  PixelOffsetControlPane::SetPixelOffsetGranularity(float pog)
{
	pixelOffsetGranularity = Clamped<float>(pog, 0, 0.5f);
}


//====================================================================
void  PixelOffsetControlPane::SetPOCPShaderNames(const std::string& pane,
											const std::string& iconButtons,
											const std::string& labels)
{
	SetShaderName(pane);
	pButtonClose->SetShaderName(iconButtons);
	pButtonLeft->SetShaderName(iconButtons);
	pButtonRight->SetShaderName(iconButtons);
	pButtonUp->SetShaderName(iconButtons);
	pButtonDown->SetShaderName(iconButtons);
	pLabelPixelOffsetX->SetShaderName(labels);
	pLabelPixelOffsetY->SetShaderName(labels);
}


//====================================================================
void  PixelOffsetControlPane::SetPOCPSubcomponentToolTipText(
											const std::string& buttonClose,
											const std::string& buttonArrows,
											const std::string& labelOffsetX,
											const std::string& labelOffsetY)
{
	pButtonClose->SetToolTipText(buttonClose);
	pButtonLeft->SetToolTipText(buttonArrows);
	pButtonRight->SetToolTipText(buttonArrows);
	pButtonUp->SetToolTipText(buttonArrows);
	pButtonDown->SetToolTipText(buttonArrows);
	pLabelPixelOffsetX->SetToolTipText(labelOffsetX);
	pLabelPixelOffsetY->SetToolTipText(labelOffsetY);
}


//====================================================================
void  PixelOffsetControlPane::SetPOCPSubcomponentTileParams(
									const gui::TileParams& buttonClose,
									const gui::TileParams& buttonLeft,
									const gui::TileParams& buttonRight,
									const gui::TileParams& buttonUp,
									const gui::TileParams& buttonDown)
{
	pButtonClose->SetTileParams(buttonClose);
	pButtonLeft->SetTileParams(buttonLeft);
	pButtonRight->SetTileParams(buttonRight);
	pButtonUp->SetTileParams(buttonUp);
	pButtonDown->SetTileParams(buttonDown);
}


//====================================================================
void  PixelOffsetControlPane::GetDrawRectangles(gui::Rectangle& _rControlArea,
											gui::Rectangle& _rBorderLeft,
											gui::Rectangle& _rBorderRight,
											gui::Rectangle& _rBorderTop,
											gui::Rectangle& _rBorderBottom) const
{
	_rControlArea = rControlArea;
	_rBorderLeft = rBorderLeft;
	_rBorderRight = rBorderRight;
	_rBorderTop = rBorderTop;
	_rBorderBottom = rBorderBottom;
}


//====================================================================
void  PixelOffsetControlPane::OnVideoRendererChanged(VideoRenderer* vr)
{
	UpdatePixelOffsetText();
}


//====================================================================
void  PixelOffsetControlPane::RecalcLayout()
{
	controlAreaOffset = (gui::Manager::Instance().GetWindowSize() - geomParams.controlAreaSize) * 0.5f;

	pLayouter->SetPosition(controlAreaOffset);
	pLayouter->SetSize(geomParams.controlAreaSize);
}


//====================================================================
void  PixelOffsetControlPane::UpdateGeometryCache()
{
	vec2  ws = gui::Manager::Instance().GetWindowSize();
	float bw = geomParams.screenBorderWidth;

	rControlArea.Set(controlAreaOffset, geomParams.controlAreaSize);
	rBorderLeft.Set(0, 0, bw, ws.y);
	rBorderRight.Set(ws.x - bw, 0, bw, ws.y);
	rBorderTop.Set(0, 0, ws.x, bw);
	rBorderBottom.Set(0, ws.y - bw, ws.x, bw);
}


//====================================================================
void  PixelOffsetControlPane::OnWindowResize(uint width, uint height)
{
	SetPosition(vec2::ZERO);
}


//====================================================================
bool  PixelOffsetControlPane::HitTestSelf(const vec2& testPoint) const
{
	return rControlArea.HitTest(testPoint);
}


//====================================================================
bool  PixelOffsetControlPane::OnCommand(uint commandID)
{
	switch(commandID) {

		case gui::CID_Close:
			ShowTopLevel(false);
			return true;

		case gui::CID_Left:
			SetupRendererPixelOffset(- pixelOffsetGranularity, 0);
			return true;

		case gui::CID_Right:
			SetupRendererPixelOffset(pixelOffsetGranularity, 0);
			return true;

		case gui::CID_Up:
			SetupRendererPixelOffset(0, - pixelOffsetGranularity);
			return true;

		case gui::CID_Down:
			SetupRendererPixelOffset(0, pixelOffsetGranularity);
			return true;
	}
	return false;
}


//====================================================================
bool  PixelOffsetControlPane::OnKeyboardQueryFocus(bool bFocusByMouse, eMouseButton buttonCode, bool& bWantFocus, bool& bNeedCharacterData)
{
	DefaultProcessQueryFocus(bFocusByMouse, buttonCode, bWantFocus, bNeedCharacterData);
	return true;
}


//====================================================================
bool  PixelOffsetControlPane::OnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch)
{
	if(! bDown) return false;

	switch(keyCode) {
		case Key_Escape:
			SendMessageToSelf(gui::Message_Command(gui::CID_Close));
			return true;

		case Key_ArrowLeft:
			SendMessageToSelf(gui::Message_Command(gui::CID_Left));
			return true;

		case Key_ArrowRight:
			SendMessageToSelf(gui::Message_Command(gui::CID_Right));
			return true;

		case Key_ArrowUp:
			SendMessageToSelf(gui::Message_Command(gui::CID_Up));
			return true;

		case Key_ArrowDown:
			SendMessageToSelf(gui::Message_Command(gui::CID_Down));
			return true;
	}
	return false;
}


//====================================================================
void  PixelOffsetControlPane::SetupRendererPixelOffset(float dx, float dy)
{
	gui::Renderer* gr = gui::Manager::Instance().GetGUIRenderer();
	if(! gr) return;

	gui::Renderer::PixelParams  pp;
	gr->GetPixelParams(pp);

	pp.offsetX += dx;
	pp.offsetY += dy;

	Clamp<float>(pp.offsetX, -1, 1);
	Clamp<float>(pp.offsetY, -1, 1);

	gr->SetPixelParams(pp);

	UpdatePixelOffsetText();
}


//====================================================================
void  PixelOffsetControlPane::UpdatePixelOffsetText()
{
	gui::Renderer* gr = gui::Manager::Instance().GetGUIRenderer();
	if(gr) {
		gui::Renderer::PixelParams  pp;
		gr->GetPixelParams(pp);

		pLabelPixelOffsetX->SetText(ToStringFix(pp.offsetX, 6, 3, true, false));
		pLabelPixelOffsetY->SetText(ToStringFix(pp.offsetY, 6, 3, true, false));
	}
	else {
		const char* t = "?";
		pLabelPixelOffsetX->SetText(t);
		pLabelPixelOffsetY->SetText(t);
	}
	ReportDirtyGeometry();
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


