

//--------------------------------------------------------------------
#include "GUI_CS0\ProfilerControlDialog.h"
#include "GUI\GUI_Manager.h"
#include "Framework\FrameworkProfiler.h"
#include "Framework\Kernel.h"
#include "BaseTools\Stream.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
ProfilerControlDialog::ProfilerControlDialog(const std::string& _title)
	:
	Dialog(_title)
{
	accumTime = 0;
	pProfiler = 0;
	HookProfiler();
	SetLog();
	SetVisibleStatsUpdateInterval();

	// Create sub-components
	pLayouter = new gui::layouters::RowLayouter(this);

	pCheckBoxRecursionHandler = new CheckBox(pLayouter, this, CID_RecursionHandler);
	pLabelRecursionHandler = new Label(pLayouter, 0, 0, "Enable recursion-handler", pCheckBoxRecursionHandler);
	pLabelAutoReset = new Label(pLayouter, 0, 0, "Auto reset:");
	pLabelMaxDepth = new Label(pLayouter, 0, 0, "Max depth:");

	pEditBoxAutoReset = new EditBox(pLayouter, this, CID_AutoReset);
	pEditBoxMaxDepth = new EditBox(pLayouter, this, CID_MaxDepth);

#ifndef VW_BUILD_DEBUG
	pEditBoxMaxDepth->SetEnabled(false);
#endif

	pLabelIterationCount = new Label(pLayouter, 0, 0, "Iteration count:");
	pLabelIterationCountValue = new Label(pLayouter, 0, 0, "");
	pLabelElapsedTime = new Label(pLayouter, 0, 0, "Elapsed seconds:");
	pLabelElapsedTimeValue = new Label(pLayouter, 0, 0, "");
	pLabelPrint = new Label(pLayouter, 0, 0, "EC/NT/BT/NP:");

	pCheckBoxPrintEntryCounts = new CheckBox(pLayouter, this, CID_PrintEntryCounts);
	pCheckBoxPrintEntryCounts->SetOn(false);

	pCheckBoxPrintNodeTimes = new CheckBox(pLayouter, this, CID_PrintNodeTimes);
	pCheckBoxPrintNodeTimes->SetOn(false);

	pCheckBoxPrintBranchTimes = new CheckBox(pLayouter, this, CID_PrintBranchTimes);
	pCheckBoxPrintBranchTimes->SetOn(false);

	pCheckBoxPrintNodePercentages = new CheckBox(pLayouter, this, CID_PrintNodePercentages);
	pCheckBoxPrintNodePercentages->SetOn(true);

	pButtonClear = new TextButton(pLayouter, this, CID_Clear, gui::Button::Style_Push, 0, "Clear");
	pButtonReset = new TextButton(pLayouter, this, CID_Reset, gui::Button::Style_Push, 0, "Reset");
	pButtonDump = new TextButton(pLayouter, this, CID_Dump, gui::Button::Style_Push, 0, "Dump");
	pButtonRunStop = new TextButton(pLayouter, this, CID_RunStop, gui::Button::Style_Push, 0, "Run/Stop");

	SetSize(270, 300);
	SetDialogSubcomponentLayout(false, false, false, false);
	SetProfilerControlDialogShaderNames();
	SetProfilerControlDialogSubcomponentToolTipText();

	OnDialogOpened();

	// Specify layout
	SetWorkAreaLayouter(pLayouter);

	using namespace gui::layouters;
	RowLayouter& l = *pLayouter;

	float varLen = 0.3f;

	l.Clear();
	l.FixedHeightRow().Target(pCheckBoxRecursionHandler).Spacer().Target(pLabelRecursionHandler);
	l.FixedHeightRow().Target(pLabelAutoReset).Spacer().Target(pEditBoxAutoReset, SF_Variable);
	l.FixedHeightRow().Target(pLabelMaxDepth).Spacer().Target(pEditBoxMaxDepth, SF_Variable);
	l.FixedHeightRow().Target(pLabelIterationCount).AutoSpacer().Target(pLabelIterationCountValue);
	l.FixedHeightRow().Target(pLabelElapsedTime).AutoSpacer().Target(pLabelElapsedTimeValue);
	l.FixedHeightRow().Target(pLabelPrint).AutoSpacer().Target(pCheckBoxPrintEntryCounts);
	l.LastRow().Spacer().Target(pCheckBoxPrintNodeTimes).Spacer().Target(pCheckBoxPrintBranchTimes);
	l.LastRow().Spacer().Target(pCheckBoxPrintNodePercentages);
	l.FixedHeightRow().Target(pButtonClear, SF_Variable, varLen).AutoSpacer();
	l.LastRow().Target(pButtonReset, SF_Variable, varLen).AutoSpacer().Target(pButtonDump, SF_Variable, varLen);
	l.FixedHeightRow().Target(pButtonRunStop, SF_Variable);
}


//====================================================================
void  ProfilerControlDialog::HookProfiler(Profiler* p)
{
	if(pProfiler) {
		pProfiler->UnregisterObserver(*this);
	}
	if(p) {
		pProfiler = p;
	}
	else {
		pProfiler = &Kernel::Instance().GetFrameworkProfiler();
	}
	pProfiler->RegisterObserver(*this);
}


//====================================================================
void  ProfilerControlDialog::SetLog(std::ostream* _pLog)
{
	if(_pLog) {
		pLog = _pLog;
	}
	else {
		pLog = &GetLogOut();
	}
}


//====================================================================
void  ProfilerControlDialog::SetVisibleStatsUpdateInterval(double secs)
{
	updateInterval = ClampedLow<double>(secs, 0);
}


//====================================================================
void  ProfilerControlDialog::SetProfilerControlDialogShaderNames(
											const std::string& label,
											const std::string& labelCB,
											const std::string& checkBox,
											const std::string& editBox,
											const std::string& textButton)
{
	pLabelRecursionHandler->SetShaderName(labelCB);
	pLabelAutoReset->SetShaderName(label);
	pLabelMaxDepth->SetShaderName(label);
	pLabelIterationCount->SetShaderName(label);
	pLabelIterationCountValue->SetShaderName(label);
	pLabelElapsedTime->SetShaderName(label);
	pLabelElapsedTimeValue->SetShaderName(label);
	pLabelPrint->SetShaderName(label);
	pCheckBoxRecursionHandler->SetShaderName(checkBox);
	pCheckBoxPrintEntryCounts->SetShaderName(checkBox);
	pCheckBoxPrintNodeTimes->SetShaderName(checkBox);
	pCheckBoxPrintBranchTimes->SetShaderName(checkBox);
	pCheckBoxPrintNodePercentages->SetShaderName(checkBox);
	pEditBoxAutoReset->SetShaderName(editBox);
	pEditBoxMaxDepth->SetShaderName(editBox);
	pButtonClear->SetShaderName(textButton);
	pButtonReset->SetShaderName(textButton);
	pButtonDump->SetShaderName(textButton);
	pButtonRunStop->SetShaderName(textButton);
}


//====================================================================
void  ProfilerControlDialog::SetProfilerControlDialogSubcomponentToolTipText(
										const std::string& sRecursionHandler,
										const std::string& sAutoReset,
										const std::string& sMaxDepth,
										const std::string& sClear,
										const std::string& sReset,
										const std::string& sDump,
										const std::string& sPrintEntryCounts,
										const std::string& sPrintNodeTimes,
										const std::string& sPrintBranchTimes,
										const std::string& sPrintNodePercentages,
										const std::string& sRunStop)
{
	pCheckBoxRecursionHandler->SetToolTipText(sRecursionHandler);
	pEditBoxAutoReset->SetToolTipText(sAutoReset);
	pEditBoxMaxDepth->SetToolTipText(sMaxDepth);
	pButtonClear->SetToolTipText(sClear);
	pButtonReset->SetToolTipText(sReset);
	pButtonDump->SetToolTipText(sDump);
	pCheckBoxPrintEntryCounts->SetToolTipText(sPrintEntryCounts);
	pCheckBoxPrintNodeTimes->SetToolTipText(sPrintNodeTimes);
	pCheckBoxPrintBranchTimes->SetToolTipText(sPrintBranchTimes);
	pCheckBoxPrintNodePercentages->SetToolTipText(sPrintNodePercentages);
	pButtonRunStop->SetToolTipText(sRunStop);
}


//====================================================================
void  ProfilerControlDialog::OnProfilerEnabled(bool bEnabled)
{
	TimeNotifyRegister();
	MatchActiveSettings();
	UpdateVisibleStats();
}


//====================================================================
bool  ProfilerControlDialog::OnCommand(uint commandID)
{
	switch(commandID) {
		case CID_RecursionHandler: {
			pProfiler->EnableRecursionHandler(pCheckBoxRecursionHandler->IsOn());
			break;
		}
		case CID_AutoReset: {
			int val;
			if(ToValue(pEditBoxAutoReset->GetText(), val)  &&  val >= 0) {
				pProfiler->SetAutoResetThreshold(val);
			}
			break;
		}
		case CID_MaxDepth: {
			int val;
			if(ToValue(pEditBoxMaxDepth->GetText(), val)  &&  val >= 0) {
				pProfiler->SetMaxExpectedStackDepth(val);
			}
			break;
		}
		case CID_Clear: {
			pProfiler->ClearNodes();
			UpdateVisibleStats();
			break;
		}
		case CID_Reset: {
			pProfiler->ResetStatistics();
			UpdateVisibleStats();
			break;
		}
		case CID_Dump: {
			profilerDumpDesc.bPrintEntryCounts = pCheckBoxPrintEntryCounts->IsOn();
			profilerDumpDesc.bPrintNodeTimes = pCheckBoxPrintNodeTimes->IsOn();
			profilerDumpDesc.bPrintBranchTimes = pCheckBoxPrintBranchTimes->IsOn();
			profilerDumpDesc.bPrintNodePercentages = pCheckBoxPrintNodePercentages->IsOn();
			profilerDumpDesc.ConstructHeaderLines();

			pProfiler->ProcessStatistics();
			OnDump(*pLog, *pProfiler, profilerDumpDesc);
			break;
		}
		case CID_RunStop: {
			pProfiler->Enable(! pProfiler->IsEnabled());
			break;
		}
		default: {
			return Dialog::OnCommand(commandID);
		}
	}
	MatchActiveSettings();
	return true;
}


//====================================================================
void  ProfilerControlDialog::OnChangedBranchVisible(bool bBranchVisible)
{
	TimeNotifyRegister();
}


//====================================================================
void  ProfilerControlDialog::OnChangedVisible()
{
	TimeNotifyRegister();
}


//====================================================================
void  ProfilerControlDialog::OnTimeDelta(double secs)
{
	accumTime += secs;
	if(accumTime >= updateInterval) {
		accumTime -= updateInterval;

		UpdateVisibleStats();
	}
}


//====================================================================
void  ProfilerControlDialog::OnDialogOpened()
{
	MatchActiveSettings();
	UpdateVisibleStats();
	TimeNotifyRegister();
}


//====================================================================
void  ProfilerControlDialog::MatchActiveSettings()
{
	pCheckBoxRecursionHandler->SetOn(pProfiler->IsEnabledRecursionHandler());
	pEditBoxAutoReset->SetText(ToString(pProfiler->GetAutoResetThreshold()));
	pEditBoxMaxDepth->SetText(ToString(pProfiler->GetMaxExpectedStackDepth()));
	pButtonRunStop->SetText(pProfiler->IsEnabled() ? "Stop" : "Run");

#ifdef VW_BUILD_DEBUG
	pEditBoxMaxDepth->SetEnabled(! pCheckBoxRecursionHandler->IsOn());
#endif
}


//====================================================================
void  ProfilerControlDialog::UpdateVisibleStats()
{
	pLabelIterationCountValue->SetText(ToStringFix(pProfiler->GetIterationCount(), 10, 0));
	pLabelElapsedTimeValue->SetText(ToStringFix(pProfiler->GetElapsedSeconds(), 10, 2));
}


//====================================================================
void  ProfilerControlDialog::TimeNotifyRegister()
{
	accumTime = 0;

	gui::Manager::Instance().TimeNotifyRegister(*this,
		pProfiler->IsEnabled()  &&  IsBranchVisible());
}


//====================================================================
void  ProfilerControlDialog::OnDump(std::ostream& os, Profiler& profiler, ProfilerDumpDesc& pdd)
{
	os << e_attention << "\nProfile statistics:\n" << e_info;
	profiler.DumpStatistics(os, pdd);
	os << std::endl;
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


