

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Layouters.h"
#include "GUI_CS0\Dialog.h"
#include "GUI_CS0\Buttons.h"
#include "GUI_CS0\EditBox.h"
#include "GUI_CS0\CheckBox.h"
#include "GUI_CS0\Label.h"
#include "BaseTools\Profiler.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
class VW_GUI_CS0_API ProfilerControlDialog : public gui_cs0::Dialog, public Profiler::Observer
{
public:
	//---------------------------------
	// ProfilerControlDialog interface.
	//---------------------------------
	ProfilerControlDialog(const std::string& title = "Profiler Control");

	void                     SetProfilerDumpDesc(const ProfilerDumpDesc& pdd = ProfilerDumpDesc()) { profilerDumpDesc = pdd; }
	const ProfilerDumpDesc&  GetProfilerDumpDesc() const { return profilerDumpDesc; }

	void       HookProfiler(Profiler* p = 0); //0 means use framework-profiler
	Profiler*  GetHookedProfiler() const { return pProfiler; }

	void           SetLog(std::ostream* pLog = 0); //0 means use framework-log
	std::ostream&  GetLog() const { return *pLog; }

	void    SetVisibleStatsUpdateInterval(double secs = 1);
	double  GetVisibleStatsUpdateInterval() const { return updateInterval; }

	void  SetProfilerControlDialogShaderNames(
				const std::string& label = DefaultShaderName(Shader_Label_Static),
				const std::string& labelCB = DefaultShaderName(Shader_Label_Active),
				const std::string& checkBox = DefaultShaderName(Shader_CheckBox),
				const std::string& editBox = DefaultShaderName(Shader_EditBox),
				const std::string& textButton = DefaultShaderName(Shader_TextButton_Push));

	void  SetProfilerControlDialogSubcomponentToolTipText(
				const std::string& sRecursionHandler = "Enable recursion handler\n(larger overhead)",
				const std::string& sAutoReset = "Auto reset statistics after N iterations\n(reduces profiler-internal memory-allocation noise)",
				const std::string& sMaxDepth = "Maximum expected profile-stack depth\nafter which warning is displayed\n(debug-only, used if recursion-handler is disabled)",
				const std::string& sClear = "Delete all nodes and statistics",
				const std::string& sReset = "Reset accumulated statistics",
				const std::string& sDump = "Dump statistics to log",
				const std::string& sPrintEntryCounts = "Print entry-counts",
				const std::string& sPrintNodeTimes = "Print node-times",
				const std::string& sPrintBranchTimes = "Print branch-times",
				const std::string& sPrintNodePercentages = "Print node-percentages",
				const std::string& sRunStop = "Run / stop profiler");

	//-----------------------------------
	// Profiler::Observer implementation.
	//-----------------------------------
	virtual  void  OnProfilerEnabled(bool bEnabled);


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  bool  OnCommand(uint commandID);
	virtual  void  OnChangedBranchVisible(bool bBranchVisible);
	virtual  void  OnChangedVisible();
	virtual  void  OnTimeDelta(double secs);

	//------------------
	// Dialog overrides.
	//------------------
	virtual  void  OnDialogOpened();
	virtual  void  OnDialogClosed() {}
	virtual  void  OnApplySettings() {}

	//---------------------
	// Command identifiers.
	//---------------------
	enum {
		CID_RecursionHandler = gui::CID_User,
		CID_AutoReset,
		CID_MaxDepth,
		CID_PrintEntryCounts,
		CID_PrintNodeTimes,
		CID_PrintBranchTimes,
		CID_PrintNodePercentages,
		CID_Clear,
		CID_Reset,
		CID_Dump,
		CID_RunStop
	};

	//------------------
	// Internal helpers.
	//------------------
	void  MatchActiveSettings();
	void  UpdateVisibleStats();
	void  TimeNotifyRegister();

	//-------------------------
	// Derived-class interface.
	//-------------------------
	virtual  void  OnDump(std::ostream& os, Profiler& profiler, ProfilerDumpDesc& pdd);


protected:
	//----------------
	// Sub-components.
	//----------------
	gui::layouters::RowLayouter*  pLayouter;

	Label*       pLabelRecursionHandler;
	Label*       pLabelAutoReset;
	Label*       pLabelMaxDepth;
	Label*       pLabelIterationCount;
	Label*       pLabelIterationCountValue;
	Label*       pLabelElapsedTime;
	Label*       pLabelElapsedTimeValue;
	Label*       pLabelPrint;
	CheckBox*    pCheckBoxRecursionHandler;
	CheckBox*    pCheckBoxPrintEntryCounts;
	CheckBox*    pCheckBoxPrintNodeTimes;
	CheckBox*    pCheckBoxPrintBranchTimes;
	CheckBox*    pCheckBoxPrintNodePercentages;
	EditBox*     pEditBoxAutoReset;
	EditBox*     pEditBoxMaxDepth;
	TextButton*  pButtonClear;
	TextButton*  pButtonReset;
	TextButton*  pButtonDump;
	TextButton*  pButtonRunStop;

	//------------
	// Properties.
	//------------
	ProfilerDumpDesc  profilerDumpDesc;
	Profiler*         pProfiler;
	std::ostream*     pLog;
	double            updateInterval;

	//--------
	// States.
	//--------
	double  accumTime;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


