

//--------------------------------------------------------------------
#include "GUI_CS0\Separators.h"
#include "GUI_CS0\ShaderNames.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_Tools.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
HorzSeparator::GeomParams::GeomParams()
{
	preferredSize = vec2(100, 8);
	parentPadding = 0;
}


//====================================================================
HorzSeparator::HorzSeparator(gui::Component* _parent)
{
	SetShaderName(DefaultShaderName(Shader_HorzSeparator));
	SetGhost(true);
	SetGeomParams();

	if(_parent) AttachTo(*_parent);
}


//====================================================================
void  HorzSeparator::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;
	ReportDirtyGeometry();
}


//====================================================================
void  HorzSeparator::RecalcSize()
{
	SetSize(geomParams.preferredSize);
}


//====================================================================
void  HorzSeparator::RecalcLayout()
{
	SetSizeX(GetSize().x - 2 * geomParams.parentPadding);
	SetPositionX(geomParams.parentPadding);
}


//====================================================================
HorzTextSeparator::GeomParams::GeomParams()
{
	preferredSize = vec2(100, 24);
	parentPadding = 0;
	textHeight = 18;
	textOffsetLeft = 12;
	textPadding = 2;
}


//====================================================================
HorzTextSeparator::HorzTextSeparator(gui::Component* _parent,
									const std::string& _text)
{
	SetShaderName(DefaultShaderName(Shader_HorzTextSeparator));
	SetGhost(true);
	SetGeomParams();
	SetText(_text);

	if(_parent) AttachTo(*_parent);
}


//====================================================================
void  HorzTextSeparator::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;
	ReportDirtyGeometry();
}


//====================================================================
void  HorzTextSeparator::SetText(const std::string& _text)
{
	text = _text;
	ReportDirtyGeometry();
}


//====================================================================
void  HorzTextSeparator::GetTextInfo(float& _textPos, float& _textWidth) const
{
	_textPos = geomParams.textOffsetLeft;
	_textWidth = textWidth;
}


//====================================================================
void  HorzTextSeparator::GetLineSegmentInfo(float& _leftSegmentLen,
					float& _rightSegmentStart, float& _rightSegmentLen) const
{
	_leftSegmentLen = leftSegmentLen;
	_rightSegmentStart = rightSegmentStart;
	_rightSegmentLen = rightSegmentLen;
}


//====================================================================
void  HorzTextSeparator::RecalcSize()
{
	SetSize(geomParams.preferredSize);
}


//====================================================================
void  HorzTextSeparator::RecalcLayout()
{
	gui::ConstShaderPtr<HorzTextSeparator::Shader> pShader = GetBaseShader();
	if(! pShader) return;

	const gui::Font* pFont = pShader->GetFont();
	if(! pFont) return;

	textWidth = pFont->CalcTextWidth(pShader->GetTextMetricsL(geomParams.textHeight), text);

	SetSizeX(GetSize().x - 2 * geomParams.parentPadding);
	SetPositionX(geomParams.parentPadding);
}


//====================================================================
void  HorzTextSeparator::UpdateGeometryCache()
{
	leftSegmentLen = geomParams.textOffsetLeft - geomParams.textPadding;
	rightSegmentStart = geomParams.textOffsetLeft + textWidth + geomParams.textPadding;
	rightSegmentLen = GetSize().x - rightSegmentStart;
}


//====================================================================
ControlGroupFrame::GeomParams::GeomParams()
{
	textHeight = 18;
	textOffsetLeft = 12;
	textPadding = 2;
}


//====================================================================
ControlGroupFrame::ControlGroupFrame(gui::Component* _parent,
									const std::string& _text)
{
	SetShaderName(DefaultShaderName(Shader_ControlGroupFrame));
	SetGhost(true);

	SetGeomParams();
	SetText(_text);

	if(_parent) AttachTo(*_parent);
}


//====================================================================
void  ControlGroupFrame::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;
	ReportDirtyGeometry();
}


//====================================================================
void  ControlGroupFrame::SetText(const std::string& _text)
{
	text = _text;
	ReportDirtyGeometry();
}


//====================================================================
void  ControlGroupFrame::GetTextInfo(float& _textPos, float& _textWidth) const
{
	_textPos = geomParams.textOffsetLeft;
	_textWidth = textWidth;
}


//====================================================================
void  ControlGroupFrame::GetTopLineSegmentInfo(float& _leftSegmentLen,
					float& _rightSegmentStart, float& _rightSegmentLen) const
{
	_leftSegmentLen = leftSegmentLen;
	_rightSegmentStart = rightSegmentStart;
	_rightSegmentLen = rightSegmentLen;
}


//====================================================================
void  ControlGroupFrame::UpdateGeometryCache()
{
	gui::ConstShaderPtr<ControlGroupFrame::Shader> pShader = GetBaseShader();
	if(! pShader) return;

	const gui::Font* pFont = pShader->GetFont();
	if(! pFont) return;

	textWidth = pFont->CalcTextWidth(pShader->GetTextMetricsL(geomParams.textHeight), text);

	leftSegmentLen = geomParams.textOffsetLeft - geomParams.textPadding;
	rightSegmentStart = geomParams.textOffsetLeft + textWidth + geomParams.textPadding;
	rightSegmentLen = GetSize().x - rightSegmentStart;
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


