

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Component.h"
#include "GUI\GUI_Layouters.h"
#include "GUI\GUI_Shader.h"
#include "private\vw_gui_cs0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
class VW_GUI_CS0_API HorzSeparator : public gui::Component
{
	VW_GUI_COMPONENT_DECLARE_SHADER_DLL(VW_GUI_CS0_API, Shader);
	VW_GUI_COMPONENT_DECLARE_CHECK_SHADER(Shader);


public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		vec2   preferredSize;
		float  parentPadding;

		GeomParams();
	};


public:
	//-------------------------
	// HorzSeparator interface.
	//-------------------------
	HorzSeparator(gui::Component* parent = 0);

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void  SetHorzSeparatorPosition(float y) { SetPosition(geomParams.parentPadding, y); }


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  RecalcSize();
	virtual  void  RecalcLayout();


protected:
	//------------
	// Properties.
	//------------
	GeomParams  geomParams;
};


//====================================================================
class VW_GUI_CS0_API HorzTextSeparator : public gui::Component
{
	VW_GUI_COMPONENT_DECLARE_SHADER_DLL(VW_GUI_CS0_API, Shader);
	VW_GUI_COMPONENT_DECLARE_CHECK_SHADER(Shader);


public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		vec2   preferredSize;
		float  parentPadding;
		float  textHeight;
		float  textOffsetLeft;
		float  textPadding;

		GeomParams();
	};


public:
	//-----------------------------
	// HorzTextSeparator interface.
	//-----------------------------
	HorzTextSeparator(gui::Component* parent = 0,
					const std::string& text = "HorzTextSeparator");

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void                SetText(const std::string& text = "HorzTextSeparator");
	const std::string&  GetText() const { return text; }

	void  SetHorzTextSeparatorPosition(float y) { SetPosition(geomParams.parentPadding, y); }

	//-----------------------------------------------
	// Additional methods intended for use by shader.
	//-----------------------------------------------
	void  GetTextInfo(float& textPos, float& textWidth) const;
	void  GetLineSegmentInfo(float& leftSegmentLen, float& rightSegmentStart, float& rightSegmentLen) const;


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  RecalcSize();
	virtual  void  RecalcLayout();
	virtual  void  UpdateGeometryCache();


protected:
	//------------
	// Properties.
	//------------
	GeomParams   geomParams;
	std::string  text;

	//-------
	// Cache.
	//-------
	float  textWidth;
	float  leftSegmentLen;
	float  rightSegmentStart;
	float  rightSegmentLen;
};


//====================================================================
class VW_GUI_CS0_API ControlGroupFrame : public gui::layouters::RowLayouter
{
	VW_GUI_COMPONENT_DECLARE_SHADER_DLL(VW_GUI_CS0_API, Shader);
	VW_GUI_COMPONENT_DECLARE_CHECK_SHADER(Shader);


public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		float  textHeight;
		float  textOffsetLeft;
		float  textPadding;

		GeomParams();
	};


public:
	ControlGroupFrame(gui::Component* parent = 0,
					const std::string& text = "ControlGroupFrame");

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void                SetText(const std::string& text = "ControlGroupFrame");
	const std::string&  GetText() const { return text; }

	//-----------------------------------------------
	// Additional methods intended for use by shader.
	//-----------------------------------------------
	void  GetTextInfo(float& textPos, float& textWidth) const;
	void  GetTopLineSegmentInfo(float& leftSegmentLen, float& rightSegmentStart, float& rightSegmentLen) const;


protected:
	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  UpdateGeometryCache();


protected:
	//------------
	// Properties.
	//------------
	GeomParams   geomParams;
	std::string  text;

	//-------
	// Cache.
	//-------
	float  textWidth;
	float  leftSegmentLen;
	float  rightSegmentStart;
	float  rightSegmentLen;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


