

//--------------------------------------------------------------------
#include "GUI_CS0\ToolBar.h"
#include "BaseTools\STLExtensions.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
ToolBar::ToolBar(const std::string& _title, bool _bVertical)
	:
	TopLevelFrame(_title)
{
	pLayouter = new gui::layouters::ArrayLayouter(this);

	TopLevelFrame::GeomParams  gp;
	gp.bResizable = false;
	gp.minSize = vec2::ZERO;
	gp.maxSize = vec2::INFINITY;
	gp.titleBarHeight = 16;
	gp.titleTextHeight = 16;
	gp.titleOffsetLeft = 0;
	gp.frameBorderWidth = 2;
	gp.resizeAreaWidth = 0;
	gp.resizeCornerExtents = vec2::ZERO;
	gp.workAreaPaddingH = 4;
	gp.workAreaPaddingT = 4;
	gp.workAreaPaddingB = 4;
	gp.buttonFrameControlsOffsetRight = 0;
	gp.buttonFrameControlsSpacing = 0;
	gp.buttonFrameControlsSize = 16;
	gp.buttonFrameControlsIconSize = 14;

	TopLevelFrame::SetGeomParams(gp);
	SetTLFSubcomponentLayout(true, false, true);
	SetSize(60, 60);

	GeomParams  gpAL;
	gpAL.bVertical = _bVertical;
	if(_bVertical) gpAL.bResizeControlsToMax = true;
	SetGeomParams(gpAL);
}


//====================================================================
void  ToolBar::SetGeomParams(const GeomParams& gp)
{
	pLayouter->SetGeomParams(gp);
}


//====================================================================
const ToolBar::GeomParams&  ToolBar::GetGeomParams() const
{
	return pLayouter->GetGeomParams();
}


//====================================================================
void  ToolBar::AttachControl(gui::Component* c)
{
	if(c) pLayouter->Attach(*c);
}


//====================================================================
void  ToolBar::DetachControl(gui::Component* c)
{
	if(c) pLayouter->Detach(*c);
}


//====================================================================
void  ToolBar::DetachAllControls()
{
	pLayouter->DetachAllChildren();
}


//====================================================================
void  ToolBar::DeleteAllControls()
{
	pLayouter->DeleteAllChildren();
}


//====================================================================
void  ToolBar::ClearSpacers()
{
	pLayouter->ClearSpacers();
}


//====================================================================
void  ToolBar::AddSpacer(const gui::Component* pBefore, float length)
{
	pLayouter->AddSpacer(gui::layouters::Spacer(pBefore, length));
}


//====================================================================
void  ToolBar::RemoveSpacer(const gui::Component* pBefore)
{
	pLayouter->RemoveSpacer(pBefore);
}


//====================================================================
bool  ToolBar::OnCommand(uint commandID)
{
	if(TopLevelFrame::OnCommand(commandID)) return true;
	return ProcessToolBarControlCommand(commandID);
}


//====================================================================
void  ToolBar::RecalcLayout()
{
	// position controls & adjust own size
	pLayouter->PropagateRecalcLayout();
	pLayouter->SetPosition(CalcTLFWorkAreaOffset());

	SetSize(pLayouter->GetSize() + CalcTLFWorkAreaFrameSize());

	// continue with base-class after auto-resize is finished
	TopLevelFrame::RecalcLayout();
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


