

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI\GUI_Component.h"
#include "GUI\GUI_Shader.h"
#include "private\vw_gui_cs0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
// Operates as service-provider: SID_ToolTip
// Supported service-commands:
// 1) SCID_QueryParams: sent by ToolTip to mouse-hover-target
// 2) SCID_Reset: sent by user to reset mouse-hover-target cache
//====================================================================
class VW_GUI_CS0_API ToolTip : public gui::Component
{
public:
	VW_GUI_COMPONENT_DECLARE_SHADER_DLL(VW_GUI_CS0_API, Shader);
	VW_GUI_COMPONENT_DECLARE_CHECK_SHADER(Shader);


public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_CS0_API GeomParams
	{
		vec2  textPadding;

		GeomParams();
	};


public:
	//-------------------
	// ToolTip interface.
	//-------------------
	ToolTip();

	void    SetMinHoverTime(double secsMinHoverTime = 0.5f);
	double  GetMinHoverTime() const { return secsMinHoverTime; }

	void    SetFadeTimes(double secsFadeIn = 0.2f, double secsFadeOut = 0.2f);
	void    GetFadeTimes(double& secsFadeIn, double& secsFadeOut);

	void               SetGeomParams(const GeomParams& gp = GeomParams());
	const GeomParams&  GetGeomParams() const { return geomParams; }

	void                SetText(const std::string& text);
	const std::string&  GetText() const { return text; }

	void  Show(const vec2& position,
				const vec2& mouseMoveIgnoreAreaPosition,
				const vec2& mouseMoveIgnoreAreaSize);

	void  Hide();

	//-----------------------------------------------
	// Additional methods intended for use by shader.
	//-----------------------------------------------
	const std::vector<std::string>  GetTextLines() const { return textLines; }
	uint                            GetNumTextLines() const { return textLines.size(); }
	const std::string&              GetTextLine(uint i) const { return textLines[i]; }
	float                           GetTextLineWidth(uint i) const { return textLineWidths[i]; }
	const vec2&                     GetTextSize() const { return textSize; }
	ubyte                           GetTranslucency() const { return alphaCurrB; }


protected:
	//--------------------------
	// MessageHandler overrides.
	//--------------------------
	virtual  void  ServiceProviderPreEventDispatch(double secsTimeDelta);
	virtual  void  ServiceProviderPostEventDispatch(double secsTimeDelta);
	virtual  bool  HandleMessage(gui::Message& msg);

	//---------------------
	// Component overrides.
	//---------------------
	virtual  void  RecalcSize();

	//------------------
	// Internal helpers.
	//------------------
	enum eAnimationState
	{
		AnimationState_FadeIn = 0,
		AnimationState_Stable,
		AnimationState_FadeOut,

		AnimationState_Count,
		AnimationState_Force32Bits = 0x7FFFFFFF
	};

	void  UpdateTranslucencyCache() { alphaCurrB = ConvertColorComponent(alphaCurr); }


protected:
	//------------
	// Properties.
	//------------
	GeomParams   geomParams;
	std::string  text;
	double       secsMinHoverTime;
	double       secsFadeIn;
	double       secsFadeOut;

	//--------
	// States.
	//--------
	eAnimationState       animState;
	float                 alphaCurr;
	gui::Rectangle        mouseMoveIgnoreArea;
	gui::MessageHandler*  pLastHoverCheckTarget;
	gui::MessageHandler*  pLastQueryTarget;

	//-------
	// Cache.
	//-------
	float                     alphaPerSecFadeIn;
	float                     alphaPerSecFadeOut;
	ubyte                     alphaCurrB;
	std::vector<std::string>  textLines;
	std::vector<float>        textLineWidths;
	vec2                      textSize;
	gui::ToolTipParams        ttParams;
};


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


