

//--------------------------------------------------------------------
#include "GUI_CS0\TopLevelPane.h"
#include "GUI\GUI_Manager.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
gui::TextAttributes __fastcall TopLevelPane::DefaultTextAttributes()
{
	gui::TextAttributes ta;
	ta.fontName = gui::std_fonts::FontName_Default;
	ta.textMetricsL.textHeight = 16;
	ta.textMetricsL.bMonospace = true;
	return ta;
}


//====================================================================
TopLevelPane::GeomParams::GeomParams()
{
	titleTextHeight = 16;
	titleOffsetTopLeft = vec2(0, 0);
	buttonOffsetTopRight = vec2(0, 0);
	iconButtonPaneControlSize = 14;
	iconButtonPaneControlTileSize = 12;
};


//====================================================================
TopLevelPane::TopLevelPane(const std::string& _title)
{
	bTitleVisible = true;
	pFont = 0;

	SetVisible(false);
	SetCanFocus(true);

	pButtonClose = new IconButton(this, this, gui::CID_Close, gui::Button::Style_Push);
	pButtonClose->SetVisible(false);

	SetTitleText(_title);
	SetGeomParams();
	SetSize(240, 120);
	SetTextAttributes();
	SetTLPSubcomponentLayout();
	SetTLPSubcomponentToolTipText();
	SetTLPSubcomponentTileParams();
	SetTLPShaderNames();
	SetClosePaneOnKey();
}


//====================================================================
void  TopLevelPane::SetTextAttributes(const gui::TextAttributes& ta)
{
	textAtts = ta;
	RequestFontRecache();
}


//====================================================================
void  TopLevelPane::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;

	IconButton::GeomParams gpIconButton;
	gpIconButton.preferredSize = vec2(geomParams.iconButtonPaneControlSize, geomParams.iconButtonPaneControlSize);
	gpIconButton.iconSize = vec2(geomParams.iconButtonPaneControlTileSize, geomParams.iconButtonPaneControlTileSize);

	pButtonClose->SetGeomParams(gpIconButton);

	ReportDirtyGeometry();
}


//====================================================================
void  TopLevelPane::SetTLPShaderNames(const std::string& topLevelPane,
									const std::string& iconButtonPaneControl)
{
	SetShaderName(topLevelPane);
	pButtonClose->SetShaderName(iconButtonPaneControl);
}


//====================================================================
void  TopLevelPane::SetTLPSubcomponentLayout(
						bool _bShowTitle, bool _bHideTitleOnMouseExit,
						bool _bShowCloseButton, bool _bHideCloseButtonOnMouseExit)
{
	bShowTitle = _bShowTitle;
	bHideTitleOnMouseExit = _bHideTitleOnMouseExit;

	bTitleVisible = bShowTitle  &&  (IsMouseOver()  ||  ! bHideTitleOnMouseExit);

	bShowCloseButton = _bShowCloseButton;
	bHideCloseButtonOnMouseExit = _bHideCloseButtonOnMouseExit;

	pButtonClose->SetVisible(bShowCloseButton  &&  (IsMouseOver()  ||  ! bHideCloseButtonOnMouseExit));
}


//====================================================================
void  TopLevelPane::SetTLPSubcomponentToolTipText(const std::string& buttonClose)
{
	pButtonClose->SetToolTipText(buttonClose);
}


//====================================================================
void  TopLevelPane::SetTLPSubcomponentTileParams(const gui::TileParams& buttonClose)
{
	pButtonClose->SetTileParams(buttonClose);
}


//====================================================================
void  TopLevelPane::SetClosePaneOnKey(bool bCloseOnKey, eKey kc)
{
	bClosePaneOnKey = bCloseOnKey;
	keyCodeClose = kc;
}


//====================================================================
void  TopLevelPane::CalcTLPMainArea(float offsetY, float sizeY) const
{
	offsetY = Max(pButtonClose->CalcEndPositionY(),
					geomParams.titleOffsetTopLeft.y + geomParams.titleTextHeight);
	sizeY = GetSize().y - offsetY;
}


//====================================================================
bool  TopLevelPane::CacheFontHandles(gui::Manager& gm)
{
	if(textAtts.fontName.empty()) {
		pFont = 0;
		return true;
	}
	pFont = gm.GetFont(textAtts.fontName);
	return (pFont != 0);
}


//====================================================================
void  TopLevelPane::RecalcLayout()
{
	pButtonClose->SetPosition(
		GetSize().x - pButtonClose->GetSize().x - geomParams.buttonOffsetTopRight.x,
		geomParams.buttonOffsetTopRight.y);
}


//====================================================================
void  TopLevelPane::OnChangedVisible()
{
	if(IsVisible()) {
		OnPaneOpened();
	}
}


//====================================================================
bool  TopLevelPane::OnCommand(uint commandID)
{
	if(commandID == gui::CID_Close) {

		ShowTopLevel(false);
		OnPaneClosed();
		return true;
	}
	return false;
}


//====================================================================
bool  TopLevelPane::OnDeactivate(bool bDeactivateByMouse)
{
	if(bShowCloseButton  &&  bHideCloseButtonOnMouseExit) {
		pButtonClose->SetVisible(false);
	}
	if(bShowTitle  &&  bHideTitleOnMouseExit) {
		bTitleVisible = false;
	}
	return true;
}


//====================================================================
bool  TopLevelPane::OnMouseEnterExit(bool bEnter, bool bMouseOverDirect, const vec2& mousePos)
{
	if(bEnter) {
		if(bShowCloseButton) {
			pButtonClose->SetVisible(true);
		}
		if(bShowTitle) {
			bTitleVisible = true;
		}
	}
	else {
		if(bShowCloseButton  &&  bHideCloseButtonOnMouseExit) {
			pButtonClose->SetVisible(false);
		}
		if(bShowTitle  &&  bHideTitleOnMouseExit) {
			bTitleVisible = false;
		}
	}
	return true;
}


//====================================================================
bool  TopLevelPane::OnMouseButton(bool bDown, eMouseButton buttonCode, bool bDoubleClick, uint repeatCount, const vec2& mousePos)
{
	if(bDown  &&  buttonCode == MouseButton_Left  &&  bDoubleClick) {
		OnPaneDock();
		return true;
	}
	return false;
}


//====================================================================
bool  TopLevelPane::OnKeyboardQueryFocus(bool bFocusByMouse, eMouseButton buttonCode, bool& bWantFocus, bool& bNeedCharacterData)
{
	DefaultProcessQueryFocus(bFocusByMouse, buttonCode, bWantFocus, bNeedCharacterData);
	return true;
}


//====================================================================
bool  TopLevelPane::OnKeyboardKey(bool bDown, eKey keyCode, uint repeatCount, char ch)
{
	if(! bDown  ||  repeatCount) return false;

	if(bClosePaneOnKey  &&  keyCode == keyCodeClose) {
		SendMessageToSelf(gui::Message_Command(gui::CID_Close));
		return true;
	}
	return false;
}


//====================================================================
void  TopLevelPane::OnPaneDock()
{
	gui::Manager& gm = gui::Manager::Instance();
	OnWindowResize(gm.GetWindowWidth(), gm.GetWindowHeight());
	UpdateGeometry();
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


