

//--------------------------------------------------------------------
#include "GUI_CS0\VideoConfigDialog.h"
#include "Framework\Application.h"


//====================================================================
#define   DEFAULT_FREQ_ID   -1


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_cs0 {


//====================================================================
VideoConfigDialog::GeomParams::GeomParams()
{
	smallCheckBoxSize = 14;
	smallLabelTextHeight = 14;
}


//====================================================================
VideoConfigDialog::VideoConfigDialog(const std::string& _title,
									uint _videoReconfigCode)
	:
	Dialog(_title)
{
	SetVideoReconfigCode(_videoReconfigCode);

	pLayouter = new gui::layouters::RowLayouter(this);

	pCheckBoxDynamicVSync = new CheckBox(pLayouterStdDialogControls, this, CID_DynamicVSync);
	pLabelDynamicVSync = new Label(pLayouterStdDialogControls, 0, 0, "DynamicVSync", pCheckBoxDynamicVSync);

	// renderer
	pHorzTextSeparatorRenderer = new HorzTextSeparator(pLayouter, "Renderer");
	pComboBoxRenderer = new ComboBox(pLayouter, this, CID_Renderer);
	pCheckBoxHardware = new CheckBox(pLayouter, this, CID_Hardware);
	pLabelHardware = new Label(pLayouter, 0, 0, "Hardware", pCheckBoxHardware);

	// display
	pHorzTextSeparatorDisplay = new HorzTextSeparator(pLayouter, "Display");
	pLabelDisplayMode = new Label(pLayouter, 0, 0, "Mode:");
	pComboBoxDisplayMode = new ComboBox(pLayouter, this, CID_DisplayMode);
	pLabelRefreshFreq = new Label(pLayouter, 0, 0, "Refresh:");
	pComboBoxRefreshFreq = new ComboBox(pLayouter, this, CID_Refresh);
	pCheckBoxFullscreen = new CheckBox(pLayouter, this, CID_Fullscreen);
	pLabelFullscreen = new Label(pLayouter, 0, 0, "Fullscreen", pCheckBoxFullscreen);
	pCheckBoxVSync = new CheckBox(pLayouter, this, CID_VSync);
	pLabelVSync = new Label(pLayouter, 0, 0, "VSync", pCheckBoxVSync);
	pCheckBoxTripple = new CheckBox(pLayouter, this, CID_Tripple);
	pLabelTripple = new Label(pLayouter, 0, 0, "TrippleBuffer", pCheckBoxTripple);

	// developer controls
	pHorzTextSeparatorDeveloper = new HorzTextSeparator(pLayouter, "Developer");
	pLabelFrameBuffer = new Label(pLayouter, 0, 0, "Buffers:");
	pComboBoxFrameBuffer = new ComboBox(pLayouter);
	pLabelVertProc = new Label(pLayouter, 0, 0, "TNL:");
	pComboBoxVertProc = new ComboBox(pLayouter, this, CID_VertProc);

	for(uint i = 0; i < VideoHints::VertexProcessingMode_Count; i++) {
		pComboBoxVertProc->AddItem(new ComboBoxItem(
			VideoHints::FormatVertexProcessingMode(VideoHints::eVertexProcessingMode(i)), i));
	}

	pCheckBoxResizable = new CheckBox(pLayouter, this, CID_Resizable);
	pLabelResizable = new Label(pLayouter, 0, 0, "Resizable", pCheckBoxResizable);
	pCheckBoxPure = new CheckBox(pLayouter, this, CID_Pure);
	pLabelPure = new Label(pLayouter, 0, 0, "PureDevice", pCheckBoxPure);
	pCheckBoxDestAlpha = new CheckBox(pLayouter, this, CID_DestAlpha);
	pLabelDestAlpha = new Label(pLayouter, 0, 0, "EnableDestAlpha", pCheckBoxDestAlpha);

	// default params
	KernelExecutionParams kep;

	SetVideoConfigDialogSubcomponentLayout(kep.bDeveloperMode);
	SetVideoConfigDialogShaderNames();
	SetVideoConfigDialogSubcomponentToolTipText();
	SetGeomParams();

	BuildLayout();
	SetWorkAreaLayouter(pLayouter);

	OnDialogOpened();
	Kernel::Instance().RegisterReconfigObserver(*this);
}


//====================================================================
void  VideoConfigDialog::SetGeomParams(const GeomParams& gp)
{
	geomParams = gp;

	Label::GeomParams  gpLabel;
	gpLabel.textHeight = geomParams.smallLabelTextHeight;
	gpLabel.minSize = vec2::ZERO;
	pLabelDynamicVSync->SetGeomParams(gpLabel);

	CheckBox::GeomParams  gpCheck;
	gpCheck.preferredSize = vec2(geomParams.smallCheckBoxSize, geomParams.smallCheckBoxSize);
	pCheckBoxDynamicVSync->SetGeomParams(gpCheck);

	ReportDirtyGeometry();
}


//====================================================================
void  VideoConfigDialog::SetVideoConfigDialogShaderNames(
										const std::string& label,
										const std::string& labelCB,
										const std::string& checkBox,
										const std::string& comboBox,
										const std::string& horzTextSeparator)
{
	pHorzTextSeparatorRenderer->SetShaderName(horzTextSeparator);
	pLabelHardware->SetShaderName(labelCB);
	pCheckBoxHardware->SetShaderName(checkBox);
	pComboBoxRenderer->SetShaderName(comboBox);

	pHorzTextSeparatorDisplay->SetShaderName(horzTextSeparator);
	pLabelDisplayMode->SetShaderName(label);
	pLabelRefreshFreq->SetShaderName(label);
	pLabelFullscreen->SetShaderName(labelCB);
	pLabelVSync->SetShaderName(labelCB);
	pLabelTripple->SetShaderName(labelCB);
	pCheckBoxFullscreen->SetShaderName(checkBox);
	pCheckBoxVSync->SetShaderName(checkBox);
	pCheckBoxTripple->SetShaderName(checkBox);
	pComboBoxDisplayMode->SetShaderName(comboBox);
	pComboBoxRefreshFreq->SetShaderName(comboBox);

	pHorzTextSeparatorDeveloper->SetShaderName(horzTextSeparator);
	pLabelFrameBuffer->SetShaderName(label);
	pLabelVertProc->SetShaderName(label);
	pLabelResizable->SetShaderName(labelCB);
	pLabelPure->SetShaderName(labelCB);
	pLabelDestAlpha->SetShaderName(labelCB);
	pCheckBoxResizable->SetShaderName(checkBox);
	pCheckBoxPure->SetShaderName(checkBox);
	pCheckBoxDestAlpha->SetShaderName(checkBox);
	pComboBoxFrameBuffer->SetShaderName(comboBox);
	pComboBoxVertProc->SetShaderName(comboBox);

	pLabelDynamicVSync->SetShaderName(labelCB);
	pCheckBoxDynamicVSync->SetShaderName(checkBox);
}


//====================================================================
void  VideoConfigDialog::SetVideoConfigDialogSubcomponentLayout(bool bShowDeveloperControls)
{
	pHorzTextSeparatorDeveloper->SetVisible(bShowDeveloperControls);
	pLabelFrameBuffer->SetVisible(bShowDeveloperControls);
	pComboBoxFrameBuffer->SetVisible(bShowDeveloperControls);
	pLabelVertProc->SetVisible(bShowDeveloperControls);
	pComboBoxVertProc->SetVisible(bShowDeveloperControls);
	pLabelResizable->SetVisible(bShowDeveloperControls);
	pCheckBoxResizable->SetVisible(bShowDeveloperControls);
	pLabelPure->SetVisible(bShowDeveloperControls);
	pCheckBoxPure->SetVisible(bShowDeveloperControls);
	pLabelDestAlpha->SetVisible(bShowDeveloperControls);
	pCheckBoxDestAlpha->SetVisible(bShowDeveloperControls);

	TopLevelFrame::GeomParams gp;
	gp.bResizable = true;
	if(bShowDeveloperControls) {
		gp.minSize = vec2(460, 320);
		gp.maxSize = vec2(620, 340);
	}
	else {
		gp.minSize = vec2(460, 230);
		gp.maxSize = vec2(620, 250);
	}
	TopLevelFrame::SetGeomParams(gp);
	SetSize(GetMinSize());

	// Specify layout
	using namespace gui::layouters;
	RowLayouter& l = *pLayouter;

	float ds = 2 * Cell::DefaultWidth;

	l.Clear();
	l.TargetHeightRow(pHorzTextSeparatorRenderer).Target(pHorzTextSeparatorRenderer, SF_Variable);
	l.FixedHeightRow().Target(pCheckBoxHardware).Spacer().Target(pLabelHardware).Spacer(ds).Target(pComboBoxRenderer, SF_Variable);

	l.TargetHeightRow(pHorzTextSeparatorDisplay).Target(pHorzTextSeparatorDisplay, SF_Variable);
	l.FixedHeightRow().Target(pLabelDisplayMode).Spacer().Target(pComboBoxDisplayMode, SF_Variable, 0.6f);
	l.LastRow().AutoSpacer().Target(pLabelRefreshFreq).Spacer().Target(pComboBoxRefreshFreq, SF_Variable, 0.3f);
	l.FixedHeightRow().Target(pCheckBoxFullscreen).Spacer().Target(pLabelFullscreen);
	l.LastRow().AutoSpacer().Target(pCheckBoxTripple).Spacer().Target(pLabelTripple);
	l.LastRow().AutoSpacer().Target(pCheckBoxVSync).Spacer().Target(pLabelVSync);

	l.TargetHeightRow(pHorzTextSeparatorDeveloper).Target(pHorzTextSeparatorDeveloper, SF_Variable);
	l.FixedHeightRow().Target(pLabelFrameBuffer).Spacer().Target(pComboBoxFrameBuffer, SF_Variable, 0.6f);
	l.LastRow().AutoSpacer().Target(pLabelVertProc).Spacer().Target(pComboBoxVertProc, SF_Variable, 0.3f);
	l.FixedHeightRow().Target(pCheckBoxResizable).Spacer().Target(pLabelResizable);
	l.LastRow().AutoSpacer().Target(pCheckBoxPure).Spacer().Target(pLabelPure);
	l.LastRow().AutoSpacer().Target(pCheckBoxDestAlpha).Spacer().Target(pLabelDestAlpha);

	ReportDirtyGeometry();
}


//====================================================================
void  VideoConfigDialog::SetVideoConfigDialogSubcomponentToolTipText(
							const std::string& sRenderer, const std::string& sHardware,
							const std::string& sDisplayMode, const std::string& sRefreshFreq,
							const std::string& sFullscreen, const std::string& sTripple,
							const std::string& sVSync, const std::string& sFrameBuffer,
							const std::string& sVertProc, const std::string& sResizable,
							const std::string& sPure, const std::string& sDestAlpha,
							const std::string& sDynamicVSync)
{
	pComboBoxRenderer->SetToolTipText(sRenderer);
	pCheckBoxHardware->SetToolTipText(sHardware);
	pComboBoxDisplayMode->SetToolTipText(sDisplayMode);
	pComboBoxRefreshFreq->SetToolTipText(sRefreshFreq);
	pCheckBoxFullscreen->SetToolTipText(sFullscreen);
	pCheckBoxTripple->SetToolTipText(sTripple);
	pCheckBoxVSync->SetToolTipText(sVSync);
	pComboBoxFrameBuffer->SetToolTipText(sFrameBuffer);
	pComboBoxVertProc->SetToolTipText(sVertProc);
	pCheckBoxResizable->SetToolTipText(sResizable);
	pCheckBoxPure->SetToolTipText(sPure);
	pCheckBoxDestAlpha->SetToolTipText(sDestAlpha);
	pCheckBoxDynamicVSync->SetToolTipText(sDynamicVSync);
}


//====================================================================
void  VideoConfigDialog::OnVideoReconfig()
{
	OnDialogOpened();
}


//====================================================================
void  VideoConfigDialog::OnVSyncReconfig(bool bVSyncEnabled)
{
	OnDialogOpened();
}


//====================================================================
bool  VideoConfigDialog::OnCommand(uint commandID)
{
	switch(commandID) {

		case CID_Renderer:
			SelectSwitchesClosestToActive();
			RefillDisplayModes();
			SelectDisplayModeClosestToActive();
			RefillRefreshFreq();
			SelectRefreshClosestToActive();
			RefillFrameBuffer();
			SelectFrameBufferClosestToActive();
			SelectVertProcClosestToActive();
			UpdateStandardButtons();
			return true;

		case CID_DisplayMode:
			RefillRefreshFreq();
			SelectRefreshClosestToActive();
			UpdateStandardButtons();
			return true;

		case CID_Hardware:
		case CID_Fullscreen:
		case CID_Tripple:
			RefillFrameBuffer();
			SelectFrameBufferClosestToActive();
			UpdateStandardButtons();
			return true;

		case CID_VSync:
		case CID_Refresh:
		case CID_FrameBuffer:
		case CID_VertProc:
		case CID_Resizable:
		case CID_Pure:
		case CID_DestAlpha:
			UpdateStandardButtons();
			return true;

		case CID_DynamicVSync:
			VideoRenderer::EnableDynamicVSyncUpdate(pCheckBoxDynamicVSync->IsOn());
			return true;
	}
	return Dialog::OnCommand(commandID);
}


//====================================================================
void  VideoConfigDialog::OnStdDialogControlsLayouterUpdate()
{
	using namespace gui::layouters;
	RowLayouter& l = *pLayouterStdDialogControls;

	Row r;
	r.Target(pCheckBoxDynamicVSync).Spacer(geomParams.smallCheckBoxSize * 0.5f).Target(pLabelDynamicVSync);
	l.LastRow().Prepend(r);
}


//====================================================================
void  VideoConfigDialog::OnDialogOpened()
{
	pCheckBoxDynamicVSync->SetOn(VideoRenderer::IsEnabledDynamicVSyncUpdate());

	RefillRenderers();
	SelectActiveRenderer();

	SelectSwitchesClosestToActive();

	RefillDisplayModes();
	SelectDisplayModeClosestToActive();

	RefillRefreshFreq();
	SelectRefreshClosestToActive();

	RefillFrameBuffer();
	SelectFrameBufferClosestToActive();

	SelectVertProcClosestToActive();

	UpdateStandardButtons();
	pButtonApply->SetEnabled(false);
}


//====================================================================
void  VideoConfigDialog::OnApplySettings()
{
	// get selected renderer
	int iSelRend = pComboBoxRenderer->GetSelectedItemIndex();
	if(iSelRend < 0) return;
	uint iRend = pComboBoxRenderer->GetItem(iSelRend).GetID();

	// cache selected-renderer info
	VideoRendererInfo  vrInfoSelected;
	Kernel::Instance().GetVideoRendererInfo(iRend, vrInfoSelected);
	const VideoProfile& vpSelected = vrInfoSelected.GetVideoProfile();

	// get selected display mode
	int iSelDMP = pComboBoxDisplayMode->GetSelectedItemIndex();
	if(iSelDMP < 0) return;
	uint iDMP = pComboBoxDisplayMode->GetItem(iSelDMP).GetID();
	const DisplayModeProperties& dmpSelected = vpSelected.GetDisplayModeProperties(iDMP);

	// get selected refresh frequency
	int iSelFreq = pComboBoxRefreshFreq->GetSelectedItemIndex();
	if(iSelFreq < 0) return;
	int iFreq = pComboBoxRefreshFreq->GetItem(iSelFreq).GetID();
	uint freqSelected = (iFreq == DEFAULT_FREQ_ID) ? 0 : dmpSelected.GetRefreshFrequency(iFreq);

	// get selected frame-buffer-config
	int iSelFBC = pComboBoxFrameBuffer->GetSelectedItemIndex();
	if(iSelFBC < 0) return;
	uint iFBC = pComboBoxFrameBuffer->GetItem(iSelFBC).GetID();
	const FrameBufferConfig& fbcSelected = vpSelected.GetFrameBufferConfig(iFBC);

	// get selected vertex-processing-mode
	int iSelVPM = pComboBoxVertProc->GetSelectedItemIndex();
	if(iSelVPM < 0) return;
	uint iVPM = pComboBoxVertProc->GetItem(iSelVPM).GetID();

	// fill video configuration
	VideoConfig vConfig;

	vConfig.displayModeConfig.width = dmpSelected.width;
	vConfig.displayModeConfig.height = dmpSelected.height;
	vConfig.displayModeConfig.bitsPerPixel = dmpSelected.bitsPerPixel;
	vConfig.displayModeConfig.refreshFrequency = freqSelected;
	vConfig.frameBufferConfig = fbcSelected;
	vConfig.resizable = pCheckBoxResizable->IsOn();
	vConfig.videoHints.vSync = pCheckBoxVSync->IsOn();
	vConfig.videoHints.pureDeviceMode = pCheckBoxPure->IsOn();
	vConfig.videoHints.fullscreenDestinationAlpha = pCheckBoxDestAlpha->IsOn();
	vConfig.videoHints.vertexProcessingMode = VideoHints::eVertexProcessingMode(iVPM);

	// active params
	VideoRendererInfo  vrInfoActive;
	VideoConfig        vConfigActive;
	if(! GetActiveRendererInfo(vrInfoActive, vConfigActive)) return;

	std::string  vrPluginName = vrInfoSelected.GetPluginInfo().GetPluginName();

	// give derived class a chance to override new settings
	OnPreApplySettings(vrPluginName, vConfig);

	if(vrInfoActive.GetPluginInfo().GetPluginName() == vrPluginName) {

		// check for null-update
		if(vConfig == vConfigActive) return;

		// check for vsync update only
		if(vConfig.IsEqualIgnoreVSync(vConfigActive)) {

			VideoRenderer* pvr = Kernel::Instance().GetActiveVideoRendererPtr();
			if(pvr) pvr->UpdateVSync(vConfig.videoHints.vSync);
			return;
		}
	}

	// full reconfig
	Kernel::Instance().RequestVideoReconfig(vrPluginName, vConfig, iVideoReconfigCode);
}


//====================================================================
bool  VideoConfigDialog::GetActiveRendererInfo(VideoRendererInfo& vri, VideoConfig& vc)
{
	if(! Kernel::Instance().GetActiveVideoRendererInfo(vri)) return false;
	VideoRenderer* pvr = Kernel::Instance().GetActiveVideoRendererPtr();
	if(! pvr) return false;
	vc = pvr->GetVideoConfig();
	return true;
}


//====================================================================
bool  VideoConfigDialog::GetSelectedRendererVideoProfile(VideoProfile& vp)
{
	int iSel = pComboBoxRenderer->GetSelectedItemIndex();
	if(iSel < 0) return false;

	VideoRendererInfo vrInfo;
	Kernel::Instance().GetVideoRendererInfo(iSel, vrInfo);

	vp = vrInfo.GetVideoProfile();
	return true;
}


//====================================================================
void  VideoConfigDialog::RefillRenderers()
{
	VideoRenderer* pvr = Kernel::Instance().GetActiveVideoRendererPtr();
	if(! pvr) return;

	pComboBoxRenderer->DeleteAllItems();

	VideoRendererInfo vrInfo;
	for(uint i = 0; i < Kernel::Instance().GetNumVideoRenderers(); i++) {

		Kernel::Instance().GetVideoRendererInfo(i, vrInfo);

		pComboBoxRenderer->AddItem(new ComboBoxItem(vrInfo.GetPluginInfo().GetPluginName(), i));
	}
}


//====================================================================
void  VideoConfigDialog::RefillDisplayModes()
{
	pComboBoxDisplayMode->DeleteAllItems();

	VideoProfile vp;
	if(! GetSelectedRendererVideoProfile(vp)) return;

	for(uint i = 0; i < vp.GetNumDisplayModeProperties(); i++) {
		const DisplayModeProperties& dmp = vp.GetDisplayModeProperties(i);

		pComboBoxDisplayMode->AddItem(new ComboBoxItem(dmp.MakeDesc(), i));
	}
}


//====================================================================
void  VideoConfigDialog::RefillRefreshFreq()
{
	pComboBoxRefreshFreq->DeleteAllItems();

	VideoProfile vp;
	if(! GetSelectedRendererVideoProfile(vp)) return;

	int iSelDM = pComboBoxDisplayMode->GetSelectedItemIndex();
	if(iSelDM < 0) return;

	// default frequency
	pComboBoxRefreshFreq->AddItem(new ComboBoxItem("Default", DEFAULT_FREQ_ID));

	const DisplayModeProperties& dmp = vp.GetDisplayModeProperties(iSelDM);
	for(uint i = 0; i < dmp.GetNumRefreshFrequencies(); i++) {
		uint freq = dmp.GetRefreshFrequency(i);
		pComboBoxRefreshFreq->AddItem(new ComboBoxItem(ToString(freq), i));
	}
}


//====================================================================
void  VideoConfigDialog::RefillFrameBuffer()
{
	pComboBoxFrameBuffer->DeleteAllItems();

	VideoProfile vp;
	if(! GetSelectedRendererVideoProfile(vp)) return;

	for(uint i = 0; i < vp.GetNumFrameBufferConfigs(); i++) {
		const FrameBufferConfig& fbc = vp.GetFrameBufferConfig(i);

		if(fbc.hardwareRasterizer != pCheckBoxHardware->IsOn()) continue;
		if(fbc.fullscreen != pCheckBoxFullscreen->IsOn()) continue;

		switch(fbc.backBufferCount) {
			case 1:
				if(pCheckBoxTripple->IsOn()) continue;
				break;
			case 2:
				if(! pCheckBoxTripple->IsOn()) continue;
				break;
			default:
				continue;
		}
		pComboBoxFrameBuffer->AddItem(new ComboBoxItem(fbc.MakeLayoutDesc(), i));
	}
}


//====================================================================
void  VideoConfigDialog::SelectActiveRenderer()
{
	VideoRendererInfo  vrInfoActive;
	VideoConfig        vConfigActive;
	if(! GetActiveRendererInfo(vrInfoActive, vConfigActive)) return;

	// match plugin-name
	for(uint iItem = 0; iItem < pComboBoxRenderer->GetNumItems(); iItem++) {
		if(pComboBoxRenderer->GetItem(iItem).GetText() == vrInfoActive.GetPluginInfo().GetPluginName()) {
			pComboBoxRenderer->SelectItem(iItem);
			return;
		}
	}
}


//====================================================================
void  VideoConfigDialog::SelectSwitchesClosestToActive()
{
	VideoRendererInfo  vrInfoActive;
	VideoConfig        vConfigActive;
	if(! GetActiveRendererInfo(vrInfoActive, vConfigActive)) return;

	pCheckBoxHardware->SetOn(vConfigActive.frameBufferConfig.hardwareRasterizer);
	pCheckBoxFullscreen->SetOn(vConfigActive.frameBufferConfig.fullscreen);
	pCheckBoxTripple->SetOn(vConfigActive.frameBufferConfig.backBufferCount == 2);
	pCheckBoxVSync->SetOn(vConfigActive.videoHints.vSync);
	pCheckBoxResizable->SetOn(vConfigActive.resizable);
	pCheckBoxPure->SetOn(vConfigActive.videoHints.pureDeviceMode);
	pCheckBoxDestAlpha->SetOn(vConfigActive.videoHints.fullscreenDestinationAlpha);
	pComboBoxVertProc->SelectItem(vConfigActive.videoHints.vertexProcessingMode);
}


//====================================================================
void  VideoConfigDialog::SelectDisplayModeClosestToActive()
{
	VideoRendererInfo  vrInfoActive;
	VideoConfig        vConfigActive;
	if(! GetActiveRendererInfo(vrInfoActive, vConfigActive)) return;

	VideoProfile  vpSelected;
	if(! GetSelectedRendererVideoProfile(vpSelected)) return;

	// find equal or first lower resolution
	uint iDiffW = UINT_MAX;
	int  iItemSel = -1;

	for(uint iItem = 0; iItem < pComboBoxDisplayMode->GetNumItems(); iItem++) {

		const DisplayModeProperties& dmp = vpSelected.GetDisplayModeProperties(
									pComboBoxDisplayMode->GetItem(iItem).GetID());

		// match bits-per-pixel
		if(dmp.bitsPerPixel != vConfigActive.displayModeConfig.bitsPerPixel) continue;

		// match width
		if(dmp.width > vConfigActive.displayModeConfig.width) continue;

		uint diffW = vConfigActive.displayModeConfig.width - dmp.width;
		if(diffW < iDiffW) {
			iDiffW = diffW;
			iItemSel = iItem;
		}
	}

	// select closest item
	if(iItemSel < 0) iItemSel = 0;
	pComboBoxDisplayMode->SelectItem(iItemSel);
}


//====================================================================
void  VideoConfigDialog::SelectRefreshClosestToActive()
{
	VideoRendererInfo  vrInfoActive;
	VideoConfig        vConfigActive;
	if(! GetActiveRendererInfo(vrInfoActive, vConfigActive)) return;

	VideoProfile  vpSelected;
	if(! GetSelectedRendererVideoProfile(vpSelected)) return;

	int iDMPSelected = pComboBoxDisplayMode->GetSelectedItemIndex();
	if(iDMPSelected < 0) return;
	const DisplayModeProperties& dmpSelected = vpSelected.GetDisplayModeProperties(
								pComboBoxDisplayMode->GetItem(iDMPSelected).GetID());

	// find equal or set default frequency
	int iItemSel = -1;

	for(uint iItem = 0; iItem < pComboBoxRefreshFreq->GetNumItems(); iItem++) {

		uint iItemFreq = 0;

		int iUserData = pComboBoxRefreshFreq->GetItem(iItem).GetID();
		if(iUserData != DEFAULT_FREQ_ID) {
			iItemFreq = dmpSelected.GetRefreshFrequency(iUserData);
		}

		if(iItemFreq == vConfigActive.displayModeConfig.refreshFrequency) {
			iItemSel = iItem;
			break;
		}
	}

	// select closest item or default (0)
	if(iItemSel < 0) iItemSel = 0;
	pComboBoxRefreshFreq->SelectItem(iItemSel);
}


//====================================================================
void  VideoConfigDialog::SelectFrameBufferClosestToActive()
{
	VideoRendererInfo  vrInfoActive;
	VideoConfig        vConfigActive;
	if(! GetActiveRendererInfo(vrInfoActive, vConfigActive)) return;

	VideoProfile  vpSelected;
	if(! GetSelectedRendererVideoProfile(vpSelected)) return;

	// re-filter video-profile to match switches
	struct DMCFilter : public DisplayModeConfigFilter {
		virtual bool Filter(const DisplayModeConfig& dmc) { return true; }
	};

	struct FBCFilter : public FrameBufferConfigFilter {
		bool hw, fs;
		uint bbc;

		FBCFilter(bool _hw, bool _fs, uint _bbc) {
			hw = _hw;
			fs = _fs;
			bbc = _bbc;
		}
		virtual bool Filter(const FrameBufferConfig& fbc) {
			return (fbc.hardwareRasterizer == hw) &&
			(fbc.fullscreen == fs) && (fbc.backBufferCount == bbc);
		}
	};

	VideoProfile  vpFiltered;
	vpSelected.Filter(vpFiltered, DMCFilter(), FBCFilter(pCheckBoxHardware->IsOn(),
						pCheckBoxFullscreen->IsOn(), (pCheckBoxTripple->IsOn() ? 2 : 1)));

	// find closest matching
	const FrameBufferConfig* pFBCClosest = vpFiltered.SelectClosestFrameBufferConfig(
													vConfigActive.frameBufferConfig);
	int iSelFBC = 0;
	if(pFBCClosest) {
		for(uint iItem = 0; iItem < pComboBoxFrameBuffer->GetNumItems(); iItem++) {

			int iFBC = pComboBoxFrameBuffer->GetItem(iItem).GetID();
			if(iFBC < 0) continue;

			if(vpSelected.GetFrameBufferConfig(iFBC) == *pFBCClosest) {
				iSelFBC = iItem;
				break;
			}
		}
	}
	pComboBoxFrameBuffer->SelectItem(iSelFBC);
}


//====================================================================
void  VideoConfigDialog::SelectVertProcClosestToActive()
{
	VideoRendererInfo  vrInfoActive;
	VideoConfig        vConfigActive;
	if(! GetActiveRendererInfo(vrInfoActive, vConfigActive)) return;

	VideoProfile  vpSelected;
	if(! GetSelectedRendererVideoProfile(vpSelected)) return;

	// find equal vertex-processing mode
	int iItemSel = -1;

	for(uint iItem = 0; iItem < pComboBoxVertProc->GetNumItems(); iItem++) {
		uint iItemVPM = pComboBoxVertProc->GetItem(iItem).GetID();

		if(VideoHints::eVertexProcessingMode(iItemVPM) == vConfigActive.videoHints.vertexProcessingMode) {
			iItemSel = iItem;
			break;
		}
	}

	// select closest item
	if(iItemSel < 0) iItemSel = 0;
	pComboBoxVertProc->SelectItem(iItemSel);
}


//====================================================================
void  VideoConfigDialog::UpdateStandardButtons()
{
	bool bEnable = true;

	if(! pComboBoxRenderer->GetNumItems()) bEnable = false;
	if(! pComboBoxDisplayMode->GetNumItems()) bEnable = false;
	if(! pComboBoxRefreshFreq->GetNumItems()) bEnable = false;
	if(! pComboBoxFrameBuffer->GetNumItems()) bEnable = false;

	pButtonOK->SetEnabled(bEnable);
	pButtonApply->SetEnabled(bEnable);
}


//********************************************************************
}//namespace gui_cs0


//********************************************************************
}//namespace vertexware


