

//--------------------------------------------------------------------
#include "GUI_SS0\Buttons.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_Tools.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
ShaderParams_Button_PushOpaque::ShaderParams_Button_PushOpaque()
{
	colorDisabledBorder = color4b::LIGHT_GRAY;
	colorDisabledGlyph = color4b::LIGHT_GRAY;
	colorDisabledBgr = color4b::GRAY;
	colorEnabledBorder = color4b::BLACK;
	colorEnabledBgrTop = color4b(128, 128, 128);
	colorEnabledBgrBottom = color4b(192, 192, 192);
	colorEnabledGlyph = color4b::BLACK;
	colorPressedBorder = colorEnabledBorder;
	colorPressedBgrTop = color4b(64, 64, 64);
	colorPressedBgrBottom = color4b(128, 128, 128);
	colorPressedGlyph = color4b::YELLOW;
	colorHotBorder = color4b::YELLOW;
	pressOffset = vec2(1, 1);
	mainBorderWidth = 1;
	hotBorderWidth = 2;
}


//====================================================================
void  ShaderParams_Button_PushOpaque::ChooseParams(
									const gui::Button& button,
									color4b& cTop, color4b& cBottom,
									color4b& cGlyph, color4b& cMainBorder,
									vec2& pressOff) const
{
	// choose offset
	pressOff.Set(0, 0);

	if(button.IsPressed()  &&  (button.GetStyle() == gui::Button::Style_Push)) {
		pressOff = pressOffset;
	}

	// choose colors
	if(button.IsEnabled()) {
		cGlyph = colorEnabledGlyph;

		if(button.IsPressed()) {
			cMainBorder = colorPressedBorder;
			cTop = colorPressedBgrTop;
			cBottom = colorPressedBgrBottom;

			if(button.GetStyle() == gui::Button::Style_Press) {
				cGlyph = colorPressedGlyph;
			}
		}
		else {
			cMainBorder = colorEnabledBorder;
			cTop = colorEnabledBgrTop;
			cBottom = colorEnabledBgrBottom;
		}
	}
	else {
		cTop = colorDisabledBgr;
		cBottom = colorDisabledBgr;
		cGlyph = colorDisabledGlyph;
		cMainBorder = colorDisabledBorder;
	}
}


//====================================================================
ShaderParams_Button_PushTranslucent::ShaderParams_Button_PushTranslucent()
{
	colorDisabled = color4b::GRAY;
	colorEnabled = color4b::WHITE;
	colorHot = color4b::YELLOW;
	alphaPressedBgr = 160;
	alphaHotBgr = 80;
}


//====================================================================
void  ShaderParams_Button_PushTranslucent::ChooseParams(
									const gui::Button& button,
									color4b& cGlyph, color4b& cBgr, bool& bDrawBgr) const
{
	if(button.IsEnabled()) {
		cGlyph = button.IsHot() ? colorHot : colorEnabled;
	}
	else {
		cGlyph = colorDisabled;
	}
	cBgr = cGlyph;
	bDrawBgr = false;

	if(button.IsPressed()) {
		cBgr.a = alphaPressedBgr;
		bDrawBgr = true;
	}
	else if(button.IsHot()) {
		cBgr.a = alphaHotBgr;
		bDrawBgr = true;
	}
}


//====================================================================
ShaderParams_Button_ToggleOpaque::ShaderParams_Button_ToggleOpaque()
{
	colorDisabled = color4b(192, 192, 192);
	colorDisabledToggled = color4b(160, 160, 160);
	colorDisabledBgr = color4b(128, 128, 128);
	colorDisabledBgrToggled = color4b(96, 96, 96);
	colorEnabledBorder = color4b::BLACK;
	colorUntoggledGlyph = color4b::BLACK;
	colorUntoggledPressedGlyph = color4b::YELLOW;
	colorToggledGlyph = color4b::YELLOW;
	colorToggledPressedGlyph = color4b::BLACK;
	colorUntoggledBgrTop = color4b(128, 128, 128);
	colorUntoggledBgrBottom = color4b(192, 192, 192);
	colorUntoggledPressedBgrTop = color4b(64, 64, 64);
	colorUntoggledPressedBgrBottom = color4b(128, 128, 128);
	colorToggledBgrTop = color4b(64, 64, 64);
	colorToggledBgrBottom = color4b(128, 128, 128);
	colorToggledPressedBgrTop = color4b(128, 128, 128);
	colorToggledPressedBgrBottom = color4b(192, 192, 192);
	colorHotBorder = color4b::YELLOW;
	mainBorderWidthUntoggled = 1;
	mainBorderWidthUntoggledPressed = 2;
	mainBorderWidthToggled = 2;
	mainBorderWidthToggledPressed = 1;
	hotBorderWidth = 2;
}


//====================================================================
void  ShaderParams_Button_ToggleOpaque::ChooseParams(
								const gui::Button& button,
								color4b& cTop, color4b& cBottom,
								color4b& cGlyph, color4b& cMainBorder,
								float& mainBorderWidth) const
{
	// choose colors
	if(button.IsEnabled()) {
		if(button.IsToggled()) {
			if(button.IsPressed()) {
				cTop = colorToggledPressedBgrTop;
				cBottom = colorToggledPressedBgrBottom;
				cGlyph = colorToggledPressedGlyph;
			}
			else {
				cTop = colorToggledBgrTop;
				cBottom = colorToggledBgrBottom;
				cGlyph = colorToggledGlyph;
			}
		}
		else {
			if(button.IsPressed()) {
				cTop = colorUntoggledPressedBgrTop;
				cBottom = colorUntoggledPressedBgrBottom;
				cGlyph = colorUntoggledPressedGlyph;
			}
			else {
				cTop = colorUntoggledBgrTop;
				cBottom = colorUntoggledBgrBottom;
				cGlyph = colorUntoggledGlyph;
			}
		}
		cMainBorder = colorEnabledBorder;
	}
	else {
		if(button.IsToggled()) {
			cTop = colorDisabledBgrToggled;
			cBottom = colorDisabledBgrToggled;
			cGlyph = colorDisabledToggled;
			cMainBorder = colorDisabledToggled;
		}
		else {
			cTop = colorDisabledBgr;
			cBottom = colorDisabledBgr;
			cGlyph = colorDisabled;
			cMainBorder = colorDisabled;
		}
	}

	// choose main-border width
	if(button.IsToggled()) {
		mainBorderWidth = button.IsPressed() ? mainBorderWidthToggledPressed : mainBorderWidthToggled;
	}
	else {
		mainBorderWidth = button.IsPressed() ? mainBorderWidthUntoggledPressed : mainBorderWidthUntoggled;
	}
}


//====================================================================
ShaderParams_Button_ToggleTranslucent::ShaderParams_Button_ToggleTranslucent()
{
	colorDisabled = color4b::GRAY;
	colorDisabledToggled = color4b::LIGHT_GRAY;
	colorUntoggled = color4b::WHITE;
	colorUntoggledPressed = color4b::YELLOW;
	colorToggled = color4b::YELLOW;
	colorToggledPressed = color4b::WHITE;
	colorBgr = color4b::YELLOW;
	alphaPressedBgr = 160;
	alphaHotBgr = 80;
}


//====================================================================
void  ShaderParams_Button_ToggleTranslucent::ChooseParams(
								const gui::Button& button,
								color4b& cGlyph, color4b& cBgr, bool& bDrawBgr) const
{
	if(button.IsEnabled()) {
		if(button.IsToggled()) {
			cGlyph = button.IsPressed() ? colorToggledPressed : colorToggled;
		}
		else {
			cGlyph = button.IsPressed() ? colorUntoggledPressed : colorUntoggled;
		}
	}
	else {
		cGlyph = button.IsToggled() ? colorDisabledToggled : colorDisabled;
	}
	cBgr = colorBgr;
	bDrawBgr = false;

	if(button.IsPressed()) {
		cBgr.a = alphaPressedBgr;
		bDrawBgr = true;
	}
	else if(button.IsHot()) {
		cBgr.a = alphaHotBgr;
		bDrawBgr = true;
	}
}


//====================================================================
CS0_TextButtonShader_PushOpaque::CS0_TextButtonShader_PushOpaque(
								const std::string& shaderName,
								const ShaderParams_Button_PushOpaque& sp,
								const gui::TextAttributes& ta)
	:
	gui_cs0::TextButton::Shader(shaderName),
	shaderParams(sp)
{
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_TextButtonShader_PushOpaque::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	if(! pFont) return;

	const gui_cs0::TextButton& textButton = gui::ComponentCast<gui_cs0::TextButton>(c);
	const gui_cs0::TextButton::GeomParams& gp = textButton.GetGeomParams();

	// choose params
	color4b  colorTop, colorBottom, colorGlyph, colorMainBorder;
	vec2     pressOffset;

	shaderParams.ChooseParams(textButton, colorTop, colorBottom,
						colorGlyph, colorMainBorder, pressOffset);

	gui::RectangleC  rBorder(pressOffset, textButton.GetSize());

	// draw background
	rBorder.SetColorTop(colorTop);
	rBorder.SetColorBottom(colorBottom);

	gr.DrawRectangle(rBorder);

	// draw text
	vec2 textOffset = 0.5f * (textButton.GetSize() - textButton.GetTextSize()) + pressOffset;

	gui::DrawTextLines(gr, *pFont, colorGlyph, textOffset,
				GetTextMetricsLCache(gp.textHeight, gp.lineHeight),
				textButton.GetTextLines());

	// draw main border
	rBorder.SetColor(colorMainBorder);
	gr.DrawRectangleFrame(rBorder, shaderParams.mainBorderWidth);

	// draw hot border
	if(textButton.IsPressed()) return;

	if(textButton.IsHot()) {
		rBorder.SetColor(shaderParams.colorHotBorder);
		rBorder.Shrink(shaderParams.mainBorderWidth);
		gr.DrawRectangleFrame(rBorder, shaderParams.hotBorderWidth);
	}
}


//====================================================================
CS0_TextButtonShader_ToggleOpaque::CS0_TextButtonShader_ToggleOpaque(
										const std::string& shaderName,
										const ShaderParams_Button_ToggleOpaque& sp,
										const gui::TextAttributes& ta)
	:
	gui_cs0::TextButton::Shader(shaderName),
	shaderParams(sp)
{
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_TextButtonShader_ToggleOpaque::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	if(! pFont) return;

	const gui_cs0::TextButton& textButton = gui::ComponentCast<gui_cs0::TextButton>(c);
	const gui_cs0::TextButton::GeomParams& gp = textButton.GetGeomParams();

	// choose params
	color4b  colorTop, colorBottom, colorGlyph, colorMainBorder;
	float    mainBorderWidth;

	shaderParams.ChooseParams(textButton, colorTop, colorBottom,
						colorGlyph, colorMainBorder, mainBorderWidth);

	// draw background
	gui::RectangleC  rBorder(vec2::ZERO, textButton.GetSize());

	rBorder.SetColorTop(colorTop);
	rBorder.SetColorBottom(colorBottom);

	gr.DrawRectangle(rBorder);

	// draw text
	vec2 textOffset = 0.5f * (textButton.GetSize() - textButton.GetTextSize());

	gui::DrawTextLines(gr, *pFont, colorGlyph, textOffset,
				GetTextMetricsLCache(gp.textHeight, gp.lineHeight),
				textButton.GetTextLines());

	// draw main border
	rBorder.SetColor(colorMainBorder);
	gr.DrawRectangleFrame(rBorder, mainBorderWidth);

	// draw hot border
	if(textButton.IsHot()) {
		rBorder.SetColor(shaderParams.colorHotBorder);
		rBorder.Shrink(mainBorderWidth);
		gr.DrawRectangleFrame(rBorder, shaderParams.hotBorderWidth);
	}
}


//====================================================================
CS0_TextButtonShader_Translucent::CS0_TextButtonShader_Translucent(
												const std::string& shaderName,
												const gui::TextAttributes& ta)
	:
	gui_cs0::TextButton::Shader(shaderName)
{
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_TextButtonShader_Translucent::DrawTextButton(
									const gui_cs0::TextButton& textButton,
									gui::Renderer& gr,
									color4b colorGlyph, color4b colorBgr, bool bDrawBgr) const
{
	if(! pFont) return;
	const gui_cs0::TextButton::GeomParams& gp = textButton.GetGeomParams();

	gui::RectangleC  rBorder(vec2::ZERO, textButton.GetSize());

	// draw background
	if(bDrawBgr) {
		rBorder.SetColor(colorBgr);
		gr.DrawRectangle(rBorder);
	}

	// draw text
	vec2 textOffset = 0.5f * (textButton.GetSize() - textButton.GetTextSize());

	gui::DrawTextLines(gr, *pFont, colorGlyph, textOffset,
				GetTextMetricsLCache(gp.textHeight, gp.lineHeight),
				textButton.GetTextLines());
}


//====================================================================
CS0_TextButtonShader_PushTranslucent::CS0_TextButtonShader_PushTranslucent(
									const std::string& shaderName,
									const ShaderParams_Button_PushTranslucent& sp,
									const gui::TextAttributes& ta)
	:
	CS0_TextButtonShader_Translucent(shaderName, ta),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_TextButtonShader_PushTranslucent::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::TextButton& textButton = gui::ComponentCast<gui_cs0::TextButton>(c);

	// choose params
	color4b  colorGlyph, colorBgr;
	bool     bDrawBgr;

	shaderParams.ChooseParams(textButton, colorGlyph, colorBgr, bDrawBgr);

	// do draw
	DrawTextButton(textButton, gr, colorGlyph, colorBgr, bDrawBgr);
}


//====================================================================
CS0_TextButtonShader_ToggleTranslucent::CS0_TextButtonShader_ToggleTranslucent(
										const std::string& shaderName,
										const ShaderParams_Button_ToggleTranslucent& sp,
										const gui::TextAttributes& ta)
	:
	CS0_TextButtonShader_Translucent(shaderName, ta),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_TextButtonShader_ToggleTranslucent::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::TextButton& textButton = gui::ComponentCast<gui_cs0::TextButton>(c);

	// choose params
	color4b  colorGlyph, colorBgr;
	bool     bDrawBgr;

	shaderParams.ChooseParams(textButton, colorGlyph, colorBgr, bDrawBgr);

	// do draw
	DrawTextButton(textButton, gr, colorGlyph, colorBgr, bDrawBgr);
}


//====================================================================
CS0_IconButtonShader_PushOpaque::CS0_IconButtonShader_PushOpaque(
								const std::string& shaderName,
								const ShaderParams_Button_PushOpaque& sp)
	:
	gui_cs0::IconButton::Shader(shaderName),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_IconButtonShader_PushOpaque::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::IconButton& iconButton = gui::ComponentCast<gui_cs0::IconButton>(c);

	// choose params
	color4b  colorTop, colorBottom, colorGlyph, colorMainBorder;
	vec2     pressOffset;

	shaderParams.ChooseParams(iconButton, colorTop, colorBottom,
							colorGlyph, colorMainBorder, pressOffset);

	gui::RectangleC  rBorder(pressOffset, iconButton.GetSize());

	// draw background
	rBorder.SetColorTop(colorTop);
	rBorder.SetColorBottom(colorBottom);

	gr.DrawRectangle(rBorder);

	// draw icon
	const gui::TileTexture* pTileTexture = iconButton.GetTileTexture();
	if(! pTileTexture) return;

	gui::RectangleCT rTile = iconButton.GetTileRectangle();

	rTile.Offset(pressOffset);
	rTile.SetColor(colorGlyph);

	gr.DrawRectangle(*pTileTexture, rTile);

	// draw main border
	rBorder.SetColor(colorMainBorder);
	gr.DrawRectangleFrame(rBorder, shaderParams.mainBorderWidth);

	// draw hot border
	if(iconButton.IsPressed()) return;

	if(iconButton.IsHot()) {
		rBorder.SetColor(shaderParams.colorHotBorder);
		rBorder.Shrink(shaderParams.mainBorderWidth);
		gr.DrawRectangleFrame(rBorder, shaderParams.hotBorderWidth);
	}
}


//====================================================================
CS0_IconButtonShader_ToggleOpaque::CS0_IconButtonShader_ToggleOpaque(
										const std::string& shaderName,
										const ShaderParams_Button_ToggleOpaque& sp)
	:
	gui_cs0::IconButton::Shader(shaderName),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_IconButtonShader_ToggleOpaque::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::IconButton& iconButton = gui::ComponentCast<gui_cs0::IconButton>(c);

	// choose params
	color4b  colorTop, colorBottom, colorGlyph, colorMainBorder;
	float    mainBorderWidth;

	shaderParams.ChooseParams(iconButton, colorTop, colorBottom,
							colorGlyph, colorMainBorder, mainBorderWidth);

	// draw background
	gui::RectangleC  rBorder(vec2::ZERO, iconButton.GetSize());

	rBorder.SetColorTop(colorTop);
	rBorder.SetColorBottom(colorBottom);

	gr.DrawRectangle(rBorder);

	// draw icon
	const gui::TileTexture* pTileTexture = iconButton.GetTileTexture();
	if(! pTileTexture) return;

	gui::RectangleCT rTile = iconButton.GetTileRectangle();
	rTile.SetColor(colorGlyph);

	gr.DrawRectangle(*pTileTexture, rTile);

	// draw main border
	rBorder.SetColor(colorMainBorder);
	gr.DrawRectangleFrame(rBorder, mainBorderWidth);

	// draw hot border
	if(iconButton.IsHot()) {
		rBorder.SetColor(shaderParams.colorHotBorder);
		rBorder.Shrink(mainBorderWidth);
		gr.DrawRectangleFrame(rBorder, shaderParams.hotBorderWidth);
	}
}


//====================================================================
CS0_IconButtonShader_Translucent::CS0_IconButtonShader_Translucent(const std::string& shaderName)
	:
	gui_cs0::IconButton::Shader(shaderName)
{
}


//====================================================================
void __fastcall CS0_IconButtonShader_Translucent::DrawIconButton(
									const gui_cs0::IconButton& iconButton,
									gui::Renderer& gr,
									color4b colorGlyph, color4b colorBgr, bool bDrawBgr)
{
	gui::RectangleC  rBorder;
	rBorder.Set(vec2::ZERO, iconButton.GetSize());

	// draw background
	if(bDrawBgr) {
		rBorder.SetColor(colorBgr);
		gr.DrawRectangle(rBorder);
	}

	// draw icon
	const gui::TileTexture* pTileTexture = iconButton.GetTileTexture();
	if(! pTileTexture) return;

	gui::RectangleCT& rTile = iconButton.GetTileRectangle();
	rTile.SetColor(colorGlyph);

	gr.DrawRectangle(*pTileTexture, rTile);
}


//====================================================================
CS0_IconButtonShader_PushTranslucent::CS0_IconButtonShader_PushTranslucent(
										const std::string& shaderName,
										const ShaderParams_Button_PushTranslucent& sp)
	:
	CS0_IconButtonShader_Translucent(shaderName),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_IconButtonShader_PushTranslucent::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::IconButton& iconButton = gui::ComponentCast<gui_cs0::IconButton>(c);

	// choose params
	color4b  colorGlyph, colorBgr;
	bool     bDrawBgr;

	shaderParams.ChooseParams(iconButton, colorGlyph, colorBgr, bDrawBgr);

	// do draw
	DrawIconButton(iconButton, gr, colorGlyph, colorBgr, bDrawBgr);
}


//====================================================================
CS0_IconButtonShader_ToggleTranslucent::CS0_IconButtonShader_ToggleTranslucent(
										const std::string& shaderName,
										const ShaderParams_Button_ToggleTranslucent& sp)
	:
	CS0_IconButtonShader_Translucent(shaderName),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_IconButtonShader_ToggleTranslucent::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::IconButton& iconButton = gui::ComponentCast<gui_cs0::IconButton>(c);

	// choose params
	color4b  colorGlyph, colorBgr;
	bool     bDrawBgr;

	shaderParams.ChooseParams(iconButton, colorGlyph, colorBgr, bDrawBgr);

	// do draw
	DrawIconButton(iconButton, gr, colorGlyph, colorBgr, bDrawBgr);
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


