

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\CheckBox.h"
#include "private\vw_gui_ss0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
class VW_GUI_SS0_API CS0_CheckBoxShader_Opaque : public gui_cs0::CheckBox::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ShaderParams
	{
		color4b  colorDisabled;
		color4b  colorDisabledBgr;
		color4b  colorEnabledBorder;
		color4b  colorEnabledIcon;
		color4b  colorEnabledBgrTop;
		color4b  colorEnabledBgrBottom;
		color4b  colorPressedBgrTop;
		color4b  colorPressedBgrBottom;
		color4b  colorHotBorder;
		vec2     iconPadding;
		vec2     pressOffset;
		float    mainBorderWidth;
		float    hotBorderWidth;
		gui::TileParams  tileParams;

		ShaderParams();

		void  ChooseParams(const gui_cs0::CheckBox& checkBox,
						color4b& cTop, color4b& cBottom,
						color4b& cIcon, color4b& cMainBorder,
						vec2& pressOffset) const;
	};


public:
	//-------------------------------------
	// CS0_CheckBoxShader_Opaque interface.
	//-------------------------------------
	CS0_CheckBoxShader_Opaque(const std::string& shaderName = "",
							const ShaderParams& sp = ShaderParams());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams());
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//-----------------------------
	// VideoResourceUser overrides.
	//-----------------------------
	virtual  bool  CacheTileTextureHandles(gui::Manager& gm);
	virtual  void  PostCacheTileTextureHandles();


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;

	//-------
	// Cache.
	//-------
	const gui::TileTexture*   pTileTexture;
	mutable gui::RectangleCT  rTile;
	vec2                      iconWidth;
};


//====================================================================
class VW_GUI_SS0_API CS0_CheckBoxShader_Translucent : public gui_cs0::CheckBox::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ShaderParams
	{
		color4b  colorDisabled;
		color4b  colorEnabled;
		color4b  colorHot;
		ubyte    alphaPressedBgr;
		ubyte    alphaHotBgr;
		vec2     iconPadding;
		float    borderWidth;
		gui::TileParams  tileParams;

		ShaderParams();

		void  ChooseParams(const gui_cs0::CheckBox& checkBox,
							color4b& cIcon, color4b& cBgr) const;
	};


public:
	//------------------------------------------
	// CS0_CheckBoxShader_Translucent interface.
	//------------------------------------------
	CS0_CheckBoxShader_Translucent(const std::string& shaderName = "",
								const ShaderParams& sp = ShaderParams());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams());
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//-----------------------------
	// VideoResourceUser overrides.
	//-----------------------------
	virtual  bool  CacheTileTextureHandles(gui::Manager& gm);
	virtual  void  PostCacheTileTextureHandles();


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;

	//-------
	// Cache.
	//-------
	const gui::TileTexture*   pTileTexture;
	mutable gui::RectangleCT  rTile;
	vec2                      iconWidth;
};


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


