

//--------------------------------------------------------------------
#include "GUI_SS0\ComboBox.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_StdObjects.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
CS0_ComboBoxShader::ShaderParams::ShaderParams()
{
	colorBaseTextDisabled = color4b::DARK_GRAY;
	colorBaseTextEnabled = color4b::BLACK;
	colorBaseTextHot = color4b::YELLOW;

	colorPopupText = color4b(255, 255, 255, 192);
	colorPopupBgr1 = color4b(0, 0, 255, 96);
	colorPopupBgr2 = color4b(0, 0, 128, 192);

	colorPaneText = color4b::BLACK;
	colorPaneBgrHot1 = color4b(0, 0, 255, 192);
	colorPaneBgrHot2 = color4b(0, 0, 128, 192);
	colorPaneTextHot = color4b::YELLOW;
	colorPaneTextSelected = color4b::YELLOW;

	baseDisabled.c1 = color4b::LIGHT_GRAY;
	baseDisabled.c2 = color4b::LIGHT_GRAY;
	baseDisabled.cBorder = color4b::DARK_GRAY;
	baseDisabled.widthBorder = 1;

	baseEnabled.c1 = color4b(157, 185, 235);
	baseEnabled.c2 = color4b(122, 150, 223);
	baseEnabled.cBorder = color4b::BLACK;
	baseEnabled.widthBorder = 1;

	baseHot.c1 = color4b(0, 0, 255);
	baseHot.c2 = color4b(0, 0, 128);
	baseHot.cBorder = color4b(0, 0, 128);
	baseHot.widthBorder = 1;

	pane.c1 = color4b(157, 185, 235);
	pane.c2 = color4b(122, 150, 223);
	pane.cBorder = color4b::BLACK;
	pane.widthBorder = 1;

	paneButtonDisabled = baseEnabled;
	paneButtonEnabled = baseEnabled;
	paneButtonHot = baseHot;
	paneButtonPressed = baseHot;
	paneButtonPressed.cBorder = color4b::YELLOW;

	openButtonDisabled = baseDisabled;
	openButtonEnabled = baseEnabled;
	openButtonHot = baseHot;

	tPaneButtonDisabled.color = color4b(0, 0, 0, 64);
	tPaneButtonDisabled.shrink = 1;
	tPaneButtonEnabled.color = color4b::BLACK;
	tPaneButtonEnabled.shrink = 1;
	tPaneButtonHot.color = color4b::YELLOW;
	tPaneButtonHot.shrink = 1;
	tPaneButtonPressed.color = color4b::YELLOW;
	tPaneButtonPressed.shrink = 2;

	tOpenButtonDisabled.color = color4b::DARK_GRAY;
	tOpenButtonDisabled.shrink = 4;
	tOpenButtonEnabled.color = color4b::BLACK;
	tOpenButtonEnabled.shrink = 4;
	tOpenButtonHot.color = color4b::YELLOW;
	tOpenButtonHot.shrink = 4;

	using namespace gui::std_tile_textures;

	tileParamsOpenTop = MakeTileParams(Tile_ArrowUp);
	tileParamsOpenBottom = MakeTileParams(Tile_ArrowDown);
	tileParamsOpenCenter = MakeTileParams(Tile_ArrowUpDown);
	tileParamsBackward = MakeTileParams(Tile_ArrowUp);
	tileParamsForward = MakeTileParams(Tile_ArrowDown);
}


//====================================================================
void  CS0_ComboBoxShader::ShaderParams::ChooseParams(
								const gui_cs0::ComboBox& cbox,
								const gui_cs0::ComboBoxButton& buttonOpen,
								color4b& cBaseText,
								ParamBlockArea& pbaBase, ParamBlockArea& pbaButtonOpen,
								ParamBlockTile& pbtButtonOpen) const
{
	if(cbox.IsEnabled()) {
		if(cbox.IsHot()) {
			pbaBase = baseHot;
			cBaseText = colorBaseTextHot;
		}
		else {
			pbaBase = baseEnabled;
			cBaseText = colorBaseTextEnabled;
		}
	}
	else {
		pbaBase = baseDisabled;
		cBaseText = colorBaseTextDisabled;
	}

	if(buttonOpen.IsEnabled()) {
		if(buttonOpen.IsHot()) {
			pbaButtonOpen = openButtonHot;
			pbtButtonOpen = tOpenButtonHot;
		}
		else {
			pbaButtonOpen = openButtonEnabled;
			pbtButtonOpen = tOpenButtonEnabled;
		}
	}
	else {
		pbaButtonOpen = openButtonDisabled;
		pbtButtonOpen = tOpenButtonDisabled;
	}
}


//====================================================================
void  CS0_ComboBoxShader::ShaderParams::ChooseParams(
							const gui_cs0::ComboBoxPaneButton& button,
							ParamBlockArea& pbaPaneButton, ParamBlockTile& pbtPaneButton) const
{
	if(button.IsEnabled()) {
		if(button.IsPressed()) {
			pbaPaneButton = paneButtonPressed;
			pbtPaneButton = tPaneButtonPressed;
		}
		else if(button.IsHot()) {
			pbaPaneButton = paneButtonHot;
			pbtPaneButton = tPaneButtonHot;
		}
		else {
			pbaPaneButton = paneButtonEnabled;
			pbtPaneButton = tPaneButtonEnabled;
		}
	}
	else {
		pbaPaneButton = paneButtonDisabled;
		pbtPaneButton = tPaneButtonDisabled;
	}
}


//====================================================================
void  CS0_ComboBoxShader::ShaderParams::CalcPaneTileRectangle(gui::Rectangle& rTile,
						const gui::Rectangle& rPaneButton, const ParamBlockTile& pbt) const
{
	rTile.height = rPaneButton.height - pbt.shrink;
	rTile.top = rPaneButton.top + (rPaneButton.height - rTile.height) * 0.5;
	rTile.width = rTile.height;
	rTile.left = rPaneButton.left + (rPaneButton.width - rTile.width) * 0.5f;
}


//====================================================================
CS0_ComboBoxShader::CS0_ComboBoxShader(const std::string& shaderName,
									const ShaderParams& sp,
									const gui::TextAttributes& ta)
	:
	gui_cs0::ComboBox::Shader(shaderName)
{
	pTileTexOpenTop = 0;
	pTileTexOpenBottom = 0;
	pTileTexOpenCenter = 0;
	pTileTexBackward = 0;
	pTileTexForward = 0;

	SetTextAttributes(ta);

	SetShaderParams(sp);
}


//====================================================================
void  CS0_ComboBoxShader::SetShaderParams(const ShaderParams& sp)
{
	shaderParams = sp;

	rTileOpenTop.SetTexTransform(shaderParams.tileParamsOpenTop.texTransform);
	rTileOpenBottom.SetTexTransform(shaderParams.tileParamsOpenBottom.texTransform);
	rTileOpenCenter.SetTexTransform(shaderParams.tileParamsOpenCenter.texTransform);
	rTileBackward.SetTexTransform(shaderParams.tileParamsBackward.texTransform);
	rTileForward.SetTexTransform(shaderParams.tileParamsForward.texTransform);

	RequestTileTextureRecache();
}


//====================================================================
void  CS0_ComboBoxShader::Shade(const gui_cs0::ComboBox& cbox, gui::Renderer& gr) const
{
	const gui_cs0::ComboBoxButton& buttonOpen = cbox.GetButtonOpen();
	const gui_cs0::ComboBox::GeomParams& gp = cbox.GetGeomParams();

	// choose params
	color4b         cBaseText;
	ParamBlockArea  pbaBase, pbaButtonOpen;
	ParamBlockTile  pbtButtonOpen;

	shaderParams.ChooseParams(cbox, buttonOpen, cBaseText,
						pbaBase, pbaButtonOpen, pbtButtonOpen);

	// draw base
	gui::RectangleC  rBox(vec2::ZERO, cbox.GetSize());
	rBox.SetColorTop(pbaBase.c1);
	rBox.SetColorBottom(pbaBase.c2);
	gr.DrawRectangle(rBox);

	rBox.SetColor(pbaBase.cBorder);
	gr.DrawRectangleFrame(rBox, pbaBase.widthBorder);

	// draw open-button
	gui::RectangleC  rButton(buttonOpen.GetPosition(), buttonOpen.GetSize());
	rButton.SetColorTop(pbaButtonOpen.c1);
	rButton.SetColorBottom(pbaButtonOpen.c2);
	gr.DrawRectangle(rButton);

	rButton.SetColor(pbaButtonOpen.cBorder);
	gr.DrawRectangleFrame(rButton, pbaButtonOpen.widthBorder);

	gui::RectangleCT*        prTileOpen = 0;
	const gui::TileTexture*  pTileTexOpen = 0;

	switch(gp.panePosition) {
		case gui_cs0::ComboBox::PanePosition_Top:
			prTileOpen = &rTileOpenTop;
			pTileTexOpen = pTileTexOpenTop;
			break;
		case gui_cs0::ComboBox::PanePosition_Bottom:
			prTileOpen = &rTileOpenBottom;
			pTileTexOpen = pTileTexOpenBottom;
			break;
		case gui_cs0::ComboBox::PanePosition_Center:
			prTileOpen = &rTileOpenCenter;
			pTileTexOpen = pTileTexOpenCenter;
			break;
		default:
			prTileOpen = &rTileOpenBottom;
			pTileTexOpen = pTileTexOpenBottom;
			break;
	}

	prTileOpen->SetColor(pbtButtonOpen.color);
	prTileOpen->CopyGeometryParamsFrom(rButton);
	prTileOpen->Shrink(pbtButtonOpen.shrink);
	gr.DrawRectangle(*pTileTexOpen, *prTileOpen);

	// draw selected-item text
	if(! pFont) return;

	vec2  textOffset(gp.textPaddingBox, (cbox.GetSize().y - gp.textHeightBox) * 0.5f);
	if(! gp.bOpenButtonRightAlign) textOffset.x += rButton.width;

	gui::DrawTextParams  dtp(cBaseText, textOffset, GetTextMetricsLCache(gp.textHeightBox));

	gr.DrawText(*pFont, cbox.GetSelectedItemVisibleTextLength(),
				cbox.GetSelectedItemText().c_str(), dtp);

	// draw selected-item popup
	if(cbox.GetShowSelectedItemPopup()) {
		if(! gp.bOpenButtonRightAlign) dtp.positionTopLeft.x = gp.textPaddingBox;

		rBox.top += pbaBase.widthBorder;
		rBox.height -= 2 * pbaBase.widthBorder;
		rBox.left += pbaBase.widthBorder;
		rBox.width = Max(rBox.width, cbox.GetSelectedItemTextWidth() + gp.textPaddingBox * 2);

		rBox.SetColorTop(shaderParams.colorPopupBgr1);
		rBox.SetColorBottom(shaderParams.colorPopupBgr2);
		gr.DrawRectangle(rBox);

		dtp.color = shaderParams.colorPopupText;
		gr.DrawText(*pFont, cbox.GetSelectedItemText(), dtp);
	}
}


//====================================================================
void  CS0_ComboBoxShader::Shade(const gui_cs0::ComboBoxPane& pane, gui::Renderer& gr) const
{
	const gui_cs0::ComboBox& cbox = pane.GetComboBox();
	const gui_cs0::ComboBox::GeomParams& gp = cbox.GetGeomParams();

	// draw pane
	gui::RectangleC  r(vec2::ZERO, pane.GetSize());
	r.SetColorLeft(shaderParams.pane.c1);
	r.SetColorRight(shaderParams.pane.c2);
	gr.DrawRectangle(r);

	// draw pane
	r.SetColor(shaderParams.pane.cBorder);
	gr.DrawRectangleFrame(r, shaderParams.pane.widthBorder);

	// draw buttons
	if(pane.GetButtonsVisible()) {

		// choose params
		ParamBlockArea pbaBackward, pbaForward;
		ParamBlockTile pbtBackward, pbtForward;

		shaderParams.ChooseParams(pane.GetButtonBackward(), pbaBackward, pbtBackward);
		shaderParams.ChooseParams(pane.GetButtonForward(), pbaForward, pbtForward);

		// backward button
		r.SetColorTop(pbaBackward.c1);
		r.SetColorBottom(pbaBackward.c2);

		r.height = pane.GetButtonBackward().GetSize().y;
		gr.DrawRectangle(r);

		shaderParams.CalcPaneTileRectangle(rTileBackward, r, pbtBackward);
		rTileBackward.SetColor(pbtBackward.color);
		gr.DrawRectangle(*pTileTexBackward, rTileBackward);

		r.SetColor(pbaBackward.cBorder);
		gr.DrawRectangleFrame(r, pbaBackward.widthBorder);

		// forward button
		r.SetColorTop(pbaForward.c2);
		r.SetColorBottom(pbaForward.c1);

		r.top = pane.GetButtonForward().GetPosition().y;
		gr.DrawRectangle(r);

		shaderParams.CalcPaneTileRectangle(rTileForward, r, pbtForward);
		rTileForward.SetColor(pbtForward.color);
		gr.DrawRectangle(*pTileTexForward, rTileForward);

		r.SetColor(pbaForward.cBorder);
		gr.DrawRectangleFrame(r, pbaForward.widthBorder);
	}

	// draw items
	if(! pFont) return;

	int first, last;
	if(! pane.GetVisibleItemRange(first, last)) {
		first = last + 1;
	}

	int iSelectedItem = cbox.GetSelectedItemIndex();
	int iHotItem = pane.GetHotItem();

	r.left = shaderParams.pane.widthBorder;

	float  hotBgrWidth = pane.GetSize().x - 2 * shaderParams.pane.widthBorder;
	float  maxItemWidth = pane.GetSize().x - 2 * gp.textPaddingPane;
	float  itemRefY = pane.GetButtonsVisible() ? pane.GetButtonBackward().GetSize().y : 0;

	vec2 textOffset(gp.textPaddingPane, (gp.itemHeightPane - gp.textHeightPane) * 0.5f);
	textOffset.y += itemRefY;

	gui::DrawTextParams  dtp(color4b(), textOffset, GetTextMetricsLCache(gp.textHeightPane));

	int curr = first;
	while(curr <= last) {

		// choose color
		if(curr == iHotItem) {
			dtp.color = shaderParams.colorPaneTextHot;

			// draw hot-bgr
			r.top = itemRefY + gp.itemHeightPane * (iHotItem - first);
			r.height = gp.itemHeightPane;

			if(! pane.GetButtonsVisible()) {
				if(curr == first) {
					r.height -= shaderParams.pane.widthBorder;
					r.top += shaderParams.pane.widthBorder;
				}
				else if(curr == last) {
					r.height -= shaderParams.pane.widthBorder;
				}
			}
			float fullItemWidth = cbox.GetItemFullTextWidthPane(curr);

			if(fullItemWidth > maxItemWidth) {
				r.width = fullItemWidth + 2 * gp.textPaddingPane;
			}
			else {
				r.width = hotBgrWidth;
			}

			r.SetColorLeft(shaderParams.colorPaneBgrHot1);
			r.SetColorRight(shaderParams.colorPaneBgrHot2);
			gr.DrawRectangle(r);
		}
		else if(curr == iSelectedItem) {
			dtp.color = shaderParams.colorPaneTextSelected;
		}
		else {
			dtp.color = shaderParams.colorPaneText;
		}

		// do draw text
		if(curr == iHotItem) {
			gr.DrawText(*pFont, cbox.GetItem(curr).GetText(), dtp);
		}
		else {
			gr.DrawText(*pFont, cbox.GetItemVisibleTextLengthPane(curr),
						cbox.GetItem(curr).GetText().c_str(), dtp);
		}

		dtp.positionTopLeft.y += gp.itemHeightPane;
		curr++;
	}
}


//====================================================================
bool  CS0_ComboBoxShader::CacheTileTextureHandles(gui::Manager& gm)
{
	pTileTexOpenTop = gm.GetTileTexture(shaderParams.tileParamsOpenTop.texName);
	pTileTexOpenBottom = gm.GetTileTexture(shaderParams.tileParamsOpenBottom.texName);
	pTileTexOpenCenter = gm.GetTileTexture(shaderParams.tileParamsOpenCenter.texName);
	pTileTexBackward = gm.GetTileTexture(shaderParams.tileParamsBackward.texName);
	pTileTexForward = gm.GetTileTexture(shaderParams.tileParamsForward.texName);

	return
	(pTileTexOpenTop != 0  &&  pTileTexOpenBottom != 0  &&  pTileTexOpenCenter != 0  &&
	pTileTexBackward != 0  &&  pTileTexForward != 0);
}


//====================================================================
void  CS0_ComboBoxShader::PostCacheTileTextureHandles()
{
	pTileTexOpenTop->GetTileCoords(shaderParams.tileParamsOpenTop.texRow, shaderParams.tileParamsOpenTop.texCol, rTileOpenTop);
	pTileTexOpenBottom->GetTileCoords(shaderParams.tileParamsOpenBottom.texRow, shaderParams.tileParamsOpenBottom.texCol, rTileOpenBottom);
	pTileTexOpenCenter->GetTileCoords(shaderParams.tileParamsOpenCenter.texRow, shaderParams.tileParamsOpenCenter.texCol, rTileOpenCenter);
	pTileTexBackward->GetTileCoords(shaderParams.tileParamsBackward.texRow, shaderParams.tileParamsBackward.texCol, rTileBackward);
	pTileTexForward->GetTileCoords(shaderParams.tileParamsForward.texRow, shaderParams.tileParamsForward.texCol, rTileForward);
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


