

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\ComboBox.h"
#include "private\vw_gui_ss0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
class VW_GUI_SS0_API CS0_ComboBoxShader : public gui_cs0::ComboBox::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ParamBlockArea
	{
		color4b  c1;
		color4b  c2;
		color4b  cBorder;
		float    widthBorder;
	};

	struct VW_GUI_SS0_API ParamBlockTile
	{
		color4b  color;
		float    shrink;
	};

	struct VW_GUI_SS0_API ShaderParams
	{
		color4b  colorBaseTextDisabled;
		color4b  colorBaseTextEnabled;
		color4b  colorBaseTextHot;
		color4b  colorPopupText;
		color4b  colorPopupBgr1;
		color4b  colorPopupBgr2;
		color4b  colorPaneText;
		color4b  colorPaneBgrHot1;
		color4b  colorPaneBgrHot2;
		color4b  colorPaneTextHot;
		color4b  colorPaneTextSelected;

		ParamBlockArea  baseDisabled;
		ParamBlockArea  baseEnabled;
		ParamBlockArea  baseHot;

		ParamBlockArea  pane;
		ParamBlockArea  paneButtonDisabled;
		ParamBlockArea  paneButtonEnabled;
		ParamBlockArea  paneButtonHot;
		ParamBlockArea  paneButtonPressed;

		ParamBlockArea  openButtonDisabled;
		ParamBlockArea  openButtonEnabled;
		ParamBlockArea  openButtonHot;

		ParamBlockTile  tOpenButtonDisabled;
		ParamBlockTile  tOpenButtonEnabled;
		ParamBlockTile  tOpenButtonHot;

		ParamBlockTile  tPaneButtonDisabled;
		ParamBlockTile  tPaneButtonEnabled;
		ParamBlockTile  tPaneButtonHot;
		ParamBlockTile  tPaneButtonPressed;

		gui::TileParams  tileParamsOpenTop;
		gui::TileParams  tileParamsOpenBottom;
		gui::TileParams  tileParamsOpenCenter;
		gui::TileParams  tileParamsBackward;
		gui::TileParams  tileParamsForward;

		ShaderParams();

		void  ChooseParams(const gui_cs0::ComboBox& cbox,
						const gui_cs0::ComboBoxButton& buttonOpen,
						color4b& cBaseText,
						ParamBlockArea& pbaBase, ParamBlockArea& pbaButtonOpen,
						ParamBlockTile& pbtButtonOpen) const;

		void  ChooseParams(const gui_cs0::ComboBoxPaneButton& button,
						ParamBlockArea& pbaPaneButton, ParamBlockTile& pbtPaneButton) const;

		void  CalcPaneTileRectangle(gui::Rectangle& rTile,
						const gui::Rectangle& rPaneButton, const ParamBlockTile& pbt) const;
	};


public:
	//------------------------------
	// CS0_ComboBoxShader interface.
	//------------------------------
	CS0_ComboBoxShader(const std::string& shaderName = "",
						const ShaderParams& sp = ShaderParams(),
						const gui::TextAttributes& ta = DefaultTextAttributes());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams());
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------------
	// ComboBox::Shader implementation.
	//---------------------------------
	virtual  void  Shade(const gui_cs0::ComboBox& cbox, gui::Renderer& gr) const;
	virtual  void  Shade(const gui_cs0::ComboBoxPane& pane, gui::Renderer& gr) const;


protected:
	//-----------------------------
	// VideoResourceUser overrides.
	//-----------------------------
	virtual  bool  CacheTileTextureHandles(gui::Manager& gm);
	virtual  void  PostCacheTileTextureHandles();


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;

	//-------
	// Cache.
	//-------
	const gui::TileTexture*   pTileTexOpenTop;
	const gui::TileTexture*   pTileTexOpenBottom;
	const gui::TileTexture*   pTileTexOpenCenter;
	const gui::TileTexture*   pTileTexBackward;
	const gui::TileTexture*   pTileTexForward;
	mutable gui::RectangleCT  rTileOpenTop;
	mutable gui::RectangleCT  rTileOpenBottom;
	mutable gui::RectangleCT  rTileOpenCenter;
	mutable gui::RectangleCT  rTileBackward;
	mutable gui::RectangleCT  rTileForward;
};


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


