

//--------------------------------------------------------------------
#include "GUI_SS0\EditBox.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_Tools.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
CS0_EditBoxShader::ShaderParams::ShaderParams()
{
	colorDisabledBgr = color4b::GRAY;
	colorDisabledBorder = color4b::LIGHT_GRAY;
	colorDisabledText = color4b::LIGHT_GRAY;

	colorEnabledBgr = color4b(224, 224, 200);
	colorEnabledBorder = color4b::BLACK;
	colorEnabledText = color4b::BLACK;

	colorFocusedBgr = color4b(224, 224, 200);
	colorFocusedBorder = color4b::BLUE;
	colorFocusedText = color4b::BLUE;

	borderWidth = 1;

	colorCaret = color4b::RED;
	caretWidth = 1;
}


//====================================================================
void  CS0_EditBoxShader::ShaderParams::ChooseParams(const gui_cs0::EditBox& editBox,
									color4b& cBgr, color4b& cBorder, color4b& cText) const
{
	if(editBox.IsEnabled()) {
		if(editBox.IsFocused()) {
			cBgr = colorFocusedBgr;
			cBorder = colorFocusedBorder;
			cText = colorFocusedText;
		}
		else {
			cBgr = colorEnabledBgr;
			cBorder = colorEnabledBorder;
			cText = colorEnabledText;
		}
	}
	else {
		cBgr = colorDisabledBgr;
		cBorder = colorDisabledBorder;
		cText = colorDisabledText;
	}
}


//====================================================================
gui::TextAttributes __fastcall CS0_EditBoxShader::DefaultTextAttributes()
{
	gui::TextAttributes ta = gui::BaseShader::DefaultTextAttributes();
	ta.textMetricsL.textHeight = 20;
	return ta;
}


//====================================================================
CS0_EditBoxShader::CS0_EditBoxShader(const std::string& shaderName,
									const ShaderParams& sp,
									const gui::TextAttributes& ta)
	:
	gui_cs0::EditBox::Shader(shaderName),
	shaderParams(sp)
{
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_EditBoxShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::EditBox& editBox = gui::ComponentCast<gui_cs0::EditBox>(c);

	// choose params
	color4b  cBgr, cBorder, cText;
	shaderParams.ChooseParams(editBox, cBgr, cBorder, cText);

	gui::RectangleC  r(vec2::ZERO, editBox.GetSize());

	// draw background
	r.SetColor(cBgr);
	gr.DrawRectangle(r);

	// draw border
	r.SetColor(cBorder);
	gr.DrawRectangleFrame(r, shaderParams.borderWidth);

	// draw text
	if(! pFont) return;

	const gui_cs0::EditBox::GeomParams& gp = editBox.GetGeomParams();
	gui::TextMetricsL& tml = GetTextMetricsLCache(gp.textHeight);

	vec2  textOffset(gp.textPadding, (editBox.GetSize().y - tml.textHeight) * 0.5f);

	uint firstVisible, maxVisible;
	editBox.GetVisibleTextInfo(firstVisible, maxVisible);

	gui::DrawTextParams  dtp(cText, textOffset, tml);
	gr.DrawText(*pFont, maxVisible, editBox.GetText().c_str() + firstVisible, dtp);

	// draw caret
	if(editBox.IsFocused()) {
		r.Set(editBox.GetCaretDrawPosition(), textOffset.y, shaderParams.caretWidth, tml.textHeight);
		r.SetColor(shaderParams.colorCaret);
		gr.DrawRectangle(r);
	}
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


