

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "GUI_CS0\Label.h"
#include "private\vw_gui_ss0_exports.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
class VW_GUI_SS0_API CS0_LabelShader_Static : public gui_cs0::Label::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ShaderParams
	{
		color4b  colorDisabled;
		color4b  colorEnabled;
		color4b  colorHot;

		ShaderParams();
	};


public:
	//----------------------------------
	// CS0_LabelShader_Static interface.
	//----------------------------------
	CS0_LabelShader_Static(const std::string& shaderName = "",
							const ShaderParams& sp = ShaderParams(),
							const gui::TextAttributes& ta = DefaultTextAttributes());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams()) { shaderParams = sp; }
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;
};


//====================================================================
class VW_GUI_SS0_API CS0_LabelShader_Active : public gui_cs0::Label::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ShaderParams
	{
		color4b  colorDisabledText;
		color4b  colorEnabledText;
		color4b  colorHotBorder;
		color4b  colorPressedBorder;
		float    borderWidth;
		vec2     borderOffset;

		ShaderParams();
	};


public:
	//----------------------------------
	// CS0_LabelShader_Active interface.
	//----------------------------------
	CS0_LabelShader_Active(const std::string& shaderName = "",
							const ShaderParams& sp = ShaderParams(),
							const gui::TextAttributes& ta = DefaultTextAttributes());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams()) { shaderParams = sp; }
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;
};


//====================================================================
class VW_GUI_SS0_API CS0_LabelShader_Link : public gui_cs0::Label::Shader
{
public:
	//----------------
	// Parameter sets.
	//----------------
	struct VW_GUI_SS0_API ShaderParams
	{
		color4b  colorDisabled;
		color4b  colorEnabled;
		color4b  colorHotText;
		color4b  colorHotUnderline;
		color4b  colorPressedText;
		color4b  colorPressedUnderline;
		float    underlineWidth;
		float    underlineOffset;

		ShaderParams();

		void  ChooseParams(const gui_cs0::Label& label,
					color4b& cText, color4b& cUnderline, bool& bDrawUnderline) const;
	};


public:
	//--------------------------------
	// CS0_LabelShader_Link interface.
	//--------------------------------
	CS0_LabelShader_Link(const std::string& shaderName = "",
						const ShaderParams& sp = ShaderParams(),
						const gui::TextAttributes& ta = DefaultTextAttributes());

	void                 SetShaderParams(const ShaderParams& sp = ShaderParams()) { shaderParams = sp; }
	const ShaderParams&  GetShaderParams() const { return shaderParams; }

	//---------------------------
	// BaseShader implementation.
	//---------------------------
	virtual  void  Shade(const gui::Component& c, gui::Renderer& gr) const;


protected:
	//------------
	// Properties.
	//------------
	ShaderParams  shaderParams;
};


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


