

//--------------------------------------------------------------------
#include "GUI_SS0\LogoSpinners.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
CS0_LogoSpinnerShader::ShaderParams::ShaderParams()
{
	colorFrontTop = color4b(255, 255, 255);
	colorFrontBottom = color4b(64, 64, 64);
	colorBackTop = color4b(192, 192, 192);
	colorBackBottom = color4b(0, 0, 0);
	colorFrontTopHot = color4b(255, 255, 0);
	colorFrontBottomHot = color4b(64, 64, 0);
	colorBackTopHot = color4b(0, 0, 255);
	colorBackBottomHot = color4b(0, 0, 64);
}


//====================================================================
void  CS0_LogoSpinnerShader::ShaderParams::ChooseParams(
							const gui_cs0::LogoSpinner& logoSpinner,
							color4b& cFrontTop, color4b& cFrontBottom,
							color4b& cBackTop, color4b& cBackBottom) const
{
	if(logoSpinner.IsHot()) {
		cFrontTop = colorFrontTopHot;
		cFrontBottom = colorFrontBottomHot;
		cBackTop = colorBackTopHot;
		cBackBottom = colorBackBottomHot;
	}
	else {
		cFrontTop = colorFrontTop;
		cFrontBottom = colorFrontBottom;
		cBackTop = colorBackTop;
		cBackBottom = colorBackBottom;
	}
}


//====================================================================
CS0_LogoSpinnerShader::CS0_LogoSpinnerShader(const std::string& shaderName,
											const ShaderParams& sp)
	:
	gui_cs0::LogoSpinner::Shader(shaderName),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_LogoSpinnerShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::LogoSpinner& logo = gui::ComponentCast<gui_cs0::LogoSpinner>(c);

	const gui::TileTexture* pTileTexture = logo.GetTileTexture();
	gui::RectangleCT& rFront = logo.GetTileRectangleFront();
	gui::RectangleCT& rBack = logo.GetTileRectangleBack();

	// setup colors
	color4b cFrontTop, cFrontBottom, cBackTop, cBackBottom;
	shaderParams.ChooseParams(logo, cFrontTop, cFrontBottom, cBackTop, cBackBottom);

	rFront.SetColorTop(cFrontTop);
	rFront.SetColorBottom(cFrontBottom);
	rBack.SetColorTop(cBackTop);
	rBack.SetColorBottom(cBackBottom);

	// do shade
	gui::Renderer::TransformSentry  ts(gr, logo.GetDrawTransform());

	if(pTileTexture) {
		gr.DrawRectangle(*pTileTexture, rFront);
		gr.DrawRectangle(*pTileTexture, rBack);
	}
	else {
		gr.DrawRectangle(rFront);
		gr.DrawRectangle(rBack);
	}
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


