

//--------------------------------------------------------------------
#include "GUI_SS0\Menu.h"
#include "GUI\GUI_Manager.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
CS0_MenuBarShader::ShaderParams::ShaderParams()
{
	colorBgrTop = color4b(157, 185, 235);
	colorBgrBottom = color4b(122, 150, 223);
	colorBorder = colorBgrTop;
	colorBgrTopActive = color4b(61, 149, 255);
	colorBgrBottomActive = color4b(0, 84, 227);
	colorBorderActive = colorBgrTopActive;
	borderWidth = 1;
	colorBorderMoving = colorBgrBottomActive;
	borderWidthMoving = 1;
	colorBorderMovingInDockArea = color4b::RED;
	borderWidthMovingInDockArea = 1;
}


//====================================================================
CS0_MenuBarShader::CS0_MenuBarShader(const std::string& shaderName,
									const ShaderParams& sp)
	:
	gui_cs0::MenuBar::Shader(shaderName),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_MenuBarShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::MenuBar& mbar = gui::ComponentCast<gui_cs0::MenuBar>(c);

	gui::RectangleC r(vec2::ZERO, mbar.GetSize());

	// draw background
	if(mbar.IsActive()) {
		r.SetColorTop(shaderParams.colorBgrTopActive);
		r.SetColorBottom(shaderParams.colorBgrBottomActive);
	}
	else {
		r.SetColorTop(shaderParams.colorBgrTop);
		r.SetColorBottom(shaderParams.colorBgrBottom);
	}
	gr.DrawRectangle(r);

	// draw border
	float borderWidth = 0;
	if(mbar.IsMovingInDockSensorArea()) {
		r.SetColor(shaderParams.colorBorderMovingInDockArea);
		borderWidth = shaderParams.borderWidthMovingInDockArea;
	}
	else if(mbar.IsMoving()) {
		r.SetColor(shaderParams.colorBorderMoving);
		borderWidth = shaderParams.borderWidthMoving;
	}
	else {
		r.SetColor(mbar.IsActive() ? shaderParams.colorBorderActive : shaderParams.colorBorder);
		borderWidth = shaderParams.borderWidth;
	}
	if(borderWidth) {
		gr.DrawRectangleFrame(r, borderWidth);
	}
}


//====================================================================
CS0_MenuTitleShader::ShaderParams::ShaderParams()
{
	colorDisabled = color4b::DARK_GRAY;
	colorEnabled = color4b::BLACK;
	colorActive = color4b::WHITE;
	colorHot = color4b::RED;
}


//====================================================================
CS0_MenuTitleShader::CS0_MenuTitleShader(const std::string& shaderName,
										const ShaderParams& sp,
										const gui::TextAttributes& ta)
	:
	gui_cs0::MenuTitle::Shader(shaderName),
	shaderParams(sp)
{
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_MenuTitleShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::MenuTitle& mtitle = gui::ComponentCast<gui_cs0::MenuTitle>(c);

	// draw text
	if(! pFont) return;

	color4b cText = shaderParams.colorDisabled;
	if(mtitle.IsEnabled()) {
		if(mtitle.IsHot()) {
			cText = shaderParams.colorHot;
		}
		else if(mtitle.IsTitleActive()) {
			cText = shaderParams.colorActive;
		}
		else {
			cText = shaderParams.colorEnabled;
		}
	}

	gui::DrawTextParams  dtp(cText, vec2::ZERO, GetTextMetricsLCache(mtitle.GetGeomParams().textHeight));
	gr.DrawText(*pFont, mtitle.GetText(), dtp);
}


//====================================================================
CS0_MenuPaneShader::ShaderParams::ShaderParams()
{
	colorBgrLeft = color4b(61, 149, 255);
	colorBgrRight = color4b(0, 84, 227);
	colorBorder = colorBgrLeft;
	widthBorder = 1;
}


//====================================================================
CS0_MenuPaneShader::CS0_MenuPaneShader(const std::string& shaderName,
										const ShaderParams& sp)
	:
	gui_cs0::MenuPane::Shader(shaderName),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_MenuPaneShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::MenuPane& pane = gui::ComponentCast<gui_cs0::MenuPane>(c);

	gui::RectangleC r(vec2::ZERO, pane.GetSize());

	// draw background
	r.SetColorLeft(shaderParams.colorBgrLeft);
	r.SetColorRight(shaderParams.colorBgrRight);
	gr.DrawRectangle(r);

	// draw border
	r.SetColor(shaderParams.colorBorder);
	if(shaderParams.widthBorder) {
		gr.DrawRectangleFrame(r, shaderParams.widthBorder);
	}
}


//====================================================================
CS0_MenuItemShader::ShaderParams::ShaderParams()
{
	colorTextDisabled = color4b::LIGHT_GRAY;
	colorTextEnabled = color4b::BLACK;
	colorTextHot = color4b::WHITE;
	colorBgrLeftHot = color4b(0, 0, 255);
	colorBgrRightHot = color4b(0, 0, 128);
	colorTileDisabled = colorTextDisabled;
	colorTileEnabled = colorTextEnabled;
	colorTileHot = colorTextHot;
	tileShrink = 4;

	using namespace gui::std_tile_textures;

	tileParamsOn = MakeTileParams(Tile_Check);
	tileParamsSelected = MakeTileParams(Tile_Bullet);
	tileParamsOpen = MakeTileParams(Tile_ArrowRight);
}


//====================================================================
CS0_MenuItemShader::CS0_MenuItemShader(const std::string& shaderName,
										const ShaderParams& sp,
										const gui::TextAttributes& ta)
	:
	gui_cs0::MenuItem::Shader(shaderName)
{
	pTileTexOn = 0;
	pTileTexSelected = 0;
	pTileTexOpen = 0;

	SetShaderParams(sp);
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_MenuItemShader::SetShaderParams(const ShaderParams& sp)
{
	shaderParams = sp;

	rTileOn.SetTexTransform(shaderParams.tileParamsOn.texTransform);
	rTileSelected.SetTexTransform(shaderParams.tileParamsSelected.texTransform);
	rTileOpen.SetTexTransform(shaderParams.tileParamsOpen.texTransform);

	RequestTileTextureRecache();
}


//====================================================================
void  CS0_MenuItemShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::MenuItem& item = gui::ComponentCast<gui_cs0::MenuItem>(c);
	const gui_cs0::MenuItem::GeomParams& gp = item.GetGeomParams();

	// draw background
	if(item.IsEnabled()  &&  item.IsHot()) {
		gui::RectangleC r(vec2::ZERO, item.GetSize());
		r.SetColorLeft(shaderParams.colorBgrLeftHot);
		r.SetColorRight(shaderParams.colorBgrRightHot);
		gr.DrawRectangle(r);
	}

	// choose params
	color4b cText = shaderParams.colorTextDisabled;
	color4b cTile = shaderParams.colorTileDisabled;

	if(item.IsEnabled()) {
		if(item.IsHot()) {
			cText = shaderParams.colorTextHot;
			cTile = shaderParams.colorTileHot;
		}
		else {
			cText = shaderParams.colorTextEnabled;
			cTile = shaderParams.colorTileEnabled;
		}
	}

	// draw text
	if(! pFont) return;
	vec2 textOffset(gp.padding + gp.iconSize + gp.spacingIconText,
					(gp.itemHeight - gp.textHeight) * 0.5f);

	gui::DrawTextParams  dtp(cText, textOffset, GetTextMetricsLCache(gp.textHeight));
	gr.DrawText(*pFont, item.GetText(), dtp);

	// draw tiles
	if(item.IsOn()  &&  pTileTexOn) {
		PrepareTileRectangle(rTileOn, cTile, gp.padding, item, gp);
		gr.DrawRectangle(*pTileTexOn, rTileOn);
		return;
	}
	if(item.IsSelected()  &&  pTileTexSelected) {
		PrepareTileRectangle(rTileSelected, cTile, gp.padding, item, gp);
		gr.DrawRectangle(*pTileTexSelected, rTileSelected);
		return;
	}
	if(item.GetNumChildren()  &&  pTileTexOpen) {
		PrepareTileRectangle(rTileOpen, cTile, item.GetSize().x - gp.padding - gp.iconSize, item, gp);
		gr.DrawRectangle(*pTileTexOpen, rTileOpen);
	}
}


//====================================================================
void  CS0_MenuItemShader::PrepareTileRectangle(gui::RectangleCT& r,
								color4b color, float leftOffset,
								const gui_cs0::MenuItem& mi,
								const gui_cs0::MenuItem::GeomParams& gp) const
{
	r.width = r.height = gp.iconSize - shaderParams.tileShrink * 2;
	r.left = leftOffset + (gp.iconSize - r.width) * 0.5;
	r.top = (mi.GetSize().y - r.height) * 0.5f;
	r.SetColor(color);
}


//====================================================================
bool  CS0_MenuItemShader::CacheTileTextureHandles(gui::Manager& gm)
{
	pTileTexOn = gm.GetTileTexture(shaderParams.tileParamsOn.texName);
	pTileTexSelected = gm.GetTileTexture(shaderParams.tileParamsSelected.texName);
	pTileTexOpen = gm.GetTileTexture(shaderParams.tileParamsOpen.texName);

	return (pTileTexOn != 0  &&  pTileTexSelected != 0  &&  pTileTexOpen != 0);
}


//====================================================================
void  CS0_MenuItemShader::PostCacheTileTextureHandles()
{
	pTileTexOn->GetTileCoords(shaderParams.tileParamsOn.texRow, shaderParams.tileParamsOn.texCol, rTileOn);
	pTileTexSelected->GetTileCoords(shaderParams.tileParamsSelected.texRow, shaderParams.tileParamsSelected.texCol, rTileSelected);
	pTileTexOpen->GetTileCoords(shaderParams.tileParamsOpen.texRow, shaderParams.tileParamsOpen.texCol, rTileOpen);
}


//====================================================================
CS0_MenuSeparatorShader::ShaderParams::ShaderParams()
{
	colorLeft = color4b(61, 149, 255);
	colorRight = color4b::BLACK;
	padding = 2;
	lineWidth = 1;
}


//====================================================================
CS0_MenuSeparatorShader::CS0_MenuSeparatorShader(const std::string& shaderName,
												const ShaderParams& sp)
	:
	gui_cs0::MenuSeparator::Shader(shaderName),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_MenuSeparatorShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::MenuSeparator& msep = gui::ComponentCast<gui_cs0::MenuSeparator>(c);

	gui::RectangleC r(
		shaderParams.padding, (msep.GetSize().y - shaderParams.lineWidth) * 0.5f,
		msep.GetSize().x - 2 * shaderParams.padding, shaderParams.lineWidth);

	r.SetColorLeft(shaderParams.colorLeft);
	r.SetColorRight(shaderParams.colorRight);
	gr.DrawRectangle(r);
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


