

//--------------------------------------------------------------------
#include "GUI_SS0\PropertyFrame.h"
#include "GUI\GUI_Manager.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
CS0_PropertyTabShader::ShaderParams::ShaderParams()
{
	colorBgrTop = color4b(192, 192, 192);
	colorBgrBottom = color4b(160, 160, 160);
	colorBgrTopHotSelected = color4b(224, 224, 224);
	colorBgrBottomHotSelected = color4b(224, 224, 224);
	colorBorder = color4b::BLACK;
	borderWidth = 1;
	colorHiliteTop = color4b(160, 160, 0);
	colorHiliteBottom = color4b(192, 192, 0);
	hiliteWidth = 4;
	colorText = color4b::BLACK;
	colorTextSelected = color4b::BLACK;
	colorTextHot = color4b::BLUE;
}


//====================================================================
void  CS0_PropertyTabShader::ShaderParams::ChooseParams(const gui_cs0::PropertyTab& tab,
								color4b& cBgrTop, color4b& cBgrBottom) const
{
	if(tab.IsSelected()  ||  tab.IsHot()) {
		cBgrTop = colorBgrTopHotSelected;
		cBgrBottom = colorBgrBottomHotSelected;
	}
	else {
		cBgrTop = colorBgrTop;
		cBgrBottom = colorBgrBottom;
	}
}


//====================================================================
CS0_PropertyTabShader::CS0_PropertyTabShader(const std::string& shaderName,
										const ShaderParams& sp,
										const gui::TextAttributes& ta)
	:
	gui_cs0::PropertyTab::Shader(shaderName),
	shaderParams(sp)
{
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_PropertyTabShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::PropertyTab& tab = gui::ComponentCast<gui_cs0::PropertyTab>(c);

	if(! pFont) return;

	// choose params
	color4b cBgrTop, cBgrBottom;
	shaderParams.ChooseParams(tab, cBgrTop, cBgrBottom);

	gui::RectangleC r(vec2::ZERO, tab.GetSize());

	// draw background
	r.SetColorTop(cBgrTop);
	r.SetColorBottom(cBgrBottom);
	gr.DrawRectangle(r);

	// draw border
	r.SetColor(shaderParams.colorBorder);
	r.width = shaderParams.borderWidth;
	gr.DrawRectangle(r);
	r.left = tab.GetSize().x - shaderParams.borderWidth;
	gr.DrawRectangle(r);

	// draw hilite
	r.Set(shaderParams.borderWidth, 0, tab.GetSize().x - 2 * shaderParams.borderWidth, shaderParams.borderWidth);

	if(tab.IsSelected()  ||  tab.IsHot()) {
		r.height = shaderParams.hiliteWidth;
		r.SetColorTop(shaderParams.colorHiliteTop);
		r.SetColorBottom(shaderParams.colorHiliteBottom);
	}
	gr.DrawRectangle(r);

	// draw text
	vec2 textOffset = 0.5f * (tab.GetSize() - tab.GetTextSize());

	color4b textColor;
	if(tab.IsHot()) textColor = shaderParams.colorTextHot;
	else if(tab.IsSelected()) textColor = shaderParams.colorTextSelected;
	else textColor = shaderParams.colorText;

	gui::DrawTextParams  dtp(textColor, textOffset, GetTextMetricsLCache(tab.GetTextHeight()));
	gr.DrawText(*pFont, tab.GetText(), dtp);
}


//====================================================================
CS0_PropertyPageShader::ShaderParams::ShaderParams()
{
	colorBgrTop = color4b(224, 224, 224);
	colorBgrBottom = color4b(192, 192, 192);
	colorBorder = color4b::BLACK;
	borderWidth = 1;
}


//====================================================================
CS0_PropertyPageShader::CS0_PropertyPageShader(const std::string& shaderName,
												const ShaderParams& sp)
	:
	gui_cs0::PropertyPage::Shader(shaderName),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_PropertyPageShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::PropertyPage& page = gui::ComponentCast<gui_cs0::PropertyPage>(c);

	gui::RectangleC r(vec2::ZERO, page.GetSize());

	// draw background
	r.SetColorTop(shaderParams.colorBgrTop);
	r.SetColorBottom(shaderParams.colorBgrBottom);
	gr.DrawRectangle(r);

	// draw border left
	r.SetColor(shaderParams.colorBorder);

	r.top = shaderParams.borderWidth;
	r.width = shaderParams.borderWidth;
	r.height -= 2 * shaderParams.borderWidth;
	gr.DrawRectangle(r);

	// draw border right
	r.left = page.GetSize().x - shaderParams.borderWidth;
	gr.DrawRectangle(r);

	// draw border bottom
	r.Set(0, page.GetSize().y - shaderParams.borderWidth,
			page.GetSize().x, shaderParams.borderWidth);
	gr.DrawRectangle(r);

	// draw border top
	float tabOffset, tabLen;
	page.GetTabPosition(tabOffset, tabLen);

	r.top = 0;
	r.width = tabOffset + shaderParams.borderWidth;
	gr.DrawRectangle(r);

	r.left = tabOffset + tabLen - shaderParams.borderWidth;
	r.width = page.GetSize().x - r.left;
	gr.DrawRectangle(r);
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


