

//--------------------------------------------------------------------
#include "GUI_SS0\RadioButton.h"
#include "GUI\GUI_Manager.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
CS0_RadioButtonShader::ShaderParams::ShaderParams()
{
	colorDisabled = color4b::GRAY;
	colorDisabledBgr = color4b::LIGHT_GRAY;
	colorEnabledBorder = color4b::BLACK;
	colorEnabledCenterLeft = color4b(64, 64, 64);
	colorEnabledCenterRight = color4b::BLACK;
	colorEnabledBgrLeft = color4b(224, 224, 224);
	colorEnabledBgrRight = colorEnabledBgrLeft;
	colorHotBgrLeft = color4b(128, 128, 0);
	colorHotBgrRight = color4b::YELLOW;
	colorPressedBgrLeft = color4b(128, 128, 128);
	colorPressedBgrRight = color4b(192, 192, 192);
	shrink = 0;
	widthBorder = 1;
	widthOutterRing = 4;
	tileParams = gui::std_tile_textures::MakeTileParams(gui::std_tile_textures::Tile_Bullet);
}


//====================================================================
CS0_RadioButtonShader::CS0_RadioButtonShader(const std::string& shaderName,
											const ShaderParams& sp)
	:
	gui_cs0::RadioButton::Shader(shaderName),
	shaderParams(sp)
{
	pTileTexture = 0;
}


//====================================================================
void  CS0_RadioButtonShader::SetShaderParams(const ShaderParams& sp)
{
	shaderParams = sp;
	RequestTileTextureRecache();
}


//====================================================================
bool  CS0_RadioButtonShader::CacheTileTextureHandles(gui::Manager& gm)
{
	pTileTexture = gm.GetTileTexture(shaderParams.tileParams.texName);
	return (pTileTexture != 0);
}


//====================================================================
void  CS0_RadioButtonShader::PostCacheTileTextureHandles()
{
	if(pTileTexture) {
		pTileTexture->GetTileCoords(shaderParams.tileParams.texRow, shaderParams.tileParams.texCol, rTile);
	}
	rTile.SetTexTransform(shaderParams.tileParams.texTransform);
}


//====================================================================
void  CS0_RadioButtonShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::RadioButton& radioButton = gui::ComponentCast<gui_cs0::RadioButton>(c);

	if(! pTileTexture) return;

	rTile.Set(vec2::ZERO, radioButton.GetSize());

	// choose params
	color4b  cBgrLeft, cBgrRight, cBorder, cCenterLeft, cCenterRight;

	if(radioButton.IsEnabled()) {
		if(radioButton.IsPressed()) {
			cBgrLeft = shaderParams.colorPressedBgrLeft;
			cBgrRight = shaderParams.colorPressedBgrRight;
		}
		else if(radioButton.IsHot()) {
			cBgrLeft = shaderParams.colorHotBgrLeft;
			cBgrRight = shaderParams.colorHotBgrRight;
		}
		else {
			cBgrLeft = shaderParams.colorEnabledBgrLeft;
			cBgrRight = shaderParams.colorEnabledBgrRight;
		}
		cBorder = shaderParams.colorEnabledBorder;
		cCenterLeft = shaderParams.colorEnabledCenterLeft;
		cCenterRight = shaderParams.colorEnabledCenterRight;
	}
	else {
		cBgrLeft = shaderParams.colorDisabledBgr;
		cBgrRight = shaderParams.colorDisabledBgr;
		cBorder = shaderParams.colorDisabled;
		cCenterLeft = shaderParams.colorDisabled;
		cCenterRight = shaderParams.colorDisabled;
	}

	// draw border
	rTile.Shrink(shaderParams.shrink);
	rTile.SetColor(cBorder);
	gr.DrawRectangle(*pTileTexture, rTile);

	// draw background
	rTile.Shrink(shaderParams.widthBorder);
	rTile.SetColorLeft(cBgrLeft);
	rTile.SetColorRight(cBgrRight);
	gr.DrawRectangle(*pTileTexture, rTile);

	// draw selection mark
	if(radioButton.IsSelected()) {
		rTile.Shrink(shaderParams.widthOutterRing);
		rTile.SetColorLeft(cCenterLeft);
		rTile.SetColorRight(cCenterRight);
		gr.DrawRectangle(*pTileTexture, rTile);
	}
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


