

//--------------------------------------------------------------------
#include "GUI_SS0\ScrollBar.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_StdObjects.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
CS0_ScrollBarShader::ShaderParams::ShaderParams()
{
	baseDisabled.c1 = color4b::LIGHT_GRAY;
	baseDisabled.c2 = color4b::LIGHT_GRAY;
	baseDisabled.cBorder = color4b::DARK_GRAY;
	baseDisabled.widthBorder = 1;

	baseEnabled.c1 = color4b(0, 192, 0);
	baseEnabled.c2 = color4b(0, 128, 0);
	baseEnabled.cBorder = color4b::BLACK;
	baseEnabled.widthBorder = 1;

	baseFocused.c1 = baseEnabled.c1;
	baseFocused.c2 = baseEnabled.c2;
	baseFocused.cBorder = color4b::BLACK;
	baseFocused.widthBorder = 1;

	baseHot = baseFocused;

	thumbDisabled.c1 = color4b::LIGHT_GRAY;
	thumbDisabled.c2 = color4b::LIGHT_GRAY;
	thumbDisabled.cBorder = color4b::DARK_GRAY;
	thumbDisabled.widthBorder = 1;

	thumbEnabled.c1 = color4b(157, 185, 235);
	thumbEnabled.c2 = color4b(122, 150, 223);
	thumbEnabled.cBorder = color4b::BLACK;
	thumbEnabled.widthBorder = 1;

	thumbHot.c1 = color4b(61, 149, 255);
	thumbHot.c2 = color4b(0, 84, 227);
	thumbHot.cBorder = color4b::BLUE;
	thumbHot.widthBorder = 1;

	thumbGrabbed.c1 = color4b(61, 149, 255);
	thumbGrabbed.c2 = color4b(0, 84, 227);
	thumbGrabbed.cBorder = color4b::RED;
	thumbGrabbed.widthBorder = 2;

	buttonDisabled.c1 = color4b::LIGHT_GRAY;
	buttonDisabled.c2 = color4b::LIGHT_GRAY;
	buttonDisabled.cBorder = color4b::DARK_GRAY;
	buttonDisabled.widthBorder = 1;

	buttonEnabled.c1 = color4b(157, 185, 235);
	buttonEnabled.c2 = color4b(122, 150, 223);
	buttonEnabled.cBorder = color4b::BLACK;
	buttonEnabled.widthBorder = 1;

	buttonHot.c1 = color4b(61, 149, 255);
	buttonHot.c2 = color4b(0, 84, 227);
	buttonHot.cBorder = color4b::BLUE;
	buttonHot.widthBorder = 1;

	buttonPressed.c1 = color4b(61, 149, 255);
	buttonPressed.c2 = color4b(0, 84, 227);
	buttonPressed.cBorder = color4b::RED;
	buttonPressed.widthBorder = 2;

	tButtonDisabled.color = color4b::DARK_GRAY;
	tButtonDisabled.shrink = 2;

	tButtonEnabled.color = color4b::WHITE;
	tButtonEnabled.shrink = 2;

	tButtonHot.color = color4b::WHITE;
	tButtonHot.shrink = 2;

	tButtonPressed.color = color4b::WHITE;
	tButtonPressed.shrink = 3;

	using namespace gui::std_tile_textures;

	tileParamsUp = MakeTileParams(Tile_ArrowUp);
	tileParamsDown = MakeTileParams(Tile_ArrowDown);
	tileParamsLeft = MakeTileParams(Tile_ArrowLeft);
	tileParamsRight = MakeTileParams(Tile_ArrowRight);
}


//====================================================================
void  CS0_ScrollBarShader::ShaderParams::ChooseParams(
								const gui_cs0::ScrollBarButton& button,
								ParamBlockArea& pba, ParamBlockTile& pbt) const
{
	if(button.IsEnabled()) {
		if(button.IsPressed()) {
			pba = buttonPressed;
			pbt = tButtonPressed;
		}
		else if(button.IsHot()) {
			pba = buttonHot;
			pbt = tButtonHot;
		}
		else {
			pba = buttonEnabled;
			pbt = tButtonEnabled;
		}
	}
	else {
		pba = buttonDisabled;
		pbt = tButtonDisabled;
	}
}


//====================================================================
void  CS0_ScrollBarShader::ShaderParams::ChooseParams(
					const gui_cs0::ScrollBar& sbar,
					ParamBlockArea& pbaBase, ParamBlockArea& pbaThumb,
					ParamBlockArea& pbaButtonBackward, ParamBlockTile& pbtButtonBackward,
					ParamBlockArea& pbaButtonForward, ParamBlockTile& pbtButtonForward) const
{
	// base params
	if(sbar.IsEnabled()) {
		if(sbar.IsHot()) pbaBase = baseHot;
		else if(sbar.IsFocused()) pbaBase = baseFocused;
		else pbaBase = baseEnabled;
	}
	else {
		pbaBase = baseDisabled;
	}

	// thumb params
	const gui_cs0::ScrollBarThumb& thumb = sbar.GetThumb();

	if(thumb.IsEnabled()) {
		if(thumb.IsGrabbed()) pbaThumb = thumbGrabbed;
		else if(thumb.IsHot()) pbaThumb = thumbHot;
		else pbaThumb = thumbEnabled;
	}
	else {
		pbaThumb = thumbDisabled;
	}

	// backward-button params
	ChooseParams(sbar.GetButtonBackward(), pbaButtonBackward, pbtButtonBackward);

	// forward-button params
	ChooseParams(sbar.GetButtonForward(), pbaButtonForward, pbtButtonForward);
}


//====================================================================
CS0_ScrollBarShader::CS0_ScrollBarShader(const std::string& shaderName,
										const ShaderParams& sp)
	:
	gui_cs0::ScrollBar::Shader(shaderName)
{
	pTileTexUp = 0;
	pTileTexDown = 0;
	pTileTexLeft = 0;
	pTileTexRight = 0;

	SetShaderParams(sp);
}


//====================================================================
void  CS0_ScrollBarShader::SetShaderParams(const ShaderParams& sp)
{
	shaderParams = sp;

	rTileUp.SetTexTransform(shaderParams.tileParamsUp.texTransform);
	rTileDown.SetTexTransform(shaderParams.tileParamsDown.texTransform);
	rTileLeft.SetTexTransform(shaderParams.tileParamsLeft.texTransform);
	rTileRight.SetTexTransform(shaderParams.tileParamsRight.texTransform);

	RequestTileTextureRecache();
}


//====================================================================
void  CS0_ScrollBarShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::ScrollBar& sbar = gui::ComponentCast<gui_cs0::ScrollBar>(c);

	// get draw areas
	gui::RectangleC  rBase, rThumb, rButtonBackward, rButtonForward;

	sbar.GetDrawRectangles(rBase, rThumb, rButtonBackward, rButtonForward);

	// choose params
	ParamBlockArea  pbaBase, pbaThumb, pbaButtonBackward, pbaButtonForward;
	ParamBlockTile  pbtButtonBackward, pbtButtonForward;

	shaderParams.ChooseParams(sbar, pbaBase, pbaThumb,
							pbaButtonBackward, pbtButtonBackward,
							pbaButtonForward, pbtButtonForward);

	if(sbar.IsVertical()) {
		rBase.SetColorLeft(pbaBase.c1);
		rBase.SetColorRight(pbaBase.c2);

		rThumb.SetColorLeft(pbaThumb.c1);
		rThumb.SetColorRight(pbaThumb.c2);

		rButtonBackward.SetColorLeft(pbaButtonBackward.c1);
		rButtonBackward.SetColorRight(pbaButtonBackward.c2);

		rButtonForward.SetColorLeft(pbaButtonForward.c1);
		rButtonForward.SetColorRight(pbaButtonForward.c2);
	}
	else {
		rBase.SetColorTop(pbaBase.c1);
		rBase.SetColorBottom(pbaBase.c2);

		rThumb.SetColorTop(pbaThumb.c1);
		rThumb.SetColorBottom(pbaThumb.c2);

		rButtonBackward.SetColorTop(pbaButtonBackward.c1);
		rButtonBackward.SetColorBottom(pbaButtonBackward.c2);

		rButtonForward.SetColorTop(pbaButtonForward.c1);
		rButtonForward.SetColorBottom(pbaButtonForward.c2);
	}

	// do draw
	gr.DrawRectangle(rBase);
	if(pbaBase.widthBorder) {
		rBase.SetColor(pbaBase.cBorder);
		gr.DrawRectangleFrame(rBase, pbaBase.widthBorder);
	}

	gr.DrawRectangle(rThumb);
	if(pbaThumb.widthBorder) {
		rThumb.SetColor(pbaThumb.cBorder);
		gr.DrawRectangleFrame(rThumb, pbaThumb.widthBorder);
	}

	gr.DrawRectangle(rButtonBackward);
	if(pbaButtonBackward.widthBorder) {
		rButtonBackward.SetColor(pbaButtonBackward.cBorder);
		gr.DrawRectangleFrame(rButtonBackward, pbaButtonBackward.widthBorder);
	}

	gr.DrawRectangle(rButtonForward);
	if(pbaButtonForward.widthBorder) {
		rButtonForward.SetColor(pbaButtonForward.cBorder);
		gr.DrawRectangleFrame(rButtonForward, pbaButtonForward.widthBorder);
	}

	if(sbar.IsVertical()) {
		rTileUp.SetColor(pbtButtonBackward.color);
		rTileUp.CopyGeometryParamsFrom(rButtonBackward);
		rTileUp.Shrink(pbtButtonBackward.shrink);
		gr.DrawRectangle(*pTileTexUp, rTileUp);

		rTileDown.SetColor(pbtButtonForward.color);
		rTileDown.CopyGeometryParamsFrom(rButtonForward);
		rTileDown.Shrink(pbtButtonForward.shrink);
		gr.DrawRectangle(*pTileTexDown, rTileDown);
	}
	else {
		rTileLeft.SetColor(pbtButtonBackward.color);
		rTileLeft.CopyGeometryParamsFrom(rButtonBackward);
		rTileLeft.Shrink(pbtButtonBackward.shrink);
		gr.DrawRectangle(*pTileTexLeft, rTileLeft);

		rTileRight.SetColor(pbtButtonForward.color);
		rTileRight.CopyGeometryParamsFrom(rButtonForward);
		rTileRight.Shrink(pbtButtonForward.shrink);
		gr.DrawRectangle(*pTileTexRight, rTileRight);
	}
}


//====================================================================
bool  CS0_ScrollBarShader::CacheTileTextureHandles(gui::Manager& gm)
{
	pTileTexUp = gm.GetTileTexture(shaderParams.tileParamsUp.texName);
	pTileTexDown = gm.GetTileTexture(shaderParams.tileParamsUp.texName);
	pTileTexLeft = gm.GetTileTexture(shaderParams.tileParamsUp.texName);
	pTileTexRight = gm.GetTileTexture(shaderParams.tileParamsUp.texName);

	return (pTileTexUp  &&  pTileTexDown  &&  pTileTexLeft  &&  pTileTexRight);
}


//====================================================================
void  CS0_ScrollBarShader::PostCacheTileTextureHandles()
{
	pTileTexUp->GetTileCoords(shaderParams.tileParamsUp.texRow, shaderParams.tileParamsUp.texCol, rTileUp);
	pTileTexDown->GetTileCoords(shaderParams.tileParamsDown.texRow, shaderParams.tileParamsDown.texCol, rTileDown);
	pTileTexLeft->GetTileCoords(shaderParams.tileParamsLeft.texRow, shaderParams.tileParamsLeft.texCol, rTileLeft);
	pTileTexRight->GetTileCoords(shaderParams.tileParamsRight.texRow, shaderParams.tileParamsRight.texCol, rTileRight);
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


