

//--------------------------------------------------------------------
#include "GUI_SS0\Separators.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_Tools.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
CS0_HorzSeparatorShader::ShaderParams::ShaderParams()
{
	colorLine = color4b::BLACK;
	lineWidth = 1;
	lineOffsetFromCenter = 0;
}


//====================================================================
CS0_HorzSeparatorShader::CS0_HorzSeparatorShader(const std::string& shaderName,
												const ShaderParams& sp)
	:
	gui_cs0::HorzSeparator::Shader(shaderName),
	shaderParams(sp)
{
}


//====================================================================
void  CS0_HorzSeparatorShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::HorzSeparator& hsep = gui::ComponentCast<gui_cs0::HorzSeparator>(c);

	float lineOffsetY = (hsep.GetSize().y - shaderParams.lineWidth) * 0.5f + shaderParams.lineOffsetFromCenter;

	gui::RectangleC  r(0, lineOffsetY, hsep.GetSize().x, shaderParams.lineWidth);

	r.SetColor(shaderParams.colorLine);
	gr.DrawRectangle(r);
}


//====================================================================
CS0_HorzTextSeparatorShader::ShaderParams::ShaderParams()
{
	colorLine = color4b::BLACK;
	colorText = color4b::BLUE;
	lineWidth = 1;
	lineOffsetFromCenter = 0;
}


//====================================================================
CS0_HorzTextSeparatorShader::CS0_HorzTextSeparatorShader(const std::string& shaderName,
													const ShaderParams& sp,
													const gui::TextAttributes& ta)
	:
	gui_cs0::HorzTextSeparator::Shader(shaderName),
	shaderParams(sp)
{
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_HorzTextSeparatorShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::HorzTextSeparator& htsep = gui::ComponentCast<gui_cs0::HorzTextSeparator>(c);
	const gui_cs0::HorzTextSeparator::GeomParams& gp = htsep.GetGeomParams();

	float leftSegmentLen, rightSegmentStart, rightSegmentLen;
	htsep.GetLineSegmentInfo(leftSegmentLen, rightSegmentStart, rightSegmentLen);

	float lineOffsetY = (htsep.GetSize().y - shaderParams.lineWidth) * 0.5f + shaderParams.lineOffsetFromCenter;

	// left segment
	gui::RectangleC  r(0, lineOffsetY, leftSegmentLen, shaderParams.lineWidth);
	r.SetColor(shaderParams.colorLine);
	gr.DrawRectangle(r);

	// right segment
	r.left = rightSegmentStart;
	r.width = rightSegmentLen;
	gr.DrawRectangle(r);

	// draw text
	if(! pFont) return;

	float textPos, textWidth;
	htsep.GetTextInfo(textPos, textWidth);

	float textOffsetY = (htsep.GetSize().y - gp.textHeight) * 0.5f;

	gui::DrawTextParams  dtp(shaderParams.colorText,
			vec2(textPos, textOffsetY), GetTextMetricsLCache(gp.textHeight));

	gr.DrawText(*pFont, htsep.GetText(), dtp);
}


//====================================================================
CS0_ControlGroupFrameShader::ShaderParams::ShaderParams()
{
	colorLine = color4b::BLACK;
	colorText = color4b::BLUE;
	lineWidth = 1;
	lineOffsetFromCenter = 0;
}


//====================================================================
CS0_ControlGroupFrameShader::CS0_ControlGroupFrameShader(const std::string& shaderName,
												const ShaderParams& sp,
												const gui::TextAttributes& ta)
	:
	gui_cs0::ControlGroupFrame::Shader(shaderName),
	shaderParams(sp)
{
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_ControlGroupFrameShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::ControlGroupFrame& cgf = gui::ComponentCast<gui_cs0::ControlGroupFrame>(c);
	const gui_cs0::ControlGroupFrame::GeomParams& gp = cgf.GetGeomParams();

	float leftSegmentLen, rightSegmentStart, rightSegmentLen;
	cgf.GetTopLineSegmentInfo(leftSegmentLen, rightSegmentStart, rightSegmentLen);

	float lineOffsetY = (gp.textHeight - shaderParams.lineWidth) * 0.5f + shaderParams.lineOffsetFromCenter;

	// top-left segment
	gui::RectangleC  r(0, lineOffsetY, leftSegmentLen, shaderParams.lineWidth);
	r.SetColor(shaderParams.colorLine);
	gr.DrawRectangle(r);

	// top-right segment
	r.left = rightSegmentStart;
	r.width = rightSegmentLen;
	gr.DrawRectangle(r);

	// bottom line
	r.Set(0, cgf.GetSize().y - shaderParams.lineWidth, cgf.GetSize().x, shaderParams.lineWidth);
	gr.DrawRectangle(r);

	// left line
	r.top = lineOffsetY + shaderParams.lineWidth;
	r.width = shaderParams.lineWidth;
	r.height = cgf.GetSize().y - r.top - shaderParams.lineWidth;
	gr.DrawRectangle(r);

	// right line
	r.left = cgf.GetSize().x - shaderParams.lineWidth;
	gr.DrawRectangle(r);

	// draw text
	if(! pFont) return;

	float textPos, textWidth;
	cgf.GetTextInfo(textPos, textWidth);

	gui::DrawTextParams  dtp(shaderParams.colorText,
				vec2(textPos, 0), GetTextMetricsLCache(gp.textHeight));

	gr.DrawText(*pFont, cgf.GetText(), dtp);
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


