

//--------------------------------------------------------------------
#include "GUI_SS0\ShaderFactory.h"
#include "GUI\GUI_Manager.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
ShaderFactory::ShaderFactory()
{
	SetAllCreateFlags(true);

	spMenuBarIconButtonBarControl.alphaHotBgr = 0;
	spMenuBarIconButtonBarControl.alphaPressedBgr = 0;
	spMenuBarIconButtonBarControl.colorEnabled = spMenuTitle.colorEnabled;
	spMenuBarIconButtonBarControl.colorHot = spMenuTitle.colorHot;

	spPixelOffsetControlPane.colorControlArea = color4b(61, 149, 255);
	spPixelOffsetControlPane.colorScreenBorder = color4b::WHITE;
	spPixelOffsetControlPaneIconButton.colorDisabled = color4b::BLACK;
	spPixelOffsetControlPaneIconButton.colorEnabled = color4b::BLACK;
	spPixelOffsetControlPaneIconButton.colorHot = color4b::WHITE;
	spPixelOffsetControlPaneIconButton.alphaHotBgr = 0;
	spPixelOffsetControlPaneIconButton.alphaPressedBgr = 0;
	spPixelOffsetControlPaneLabel.colorDisabled = color4b::BLACK;
	spPixelOffsetControlPaneLabel.colorEnabled = color4b::BLACK;
	spPixelOffsetControlPaneLabel.colorHot = color4b::BLACK;
	taPixelOffsetControlPaneLabel = CS0_LabelShader_Static::DefaultTextAttributes();
	taPixelOffsetControlPaneLabel.fontName = gui::std_fonts::FontName_Console;
	taPixelOffsetControlPaneLabel.textMetricsL.bMonospace = true;
	taPixelOffsetControlPaneLabel.textMetricsL.textHeight = 20;

	spConsoleOutputBgr.borderWidth = 0;
	spConsoleScrollBar.baseDisabled.widthBorder = 0;
	spConsoleScrollBar.baseEnabled.widthBorder = 0;
	spConsoleScrollBar.baseFocused.widthBorder = 0;
	spConsoleScrollBar.baseHot.widthBorder = 0;
	spConsoleScrollBar.thumbDisabled.widthBorder = 0;
	spConsoleScrollBar.thumbEnabled.widthBorder = 0;
	spConsoleScrollBar.thumbHot.widthBorder = 0;
	spConsoleScrollBar.thumbGrabbed.widthBorder = 1;
	spConsoleScrollBar.buttonDisabled.widthBorder = 0;
	spConsoleScrollBar.buttonEnabled.widthBorder = 0;
	spConsoleScrollBar.buttonHot.widthBorder = 0;
	spConsoleScrollBar.buttonPressed.widthBorder = 1;
	spConsoleEditBox.colorEnabledBgr = color4b(32, 64, 64);
	spConsoleEditBox.colorFocusedBgr = color4b(64, 128, 128);
	spConsoleEditBox.colorEnabledText = color4b::BLACK;
	spConsoleEditBox.colorFocusedText = color4b::WHITE;
	spConsoleEditBox.colorEnabledBorder = color4b::BLACK;
	spConsoleEditBox.colorFocusedBorder = color4b::BLACK;
	spConsoleEditBox.colorCaret = color4b::RED;
	spConsoleEditBox.caretWidth = 2;
	spConsoleEditBox.borderWidth = 0;
	taConsoleEditBox.fontName = gui::std_fonts::FontName_Console;
	taConsoleEditBox.textMetricsL.bMonospace = true;

	spTopLevelFrame.colorTitleText = color4b(216, 228, 248);
	spTopLevelFrame.colorTitleTextActive = color4b::WHITE;

	spTopLevelFrameIconButtonFrameControlPush.colorDisabled = color4b(216, 228, 248);
	spTopLevelFrameIconButtonFrameControlPush.colorEnabled = spTopLevelFrameIconButtonFrameControlPush.colorDisabled;
	spTopLevelFrameIconButtonFrameControlPush.colorHot = color4b::YELLOW;
	spTopLevelFrameIconButtonFrameControlPush.alphaHotBgr = 0;
	spTopLevelFrameIconButtonFrameControlPush.alphaPressedBgr = 0;

	spTopLevelFrameIconButtonFrameControlToggle.colorDisabled = color4b(216, 228, 248);
	spTopLevelFrameIconButtonFrameControlToggle.colorUntoggled = spTopLevelFrameIconButtonFrameControlToggle.colorDisabled;
	spTopLevelFrameIconButtonFrameControlToggle.colorToggled = color4b::YELLOW;
	spTopLevelFrameIconButtonFrameControlToggle.alphaHotBgr = 0;
	spTopLevelFrameIconButtonFrameControlToggle.alphaPressedBgr = 0;

	spTopLevelFrameTextButtonFrameControlPush.alphaHotBgr = 0;
	spTopLevelFrameTextButtonFrameControlPush.alphaPressedBgr = 0;

	spTopLevelFrameTextButtonFrameControlToggle.alphaHotBgr = 0;
	spTopLevelFrameTextButtonFrameControlToggle.alphaPressedBgr = 0;

	spTopLevelPaneIconButtonPaneControl.colorDisabled = color4b(255, 255, 0, 160);
	spTopLevelPaneIconButtonPaneControl.colorEnabled = spTopLevelPaneIconButtonPaneControl.colorDisabled;
	spTopLevelPaneIconButtonPaneControl.colorHot = color4b(255, 255, 0);
	spTopLevelPaneIconButtonPaneControl.alphaHotBgr = 0;
	spTopLevelPaneIconButtonPaneControl.alphaPressedBgr = 0;

	spOverlayLabelStatic.colorDisabled = color4b(255, 255, 255, 192);
	spOverlayLabelStatic.colorEnabled = spOverlayLabelStatic.colorDisabled;
	spOverlayLabelStatic.colorHot = spOverlayLabelStatic.colorDisabled;
}


//====================================================================
void  ShaderFactory::SetAllCreateFlags(bool bCreate)
{
	for(uint i = 0; i < gui_cs0::Shader_Count; i++) {
		SetCreateFlag(gui_cs0::eShader(i), bCreate);
	}
}


//====================================================================
void  ShaderFactory::SetCreateFlag(gui_cs0::eShader s, bool bCreate)
{
	createFlags[s] = bCreate;
}


//====================================================================
void  ShaderFactory::CreateShaders() const
{
	using namespace gui_cs0;

	if(IsCreateFlagSet(Shader_Label_Static)) {
		new CS0_LabelShader_Static(DefaultShaderName(Shader_Label_Static), spLabelStatic);
	}
	if(IsCreateFlagSet(Shader_Label_Active)) {
		new CS0_LabelShader_Active(DefaultShaderName(Shader_Label_Active), spLabelActive);
	}
	if(IsCreateFlagSet(Shader_Label_Link)) {
		new CS0_LabelShader_Link(DefaultShaderName(Shader_Label_Link), spLabelLink);
	}
	if(IsCreateFlagSet(Shader_TextButton_Press)) {
		new CS0_TextButtonShader_PushOpaque(DefaultShaderName(Shader_TextButton_Press), spTextButtonPush);
	}
	if(IsCreateFlagSet(Shader_TextButton_Push)) {
		new CS0_TextButtonShader_PushOpaque(DefaultShaderName(Shader_TextButton_Push), spTextButtonPush);
	}
	if(IsCreateFlagSet(Shader_TextButton_Toggle)) {
		new CS0_TextButtonShader_ToggleOpaque(DefaultShaderName(Shader_TextButton_Toggle), spTextButtonToggle);
	}
	if(IsCreateFlagSet(Shader_IconButton_Press)) {
		new CS0_IconButtonShader_PushOpaque(DefaultShaderName(Shader_IconButton_Press), spIconButtonPush);
	}
	if(IsCreateFlagSet(Shader_IconButton_Push)) {
		new CS0_IconButtonShader_PushOpaque(DefaultShaderName(Shader_IconButton_Push), spIconButtonPush);
	}
	if(IsCreateFlagSet(Shader_IconButton_Toggle)) {
		new CS0_IconButtonShader_ToggleOpaque(DefaultShaderName(Shader_IconButton_Toggle), spIconButtonToggle);
	}
	if(IsCreateFlagSet(Shader_CheckBox)) {
		new CS0_CheckBoxShader_Opaque(DefaultShaderName(Shader_CheckBox), spCheckBox);
	}
	if(IsCreateFlagSet(Shader_RadioButton)) {
		new CS0_RadioButtonShader(DefaultShaderName(Shader_RadioButton), spRadioButton);
	}
	if(IsCreateFlagSet(Shader_EditBox)) {
		new CS0_EditBoxShader(DefaultShaderName(Shader_EditBox), spEditBox);
	}
	if(IsCreateFlagSet(Shader_ComboBox)) {
		new CS0_ComboBoxShader(DefaultShaderName(Shader_ComboBox), spComboBox);
	}
	if(IsCreateFlagSet(Shader_ScrollBar)) {
		new CS0_ScrollBarShader(DefaultShaderName(Shader_ScrollBar), spScrollBar);
	}
	if(IsCreateFlagSet(Shader_HorzSeparator)) {
		new CS0_HorzSeparatorShader(DefaultShaderName(Shader_HorzSeparator), spHorzSeparator);
	}
	if(IsCreateFlagSet(Shader_HorzTextSeparator)) {
		new CS0_HorzTextSeparatorShader(DefaultShaderName(Shader_HorzTextSeparator), spHorzTextSeparator);
	}
	if(IsCreateFlagSet(Shader_ControlGroupFrame)) {
		new CS0_ControlGroupFrameShader(DefaultShaderName(Shader_ControlGroupFrame), spControlGroupFrame);
	}
	if(IsCreateFlagSet(Shader_LogoSpinner)) {
		new CS0_LogoSpinnerShader(DefaultShaderName(Shader_LogoSpinner), spLogoSpinner);
	}
	if(IsCreateFlagSet(Shader_ToolTip)) {
		new CS0_ToolTipShader(DefaultShaderName(Shader_ToolTip), spToolTip);
	}
	if(IsCreateFlagSet(Shader_RubberBand)) {
		new CS0_RubberbandShader(DefaultShaderName(Shader_RubberBand), spRubberBand);
	}
	if(IsCreateFlagSet(Shader_MenuBar)) {
		new CS0_MenuBarShader(DefaultShaderName(Shader_MenuBar), spMenuBar);
	}
	if(IsCreateFlagSet(Shader_MenuBar_IconButton_BarControl)) {
		new CS0_IconButtonShader_PushTranslucent(DefaultShaderName(Shader_MenuBar_IconButton_BarControl), spMenuBarIconButtonBarControl);
	}
	if(IsCreateFlagSet(Shader_MenuTitle)) {
		new CS0_MenuTitleShader(DefaultShaderName(Shader_MenuTitle), spMenuTitle);
	}
	if(IsCreateFlagSet(Shader_MenuPane)) {
		new CS0_MenuPaneShader(DefaultShaderName(Shader_MenuPane), spMenuPane);
	}
	if(IsCreateFlagSet(Shader_MenuSeparator)) {
		new CS0_MenuSeparatorShader(DefaultShaderName(Shader_MenuSeparator), spMenuSeparator);
	}
	if(IsCreateFlagSet(Shader_MenuItem)) {
		new CS0_MenuItemShader(DefaultShaderName(Shader_MenuItem), spMenuItem);
	}
	if(IsCreateFlagSet(Shader_PixelOffsetControlPane)) {
		new CS0_PixelOffsetControlPaneShader(DefaultShaderName(Shader_PixelOffsetControlPane), spPixelOffsetControlPane);
	}
	if(IsCreateFlagSet(Shader_PixelOffsetControlPane_IconButton)) {
		new CS0_IconButtonShader_PushTranslucent(DefaultShaderName(Shader_PixelOffsetControlPane_IconButton), spPixelOffsetControlPaneIconButton);
	}
	if(IsCreateFlagSet(Shader_PixelOffsetControlPane_Label)) {
		new CS0_LabelShader_Static(DefaultShaderName(Shader_PixelOffsetControlPane_Label), spPixelOffsetControlPaneLabel, taPixelOffsetControlPaneLabel);
	}
	if(IsCreateFlagSet(Shader_PropertyTab)) {
		new CS0_PropertyTabShader(DefaultShaderName(Shader_PropertyTab), spPropertyTab);
	}
	if(IsCreateFlagSet(Shader_PropertyPage)) {
		new CS0_PropertyPageShader(DefaultShaderName(Shader_PropertyPage), spPropertyPage);
	}
	if(IsCreateFlagSet(Shader_Console_OutputBgr)) {
		new CS0_ConsoleOutputBgrShader(DefaultShaderName(Shader_Console_OutputBgr), spConsoleOutputBgr);
	}
	if(IsCreateFlagSet(Shader_Console_EditBox)) {
		new CS0_EditBoxShader(DefaultShaderName(Shader_Console_EditBox), spConsoleEditBox, taConsoleEditBox);
	}
	if(IsCreateFlagSet(Shader_Console_ScrollBar)) {
		new CS0_ScrollBarShader(DefaultShaderName(Shader_Console_ScrollBar), spConsoleScrollBar);
	}
	if(IsCreateFlagSet(Shader_TopLevelFrame)) {
		new CS0_TopLevelFrameShader(DefaultShaderName(Shader_TopLevelFrame), spTopLevelFrame);
	}
	if(IsCreateFlagSet(Shader_TopLevelFrame_IconButton_FrameControl_Push)) {
		new CS0_IconButtonShader_PushTranslucent(DefaultShaderName(Shader_TopLevelFrame_IconButton_FrameControl_Push), spTopLevelFrameIconButtonFrameControlPush);
	}
	if(IsCreateFlagSet(Shader_TopLevelFrame_IconButton_FrameControl_Toggle)) {
		new CS0_IconButtonShader_ToggleTranslucent(DefaultShaderName(Shader_TopLevelFrame_IconButton_FrameControl_Toggle), spTopLevelFrameIconButtonFrameControlToggle);
	}
	if(IsCreateFlagSet(Shader_TopLevelFrame_TextButton_FrameControl_Push)) {
		new CS0_TextButtonShader_PushTranslucent(DefaultShaderName(Shader_TopLevelFrame_TextButton_FrameControl_Push), spTopLevelFrameTextButtonFrameControlPush);
	}
	if(IsCreateFlagSet(Shader_TopLevelFrame_TextButton_FrameControl_Toggle)) {
		new CS0_TextButtonShader_ToggleTranslucent(DefaultShaderName(Shader_TopLevelFrame_TextButton_FrameControl_Toggle), spTopLevelFrameTextButtonFrameControlToggle);
	}
	if(IsCreateFlagSet(Shader_TopLevelPane)) {
		new CS0_TopLevelPaneShader(DefaultShaderName(Shader_TopLevelPane), spTopLevelPane);
	}
	if(IsCreateFlagSet(Shader_TopLevelPane_IconButton_PaneControl)) {
		new CS0_IconButtonShader_PushTranslucent(DefaultShaderName(Shader_TopLevelPane_IconButton_PaneControl), spTopLevelPaneIconButtonPaneControl);
	}
	if(IsCreateFlagSet(Shader_Overlay_Label_Static)) {
		new CS0_LabelShader_Static(DefaultShaderName(Shader_Overlay_Label_Static), spOverlayLabelStatic);
	}
	if(IsCreateFlagSet(Shader_Overlay_TextButton_Press)) {
		new CS0_TextButtonShader_PushTranslucent(DefaultShaderName(Shader_Overlay_TextButton_Press), spOverlayTextButtonPush);
	}
	if(IsCreateFlagSet(Shader_Overlay_TextButton_Push)) {
		new CS0_TextButtonShader_PushTranslucent(DefaultShaderName(Shader_Overlay_TextButton_Push), spOverlayTextButtonPush);
	}
	if(IsCreateFlagSet(Shader_Overlay_TextButton_Toggle)) {
		new CS0_TextButtonShader_ToggleTranslucent(DefaultShaderName(Shader_Overlay_TextButton_Toggle), spOverlayTextButtonToggle);
	}
	if(IsCreateFlagSet(Shader_Overlay_IconButton_Press)) {
		new CS0_IconButtonShader_PushTranslucent(DefaultShaderName(Shader_Overlay_IconButton_Press), spOverlayIconButtonPush);
	}
	if(IsCreateFlagSet(Shader_Overlay_IconButton_Push)) {
		new CS0_IconButtonShader_PushTranslucent(DefaultShaderName(Shader_Overlay_IconButton_Push), spOverlayIconButtonPush);
	}
	if(IsCreateFlagSet(Shader_Overlay_IconButton_Toggle)) {
		new CS0_IconButtonShader_ToggleTranslucent(DefaultShaderName(Shader_Overlay_IconButton_Toggle), spOverlayIconButtonToggle);
	}
	if(IsCreateFlagSet(Shader_Overlay_CheckBox)) {
		new CS0_CheckBoxShader_Translucent(DefaultShaderName(Shader_Overlay_CheckBox), spOverlayCheckBox);
	}
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


