

//--------------------------------------------------------------------
#include "GUI_SS0\ToolTip.h"
#include "GUI\GUI_Manager.h"
#include "GUI\GUI_Tools.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
CS0_ToolTipShader::ShaderParams::ShaderParams()
{
	colorBorder = color4b::BLACK;
	colorBgr = color4b::YELLOW;
	colorText = color4b::BLACK;
	borderWidth = 1;
}


//====================================================================
gui::TextAttributes __fastcall CS0_ToolTipShader::DefaultTextAttributes()
{
	gui::TextAttributes ta = gui::BaseShader::DefaultTextAttributes();
	ta.textMetricsL.textHeight = 14;
	return ta;
}


//====================================================================
CS0_ToolTipShader::CS0_ToolTipShader(const std::string& shaderName,
									const ShaderParams& sp,
									const gui::TextAttributes& ta)
	:
	gui_cs0::ToolTip::Shader(shaderName),
	shaderParams(sp)
{
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_ToolTipShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::ToolTip& toolTip = gui::ComponentCast<gui_cs0::ToolTip>(c);

	ubyte alpha = toolTip.GetTranslucency();

	gui::RectangleC  rBorder(vec2::ZERO, toolTip.GetSize());

	// draw background
	color4b cBgr = shaderParams.colorBgr;
	cBgr.a = alpha;

	rBorder.SetColor(cBgr);
	gr.DrawRectangle(rBorder);

	// draw border
	color4b cBorder = shaderParams.colorBorder;
	cBorder.a = alpha;

	rBorder.SetColor(cBorder);
	gr.DrawRectangleFrame(rBorder, shaderParams.borderWidth);

	// draw text
	if(! pFont) return;

	color4b cText = shaderParams.colorText;
	cText.a = alpha;

	gui::DrawTextLines(gr, *pFont, cText,
				0.5f * (toolTip.GetSize() - toolTip.GetTextSize()),
				GetTextAttributes().textMetricsL, toolTip.GetTextLines());
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


