

//--------------------------------------------------------------------
#include "GUI_SS0\TopLevelFrame.h"
#include "GUI\GUI_Manager.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
CS0_TopLevelFrameShader::ShaderParams::ShaderParams()
{
	colorMainArea1 = color4b(236, 233, 216);
	colorMainArea2 = color4b(180, 177, 160);
	colorTitleBar1 = color4b(157, 185, 235);
	colorTitleBar2 = color4b(122, 150, 223);
	colorTitleBarActive1 = color4b(61, 149, 255);
	colorTitleBarActive2 = color4b(0, 84, 227);
	colorTitleText = color4b(216, 228, 248);
	colorTitleTextActive = color4b::WHITE;
}


//====================================================================
CS0_TopLevelFrameShader::CS0_TopLevelFrameShader(const std::string& shaderName,
											const ShaderParams& sp,
											const gui::TextAttributes ta)
	:
	gui_cs0::TopLevelFrame::Shader(shaderName),
	shaderParams(sp)
{
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_TopLevelFrameShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::TopLevelFrame& tlf = gui::ComponentCast<gui_cs0::TopLevelFrame>(c);
	const gui_cs0::TopLevelFrame::GeomParams& gp = tlf.GetGeomParams();

	// draw main-area
	gui::RectangleC  r(vec2::ZERO, tlf.GetSize());
	r.SetColorLeft(shaderParams.colorMainArea1);
	r.SetColorRight(shaderParams.colorMainArea2);

	gr.DrawRectangle(r);

	// draw title-bar
	r.height = gp.titleBarHeight;

	if(tlf.IsActive()) {
		r.SetColorTop(shaderParams.colorTitleBarActive1);
		r.SetColorBottom(shaderParams.colorTitleBarActive2);
	}
	else {
		r.SetColorTop(shaderParams.colorTitleBar1);
		r.SetColorBottom(shaderParams.colorTitleBar2);
	}
	gr.DrawRectangle(r);

	// setup frame color
	r.SetColor(tlf.IsActive() ? shaderParams.colorTitleBarActive2 : shaderParams.colorTitleBar2);

	// draw bottom-frame
	r.height = gp.frameBorderWidth;
	r.top = tlf.GetSize().y - gp.frameBorderWidth;

	gr.DrawRectangle(r);

	// draw left-frame
	r.top = gp.titleBarHeight;
	r.height = tlf.GetSize().y - gp.frameBorderWidth - gp.titleBarHeight;
	r.left = 0;
	r.width = gp.frameBorderWidth;

	gr.DrawRectangle(r);

	// draw right-frame
	r.left = tlf.GetSize().x - gp.frameBorderWidth;

	gr.DrawRectangle(r);

	// draw title
	if(tlf.IsTitleVisible()) {
		if(! pFont) return;

		gui::DrawTextParams  dtp(
			tlf.IsActive() ? shaderParams.colorTitleTextActive : shaderParams.colorTitleText,
			tlf.GetTitleOffset(), GetTextMetricsLCache(gp.titleTextHeight));

		gr.DrawText(*pFont, tlf.GetTitleText(), dtp);
	}
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


