

//--------------------------------------------------------------------
#include "GUI_SS0\TopLevelPane.h"
#include "GUI\GUI_Manager.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gui_ss0 {


//====================================================================
CS0_TopLevelPaneShader::ShaderParams::ShaderParams()
{
	colorBgr = color4b(0, 0, 128, 64);
	colorBgrActive = color4b(0, 0, 128, 128);
	colorBgrHot = color4b(0, 0, 128, 192);

	colorTitle = color4b(255, 255, 0, 160);
	colorTitleActive = color4b(255, 255, 0);
	colorTitleHot = color4b(255, 255, 0);
}


//====================================================================
void  CS0_TopLevelPaneShader::ShaderParams::ChooseParams(
										const gui_cs0::TopLevelPane& tlp,
										color4b& cBgr, color4b& cTitle) const
{
	if(tlp.IsMouseOver()) {
		cBgr = colorBgrHot;
		cTitle = colorTitleHot;
		return;
	}
	if(tlp.IsActive()) {
		cBgr = colorBgrActive;
		cTitle = colorTitleActive;
		return;
	}
	cBgr = colorBgr;
	cTitle = colorTitle;
}


//====================================================================
CS0_TopLevelPaneShader::CS0_TopLevelPaneShader(const std::string& shaderName,
										const ShaderParams& sp,
										const gui::TextAttributes ta)
	:
	gui_cs0::TopLevelPane::Shader(shaderName),
	shaderParams(sp)
{
	SetTextAttributes(ta);
}


//====================================================================
void  CS0_TopLevelPaneShader::Shade(const gui::Component& c, gui::Renderer& gr) const
{
	const gui_cs0::TopLevelPane& tlp = gui::ComponentCast<gui_cs0::TopLevelPane>(c);

	// choose params
	color4b  cBgr, cTitle;
	shaderParams.ChooseParams(tlp, cBgr, cTitle);

	// draw background
	gui::RectangleC  r(vec2::ZERO, tlp.GetSize());
	r.SetColor(cBgr);
	gr.DrawRectangle(r);

	// draw title
	if(tlp.IsTitleVisible()) {
		if(! pFont) return;

		gui::DrawTextParams  dtp(cTitle, tlp.GetGeomParams().titleOffsetTopLeft,
				GetTextMetricsLCache(tlp.GetGeomParams().titleTextHeight));

		gr.DrawText(*pFont, tlp.GetTitleText(), dtp);
	}
}


//********************************************************************
}//namespace gui_ss0


//********************************************************************
}//namespace vertexware


