

//--------------------------------------------------------------------
#include "private\AppSubsysManager.h"
#include "private\vw_kernel.h"
#include "BaseTools\Stream.h"


//====================================================================
AppSubsysManager::AppSubsysManager()
{
	bSubsysInitalized = false;
	bSubsysConnectedToWindow = false;
	iReconfigCount = 0;
	reconfigCode = 0;
}


//====================================================================
AppSubsysManager::~AppSubsysManager()
{
}


//====================================================================
void  AppSubsysManager::InitializeSubsystems()
{
	if(bSubsysInitalized) return;
	vw_kernel& k = vw_kernel::Instance();

	olog << caption("app-specific-subsystems init");
	k.GetApplication().InitializeSubsystems(iReconfigCount, reconfigCode);
	iReconfigCount++;

	olog << nl;
	bSubsysInitalized = true;
}


//====================================================================
void  AppSubsysManager::CleanupSubsystems()
{
	if(! bSubsysInitalized) return;
	vw_kernel& k = vw_kernel::Instance();

	olog << caption("app-specific-subsystems cleanup");
	k.GetApplication().CleanupSubsystems(iReconfigCount, reconfigCode);

	olog << nl;
	bSubsysInitalized = false;
}


//====================================================================
void  AppSubsysManager::ConnectSubsystemsToAppWindow(VideoSubsystemWindow& w)
{
	if(! bSubsysInitalized) return;
	if(bSubsysConnectedToWindow) return;

	olog << caption("connecting app-specific-subsystems to video-window");
	vw_kernel::Instance().GetApplication().ConnectSubsystemsToAppWindow(w.GetHWND(), w.IsFullscreen());
	olog << nl;

	bSubsysConnectedToWindow = true;
}


//====================================================================
void  AppSubsysManager::DisconnectSubsystemsFromAppWindow()
{
	if(! bSubsysInitalized) return;
	if(! bSubsysConnectedToWindow) return;

	olog << caption("disconnecting app-specific-subsystems from video-window");
	vw_kernel::Instance().GetApplication().DisconnectSubsystemsFromAppWindow();
	olog << nl;

	bSubsysConnectedToWindow = false;
}

