

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "private\PluginManager.h"
#include "Framework\HIDDriver.h"
#include "Framework\HIDDriver0.h"
#include "Framework\HIDDriver8.h"
#include "Framework\DirectInputProxy.h"
#include "Framework\DirectInputProxy8.h"
#include "Framework\HIDController.h"
#include "HIDControllers\Keyboard.h"
#include "HIDControllers\Mouse.h"
#include "BaseTools\Memory.h"
#include <vector>

using namespace vertexware;


//====================================================================
class HIDManager
{
public:
	//------------------------
	// device-related pointers
	//------------------------
	class DeviceLink
	{
	public:
		DeviceLink(HIDDriverPlugin* _pPlugin = 0, HIDDriver* _pDriver = 0,
					HumanInterfaceDevice* _pDevice = 0)
		{
			pPlugin = _pPlugin;
			pDriver = _pDriver;
			pDevice = _pDevice;
		}

		void  DeleteDevice() { SafeDelete(pDevice); }

		HIDDriverPlugin*       GetPlugin() { return pPlugin; }
		HIDDriver*             GetDriver() { return pDriver; }
		HumanInterfaceDevice*  GetDevice() { return pDevice; }

	private:
		HIDDriverPlugin*       pPlugin;
		HIDDriver*             pDriver;
		HumanInterfaceDevice*  pDevice;
	};


public:
	HIDManager();
	~HIDManager();

	void  SetReconfigParams(const DirectInputConfig& diConfig, uint reconfigCode);

	void  InitializeDevices();
	void  CleanupDevices();

	void  ConnectDevicesToWindow(VideoSubsystemWindow& window);
	void  DisconnectDevicesFromWindow();

	void  OnAppWindowActive(bool bActive);
	void  OnAppWindowGeometryChanged();

	void  GetActiveDirectInputConfig(DirectInputConfig& diConfig);

	uint                   GetNumDevices();
	HumanInterfaceDevice*  GetDevice(uint i);

	void                          SetAutoSamplingConfig(const HIDAutoSamplingConfig& asc);
	const HIDAutoSamplingConfig&  GetAutoSamplingConfig() { return autoSamplingConfig; }
	void                          AutoSampleDevices();


private:
	std::string  MakeShortDesc(DeviceLink& deviceLink);
	void         CleanupDevice(DeviceLink& deviceLink);
	void         ExtractNonSystemDevices();

	// driver-interface / direct-input-version wrappers
	void                   EnumerateDirectInputDevices();
	uint                   ProbeDevices(HIDDriver& hidd);
	HumanInterfaceDevice*  CreateDevice(HIDDriver& hidd, uint i);
	HumanInterfaceDevice*  CreateSystemKeyboard();
	HumanInterfaceDevice*  CreateSystemMouse();

	// direct-input-8 specific
	static BOOL CALLBACK  DIEnumDevicesCallback8(const DIDEVICEINSTANCE* deviceInfo, void* v);
	void                  DispatchDirectInputDevice8(const DirectInputDeviceInstanceInfo8& didii);
	bool                  CanHandleDevice8(HIDDriver& hidd, const DirectInputDeviceInstanceInfo8& didii);


private:
	bool  bDevicesInitialized;
	bool  bDevicesConnectedToWindow;

	DirectInputConfig  directInputConfig;
	DirectInputProxy*  pDirectInputProxy;

	std::vector<HumanInterfaceDevice*>  nonSystemDevices;

	HumanInterfaceDevice*  pSystemKeyboard;
	HumanInterfaceDevice*  pSystemMouse;

	bool  bFoundDISysKeyboard;
	bool  bFoundDISysMouse;

	DirectInputDeviceInstanceInfo  didiiSysKeyboard;
	DirectInputDeviceInstanceInfo  didiiSysMouse;

	std::vector<DeviceLink>  deviceLinks;
	HIDAutoSamplingConfig    autoSamplingConfig;

	uint               iReconfigCount;
	uint               reconfigCode;
	DirectInputConfig  diConfigReconfig;
};


