

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\SyncPrimitives.h"
#include "BaseTools\Thread.h"
#include "BaseTools\ConsoleOutputWindow.h"
#include "Framework\Kernel.h"

using namespace vertexware;


//====================================================================
class LogWindowThread : public Thread
{
public:
	enum eWindowPosition
	{
		WindowPosition_Center = 0,
		WindowPosition_TopLeft,
		WindowPosition_TopRight,

		WindowPosition_Count,
		WindowPosition_Force32Bits = 0x7FFFFFFF
	};


public:
	LogWindowThread();

	void  SetTitle(const std::string& s);
	void  EnableCloseGadget(bool bEnableCloseGadget, bool bCallKernelShutdownOnClose);
	void  SetWindowPosition(eWindowPosition pos = WindowPosition_Center);
	void  SplashWindow();
	void  UnsplashWindow();
	void  FocusOnLastLine();

	TopLevelWindow*  GetWindow() { return pConOutWin; }


protected:
	virtual  uint  DoWork();


private:
	void  RealizeTitle();
	void  RealizeCloseGadget();
	void  RealizeLayout();
	void  RealizeAutoActivate();

	void  EnterDynamicUpdate();
	void  LeaveDynamicUpdate();

	//-----------------------
	class DynamicUpdateSentry
	{
	private:
		LogWindowThread&  lwt;
	public:
		DynamicUpdateSentry(LogWindowThread& _lwt) : lwt(_lwt) {
			lwt.EnterDynamicUpdate();
		}
		~DynamicUpdateSentry() {
			lwt.LeaveDynamicUpdate();
		}
	};
	friend class DynamicUpdateSentry;


private:
	ConsoleOutputWindow*  pConOutWin;

	CriticalSection  cs;
	bool  bUpdateTitle;
	bool  bUpdateCloseGadget;
	bool  bUpdateLayout;
	bool  bAutoActivate;

	std::string      sTitle;
	bool             bEnableCloseGadget;
	bool             bCallKernelShutdownOnClose;
	eWindowPosition  windowPosition;
	bool             bSplashWindow;
	bool             bUnsplashWindow;
};


