

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "private\PluginTypes.h"
#include "BaseTools\FileSystem.h"
#include <vector>

using namespace vertexware;


//====================================================================
class PluginManager : public file_system::FileSystemWalker
{
public:
	PluginManager();
	~PluginManager();

	void  DisablePlugin(const std::string& pluginName);
	void  DisableAllHIDDriverPlugins();

	void  LoadAvailablePlugins(const std::string& pluginDirectory);

	bool  SetPluginParameter(const std::string& pluginName, const NamedParameter& np);
	bool  GetPluginParameter(const std::string& pluginName, const std::string& parameterName, NamedParameter& out);

	//---------------------------
	// Generic-extension plugins.
	//---------------------------
	void  ScanGenericExtensionPlugins();

	uint                     GetNumAppUsedGenericExtensionPlugins();
	GenericExtensionPlugin*  GetAppUsedGenericExtensionPlugin(uint i);

	GenericExtensionPlugin*  FindAppUsedGenericExtensionPlugin(
										const std::string* pPluginName,
										const std::string* pGenericExtensionInterfaceName);

	uint                     GetNumKernelUsedGenericExtensionPlugins();
	GenericExtensionPlugin*  GetKernelUsedGenericExtensionPlugin(uint i);

	GenericExtensionPlugin*  FindKernelUsedGenericExtensionPlugin(
										const std::string* pPluginName,
										const std::string* pGenericExtensionInterfaceName);

	//---------------------
	// Application plugins.
	//---------------------
	void                ScanApplicationPlugins(const std::string& requestedPluginName);
	ApplicationPlugin*  GetApplicationPlugin();

	//----------------------------
	// Direct-input-proxy plugins.
	//----------------------------
	void                     ScanDirectInputProxyPlugins();
	DirectInputProxyPlugin*  GetDirectInputProxyPlugin(uint directInputVersion);

	//--------------------
	// HID-driver plugins.
	//--------------------
	void              ScanHIDDriverPlugins();
	uint              GetNumHIDDriverPlugins();
	HIDDriverPlugin*  GetHIDDriverPlugin(uint i);

	//-------------------------
	// Video-subsystem plugins.
	//-------------------------
	void                   ScanVideoSubsystemPlugins();
	VideoSubsystemPlugin*  FindVideoSubsystemPlugin(const std::string& interfaceName);

	//------------------------
	// Video-renderer plugins.
	//------------------------
	void                   ScanVideoRendererPlugins();
	uint                   GetNumVideoRendererPlugins();
	VideoRendererPlugin*   GetVideoRendererPlugin(uint i);
	VideoRendererPlugin*   FindVideoRendererPlugin(const std::string& pluginName);


protected:
	//----------------------------
	// FileSystemWalker overrides.
	//----------------------------
	virtual void OnFile(uint, uint, const std::string& fileName, const std::string& path);

	virtual bool OnDirectory(uint, uint, const std::string&, const std::string&)
	{ return false; }


private:
	void  StorePlugin(VertexwarePlugin& vwp);
	void  CheckForDuplicatePlugins();
	void  ClassifyPlugins();
	void  DeletePlugin(VertexwarePlugin* pPlugin);
	void  CheckForDuplicateDriverPriorities(std::vector<HIDDriverPlugin*>& hiddPlugins);
	bool  ExtractDirectInputVersion(DirectInputProxyPlugin& dipp);
	bool  ExtractDirectInputVersion(HIDDriverPlugin& hiddp);

	VertexwarePlugin*  FindPlugin(const std::string& pluginName);


private:
	std::vector<std::string>  disabledPluginNames;
	bool                      bDisableAllHIDDriverPlugins;

	std::vector<VertexwarePlugin*>        plugins; //owner container
	std::vector<GenericExtensionPlugin*>  appUsedGenericExtensionPlugins;
	std::vector<GenericExtensionPlugin*>  kernelUsedGenericExtensionPlugins;
	std::vector<ApplicationPlugin*>       applicationPlugins;
	std::vector<DirectInputProxyPlugin*>  directInputProxyPlugins;
	std::vector<HIDDriverPlugin*>         hidDriverPlugins;
	std::vector<VideoSubsystemPlugin*>    videoSubsystemPlugins;
	std::vector<VideoRendererPlugin*>     videoRendererPlugins;
};

