

//--------------------------------------------------------------------
#include "private\PluginTypes.h"
#include "BaseTools\Memory.h"
#include "BaseTools\Debug.h"

using namespace vertexware;


//====================================================================
VertexwarePlugin::VertexwarePlugin(const std::string& fileName) :
	DynamicLinkLibrary(fileName)
{
	Clear();
}


//====================================================================
VertexwarePlugin::VertexwarePlugin()
{
	Clear();
}


//====================================================================
VertexwarePlugin::~VertexwarePlugin()
{
	if(bAutoDeletePluginObject) {
		DeletePluginObject();
	}
}


//====================================================================
void  VertexwarePlugin::Clear()
{
	pfnPluginAccessPoint = 0;
	pPluginDescriptor = 0;
	pPluginObject = 0;
	bAutoDeletePluginObject = false;
}


//====================================================================
void  VertexwarePlugin::FillImportTable(DLLImportTable& table)
{
	table.Add(VW_PLUGIN_ACCESS_POINT_NAME, &pfnPluginAccessPoint);
}


//====================================================================
void  VertexwarePlugin::PostImport(bool bAllImported, uint)
{
	if(bAllImported) {
		// cache pointer to plugin-descriptor
		pPluginDescriptor = &pfnPluginAccessPoint();
	}
}


//====================================================================
void  VertexwarePlugin::SafeCopyFrom(const VertexwarePlugin& vwp)
{
	DynamicLinkLibrary::SafeCopyFrom(vwp);
	pfnPluginAccessPoint = vwp.pfnPluginAccessPoint;
	pPluginDescriptor = vwp.pPluginDescriptor;
	pPluginObject = vwp.pPluginObject;
	bAutoDeletePluginObject = false;
}


//====================================================================
bool  VertexwarePlugin::CreatePluginObject()
{
	pPluginObject = pPluginDescriptor->CreateAndSetupPlugin();
	return (pPluginObject != 0);
}


//====================================================================
void  VertexwarePlugin::DeletePluginObject()
{
	SafeDelete(pPluginObject);
}


//====================================================================
void  VertexwarePlugin::EnableAutoDeletePluginObject(bool bEnable)
{
	bAutoDeletePluginObject = bEnable;
}

