

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\DynamicLinkLibrary.h"
#include "Framework\Plugin.h"
#include "Framework\GenericExtension.h"
#include "Framework\Application.h"
#include "Framework\DirectInputProxy.h"
#include "Framework\HIDDriver.h"
#include "Framework\VideoSubsystem.h"
#include "Framework\VideoRenderer.h"

using namespace vertexware;


//====================================================================
class VertexwarePlugin : public DynamicLinkLibrary
{
public:
	VertexwarePlugin(const std::string& fileName);
	virtual ~VertexwarePlugin();

	//-------------------------
	// DLL overrides & helpers.
	//-------------------------
	virtual  void  FillImportTable(DLLImportTable& table);
	virtual  void  PostImport(bool bAllImported, uint);
	void           SafeCopyFrom(const VertexwarePlugin& vwp);

	VertexwarePlugin(const VertexwarePlugin& vwp) { operator=(vwp); }
	VertexwarePlugin&  operator=(const VertexwarePlugin& vwp) { SafeCopyFrom(vwp); return *this; }

	//---------------------
	// Post-import methods.
	//---------------------
	const PluginInfo&  GetPluginInfo() const { return pPluginDescriptor->GetPluginInfo(); }

	//--------------------------
	// Plugin-object management.
	//--------------------------
	bool     CreatePluginObject();
	void     DeletePluginObject();
	void     EnableAutoDeletePluginObject(bool bEnable);
	Plugin*  GetPluginObject() { return pPluginObject; }


protected:
	VertexwarePlugin();


private:
	void  Clear();


private:
	VW_PFN_PLUGIN_ACCESS_POINT  pfnPluginAccessPoint;
	PluginDescriptor*           pPluginDescriptor;
	Plugin*                     pPluginObject;
	bool                        bAutoDeletePluginObject;
};


//====================================================================
class GenericExtensionPlugin : public VertexwarePlugin
{
public:
	GenericExtension*  GetGenericExtension() {
		return static_cast<GenericExtension*>(GetPluginObject());
	}

public:
	std::string  genericExtensionInterfaceName;
	bool         bUsedByKernel;
	bool         bUsedByApp;
};


//====================================================================
class ApplicationPlugin : public VertexwarePlugin
{
public:
	Application*  GetApplication() {
		return static_cast<Application*>(GetPluginObject());
	}
};


//====================================================================
class DirectInputProxyPlugin : public VertexwarePlugin
{
public:
	DirectInputProxy*  GetDirectInputProxy() {
		return static_cast<DirectInputProxy*>(GetPluginObject());
	}

public:
	std::string  directInputProxyInterfaceName;
	uint         directInputVersion;
};


//====================================================================
class HIDDriverPlugin : public VertexwarePlugin
{
public:
	HIDDriver*  GetHIDDriver() {
		return static_cast<HIDDriver*>(GetPluginObject());
	}

public:
	std::string  hidDriverInterfaceName;
	uint         directInputVersion;
	uint         defaultDriverPriority;
	uint         finalDriverPriority;
};


//====================================================================
class VideoSubsystemPlugin : public VertexwarePlugin
{
public:
	VideoSubsystem*  GetVideoSubsystem() {
		return static_cast<VideoSubsystem*>(GetPluginObject());
	}

public:
	std::string   videoSubsystemInterfaceName;
	VideoProfile  videoProfileFiltered;
	VideoProfile  videoProfileUnfiltered;
};


//====================================================================
class VideoRendererPlugin : public VertexwarePlugin
{
public:
	VideoRenderer*  GetVideoRenderer() {
		return static_cast<VideoRenderer*>(GetPluginObject());
	}

public:
	std::string  videoRendererInterfaceName;
	std::string  videoSubsystemInterfaceName;
};


