

//--------------------------------------------------------------------
#include "private\SystemKeyboard.h"
#include "Framework\KernelMacros.h"
#include "BaseTools\WinUtils.h"
#include "BaseTools\Stream.h"
#include "BaseTools\Debug.h"

using namespace vertexware;


//********************************************************************
namespace {


//====================================================================
void __fastcall  FillKeyCodeMapDI(ubyte map[Key_Count])
{
	// keyboard row 1
	map[ Key_Escape ] = DIK_ESCAPE;
	map[ Key_F1     ] = DIK_F1;
	map[ Key_F2     ] = DIK_F2;
	map[ Key_F3     ] = DIK_F3;
	map[ Key_F4     ] = DIK_F4;
	map[ Key_F5     ] = DIK_F5;
	map[ Key_F6     ] = DIK_F6;
	map[ Key_F7     ] = DIK_F7;
	map[ Key_F8     ] = DIK_F8;
	map[ Key_F9     ] = DIK_F9;
	map[ Key_F10    ] = DIK_F10;
	map[ Key_F11    ] = DIK_F11;
	map[ Key_F12    ] = DIK_F12;

	// keyboard row 2
	map[ Key_GraveAccent ] = DIK_GRAVE;
	map[ Key_1           ] = DIK_1;
	map[ Key_2           ] = DIK_2;
	map[ Key_3           ] = DIK_3;
	map[ Key_4           ] = DIK_4;
	map[ Key_5           ] = DIK_5;
	map[ Key_6           ] = DIK_6;
	map[ Key_7           ] = DIK_7;
	map[ Key_8           ] = DIK_8;
	map[ Key_9           ] = DIK_9;
	map[ Key_0           ] = DIK_0;
	map[ Key_Minus       ] = DIK_MINUS;
	map[ Key_Equals      ] = DIK_EQUALS;
	map[ Key_Backspace   ] = DIK_BACK;

	// keyboard row 3
	map[ Key_Tab          ] = DIK_TAB;
	map[ Key_Q            ] = DIK_Q;
	map[ Key_W            ] = DIK_W;
	map[ Key_E            ] = DIK_E;
	map[ Key_R            ] = DIK_R;
	map[ Key_T            ] = DIK_T;
	map[ Key_Y            ] = DIK_Y;
	map[ Key_U            ] = DIK_U;
	map[ Key_I            ] = DIK_I;
	map[ Key_O            ] = DIK_O;
	map[ Key_P            ] = DIK_P;
	map[ Key_LeftBracket  ] = DIK_LBRACKET;
	map[ Key_RightBracket ] = DIK_RBRACKET;
	map[ Key_Enter        ] = DIK_RETURN;

	// keyboard row 4
	map[ Key_CapsLock   ] = DIK_CAPITAL;
	map[ Key_A          ] = DIK_A;
	map[ Key_S          ] = DIK_S;
	map[ Key_D          ] = DIK_D;
	map[ Key_F          ] = DIK_F;
	map[ Key_G          ] = DIK_G;
	map[ Key_H          ] = DIK_H;
	map[ Key_J          ] = DIK_J;
	map[ Key_K          ] = DIK_K;
	map[ Key_L          ] = DIK_L;
	map[ Key_Semicolon  ] = DIK_SEMICOLON;
	map[ Key_Apostrophe ] = DIK_APOSTROPHE;
	map[ Key_Backslash  ] = DIK_BACKSLASH;

	// keyboard row 5
	map[ Key_LeftShift  ] = DIK_LSHIFT;
	map[ Key_OEM102     ] = DIK_OEM_102;
	map[ Key_Z          ] = DIK_Z;
	map[ Key_X          ] = DIK_X;
	map[ Key_C          ] = DIK_C;
	map[ Key_V          ] = DIK_V;
	map[ Key_B          ] = DIK_B;
	map[ Key_N          ] = DIK_N;
	map[ Key_M          ] = DIK_M;
	map[ Key_Comma      ] = DIK_COMMA;
	map[ Key_Period     ] = DIK_PERIOD;
	map[ Key_Slash      ] = DIK_SLASH;
	map[ Key_RightShift ] = DIK_RSHIFT;

	// keyboard row 6
	map[ Key_LeftControl  ] = DIK_LCONTROL;
	map[ Key_LeftWin      ] = DIK_LWIN;
	map[ Key_LeftAlt      ] = DIK_LALT;
	map[ Key_Space        ] = DIK_SPACE;
	map[ Key_RightAlt     ] = DIK_RALT;
	map[ Key_RightWin     ] = DIK_RWIN;
	map[ Key_AppMenu      ] = DIK_APPS;
	map[ Key_RightControl ] = DIK_RCONTROL;

	// keyboard special 1
	map[ Key_PrintScreen ] = DIK_SYSRQ;
	map[ Key_ScrollLock  ] = DIK_SCROLL;
	map[ Key_Pause       ] = DIK_PAUSE;

	// keyboard special 2
	map[ Key_Insert   ] = DIK_INSERT;
	map[ Key_Delete   ] = DIK_DELETE;
	map[ Key_Home     ] = DIK_HOME;
	map[ Key_End      ] = DIK_END;
	map[ Key_PageUp   ] = DIK_PRIOR;
	map[ Key_PageDown ] = DIK_NEXT;

	// keyboard special 3
	map[ Key_ArrowUp    ] = DIK_UP;
	map[ Key_ArrowDown  ] = DIK_DOWN;
	map[ Key_ArrowLeft  ] = DIK_LEFT;
	map[ Key_ArrowRight ] = DIK_RIGHT;

	// keyboard numpad
	map[ Key_NumLock     ] = DIK_NUMLOCK;
	map[ Key_NumDivide   ] = DIK_DIVIDE;
	map[ Key_NumMultiply ] = DIK_MULTIPLY;
	map[ Key_NumSubtract ] = DIK_SUBTRACT;
	map[ Key_NumAdd      ] = DIK_ADD;
	map[ Key_NumEnter    ] = DIK_NUMPADENTER;
	map[ Key_NumDecimal  ] = DIK_DECIMAL;
	map[ Key_Num0        ] = DIK_NUMPAD0;
	map[ Key_Num1        ] = DIK_NUMPAD1;
	map[ Key_Num2        ] = DIK_NUMPAD2;
	map[ Key_Num3        ] = DIK_NUMPAD3;
	map[ Key_Num4        ] = DIK_NUMPAD4;
	map[ Key_Num5        ] = DIK_NUMPAD5;
	map[ Key_Num6        ] = DIK_NUMPAD6;
	map[ Key_Num7        ] = DIK_NUMPAD7;
	map[ Key_Num8        ] = DIK_NUMPAD8;
	map[ Key_Num9        ] = DIK_NUMPAD9;
}


//====================================================================
void __fastcall  FillKeyCodeMapWin32(ubyte map[Key_Count])
{
	// keyboard row 1
	map[ Key_Escape ] = VK_ESCAPE;
	map[ Key_F1     ] = VK_F1;
	map[ Key_F2     ] = VK_F2;
	map[ Key_F3     ] = VK_F3;
	map[ Key_F4     ] = VK_F4;
	map[ Key_F5     ] = VK_F5;
	map[ Key_F6     ] = VK_F6;
	map[ Key_F7     ] = VK_F7;
	map[ Key_F8     ] = VK_F8;
	map[ Key_F9     ] = VK_F9;
	map[ Key_F10    ] = VK_F10;
	map[ Key_F11    ] = 0x7a;
	map[ Key_F12    ] = 0x7b;

	// keyboard row 2
	map[ Key_GraveAccent ] = 0xc0;
	map[ Key_1           ] = '1';
	map[ Key_2           ] = '2';
	map[ Key_3           ] = '3';
	map[ Key_4           ] = '4';
	map[ Key_5           ] = '5';
	map[ Key_6           ] = '6';
	map[ Key_7           ] = '7';
	map[ Key_8           ] = '8';
	map[ Key_9           ] = '9';
	map[ Key_0           ] = '0';
	map[ Key_Minus       ] = 0xbf;
	map[ Key_Equals      ] = 0xbb;
	map[ Key_Backspace   ] = VK_BACK;

	// keyboard row 3
	map[ Key_Tab          ] = VK_TAB;
	map[ Key_Q            ] = 'Q';
	map[ Key_W            ] = 'W';
	map[ Key_E            ] = 'E';
	map[ Key_R            ] = 'R';
	map[ Key_T            ] = 'T';
	map[ Key_Y            ] = 0x5a;
	map[ Key_U            ] = 'U';
	map[ Key_I            ] = 'I';
	map[ Key_O            ] = 'O';
	map[ Key_P            ] = 'P';
	map[ Key_LeftBracket  ] = 0xdb;
	map[ Key_RightBracket ] = 0xdd;
	map[ Key_Enter        ] = VK_RETURN;

	// keyboard row 4
	map[ Key_CapsLock   ] = VK_CAPITAL;
	map[ Key_A          ] = 'A';
	map[ Key_S          ] = 'S';
	map[ Key_D          ] = 'D';
	map[ Key_F          ] = 'F';
	map[ Key_G          ] = 'G';
	map[ Key_H          ] = 'H';
	map[ Key_J          ] = 'J';
	map[ Key_K          ] = 'K';
	map[ Key_L          ] = 'L';
	map[ Key_Semicolon  ] = 0xba;
	map[ Key_Apostrophe ] = 0xde;
	map[ Key_Backslash  ] = 0xdc;

	// keyboard row 5
	map[ Key_LeftShift  ] = 0xa0;
	map[ Key_OEM102     ] = 0xe2;
	map[ Key_Z          ] = 0x59;
	map[ Key_X          ] = 'X';
	map[ Key_C          ] = 'C';
	map[ Key_V          ] = 'V';
	map[ Key_B          ] = 'B';
	map[ Key_N          ] = 'N';
	map[ Key_M          ] = 'M';
	map[ Key_Comma      ] = 0xbc;
	map[ Key_Period     ] = 0xbe;
	map[ Key_Slash      ] = 0xbd;
	map[ Key_RightShift ] = 0xa1;

	// keyboard row 6
	map[ Key_LeftControl  ] = 0xa2;
	map[ Key_LeftWin      ] = 0x5b;
	map[ Key_LeftAlt      ] = 0xa4;
	map[ Key_Space        ] = VK_SPACE;
	map[ Key_RightAlt     ] = 0xa5;
	map[ Key_RightWin     ] = 0x5c;
	map[ Key_AppMenu      ] = VK_APPS;
	map[ Key_RightControl ] = 0xa3;

	// keyboard special 1
	map[ Key_PrintScreen ] = VK_SNAPSHOT;
	map[ Key_ScrollLock  ] = VK_SCROLL;
	map[ Key_Pause       ] = VK_PAUSE;

	// keyboard special 2
	map[ Key_Insert   ] = VK_INSERT;
	map[ Key_Delete   ] = VK_DELETE;
	map[ Key_Home     ] = VK_HOME;
	map[ Key_End      ] = VK_END;
	map[ Key_PageUp   ] = VK_PRIOR;
	map[ Key_PageDown ] = VK_NEXT;

	// keyboard special 3
	map[ Key_ArrowUp    ] = VK_UP;
	map[ Key_ArrowDown  ] = VK_DOWN;
	map[ Key_ArrowLeft  ] = VK_LEFT;
	map[ Key_ArrowRight ] = VK_RIGHT;

	// keyboard numpad
	map[ Key_NumLock     ] = VK_NUMLOCK;
	map[ Key_NumDivide   ] = VK_DIVIDE;
	map[ Key_NumMultiply ] = VK_MULTIPLY;
	map[ Key_NumSubtract ] = VK_SUBTRACT;
	map[ Key_NumAdd      ] = VK_ADD;
	map[ Key_NumEnter    ] = 0; //available only through VK_RETURN
	map[ Key_NumDecimal  ] = VK_DECIMAL;
	map[ Key_Num0        ] = VK_NUMPAD0;
	map[ Key_Num1        ] = VK_NUMPAD1;
	map[ Key_Num2        ] = VK_NUMPAD2;
	map[ Key_Num3        ] = VK_NUMPAD3;
	map[ Key_Num4        ] = VK_NUMPAD4;
	map[ Key_Num5        ] = VK_NUMPAD5;
	map[ Key_Num6        ] = VK_NUMPAD6;
	map[ Key_Num7        ] = VK_NUMPAD7;
	map[ Key_Num8        ] = VK_NUMPAD8;
	map[ Key_Num9        ] = VK_NUMPAD9;
}


//====================================================================
ubyte  g_keyCodeMapDI[Key_Count];
ubyte  g_keyCodeMapWin32[Key_Count];


//====================================================================
struct InitKeyMaps
{
	InitKeyMaps() {
		FillKeyCodeMapDI(g_keyCodeMapDI);
		FillKeyCodeMapWin32(g_keyCodeMapWin32);
	}
} g_initKeyMaps;


//********************************************************************
}//namespace


//====================================================================
SystemKeyboard8::SystemKeyboard8(DirectInputProxy8* dip8,
								const DirectInputDeviceInstanceInfo& didii)
	:
	Keyboard(true)
{
	// init parent-hid
	SetParentHID(*this);

	olog << "Creating SystemKeyboard.\n";

	pDirectInputProxy8 = dip8;
	pAppWindow = 0;
	iDIDevice = 0;
	bDIDeviceConnectedToWindow = false;
	bDIDeviceActive = false;
	bUsingExclusiveMode = false;
	bUseGetAsyncKeyState = false;

	CreateDIDevice();

	DeviceInfo().SetDeviceDescription("SystemKeyboard");
	DeviceInfo().SetDirectInputOnly(false);
	DeviceInfo().SetCanUseDirectInput(pDirectInputProxy8 != 0);
	if(dip8) {
		DeviceInfo().SetDirectInputDeviceInstanceInfo(didii);
	}

	UpdateUsingDirectInput(GetUseDirectInputAtInit() ? ToBool(pDirectInputProxy8) : false);
}


//====================================================================
SystemKeyboard8::~SystemKeyboard8()
{
	DisconnectFromAppWindow();
	ReleaseDIDevice();
}


//====================================================================
void  SystemKeyboard8::ConnectToAppWindow(TopLevelWindow* _pAppWindow,
										HWND hwndAppWindow, bool bFullscreen)
{
	VW_ASSERT(_pAppWindow, "Invalid argument.");

	// setup win32 hooks
	pAppWindow = _pAppWindow;
	bUsingExclusiveMode = false;
	SetupKeyboardMessageDispatch(false, false);

	// setup direct-input device behavior
	if(iDIDevice) {
		bDIDeviceConnectedToWindow = false;

		iDIDevice->Unacquire();

		DWORD dwFlags = DISCL_FOREGROUND;
		if(bFullscreen  &&  pDirectInputProxy8->IsAllowedFullscreenExclusiveMode()) {
			dwFlags |= DISCL_EXCLUSIVE;
		}
		else {
			dwFlags |= DISCL_NONEXCLUSIVE;
		}
		olog << "Setting DirectInput cooperative level: " << pDirectInputProxy8->GetCooperativeLevelDesc(dwFlags) << nl;

		HRESULT hr = iDIDevice->SetCooperativeLevel(pAppWindow->GetHWND(), dwFlags);
		if(SUCCEEDED(hr)) {
			bUsingExclusiveMode = ToBool(dwFlags & DISCL_EXCLUSIVE);
			bDIDeviceConnectedToWindow = true;
		}
		else {
			olog << e_error << "SetCooperativeLevel failed.\n" << e_info;
		}
	}
	OnUseDirectInput(IsUsingDirectInput());
}


//====================================================================
void  SystemKeyboard8::DisconnectFromAppWindow()
{
	// remove win32 hooks
	ResetKeyboardMessageDispatch();
	pAppWindow = 0;

	// disconnect direct-input device from window
	ActivateDIDevice(false);
	bDIDeviceConnectedToWindow = false;
}


//====================================================================
void  SystemKeyboard8::OnStartAsyncSampling(HANDLE samplingThreadHandle, uint samplingThreadID)
{
	KernelThreadInfo kti;
	Kernel::Instance().GetKernelThreadInfo(kti);
	BOOL bOK = ::AttachThreadInput(samplingThreadID, kti.threadID, TRUE);
	bUseGetAsyncKeyState = ! bOK;
}


//====================================================================
void  SystemKeyboard8::OnStopAsyncSampling(HANDLE samplingThreadHandle, uint samplingThreadID)
{
	KernelThreadInfo kti;
	Kernel::Instance().GetKernelThreadInfo(kti);
	::AttachThreadInput(samplingThreadID, kti.threadID, FALSE);
	bUseGetAsyncKeyState = false;
}


//====================================================================
bool  SystemKeyboard8::SampleInputData()
{
	PreSampleUpdate();

	//----------------------
	// Use DirectInputDevice
	//----------------------
	if(bDIDeviceActive) {

		// get direct-input keyboard state
		HRESULT hr = iDIDevice->GetDeviceState(sizeof(diKeyboardState), &diKeyboardState);

		// handle errors
		if(FAILED(hr)) {
			switch(hr) {
				case DIERR_NOTACQUIRED:
				case DIERR_INPUTLOST:
					// device is not acquired or data stream was lost, try to reacquire
					iDIDevice->Unacquire();
					AcquireDIDevice();
					break;

				case E_PENDING:
					return false;
			}
			ResetKeyStates();
			return false;
		}

		// map key codes and report key states
		for(uint i = 0; i < Key_Count; i++) {
			if(diKeyboardState[ g_keyCodeMapDI[i] ] & 0x80) {
				SetKeyPressed(eKey(i));
			}
		}
	}

	//--------------------
	// Use Win32 functions
	//--------------------
	else {
		// get win32 keyboard state
		// NOTES: GetKeyboardState() works only if called from the same
		// thread that created the message-queue, so override this behavior
		// by calling AttachThreadInput(). If attach fails use GetAsyncKeyState().

		if(bUseGetAsyncKeyState) {
			// num-lock / caps-lock toggle will not work when mapping characters
			for(uint vk = 0; vk < 256; vk++) {
				win32KeyboardState[vk] = (ushort(::GetAsyncKeyState(vk)) >> 8) & 0x80;
			}
		}
		else {
			::GetKeyboardState(win32KeyboardState);
		}

		// map key codes and report key states
		for(uint i = 0; i < Key_Count; i++) {
			if(win32KeyboardState[ g_keyCodeMapWin32[i] ] & 0x80) {
				SetKeyPressed(eKey(i));
			}
		}
	}

	// update transitions
	PostSampleUpdate();

	// synthesize win32-keyboard-state if necessary
	if(IsEnabledCharacterMapping()  &&  bDIDeviceActive) {

		if(bUsingExclusiveMode) {
			ClearKeyboardState(win32KeyboardState);

			for(uint i = 0; i < Key_Count; i++) {
				win32KeyboardState[ g_keyCodeMapWin32[i] ] = diKeyboardState[ g_keyCodeMapDI[i] ] & 0x80;
			}

			// update additional virtual-keys
			win32KeyboardState[VK_CONTROL] = win32KeyboardState[VK_LCONTROL] | win32KeyboardState[VK_RCONTROL];
			win32KeyboardState[VK_SHIFT] = win32KeyboardState[VK_LSHIFT] | win32KeyboardState[VK_RSHIFT];
			win32KeyboardState[VK_MENU] = win32KeyboardState[VK_LMENU] | win32KeyboardState[VK_RMENU];

			// update key-lock toggles
			if(KeyDown(Key_NumLock)) Flip(bNumLock);
			if(KeyDown(Key_CapsLock)) Flip(bCapsLock);
			if(KeyDown(Key_ScrollLock)) Flip(bScrollLock);

			win32KeyboardState[VK_NUMLOCK] = bNumLock ? 0x01 : 0x00;
			win32KeyboardState[VK_CAPITAL] = bCapsLock ? 0x01 : 0x00;
			win32KeyboardState[VK_SCROLL]  = bScrollLock ? 0x01 : 0x00;
		}
		else {
			::GetKeyboardState(win32KeyboardState);
		}
	}

	// sampling succeeded
	return true;
}


//====================================================================
bool  SystemKeyboard8::OnKeyDown(WPARAM virtKeyCode, LPARAM keyData)
{
	// not used
	return true;
}


//====================================================================
bool  SystemKeyboard8::OnKeyUp(WPARAM virtKeyCode, LPARAM keyData)
{
	// not used
	return true;
}


//====================================================================
bool  SystemKeyboard8::OnChar(char ch)
{
	// not used
	return true;
}


//====================================================================
bool  SystemKeyboard8::MapKeyCodeToCharacter(eKey keyCode, char& ch) const
{
	// ToAsciiEx ignores VK_NUMLOCK
	if(keyCode >= Key_NumDecimal  &&  keyCode <= Key_Num9) {
		// if num-lock is not toggled, ignore
		if(! (win32KeyboardState[VK_NUMLOCK] & 0x01)) return false;
	}

	HKL hkl = ::GetKeyboardLayout(0);
	uint virtKeyCode = g_keyCodeMapWin32[ keyCode ];
	uint scanCode = ::MapVirtualKeyEx(virtKeyCode, 0, hkl);

	ushort result;
	if(::ToAsciiEx(virtKeyCode, scanCode, win32KeyboardState, &result, 0, hkl) == 1) {
		ch = result & 0x00FF;
		return true;
	}
	return false;
}


//====================================================================
void  SystemKeyboard8::OnUseDirectInput(bool bUseDI)
{
	ActivateDIDevice(bUseDI);
	HumanInterfaceDevice::UpdateUsingDirectInput(bDIDeviceActive);
}


//====================================================================
void  SystemKeyboard8::OnChangedProcessingMode()
{
}


//====================================================================
void  SystemKeyboard8::CreateDIDevice()
{
	iDIDevice = 0;
	if(! pDirectInputProxy8) return;

	// create direct-input system-keyboard
	HRESULT hr = pDirectInputProxy8->GetDirectInputObject()->CreateDevice(
						pDirectInputProxy8->GetGuidSysKeyboard(), &iDIDevice, 0);

	if(SUCCEEDED(hr)) {
		hr = iDIDevice->SetDataFormat(&pDirectInputProxy8->GetDataFormatKeyboard());

		if(SUCCEEDED(hr)) {
			ExtractHardwareProperties();
			return;
		}
		olog << e_error << "Cannot set DirectInput data-format.\n" << e_info;
	}
	else {
		iDIDevice = 0;
		olog << e_error << "Cannot create DirectInput device.\n" << e_info;
	}
	win_utils::SafeRelease(iDIDevice);
}


//====================================================================
void  SystemKeyboard8::ReleaseDIDevice()
{
	if(iDIDevice) {
		iDIDevice->Unacquire();
	}
	win_utils::SafeRelease(iDIDevice);
}


//====================================================================
void  SystemKeyboard8::ActivateDIDevice(bool bActivate)
{
	bDIDeviceActive = false;

	if(iDIDevice  &&  bDIDeviceConnectedToWindow) {
		if(bActivate) {
			bDIDeviceActive = true;
			AcquireDIDevice();
		}
		else {
			iDIDevice->Unacquire();
		}
	}

	ClearKeyboardState(diKeyboardState);
	ClearKeyboardState(win32KeyboardState);
}


//====================================================================
void  SystemKeyboard8::AcquireDIDevice()
{
	// update key-lock toggles
	if(bUsingExclusiveMode) {
		::GetKeyboardState(win32KeyboardState);

		bNumLock = win32KeyboardState[VK_NUMLOCK] & 0x01;
		bCapsLock = win32KeyboardState[VK_CAPITAL] & 0x01;
		bScrollLock = win32KeyboardState[VK_SCROLL] & 0x01;
	}

	iDIDevice->Acquire();
}


//====================================================================
void  SystemKeyboard8::ResetKeyboardMessageDispatch()
{
	if(! pAppWindow) return;

	pAppWindow->SetKeyboardMessageHook(0);
	pAppWindow->EnableMessageDispatchKeyboardKeyStates(true);
	pAppWindow->EnableMessageDispatchKeyboardCharEvents(true);
}


//====================================================================
void  SystemKeyboard8::SetupKeyboardMessageDispatch(
							bool bNeedKeyEvents, bool bNeedCharEvents)
{
	if(! pAppWindow) return;

	pAppWindow->SetKeyboardMessageHook(this);
	pAppWindow->EnableMessageDispatchKeyboardKeyStates(bNeedKeyEvents);
	pAppWindow->EnableMessageDispatchKeyboardCharEvents(bNeedCharEvents);
}


//====================================================================
void  SystemKeyboard8::ClearKeyboardState(ubyte keyboardState[256])
{
	::ZeroMemory(keyboardState, 256);
}


//====================================================================
void  SystemKeyboard8::ExtractHardwareProperties()
{
	if(! iDIDevice) return;
}

