

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\DirectInputProxy8.h"
#include "HIDControllers\Keyboard.h"
#include "BaseTools\TopLevelWindow.h"

using namespace vertexware;


//====================================================================
class SystemKeyboard8 : public HumanInterfaceDevice,
						public Keyboard,
						public KeyboardMessageHook
{
public:
	SystemKeyboard8(DirectInputProxy8* dip8, const DirectInputDeviceInstanceInfo& didii);
	virtual ~SystemKeyboard8();


	//-------------------------------------
	// HumanInterfaceDevice implementation.
	//-------------------------------------
	virtual  uint            GetNumControllers() { return 1; }
	virtual  HIDController&  GetController(uint i) { return *this; }

	virtual  void  ConnectToAppWindow(TopLevelWindow* pAppWindow,
									HWND hwndAppWindow, bool bFullscreen);

	virtual  void  DisconnectFromAppWindow();

	virtual  void  OnStartAsyncSampling(HANDLE samplingThreadHandle, uint samplingThreadID);
	virtual  void  OnStopAsyncSampling(HANDLE samplingThreadHandle, uint samplingThreadID);

	virtual  bool  SampleInputData();


	//-------------------------------
	// KeyboardMessageHook overrides.
	//-------------------------------
	virtual  bool  OnKeyDown(WPARAM virtKeyCode, LPARAM keyData);
	virtual  bool  OnKeyUp(WPARAM virtKeyCode, LPARAM keyData);
	virtual  bool  OnChar(char ch);


	//-------------------------
	// Keyboard implementation.
	//-------------------------
	virtual  bool  MapKeyCodeToCharacter(eKey keyCode, char& ch) const;


protected:
	//-------------------------------------
	// HumanInterfaceDevice implementation.
	//-------------------------------------
	virtual  void  OnUseDirectInput(bool bUseDI);

	//-------------------------
	// Keyboard implementation.
	//-------------------------
	virtual  void  OnChangedProcessingMode();


private:
	void  CreateDIDevice();
	void  ReleaseDIDevice();
	void  ActivateDIDevice(bool bActivate);
	void  AcquireDIDevice();
	void  ResetKeyboardMessageDispatch();
	void  SetupKeyboardMessageDispatch(bool bNeedKeyEvents, bool bNeedCharEvents);
	void  ClearKeyboardState(ubyte keyboardState[256]);
	void  ExtractHardwareProperties();


private:
	DirectInputProxy8*    pDirectInputProxy8;
	TopLevelWindow*       pAppWindow;
	IDirectInputDevice8*  iDIDevice;

	bool   bDIDeviceConnectedToWindow;
	bool   bDIDeviceActive;
	bool   bUsingExclusiveMode;
	bool   bUseGetAsyncKeyState;

	ubyte  win32KeyboardState[256];
	ubyte  diKeyboardState[256];

	bool   bNumLock;
	bool   bCapsLock;
	bool   bScrollLock;
};


