

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\DirectInputProxy8.h"
#include "HIDControllers\Mouse.h"
#include "BaseTools\TopLevelWindow.h"
#include "BaseTools\WinUtils.h"

using namespace vertexware;


//====================================================================
class SystemMouse8 : public HumanInterfaceDevice,
					public Mouse,
					public MouseMessageHook
{
public:
	SystemMouse8(DirectInputProxy8* dip8, const DirectInputDeviceInstanceInfo& didii);
	virtual ~SystemMouse8();


	//-------------------------------------
	// HumanInterfaceDevice implementation.
	//-------------------------------------
	virtual  uint            GetNumControllers() { return 1; }
	virtual  HIDController&  GetController(uint i) { return *this; }

	virtual  void  ConnectToAppWindow(TopLevelWindow* pAppWindow,
									HWND hwndAppWindow, bool bFullscreen);

	virtual  void  DisconnectFromAppWindow();

	virtual  void  OnAppWindowActive(bool bActive);
	virtual  void  OnAppWindowGeometryChanged();

	virtual  bool  SampleInputData();


	//----------------------------
	// MouseMessageHook overrides.
	//----------------------------
	virtual  bool  OnMouseMove(const win_utils::Point& mousePosClient,
								WPARAM keyFlags);

	virtual  bool  OnMouseWheel(int delta,
								const win_utils::Point& mousePosClient,
								WPARAM keyFlags);

	virtual  bool  OnMouseButtonLeft(bool bDown,
								const win_utils::Point& mousePosClient,
								WPARAM keyFlags);

	virtual  bool  OnMouseButtonMiddle(bool bDown,
								const win_utils::Point& mousePosClient,
								WPARAM keyFlags);

	virtual  bool  OnMouseButtonRight(bool bDown,
								const win_utils::Point& mousePosClient,
								WPARAM keyFlags);


protected:
	//-------------------------------------
	// HumanInterfaceDevice implementation.
	//-------------------------------------
	virtual  void  OnUseDirectInput(bool bUseDI);

	//----------------------
	// Mouse implementation.
	//----------------------
	virtual  void  OnInputModeChanged();
	virtual  void  OnSyncWithSystemCursorChanged();


private:
	void  SetupCursorStates();
	void  ClipPosition(win_utils::Point& pos);
	bool  IsPositionInsideClientArea(const win_utils::Point& pos);
	void  CreateDIDevice();
	void  ReleaseDIDevice();
	void  CacheDIDeviceProperties();
	void  ActivateDIDevice(bool bActivate);
	void  ResetMouseMessageDispatch();
	void  SetupMouseMessageDispatch(bool bUsingDI);
	void  ExtractHardwareProperties();


private:
	DirectInputProxy8*    pDirectInputProxy8;
	TopLevelWindow*       pAppWindow;
	IDirectInputDevice8*  iDIDevice;
	bool                  bDIDeviceConnectedToWindow;
	bool                  bDIDeviceActive;
	int                   DIWheelGranularity;
	int                   wheelAbsMax;
	RECT                  appClientRect;
	RECT                  appClientRectGuardBand;
	win_utils::Point      appClientSize;
	win_utils::Point      appClientCenter;
	win_utils::Point      lastCursorPos;
	bool                  bUsingExclusiveMode;
	bool                  bAppWindowFullscreen;
	bool                  bAppWindowActiveCycle;
	bool                  bRelativeModeBeforeAppWindowActiveCycle;
};

