

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "Framework\VideoSubsystem.h"
#include "Framework\VideoRenderer.h"

using namespace vertexware;


//====================================================================
class VideoManager : public VideoSubsystemWindowObserver
{
public:
	VideoManager();
	~VideoManager();

	void  SetReconfigParams(const std::string& vrPluginName,
							const VideoConfig& vc, uint reconfigCode);

	void  InitializeWindow();
	void  CleanupWindow();

	void  InitializeResources(bool bPostReset);
	void  CleanupResources(bool bPreReset);

	void  ErrorCheck();
	bool  IsReadyToRender();
	void  Render(const KernelExecutionParams& kep);
	void  Fix();

	void  SetMainWindowGeometry(const MainWindowGeometry& mwg);
	void  GetMainWindowGeometry(MainWindowGeometry& mwg);

	VideoSubsystemWindow*  GetVideoWindow() { return pVideoWindow; }
	VideoRenderer*         GetVideoRenderer() { return pVideoRenderer; }


	//---------------------------------------------
	// VideoSubsystemWindowObserver implementation.
	//---------------------------------------------
	virtual  void  OnDisplaySettingsChanged();
	virtual  void  OnVideoWindowActivated(bool bActive);
	virtual  void  OnVideoWindowMoved();
	virtual  void  OnVideoWindowResized(TopLevelWindow::eResizeAction action);
	virtual  void  OnVideoWindowPaint();
	virtual  void  OnSystemMenuLoopActive(bool bLoopActive);
	virtual  void  OnSystemResizeMoveLoopActive(bool bLoopActive);


private:
	bool  NeedsReset();
	bool  ResetVideoSubsystem();
	void  ReportInitVideoRendererStates(bool bPostReset);
	void  ReportWindowResize();
	void  ShowSystemCursor(bool bShow);


private:
	bool  bSystemCursorVisible;
	bool  bWindowInitialized;
	bool  bResourcesInitialized;
	uint  iWindowCreateCount;

	bool  bResizeMoveSystemLoopActive;
	uint  lastWindowWidth;
	uint  lastWindowHeight;
	uint  iResetCount;

	VideoRenderer*         pVideoRenderer;
	VideoSubsystem*        pVideoSubsystem;
	VideoSubsystemWindow*  pVideoWindow;

	bool         bLastKnownGoodConfigValid;
	std::string  lastKnownGoodPluginName;
	VideoConfig  lastKnownGoodConfig;

	uint         iReconfigCount;
	uint         reconfigCode;
	std::string  vrPluginNameReconfig;
	VideoConfig  videoConfigReconfig;
};

