

//--------------------------------------------------------------------
#include "GLTools\ErrorHandling.h"
#include "GLTools\Extensions.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gl_tools {


//====================================================================
std::string __fastcall GLErrorString(GLenum errorCode)
{
	switch(errorCode) {

	case GL_NO_ERROR: return "no error";
	case GL_INVALID_ENUM: return "invalid enumerant";
	case GL_INVALID_VALUE: return "invalid value";
	case GL_INVALID_OPERATION: return "invalid operation";
	case GL_STACK_OVERFLOW: return "stack overflow";
	case GL_STACK_UNDERFLOW: return "stack underflow";
	case GL_OUT_OF_MEMORY: return "out of memory";
	case GL_TABLE_TOO_LARGE: return "table too large";
	}
	return "unknown error code";
}


//====================================================================
std::string __fastcall FormatGLError(GLenum errorCode,
									const std::string& testPointName)
{
	std::string s("OpenGL failed with code: ");

	s.append(GLErrorString(errorCode));

	if(testPointName.length()) {
		s.append(" @ ").append(testPointName);
	}
	return s;
}


//====================================================================
bool __fastcall GLErrorCheck(const std::string& testPointName,
							GLenum* pOutErrCode,
							std::string* pOutErrDesc)
{
	GLenum errorCode = glGetError();
	if(errorCode == GL_NO_ERROR) return false;
	if(pOutErrCode) *pOutErrCode = errorCode;
	if(pOutErrDesc) *pOutErrDesc = FormatGLError(errorCode, testPointName);
	return true;
}


//====================================================================
void __fastcall GLErrorCheckE(const std::string& testPointName)
{
	GLenum errorCode = glGetError();
	if(errorCode != GL_NO_ERROR) {
		VW_THROW(OpenGLException, FormatGLError(errorCode, testPointName));
	}
}


//********************************************************************
}//namespace gl_tools


//********************************************************************
}//namespace vertexware


