

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "BaseTools\Exception.h"
#include "Framework\FrameworkExceptions.h"
#include "private\vw_opengl_exports.h"
#include <gl\gl.h>


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gl_tools {


//====================================================================
VW_DERIVE_EXCEPTION( OpenGLException, VideoRendererException )


//====================================================================
// Replacement for gluErrorString(). Handles only GL error codes.
// Supports the following codes:
//    GL_NO_ERROR
//    GL_INVALID_ENUM
//    GL_INVALID_VALUE
//    GL_INVALID_OPERATION
//    GL_STACK_OVERFLOW
//    GL_STACK_UNDERFLOW
//    GL_OUT_OF_MEMORY
//    GL_TABLE_TOO_LARGE
//====================================================================
VW_OPENGL_API
std::string __fastcall GLErrorString(GLenum errorCode);


//====================================================================
// Returns GL error description.
//====================================================================
VW_OPENGL_API
std::string __fastcall FormatGLError(GLenum errorCode,
									const std::string& testPointName = "");


//====================================================================
// Returns true if GL error occured.
//====================================================================
VW_OPENGL_API
bool __fastcall GLErrorCheck(const std::string& testPointName = "",
							GLenum* pOutErrCode = 0,
							std::string* pOutErrDesc = 0);


//====================================================================
// Throws OpenGLException if GL error occured.
//====================================================================
VW_OPENGL_API
void __fastcall GLErrorCheckE(const std::string& testPointName = "");


//====================================================================
// Throws TException exception if <hr> indicates D3D8 error.
//====================================================================
template<typename TException>
inline void __fastcall GLErrorCheckET(const std::string& testPointName)
{
	GLenum errorCode = glGetError();
	if(errorCode != GL_NO_ERROR) {
		VW_THROW(TException, FormatGLError(errorCode, testPointName));
	}
}


//********************************************************************
}//namespace gl_tools


//********************************************************************
}//namespace vertexware


