

//--------------------------------------------------------------------
#include "GLTools\Extensions.h"
#include "BaseTools\String.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gl_tools {


//====================================================================
void __fastcall ExtractVersionNumbers(uint& outMajor, uint& outMinor,
												uint& outRelease)
{
	outMajor = outMinor = outRelease = 0;

	// allowed forms: a) major.minor b) major.minor.release
	std::string s(reinterpret_cast<const char*>(glGetString(GL_VERSION)));
	const char dot = '.';

	// extract major
	uint pos = s.find(dot);
	if(pos == std::string::npos) return;
	ToValue<uint>(s.substr(0, pos), outMajor, 0);
	s.erase(0, pos + 1);

	// extract minor
	pos = s.find(dot);
	if(pos == std::string::npos) {
		ToValue<uint>(s, outMinor, 0);
		return;
	}
	ToValue<uint>(s.substr(0, pos), outMinor, 0);
	s.erase(0, pos + 1);

	// extract release
	ToValue<uint>(s, outRelease, 0);
}


//====================================================================
void __fastcall GetExtensionsString(HDC hdc, std::string& outExtensionsString)
{
	// GL extensions
	outExtensionsString.assign(reinterpret_cast<const char*>(glGetString(GL_EXTENSIONS)));

	// WGL extensions
	typedef  const char* (APIENTRY *PFNWGLGETEXTENSIONSTRINGARB)(HDC);

	PFNWGLGETEXTENSIONSTRINGARB  wglGetExtensionsStringARB =
		(PFNWGLGETEXTENSIONSTRINGARB)(::wglGetProcAddress("wglGetExtensionsStringARB"));

	if(wglGetExtensionsStringARB) {
		outExtensionsString.append(1, ' ');
		outExtensionsString.append(wglGetExtensionsStringARB(hdc));
	}
}


//====================================================================
bool __fastcall IsExtensionSupported(const std::string& extensionsString,
										const std::string& extensionName)
{
	if(extensionName.empty()) return false;
	const std::string  wsChars("\t\n ");

	std::string::size_type seekPos = 0;
	while(seekPos < extensionsString.length()) {

		std::string::size_type startPos = extensionsString.find(extensionName, seekPos);
		if(startPos == std::string::npos) return false;

		uint endPos = startPos + extensionName.length();
		if(endPos >= extensionsString.length()) return true;

		switch(extensionsString[endPos]) {
			case '\t':
			case '\n':
			case ' ':
				return true;
		}

		seekPos = extensionsString.find_first_of(wsChars, seekPos);
		if(seekPos == std::string::npos) return false;
		seekPos = extensionsString.find_first_not_of(wsChars, seekPos);
		if(seekPos == std::string::npos) return false;
	}
	return false;
}


//====================================================================
void* __fastcall GetExtensionProc(const char* extensionProcName)
{
	if(! extensionProcName) return 0;
	return ::wglGetProcAddress(extensionProcName);
}


//********************************************************************
}//namespace gl_tools


//********************************************************************
namespace gl_ext {


//====================================================================
std::string __fastcall  GetExtensionName(eExtension e)
{
	switch(e) {

	case Extension_GL_CORE_12: return "GL_CORE_12";
	case Extension_GL_CORE_13: return "GL_CORE_13";

	case Extension_GL_ARB_imaging: return "GL_ARB_imaging";
	case Extension_GL_ARB_matrix_palette: return "GL_ARB_matrix_palette";
	case Extension_GL_ARB_multisample: return "GL_ARB_multisample";
	case Extension_GL_ARB_multitexture: return "GL_ARB_multitexture";
	case Extension_GL_ARB_point_parameters: return "GL_ARB_point_parameters";
	case Extension_GL_ARB_texture_compression: return "GL_ARB_texture_compression";
	case Extension_GL_ARB_texture_env_add: return "GL_ARB_texture_env_add";
	case Extension_GL_ARB_texture_env_dot3: return "GL_ARB_texture_env_dot3";
	case Extension_GL_ARB_texture_env_combine: return "GL_ARB_texture_env_combine";
	case Extension_GL_ARB_texture_env_crossbar: return "GL_ARB_texture_env_crossbar";
	case Extension_GL_ARB_texture_border_clamp: return "GL_ARB_texture_border_clamp";
	case Extension_GL_ARB_texture_cube_map: return "GL_ARB_texture_cube_map";
	case Extension_GL_ARB_transpose_matrix: return "GL_ARB_transpose_matrix";
	case Extension_GL_ARB_vertex_blend: return "GL_ARB_vertex_blend";

	case Extension_GL_EXT_abgr: return "GL_EXT_abgr";
	case Extension_GL_EXT_bgra: return "GL_EXT_bgra";
	case Extension_GL_EXT_compiled_vertex_array: return "GL_EXT_compiled_vertex_array";
	case Extension_GL_EXT_fog_coord: return "GL_EXT_fog_coord";
	case Extension_GL_EXT_multi_draw_arrays: return "GL_EXT_multi_draw_arrays";
	case Extension_GL_EXT_point_parameters: return "GL_EXT_point_parameters";
	case Extension_GL_EXT_secondary_color: return "GL_EXT_secondary_color";
	case Extension_GL_EXT_stencil_wrap: return "GL_EXT_stencil_wrap";
	case Extension_GL_EXT_texture_filter_anisotropic: return "GL_EXT_texture_filter_anisotropic";
	case Extension_GL_EXT_texture_lod_bias: return "GL_EXT_texture_lod_bias";
	case Extension_GL_EXT_vertex_weighting: return "GL_EXT_vertex_weighting";

	case Extension_GL_ATI_pn_triangles: return "GL_ATI_pn_triangles";

	case Extension_GL_NV_blend_square: return "GL_NV_blend_square";
	case Extension_GL_NV_copy_depth_to_color: return "GL_NV_copy_depth_to_color";
	case Extension_GL_NV_evaluators: return "GL_NV_evaluators";
	case Extension_GL_NV_fence: return "GL_NV_fence";
	case Extension_GL_NV_fog_distance: return "GL_NV_fog_distance";
	case Extension_GL_NV_light_max_exponent: return "GL_NV_light_max_exponent";
	case Extension_GL_NV_packed_depth_stencil: return "GL_NV_packed_depth_stencil";
	case Extension_GL_NV_register_combiners: return "GL_NV_register_combiners";
	case Extension_GL_NV_register_combiners2: return "GL_NV_register_combiners2";
	case Extension_GL_NV_texgen_reflection: return "GL_NV_texgen_reflection";
	case Extension_GL_NV_texture_env_combine4: return "GL_NV_texture_env_combine4";
	case Extension_GL_NV_texture_rectangle: return "GL_NV_texture_rectangle";
	case Extension_GL_NV_texture_shader: return "GL_NV_texture_shader";
	case Extension_GL_NV_texture_shader2: return "GL_NV_texture_shader2";
	case Extension_GL_NV_vertex_array_range: return "GL_NV_vertex_array_range";
	case Extension_GL_NV_vertex_array_range2: return "GL_NV_vertex_array_range2";
	case Extension_GL_NV_vertex_program: return "GL_NV_vertex_program";

	case Extension_GL_SGIS_generate_mipmap: return "GL_SGIS_generate_mipmap";
	case Extension_GL_SGIX_shadow: return "GL_SGIX_shadow";
	case Extension_GL_SGIX_depth_texture: return "GL_SGIX_depth_texture";
	}
	return "Extension_Invalid";
}


//====================================================================
eExtension __fastcall  GetExtensionFromName(const std::string& name)
{
	for(uint i = 0; i < Extension_Count; i++) {
		if(GetExtensionName(eExtension(i)) == name) return eExtension(i);
	}
	return Extension_Invalid;
}


//====================================================================
// GL implementation info
//====================================================================
bool  g_GLReady = false;
uint  g_VersionGLMajor = 0;
uint  g_VersionGLMinor = 0;
uint  g_VersionGLRelease = 0;
bool  g_SupportedExtension[Extension_Count];


//====================================================================
bool __fastcall  IsGLReady() { return g_GLReady; }
uint __fastcall  GetVersionGLMajor() { return g_VersionGLMajor; }
uint __fastcall  GetVersionGLMinor() { return g_VersionGLMinor; }
uint __fastcall  GetVersionGLRelease() { return g_VersionGLRelease; }
bool __fastcall  IsSupported(eExtension e) { return g_SupportedExtension[e]; }


//====================================================================
void __fastcall  SetGLReady(bool bReady) { g_GLReady = bReady; }
void __fastcall  SetVersionGLMajor(uint v) { g_VersionGLMajor = v; }
void __fastcall  SetVersionGLMinor(uint v) { g_VersionGLMinor = v; }
void __fastcall  SetVersionGLRelease(uint v) { g_VersionGLRelease = v; }
void __fastcall  SetSupported(eExtension e, bool s) { g_SupportedExtension[e] = s; }


//====================================================================
void __fastcall  ClearImplementationInfo()
{
	g_VersionGLMajor = 0;
	g_VersionGLMinor = 0;
	g_VersionGLRelease = 0;

	for(uint i = 0; i < Extension_Count; i++) {
		g_SupportedExtension[i] = false;
	}
}


//====================================================================
struct ImpInfoInit
{
	ImpInfoInit() { ClearImplementationInfo(); }
}
g_impInfoInit;


//====================================================================
// GL_CORE_12
//====================================================================
PFNGLDRAWRANGEELEMENTS glDrawRangeElements = 0;
PFNGLTEXIMAGE3D glTexImage3D = 0;
PFNGLTEXSUBIMAGE3D glTexSubImage3D = 0;
PFNGLCOPYTEXSUBIMAGE3D glCopyTexSubImage3D = 0;


//====================================================================
// GL_CORE_13
//====================================================================
PFNGLACTIVETEXTURE glActiveTexture = 0;
PFNGLCLIENTACTIVETEXTURE glClientActiveTexture = 0;
PFNGLMULTITEXCOORD1D glMultiTexCoord1d = 0;
PFNGLMULTITEXCOORD1DV glMultiTexCoord1dv = 0;
PFNGLMULTITEXCOORD1F glMultiTexCoord1f = 0;
PFNGLMULTITEXCOORD1FV glMultiTexCoord1fv = 0;
PFNGLMULTITEXCOORD1I glMultiTexCoord1i = 0;
PFNGLMULTITEXCOORD1IV glMultiTexCoord1iv = 0;
PFNGLMULTITEXCOORD1S glMultiTexCoord1s = 0;
PFNGLMULTITEXCOORD1SV glMultiTexCoord1sv = 0;
PFNGLMULTITEXCOORD2D glMultiTexCoord2d = 0;
PFNGLMULTITEXCOORD2DV glMultiTexCoord2dv = 0;
PFNGLMULTITEXCOORD2F glMultiTexCoord2f = 0;
PFNGLMULTITEXCOORD2FV glMultiTexCoord2fv = 0;
PFNGLMULTITEXCOORD2I glMultiTexCoord2i = 0;
PFNGLMULTITEXCOORD2IV glMultiTexCoord2iv = 0;
PFNGLMULTITEXCOORD2S glMultiTexCoord2s = 0;
PFNGLMULTITEXCOORD2SV glMultiTexCoord2sv = 0;
PFNGLMULTITEXCOORD3D glMultiTexCoord3d = 0;
PFNGLMULTITEXCOORD3DV glMultiTexCoord3dv = 0;
PFNGLMULTITEXCOORD3F glMultiTexCoord3f = 0;
PFNGLMULTITEXCOORD3FV glMultiTexCoord3fv = 0;
PFNGLMULTITEXCOORD3I glMultiTexCoord3i = 0;
PFNGLMULTITEXCOORD3IV glMultiTexCoord3iv = 0;
PFNGLMULTITEXCOORD3S glMultiTexCoord3s = 0;
PFNGLMULTITEXCOORD3SV glMultiTexCoord3sv = 0;
PFNGLMULTITEXCOORD4D glMultiTexCoord4d = 0;
PFNGLMULTITEXCOORD4DV glMultiTexCoord4dv = 0;
PFNGLMULTITEXCOORD4F glMultiTexCoord4f = 0;
PFNGLMULTITEXCOORD4FV glMultiTexCoord4fv = 0;
PFNGLMULTITEXCOORD4I glMultiTexCoord4i = 0;
PFNGLMULTITEXCOORD4IV glMultiTexCoord4iv = 0;
PFNGLMULTITEXCOORD4S glMultiTexCoord4s = 0;
PFNGLMULTITEXCOORD4SV glMultiTexCoord4sv = 0;
PFNGLLOADTRANSPOSEMATRIXF glLoadTransposeMatrixf = 0;
PFNGLLOADTRANSPOSEMATRIXD glLoadTransposeMatrixd = 0;
PFNGLMULTTRANSPOSEMATRIXF glMultTransposeMatrixf = 0;
PFNGLMULTTRANSPOSEMATRIXD glMultTransposeMatrixd = 0;
PFNGLCOMPRESSEDTEXIMAGE1D glCompressedTexImage1D = 0;
PFNGLCOMPRESSEDTEXIMAGE2D glCompressedTexImage2D = 0;
PFNGLCOMPRESSEDTEXIMAGE3D glCompressedTexImage3D = 0;
PFNGLCOMPRESSEDTEXSUBIMAGE1D glCompressedTexSubImage1D = 0;
PFNGLCOMPRESSEDTEXSUBIMAGE2D glCompressedTexSubImage2D = 0;
PFNGLCOMPRESSEDTEXSUBIMAGE3D glCompressedTexSubImage3D = 0;
PFNGLGETCOMPRESSEDTEXIMAGE glGetCompressedTexImage = 0;
PFNGLSAMPLECOVERAGE glSampleCoverage = 0;


//====================================================================
// GL_ARB_imaging
//====================================================================
PFNGLBLENDCOLOR glBlendColor = 0;
PFNGLBLENDEQUATION glBlendEquation = 0;
PFNGLCOLORTABLE glColorTable = 0;
PFNGLCOLORTABLEPARAMETERFV glColorTableParameterfv = 0;
PFNGLCOLORTABLEPARAMETERIV glColorTableParameteriv = 0;
PFNGLCOPYCOLORTABLE glCopyColorTable = 0;
PFNGLGETCOLORTABLE glGetColorTable = 0;
PFNGLGETCOLORTABLEPARAMETERFV glGetColorTableParameterfv = 0;
PFNGLGETCOLORTABLEPARAMETERIV glGetColorTableParameteriv = 0;
PFNGLCOLORSUBTABLE glColorSubTable = 0;
PFNGLCOPYCOLORSUBTABLE glCopyColorSubTable = 0;
PFNGLCONVOLUTIONFILTER1D glConvolutionFilter1D = 0;
PFNGLCONVOLUTIONFILTER2D glConvolutionFilter2D = 0;
PFNGLCONVOLUTIONPARAMETERF glConvolutionParameterf = 0;
PFNGLCONVOLUTIONPARAMETERFV glConvolutionParameterfv = 0;
PFNGLCONVOLUTIONPARAMETERI glConvolutionParameteri = 0;
PFNGLCONVOLUTIONPARAMETERIV glConvolutionParameteriv = 0;
PFNGLCOPYCONVOLUTIONFILTER1D glCopyConvolutionFilter1D = 0;
PFNGLCOPYCONVOLUTIONFILTER2D glCopyConvolutionFilter2D = 0;
PFNGLGETCONVOLUTIONFILTER glGetConvolutionFilter = 0;
PFNGLGETCONVOLUTIONPARAMETERFV glGetConvolutionParameterfv = 0;
PFNGLGETCONVOLUTIONPARAMETERIV glGetConvolutionParameteriv = 0;
PFNGLGETSEPARABLEFILTER glGetSeparableFilter = 0;
PFNGLSEPARABLEFILTER2D glSeparableFilter2D = 0;
PFNGLGETHISTOGRAM glGetHistogram = 0;
PFNGLGETHISTOGRAMPARAMETERFV glGetHistogramParameterfv = 0;
PFNGLGETHISTOGRAMPARAMETERIV glGetHistogramParameteriv = 0;
PFNGLGETMINMAX glGetMinmax = 0;
PFNGLGETMINMAXPARAMETERFV glGetMinmaxParameterfv = 0;
PFNGLGETMINMAXPARAMETERIV glGetMinmaxParameteriv = 0;
PFNGLHISTOGRAM glHistogram = 0;
PFNGLMINMAX glMinmax = 0;
PFNGLRESETHISTOGRAM glResetHistogram = 0;
PFNGLRESETMINMAX glResetMinmax = 0;


//====================================================================
// GL_ARB_matrix_palette
//====================================================================
PFNGLCURRENTPALETTEMATRIXARB glCurrentPaletteMatrixARB = 0;
PFNGLMATRIXINDEXUBVARB glMatrixIndexubvARB = 0;
PFNGLMATRIXINDEXUSVARB glMatrixIndexusvARB = 0;
PFNGLMATRIXINDEXUIVARB glMatrixIndexuivARB = 0;
PFNGLMATRIXINDEXPOINTERARB glMatrixIndexPointerARB = 0;


//====================================================================
// GL_ARB_multisample
//====================================================================
PFNGLSAMPLECOVERAGEARB glSampleCoverageARB = 0;


//====================================================================
// GL_ARB_multitexture
//====================================================================
PFNGLACTIVETEXTUREARB glActiveTextureARB = 0;
PFNGLCLIENTACTIVETEXTUREARB glClientActiveTextureARB = 0;
PFNGLMULTITEXCOORD1DARB glMultiTexCoord1dARB = 0;
PFNGLMULTITEXCOORD1DVARB glMultiTexCoord1dvARB = 0;
PFNGLMULTITEXCOORD1FARB glMultiTexCoord1fARB = 0;
PFNGLMULTITEXCOORD1FVARB glMultiTexCoord1fvARB = 0;
PFNGLMULTITEXCOORD1IARB glMultiTexCoord1iARB = 0;
PFNGLMULTITEXCOORD1IVARB glMultiTexCoord1ivARB = 0;
PFNGLMULTITEXCOORD1SARB glMultiTexCoord1sARB = 0;
PFNGLMULTITEXCOORD1SVARB glMultiTexCoord1svARB = 0;
PFNGLMULTITEXCOORD2DARB glMultiTexCoord2dARB = 0;
PFNGLMULTITEXCOORD2DVARB glMultiTexCoord2dvARB = 0;
PFNGLMULTITEXCOORD2FARB glMultiTexCoord2fARB = 0;
PFNGLMULTITEXCOORD2FVARB glMultiTexCoord2fvARB = 0;
PFNGLMULTITEXCOORD2IARB glMultiTexCoord2iARB = 0;
PFNGLMULTITEXCOORD2IVARB glMultiTexCoord2ivARB = 0;
PFNGLMULTITEXCOORD2SARB glMultiTexCoord2sARB = 0;
PFNGLMULTITEXCOORD2SVARB glMultiTexCoord2svARB = 0;
PFNGLMULTITEXCOORD3DARB glMultiTexCoord3dARB = 0;
PFNGLMULTITEXCOORD3DVARB glMultiTexCoord3dvARB = 0;
PFNGLMULTITEXCOORD3FARB glMultiTexCoord3fARB = 0;
PFNGLMULTITEXCOORD3FVARB glMultiTexCoord3fvARB = 0;
PFNGLMULTITEXCOORD3IARB glMultiTexCoord3iARB = 0;
PFNGLMULTITEXCOORD3IVARB glMultiTexCoord3ivARB = 0;
PFNGLMULTITEXCOORD3SARB glMultiTexCoord3sARB = 0;
PFNGLMULTITEXCOORD3SVARB glMultiTexCoord3svARB = 0;
PFNGLMULTITEXCOORD4DARB glMultiTexCoord4dARB = 0;
PFNGLMULTITEXCOORD4DVARB glMultiTexCoord4dvARB = 0;
PFNGLMULTITEXCOORD4FARB glMultiTexCoord4fARB = 0;
PFNGLMULTITEXCOORD4FVARB glMultiTexCoord4fvARB = 0;
PFNGLMULTITEXCOORD4IARB glMultiTexCoord4iARB = 0;
PFNGLMULTITEXCOORD4IVARB glMultiTexCoord4ivARB = 0;
PFNGLMULTITEXCOORD4SARB glMultiTexCoord4sARB = 0;
PFNGLMULTITEXCOORD4SVARB glMultiTexCoord4svARB = 0;


//====================================================================
// GL_ARB_point_parameters
//====================================================================
PFNGLPOINTPARAMETERFARB glPointParameterfARB = 0;
PFNGLPOINTPARAMETERFVARB glPointParameterfvARB = 0;


//====================================================================
// GL_ARB_texture_compression
//====================================================================
PFNGLCOMPRESSEDTEXIMAGE1DARB glCompressedTexImage1DARB = 0;
PFNGLCOMPRESSEDTEXIMAGE2DARB glCompressedTexImage2DARB = 0;
PFNGLCOMPRESSEDTEXIMAGE3DARB glCompressedTexImage3DARB = 0;
PFNGLCOMPRESSEDTEXSUBIMAGE1DARB glCompressedTexSubImage1DARB = 0;
PFNGLCOMPRESSEDTEXSUBIMAGE2DARB glCompressedTexSubImage2DARB = 0;
PFNGLCOMPRESSEDTEXSUBIMAGE3DARB glCompressedTexSubImage3DARB = 0;
PFNGLGETCOMPRESSEDTEXIMAGEARB glGetCompressedTexImageARB = 0;


//====================================================================
// GL_ARB_transpose_matrix
//====================================================================
PFNGLLOADTRANSPOSEMATRIXFARB glLoadTransposeMatrixfARB = 0;
PFNGLLOADTRANSPOSEMATRIXDARB glLoadTransposeMatrixdARB = 0;
PFNGLMULTTRANSPOSEMATRIXFARB glMultTransposeMatrixfARB = 0;
PFNGLMULTTRANSPOSEMATRIXDARB glMultTransposeMatrixdARB = 0;


//====================================================================
// GL_ARB_vertex_blend
//====================================================================
PFNGLWEIGHTBVARB glWeightbvARB = 0;
PFNGLWEIGHTSVARB glWeightsvARB = 0;
PFNGLWEIGHTIVARB glWeightivARB = 0;
PFNGLWEIGHTFVARB glWeightfvARB = 0;
PFNGLWEIGHTDVARB glWeightdvARB = 0;
PFNGLWEIGHTUBVARB glWeightubvARB = 0;
PFNGLWEIGHTUSVARB glWeightusvARB = 0;
PFNGLWEIGHTUIVARB glWeightuivARB = 0;
PFNGLWEIGHTPOINTERARB glWeightPointerARB = 0;
PFNGLVERTEXBLENDARB glVertexBlendARB = 0;


//====================================================================
// GL_EXT_compiled_vertex_array
//====================================================================
PFNGLLOCKARRAYSEXT glLockArraysEXT = 0;
PFNGLUNLOCKARRAYSEXT glUnlockArraysEXT = 0;


//====================================================================
// GL_EXT_fog_coord
//====================================================================
PFNGLFOGCOORDFEXT glFogCoordfEXT = 0;
PFNGLFOGCOORDFVEXT glFogCoordfvEXT = 0;
PFNGLFOGCOORDDEXT glFogCoorddEXT = 0;
PFNGLFOGCOORDDVEXT glFogCoorddvEXT = 0;
PFNGLFOGCOORDPOINTEREXT glFogCoordPointerEXT = 0;


//====================================================================
// GL_EXT_multi_draw_arrays
//====================================================================
PFNGLMULTIDRAWARRAYSEXT glMultiDrawArraysEXT = 0;
PFNGLMULTIDRAWELEMENTSEXT glMultiDrawElementsEXT = 0;


//====================================================================
// GL_EXT_point_parameters
//====================================================================
PFNGLPOINTPARAMETERFEXT glPointParameterfEXT = 0;
PFNGLPOINTPARAMETERFVEXT glPointParameterfvEXT = 0;


//====================================================================
// GL_EXT_secondary_color
//====================================================================
PFNGLSECONDARYCOLOR3BEXT glSecondaryColor3bEXT = 0;
PFNGLSECONDARYCOLOR3BVEXT glSecondaryColor3bvEXT = 0;
PFNGLSECONDARYCOLOR3DEXT glSecondaryColor3dEXT = 0;
PFNGLSECONDARYCOLOR3DVEXT glSecondaryColor3dvEXT = 0;
PFNGLSECONDARYCOLOR3FEXT glSecondaryColor3fEXT = 0;
PFNGLSECONDARYCOLOR3FVEXT glSecondaryColor3fvEXT = 0;
PFNGLSECONDARYCOLOR3IEXT glSecondaryColor3iEXT = 0;
PFNGLSECONDARYCOLOR3IVEXT glSecondaryColor3ivEXT = 0;
PFNGLSECONDARYCOLOR3SEXT glSecondaryColor3sEXT = 0;
PFNGLSECONDARYCOLOR3SVEXT glSecondaryColor3svEXT = 0;
PFNGLSECONDARYCOLOR3UBEXT glSecondaryColor3ubEXT = 0;
PFNGLSECONDARYCOLOR3UBVEXT glSecondaryColor3ubvEXT = 0;
PFNGLSECONDARYCOLOR3UIEXT glSecondaryColor3uiEXT = 0;
PFNGLSECONDARYCOLOR3UIVEXT glSecondaryColor3uivEXT = 0;
PFNGLSECONDARYCOLOR3USEXT glSecondaryColor3usEXT = 0;
PFNGLSECONDARYCOLOR3USVEXT glSecondaryColor3usvEXT = 0;
PFNGLSECONDARYCOLORPOINTEREXT glSecondaryColorPointerEXT = 0;


//====================================================================
// GL_EXT_vertex_weighting
//====================================================================
PFNGLVERTEXWEIGHTFEXT glVertexWeightfEXT = 0;
PFNGLVERTEXWEIGHTFVEXT glVertexWeightfvEXT = 0;
PFNGLVERTEXWEIGHTPOINTEREXT glVertexWeightPointerEXT = 0;


//====================================================================
// GL_ATI_pn_triangles
//====================================================================
PFNGLPNTRIANGLESIATI glPNTrianglesiATI = 0;
PFNGLPNTRIANGLESFATI glPNTrianglesfATI = 0;


//====================================================================
// GL_NV_evaluators
//====================================================================
PFNGLMAPCONTROLPOINTSNV glMapControlPointsNV = 0;
PFNGLMAPPARAMETERIVNV glMapParameterivNV = 0;
PFNGLMAPPARAMETERFVNV glMapParameterfvNV = 0;
PFNGLGETMAPCONTROLPOINTSNV glGetMapControlPointsNV = 0;
PFNGLGETMAPPARAMETERIVNV glGetMapParameterivNV = 0;
PFNGLGETMAPPARAMETERFVNV glGetMapParameterfvNV = 0;
PFNGLGETMAPATTRIBPARAMETERIVNV glGetMapAttribParameterivNV = 0;
PFNGLGETMAPATTRIBPARAMETERFVNV glGetMapAttribParameterfvNV = 0;
PFNGLEVALMAPSNV glEvalMapsNV = 0;


//====================================================================
// GL_NV_fence
//====================================================================
PFNGLGENFENCESNV glGenFencesNV = 0;
PFNGLDELETEFENCESNV glDeleteFencesNV = 0;
PFNGLSETFENCENV glSetFenceNV = 0;
PFNGLTESTFENCENV glTestFenceNV = 0;
PFNGLFINISHFENCENV glFinishFenceNV = 0;
PFNGLISFENCENV glIsFenceNV = 0;
PFNGLGETFENCEIVNV glGetFenceivNV = 0;


//====================================================================
// GL_NV_register_combiners
//====================================================================
PFNGLCOMBINERPARAMETERFVNV glCombinerParameterfvNV = 0;
PFNGLCOMBINERPARAMETERFNV glCombinerParameterfNV = 0;
PFNGLCOMBINERPARAMETERIVNV glCombinerParameterivNV = 0;
PFNGLCOMBINERPARAMETERINV glCombinerParameteriNV = 0;
PFNGLCOMBINERINPUTNV glCombinerInputNV = 0;
PFNGLCOMBINEROUTPUTNV glCombinerOutputNV = 0;
PFNGLFINALCOMBINERINPUTNV glFinalCombinerInputNV = 0;
PFNGLGETCOMBINERINPUTPARAMETERFVNV glGetCombinerInputParameterfvNV = 0;
PFNGLGETCOMBINERINPUTPARAMETERIVNV glGetCombinerInputParameterivNV = 0;
PFNGLGETCOMBINEROUTPUTPARAMETERFVNV glGetCombinerOutputParameterfvNV = 0;
PFNGLGETCOMBINEROUTPUTPARAMETERIVNV glGetCombinerOutputParameterivNV = 0;
PFNGLGETFINALCOMBINERINPUTPARAMETERFVNV glGetFinalCombinerInputParameterfvNV = 0;
PFNGLGETFINALCOMBINERINPUTPARAMETERIVNV glGetFinalCombinerInputParameterivNV = 0;


//====================================================================
// GL_NV_register_combiners2
//====================================================================
PFNGLCOMBINERSTAGEPARAMETERFVNV glCombinerStageParameterfvNV = 0;
PFNGLGETCOMBINERSTAGEPARAMETERFVNV glGetCombinerStageParameterfvNV = 0;


//====================================================================
// GL_NV_vertex_array_range
//====================================================================
PFNGLFLUSHVERTEXARRAYRANGENV glFlushVertexArrayRangeNV = 0;
PFNGLVERTEXARRAYRANGENV glVertexArrayRangeNV = 0;


//====================================================================
// GL_NV_vertex_program
//====================================================================
PFNGLBINDPROGRAMNV glBindProgramNV = 0;
PFNGLDELETEPROGRAMSNV glDeleteProgramsNV = 0;
PFNGLEXECUTEPROGRAMNV glExecuteProgramNV = 0;
PFNGLGENPROGRAMSNV glGenProgramsNV = 0;
PFNGLAREPROGRAMSRESIDENTNV glAreProgramsResidentNV = 0;
PFNGLREQUESTRESIDENTPROGRAMSNV glRequestResidentProgramsNV = 0;
PFNGLGETPROGRAMPARAMETERFVNV glGetProgramParameterfvNV = 0;
PFNGLGETPROGRAMPARAMETERDVNV glGetProgramParameterdvNV = 0;
PFNGLGETPROGRAMIVNV glGetProgramivNV = 0;
PFNGLGETPROGRAMSTRINGNV glGetProgramStringNV = 0;
PFNGLGETTRACKMATRIXIVNV glGetTrackMatrixivNV = 0;
PFNGLGETVERTEXATTRIBDVNV glGetVertexAttribdvNV = 0;
PFNGLGETVERTEXATTRINFVNV glGetVertexAttribfvNV = 0;
PFNGLGETVERTEXATTRIBIVNV glGetVertexAttribivNV = 0;
PFNGLGETVERTEXATTRIBPOINTERVNV glGetVertexAttribPointervNV = 0;
PFNGLISPROGRAMNV glIsProgramNV = 0;
PFNGLLOADPROGRAMNV glLoadProgramNV = 0;
PFNGLPROGRAMPARAMETER4FNV glProgramParameter4fNV = 0;
PFNGLPROGRAMPARAMETER4DNV glProgramParameter4dNV = 0;
PFNGLPROGRAMPARAMETER4DVNV glProgramParameter4dvNV = 0;
PFNGLPROGRAMPARAMETER4FVNV glProgramParameter4fvNV = 0;
PFNGLPROGRAMPARAMETERS4DVNV glProgramParameters4dvNV = 0;
PFNGLPROGRAMPARAMETERS4FVNV glProgramParameters4fvNV = 0;
PFNGLTRACKMATRIXNV glTrackMatrixNV = 0;
PFNGLVERTEXATTRIBPPOINTERNV glVertexAttribPointerNV = 0;
PFNGLVERTEXATTRIB1SNV glVertexAttrib1sNV = 0;
PFNGLVERTEXATTRIB1FNV glVertexAttrib1fNV = 0;
PFNGLVERTEXATTRIB1DNV glVertexAttrib1dNV = 0;
PFNGLVERTEXATTRIB2SNV glVertexAttrib2sNV = 0;
PFNGLVERTEXATTRIB2FNV glVertexAttrib2fNV = 0;
PFNGLVERTEXATTRIB2DNV glVertexAttrib2dNV = 0;
PFNGLVERTEXATTRIB3SNV glVertexAttrib3sNV = 0;
PFNGLVERTEXATTRIB3FNV glVertexAttrib3fNV = 0;
PFNGLVERTEXATTRIB3DNV glVertexAttrib3dNV = 0;
PFNGLVERTEXATTRIB4SNV glVertexAttrib4sNV = 0;
PFNGLVERTEXATTRIB4FNV glVertexAttrib4fNV = 0;
PFNGLVERTEXATTRIB4DNV glVertexAttrib4dNV = 0;
PFNGLVERTEXATTRIB4UBNV glVertexAttrib4ubNV = 0;
PFNGLVERTEXATTRIB1SVNV glVertexAttrib1svNV = 0;
PFNGLVERTEXATTRIB1FVNV glVertexAttrib1fvNV = 0;
PFNGLVERTEXATTRIB1DVNV glVertexAttrib1dvNV = 0;
PFNGLVERTEXATTRIB2SVNV glVertexAttrib2svNV = 0;
PFNGLVERTEXATTRIB2FVNV glVertexAttrib2fvNV = 0;
PFNGLVERTEXATTRIB2DVNV glVertexAttrib2dvNV = 0;
PFNGLVERTEXATTRIB3SVNV glVertexAttrib3svNV = 0;
PFNGLVERTEXATTRIB3FVNV glVertexAttrib3fvNV = 0;
PFNGLVERTEXATTRIB3DVNV glVertexAttrib3dvNV = 0;
PFNGLVERTEXATTRIB4SVNV glVertexAttrib4svNV = 0;
PFNGLVERTEXATTRIB4FVNV glVertexAttrib4fvNV = 0;
PFNGLVERTEXATTRIB4DVNV glVertexAttrib4dvNV = 0;
PFNGLVERTEXATTRIB4UBVNV glVertexAttrib4ubvNV = 0;
PFNGLVERTEXATTRIBS1SVNV glVertexAttribs1svNV = 0;
PFNGLVERTEXATTRIBS1FVNV glVertexAttribs1fvNV = 0;
PFNGLVERTEXATTRIBS1DVNV glVertexAttribs1dvNV = 0;
PFNGLVERTEXATTRIBS2SVNV glVertexAttribs2svNV = 0;
PFNGLVERTEXATTRIBS2FVNV glVertexAttribs2fvNV = 0;
PFNGLVERTEXATTRIBS2DVNV glVertexAttribs2dvNV = 0;
PFNGLVERTEXATTRIBS3SVNV glVertexAttribs3svNV = 0;
PFNGLVERTEXATTRIBS3FVNV glVertexAttribs3fvNV = 0;
PFNGLVERTEXATTRIBS3DVNV glVertexAttribs3dvNV = 0;
PFNGLVERTEXATTRIBS4SVNV glVertexAttribs4svNV = 0;
PFNGLVERTEXATTRIBS4FVNV glVertexAttribs4fvNV = 0;
PFNGLVERTEXATTRIBS4DVNV glVertexAttribs4dvNV = 0;
PFNGLVERTEXATTRIBS4UBVNV glVertexAttribs4ubvNV = 0;


//********************************************************************
}//namespace gl_ext


//********************************************************************
}//namespace vertexware


