

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Types.h"
#include "private\vw_opengl_exports.h"
#include <string>
#include <gl\gl.h>


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gl_tools {


//====================================================================
// Extracts GL version numbers.
//====================================================================
VW_OPENGL_API
void __fastcall ExtractVersionNumbers(uint& outMajor, uint& outMinor,
													uint& outRelease);


//====================================================================
// Returns GL + WGL list of extensions.
//====================================================================
VW_OPENGL_API
void __fastcall GetExtensionsString(HDC hdc, std::string& outExtensionsString);


//====================================================================
// Returns true if given extension is supported.
//====================================================================
VW_OPENGL_API
bool __fastcall IsExtensionSupported(const std::string& extensionsString,
									const std::string& extensionName);


//====================================================================
// Returns valid function-pointer if given extension-entry-point is found.
//====================================================================
VW_OPENGL_API
void* __fastcall GetExtensionProc(const char* extensionProcName);


//********************************************************************
}//namespace gl_tools


//********************************************************************
namespace gl_ext {


//====================================================================
// Extension list
//====================================================================
enum eExtension
{
	Extension_Invalid = -1,

	Extension_GL_CORE_12 = 0,
	Extension_GL_CORE_13,

	Extension_GL_ARB_imaging,
	Extension_GL_ARB_matrix_palette,
	Extension_GL_ARB_multisample,
	Extension_GL_ARB_multitexture,
	Extension_GL_ARB_point_parameters,
	Extension_GL_ARB_texture_compression,
	Extension_GL_ARB_texture_env_add,
	Extension_GL_ARB_texture_env_dot3,
	Extension_GL_ARB_texture_env_combine,
	Extension_GL_ARB_texture_env_crossbar,
	Extension_GL_ARB_texture_border_clamp,
	Extension_GL_ARB_texture_cube_map,
	Extension_GL_ARB_transpose_matrix,
	Extension_GL_ARB_vertex_blend,

	Extension_GL_EXT_abgr,
	Extension_GL_EXT_bgra,
	Extension_GL_EXT_compiled_vertex_array,
	Extension_GL_EXT_fog_coord,
	Extension_GL_EXT_multi_draw_arrays,
	Extension_GL_EXT_point_parameters,
	Extension_GL_EXT_secondary_color,
	Extension_GL_EXT_stencil_wrap,
	Extension_GL_EXT_texture_filter_anisotropic,
	Extension_GL_EXT_texture_lod_bias,
	Extension_GL_EXT_vertex_weighting,

	Extension_GL_ATI_pn_triangles,

	Extension_GL_NV_blend_square,
	Extension_GL_NV_copy_depth_to_color,
	Extension_GL_NV_evaluators,
	Extension_GL_NV_fence,
	Extension_GL_NV_fog_distance,
	Extension_GL_NV_light_max_exponent,
	Extension_GL_NV_packed_depth_stencil,
	Extension_GL_NV_register_combiners,
	Extension_GL_NV_register_combiners2,
	Extension_GL_NV_texgen_reflection,
	Extension_GL_NV_texture_env_combine4,
	Extension_GL_NV_texture_rectangle,
	Extension_GL_NV_texture_shader,
	Extension_GL_NV_texture_shader2,
	Extension_GL_NV_vertex_array_range,
	Extension_GL_NV_vertex_array_range2,
	Extension_GL_NV_vertex_program,

	Extension_GL_SGIS_generate_mipmap,
	Extension_GL_SGIX_shadow,
	Extension_GL_SGIX_depth_texture,

	Extension_Count,
	Extension_Force32Bits = 0x7FFFFFFF
};


//====================================================================
// Returns "Extension_Invalid" if not valid.
//====================================================================
VW_OPENGL_API std::string __fastcall  GetExtensionName(eExtension e);


//====================================================================
// Returns Extension_Invalid if not valid.
//====================================================================
VW_OPENGL_API eExtension __fastcall  GetExtensionFromName(const std::string& name);


//====================================================================
// GL implementation info.
//====================================================================
VW_OPENGL_API bool __fastcall  IsGLReady();
VW_OPENGL_API uint __fastcall  GetVersionGLMajor();
VW_OPENGL_API uint __fastcall  GetVersionGLMinor();
VW_OPENGL_API uint __fastcall  GetVersionGLRelease();
VW_OPENGL_API bool __fastcall  IsSupported(eExtension e);


//====================================================================
VW_OPENGL_API void __fastcall  SetGLReady(bool bReady);
VW_OPENGL_API void __fastcall  ClearImplementationInfo();
VW_OPENGL_API void __fastcall  SetVersionGLMajor(uint v);
VW_OPENGL_API void __fastcall  SetVersionGLMinor(uint v);
VW_OPENGL_API void __fastcall  SetVersionGLRelease(uint v);
VW_OPENGL_API void __fastcall  SetSupported(eExtension e, bool s);


//====================================================================
// GL_CORE_12, GL_CORE_13, GL_ARB_imaging constants
//====================================================================
#ifndef GL_VERSION_1_3

#ifdef GL_VERSION_1_2
#error Unhandled OpenGL-system-header version: GL_VERSION_1_2
#endif //GL_VERSION_1_2

#define GL_VERSION_1_3    1
#define GL_VERSION_1_2    1
#define GL_ARB_imaging    1


//--------------------------------------------------------------------
#define GL_RESCALE_NORMAL                                       0x803A
#define GL_CLAMP_TO_EDGE                                        0x812F
#define GL_MAX_ELEMENTS_VERTICES                                0x80E8
#define GL_MAX_ELEMENTS_INDICES                                 0x80E9
#define GL_BGR                                                  0x80E0
#define GL_BGRA                                                 0x80E1
#define GL_UNSIGNED_BYTE_3_3_2                                  0x8032
#define GL_UNSIGNED_BYTE_2_3_3_REV                              0x8362
#define GL_UNSIGNED_SHORT_5_6_5                                 0x8363
#define GL_UNSIGNED_SHORT_5_6_5_REV                             0x8364
#define GL_UNSIGNED_SHORT_4_4_4_4                               0x8033
#define GL_UNSIGNED_SHORT_4_4_4_4_REV                           0x8365
#define GL_UNSIGNED_SHORT_5_5_5_1                               0x8034
#define GL_UNSIGNED_SHORT_1_5_5_5_REV                           0x8366
#define GL_UNSIGNED_INT_8_8_8_8                                 0x8035
#define GL_UNSIGNED_INT_8_8_8_8_REV                             0x8367
#define GL_UNSIGNED_INT_10_10_10_2                              0x8036
#define GL_UNSIGNED_INT_2_10_10_10_REV                          0x8368
#define GL_LIGHT_MODEL_COLOR_CONTROL                            0x81F8
#define GL_SINGLE_COLOR                                         0x81F9
#define GL_SEPARATE_SPECULAR_COLOR                              0x81FA
#define GL_TEXTURE_MIN_LOD                                      0x813A
#define GL_TEXTURE_MAX_LOD                                      0x813B
#define GL_TEXTURE_BASE_LEVEL                                   0x813C
#define GL_TEXTURE_MAX_LEVEL                                    0x813D
#define GL_SMOOTH_POINT_SIZE_RANGE                              0x0B12
#define GL_SMOOTH_POINT_SIZE_GRANULARITY                        0x0B13
#define GL_SMOOTH_LINE_WIDTH_RANGE                              0x0B22
#define GL_SMOOTH_LINE_WIDTH_GRANULARITY                        0x0B23
#define GL_ALIASED_POINT_SIZE_RANGE                             0x846D
#define GL_ALIASED_LINE_WIDTH_RANGE                             0x846E
#define GL_PACK_SKIP_IMAGES                                     0x806B
#define GL_PACK_IMAGE_HEIGHT                                    0x806C
#define GL_UNPACK_SKIP_IMAGES                                   0x806D
#define GL_UNPACK_IMAGE_HEIGHT                                  0x806E
#define GL_TEXTURE_3D                                           0x806F
#define GL_PROXY_TEXTURE_3D                                     0x8070
#define GL_TEXTURE_DEPTH                                        0x8071
#define GL_TEXTURE_WRAP_R                                       0x8072
#define GL_MAX_3D_TEXTURE_SIZE                                  0x8073
#define GL_TEXTURE_BINDING_3D                                   0x806A


//--------------------------------------------------------------------
#define GL_COLOR_TABLE                                          0x80D0
#define GL_POST_CONVOLUTION_COLOR_TABLE                         0x80D1
#define GL_POST_COLOR_MATRIX_COLOR_TABLE                        0x80D2
#define GL_PROXY_COLOR_TABLE                                    0x80D3
#define GL_PROXY_POST_CONVOLUTION_COLOR_TABLE                   0x80D4
#define GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE                  0x80D5
#define GL_COLOR_TABLE_SCALE                                    0x80D6
#define GL_COLOR_TABLE_BIAS                                     0x80D7
#define GL_COLOR_TABLE_FORMAT                                   0x80D8
#define GL_COLOR_TABLE_WIDTH                                    0x80D9
#define GL_COLOR_TABLE_RED_SIZE                                 0x80DA
#define GL_COLOR_TABLE_GREEN_SIZE                               0x80DB
#define GL_COLOR_TABLE_BLUE_SIZE                                0x80DC
#define GL_COLOR_TABLE_ALPHA_SIZE                               0x80DD
#define GL_COLOR_TABLE_LUMINANCE_SIZE                           0x80DE
#define GL_COLOR_TABLE_INTENSITY_SIZE                           0x80DF


//--------------------------------------------------------------------
#define GL_CONVOLUTION_1D                                       0x8010
#define GL_CONVOLUTION_2D                                       0x8011
#define GL_SEPARABLE_2D                                         0x8012
#define GL_CONVOLUTION_BORDER_MODE                              0x8013
#define GL_CONVOLUTION_FILTER_SCALE                             0x8014
#define GL_CONVOLUTION_FILTER_BIAS                              0x8015
#define GL_REDUCE                                               0x8016
#define GL_CONVOLUTION_FORMAT                                   0x8017
#define GL_CONVOLUTION_WIDTH                                    0x8018
#define GL_CONVOLUTION_HEIGHT                                   0x8019
#define GL_MAX_CONVOLUTION_WIDTH                                0x801A
#define GL_MAX_CONVOLUTION_HEIGHT                               0x801B
#define GL_POST_CONVOLUTION_RED_SCALE                           0x801C
#define GL_POST_CONVOLUTION_GREEN_SCALE                         0x801D
#define GL_POST_CONVOLUTION_BLUE_SCALE                          0x801E
#define GL_POST_CONVOLUTION_ALPHA_SCALE                         0x801F
#define GL_POST_CONVOLUTION_RED_BIAS                            0x8020
#define GL_POST_CONVOLUTION_GREEN_BIAS                          0x8021
#define GL_POST_CONVOLUTION_BLUE_BIAS                           0x8022
#define GL_POST_CONVOLUTION_ALPHA_BIAS                          0x8023
#define GL_CONSTANT_BORDER                                      0x8151
#define GL_REPLICATE_BORDER                                     0x8153
#define GL_CONVOLUTION_BORDER_COLOR                             0x8154
#define GL_COLOR_MATRIX                                         0x80B1
#define GL_COLOR_MATRIX_STACK_DEPTH                             0x80B2
#define GL_MAX_COLOR_MATRIX_STACK_DEPTH                         0x80B3
#define GL_POST_COLOR_MATRIX_RED_SCALE                          0x80B4
#define GL_POST_COLOR_MATRIX_GREEN_SCALE                        0x80B5
#define GL_POST_COLOR_MATRIX_BLUE_SCALE                         0x80B6
#define GL_POST_COLOR_MATRIX_ALPHA_SCALE                        0x80B7
#define GL_POST_COLOR_MATRIX_RED_BIAS                           0x80B8
#define GL_POST_COLOR_MATRIX_GREEN_BIAS                         0x80B9
#define GL_POST_COLOR_MATRIX_BLUE_BIAS                          0x80BA
#define GL_POST_COLOR_MATRIX_ALPHA_BIAS                         0x80BB
#define GL_HISTOGRAM                                            0x8024
#define GL_PROXY_HISTOGRAM                                      0x8025
#define GL_HISTOGRAM_WIDTH                                      0x8026
#define GL_HISTOGRAM_FORMAT                                     0x8027
#define GL_HISTOGRAM_RED_SIZE                                   0x8028
#define GL_HISTOGRAM_GREEN_SIZE                                 0x8029
#define GL_HISTOGRAM_BLUE_SIZE                                  0x802A
#define GL_HISTOGRAM_ALPHA_SIZE                                 0x802B
#define GL_HISTOGRAM_LUMINANCE_SIZE                             0x802C
#define GL_HISTOGRAM_SINK                                       0x802D
#define GL_MINMAX                                               0x802E
#define GL_MINMAX_FORMAT                                        0x802F
#define GL_MINMAX_SINK                                          0x8030
#define GL_TABLE_TOO_LARGE                                      0x8031
#define GL_BLEND_EQUATION                                       0x8009
#define GL_MIN                                                  0x8007
#define GL_MAX                                                  0x8008
#define GL_FUNC_ADD                                             0x8006
#define GL_FUNC_SUBTRACT                                        0x800A
#define GL_FUNC_REVERSE_SUBTRACT                                0x800B
#define GL_BLEND_COLOR                                          0x8005


//--------------------------------------------------------------------
#define GL_TEXTURE0                                             0x84C0
#define GL_TEXTURE1                                             0x84C1
#define GL_TEXTURE2                                             0x84C2
#define GL_TEXTURE3                                             0x84C3
#define GL_TEXTURE4                                             0x84C4
#define GL_TEXTURE5                                             0x84C5
#define GL_TEXTURE6                                             0x84C6
#define GL_TEXTURE7                                             0x84C7
#define GL_TEXTURE8                                             0x84C8
#define GL_TEXTURE9                                             0x84C9
#define GL_TEXTURE10                                            0x84CA
#define GL_TEXTURE11                                            0x84CB
#define GL_TEXTURE12                                            0x84CC
#define GL_TEXTURE13                                            0x84CD
#define GL_TEXTURE14                                            0x84CE
#define GL_TEXTURE15                                            0x84CF
#define GL_TEXTURE16                                            0x84D0
#define GL_TEXTURE17                                            0x84D1
#define GL_TEXTURE18                                            0x84D2
#define GL_TEXTURE19                                            0x84D3
#define GL_TEXTURE20                                            0x84D4
#define GL_TEXTURE21                                            0x84D5
#define GL_TEXTURE22                                            0x84D6
#define GL_TEXTURE23                                            0x84D7
#define GL_TEXTURE24                                            0x84D8
#define GL_TEXTURE25                                            0x84D9
#define GL_TEXTURE26                                            0x84DA
#define GL_TEXTURE27                                            0x84DB
#define GL_TEXTURE28                                            0x84DC
#define GL_TEXTURE29                                            0x84DD
#define GL_TEXTURE30                                            0x84DE
#define GL_TEXTURE31                                            0x84DF
#define GL_ACTIVE_TEXTURE                                       0x84E0
#define GL_CLIENT_ACTIVE_TEXTURE                                0x84E1
#define GL_MAX_TEXTURE_UNITS                                    0x84E2


//--------------------------------------------------------------------
#define GL_NORMAL_MAP                                           0x8511
#define GL_REFLECTION_MAP                                       0x8512
#define GL_TEXTURE_CUBE_MAP                                     0x8513
#define GL_TEXTURE_BINDING_CUBE_MAP                             0x8514
#define GL_TEXTURE_CUBE_MAP_POSITIVE_X                          0x8515
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_X                          0x8516
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Y                          0x8517
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Y                          0x8518
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Z                          0x8519
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Z                          0x851A
#define GL_PROXY_TEXTURE_CUBE_MAP                               0x851B
#define GL_MAX_CUBE_MAP_TEXTURE_SIZE                            0x851C


//--------------------------------------------------------------------
#define GL_COMPRESSED_ALPHA                                     0x84E9
#define GL_COMPRESSED_LUMINANCE                                 0x84EA
#define GL_COMPRESSED_LUMINANCE_ALPHA                           0x84EB
#define GL_COMPRESSED_INTENSITY                                 0x84EC
#define GL_COMPRESSED_RGB                                       0x84ED
#define GL_COMPRESSED_RGBA                                      0x84EE
#define GL_TEXTURE_COMPRESSION_HINT                             0x84EF
#define GL_TEXTURE_COMPRESSED_IMAGE_SIZE                        0x86A0
#define GL_TEXTURE_COMPRESSED                                   0x86A1
#define GL_NUM_COMPRESSED_TEXTURE_FORMATS                       0x86A2
#define GL_COMPRESSED_TEXTURE_FORMATS                           0x86A3


//--------------------------------------------------------------------
#define GL_MULTISAMPLE                                          0x809D
#define GL_SAMPLE_ALPHA_TO_COVERAGE                             0x809E
#define GL_SAMPLE_ALPHA_TO_ONE                                  0x809F
#define GL_SAMPLE_COVERAGE                                      0x80A0
#define GL_SAMPLE_BUFFERS                                       0x80A8
#define GL_SAMPLES                                              0x80A9
#define GL_SAMPLE_COVERAGE_VALUE                                0x80AA
#define GL_SAMPLE_COVERAGE_INVERT                               0x80AB
#define GL_MULTISAMPLE_BIT                                      0x20000000


//--------------------------------------------------------------------
#define GL_TRANSPOSE_MODELVIEW_MATRIX                           0x84E3
#define GL_TRANSPOSE_PROJECTION_MATRIX                          0x84E4
#define GL_TRANSPOSE_TEXTURE_MATRIX                             0x84E5
#define GL_TRANSPOSE_COLOR_MATRIX                               0x84E6


//--------------------------------------------------------------------
#define GL_COMBINE                                              0x8570
#define GL_COMBINE_RGB                                          0x8571
#define GL_COMBINE_ALPHA                                        0x8572
#define GL_SOURCE0_RGB                                          0x8580
#define GL_SOURCE1_RGB                                          0x8581
#define GL_SOURCE2_RGB                                          0x8582
#define GL_SOURCE0_ALPHA                                        0x8588
#define GL_SOURCE1_ALPHA                                        0x8589
#define GL_SOURCE2_ALPHA                                        0x858A
#define GL_OPERAND0_RGB                                         0x8590
#define GL_OPERAND1_RGB                                         0x8591
#define GL_OPERAND2_RGB                                         0x8592
#define GL_OPERAND0_ALPHA                                       0x8598
#define GL_OPERAND1_ALPHA                                       0x8599
#define GL_OPERAND2_ALPHA                                       0x859A
#define GL_RGB_SCALE                                            0x8573
#define GL_ADD_SIGNED                                           0x8574
#define GL_INTERPOLATE                                          0x8575
#define GL_SUBTRACT                                             0x84E7
#define GL_CONSTANT                                             0x8576
#define GL_PRIMARY_COLOR                                        0x8577
#define GL_PREVIOUS                                             0x8578


//--------------------------------------------------------------------
#define GL_DOT3_RGB                                             0x86AE
#define GL_DOT3_RGBA                                            0x86AF


//--------------------------------------------------------------------
#define GL_CLAMP_TO_BORDER                                      0x812D


//--------------------------------------------------------------------
#endif //!GL_VERSION_1_3


//====================================================================
// GL_CORE_12 entry points
//====================================================================
typedef void (APIENTRY* PFNGLDRAWRANGEELEMENTS)(GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, const GLvoid* indices);
typedef void (APIENTRY* PFNGLTEXIMAGE3D)(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLenum format, GLenum type, const GLvoid* pixels);
typedef void (APIENTRY* PFNGLTEXSUBIMAGE3D)(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, const GLvoid* pixels);
typedef void (APIENTRY* PFNGLCOPYTEXSUBIMAGE3D)(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLint x, GLint y, GLsizei width, GLsizei height);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLDRAWRANGEELEMENTS glDrawRangeElements;
VW_OPENGL_API extern PFNGLTEXIMAGE3D glTexImage3D;
VW_OPENGL_API extern PFNGLTEXSUBIMAGE3D glTexSubImage3D;
VW_OPENGL_API extern PFNGLCOPYTEXSUBIMAGE3D glCopyTexSubImage3D;


//====================================================================
// GL_CORE_13 entry points
//====================================================================
typedef void (APIENTRY* PFNGLACTIVETEXTURE)(GLenum texture);
typedef void (APIENTRY* PFNGLCLIENTACTIVETEXTURE)(GLenum texture);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1D)(GLenum target, GLdouble s);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1DV)(GLenum target, const GLdouble* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1F)(GLenum target, GLfloat s);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1FV)(GLenum target, const GLfloat* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1I)(GLenum target, GLint s);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1IV)(GLenum target, const GLint* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1S)(GLenum target, GLshort s);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1SV)(GLenum target, const GLshort* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2D)(GLenum target, GLdouble s, GLdouble t);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2DV)(GLenum target, const GLdouble* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2F)(GLenum target, GLfloat s, GLfloat t);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2FV)(GLenum target, const GLfloat* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2I)(GLenum target, GLint s, GLint t);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2IV)(GLenum target, const GLint* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2S)(GLenum target, GLshort s, GLshort t);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2SV)(GLenum target, const GLshort* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3D)(GLenum target, GLdouble s, GLdouble t, GLdouble r);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3DV)(GLenum target, const GLdouble* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3F)(GLenum target, GLfloat s, GLfloat t, GLfloat r);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3FV)(GLenum target, const GLfloat* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3I)(GLenum target, GLint s, GLint t, GLint r);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3IV)(GLenum target, const GLint* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3S)(GLenum target, GLshort s, GLshort t, GLshort r);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3SV)(GLenum target, const GLshort* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4D)(GLenum target, GLdouble s, GLdouble t, GLdouble r, GLdouble q);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4DV)(GLenum target, const GLdouble* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4F)(GLenum target, GLfloat s, GLfloat t, GLfloat r, GLfloat q);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4FV)(GLenum target, const GLfloat* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4I)(GLenum target, GLint s, GLint t, GLint r, GLint q);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4IV)(GLenum target, const GLint* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4S)(GLenum target, GLshort s, GLshort t, GLshort r, GLshort q);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4SV)(GLenum target, const GLshort* v);
typedef void (APIENTRY* PFNGLLOADTRANSPOSEMATRIXF)(const GLfloat* m);
typedef void (APIENTRY* PFNGLLOADTRANSPOSEMATRIXD)(const GLdouble* m);
typedef void (APIENTRY* PFNGLMULTTRANSPOSEMATRIXF)(const GLfloat* m);
typedef void (APIENTRY* PFNGLMULTTRANSPOSEMATRIXD)(const GLdouble* m);
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXIMAGE1D)(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLint border, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXIMAGE2D)(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLint border, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXIMAGE3D)(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXSUBIMAGE1D)(GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXSUBIMAGE2D)(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXSUBIMAGE3D)(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLGETCOMPRESSEDTEXIMAGE)(GLenum target, GLint level, GLvoid* img);
typedef void (APIENTRY* PFNGLSAMPLECOVERAGE)(GLclampf value, GLboolean invert);
typedef void (APIENTRY* PFNGLSAMPLEPASS)(GLenum pass);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLACTIVETEXTURE glActiveTexture;
VW_OPENGL_API extern PFNGLCLIENTACTIVETEXTURE glClientActiveTexture;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1D glMultiTexCoord1d;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1DV glMultiTexCoord1dv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1F glMultiTexCoord1f;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1FV glMultiTexCoord1fv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1I glMultiTexCoord1i;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1IV glMultiTexCoord1iv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1S glMultiTexCoord1s;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1SV glMultiTexCoord1sv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2D glMultiTexCoord2d;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2DV glMultiTexCoord2dv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2F glMultiTexCoord2f;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2FV glMultiTexCoord2fv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2I glMultiTexCoord2i;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2IV glMultiTexCoord2iv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2S glMultiTexCoord2s;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2SV glMultiTexCoord2sv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3D glMultiTexCoord3d;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3DV glMultiTexCoord3dv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3F glMultiTexCoord3f;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3FV glMultiTexCoord3fv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3I glMultiTexCoord3i;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3IV glMultiTexCoord3iv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3S glMultiTexCoord3s;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3SV glMultiTexCoord3sv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4D glMultiTexCoord4d;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4DV glMultiTexCoord4dv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4F glMultiTexCoord4f;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4FV glMultiTexCoord4fv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4I glMultiTexCoord4i;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4IV glMultiTexCoord4iv;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4S glMultiTexCoord4s;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4SV glMultiTexCoord4sv;
VW_OPENGL_API extern PFNGLLOADTRANSPOSEMATRIXF glLoadTransposeMatrixf;
VW_OPENGL_API extern PFNGLLOADTRANSPOSEMATRIXD glLoadTransposeMatrixd;
VW_OPENGL_API extern PFNGLMULTTRANSPOSEMATRIXF glMultTransposeMatrixf;
VW_OPENGL_API extern PFNGLMULTTRANSPOSEMATRIXD glMultTransposeMatrixd;
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXIMAGE1D glCompressedTexImage1D;
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXIMAGE2D glCompressedTexImage2D;
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXIMAGE3D glCompressedTexImage3D;
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXSUBIMAGE1D glCompressedTexSubImage1D;
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXSUBIMAGE2D glCompressedTexSubImage2D;
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXSUBIMAGE3D glCompressedTexSubImage3D;
VW_OPENGL_API extern PFNGLGETCOMPRESSEDTEXIMAGE glGetCompressedTexImage;
VW_OPENGL_API extern PFNGLSAMPLECOVERAGE glSampleCoverage;


//====================================================================
// GL_ARB_imaging entry points
//====================================================================
typedef void (APIENTRY* PFNGLBLENDCOLOR)(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);
typedef void (APIENTRY* PFNGLBLENDEQUATION)(GLenum mode);
typedef void (APIENTRY* PFNGLCOLORTABLE)(GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, const GLvoid* table);
typedef void (APIENTRY* PFNGLCOLORTABLEPARAMETERFV)(GLenum target, GLenum pname, const GLfloat* params);
typedef void (APIENTRY* PFNGLCOLORTABLEPARAMETERIV)(GLenum target, GLenum pname, const GLint* params);
typedef void (APIENTRY* PFNGLCOPYCOLORTABLE)(GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width);
typedef void (APIENTRY* PFNGLGETCOLORTABLE)(GLenum target, GLenum format, GLenum type, GLvoid* table);
typedef void (APIENTRY* PFNGLGETCOLORTABLEPARAMETERFV)(GLenum target, GLenum pname, GLfloat* params);
typedef void (APIENTRY* PFNGLGETCOLORTABLEPARAMETERIV)(GLenum target, GLenum pname, GLint* params);
typedef void (APIENTRY* PFNGLCOLORSUBTABLE)(GLenum target, GLsizei start, GLsizei count, GLenum format, GLenum type, const GLvoid* data);
typedef void (APIENTRY* PFNGLCOPYCOLORSUBTABLE)(GLenum target, GLsizei start, GLint x, GLint y, GLsizei width);
typedef void (APIENTRY* PFNGLCONVOLUTIONFILTER1D)(GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, const GLvoid* image);
typedef void (APIENTRY* PFNGLCONVOLUTIONFILTER2D)(GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid* image);
typedef void (APIENTRY* PFNGLCONVOLUTIONPARAMETERF)(GLenum target, GLenum pname, GLfloat params);
typedef void (APIENTRY* PFNGLCONVOLUTIONPARAMETERFV)(GLenum target, GLenum pname, const GLfloat* params);
typedef void (APIENTRY* PFNGLCONVOLUTIONPARAMETERI)(GLenum target, GLenum pname, GLint params);
typedef void (APIENTRY* PFNGLCONVOLUTIONPARAMETERIV)(GLenum target, GLenum pname, const GLint* params);
typedef void (APIENTRY* PFNGLCOPYCONVOLUTIONFILTER1D)(GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width);
typedef void (APIENTRY* PFNGLCOPYCONVOLUTIONFILTER2D)(GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height);
typedef void (APIENTRY* PFNGLGETCONVOLUTIONFILTER)(GLenum target, GLenum format, GLenum type, GLvoid* image);
typedef void (APIENTRY* PFNGLGETCONVOLUTIONPARAMETERFV)(GLenum target, GLenum pname, GLfloat* params);
typedef void (APIENTRY* PFNGLGETCONVOLUTIONPARAMETERIV)(GLenum target, GLenum pname, GLint* params);
typedef void (APIENTRY* PFNGLGETSEPARABLEFILTER)(GLenum target, GLenum format, GLenum type, GLvoid* row, GLvoid* column, GLvoid* span);
typedef void (APIENTRY* PFNGLSEPARABLEFILTER2D)(GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid* row, const GLvoid* column);
typedef void (APIENTRY* PFNGLGETHISTOGRAM)(GLenum target, GLboolean reset, GLenum format, GLenum type, GLvoid* values);
typedef void (APIENTRY* PFNGLGETHISTOGRAMPARAMETERFV)(GLenum target, GLenum pname, GLfloat* params);
typedef void (APIENTRY* PFNGLGETHISTOGRAMPARAMETERIV)(GLenum target, GLenum pname, GLint* params);
typedef void (APIENTRY* PFNGLGETMINMAX)(GLenum target, GLboolean reset, GLenum format, GLenum type, GLvoid* values);
typedef void (APIENTRY* PFNGLGETMINMAXPARAMETERFV)(GLenum target, GLenum pname, GLfloat* params);
typedef void (APIENTRY* PFNGLGETMINMAXPARAMETERIV)(GLenum target, GLenum pname, GLint* params);
typedef void (APIENTRY* PFNGLHISTOGRAM)(GLenum target, GLsizei width, GLenum internalformat, GLboolean sink);
typedef void (APIENTRY* PFNGLMINMAX)(GLenum target, GLenum internalformat, GLboolean sink);
typedef void (APIENTRY* PFNGLRESETHISTOGRAM)(GLenum target);
typedef void (APIENTRY* PFNGLRESETMINMAX)(GLenum target);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLBLENDCOLOR glBlendColor;
VW_OPENGL_API extern PFNGLBLENDEQUATION glBlendEquation;
VW_OPENGL_API extern PFNGLCOLORTABLE glColorTable;
VW_OPENGL_API extern PFNGLCOLORTABLEPARAMETERFV glColorTableParameterfv;
VW_OPENGL_API extern PFNGLCOLORTABLEPARAMETERIV glColorTableParameteriv;
VW_OPENGL_API extern PFNGLCOPYCOLORTABLE glCopyColorTable;
VW_OPENGL_API extern PFNGLGETCOLORTABLE glGetColorTable;
VW_OPENGL_API extern PFNGLGETCOLORTABLEPARAMETERFV glGetColorTableParameterfv;
VW_OPENGL_API extern PFNGLGETCOLORTABLEPARAMETERIV glGetColorTableParameteriv;
VW_OPENGL_API extern PFNGLCOLORSUBTABLE glColorSubTable;
VW_OPENGL_API extern PFNGLCOPYCOLORSUBTABLE glCopyColorSubTable;
VW_OPENGL_API extern PFNGLCONVOLUTIONFILTER1D glConvolutionFilter1D;
VW_OPENGL_API extern PFNGLCONVOLUTIONFILTER2D glConvolutionFilter2D;
VW_OPENGL_API extern PFNGLCONVOLUTIONPARAMETERF glConvolutionParameterf;
VW_OPENGL_API extern PFNGLCONVOLUTIONPARAMETERFV glConvolutionParameterfv;
VW_OPENGL_API extern PFNGLCONVOLUTIONPARAMETERI glConvolutionParameteri;
VW_OPENGL_API extern PFNGLCONVOLUTIONPARAMETERIV glConvolutionParameteriv;
VW_OPENGL_API extern PFNGLCOPYCONVOLUTIONFILTER1D glCopyConvolutionFilter1D;
VW_OPENGL_API extern PFNGLCOPYCONVOLUTIONFILTER2D glCopyConvolutionFilter2D;
VW_OPENGL_API extern PFNGLGETCONVOLUTIONFILTER glGetConvolutionFilter;
VW_OPENGL_API extern PFNGLGETCONVOLUTIONPARAMETERFV glGetConvolutionParameterfv;
VW_OPENGL_API extern PFNGLGETCONVOLUTIONPARAMETERIV glGetConvolutionParameteriv;
VW_OPENGL_API extern PFNGLGETSEPARABLEFILTER glGetSeparableFilter;
VW_OPENGL_API extern PFNGLSEPARABLEFILTER2D glSeparableFilter2D;
VW_OPENGL_API extern PFNGLGETHISTOGRAM glGetHistogram;
VW_OPENGL_API extern PFNGLGETHISTOGRAMPARAMETERFV glGetHistogramParameterfv;
VW_OPENGL_API extern PFNGLGETHISTOGRAMPARAMETERIV glGetHistogramParameteriv;
VW_OPENGL_API extern PFNGLGETMINMAX glGetMinmax;
VW_OPENGL_API extern PFNGLGETMINMAXPARAMETERFV glGetMinmaxParameterfv;
VW_OPENGL_API extern PFNGLGETMINMAXPARAMETERIV glGetMinmaxParameteriv;
VW_OPENGL_API extern PFNGLHISTOGRAM glHistogram;
VW_OPENGL_API extern PFNGLMINMAX glMinmax;
VW_OPENGL_API extern PFNGLRESETHISTOGRAM glResetHistogram;
VW_OPENGL_API extern PFNGLRESETMINMAX glResetMinmax;


//====================================================================
// GL_ARB_matrix_palette
//====================================================================
#ifndef GL_ARB_matrix_palette
#define GL_ARB_matrix_palette    1


//--------------------------------------------------------------------
#define GL_MATRIX_PALETTE_ARB                                   0x8840
#define GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB                   0x8841
#define GL_MAX_PALETTE_MATRICES_ARB                             0x8842
#define GL_CURRENT_PALETTE_MATRIX_ARB                           0x8843
#define GL_MATRIX_INDEX_ARRAY_ARB                               0x8844
#define GL_CURRENT_MATRIX_INDEX_ARB                             0x8845
#define GL_MATRIX_INDEX_ARRAY_SIZE_ARB                          0x8846
#define GL_MATRIX_INDEX_ARRAY_TYPE_ARB                          0x8847
#define GL_MATRIX_INDEX_ARRAY_STRIDE_ARB                        0x8848
#define GL_MATRIX_INDEX_ARRAY_POINTER_ARB                       0x8849


//--------------------------------------------------------------------
#endif //!GL_ARB_matrix_palette


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLCURRENTPALETTEMATRIXARB)(GLint index);
typedef void (APIENTRY* PFNGLMATRIXINDEXUBVARB)(GLint size, GLubyte* indices);
typedef void (APIENTRY* PFNGLMATRIXINDEXUSVARB)(GLint size, GLushort* indices);
typedef void (APIENTRY* PFNGLMATRIXINDEXUIVARB)(GLint size, GLuint* indices);
typedef void (APIENTRY* PFNGLMATRIXINDEXPOINTERARB)(GLint size, GLenum type, GLsizei stride, GLvoid* pointer);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLCURRENTPALETTEMATRIXARB glCurrentPaletteMatrixARB;
VW_OPENGL_API extern PFNGLMATRIXINDEXUBVARB glMatrixIndexubvARB;
VW_OPENGL_API extern PFNGLMATRIXINDEXUSVARB glMatrixIndexusvARB;
VW_OPENGL_API extern PFNGLMATRIXINDEXUIVARB glMatrixIndexuivARB;
VW_OPENGL_API extern PFNGLMATRIXINDEXPOINTERARB glMatrixIndexPointerARB;


//====================================================================
// GL_ARB_multisample
//====================================================================
#ifndef GL_ARB_multisample
#define GL_ARB_multisample    1


//--------------------------------------------------------------------
#define GL_MULTISAMPLE_ARB                                      0x809D
#define GL_SAMPLE_ALPHA_TO_COVERAGE_ARB                         0x809E
#define GL_SAMPLE_ALPHA_TO_ONE_ARB                              0x809F
#define GL_SAMPLE_COVERAGE_ARB                                  0x80A0
#define GL_SAMPLE_BUFFERS_ARB                                   0x80A8
#define GL_SAMPLES_ARB                                          0x80A9
#define GL_SAMPLE_COVERAGE_VALUE_ARB                            0x80AA
#define GL_SAMPLE_COVERAGE_INVERT_ARB                           0x80AB
#define GL_MULTISAMPLE_BIT_ARB                                  0x20000000


//--------------------------------------------------------------------
#endif //!GL_ARB_multisample


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLSAMPLECOVERAGEARB)(GLclampf value, GLboolean invert);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLSAMPLECOVERAGEARB glSampleCoverageARB;


//====================================================================
// GL_ARB_multitexture
//====================================================================
#ifndef GL_ARB_multitexture
#define GL_ARB_multitexture    1


//--------------------------------------------------------------------
#define GL_TEXTURE0_ARB                                         0x84C0
#define GL_TEXTURE1_ARB                                         0x84C1
#define GL_TEXTURE2_ARB                                         0x84C2
#define GL_TEXTURE3_ARB                                         0x84C3
#define GL_TEXTURE4_ARB                                         0x84C4
#define GL_TEXTURE5_ARB                                         0x84C5
#define GL_TEXTURE6_ARB                                         0x84C6
#define GL_TEXTURE7_ARB                                         0x84C7
#define GL_TEXTURE8_ARB                                         0x84C8
#define GL_TEXTURE9_ARB                                         0x84C9
#define GL_TEXTURE10_ARB                                        0x84CA
#define GL_TEXTURE11_ARB                                        0x84CB
#define GL_TEXTURE12_ARB                                        0x84CC
#define GL_TEXTURE13_ARB                                        0x84CD
#define GL_TEXTURE14_ARB                                        0x84CE
#define GL_TEXTURE15_ARB                                        0x84CF
#define GL_TEXTURE16_ARB                                        0x84D0
#define GL_TEXTURE17_ARB                                        0x84D1
#define GL_TEXTURE18_ARB                                        0x84D2
#define GL_TEXTURE19_ARB                                        0x84D3
#define GL_TEXTURE20_ARB                                        0x84D4
#define GL_TEXTURE21_ARB                                        0x84D5
#define GL_TEXTURE22_ARB                                        0x84D6
#define GL_TEXTURE23_ARB                                        0x84D7
#define GL_TEXTURE24_ARB                                        0x84D8
#define GL_TEXTURE25_ARB                                        0x84D9
#define GL_TEXTURE26_ARB                                        0x84DA
#define GL_TEXTURE27_ARB                                        0x84DB
#define GL_TEXTURE28_ARB                                        0x84DC
#define GL_TEXTURE29_ARB                                        0x84DD
#define GL_TEXTURE30_ARB                                        0x84DE
#define GL_TEXTURE31_ARB                                        0x84DF
#define GL_ACTIVE_TEXTURE_ARB                                   0x84E0
#define GL_CLIENT_ACTIVE_TEXTURE_ARB                            0x84E1
#define GL_MAX_TEXTURE_UNITS_ARB                                0x84E2


//--------------------------------------------------------------------
#endif //!GL_ARB_multitexture


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLACTIVETEXTUREARB)(GLenum texture);
typedef void (APIENTRY* PFNGLCLIENTACTIVETEXTUREARB)(GLenum texture);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1DARB)(GLenum target, GLdouble s);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1DVARB)(GLenum target, const GLdouble* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1FARB)(GLenum target, GLfloat s);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1FVARB)(GLenum target, const GLfloat* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1IARB)(GLenum target, GLint s);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1IVARB)(GLenum target, const GLint* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1SARB)(GLenum target, GLshort s);
typedef void (APIENTRY* PFNGLMULTITEXCOORD1SVARB)(GLenum target, const GLshort* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2DARB)(GLenum target, GLdouble s, GLdouble t);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2DVARB)(GLenum target, const GLdouble* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2FARB)(GLenum target, GLfloat s, GLfloat t);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2FVARB)(GLenum target, const GLfloat* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2IARB)(GLenum target, GLint s, GLint t);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2IVARB)(GLenum target, const GLint* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2SARB)(GLenum target, GLshort s, GLshort t);
typedef void (APIENTRY* PFNGLMULTITEXCOORD2SVARB)(GLenum target, const GLshort* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3DARB)(GLenum target, GLdouble s, GLdouble t, GLdouble r);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3DVARB)(GLenum target, const GLdouble* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3FARB)(GLenum target, GLfloat s, GLfloat t, GLfloat r);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3FVARB)(GLenum target, const GLfloat* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3IARB)(GLenum target, GLint s, GLint t, GLint r);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3IVARB)(GLenum target, const GLint* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3SARB)(GLenum target, GLshort s, GLshort t, GLshort r);
typedef void (APIENTRY* PFNGLMULTITEXCOORD3SVARB)(GLenum target, const GLshort* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4DARB)(GLenum target, GLdouble s, GLdouble t, GLdouble r, GLdouble q);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4DVARB)(GLenum target, const GLdouble* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4FARB)(GLenum target, GLfloat s, GLfloat t, GLfloat r, GLfloat q);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4FVARB)(GLenum target, const GLfloat* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4IARB)(GLenum target, GLint s, GLint t, GLint r, GLint q);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4IVARB)(GLenum target, const GLint* v);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4SARB)(GLenum target, GLshort s, GLshort t, GLshort r, GLshort q);
typedef void (APIENTRY* PFNGLMULTITEXCOORD4SVARB)(GLenum target, const GLshort* v);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLACTIVETEXTUREARB glActiveTextureARB;
VW_OPENGL_API extern PFNGLCLIENTACTIVETEXTUREARB glClientActiveTextureARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1DARB glMultiTexCoord1dARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1DVARB glMultiTexCoord1dvARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1FARB glMultiTexCoord1fARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1FVARB glMultiTexCoord1fvARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1IARB glMultiTexCoord1iARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1IVARB glMultiTexCoord1ivARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1SARB glMultiTexCoord1sARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD1SVARB glMultiTexCoord1svARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2DARB glMultiTexCoord2dARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2DVARB glMultiTexCoord2dvARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2FARB glMultiTexCoord2fARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2FVARB glMultiTexCoord2fvARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2IARB glMultiTexCoord2iARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2IVARB glMultiTexCoord2ivARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2SARB glMultiTexCoord2sARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD2SVARB glMultiTexCoord2svARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3DARB glMultiTexCoord3dARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3DVARB glMultiTexCoord3dvARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3FARB glMultiTexCoord3fARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3FVARB glMultiTexCoord3fvARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3IARB glMultiTexCoord3iARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3IVARB glMultiTexCoord3ivARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3SARB glMultiTexCoord3sARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD3SVARB glMultiTexCoord3svARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4DARB glMultiTexCoord4dARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4DVARB glMultiTexCoord4dvARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4FARB glMultiTexCoord4fARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4FVARB glMultiTexCoord4fvARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4IARB glMultiTexCoord4iARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4IVARB glMultiTexCoord4ivARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4SARB glMultiTexCoord4sARB;
VW_OPENGL_API extern PFNGLMULTITEXCOORD4SVARB glMultiTexCoord4svARB;


//====================================================================
// GL_ARB_point_parameters
//====================================================================
#ifndef GL_ARB_point_parameters
#define GL_ARB_point_parameters    1


//--------------------------------------------------------------------
#define GL_POINT_SIZE_MIN_ARB                                   0x8126
#define GL_POINT_SIZE_MAX_ARB                                   0x8127
#define GL_POINT_FADE_THRESHOLD_SIZE_ARB                        0x8128
#define GL_POINT_DISTANCE_ATTENUATION_ARB                       0x8129


//--------------------------------------------------------------------
#endif //!GL_ARB_point_parameters


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLPOINTPARAMETERFARB)(GLenum pname, GLfloat param);
typedef void (APIENTRY* PFNGLPOINTPARAMETERFVARB)(GLenum pname, GLfloat* params);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLPOINTPARAMETERFARB glPointParameterfARB;
VW_OPENGL_API extern PFNGLPOINTPARAMETERFVARB glPointParameterfvARB;


//====================================================================
// GL_ARB_texture_compression
//====================================================================
#ifndef GL_ARB_texture_compression
#define GL_ARB_texture_compression    1


//--------------------------------------------------------------------
#define GL_COMPRESSED_ALPHA_ARB                                 0x84E9
#define GL_COMPRESSED_LUMINANCE_ARB                             0x84EA
#define GL_COMPRESSED_LUMINANCE_ALPHA_ARB                       0x84EB
#define GL_COMPRESSED_INTENSITY_ARB                             0x84EC
#define GL_COMPRESSED_RGB_ARB                                   0x84ED
#define GL_COMPRESSED_RGBA_ARB                                  0x84EE
#define GL_TEXTURE_COMPRESSION_HINT_ARB                         0x84EF
#define GL_TEXTURE_IMAGE_SIZE_ARB                               0x86A0
#define GL_TEXTURE_COMPRESSED_ARB                               0x86A1
#define GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB                   0x86A2
#define GL_COMPRESSED_TEXTURE_FORMATS_ARB                       0x86A3


//--------------------------------------------------------------------
#endif //!GL_ARB_texture_compression


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXIMAGE1DARB)(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLint border, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXIMAGE2DARB)(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLint border, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXIMAGE3DARB)(GLenum target, GLint level, GLenum internalformat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXSUBIMAGE1DARB)(GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXSUBIMAGE2DARB)(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLCOMPRESSEDTEXSUBIMAGE3DARB)(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLsizei imageSize, const GLvoid* data);
typedef void (APIENTRY* PFNGLGETCOMPRESSEDTEXIMAGEARB)(GLenum target, GLint level, GLvoid* img);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXIMAGE1DARB glCompressedTexImage1DARB;
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXIMAGE2DARB glCompressedTexImage2DARB;
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXIMAGE3DARB glCompressedTexImage3DARB;
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXSUBIMAGE1DARB glCompressedTexSubImage1DARB;
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXSUBIMAGE2DARB glCompressedTexSubImage2DARB;
VW_OPENGL_API extern PFNGLCOMPRESSEDTEXSUBIMAGE3DARB glCompressedTexSubImage3DARB;
VW_OPENGL_API extern PFNGLGETCOMPRESSEDTEXIMAGEARB glGetCompressedTexImageARB;


//====================================================================
// GL_ARB_texture_env_add
//====================================================================


//====================================================================
// GL_ARB_texture_env_dot3
//====================================================================
#ifndef GL_ARB_texture_env_dot3
#define GL_ARB_texture_env_dot3    1


//--------------------------------------------------------------------
#define GL_DOT3_RGB_ARB                                         0x86AE
#define GL_DOT3_RGBA_ARB                                        0x86AF


//--------------------------------------------------------------------
#endif //!GL_ARB_texture_env_dot3


//====================================================================
// GL_ARB_texture_env_combine
//====================================================================
#ifndef GL_ARB_texture_env_combine
#define GL_ARB_texture_env_combine    1


//--------------------------------------------------------------------
#define GL_COMBINE_ARB                                          0x8570
#define GL_COMBINE_RGB_ARB                                      0x8571
#define GL_COMBINE_ALPHA_ARB                                    0x8572
#define GL_RGB_SCALE_ARB                                        0x8573
#define GL_ADD_SIGNED_ARB                                       0x8574
#define GL_INTERPOLATE_ARB                                      0x8575
#define GL_CONSTANT_ARB                                         0x8576
#define GL_PRIMARY_COLOR_ARB                                    0x8577
#define GL_PREVIOUS_ARB                                         0x8578
#define GL_SOURCE0_RGB_ARB                                      0x8580
#define GL_SOURCE1_RGB_ARB                                      0x8581
#define GL_SOURCE2_RGB_ARB                                      0x8582
#define GL_SOURCE0_ALPHA_ARB                                    0x8588
#define GL_SOURCE1_ALPHA_ARB                                    0x8589
#define GL_SOURCE2_ALPHA_ARB                                    0x858A
#define GL_OPERAND0_RGB_ARB                                     0x8590
#define GL_OPERAND1_RGB_ARB                                     0x8591
#define GL_OPERAND2_RGB_ARB                                     0x8592
#define GL_OPERAND0_ALPHA_ARB                                   0x8598
#define GL_OPERAND1_ALPHA_ARB                                   0x8599
#define GL_OPERAND2_ALPHA_ARB                                   0x859A


//--------------------------------------------------------------------
#endif //!GL_ARB_texture_env_combine


//====================================================================
// GL_ARB_texture_env_crossbar
//====================================================================


//====================================================================
// GL_ARB_texture_border_clamp
//====================================================================
#ifndef GL_ARB_texture_border_clamp
#define GL_ARB_texture_border_clamp    1


//--------------------------------------------------------------------
#define GL_CLAMP_TO_BORDER_ARB                                  0x812D


//--------------------------------------------------------------------
#endif //!GL_ARB_texture_border_clamp


//====================================================================
// GL_ARB_texture_cube_map
//====================================================================
#ifndef GL_ARB_texture_cube_map
#define GL_ARB_texture_cube_map    1


//--------------------------------------------------------------------
#define GL_NORMAL_MAP_ARB                                       0x8511
#define GL_REFLECTION_MAP_ARB                                   0x8512
#define GL_TEXTURE_CUBE_MAP_ARB                                 0x8513
#define GL_TEXTURE_BINDING_CUBE_MAP_ARB                         0x8514
#define GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB                      0x8515
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB                      0x8516
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB                      0x8517
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB                      0x8518
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB                      0x8519
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB                      0x851A
#define GL_PROXY_TEXTURE_CUBE_MAP_ARB                           0x851B
#define GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB                        0x851C


//--------------------------------------------------------------------
#endif //!GL_ARB_texture_cube_map


//====================================================================
// GL_ARB_transpose_matrix
//====================================================================
#ifndef GL_ARB_transpose_matrix
#define GL_ARB_transpose_matrix    1


//--------------------------------------------------------------------
#define GL_TRANSPOSE_MODELVIEW_MATRIX_ARB                       0x84E3
#define GL_TRANSPOSE_PROJECTION_MATRIX_ARB                      0x84E4
#define GL_TRANSPOSE_TEXTURE_MATRIX_ARB                         0x84E5
#define GL_TRANSPOSE_COLOR_MATRIX_ARB                           0x84E6


//--------------------------------------------------------------------
#endif //!GL_ARB_transpose_matrix


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLLOADTRANSPOSEMATRIXFARB)(const GLfloat* m);
typedef void (APIENTRY* PFNGLLOADTRANSPOSEMATRIXDARB)(const GLdouble* m);
typedef void (APIENTRY* PFNGLMULTTRANSPOSEMATRIXFARB)(const GLfloat* m);
typedef void (APIENTRY* PFNGLMULTTRANSPOSEMATRIXDARB)(const GLdouble* m);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLLOADTRANSPOSEMATRIXFARB glLoadTransposeMatrixfARB;
VW_OPENGL_API extern PFNGLLOADTRANSPOSEMATRIXDARB glLoadTransposeMatrixdARB;
VW_OPENGL_API extern PFNGLMULTTRANSPOSEMATRIXFARB glMultTransposeMatrixfARB;
VW_OPENGL_API extern PFNGLMULTTRANSPOSEMATRIXDARB glMultTransposeMatrixdARB;


//====================================================================
// GL_ARB_vertex_blend
//====================================================================
#ifndef GL_ARB_vertex_blend
#define GL_ARB_vertex_blend    1


//--------------------------------------------------------------------
#define GL_MAX_VERTEX_UNITS_ARB                                 0x86A4
#define GL_ACTIVE_VERTEX_UNITS_ARB                              0x86A5
#define GL_WEIGHT_SUM_UNITY_ARB                                 0x86A6      
#define GL_VERTEX_BLEND_ARB                                     0x86A7
#define GL_CURRENT_WEIGHT_ARB                                   0x86A8
#define GL_WEIGHT_ARRAY_TYPE_ARB                                0x86A9
#define GL_WEIGHT_ARRAY_STRIDE_ARB                              0x86AA
#define GL_WEIGHT_ARRAY_SIZE_ARB                                0x86AB
#define GL_WEIGHT_ARRAY_POINTER_ARB                             0x86AC
#define GL_WEIGHT_ARRAY_ARB                                     0x86AD
#define GL_MODELVIEW0_ARB                                       0x1700
#define GL_MODELVIEW1_ARB                                       0x850a
#define GL_MODELVIEW2_ARB                                       0x8722
#define GL_MODELVIEW3_ARB                                       0x8723
#define GL_MODELVIEW4_ARB                                       0x8724
#define GL_MODELVIEW5_ARB                                       0x8725
#define GL_MODELVIEW6_ARB                                       0x8726
#define GL_MODELVIEW7_ARB                                       0x8727
#define GL_MODELVIEW8_ARB                                       0x8728
#define GL_MODELVIEW9_ARB                                       0x8729
#define GL_MODELVIEW10_ARB                                      0x872A
#define GL_MODELVIEW11_ARB                                      0x872B
#define GL_MODELVIEW12_ARB                                      0x872C
#define GL_MODELVIEW13_ARB                                      0x872D
#define GL_MODELVIEW14_ARB                                      0x872E
#define GL_MODELVIEW15_ARB                                      0x872F
#define GL_MODELVIEW16_ARB                                      0x8730
#define GL_MODELVIEW17_ARB                                      0x8731
#define GL_MODELVIEW18_ARB                                      0x8732
#define GL_MODELVIEW19_ARB                                      0x8733
#define GL_MODELVIEW20_ARB                                      0x8734
#define GL_MODELVIEW21_ARB                                      0x8735
#define GL_MODELVIEW22_ARB                                      0x8736
#define GL_MODELVIEW23_ARB                                      0x8737
#define GL_MODELVIEW24_ARB                                      0x8738
#define GL_MODELVIEW25_ARB                                      0x8739
#define GL_MODELVIEW26_ARB                                      0x873A
#define GL_MODELVIEW27_ARB                                      0x873B
#define GL_MODELVIEW28_ARB                                      0x873C
#define GL_MODELVIEW29_ARB                                      0x873D
#define GL_MODELVIEW30_ARB                                      0x873E
#define GL_MODELVIEW31_ARB                                      0x873F


//--------------------------------------------------------------------
#endif //!GL_ARB_vertex_blend


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLWEIGHTBVARB)(GLint size, GLbyte* weights);
typedef void (APIENTRY* PFNGLWEIGHTSVARB)(GLint size, GLshort* weights);
typedef void (APIENTRY* PFNGLWEIGHTIVARB)(GLint size, GLint* weights);
typedef void (APIENTRY* PFNGLWEIGHTFVARB)(GLint size, GLfloat* weights);
typedef void (APIENTRY* PFNGLWEIGHTDVARB)(GLint size, GLdouble* weights);
typedef void (APIENTRY* PFNGLWEIGHTUBVARB)(GLint size, GLubyte* weights);
typedef void (APIENTRY* PFNGLWEIGHTUSVARB)(GLint size, GLushort* weights);
typedef void (APIENTRY* PFNGLWEIGHTUIVARB)(GLint size, GLuint* weights);
typedef void (APIENTRY* PFNGLWEIGHTPOINTERARB)(GLint size, GLenum type, GLsizei stride, GLvoid* pointer);
typedef void (APIENTRY* PFNGLVERTEXBLENDARB)(GLint count);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLWEIGHTBVARB glWeightbvARB;
VW_OPENGL_API extern PFNGLWEIGHTSVARB glWeightsvARB;
VW_OPENGL_API extern PFNGLWEIGHTIVARB glWeightivARB;
VW_OPENGL_API extern PFNGLWEIGHTFVARB glWeightfvARB;
VW_OPENGL_API extern PFNGLWEIGHTDVARB glWeightdvARB;
VW_OPENGL_API extern PFNGLWEIGHTUBVARB glWeightubvARB;
VW_OPENGL_API extern PFNGLWEIGHTUSVARB glWeightusvARB;
VW_OPENGL_API extern PFNGLWEIGHTUIVARB glWeightuivARB;
VW_OPENGL_API extern PFNGLWEIGHTPOINTERARB glWeightPointerARB;
VW_OPENGL_API extern PFNGLVERTEXBLENDARB glVertexBlendARB;


//====================================================================
// GL_EXT_abgr
//====================================================================
#ifndef GL_EXT_abgr
#define GL_EXT_abgr    1


//--------------------------------------------------------------------
#define GL_ABGR_EXT                                             0x8000


//--------------------------------------------------------------------
#endif //!GL_EXT_abgr


//====================================================================
// GL_EXT_bgra
//====================================================================
#ifndef GL_EXT_bgra
#define GL_EXT_bgra    1


//--------------------------------------------------------------------
#define GL_BGR_EXT                                              0x80E0
#define GL_BGRA_EXT                                             0x80E1


//--------------------------------------------------------------------
#endif //!GL_EXT_bgra


//====================================================================
// GL_EXT_compiled_vertex_array
//====================================================================
#ifndef GL_EXT_compiled_vertex_array
#define GL_EXT_compiled_vertex_array    1


//--------------------------------------------------------------------
#define GL_ARRAY_ELEMENT_LOCK_FIRST_EXT                         0x81A8
#define GL_ARRAY_ELEMENT_LOCK_COUNT_EXT                         0x81A9


//--------------------------------------------------------------------
#endif //!GL_EXT_compiled_vertex_array


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLLOCKARRAYSEXT)(GLint first, GLsizei count);
typedef void (APIENTRY* PFNGLUNLOCKARRAYSEXT)(void);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLLOCKARRAYSEXT glLockArraysEXT;
VW_OPENGL_API extern PFNGLUNLOCKARRAYSEXT glUnlockArraysEXT;


//====================================================================
// GL_EXT_fog_coord
//====================================================================
#ifndef GL_EXT_fog_coord
#define GL_EXT_fog_coord    1


//--------------------------------------------------------------------
#define GL_FOG_COORDINATE_SOURCE_EXT                            0x8450
#define GL_FOG_COORDINATE_EXT                                   0x8451
#define GL_FRAGMENT_DEPTH_EXT                                   0x8452
#define GL_CURRENT_FOG_COORDINATE_EXT                           0x8453
#define GL_FOG_COORDINATE_ARRAY_TYPE_EXT                        0x8454
#define GL_FOG_COORDINATE_ARRAY_STRIDE_EXT                      0x8455
#define GL_FOG_COORDINATE_ARRAY_POINTER_EXT                     0x8456
#define GL_FOG_COORDINATE_ARRAY_EXT                             0x8457


//--------------------------------------------------------------------
#endif //!GL_EXT_fog_coord


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLFOGCOORDFEXT)(GLfloat coord);
typedef void (APIENTRY* PFNGLFOGCOORDFVEXT)(const GLfloat* coord);
typedef void (APIENTRY* PFNGLFOGCOORDDEXT)(GLdouble coord);
typedef void (APIENTRY* PFNGLFOGCOORDDVEXT)(const GLdouble* coord);
typedef void (APIENTRY* PFNGLFOGCOORDPOINTEREXT)(GLenum type, GLsizei stride, const GLvoid* pointer);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLFOGCOORDFEXT glFogCoordfEXT;
VW_OPENGL_API extern PFNGLFOGCOORDFVEXT glFogCoordfvEXT;
VW_OPENGL_API extern PFNGLFOGCOORDDEXT glFogCoorddEXT;
VW_OPENGL_API extern PFNGLFOGCOORDDVEXT glFogCoorddvEXT;
VW_OPENGL_API extern PFNGLFOGCOORDPOINTEREXT glFogCoordPointerEXT;


//====================================================================
// GL_EXT_multi_draw_arrays
//====================================================================
typedef void (APIENTRY* PFNGLMULTIDRAWARRAYSEXT)(GLenum mode, GLint* first, GLsizei* count, GLsizei primcount);
typedef void (APIENTRY* PFNGLMULTIDRAWELEMENTSEXT)(GLenum mode, GLsizei* count, GLenum type, const GLvoid* *indices, GLsizei primcount);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLMULTIDRAWARRAYSEXT glMultiDrawArraysEXT;
VW_OPENGL_API extern PFNGLMULTIDRAWELEMENTSEXT glMultiDrawElementsEXT;


//====================================================================
// GL_EXT_point_parameters
//====================================================================
#ifndef GL_EXT_point_parameters
#define GL_EXT_point_parameters    1


//--------------------------------------------------------------------
#define GL_POINT_SIZE_MIN_EXT                                   0x8126
#define GL_POINT_SIZE_MAX_EXT                                   0x8127
#define GL_POINT_FADE_THRESHOLD_SIZE_EXT                        0x8128
#define GL_DISTANCE_ATTENUATION_EXT                             0x8129


//--------------------------------------------------------------------
#endif //!GL_EXT_point_parameters


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLPOINTPARAMETERFEXT)(GLenum pname, GLfloat param);
typedef void (APIENTRY* PFNGLPOINTPARAMETERFVEXT)(GLenum pname, const GLfloat* params);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLPOINTPARAMETERFEXT glPointParameterfEXT;
VW_OPENGL_API extern PFNGLPOINTPARAMETERFVEXT glPointParameterfvEXT;


//====================================================================
// GL_EXT_secondary_color
//====================================================================
#ifndef GL_EXT_secondary_color
#define GL_EXT_secondary_color    1


//--------------------------------------------------------------------
#define GL_COLOR_SUM_EXT                                        0x8458
#define GL_CURRENT_SECONDARY_COLOR_EXT                          0x8459
#define GL_SECONDARY_COLOR_ARRAY_SIZE_EXT                       0x845A
#define GL_SECONDARY_COLOR_ARRAY_TYPE_EXT                       0x845B
#define GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT                     0x845C
#define GL_SECONDARY_COLOR_ARRAY_POINTER_EXT                    0x845D
#define GL_SECONDARY_COLOR_ARRAY_EXT                            0x845E


//--------------------------------------------------------------------
#endif //!GL_EXT_secondary_color


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3BEXT)(GLbyte red, GLbyte green, GLbyte blue);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3BVEXT)(const GLbyte* v);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3DEXT)(GLdouble red, GLdouble green, GLdouble blue);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3DVEXT)(const GLdouble* v);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3FEXT)(GLfloat red, GLfloat green, GLfloat blue);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3FVEXT)(const GLfloat* v);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3IEXT)(GLint red, GLint green, GLint blue);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3IVEXT)(const GLint* v);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3SEXT)(GLshort red, GLshort green, GLshort blue);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3SVEXT)(const GLshort* v);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3UBEXT)(GLubyte red, GLubyte green, GLubyte blue);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3UBVEXT)(const GLubyte* v);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3UIEXT)(GLuint red, GLuint green, GLuint blue);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3UIVEXT)(const GLuint* v);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3USEXT)(GLushort red, GLushort green, GLushort blue);
typedef void (APIENTRY* PFNGLSECONDARYCOLOR3USVEXT)(const GLushort* v);
typedef void (APIENTRY* PFNGLSECONDARYCOLORPOINTEREXT)(GLint size, GLenum type, GLsizei stride, GLvoid* pointer);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3BEXT glSecondaryColor3bEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3BVEXT glSecondaryColor3bvEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3DEXT glSecondaryColor3dEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3DVEXT glSecondaryColor3dvEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3FEXT glSecondaryColor3fEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3FVEXT glSecondaryColor3fvEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3IEXT glSecondaryColor3iEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3IVEXT glSecondaryColor3ivEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3SEXT glSecondaryColor3sEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3SVEXT glSecondaryColor3svEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3UBEXT glSecondaryColor3ubEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3UBVEXT glSecondaryColor3ubvEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3UIEXT glSecondaryColor3uiEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3UIVEXT glSecondaryColor3uivEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3USEXT glSecondaryColor3usEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLOR3USVEXT glSecondaryColor3usvEXT;
VW_OPENGL_API extern PFNGLSECONDARYCOLORPOINTEREXT glSecondaryColorPointerEXT;


//====================================================================
// GL_EXT_stencil_wrap
//====================================================================
#ifndef GL_EXT_stencil_wrap
#define GL_EXT_stencil_wrap    1


//--------------------------------------------------------------------
#define GL_INCR_WRAP_EXT                                        0x8507
#define GL_DECR_WRAP_EXT                                        0x8508


//--------------------------------------------------------------------
#endif //!GL_EXT_stencil_wrap


//====================================================================
// GL_EXT_texture_filter_anisotropic
//====================================================================
#ifndef GL_EXT_texture_filter_anisotropic
#define GL_EXT_texture_filter_anisotropic    1


//--------------------------------------------------------------------
#define GL_TEXTURE_MAX_ANISOTROPY_EXT                           0x84FE
#define GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT                       0x84FF


//--------------------------------------------------------------------
#endif //!GL_EXT_texture_filter_anisotropic


//====================================================================
// GL_EXT_texture_lod_bias
//====================================================================
#ifndef GL_EXT_texture_lod_bias
#define GL_EXT_texture_lod_bias    1


//--------------------------------------------------------------------
#define GL_TEXTURE_FILTER_CONTROL_EXT                           0x8500
#define GL_TEXTURE_LOD_BIAS_EXT                                 0x8501
#define GL_MAX_TEXTURE_LOD_BIAS_EXT                             0x84FD


//--------------------------------------------------------------------
#endif //!GL_EXT_texture_lod_bias


//====================================================================
// GL_EXT_vertex_weighting
//====================================================================
#ifndef GL_EXT_vertex_weighting
#define GL_EXT_vertex_weighting    1


//--------------------------------------------------------------------
#define GL_MODELVIEW0_STACK_DEPTH_EXT                           0x0BA3
#define GL_MODELVIEW1_STACK_DEPTH_EXT                           0x8502
#define GL_MODELVIEW0_MATRIX_EXT                                0x0BA6
#define GL_MODELVIEW1_MATRIX_EXT                                0x8506
#define GL_VERTEX_WEIGHTING_EXT                                 0x8509
#define GL_MODELVIEW0_EXT                                       0x1700
#define GL_MODELVIEW1_EXT                                       0x850A
#define GL_CURRENT_VERTEX_WEIGHT_EXT                            0x850B
#define GL_VERTEX_WEIGHT_ARRAY_EXT                              0x850C
#define GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT                         0x850D
#define GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT                         0x850E
#define GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT                       0x850F
#define GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT                      0x8510


//--------------------------------------------------------------------
#endif //!GL_EXT_vertex_weighting


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLVERTEXWEIGHTFEXT)(GLfloat weight);
typedef void (APIENTRY* PFNGLVERTEXWEIGHTFVEXT)(const GLfloat* weight);
typedef void (APIENTRY* PFNGLVERTEXWEIGHTPOINTEREXT)(GLsizei size, GLenum type, GLsizei stride, const GLvoid* pointer);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLVERTEXWEIGHTFEXT glVertexWeightfEXT;
VW_OPENGL_API extern PFNGLVERTEXWEIGHTFVEXT glVertexWeightfvEXT;
VW_OPENGL_API extern PFNGLVERTEXWEIGHTPOINTEREXT glVertexWeightPointerEXT;


//====================================================================
// GL_ATI_pn_triangles
//====================================================================
#ifndef GL_ATI_pn_triangles
#define GL_ATI_pn_triangles    1


//--------------------------------------------------------------------
#define GL_PN_TRIANGLES_ATI                                     0x87F0
#define GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI               0x87F1
#define GL_PN_TRIANGLES_POINT_MODE_ATI                          0x87F2
#define GL_PN_TRIANGLES_NORMAL_MODE_ATI                         0x87F3
#define GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI                   0x87F4
#define GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI                   0x87F5
#define GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI                    0x87F6
#define GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI                  0x87F7
#define GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI               0x87F8


//--------------------------------------------------------------------
#endif //!GL_ATI_pn_triangles


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLPNTRIANGLESIATI)(GLenum pname, GLint param);
typedef void (APIENTRY* PFNGLPNTRIANGLESFATI)(GLenum pname, GLfloat param);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLPNTRIANGLESIATI glPNTrianglesiATI;
VW_OPENGL_API extern PFNGLPNTRIANGLESFATI glPNTrianglesfATI;


//====================================================================
// GL_NV_blend_square
//====================================================================


//====================================================================
// GL_NV_copy_depth_to_color
//====================================================================
#ifndef GL_NV_copy_depth_to_color
#define GL_NV_copy_depth_to_color    1


//--------------------------------------------------------------------
#define GL_DEPTH_STENCIL_TO_RGBA_NV                             0x886E
#define GL_DEPTH_STENCIL_TO_BGRA_NV                             0x886F


//--------------------------------------------------------------------
#endif //!GL_EXT_copy_depth_to_color


//====================================================================
// GL_NV_evaluators
//====================================================================
#ifndef GL_NV_evaluators
#define GL_NV_evaluators    1


//--------------------------------------------------------------------
#define GL_EVAL_2D_NV                                           0x86C0
#define GL_EVAL_TRIANGULAR_2D_NV                                0x86C1
#define GL_MAP_TESSELLATION_NV                                  0x86C2
#define GL_MAP_ATTRIB_U_ORDER_NV                                0x86C3
#define GL_MAP_ATTRIB_V_ORDER_NV                                0x86C4
#define GL_EVAL_FRACTIONAL_TESSELLATION_NV                      0x86C5
#define GL_EVAL_VERTEX_ATTRIB0_NV                               0x86C6
#define GL_EVAL_VERTEX_ATTRIB1_NV                               0x86C7
#define GL_EVAL_VERTEX_ATTRIB2_NV                               0x86C8
#define GL_EVAL_VERTEX_ATTRIB3_NV                               0x86C9
#define GL_EVAL_VERTEX_ATTRIB4_NV                               0x86CA
#define GL_EVAL_VERTEX_ATTRIB5_NV                               0x86CB
#define GL_EVAL_VERTEX_ATTRIB6_NV                               0x86CC
#define GL_EVAL_VERTEX_ATTRIB7_NV                               0x86CD
#define GL_EVAL_VERTEX_ATTRIB8_NV                               0x86CE
#define GL_EVAL_VERTEX_ATTRIB9_NV                               0x86CF
#define GL_EVAL_VERTEX_ATTRIB10_NV                              0x86D0
#define GL_EVAL_VERTEX_ATTRIB11_NV                              0x86D1
#define GL_EVAL_VERTEX_ATTRIB12_NV                              0x86D2
#define GL_EVAL_VERTEX_ATTRIB13_NV                              0x86D3
#define GL_EVAL_VERTEX_ATTRIB14_NV                              0x86D4
#define GL_EVAL_VERTEX_ATTRIB15_NV                              0x86D5
#define GL_MAX_MAP_TESSELLATION_NV                              0x86D6
#define GL_MAX_RATIONAL_EVAL_ORDER_NV                           0x86D7


//--------------------------------------------------------------------
#endif //!GL_NV_evaluators


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLMAPCONTROLPOINTSNV)(GLenum target, GLuint index, GLenum type, GLsizei ustride, GLsizei vstride, GLint uorder, GLint vorder, GLboolean packed, const GLvoid* points);
typedef void (APIENTRY* PFNGLMAPPARAMETERIVNV)(GLenum target, GLenum pname, const GLint* params);
typedef void (APIENTRY* PFNGLMAPPARAMETERFVNV)(GLenum target, GLenum pname, const GLfloat* params);
typedef void (APIENTRY* PFNGLGETMAPCONTROLPOINTSNV)(GLenum target, GLuint index, GLenum type, GLsizei ustride, GLsizei vstride, GLboolean packed, GLvoid* points);
typedef void (APIENTRY* PFNGLGETMAPPARAMETERIVNV)(GLenum target, GLenum pname, GLint* params);
typedef void (APIENTRY* PFNGLGETMAPPARAMETERFVNV)(GLenum target, GLenum pname, GLfloat* params);
typedef void (APIENTRY* PFNGLGETMAPATTRIBPARAMETERIVNV)(GLenum target, GLuint index, GLenum pname, GLint* params);
typedef void (APIENTRY* PFNGLGETMAPATTRIBPARAMETERFVNV)(GLenum target, GLuint index, GLenum pname, GLfloat* params);
typedef void (APIENTRY* PFNGLEVALMAPSNV)(GLenum target, GLenum mode);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLMAPCONTROLPOINTSNV glMapControlPointsNV;
VW_OPENGL_API extern PFNGLMAPPARAMETERIVNV glMapParameterivNV;
VW_OPENGL_API extern PFNGLMAPPARAMETERFVNV glMapParameterfvNV;
VW_OPENGL_API extern PFNGLGETMAPCONTROLPOINTSNV glGetMapControlPointsNV;
VW_OPENGL_API extern PFNGLGETMAPPARAMETERIVNV glGetMapParameterivNV;
VW_OPENGL_API extern PFNGLGETMAPPARAMETERFVNV glGetMapParameterfvNV;
VW_OPENGL_API extern PFNGLGETMAPATTRIBPARAMETERIVNV glGetMapAttribParameterivNV;
VW_OPENGL_API extern PFNGLGETMAPATTRIBPARAMETERFVNV glGetMapAttribParameterfvNV;
VW_OPENGL_API extern PFNGLEVALMAPSNV glEvalMapsNV;


//====================================================================
// GL_NV_fence
//====================================================================
#ifndef GL_NV_fence
#define GL_NV_fence    1


//--------------------------------------------------------------------
#define GL_ALL_COMPLETED_NV                                     0x84F2
#define GL_FENCE_STATUS_NV                                      0x84F3
#define GL_FENCE_CONDITION_NV                                   0x84F4


//--------------------------------------------------------------------
#endif //!GL_NV_fence


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLGENFENCESNV)(GLsizei n, GLuint* fences);
typedef void (APIENTRY* PFNGLDELETEFENCESNV)(GLsizei n, const GLuint* fences);
typedef void (APIENTRY* PFNGLSETFENCENV)(GLuint fence, GLenum condition);
typedef GLboolean (APIENTRY* PFNGLTESTFENCENV)(GLuint fence);
typedef void (APIENTRY* PFNGLFINISHFENCENV)(GLuint fence);
typedef GLboolean (APIENTRY* PFNGLISFENCENV)(GLuint fence);
typedef void (APIENTRY* PFNGLGETFENCEIVNV)(GLuint fence, GLenum pname, GLint* params);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLGENFENCESNV glGenFencesNV;
VW_OPENGL_API extern PFNGLDELETEFENCESNV glDeleteFencesNV;
VW_OPENGL_API extern PFNGLSETFENCENV glSetFenceNV;
VW_OPENGL_API extern PFNGLTESTFENCENV glTestFenceNV;
VW_OPENGL_API extern PFNGLFINISHFENCENV glFinishFenceNV;
VW_OPENGL_API extern PFNGLISFENCENV glIsFenceNV;
VW_OPENGL_API extern PFNGLGETFENCEIVNV glGetFenceivNV;


//====================================================================
// GL_NV_fog_distance
//====================================================================
#ifndef GL_NV_fog_distance
#define GL_NV_fog_distance    1


//--------------------------------------------------------------------
#define GL_FOG_DISTANCE_MODE_NV                                 0x855A
#define GL_EYE_RADIAL_NV                                        0x855B
#define GL_EYE_PLANE_ABSOLUTE_NV                                0x855C


//--------------------------------------------------------------------
#endif //!GL_NV_fog_distance


//====================================================================
// GL_NV_light_max_exponent
//====================================================================
#ifndef GL_NV_light_max_exponent
#define GL_NV_light_max_exponent    1


//--------------------------------------------------------------------
#define GL_MAX_SHININESS_NV                                     0x8504
#define GL_MAX_SPOT_EXPONENT_NV                                 0x8505


//--------------------------------------------------------------------
#endif //!GL_NV_light_max_exponent


//====================================================================
// GL_NV_packed_depth_stencil
//====================================================================
#ifndef GL_NV_packed_depth_stencil
#define GL_NV_packed_depth_stencil    1


//--------------------------------------------------------------------
#define GL_DEPTH_STENCIL_NV                                     0x84F9
#define GL_UNSIGNED_INT_24_8_NV                                 0x84FA


//--------------------------------------------------------------------
#endif //!GL_NV_packed_depth_stencil


//====================================================================
// GL_NV_register_combiners
//====================================================================
#ifndef GL_NV_register_combiners
#define GL_NV_register_combiners    1


//--------------------------------------------------------------------
#define GL_REGISTER_COMBINERS_NV                                0x8522
#define GL_COMBINER0_NV                                         0x8550
#define GL_COMBINER1_NV                                         0x8551
#define GL_COMBINER2_NV                                         0x8552
#define GL_COMBINER3_NV                                         0x8553
#define GL_COMBINER4_NV                                         0x8554
#define GL_COMBINER5_NV                                         0x8555
#define GL_COMBINER6_NV                                         0x8556
#define GL_COMBINER7_NV                                         0x8557
#define GL_VARIABLE_A_NV                                        0x8523
#define GL_VARIABLE_B_NV                                        0x8524
#define GL_VARIABLE_C_NV                                        0x8525
#define GL_VARIABLE_D_NV                                        0x8526
#define GL_VARIABLE_E_NV                                        0x8527
#define GL_VARIABLE_F_NV                                        0x8528
#define GL_VARIABLE_G_NV                                        0x8529
#define GL_CONSTANT_COLOR0_NV                                   0x852A
#define GL_CONSTANT_COLOR1_NV                                   0x852B
#define GL_PRIMARY_COLOR_NV                                     0x852C
#define GL_SECONDARY_COLOR_NV                                   0x852D
#define GL_SPARE0_NV                                            0x852E
#define GL_SPARE1_NV                                            0x852F
#define GL_UNSIGNED_IDENTITY_NV                                 0x8536
#define GL_UNSIGNED_INVERT_NV                                   0x8537
#define GL_EXPAND_NORMAL_NV                                     0x8538
#define GL_EXPAND_NEGATE_NV                                     0x8539
#define GL_HALF_BIAS_NORMAL_NV                                  0x853A
#define GL_HALF_BIAS_NEGATE_NV                                  0x853B
#define GL_SIGNED_IDENTITY_NV                                   0x853C
#define GL_SIGNED_NEGATE_NV                                     0x853D
#define GL_E_TIMES_F_NV                                         0x8531
#define GL_SPARE0_PLUS_SECONDARY_COLOR_NV                       0x8532
#define GL_SCALE_BY_TWO_NV                                      0x853E
#define GL_SCALE_BY_FOUR_NV                                     0x853F
#define GL_SCALE_BY_ONE_HALF_NV                                 0x8540
#define GL_BIAS_BY_NEGATIVE_ONE_HALF_NV                         0x8541
#define GL_DISCARD_NV                                           0x8530
#define GL_COMBINER_INPUT_NV                                    0x8542
#define GL_COMBINER_MAPPING_NV                                  0x8543
#define GL_COMBINER_COMPONENT_USAGE_NV                          0x8544
#define GL_COMBINER_AB_DOT_PRODUCT_NV                           0x8545
#define GL_COMBINER_CD_DOT_PRODUCT_NV                           0x8546
#define GL_COMBINER_MUX_SUM_NV                                  0x8547
#define GL_COMBINER_SCALE_NV                                    0x8548
#define GL_COMBINER_BIAS_NV                                     0x8549
#define GL_COMBINER_AB_OUTPUT_NV                                0x854A
#define GL_COMBINER_CD_OUTPUT_NV                                0x854B
#define GL_COMBINER_SUM_OUTPUT_NV                               0x854C
#define GL_NUM_GENERAL_COMBINERS_NV                             0x854E
#define GL_COLOR_SUM_CLAMP_NV                                   0x854F
#define GL_MAX_GENERAL_COMBINERS_NV                             0x854D


//--------------------------------------------------------------------
#endif //!GL_NV_register_combiners


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLCOMBINERPARAMETERFVNV)(GLenum pname, const GLfloat* params);
typedef void (APIENTRY* PFNGLCOMBINERPARAMETERFNV)(GLenum pname, GLfloat param);
typedef void (APIENTRY* PFNGLCOMBINERPARAMETERIVNV)(GLenum pname, const GLint* params);
typedef void (APIENTRY* PFNGLCOMBINERPARAMETERINV)(GLenum pname, GLint param);
typedef void (APIENTRY* PFNGLCOMBINERINPUTNV)(GLenum stage, GLenum portion, GLenum variable, GLenum input, GLenum mapping, GLenum componentUsage);
typedef void (APIENTRY* PFNGLCOMBINEROUTPUTNV)(GLenum stage, GLenum portion, GLenum abOutput, GLenum cdOutput, GLenum sumOutput, GLenum scale, GLenum bias, GLboolean abDotProduct, GLboolean cdDotProduct, GLboolean muxSum);
typedef void (APIENTRY* PFNGLFINALCOMBINERINPUTNV)(GLenum variable, GLenum input, GLenum mapping, GLenum componentUsage);
typedef void (APIENTRY* PFNGLGETCOMBINERINPUTPARAMETERFVNV)(GLenum stage, GLenum portion, GLenum variable, GLenum pname, GLfloat* params);
typedef void (APIENTRY* PFNGLGETCOMBINERINPUTPARAMETERIVNV)(GLenum stage, GLenum portion, GLenum variable, GLenum pname, GLint* params);
typedef void (APIENTRY* PFNGLGETCOMBINEROUTPUTPARAMETERFVNV)(GLenum stage, GLenum portion, GLenum pname, GLfloat* params);
typedef void (APIENTRY* PFNGLGETCOMBINEROUTPUTPARAMETERIVNV)(GLenum stage, GLenum portion, GLenum pname, GLint* params);
typedef void (APIENTRY* PFNGLGETFINALCOMBINERINPUTPARAMETERFVNV)(GLenum variable, GLenum pname, GLfloat* params);
typedef void (APIENTRY* PFNGLGETFINALCOMBINERINPUTPARAMETERIVNV)(GLenum variable, GLenum pname, GLint* params);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLCOMBINERPARAMETERFVNV glCombinerParameterfvNV;
VW_OPENGL_API extern PFNGLCOMBINERPARAMETERFNV glCombinerParameterfNV;
VW_OPENGL_API extern PFNGLCOMBINERPARAMETERIVNV glCombinerParameterivNV;
VW_OPENGL_API extern PFNGLCOMBINERPARAMETERINV glCombinerParameteriNV;
VW_OPENGL_API extern PFNGLCOMBINERINPUTNV glCombinerInputNV;
VW_OPENGL_API extern PFNGLCOMBINEROUTPUTNV glCombinerOutputNV;
VW_OPENGL_API extern PFNGLFINALCOMBINERINPUTNV glFinalCombinerInputNV;
VW_OPENGL_API extern PFNGLGETCOMBINERINPUTPARAMETERFVNV glGetCombinerInputParameterfvNV;
VW_OPENGL_API extern PFNGLGETCOMBINERINPUTPARAMETERIVNV glGetCombinerInputParameterivNV;
VW_OPENGL_API extern PFNGLGETCOMBINEROUTPUTPARAMETERFVNV glGetCombinerOutputParameterfvNV;
VW_OPENGL_API extern PFNGLGETCOMBINEROUTPUTPARAMETERIVNV glGetCombinerOutputParameterivNV;
VW_OPENGL_API extern PFNGLGETFINALCOMBINERINPUTPARAMETERFVNV glGetFinalCombinerInputParameterfvNV;
VW_OPENGL_API extern PFNGLGETFINALCOMBINERINPUTPARAMETERIVNV glGetFinalCombinerInputParameterivNV;


//====================================================================
// GL_NV_register_combiners2
//====================================================================
#ifndef GL_NV_register_combiners2
#define GL_NV_register_combiners2    1


//--------------------------------------------------------------------
#define GL_PER_STAGE_CONSTANTS_NV                               0x8535


//--------------------------------------------------------------------
#endif //!GL_NV_register_combiners2


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLCOMBINERSTAGEPARAMETERFVNV)(GLenum stage, GLenum pname, const GLfloat* params);
typedef void (APIENTRY* PFNGLGETCOMBINERSTAGEPARAMETERFVNV)(GLenum stage, GLenum pname, GLfloat* params);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLCOMBINERSTAGEPARAMETERFVNV glCombinerStageParameterfvNV;
VW_OPENGL_API extern PFNGLGETCOMBINERSTAGEPARAMETERFVNV glGetCombinerStageParameterfvNV;


//====================================================================
// GL_NV_texgen_reflection
//====================================================================
#ifndef GL_NV_texgen_reflection
#define GL_NV_texgen_reflection    1


//--------------------------------------------------------------------
#define GL_NORMAL_MAP_NV                                        0x8511
#define GL_REFLECTION_MAP_NV                                    0x8512


//--------------------------------------------------------------------
#endif //!GL_NV_texgen_reflection


//====================================================================
// GL_NV_texture_env_combine4
//====================================================================
#ifndef GL_NV_texture_env_combine4
#define GL_NV_texture_env_combine4    1


//--------------------------------------------------------------------
#define GL_COMBINE4_NV                                          0x8503
#define GL_SOURCE3_RGB_NV                                       0x8583
#define GL_SOURCE3_ALPHA_NV                                     0x858B
#define GL_OPERAND3_RGB_NV                                      0x8593
#define GL_OPERAND3_ALPHA_NV                                    0x859B


//--------------------------------------------------------------------
#endif //!GL_NV_texture_env_combine4


//====================================================================
// GL_NV_texture_rectangle
//====================================================================
#ifndef GL_NV_texture_rectangle
#define GL_NV_texture_rectangle    1


//--------------------------------------------------------------------
#define GL_TEXTURE_RECTANGLE_NV                                 0x84F5
#define GL_TEXTURE_BINDING_RECTANGLE_NV                         0x84F6
#define GL_PROXY_TEXTURE_RECTANGLE_NV                           0x84F7
#define GL_MAX_RECTANGLE_TEXTURE_SIZE_NV                        0x84F8


//--------------------------------------------------------------------
#endif //!GL_NV_texture_rectangle


//====================================================================
// GL_NV_texture_shader
//====================================================================
#ifndef GL_NV_texture_shader
#define GL_NV_texture_shader    1


//--------------------------------------------------------------------
#define GL_TEXTURE_SHADER_NV                                    0x86DE
#define GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV                 0x86D9
#define GL_SHADER_OPERATION_NV                                  0x86DF
#define GL_CULL_MODES_NV                                        0x86E0
#define GL_OFFSET_TEXTURE_MATRIX_NV                             0x86E1
#define GL_OFFSET_TEXTURE_SCALE_NV                              0x86E2
#define GL_OFFSET_TEXTURE_BIAS_NV                               0x86E3
#define GL_PREVIOUS_TEXTURE_INPUT_NV                            0x86E4
#define GL_CONST_EYE_NV                                         0x86E5
#define GL_SHADER_CONSISTENT_NV                                 0x86DD
#define GL_PASS_THROUGH_NV                                      0x86E6
#define GL_CULL_FRAGMENT_NV                                     0x86E7
#define GL_OFFSET_TEXTURE_2D_NV                                 0x86E8
#define GL_OFFSET_TEXTURE_RECTANGLE_NV                          0x864C
#define GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV                    0x864D
#define GL_DEPENDENT_AR_TEXTURE_2D_NV                           0x86E9
#define GL_DEPENDENT_GB_TEXTURE_2D_NV                           0x86EA
#define GL_DOT_PRODUCT_NV                                       0x86EC
#define GL_DOT_PRODUCT_DEPTH_REPLACE_NV                         0x86ED
#define GL_DOT_PRODUCT_TEXTURE_2D_NV                            0x86EE
#define GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV                     0x864E
#define GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV                      0x86F0
#define GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV                      0x86F1
#define GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV                      0x86F2
#define GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV            0x86F3
#define GL_HILO_NV                                              0x86F4
#define GL_DSDT_NV                                              0x86F5
#define GL_DSDT_MAG_NV                                          0x86F6
#define GL_DSDT_MAG_VIB_NV                                      0x86F7
#define GL_UNSIGNED_INT_S8_S8_8_8_NV                            0x86DA
#define GL_UNSIGNED_INT_8_8_S8_S8_REV_NV                        0x86DB
#define GL_SIGNED_RGBA_NV                                       0x86FB
#define GL_SIGNED_RGBA8_NV                                      0x86FC
#define GL_SIGNED_RGB_NV                                        0x86FE
#define GL_SIGNED_RGB8_NV                                       0x86FF
#define GL_SIGNED_LUMINANCE_NV                                  0x8701
#define GL_SIGNED_LUMINANCE8_NV                                 0x8702
#define GL_SIGNED_LUMINANCE_ALPHA_NV                            0x8703
#define GL_SIGNED_LUMINANCE8_ALPHA8_NV                          0x8704
#define GL_SIGNED_ALPHA_NV                                      0x8705
#define GL_SIGNED_ALPHA8_NV                                     0x8706
#define GL_SIGNED_INTENSITY_NV                                  0x8707
#define GL_SIGNED_INTENSITY8_NV                                 0x8708
#define GL_SIGNED_RGB_UNSIGNED_ALPHA_NV                         0x870C
#define GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV                       0x870D
#define GL_HILO16_NV                                            0x86F8
#define GL_SIGNED_HILO_NV                                       0x86F9
#define GL_SIGNED_HILO16_NV                                     0x86FA
#define GL_DSDT8_NV                                             0x8709
#define GL_DSDT8_MAG8_NV                                        0x870A
#define GL_DSDT_MAG_INTENSITY_NV                                0x86DC
#define GL_DSDT8_MAG8_INTENSITY8_NV                             0x870B
#define GL_HI_SCALE_NV                                          0x870E
#define GL_LO_SCALE_NV                                          0x870F
#define GL_DS_SCALE_NV                                          0x8710
#define GL_DT_SCALE_NV                                          0x8711
#define GL_MAGNITUDE_SCALE_NV                                   0x8712
#define GL_VIBRANCE_SCALE_NV                                    0x8713
#define GL_HI_BIAS_NV                                           0x8714
#define GL_LO_BIAS_NV                                           0x8715
#define GL_DS_BIAS_NV                                           0x8716
#define GL_DT_BIAS_NV                                           0x8717
#define GL_MAGNITUDE_BIAS_NV                                    0x8718
#define GL_VIBRANCE_BIAS_NV                                     0x8719
#define GL_TEXTURE_BORDER_VALUES_NV                             0x871A
#define GL_TEXTURE_HI_SIZE_NV                                   0x871B
#define GL_TEXTURE_LO_SIZE_NV                                   0x871C
#define GL_TEXTURE_DS_SIZE_NV                                   0x871D
#define GL_TEXTURE_DT_SIZE_NV                                   0x871E
#define GL_TEXTURE_MAG_SIZE_NV                                  0x871F


//--------------------------------------------------------------------
#endif //!GL_NV_texture_shader


//====================================================================
// GL_NV_texture_shader2
//====================================================================
#ifndef GL_NV_texture_shader2
#define GL_NV_texture_shader2    1


//--------------------------------------------------------------------
#define GL_DOT_PRODUCT_TEXTURE_3D_NV                            0x86EF
#define GL_HILO_NV                                              0x86F4
#define GL_DSDT_NV                                              0x86F5
#define GL_DSDT_MAG_NV                                          0x86F6
#define GL_DSDT_MAG_VIB_NV                                      0x86F7
#define GL_UNSIGNED_INT_S8_S8_8_8_NV                            0x86DA
#define GL_UNSIGNED_INT_8_8_S8_S8_REV_NV                        0x86DB
#define GL_SIGNED_RGBA_NV                                       0x86FB
#define GL_SIGNED_RGBA8_NV                                      0x86FC
#define GL_SIGNED_RGB_NV                                        0x86FE
#define GL_SIGNED_RGB8_NV                                       0x86FF
#define GL_SIGNED_LUMINANCE_NV                                  0x8701
#define GL_SIGNED_LUMINANCE8_NV                                 0x8702
#define GL_SIGNED_LUMINANCE_ALPHA_NV                            0x8703
#define GL_SIGNED_LUMINANCE8_ALPHA8_NV                          0x8704
#define GL_SIGNED_ALPHA_NV                                      0x8705
#define GL_SIGNED_ALPHA8_NV                                     0x8706
#define GL_SIGNED_INTENSITY_NV                                  0x8707
#define GL_SIGNED_INTENSITY8_NV                                 0x8708
#define GL_SIGNED_RGB_UNSIGNED_ALPHA_NV                         0x870C
#define GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV                       0x870D
#define GL_HILO16_NV                                            0x86F8
#define GL_SIGNED_HILO_NV                                       0x86F9
#define GL_SIGNED_HILO16_NV                                     0x86FA
#define GL_DSDT8_NV                                             0x8709
#define GL_DSDT8_MAG8_NV                                        0x870A
#define GL_DSDT_MAG_INTENSITY_NV                                0x86DC
#define GL_DSDT8_MAG8_INTENSITY8_NV                             0x870B 


//--------------------------------------------------------------------
#endif //!GL_NV_texture_shader2


//====================================================================
// GL_NV_vertex_array_range
//====================================================================
#ifndef GL_NV_vertex_array_range
#define GL_NV_vertex_array_range    1


//--------------------------------------------------------------------
#define GL_VERTEX_ARRAY_RANGE_NV                                0x851D
#define GL_VERTEX_ARRAY_RANGE_LENGTH_NV                         0x851E
#define GL_VERTEX_ARRAY_RANGE_VALID_NV                          0x851F
#define GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV                    0x8520
#define GL_VERTEX_ARRAY_RANGE_POINTER_NV                        0x8521


//--------------------------------------------------------------------
#endif //!GL_NV_vertex_array_range


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLFLUSHVERTEXARRAYRANGENV)(void);
typedef void (APIENTRY* PFNGLVERTEXARRAYRANGENV)(GLsizei size, const GLvoid* pointer);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLFLUSHVERTEXARRAYRANGENV glFlushVertexArrayRangeNV;
VW_OPENGL_API extern PFNGLVERTEXARRAYRANGENV glVertexArrayRangeNV;


//====================================================================
// GL_NV_vertex_array_range2
//====================================================================
#ifndef GL_NV_vertex_array_range2
#define GL_NV_vertex_array_range2    1


//--------------------------------------------------------------------
#define GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV                  0x8533


//--------------------------------------------------------------------
#endif //!GL_NV_vertex_array_range2


//====================================================================
// GL_NV_vertex_program
//====================================================================
#ifndef GL_NV_vertex_program
#define GL_NV_vertex_program    1


//--------------------------------------------------------------------
#define GL_VERTEX_PROGRAM_NV                                    0x8620
#define GL_VERTEX_PROGRAM_POINT_SIZE_NV                         0x8642
#define GL_VERTEX_PROGRAM_TWO_SIDE_NV                           0x8643
#define GL_VERTEX_STATE_PROGRAM_NV                              0x8621
#define GL_ATTRIB_ARRAY_SIZE_NV                                 0x8623
#define GL_ATTRIB_ARRAY_STRIDE_NV                               0x8624
#define GL_ATTRIB_ARRAY_TYPE_NV                                 0x8625
#define GL_CURRENT_ATTRIB_NV                                    0x8626
#define GL_PROGRAM_PARAMETER_NV                                 0x8644
#define GL_ATTRIB_ARRAY_POINTER_NV                              0x8645
#define GL_PROGRAM_TARGET_NV                                    0x8646
#define GL_PROGRAM_LENGTH_NV                                    0x8627
#define GL_PROGRAM_RESIDENT_NV                                  0x8647
#define GL_PROGRAM_STRING_NV                                    0x8628
#define GL_TRACK_MATRIX_NV                                      0x8648
#define GL_TRACK_MATRIX_TRANSFORM_NV                            0x8649
#define GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV                      0x862E
#define GL_MAX_TRACK_MATRICES_NV                                0x862F
#define GL_CURRENT_MATRIX_STACK_DEPTH_NV                        0x8640
#define GL_CURRENT_MATRIX_NV                                    0x8641
#define GL_VERTEX_PROGRAM_BINDING_NV                            0x864A
#define GL_PROGRAM_ERROR_POSITION_NV                            0x864B
#define GL_MODELVIEW_PROJECTION_NV                              0x8629
#define GL_MATRIX0_NV                                           0x8630
#define GL_MATRIX1_NV                                           0x8631
#define GL_MATRIX2_NV                                           0x8632
#define GL_MATRIX3_NV                                           0x8633
#define GL_MATRIX4_NV                                           0x8634
#define GL_MATRIX5_NV                                           0x8635
#define GL_MATRIX6_NV                                           0x8636
#define GL_MATRIX7_NV                                           0x8637
#define GL_IDENTITY_NV                                          0x862A
#define GL_INVERSE_NV                                           0x862B
#define GL_TRANSPOSE_NV                                         0x862C
#define GL_INVERSE_TRANSPOSE_NV                                 0x862D
#define GL_VERTEX_ATTRIB_ARRAY0_NV                              0x8650
#define GL_VERTEX_ATTRIB_ARRAY1_NV                              0x8651
#define GL_VERTEX_ATTRIB_ARRAY2_NV                              0x8652
#define GL_VERTEX_ATTRIB_ARRAY3_NV                              0x8653
#define GL_VERTEX_ATTRIB_ARRAY4_NV                              0x8654
#define GL_VERTEX_ATTRIB_ARRAY5_NV                              0x8655
#define GL_VERTEX_ATTRIB_ARRAY6_NV                              0x8656
#define GL_VERTEX_ATTRIB_ARRAY7_NV                              0x8657
#define GL_VERTEX_ATTRIB_ARRAY8_NV                              0x8658
#define GL_VERTEX_ATTRIB_ARRAY9_NV                              0x8659
#define GL_VERTEX_ATTRIB_ARRAY10_NV                             0x865A
#define GL_VERTEX_ATTRIB_ARRAY11_NV                             0x865B
#define GL_VERTEX_ATTRIB_ARRAY12_NV                             0x865C
#define GL_VERTEX_ATTRIB_ARRAY13_NV                             0x865D
#define GL_VERTEX_ATTRIB_ARRAY14_NV                             0x865E
#define GL_VERTEX_ATTRIB_ARRAY15_NV                             0x865F
#define GL_MAP1_VERTEX_ATTRIB0_4_NV                             0x8660
#define GL_MAP1_VERTEX_ATTRIB1_4_NV                             0x8661
#define GL_MAP1_VERTEX_ATTRIB2_4_NV                             0x8662
#define GL_MAP1_VERTEX_ATTRIB3_4_NV                             0x8663
#define GL_MAP1_VERTEX_ATTRIB4_4_NV                             0x8664
#define GL_MAP1_VERTEX_ATTRIB5_4_NV                             0x8665
#define GL_MAP1_VERTEX_ATTRIB6_4_NV                             0x8666
#define GL_MAP1_VERTEX_ATTRIB7_4_NV                             0x8667
#define GL_MAP1_VERTEX_ATTRIB8_4_NV                             0x8668
#define GL_MAP1_VERTEX_ATTRIB9_4_NV                             0x8669
#define GL_MAP1_VERTEX_ATTRIB10_4_NV                            0x866A
#define GL_MAP1_VERTEX_ATTRIB11_4_NV                            0x866B
#define GL_MAP1_VERTEX_ATTRIB12_4_NV                            0x866C
#define GL_MAP1_VERTEX_ATTRIB13_4_NV                            0x866D
#define GL_MAP1_VERTEX_ATTRIB14_4_NV                            0x866E
#define GL_MAP1_VERTEX_ATTRIB15_4_NV                            0x866F
#define GL_MAP2_VERTEX_ATTRIB0_4_NV                             0x8670
#define GL_MAP2_VERTEX_ATTRIB1_4_NV                             0x8671
#define GL_MAP2_VERTEX_ATTRIB2_4_NV                             0x8672
#define GL_MAP2_VERTEX_ATTRIB3_4_NV                             0x8673
#define GL_MAP2_VERTEX_ATTRIB4_4_NV                             0x8674
#define GL_MAP2_VERTEX_ATTRIB5_4_NV                             0x8675
#define GL_MAP2_VERTEX_ATTRIB6_4_NV                             0x8676
#define GL_MAP2_VERTEX_ATTRIB7_4_NV                             0x8677
#define GL_MAP2_VERTEX_ATTRIB8_4_NV                             0x8678
#define GL_MAP2_VERTEX_ATTRIB9_4_NV                             0x8679
#define GL_MAP2_VERTEX_ATTRIB10_4_NV                            0x867A
#define GL_MAP2_VERTEX_ATTRIB11_4_NV                            0x867B
#define GL_MAP2_VERTEX_ATTRIB12_4_NV                            0x867C
#define GL_MAP2_VERTEX_ATTRIB13_4_NV                            0x867D
#define GL_MAP2_VERTEX_ATTRIB14_4_NV                            0x867E
#define GL_MAP2_VERTEX_ATTRIB15_4_NV                            0x867F


//--------------------------------------------------------------------
#endif //!GL_NV_vertex_program


//--------------------------------------------------------------------
typedef void (APIENTRY* PFNGLBINDPROGRAMNV)(GLenum target, GLuint id);
typedef void (APIENTRY* PFNGLDELETEPROGRAMSNV)(GLsizei n, const GLuint* ids);
typedef void (APIENTRY* PFNGLEXECUTEPROGRAMNV)(GLenum target, GLuint id, const GLfloat* params);
typedef void (APIENTRY* PFNGLGENPROGRAMSNV)(GLsizei n, GLuint* ids);
typedef GLboolean (APIENTRY* PFNGLAREPROGRAMSRESIDENTNV)(GLsizei n, const GLuint* ids, GLboolean* residences);
typedef void (APIENTRY* PFNGLREQUESTRESIDENTPROGRAMSNV)(GLsizei n, GLuint* ids);
typedef void (APIENTRY* PFNGLGETPROGRAMPARAMETERFVNV)(GLenum target, GLuint index, GLenum pname, GLfloat* params);
typedef void (APIENTRY* PFNGLGETPROGRAMPARAMETERDVNV)(GLenum target, GLuint index, GLenum pname, GLdouble* params);
typedef void (APIENTRY* PFNGLGETPROGRAMIVNV)(GLuint id, GLenum pname, GLint* params);
typedef void (APIENTRY* PFNGLGETPROGRAMSTRINGNV)(GLuint id, GLenum pname, GLubyte* program);
typedef void (APIENTRY* PFNGLGETTRACKMATRIXIVNV)(GLenum target, GLuint address, GLenum pname, GLint* params);
typedef void (APIENTRY* PFNGLGETVERTEXATTRIBDVNV)(GLuint index, GLenum pname, GLdouble* params);
typedef void (APIENTRY* PFNGLGETVERTEXATTRINFVNV)(GLuint index, GLenum pname, GLfloat* params);
typedef void (APIENTRY* PFNGLGETVERTEXATTRIBIVNV)(GLuint index, GLenum pname, GLint* params);
typedef void (APIENTRY* PFNGLGETVERTEXATTRIBPOINTERVNV)(GLuint index, GLenum pname, GLvoid* *pointer);
typedef GLboolean (APIENTRY* PFNGLISPROGRAMNV)(GLuint id);
typedef void (APIENTRY* PFNGLLOADPROGRAMNV)(GLenum target, GLuint id, GLsizei len, const GLubyte* program);
typedef void (APIENTRY* PFNGLPROGRAMPARAMETER4FNV)(GLenum target, GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w);
typedef void (APIENTRY* PFNGLPROGRAMPARAMETER4DNV)(GLenum target, GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w);
typedef void (APIENTRY* PFNGLPROGRAMPARAMETER4DVNV)(GLenum target, GLuint index, const GLdouble* params);
typedef void (APIENTRY* PFNGLPROGRAMPARAMETER4FVNV)(GLenum target, GLuint index, const GLfloat* params);
typedef void (APIENTRY* PFNGLPROGRAMPARAMETERS4DVNV)(GLenum target, GLuint index, GLuint num, const GLdouble* params);
typedef void (APIENTRY* PFNGLPROGRAMPARAMETERS4FVNV)(GLenum target, GLuint index, GLuint num, const GLfloat* params);
typedef void (APIENTRY* PFNGLTRACKMATRIXNV)(GLenum target, GLuint address, GLenum matrix, GLenum transform);
typedef void (APIENTRY* PFNGLVERTEXATTRIBPPOINTERNV)(GLuint index, GLint size, GLenum type, GLsizei stride, const GLvoid* pointer);
typedef void (APIENTRY* PFNGLVERTEXATTRIB1SNV)(GLuint index, GLshort x);
typedef void (APIENTRY* PFNGLVERTEXATTRIB1FNV)(GLuint index, GLfloat x);
typedef void (APIENTRY* PFNGLVERTEXATTRIB1DNV)(GLuint index, GLdouble x);
typedef void (APIENTRY* PFNGLVERTEXATTRIB2SNV)(GLuint index, GLshort x, GLshort y);
typedef void (APIENTRY* PFNGLVERTEXATTRIB2FNV)(GLuint index, GLfloat x, GLfloat y);
typedef void (APIENTRY* PFNGLVERTEXATTRIB2DNV)(GLuint index, GLdouble x, GLdouble y);
typedef void (APIENTRY* PFNGLVERTEXATTRIB3SNV)(GLuint index, GLshort x, GLshort y, GLshort z);
typedef void (APIENTRY* PFNGLVERTEXATTRIB3FNV)(GLuint index, GLfloat x, GLfloat y, GLfloat z);
typedef void (APIENTRY* PFNGLVERTEXATTRIB3DNV)(GLuint index, GLdouble x, GLdouble y, GLdouble z);
typedef void (APIENTRY* PFNGLVERTEXATTRIB4SNV)(GLuint index, GLshort x, GLshort y, GLshort z, GLshort w);
typedef void (APIENTRY* PFNGLVERTEXATTRIB4FNV)(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w);
typedef void (APIENTRY* PFNGLVERTEXATTRIB4DNV)(GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w);
typedef void (APIENTRY* PFNGLVERTEXATTRIB4UBNV)(GLuint index, GLubyte x, GLubyte y, GLubyte z, GLubyte w);
typedef void (APIENTRY* PFNGLVERTEXATTRIB1SVNV)(GLuint index, const GLshort* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB1FVNV)(GLuint index, const GLfloat* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB1DVNV)(GLuint index, const GLdouble* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB2SVNV)(GLuint index, const GLshort* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB2FVNV)(GLuint index, const GLfloat* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB2DVNV)(GLuint index, const GLdouble* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB3SVNV)(GLuint index, const GLshort* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB3FVNV)(GLuint index, const GLfloat* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB3DVNV)(GLuint index, const GLdouble* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB4SVNV)(GLuint index, const GLshort* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB4FVNV)(GLuint index, const GLfloat* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB4DVNV)(GLuint index, const GLdouble* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIB4UBVNV)(GLuint index, const GLubyte* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS1SVNV)(GLuint index, GLsizei n, const GLshort* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS1FVNV)(GLuint index, GLsizei n, const GLfloat* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS1DVNV)(GLuint index, GLsizei n, const GLdouble* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS2SVNV)(GLuint index, GLsizei n, const GLshort* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS2FVNV)(GLuint index, GLsizei n, const GLfloat* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS2DVNV)(GLuint index, GLsizei n, const GLdouble* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS3SVNV)(GLuint index, GLsizei n, const GLshort* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS3FVNV)(GLuint index, GLsizei n, const GLfloat* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS3DVNV)(GLuint index, GLsizei n, const GLdouble* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS4SVNV)(GLuint index, GLsizei n, const GLshort* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS4FVNV)(GLuint index, GLsizei n, const GLfloat* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS4DVNV)(GLuint index, GLsizei n, const GLdouble* v);
typedef void (APIENTRY* PFNGLVERTEXATTRIBS4UBVNV)(GLuint index, GLsizei n, const GLubyte* v);


//--------------------------------------------------------------------
VW_OPENGL_API extern PFNGLBINDPROGRAMNV glBindProgramNV;
VW_OPENGL_API extern PFNGLDELETEPROGRAMSNV glDeleteProgramsNV;
VW_OPENGL_API extern PFNGLEXECUTEPROGRAMNV glExecuteProgramNV;
VW_OPENGL_API extern PFNGLGENPROGRAMSNV glGenProgramsNV;
VW_OPENGL_API extern PFNGLAREPROGRAMSRESIDENTNV glAreProgramsResidentNV;
VW_OPENGL_API extern PFNGLREQUESTRESIDENTPROGRAMSNV glRequestResidentProgramsNV;
VW_OPENGL_API extern PFNGLGETPROGRAMPARAMETERFVNV glGetProgramParameterfvNV;
VW_OPENGL_API extern PFNGLGETPROGRAMPARAMETERDVNV glGetProgramParameterdvNV;
VW_OPENGL_API extern PFNGLGETPROGRAMIVNV glGetProgramivNV;
VW_OPENGL_API extern PFNGLGETPROGRAMSTRINGNV glGetProgramStringNV;
VW_OPENGL_API extern PFNGLGETTRACKMATRIXIVNV glGetTrackMatrixivNV;
VW_OPENGL_API extern PFNGLGETVERTEXATTRIBDVNV glGetVertexAttribdvNV;
VW_OPENGL_API extern PFNGLGETVERTEXATTRINFVNV glGetVertexAttribfvNV;
VW_OPENGL_API extern PFNGLGETVERTEXATTRIBIVNV glGetVertexAttribivNV;
VW_OPENGL_API extern PFNGLGETVERTEXATTRIBPOINTERVNV glGetVertexAttribPointervNV;
VW_OPENGL_API extern PFNGLISPROGRAMNV glIsProgramNV;
VW_OPENGL_API extern PFNGLLOADPROGRAMNV glLoadProgramNV;
VW_OPENGL_API extern PFNGLPROGRAMPARAMETER4FNV glProgramParameter4fNV;
VW_OPENGL_API extern PFNGLPROGRAMPARAMETER4DNV glProgramParameter4dNV;
VW_OPENGL_API extern PFNGLPROGRAMPARAMETER4DVNV glProgramParameter4dvNV;
VW_OPENGL_API extern PFNGLPROGRAMPARAMETER4FVNV glProgramParameter4fvNV;
VW_OPENGL_API extern PFNGLPROGRAMPARAMETERS4DVNV glProgramParameters4dvNV;
VW_OPENGL_API extern PFNGLPROGRAMPARAMETERS4FVNV glProgramParameters4fvNV;
VW_OPENGL_API extern PFNGLTRACKMATRIXNV glTrackMatrixNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBPPOINTERNV glVertexAttribPointerNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB1SNV glVertexAttrib1sNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB1FNV glVertexAttrib1fNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB1DNV glVertexAttrib1dNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB2SNV glVertexAttrib2sNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB2FNV glVertexAttrib2fNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB2DNV glVertexAttrib2dNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB3SNV glVertexAttrib3sNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB3FNV glVertexAttrib3fNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB3DNV glVertexAttrib3dNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB4SNV glVertexAttrib4sNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB4FNV glVertexAttrib4fNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB4DNV glVertexAttrib4dNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB4UBNV glVertexAttrib4ubNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB1SVNV glVertexAttrib1svNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB1FVNV glVertexAttrib1fvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB1DVNV glVertexAttrib1dvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB2SVNV glVertexAttrib2svNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB2FVNV glVertexAttrib2fvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB2DVNV glVertexAttrib2dvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB3SVNV glVertexAttrib3svNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB3FVNV glVertexAttrib3fvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB3DVNV glVertexAttrib3dvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB4SVNV glVertexAttrib4svNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB4FVNV glVertexAttrib4fvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB4DVNV glVertexAttrib4dvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIB4UBVNV glVertexAttrib4ubvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS1SVNV glVertexAttribs1svNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS1FVNV glVertexAttribs1fvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS1DVNV glVertexAttribs1dvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS2SVNV glVertexAttribs2svNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS2FVNV glVertexAttribs2fvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS2DVNV glVertexAttribs2dvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS3SVNV glVertexAttribs3svNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS3FVNV glVertexAttribs3fvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS3DVNV glVertexAttribs3dvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS4SVNV glVertexAttribs4svNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS4FVNV glVertexAttribs4fvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS4DVNV glVertexAttribs4dvNV;
VW_OPENGL_API extern PFNGLVERTEXATTRIBS4UBVNV glVertexAttribs4ubvNV;


//====================================================================
// GL_SGIS_generate_mipmap
//====================================================================
#ifndef GL_SGIS_generate_mipmap
#define GL_SGIS_generate_mipmap    1


//--------------------------------------------------------------------
#define GL_GENERATE_MIPMAP_SGIS                                 0x8191
#define GL_GENERATE_MIPMAP_HINT_SGIS                            0x8192


//--------------------------------------------------------------------
#endif //!GL_SGIS_generate_mipmap


//====================================================================
// GL_SGIX_shadow
//====================================================================
#ifndef GL_SGIX_shadow
#define GL_SGIX_shadow    1


//--------------------------------------------------------------------
#define GL_TEXTURE_COMPARE_SGIX                                 0x819A
#define GL_TEXTURE_COMPARE_OPERATOR_SGIX                        0x819B
#define GL_TEXTURE_LEQUAL_R_SGIX                                0x819C
#define GL_TEXTURE_GEQUAL_R_SGIX                                0x819D


//--------------------------------------------------------------------
#endif //!GL_SGIX_shadow


//====================================================================
// GL_SGIX_depth_texture
//====================================================================
#ifndef GL_SGIX_depth_texture
#define GL_SGIX_depth_texture    1


//--------------------------------------------------------------------
#define GL_DEPTH_COMPONENT16_SGIX                               0x81A5
#define GL_DEPTH_COMPONENT24_SGIX                               0x81A6
#define GL_DEPTH_COMPONENT32_SGIX                               0x81A7


//--------------------------------------------------------------------
#endif //!GL_SGIX_depth_texture


//********************************************************************
}//namespace gl_ext


//********************************************************************
}//namespace vertexware


