

//--------------------------------------------------------------------
#include "GLTools\States.h"
#include "BaseTools\Constructs.h"
#include "GLTools\Extensions.h"
#include "GLTools\ErrorHandling.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gl_tools {


//====================================================================
void __fastcall SetupPixelPack(GLboolean bSwapBytes,
								uint rowAlignmentInBytes,
								uint rowLengthInPixels,
								uint imageHeightInPixels)
{
	glPixelStorei(GL_PACK_SWAP_BYTES, bSwapBytes);
	glPixelStorei(GL_PACK_ALIGNMENT, rowAlignmentInBytes);
	glPixelStorei(GL_PACK_ROW_LENGTH, rowLengthInPixels);
	if(gl_ext::IsSupported(gl_ext::Extension_GL_CORE_12)) {
		glPixelStorei(GL_PACK_IMAGE_HEIGHT, imageHeightInPixels);
	}
}


//====================================================================
PixelPackSentry::PixelPackSentry(GLboolean bSwapBytes,
								uint rowAlignmentInBytes,
								uint rowLengthInPixels,
								uint imageHeightInPixels)
{
	swapBytes = gl_tools::GetInteger(GL_PACK_SWAP_BYTES);
	rowAlignment = gl_tools::GetInteger(GL_PACK_ALIGNMENT);
	rowLength = gl_tools::GetInteger(GL_PACK_ROW_LENGTH);
	if(gl_ext::IsSupported(gl_ext::Extension_GL_CORE_12)) {
		imageHeight = gl_tools::GetInteger(GL_PACK_IMAGE_HEIGHT);
	}
	SetupPixelPack(bSwapBytes, rowAlignmentInBytes, rowLengthInPixels, imageHeightInPixels);
}


//====================================================================
PixelPackSentry::~PixelPackSentry()
{
	SetupPixelPack(swapBytes, rowAlignment, rowLength, imageHeight);
}


//====================================================================
void __fastcall SetupPixelUnpack(GLboolean bSwapBytes,
								uint rowAlignmentInBytes,
								uint rowLengthInPixels,
								uint imageHeightInPixels)
{
	glPixelStorei(GL_UNPACK_SWAP_BYTES, bSwapBytes);
	glPixelStorei(GL_UNPACK_ALIGNMENT, rowAlignmentInBytes);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, rowLengthInPixels);
	if(gl_ext::IsSupported(gl_ext::Extension_GL_CORE_12)) {
		glPixelStorei(GL_UNPACK_IMAGE_HEIGHT, imageHeightInPixels);
	}
}


//====================================================================
PixelUnpackSentry::PixelUnpackSentry(GLboolean bSwapBytes,
									uint rowAlignmentInBytes,
									uint rowLengthInPixels,
									uint imageHeightInPixels)
{
	swapBytes = gl_tools::GetInteger(GL_UNPACK_SWAP_BYTES);
	rowAlignment = gl_tools::GetInteger(GL_UNPACK_ALIGNMENT);
	rowLength = gl_tools::GetInteger(GL_UNPACK_ROW_LENGTH);
	if(gl_ext::IsSupported(gl_ext::Extension_GL_CORE_12)) {
		imageHeight = gl_tools::GetInteger(GL_UNPACK_IMAGE_HEIGHT);
	}
	SetupPixelUnpack(bSwapBytes, rowAlignmentInBytes, rowLengthInPixels, imageHeightInPixels);
}


//====================================================================
PixelUnpackSentry::~PixelUnpackSentry()
{
	SetupPixelUnpack(swapBytes, rowAlignment, rowLength, imageHeight);
}


//====================================================================
ReadBufferSentry::ReadBufferSentry(GLenum newReadBufferMode)
{
	lastReadBufferMode = gl_tools::GetInteger(GL_READ_BUFFER);
	glReadBuffer(newReadBufferMode);
}


//====================================================================
ReadBufferSentry::~ReadBufferSentry()
{
	glReadBuffer(lastReadBufferMode);
}


//====================================================================
DrawBufferSentry::DrawBufferSentry(GLenum newDrawBufferMode)
{
	lastDrawBufferMode = gl_tools::GetInteger(GL_DRAW_BUFFER);
	glDrawBuffer(newDrawBufferMode);
}


//====================================================================
DrawBufferSentry::~DrawBufferSentry()
{
	glDrawBuffer(lastDrawBufferMode);
}


//********************************************************************
}//namespace gl_tools


//********************************************************************
}//namespace vertexware


