

//--------------------------------------------------------------------
#include "GLTools\Transforms.h"
#include "MathTools\MathBase.h"
#include "MathTools\TransformTools.h"


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gl_tools {


//====================================================================
void __fastcall Perspective(GLdouble fovYDeg, GLdouble aspectWToH, GLdouble zNear, GLdouble zFar)
{
	GLdouble halfH = zNear * TanDeg(fovYDeg * 0.5);
	GLdouble halfW = halfH * aspectWToH;

	glFrustum(-halfW, halfW, -halfH, halfH, zNear, zFar);
}


//====================================================================
void __fastcall View(const vec3& rotation,
					const vec3& position,
					bool bProjectionCompensation)
{
	mat4  mT, mY, mX, mZ;

	GenerateTranslateVM(mT, -position.x, -position.y, -position.z);
	GenerateRotateVM(mY, -rotation.y, Axis_Y);
	GenerateRotateVM(mX, -rotation.x, Axis_X);
	GenerateRotateVM(mZ, -rotation.z, Axis_Z);

	mat4  mFinal = mT * mY * mX * mZ;

	if(bProjectionCompensation) {

		mat4  mP;
		GenerateNonUniformScale(mP, -1, 1, -1);

		glLoadMatrixf(mFinal * mP);
	}
	else {
		glLoadMatrixf(mFinal);
	}
}


//********************************************************************
}//namespace gl_tools


//********************************************************************
}//namespace vertexware


