

//--------------------------------------------------------------------
#pragma once


//--------------------------------------------------------------------
#include "BaseTools\Memory.h"
#include "GLTools\States.h"
#include "GLTools\Extensions.h"
#include "private\vw_opengl_exports.h"
#include <gl\gl.h>


//********************************************************************
namespace vertexware {


//********************************************************************
namespace gl_tools {


//====================================================================
template<typename TVertex>
class AdaptiveVertexBuffer : public AdaptiveBuffer<TVertex>
{
public:
	typedef  TVertex  VertexType;


public:
	AdaptiveVertexBuffer(uint minSize = 0, uint minStableCycles = 0) :
		AdaptiveBuffer<TVertex>(minSize, minStableCycles)
	{
		bAutoHookPointersToGLOnRealloc = false;
	}

	void  SetAutoHookPointersToGLOnRealloc(bool bAuto) {
		bAutoHookPointersToGLOnRealloc = bAuto;
	}

	void  HookPointersToGL() const {
		if(gl_ext::IsGLReady()  &&  ToPtr()) DoHookPointersToGL(ToPtr());
	}


private:
	bool  bAutoHookPointersToGLOnRealloc;


protected:
	virtual  void  DoAllocate(uint newSize)
	{
		AdaptiveBuffer<TVertex>::DoAllocate(newSize);
		if(bAutoHookPointersToGLOnRealloc) HookPointersToGL();
	}

	// implemented by derived class
	virtual  void  DoHookPointersToGL(const TVertex* pVertexZero) const = 0;
};


//********************************************************************
}//namespace gl_tools


//********************************************************************
}//namespace vertexware


